package ghmessen;

import static ghmessen.groesse.darstellung.AnpassungEnum.Ausgleichsgerade;
import static ghmessen.groesse.darstellung.AnpassungEnum.Hyperbel;
import static ghmessen.groesse.darstellung.AnpassungEnum.Normalparabel;
import static ghmessen.groesse.darstellung.AnpassungEnum.Parabel;
import static ghmessen.groesse.darstellung.AnpassungEnum.QuadHyperbel;
import static ghmessen.groesse.darstellung.AnpassungEnum.Ursprungsgerade;
import static ghmessen.groesse.darstellung.AnpassungEnum.freieFunktion;
import static ghmessen.groesse.wertetabelle.XAchseTyp.*;
import ghmessen.components.GreekKeyEventDispatcher;
import ghmessen.filefilter.CL4FileFilter;
import ghmessen.filefilter.CV4FileFilter;
import ghmessen.filefilter.FileFormatException;
import ghmessen.groesse.Art;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.Messgroesse;
import ghmessen.groesse.darstellung.AnpassungEnum;
import ghmessen.groesse.darstellung.AnpassungFrei;
import ghmessen.groesse.darstellung.AnpassungMittelwert;
import ghmessen.groesse.darstellung.AnpassungRegression;
import ghmessen.groesse.darstellung.Auswertung;
import ghmessen.groesse.darstellung.LinienAuswertung;
import ghmessen.groesse.darstellung.TextAuswertung;
import ghmessen.parser.FreieFunktionParser;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.InputMismatchException;
import java.util.Locale;
import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.Vector;
import java.util.concurrent.ConcurrentSkipListSet;

import org.codehaus.janino.ExpressionEvaluator;

public class CL4File {
	public static boolean DEBUG = false;

	@SuppressWarnings({ "serial" })
	private static final HashMap<String, String> operators = new HashMap<String, String>() {
		{
			put("last ", "n-1");
			put("next ", "n");
		}
	};

	public enum AUSWERTUNG {
		TEXTMARKIERUNG(0, 2, true), SENKRECHTELINIE(1, 1), WAAGERECHTELINIE(2,
				1), MITTELWERT(10, 1), AUSGLEICHSGERADE(20, 2), URSPRUNGSGERADE(
				21, 2), PARABEL(30, 3), NORMALPARABEL(31, 3), HYPERBEL(40, 2), QUADHYPERBEL(
				41, 3), EFUNKTION(45, 3), FREIEFUNKTION(46, 4, true);

		private int nr;
		private int doubleAnzahl;
		private boolean hatText;

		private AUSWERTUNG(int nr) {
			this(nr, 0, false);
		}

		private AUSWERTUNG(int nr, int doubleAnzahl) {
			this(nr, doubleAnzahl, false);
		}

		private AUSWERTUNG(int nr, int doubleAnzahl, boolean hatText) {
			this.nr = nr;
			this.doubleAnzahl = doubleAnzahl;
			this.hatText = hatText;
		}

		public int getNr() {
			return nr;
		}

		public int getDoubleAnzahl() {
			return doubleAnzahl;
		}

		public boolean hatText() {
			return hatText;
		}

		public static AUSWERTUNG toAuswertung(int nr) {
			for (AUSWERTUNG a : values())
				if (nr == a.nr)
					return a;
			return null;
		}

		public Auswertung toAuswertung(Double[] x, Double[] y, String text,
				Font font, Color farbe, BasicStroke linie, Double[] parameter) {
			AnpassungEnum anpassung = null;
			switch (this) {
			default:
				return null;
			case TEXTMARKIERUNG:
				return new TextAuswertung(font, text, farbe,
						new Point2D.Double(parameter[0], parameter[1]), 1);
			case MITTELWERT:
				return new AnpassungMittelwert(x, y);
			case SENKRECHTELINIE:
				return new LinienAuswertung(true, linie, new Point2D.Double(
						parameter[0], 0.0), 1);
			case WAAGERECHTELINIE:
				return new LinienAuswertung(false, linie, new Point2D.Double(
						0.0, parameter[0]), 1);
			case FREIEFUNKTION:
				FreieFunktionParser p = new FreieFunktionParser();
				ExpressionEvaluator f = p.getAusdruck(text);
				double von = x[0];
				double bis = x[x.length - 1];
				double[] param = new double[parameter.length];
				System.arraycopy(parameter, 0, parameter, 0, parameter.length);
				return new AnpassungFrei(freieFunktion, x, y, f, param, von,
						bis, 0, 0, x.length - 1, "t", "y", 3, 3, null, null, text, farbe);
			case URSPRUNGSGERADE:
				anpassung = Ursprungsgerade;
				break;
			case AUSGLEICHSGERADE:
				anpassung = Ausgleichsgerade;
				break;
			case PARABEL:
				anpassung = Parabel;
				break;
			case NORMALPARABEL:
				anpassung = Normalparabel;
				break;
			case HYPERBEL:
				anpassung = Hyperbel;
				break;
			case QUADHYPERBEL:
				anpassung = QuadHyperbel;
				break;
			}
			return new AnpassungRegression(anpassung, x, y, farbe);
		}
	}

	// private class CL4Cassy {
	// int nr;
	// int a;
	// int b;
	// }

	private boolean IS_CL4 = true;
	private Vector<String[]> groessen;
	private Vector<Integer> groessenReihe;
	private int groessenAnzahl = 0;
	// private CL4Cassy[] cassys;
	private String mp;
	private Double[][] werte;
	private int nanZeilen = 0;
	private Vector<String[]> darstellung;
	private String kommentar;
	private Vector<String> auswertung;
	private int fensterTeilung;
	private double mittelZeit;
	private int nr = 0;
	private ConcurrentSkipListSet<Integer> frequenz;
	private ConcurrentSkipListSet<Integer> ignoriert;

	public CL4File() {
		IS_CL4 = true;
	}

	public void readFile(File f) throws FileFormatException,
			FileNotFoundException {

		Scanner in = new Scanner(f, "ISO-8859-15");
		in.useLocale(Locale.US); // Dezimalpunkt!

		nr = 1;
		frequenz = new ConcurrentSkipListSet<Integer>();
		ignoriert = new ConcurrentSkipListSet<Integer>();

		/* Dateikennung */
		if (new CL4FileFilter().accept(f))
			IS_CL4 = true;
		else if (new CV4FileFilter().accept(f))
			IS_CL4 = false;
		else {
			in.close();
			throw new FileFormatException(
					"line 1: Dateikennung CL4/CV4 nicht gefunden.");
		}
		in.nextLine(); // 1. Zeile ignoriert (CL4/CV4)
		fensterTeilung = in.nextInt();
		if (in.hasNextDouble())
			mittelZeit = in.nextDouble();
		else
			mittelZeit = Konstanten.GROESSE_MITTELZEIT_STANDARD;
		in.nextLine();
		nr++;
		/* unabhängige Größen n,t,f */
		String[][] ntf = new String[3][4];
		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 4; j++)
				ntf[i][j] = in.nextLine();
		nr += 12;
		/* abhängige Größen */
		groessen = new Vector<String[]>();
		groessenReihe = new Vector<Integer>();
		groessenAnzahl = in.nextInt();
		in.nextInt(); // 2. Zahl ignoriert
		in.nextLine();
		nr++;
		String[] groesse;
		for (int i = 0; i < groessenAnzahl; i++) {
			groesse = leseZeilen(6, in);
			Scanner scg = new Scanner(groesse[0]);
			int art = scg.nextInt();
			scg.close();
			bearbeiteGroesse(groesse, i);
			if (art == 16) {
				leseZeilen(2, in);
				nr += 8;
			} else
				nr += 6;
		}
		/* Wertetabelle Zeittabelle */
		int zs = in.nextInt(), zz = in.nextInt();
		in.nextLine();
		nr++;
		String[] zeittabelle = leseZeilen(zz, in);
		if (IS_CL4) {
			werte = new Double[zz][groessenAnzahl + UNABHAENGIG];
		} else {
			nanZeilen = zaehleNANZeilen(zeittabelle);
			if (nanZeilen != 0) {
				zz = laengsteMessreihe(zeittabelle);
				for (int i = 0; i < nanZeilen; i++) {
					for (int ii = 0; ii < groessenAnzahl; ii++) {
						if (ignoriert.contains(ii))
							continue;
						// Rauten hinzufügen
						String[] kopie = groessen.get(ii).clone();
						groessen.add(kopie);
						groessenReihe.add(i + 1);
					}
				}
			}
			werte = new Double[zz][groessenAnzahl + UNABHAENGIG + nanZeilen
					* (groessenAnzahl - ignoriert.size())];
		}
		bearbeiteZeitTabelle(zz, zs, werte, zeittabelle);
		/* Wertetabelle Frequenztabelle */
		int fs = in.nextInt(), fz = in.nextInt();
		in.nextLine();
		nr++;
		if (zz < fz) {
			werte = Arrays.copyOf(werte, fz);
			for (int i = zz; i < fz; i++)
				werte[i] = new Double[groessen.size() + UNABHAENGIG];
		}
		bearbeiteFrequenzTabelle(fz, fs, werte, leseZeilen(fz, in));
		/* Cassys 8 Zeilen */
		// cassys = new CL4Cassy[8];
		bearbeiteCassys(leseZeilen(8, in));
		/* Kommentarzeilen */
		int nk = in.nextInt();
		in.nextLine();
		nr++;
		{
			String[] temp = leseZeilen(nk, in);
			kommentar = "";
			for (int i = 0; i < nk; i++)
				kommentar = kommentar + temp[i] + "\n";
		}
		/* Messparameter */
		mp = "";
		nr++;
		bearbeiteMessparameter(in.nextLine());
		/* Darstellungen */
		darstellung = new Vector<String[]>();
		int nd = in.nextInt();
		in.nextLine();
		nr++;
		for (int i = 0; i < nd; i++) {
			bearbeiteDarstellung(leseZeilen(2, in));
			nr += 2;
		}
		/* Modelle */
		@SuppressWarnings("unused")
		int nm = in.nextInt();
		in.nextLine();
		nr++;
		@SuppressWarnings("unused")
		String letzteAuswertung = in.nextLine();
		nr++;
		/* Auswertungen */
		int na = in.nextInt();
		in.nextLine();
		nr++;
		if (na > 0)
			auswertung = new Vector<String>();
		String s;
		for (int i = 0; i < na; i++) {
			s = in.nextLine();
			try {
				if (AUSWERTUNG.toAuswertung(Integer.parseInt(s.substring(0,
						s.indexOf(" ")))) != null)
					auswertung.add(s);
			} catch (IndexOutOfBoundsException e) {
				if (DEBUG)
					e.printStackTrace();
			} catch (NumberFormatException e) {
				if (DEBUG)
					e.printStackTrace();
			}
			nr++;
		}
	}

	private void bearbeiteCassys(String[] a) throws FileFormatException {

		for (int i = 0; i < 8; i++) {
			// Scanner line = new Scanner(a[i]);
			// die Bearbeitung ist vorläufig noch unwichtig
			/*
			 * cassys[i].nr = line.nextInt(); if(cassys[i].nr > -1){
			 * if(line.hasNextInt()) cassys[i].a = line.nextInt();
			 * if(line.hasNextInt()) cassys[i].b = line.nextInt(); }
			 */
		}
		nr += 8;
	}

	private String[] leseZeilen(int n, Scanner in) throws FileFormatException {
		return leseZeilen(n, in, nr);
	}

	public static String[] leseZeilen(int n, Scanner in, int nr)
			throws FileFormatException {
		String[] result = new String[n];
		for (int i = 0; i < n; i++)
			try {
				result[i] = in.nextLine();
			} catch (NoSuchElementException e) {
				throw new FileFormatException("line " + (nr + i)
						+ ": ist nicht vorhanden.");
			} catch (IllegalStateException e) {
				throw new FileFormatException("line " + (nr + i)
						+ ": Dateende erreicht.");
			}
		return result;
	}

	private Double[] leseDoubleListe(String s, int n, boolean nullErlaubt)
			throws FileFormatException {
		return leseDoubleListe(s, n, nr, nullErlaubt);
	}

	public static Double[] leseDoubleListe(String s, int n, int nr,
			boolean nullErlaubt) throws FileFormatException {
		Scanner din = new Scanner(s);
		din.useLocale(Locale.US);
		try {
			Double[] result = new Double[n];
			for (int i = 0; i < n; i++)
				try {
					result[i] = din.nextDouble();
					if (result[i] != null & result[i].equals(Double.NaN))
						result[i] = null;
				} catch (InputMismatchException e) {
					if (!nullErlaubt)
						throw new FileFormatException("line " + nr
								+ ": Double(" + i + ") ist nicht vorhanden.");
				} catch (NoSuchElementException e) {
					if (!nullErlaubt)
						throw new FileFormatException("line " + nr
								+ ": Double(" + i + ") ist nicht vorhanden.");
				} catch (IllegalStateException e) {
					if (!nullErlaubt)
						throw new FileFormatException("line " + nr
								+ ": Double(" + i + ") ist nicht vorhanden.");
				}
			return result;
		} finally {
			din.close();
		}
	}

	private int zaehleNANZeilen(String[] zeilen) {
		int ret = 0;
		for (String zeile : zeilen)
			if (zeile.startsWith("NAN"))
				ret++;
		return ret;
	}

	private int laengsteMessreihe(String[] zeilen) {
		int ret = 0;
		int z = zeilen.length - 1;
		Scanner a0 = new Scanner(zeilen[z]);
		a0.useLocale(Locale.US);
		try {
			ret = a0.nextInt();
			z--;
			while (z > 0) {
				if (zeilen[z].startsWith("NAN")) {
					z--;
					if (z > 0) {
						if (a0 != null)
							a0.close();
						a0 = new Scanner(zeilen[z]);
						a0.useLocale(Locale.US);
						int neu = a0.nextInt();
						if (neu > ret)
							ret = neu;
					}
				}
				z--;
			}
			return ret;
		} finally {
			a0.close();
		}
	}

	private void bearbeiteGroesse(String a[], int index)
			throws FileFormatException {
		Scanner a0 = new Scanner(a[0]);
		a0.useLocale(Locale.US);
		try {
			int artInt = a0.nextInt();
			int typ;
			try {
				typ = a0.nextInt();
			} catch (InputMismatchException e) {
				typ = -1;
			}
			Art art = Art.fromCL4(artInt, typ);
			if (art == Art.Unbekannt) {
				ignoriert.add(index);
				return;
			}
			int bezug = -1;
			switch (art) {
			default:
				break;
			case Klasse:
				bezug = typ;
				break;
			case FFT:
				frequenz.add(UNABHAENGIG + index);
			case Ableitung:
			case Integral:
			case Mittelwert:
			case Histogramm:
				bezug = a0.nextInt();
				break;
			}
			groessen.add(a);
			groessenReihe.add(0);
			if (bezug > groessen.size() - 1)
				throw new FileFormatException("line " + nr
						+ ": bearbeiteGroesse - unmöglicher Bezug(" + bezug
						+ ")");
		} finally {
			a0.close();
		}
	}

	private void bearbeiteZeitTabelle(int z, int s, Double[][] werte, String[] a)
			throws FileFormatException {
		if (z < 1)
			return;
		Double[] tmp = null;
		int rOfs = 0;
		int i = 0;
		int zeile = 0;
		while (zeile < a.length) {
			tmp = leseDoubleListe(a[zeile].replaceAll("NAN", "NaN"), s, true);
			if (tmp[0] == null && !IS_CL4) {
				rOfs += groessenAnzahl - ignoriert.size();
				i = 0;
				zeile++;
				continue;
			}
			if (IS_CL4) {
				werte[i][T.ordinal()] = tmp[1];
				werte[i][N.ordinal()] = tmp[0];
				werte[i][F.ordinal()] = werte[i][BETA.ordinal()] = werte[i][LAMBDA.ordinal()] = werte[i][ENERGIE.ordinal()] = 0.0;
			} else {
				werte[i][ENERGIE.ordinal()] = tmp[1];
				werte[i][N.ordinal()] = tmp[0];
				werte[i][F.ordinal()] = werte[i][BETA.ordinal()] = werte[i][LAMBDA.ordinal()] = werte[i][T.ordinal()] = 0.0;
			}
			for (int j0 = UNABHAENGIG, j1 = 2; j0 < s + UNABHAENGIG
					- ignoriert.size() - 2; j0++, j1++) {
				while (frequenz.contains(j0))
					j0++;
				while (ignoriert.contains(j1 - 2))
					j1++;
				werte[i][j0 + rOfs] = tmp[j1];
			}
			i++;
			zeile++;
		}
	}

	private void bearbeiteFrequenzTabelle(int z, int s, Double[][] werte,
			String[] a) throws FileFormatException {
		if (z < 1)
			return;
		Double[] tmp = null;
		if (z < a.length)
			throw new FileFormatException("line " + nr
					+ ": Tabellenlänge zu kurz z(" + z + ") a.length("
					+ a.length + ")");
		if (s != (frequenz.size() + 1))
			throw new FileFormatException("line " + nr
					+ ": Tabellenspalten falsch s(" + s + ") frequenz.size("
					+ frequenz.size() + ")");

		for (int i = 0; i < z; i++) {
			tmp = leseDoubleListe(a[i].replaceAll("NAN", "NaN"), s, true);
			werte[i][F.ordinal()] = tmp[0];
			Integer[] spalte = frequenz.toArray(new Integer[0]);
			for (int j = 0; j < spalte.length; j++)
				werte[i][spalte[j]] = tmp[j + 1];
		}
	}

	private void bearbeiteMessparameter(String s) throws FileFormatException {
		if (mp == null)
			throw new FileFormatException("line " + nr
					+ ": Messparameter mp ist nicht initialisiert.");
		int pos = s.lastIndexOf(" ");
		leseDoubleListe(s.substring(0, pos), 12, false);
		mp = s;
	}

	private String getSymbol(int n) throws FileFormatException {
		switch (n) {
		case 0:
			return "n";
		case 1:
			return IS_CL4 ? "t" : "E";
		case 2:
			return "f";
		default:
			try {
				return groessen.get(n - 3)[3];
			} catch (NullPointerException e) {
				throw new FileFormatException("line " + nr
						+ ": Die Größenliste ist nicht initialisiert.");
			} catch (IndexOutOfBoundsException e) {
				if (DEBUG)
					System.out.println("line " + nr + ": Falsche Größennummer("
							+ n + ")Histogramm?");
				return null;
				// Histogramm ignorieren: throw new
				// FileFormatException("line "+nr+": Falsche Größennummer("+n+")");
			}
		}
	}

	private void bearbeiteDarstellung(String[] s) throws FileFormatException {
		if (s == null || s.length != 2)
			throw new FileFormatException("line " + nr
					+ ": Darstellung ist fehlerhaft.");
		if (darstellung == null)
			throw new FileFormatException("line " + nr
					+ ": Darstellungsliste ist nicht initialisiert.");
		Scanner s2i = new Scanner(s[1]);
		s2i.useLocale(Locale.US);
		try {
			String[] ySymbol = new String[8];
			int[] yS = new int[8];
			double[] yMin = new double[8], yMax = new double[8];
			int groesse;
			int ii = 0;
			while (ii < 8) {
				groesse = s2i.nextInt();
				if (groesse != -1) {
					ySymbol[ii] = getSymbol(groesse);
					yS[ii] = s2i.nextInt();
					yMin[ii] = s2i.nextDouble();
					yMax[ii] = s2i.nextDouble();
				}
				ii++;
			}
			s2i.nextInt(); // 1 Zahl (-1) ignoriert
			if (ySymbol[1] == null)
				return;
			darstellung.add(s);
			s2i.nextInt();
			s2i.nextInt();
			s2i.nextInt();
			s2i.nextInt();
			s2i.nextInt(); // 5 Zahlen (10101) ignoriert
			ii = 0;
			while (ii < 8 && ySymbol[ii] != null) {
				if (yS[ii] < 0 || yS[ii] > 4)
					throw new FileFormatException("line " + nr
							+ ": Darstellung hat die fehlerhafte Skalierung("
							+ yS[ii] + ") für die Spalte(" + ii + ")");
				ii++;
			}
			s2i.nextInt();
			if (s2i.hasNextInt())
				s2i.nextInt();
			if (s2i.hasNextInt())
				s2i.nextInt(); // 3 Zahlen (010) ignoriert
		} finally {
			s2i.close();
		}
	}

	public boolean isCL4() {
		return IS_CL4;
	}
	
	public int getGroesseAnzahl() {
		return (groessen != null ? groessen.size() : 0);
	}

	public String[] getGroesse(int i) {
		if (groessen != null && groessen.size() > i)
			return groessen.get(i);
		else
			return null;
	}

	public int getGroessenReihe(int i) {
		if (groessenReihe != null && groessenReihe.size() > i)
			return groessenReihe.get(i);
		else
			return 0;
	}

	public int getFensterTeilung() {
		return fensterTeilung;
	}

	public double getMittelZeit() {
		return mittelZeit;
	}

	public String getMessparameter() {
		return mp;
	}

	public Double[][] getWerteTabelle() {
		return werte;
	}

	public String getKommentar() {
		return kommentar;
	}

	public int getDarstellungAnzahl() {
		return (darstellung != null ? darstellung.size() : 0);
	}

	public String[] getDarstellung(int i) {
		if (darstellung != null && darstellung.size() > i)
			return darstellung.get(i);
		else
			return null;
	}

	public int getAuswertungsAnzahl() {
		return auswertung != null ? auswertung.size() : 0;
	}

	public String getAuswertung(int i) {
		if (auswertung != null & auswertung.size() > i)
			return auswertung.get(i);
		else
			return null;
	}

	public void writeFile(File f) throws FileFormatException {
		throw new FileFormatException(
				"Die Funktion CL4File.writeFile ist nicht implementiert.");
	}

	public static String getSymbolCL4(String cl4symbol) {
		return GreekKeyEventDispatcher.toGreek(cl4symbol);
	}

	public static String getSymbolCL4(String cl4symbol, Messgroesse m) {
		String result = getSymbolCL4(cl4symbol);
		if (result.startsWith(m.getSymbol()) && result.indexOf("_") < 0)
			result = m.getSymbol() + "_"
					+ result.substring(m.getSymbol().length());
		return result;
	}

	public static String replaceOperator(String cl4formel,
			HashMap<String, String> syms, String op) {
		if (!operators.containsKey(op))
			return cl4formel;
		int index = cl4formel.indexOf(op);
		if (index < 0)
			return cl4formel;
		String symbol = null;
		try {
			for (String key : syms.keySet())
				if (cl4formel.substring(index + op.length()).startsWith(key))
					symbol = key;
			if (symbol == null)
				for (String key : new String[] { "t", "n", "f", "\u03B2",
						"\u03BB", "E" })
					if (cl4formel.substring(index + op.length())
							.startsWith(key))
						symbol = key;
		} catch (IndexOutOfBoundsException ex) {
			if (DEBUG)
				ex.printStackTrace();
		}
		if (symbol == null)
			return cl4formel;
		cl4formel = cl4formel.replaceAll(op + symbol,
				symbol + "[" + operators.get(op) + "]");
		return cl4formel;
	}

	public static String getFormelCL4(Groessenliste gl, String symbol,
			String cl4symbol, String cl4formel) {
		if (cl4formel.indexOf("_") > -1)
			cl4formel = cl4formel.replaceAll("_", "");
		cl4formel = GreekKeyEventDispatcher.toGreek(cl4formel);
		HashMap<String, String> map = new HashMap<String, String>();
		map.put(cl4symbol, symbol);
		for (Groesse g : gl)
			map.put(g.getSymbolCL4(), g.getSymbol());
		for (String key : operators.keySet())
			while (cl4formel.indexOf(key) > -1)
				cl4formel = replaceOperator(cl4formel, map, key);
		for (String key : map.keySet())
			cl4formel = cl4formel.replaceAll(key, map.get(key));
		return cl4formel;
	}

}
