package ghmessen;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.prefs.BackingStoreException;
import java.util.prefs.InvalidPreferencesFormatException;
import java.util.prefs.Preferences;

public class Eigenschaften {
	private static final boolean DEBUG = false;
	private static int pieceLength = Preferences.MAX_VALUE_LENGTH * 3 / 4;

	private static String filename = "GHMessen.xml";
	private static String[] pfad = new String[]{ "/etc/", System.getProperty("user.home")+"/." };
	
	static private byte[] object2Bytes(Object o) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(o);
		return baos.toByteArray();
	}

	static private Object bytes2Object(byte raw[]) throws IOException,
			ClassNotFoundException {
		ByteArrayInputStream bais = new ByteArrayInputStream(raw);
		ObjectInputStream ois = new ObjectInputStream(bais);
		Object o = ois.readObject();
		return o;
	}

	static private byte[][] breakIntoPieces(byte raw[]) {
		int numPieces = (raw.length + pieceLength - 1) / pieceLength;
		byte pieces[][] = new byte[numPieces][];
		for (int i = 0; i < numPieces; ++i) {
			int startByte = i * pieceLength;
			int endByte = startByte + pieceLength;
			if (endByte > raw.length)
				endByte = raw.length;
			int length = endByte - startByte;
			pieces[i] = new byte[length];
			System.arraycopy(raw, startByte, pieces[i], 0, length);
		}
		return pieces;
	}

	static private byte[] combinePieces(byte pieces[][]) {
		int length = 0;
		for (int i = 0; i < pieces.length; ++i) {
			length += pieces[i].length;
		}
		byte raw[] = new byte[length];
		int cursor = 0;
		for (int i = 0; i < pieces.length; ++i) {
			System.arraycopy(pieces[i], 0, raw, cursor, pieces[i].length);
			cursor += pieces[i].length;
		}
		return raw;
	}

	static private void writePieces(Preferences prefs, String key,
			byte[][] pieces) throws BackingStoreException {
		Preferences child = prefs.node(GHMessen.EIGENSCHAFTEN).node(key);
		child.clear();
		for (int i = 0; i < pieces.length; ++i) {
			child.putByteArray("" + i, pieces[i]);
		}
	}

	static private byte[][] readPieces(Preferences prefs, String key)
			throws BackingStoreException {
		Preferences child = prefs.node(GHMessen.EIGENSCHAFTEN).node(key);
		byte[][] pieces;
		int numPieces = child.keys().length;
		pieces = new byte[numPieces][];
		for (int i = 0; i < numPieces; ++i) {
			pieces[i] = child.getByteArray("" + i, null);
		}
		return pieces;
	}

	static public void putObject(Preferences prefs, String key, Object o)
			throws IOException, BackingStoreException, ClassNotFoundException {
		if (DEBUG) {
			System.out.println("[Eigenschaften] - putObject: key=" + key);
		}
		byte raw[] = object2Bytes(o);
		byte pieces[][] = breakIntoPieces(raw);
		writePieces(prefs, key, pieces);
	}

	static public Object getObject(Preferences prefs, String key)
			throws IOException, BackingStoreException, ClassNotFoundException {
		if (DEBUG) {
			System.out.println("[Eigenschaften] - getObject: key=" + key);
		}
		byte pieces[][] = readPieces(prefs, key);
		byte raw[] = combinePieces(pieces);
		Object o = bytes2Object(raw);
		return o;
	}

	static public void writeConfig(Preferences prefs) throws IOException, BackingStoreException{
		File outf = new File(pfad[pfad.length-1] + filename);
		if(outf.exists() && outf.canWrite() || outf.createNewFile()){
			FileOutputStream os = new FileOutputStream(outf);
			prefs.node(GHMessen.EIGENSCHAFTEN).exportSubtree(os);
			os.close();
		}
	}
	
	static public void readConfig() throws IOException, InvalidPreferencesFormatException{
		for(int i = pfad.length;i>0;i--){
			File inf = new File(pfad[i-1] + filename);
			if(inf.exists() && inf.canRead()){
				FileInputStream is = new FileInputStream(inf);
				Preferences.importPreferences(is);
			}
		}
	}
}
