/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen;

import static ghmessen.Konstanten.AUSBLENDEN_INTERVALL;
import static ghmessen.Konstanten.DATEIVERSION;
import static ghmessen.filefilter.AlleFileFilter.FORMAT.CL4;
import static ghmessen.filefilter.AlleFileFilter.FORMAT.GHM;
import static ghmessen.filefilter.AlleFileFilter.FORMAT.GHMX;
import static javax.swing.GroupLayout.PREFERRED_SIZE;
import static javax.swing.GroupLayout.Alignment.LEADING;
import static javax.swing.LayoutStyle.ComponentPlacement.UNRELATED;
import ghmessen.adapter.AdapterListe;
import ghmessen.adapter.EAdapterException;
import ghmessen.components.ImageTransferHandler;
import ghmessen.components.MessgeraeteToolBar;
import ghmessen.dialoge.EigenschaftenBox;
import ghmessen.dialoge.EinstellungenBox;
import ghmessen.dialoge.GHMessenAboutBox;
import ghmessen.dialoge.MessparameterBox;
import ghmessen.dialoge.XRayMessparameterBox;
import ghmessen.drucken.DruckTabelleDlg;
import ghmessen.drucken.DruckerEinstellung;
import ghmessen.filefilter.AlleFileFilter;
import ghmessen.filefilter.AlleFileFilter.FORMAT;
import ghmessen.filefilter.CL4FileFilter;
import ghmessen.filefilter.FileFormatException;
import ghmessen.filefilter.GHMFileFilter;
import ghmessen.filefilter.GHMXFileFilter;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Darstellungsliste;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.WerteTabelle;
import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetStatus;
import ghmessen.instrumente.MessgeraeteMap;
import ghmessen.messtasks.AnzahlTimerTask;
import ghmessen.messtasks.MessTimerTask;
import ghmessen.messtasks.OszilloskopTimerTask;
import ghmessen.messtasks.XRayTimerTask;
import ghmessen.messtasks.ZeitTimerTask;
import ghmessen.util.CursorTk;

import java.awt.AWTException;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.BevelBorder;
import javax.swing.event.TableModelListener;
import javax.swing.filechooser.FileFilter;

import org.jdesktop.application.Action;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.observablecollections.ObservableList;
import org.jdesktop.observablecollections.ObservableListListener;
import org.xml.sax.SAXException;

public class GHMessenView extends FrameView implements ObservableListListener, AutoCloseable {
	private static final boolean DEBUG = false;

	private static final FileFilter[] dateifilter = new FileFilter[] {
			new AlleFileFilter(), new GHMFileFilter(), new GHMXFileFilter(),
			new CL4FileFilter() };
	private static final int GHMFILTER = 1;
	private static final int GHMXFILTER = 2;
	@SuppressWarnings("unused")
	private static final int CL4FILTER = 3;
	private File datei = null;
	private File dateinameAlt = null;
	private String appTitel;
	public Messparameter mp; // Messparameter
	public AdapterListe al; // Angeschlossene Adapter (u.a. Cassys)
	public Groessenliste gl; // alle abhängigen Größen
	public WerteTabelle gt; // Alle Messwerte, gl muss zuerst initialisiert
							// werden
	public Darstellungsliste ansicht; // Panels mit den Ansichten
	public String kommentar; // Inhalt des Kommentartextbereichs
	public ScheduledThreadPoolExecutor zeitgeber;
	private MessTimerTask messung; // die laufende Messung
	// Zugriff auf die laufende Messung
	private ScheduledFuture<?> laufendeMessung;
	// Timer für die Aktualisierung des progressBar
	private ScheduledFuture<?> progressBarTimer = null;
	public MessgeraeteMap messgeraete;
	// existierende Messgeräte
	private MessgeraeteToolBar tbMessgeraete;
	private static ActionMap actionMap;
	private InputMap iMap;
	private ActionMap aMap;
	private static ResourceMap resourceMap;

	public GHMessenView(SingleFrameApplication app) {
		super(app);
		kommentar = "";
		zeitgeber = new ScheduledThreadPoolExecutor(20);
		gl = new Groessenliste();
		al = new AdapterListe(gl, zeitgeber);
		mp = new Messparameter(gl, al);
		gl.addObservableListListener(this);
		gt = new WerteTabelle(gl, mp);
		ansicht = new Darstellungsliste();
		mp.addPropertyChangeListener("einzelEin", ansicht);
		ansicht.addObservableListListener(this);
		messgeraete = new MessgeraeteMap(gl, al);
		laufendeMessung = null;
		initComponents();
		messgeraete.addObservableMapListener(tbMessgeraete);
		if (GHMessen.hilfe != null && GHMessen.hSet != null)
			GHMessen.hilfe.enableHelpKey(getRootPane(), "hauptfenster",
					GHMessen.hSet);
	}

	public void checkGroupMemberships() throws IOException {
		Process bp = new ProcessBuilder("id","-nG").start();
		boolean isInLock = false, isInDialout = false;
		try(Scanner in = new Scanner(bp.getInputStream())) {
			String[] as;
			while(in.hasNextLine()) {
				as = in.nextLine().split(" ");
				for(String s : as) {
					if(s.matches("lock"))
						isInLock = true;
					if(s.matches("dialout"))
						isInDialout = true;
					if(isInLock && isInDialout)
						break;
				}
			}
		}
		if(!isInLock || !isInDialout) {
			String msg = 
					(!isInLock && !isInDialout? "in den Gruppen <em>lock</em> und <em>dialout</em>"
							: (!isInLock? "in der Gruppe <em>lock</em>" : "in der Gruppe <em>dialout</em>"));
			JOptionPane.showMessageDialog( null, 
					"<html><p>Der Benutzer "+System.getenv("USER") + " ist nicht Mitglied "
					+ msg + ".</p><p> Der Zugriff auf serielle CASSYs wird eventuell nicht funktionieren.</p></html>", 
					"Fehlende Zugriffsrechte", 
					JOptionPane.ERROR_MESSAGE);
		}
	}
	
	public void close() {
		if (al != null)
			al.close();
	}
	
	private boolean toolAnzeigenAn = false;

	public void setMessenSelected(boolean select) {
		if (!mp.isEinzelEin() && mp.getIntervall() < AUSBLENDEN_INTERVALL) {
			// Messgeraete ausblenden
			if (select && !mp.isEinzelEin() && toolAnzeigen.isSelected()) {
				toolAnzeigenAn = true;
				toolAnzeigen.doClick();
			} else if (!select && toolAnzeigenAn && !toolAnzeigen.isSelected()) {
				toolAnzeigenAn = false;
				toolAnzeigen.doClick();
			}
		} else if (!mp.isEinzelEin()) {
			// Messgeraete nicht ausblenden, aber GoIO liest nur gespeicherte
			// Werte
			if (messgeraete != null)
				for (Messgeraet m : messgeraete.values())
					m.setEinlesen(!select);
		}
		itemAnzeigen.setSelected(toolAnzeigen.isSelected());
		toolMessen.setSelected(select);
		if (select) {
			toolMessen.setAction(actionMap.get("stoppeMessung"));
			toolMessen.setText("");
			menuItemMessen.setAction(actionMap.get("stoppeMessung"));
			iMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0, false),
					"stoppeMessung");
		} else {
			toolMessen.setAction(actionMap.get("starteMessung"));
			toolMessen.setText("");
			menuItemMessen.setAction(actionMap.get("starteMessung"));
			iMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0, false),
					"starteMessung");
		}
	}

	@Action
	public void showHelpBox() {
		if (GHMessen.hilfe != null)
			GHMessen.hilfe.setDisplayed(true);
	}

	@Action
	public void showAboutBox() {
		if (aboutBox == null) {
			JFrame mainFrame = GHMessen.getApplication().getMainFrame();
			aboutBox = new GHMessenAboutBox(mainFrame);
			aboutBox.setLocationRelativeTo(mainFrame);
		}
		aboutBox.setVisible(true);
	}

	@Action
	public void toggleEinstellungenBox(javax.swing.Action a) {
		toolEinstellungenBox.doClick();
	}
	
	@Action
	public void showEinstellungenBox(javax.swing.Action a) {
		showEinstellungenBox(a, EinstellungenBox.TABINTERFACE);
	}

	public void showEinstellungenBox(javax.swing.Action a, int reiter) {
		if (einstellungenBox == null) {
			JFrame mainFrame = GHMessen.getApplication().getMainFrame();
			einstellungenBox = new EinstellungenBox(mainFrame, al, ansicht, gl);
			einstellungenBox.setLocationRelativeTo(mainFrame);
			einstellungenBox.setKommentar(kommentar);
			einstellungenBox.addComponentListener(new ComponentAdapter(){
				@Override
				public void componentShown(ComponentEvent e) {
					javax.swing.Action a = actionMap.get("showEinstellungenBox");
					if(a != null){
						Boolean selected = (Boolean)a.getValue(javax.swing.Action.SELECTED_KEY);
						if(selected != null && !selected)
							a.putValue(javax.swing.Action.SELECTED_KEY, true);
					}
				}
				@Override
				public void componentHidden(ComponentEvent e) {
					javax.swing.Action a = actionMap.get("showEinstellungenBox");
					if(a != null){
						Boolean selected = (Boolean)a.getValue(javax.swing.Action.SELECTED_KEY);
						if(selected != null && selected)
							a.putValue(javax.swing.Action.SELECTED_KEY, false);
					}
				}
			});
		}
		if(a != null && a.getValue(javax.swing.Action.SELECTED_KEY) != null)
			einstellungenBox.setVisible((Boolean)a.getValue(javax.swing.Action.SELECTED_KEY));
		else
			einstellungenBox.setVisible(true);
		einstellungenBox.waehleKarteireiter(reiter);
	}

	@Action 
	public void toggleParameters(javax.swing.Action a) {
		if(toolParameters != null)
			toolParameters.doClick();
	}
	
	@Action
	public void showParameters(javax.swing.Action a) {
		if (parametersBox == null) {
			CursorTk.startWaitCursor(getComponent());
			JFrame mainFrame = GHMessen.getApplication().getMainFrame();
			if (gl.hatXRayGroesse())
				parametersBox = new XRayMessparameterBox(mainFrame, false, mp,
						al, gl);
			else
				parametersBox = new MessparameterBox(mainFrame, false, mp, al,
						gl);
			parametersBox.setLocationRelativeTo(mainFrame);
			parametersBox.addComponentListener(new ComponentAdapter(){
				@Override
				public void componentShown(ComponentEvent e){
					javax.swing.Action a = actionMap.get("showParameters");
					if(a != null){
						Boolean selected = (Boolean)a.getValue(javax.swing.Action.SELECTED_KEY);
						if(selected != null && !selected)
							a.putValue(javax.swing.Action.SELECTED_KEY, true);
					}
				}
				@Override
				public void componentHidden(ComponentEvent e){
					javax.swing.Action a = actionMap.get("showParameters");
					if(a != null){
						Boolean selected = (Boolean)a.getValue(javax.swing.Action.SELECTED_KEY);
						if(selected != null && selected)
							a.putValue(javax.swing.Action.SELECTED_KEY, false);
					}
				}
			});
			CursorTk.stopWaitCursor(getComponent());
		}
		if(a != null && a.getValue(javax.swing.Action.SELECTED_KEY) != null) {
			Boolean b = (Boolean)a.getValue(javax.swing.Action.SELECTED_KEY);
			parametersBox.setVisible(b);
		}
		else
			parametersBox.setVisible(true);
	}

	@Action
	public void showEigenschaften() {
		if (eigenschaftenBox == null) {
			JFrame mainFrame = GHMessen.getApplication().getMainFrame();
			eigenschaftenBox = new EigenschaftenBox(mainFrame, false,
					GHMessen.getApplication());
			eigenschaftenBox.setLocationRelativeTo(mainFrame);
		} else
			eigenschaftenBox.initialisieren();
		eigenschaftenBox.setVisible(true);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(GHMessenView.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(GHMessenView.class);
		appTitel = resourceMap.getString("Application.title");
		mainPanel = new javax.swing.JPanel();
		mainPanel.setSize(new Dimension(600, 400));
		mainPanel.setMinimumSize(new Dimension(600, 400));
		mainPanel.setPreferredSize(new Dimension(1000, 700));
		menuBar = new javax.swing.JMenuBar();
		statusPanel = new javax.swing.JPanel();
		statusMessageLabel = new javax.swing.JLabel();
		statusMessageLabel.setForeground(Color.BLUE);
		statusAnimationLabel = new javax.swing.JLabel();
		statusAnimationLabel.setSize(new Dimension(150, 0));
		statusAnimationLabel.setRequestFocusEnabled(false);
		statusAnimationLabel.setForeground(Color.RED);
		statusAnimationLabel.setFocusable(false);
		statusAnimationLabel.setFocusTraversalKeysEnabled(false);
		statusAnimationLabel.setBounds(new Rectangle(2, 2, 2, 2));
		statusAnimationLabel.setBorder(new BevelBorder(BevelBorder.LOWERED,
				null, null, null, null));
		statusAnimationLabel.setBackground(UIManager
				.getColor("ComboBox.background"));
		statusAnimationLabel.setFont(new Font("Dialog", Font.BOLD, 14));
		progressBar = new javax.swing.JProgressBar();

		// SELECTED_KEY-Werte für Actions müssen vordefiniert werden.
		actionMap.get("showEinstellungenBox").putValue(javax.swing.Action.SELECTED_KEY, false);
		actionMap.get("showParameters").putValue(javax.swing.Action.SELECTED_KEY, false);
		
		// Werkzeugleiste
		javax.swing.JButton einButton;

		toolBar = new javax.swing.JToolBar();
		toolBar.setLayout(new BoxLayout(toolBar, BoxLayout.LINE_AXIS));
		toolBar.setFloatable(false);
		toolBar.setRollover(true);

		einButton = new JButton(actionMap.get("dateiNeu"));
		toolBar.add(einButton);

		einButton = new javax.swing.JButton(actionMap.get("showLadenBox"));
		toolBar.add(einButton);

		einButton = new javax.swing.JButton(actionMap.get("showSpeichernBox"));
		toolBar.add(einButton);

		einButton = new javax.swing.JButton(
				actionMap.get("showSpeichernAlsBox"));
		toolBar.add(einButton);

		einButton = new javax.swing.JButton(actionMap.get("showPrintBox"));
		toolBar.add(einButton);

		toolMessen = new javax.swing.JToggleButton();
		toolMessen.setAction(actionMap.get("starteMessung"));
		toolMessen.addPropertyChangeListener("enabled",
				new PropertyChangeListener() {

					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						if (actionMap != null && actionMap.get("showParameters") != null)
							actionMap.get("showParameters").setEnabled((Boolean) evt
									.getNewValue());
					}

				});
		actionMap.get("starteMessung").setEnabled(false);
		toolBar.add(toolMessen);

		actionMap.get("showParameters").setEnabled(false);
		toolBar.add(toolParameters = new JToggleButton(actionMap.get("showParameters")));

		toolBar.add(toolEinstellungenBox = new JToggleButton(actionMap.get("showEinstellungenBox")));

		einButton = new javax.swing.JButton(actionMap.get("showAboutBox"));
		toolBar.add(einButton);

		toolAnzeigen = new javax.swing.JToggleButton(
				actionMap.get("setzeAnzeigeAlle"));
		toolAnzeigen.setSelected(true);
		toolBar.add(toolAnzeigen);

		toolBar.add(Box.createHorizontalStrut(20));

		tbMessgeraete = new MessgeraeteToolBar(messgeraete, toolAnzeigen);
		toolBar.add(tbMessgeraete);

		toolBar.add(Box.createHorizontalGlue());

		einButton = new JButton(actionMap.get("kopiereFenster"));
		toolBar.add(einButton);
		for (Component c : toolBar.getComponents())
			if (c instanceof JButton) {
				JButton b = (JButton) c;
				b.setHideActionText(true);
				b.setFocusable(false);
			} else if (c instanceof JToggleButton) {
				JToggleButton b = (JToggleButton) c;
				b.setHideActionText(true);
				b.setFocusable(false);
			}

		// Hauptfenster zusammenfügen
		setComponent(mainPanel);
		setMenuBar(menuBar);
		setStatusBar(statusPanel);
		setToolBar(toolBar);

		mainPanel.setName("mainPanel");
		mainPanel.setEnabled(false);

		darstellung = new MesswertePanel();
		darstellung.setVisible(false);
		actionMap.get("showPrintBox").setEnabled(false);

		javax.swing.GroupLayout gl_mainPanel = new javax.swing.GroupLayout(
				mainPanel);
		gl_mainPanel.setHorizontalGroup(gl_mainPanel.createParallelGroup(
				LEADING).addComponent(darstellung, PREFERRED_SIZE,
				PREFERRED_SIZE, Short.MAX_VALUE));
		gl_mainPanel.setVerticalGroup(gl_mainPanel.createParallelGroup(LEADING)
				.addComponent(darstellung, PREFERRED_SIZE, PREFERRED_SIZE,
						Short.MAX_VALUE));
		mainPanel.setLayout(gl_mainPanel);

		// Menüleiste
		javax.swing.JMenu einMenu;
		javax.swing.JMenuItem einMItem;

		// Datei-Menü
		einMenu = new JMenu(resourceMap.getString("fileMenu.text")); // NOI18N
		einMenu.setMnemonic(KeyStroke.getKeyStroke(
				resourceMap.getString("fileMenu.mnemonic")).getKeyCode());
		einMenu.add(new JMenuItem(actionMap.get("dateiNeu")));
		einMenu.add(new JSeparator());
		einMenu.add(menuRecentFiles = new JMenu(resourceMap.getString("recentFilesMenu.text")));
		updateRecentFilesMenu();
		einMenu.add(new JSeparator());
		einMenu.add(new JMenuItem(actionMap.get("showLadenBox")));
		einMenu.add(new JMenuItem(actionMap.get("showSpeichernBox")));
		einMenu.add(new JMenuItem(actionMap.get("showSpeichernAlsBox")));
		einMenu.add(new JSeparator());
		einMItem = new JMenuItem(actionMap.get("showPrintBox"));
		einMItem.setEnabled(false);
		einMenu.add(einMItem);
		einMenu.add(new JMenuItem(actionMap.get("showDruckerEinstellungenBox")));
		einMenu.add(new JSeparator());
		einMItem = new JMenuItem(actionMap.get("quit"));
		einMItem.setText(resourceMap.getString("exitMenuItem.text"));
		einMItem.setToolTipText(resourceMap
				.getString("exitMenuItem.toolTipText"));
		einMenu.add(einMItem);
		menuBar.add(einMenu);

		// Ansicht-Menü
		einMenu = new JMenu(resourceMap.getString("viewMenu.text"));
		einMenu.setMnemonic(KeyStroke.getKeyStroke(
				resourceMap.getString("viewMenu.mnemonic")).getKeyCode());
		itemAnzeigen = new JCheckBoxMenuItem(actionMap.get("setzeAnzeigeAlle"));
		itemAnzeigen.setSelected(toolAnzeigen.isSelected());
		einMenu.add(itemAnzeigen);
		menuBar.add(einMenu);

		// Messung-Menü
		einMenu = new JMenu(resourceMap.getString("measureMenu.text"));
		einMenu.setMnemonic(KeyStroke.getKeyStroke(
				resourceMap.getString("measureMenu.mnemonic")).getKeyCode());
		menuItemMessen = new JMenuItem("Messen");
		menuItemMessen.setAction(actionMap.get("starteMessung"));
		einMenu.add(menuItemMessen);
		einMenu.add(new JCheckBoxMenuItem(actionMap.get("showParameters")));
		menuBar.add(einMenu);

		// Einstellungen-Menü
		einMenu = new JMenu(resourceMap.getString("optionsMenu.text"));
		einMenu.setMnemonic(KeyStroke.getKeyStroke(
				resourceMap.getString("optionsMenu.mnemonic")).getKeyCode());
		einMenu.add(new JCheckBoxMenuItem(actionMap.get("showEinstellungenBox")));
		einMenu.add(new JSeparator());
		einMenu.add(new JMenuItem(actionMap.get("showEigenschaften")));
		menuBar.add(einMenu);

		// Hilfe-Menü
		einMenu = new JMenu(resourceMap.getString("helpMenu.text"));
		einMenu.setMnemonic(KeyStroke.getKeyStroke(
				resourceMap.getString("helpMenu.mnemonic")).getKeyCode());
		einMItem = new JMenuItem(actionMap.get("showHelpBox"));
		einMItem.setEnabled(GHMessen.hilfe != null);
		einMenu.add(einMItem);
		einMenu.add(new JSeparator());
		einMenu.add(new JMenuItem(actionMap.get("showAboutBox")));
		menuBar.add(einMenu);

		statusAnimationLabel.setHorizontalAlignment(SwingConstants.CENTER);

		progressBar.setValue(0);
		progressBar.setStringPainted(true);

		javax.swing.GroupLayout gl_statusPanel = new javax.swing.GroupLayout(
				statusPanel);
		statusPanel.setLayout(gl_statusPanel);
		gl_statusPanel.setHorizontalGroup(gl_statusPanel
				.createSequentialGroup()
				.addContainerGap()
				.addComponent(progressBar, PREFERRED_SIZE, PREFERRED_SIZE,
						PREFERRED_SIZE).addPreferredGap(UNRELATED)
				.addComponent(statusAnimationLabel, 200, 200, 200)
				.addPreferredGap(UNRELATED).addComponent(statusMessageLabel)
				.addContainerGap());
		gl_statusPanel.setVerticalGroup(gl_statusPanel
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_statusPanel.createParallelGroup(Alignment.CENTER)
								.addComponent(statusMessageLabel, 30, 30, 30)
								.addComponent(statusAnimationLabel, 30, 30, 30)
								.addComponent(progressBar, 30, 30, 30))
				.addContainerGap());

		initKeyBindings();
		getRootPane().getParent().setMinimumSize(new Dimension(800, 600));
		try {
			checkGroupMemberships();
		} catch (IOException e) {
			if(DEBUG)
				e.printStackTrace();
		}
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel mainPanel;
	private javax.swing.JMenuBar menuBar;
	private javax.swing.JProgressBar progressBar;
	private javax.swing.JLabel statusAnimationLabel;
	public javax.swing.JLabel statusMessageLabel;
	private javax.swing.JPanel statusPanel;
	private javax.swing.JToggleButton toolAnzeigen;
	private javax.swing.JToggleButton toolParameters;
	private javax.swing.JToggleButton toolEinstellungenBox;
	private javax.swing.JCheckBoxMenuItem itemAnzeigen;
	private javax.swing.JToolBar toolBar;
	private javax.swing.JToggleButton toolMessen;
	private javax.swing.JMenuItem menuItemMessen;
	private javax.swing.JMenu menuRecentFiles;
	public MesswertePanel darstellung;
	// End of variables declaration//GEN-END:variables
	// Eigene Variablen
	private JDialog aboutBox;
	public EinstellungenBox einstellungenBox;
	private MessparameterBox parametersBox;
	private EigenschaftenBox eigenschaftenBox;

	public void setEnableMessung(boolean enable) {
		actionMap.get("starteMessung").setEnabled(enable);
		actionMap.get("stoppeMessung").setEnabled(enable);
	}

	public boolean isMessungEnabled() {
		return actionMap.get("starteMessung").isEnabled()
				|| actionMap.get("stoppeMessung").isEnabled();
	}

	@Action
	public void dateiNeu() {
		JFrame mainFrame = GHMessen.getApplication().getMainFrame();
		if (JOptionPane.showConfirmDialog(mainFrame,
				"Alle Änderungen verwerfen", "Neu",
				JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE) != JOptionPane.OK_OPTION)
			return;
		datei = null;
		dateinameAlt = null;
		getFrame().setTitle(appTitel);
		setzeZurueck();
		showEinstellungenBox(null);
	}

	private void setzeZurueck() {
		if (laufendeMessung != null)
			stoppeMessung();
		mp.reset();
		if (einstellungenBox != null) {
			einstellungenBox.getCassyPanel().resetAll();
			einstellungenBox.setKommentar("");
		} else
			kommentar = "";
		gt.setNumRows(0);
		ansicht.clear();
		gl.clear();
		messgeraete.clear();
	}

	public void dateiLaden(File f) {
		try {
			CursorTk.startWaitCursor(this.getComponent());
			FORMAT format = ((AlleFileFilter) dateifilter[0]).getFormat(f);
			ObjectInputStream in = null;
			CL4File cl4 = null;
			GHMXFile.Reader ghmx = null;
			if (format == GHM) {
				in = new ObjectInputStream(new FileInputStream(f));
				String df = in.readUTF();
				if (!df.equals(DATEIVERSION)) {
					in.close();
					throw new FileFormatException(
							"<html>Das Dateiformat ist unbekannt oder veraltet(<b>"
									+ df
									+ "</b>) <br>an Stelle von(<b>"
									+ DATEIVERSION
									+ "</b>) und kann daher nicht eingelesen werden.</html>");
				}
			} else if (format == GHMX) {
				ghmx = new GHMXFile().new Reader(f);
				ghmx.parse();
			} else if (format == CL4) {
				cl4 = new CL4File();
				cl4.readFile(f);
			}
			setzeZurueck();
			Messparameter tmpGHMmp = null;
			if (format == GHM) {
				if (einstellungenBox != null)
					einstellungenBox.setKommentar(in.readUTF());
				else
					kommentar = (String) in.readUTF();
				gl.addAll((Groessenliste) in.readObject());
				tmpGHMmp = (Messparameter) in.readObject();
				WerteTabelle tmp = (WerteTabelle) in.readObject();
				gt.addRows(tmp.getDataVector());
				tmp = null;
			} else if (format == GHMX && ghmx != null) {
				if (einstellungenBox != null)
					einstellungenBox.setKommentar(ghmx.getKommentar());
				else
					kommentar = ghmx.getKommentar();
				gl.addAll(ghmx.getGroessenliste());
				gt.addRows(ghmx.getWerte());
			} else {
				if (einstellungenBox != null)
					einstellungenBox.setKommentar(cl4.getKommentar());
				else
					kommentar = cl4.getKommentar();
				for (int i = 0; i < cl4.getGroesseAnzahl(); i++) {
					Groesse g = Groesse.createFromCL4(gl, cl4.getGroesse(i),
							cl4.getGroessenReihe(i));
					gl.add(i, g);
				}
				gt.addRows(cl4.getWerteTabelle());
			}
			ansicht.clear(); // Standardansicht beseitigen
			if (format == GHM) {
				ansicht.addAll((Darstellungsliste) in.readObject());
				geraeteAnzeigeLaden(in);
				darstellung.setDividerLocation((Integer) in.readObject());
				mp.init(tmpGHMmp);
				in.close();
			} else if (format == GHMX) {
				mp.init(ghmx.getMessparameter());
				ansicht.addAll(ghmx.getDarstellungsliste(), gt);
				geraeteAnzeigeLaden(ghmx.sindMessgeraeteSichtbar(),
						ghmx.getMessgeraeteStatus());
				darstellung.setDividerLocation(ghmx.getTrenner());
			} else {
				mp.setFromCL4(gl, cl4.getMessparameter());
				for (int i = 0; i < cl4.getDarstellungAnzahl(); i++) {
					Darstellung d = new Darstellung(gt);
					d.setFromCL4(cl4.getDarstellung(i), cl4.isCL4());
					if ("Frequenzspektrum".equals(d.getName()))
						ansicht.addSpektrum(d);
					else if ("Histogramm".equals(d.getName()))
						ansicht.addHistogramm(d);
					else
						ansicht.add(d);
				}
				darstellung.setDividerLocation(cl4.getFensterTeilung());
				GHMessen.getApplication().ghmMittelZeit = cl4.getMittelZeit();
				for (int i = 0; i < cl4.getAuswertungsAnzahl(); i++) {
					ansicht.setFromCL4(cl4.getAuswertung(i));
				}
			}
			datei = f;

			if (format != GHMX) {
				String name = f.getPath();
				if (name.contains(".")
						&& !name.endsWith(((GHMXFileFilter) dateifilter[GHMXFILTER])
								.getEndung())) {
					name = name.substring(0, name.lastIndexOf("."));
					datei = new File(name);
				}
			}
			dateinameAlt = null;
			getFrame().setTitle(appTitel + " - " + datei.getName());
			GHMessen.appendToRecentFiles(datei.getAbsolutePath());
			updateRecentFilesMenu();
		} catch (FileFormatException e) {
			if (DEBUG)
				e.printStackTrace();
			JOptionPane.showMessageDialog(GHMessen.getApplication()
					.getMainFrame(),
					e.getMessage() + " Name(" + (f != null ? f.getName() : "")
							+ ")", "Falsches Dateiformat",
					JOptionPane.INFORMATION_MESSAGE);
		} catch (FileNotFoundException e) {
			if (DEBUG)
				e.printStackTrace();
			JOptionPane.showMessageDialog(GHMessen.getApplication()
					.getMainFrame(),
					e.getMessage() + " Name(" + datei.getName() + ")",
					"Datei nicht gefunden", JOptionPane.INFORMATION_MESSAGE);
		} catch (IOException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (ClassNotFoundException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (SAXException e) {
			if (DEBUG)
				e.printStackTrace();
		} finally {
			CursorTk.stopWaitCursor(this.getComponent());
		}
	}

	@Action
	public void showLadenBox() {
		JFrame mainFrame = GHMessen.getApplication().getMainFrame();
		JFileChooser ladenBox = new JFileChooser();
		ladenBox.setFileFilter(dateifilter[0]);
		for (int i = 1; i < dateifilter.length; i++)
			ladenBox.addChoosableFileFilter(dateifilter[i]);
		if (ladenBox.showOpenDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
			File f = ladenBox.getSelectedFile();
			if (!f.exists() || !f.canRead()) {
				JOptionPane
						.showMessageDialog(
								mainFrame,
								"Die Datei existiert nicht oder kann nicht gelesen werden.",
								"Öffnen", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
			dateiLaden(f);
		}
	}

	@Action
	public void showSpeichernAlsBox() {
		if (datei == null)
			showSpeichernBox();
		else {
			dateinameAlt = datei;
			datei = null;
			showSpeichernBox();
		}
	}

	@Action
	public void showSpeichernBox() {
		boolean neu = false;
		JFrame mainFrame = GHMessen.getApplication().getMainFrame();
		JFileChooser speichernBox = new JFileChooser();
		speichernBox.setFileFilter(dateifilter[GHMXFILTER]);
		speichernBox.addChoosableFileFilter(dateifilter[GHMFILTER]);
		if (datei != null
				|| speichernBox.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
			if (datei == null) {
				datei = speichernBox.getSelectedFile();
				neu = true;
			}
		} else {
			datei = dateinameAlt;
			return;
		}
		if (!datei.getName().endsWith(
				((GHMXFileFilter) dateifilter[GHMXFILTER]).getEndung())
				&& !datei.getName().endsWith(
						((GHMFileFilter) dateifilter[GHMFILTER]).getEndung()))
			datei = new File(datei.getPath()
					+ ((GHMXFileFilter) dateifilter[GHMXFILTER]).getEndung());
		try {
			if (neu && !datei.createNewFile() && datei.canWrite()) {
				if (JOptionPane
						.showConfirmDialog(
								mainFrame,
								"Die Datei existiert bereits. Soll die Datei überschrieben werden?",
								"Speichern", JOptionPane.OK_CANCEL_OPTION,
								JOptionPane.WARNING_MESSAGE) != JOptionPane.OK_OPTION) {
					datei = dateinameAlt;
					return;
				}
			}
			FORMAT format = ((AlleFileFilter) dateifilter[0]).getFormat(datei);
			if (format == GHM) {
				ObjectOutputStream out = new ObjectOutputStream(
						new FileOutputStream(datei));
				out.writeUTF(DATEIVERSION);
				if (einstellungenBox != null)
					out.writeUTF(einstellungenBox.getKommentar());
				else
					out.writeUTF(kommentar);
				out.writeObject(gl);
				out.writeObject(mp);
				TableModelListener[] sichereListe = gt.leereListenerListe();
				out.writeObject(gt);
				gt.fuelleListenerListe(sichereListe);
				out.writeObject(ansicht);
				geraeteAnzeigeSichern(out);
				out.writeObject(darstellung.getDividerLocation());
				out.close();
			} else if (format == GHMX) {
				GHMXFile.Writer outf = new GHMXFile().new Writer(datei);
				outf.writeXmlProlog();
				if (einstellungenBox != null)
					outf.writeKommentar(einstellungenBox.getKommentar());
				else
					outf.writeKommentar(kommentar);
				gl.writeToXml(outf.getTransformerHandler());
				mp.writeToXml(outf.getTransformerHandler());
				gt.writeToXml(outf.getTransformerHandler());
				ansicht.writeToXml(outf.getTransformerHandler());
				outf.writeMessgeraeteSichtbar(toolAnzeigen.isSelected());
				tbMessgeraete.writeToXml(outf.getTransformerHandler());
				outf.writeDividerLocation(darstellung.getDividerLocation());
				outf.writeXmlEpilog();
				outf.close();
			}
			getFrame().setTitle(appTitel + " - " + datei.getName());
			GHMessen.appendToRecentFiles(datei.getAbsolutePath());
			updateRecentFilesMenu();
		} catch (IOException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (SAXException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	@Action
	public void showPrintBox() {
		if (darstellung == null)
			return;
		JTable tabelle = darstellung.getSelectedTable();
		String titel = darstellung.getSelectedTableName();
		if (tabelle == null || titel == null)
			return;
		DruckTabelleDlg dlg = new DruckTabelleDlg(this.getFrame(), tabelle,
				"Tabelle " + titel);
		dlg.setLocationRelativeTo(this.getFrame());
		dlg.setVisible(true);
	}

	@Action
	public void showDruckerEinstellungenBox() {
		GHMessen app = GHMessen.getApplication();
		DruckerEinstellung dlg = new DruckerEinstellung(app.druckDienst,
				app.druckAttribut);
		if (dlg.zeigeDialog()) {
			app.druckDienst = dlg.getDruckDienst();
			app.druckAttribut = dlg.getDruckAttribut();
		}
	}

	private void geraeteAnzeigeSichern(ObjectOutputStream out)
			throws IOException {
		out.writeBoolean(toolAnzeigen.isSelected());
		out.writeObject(tbMessgeraete.saveStatusMap());
	}

	@SuppressWarnings("unchecked")
	private void geraeteAnzeigeLaden(ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		geraeteAnzeigeLaden(in.readBoolean(),
				(Map<String, MessgeraetStatus>) in.readObject());
	}

	private void geraeteAnzeigeLaden(boolean anzeigen,
			Map<String, MessgeraetStatus> geraete) {
		tbMessgeraete.restoreStatusMap(geraete);
		toolAnzeigen.setSelected(anzeigen);
		itemAnzeigen.setSelected(anzeigen);
		setzeAnzeigeAlle(null);
	}

	@Action
	public void fuehreMessungAus() {
		if (laufendeMessung == null) {
			starteMessung();
		} else {
			stoppeMessung();
		}
	}

	@Action
	public void starteMessung() {
		boolean hatOszilloskopKanal = gl.hatOszilloskopGroesse();
		boolean hatTimerKanal = gl.hatTimerGroesse();
		boolean istOszilloskopCassy = al.istOszilloskopVorhanden();
		setMessenSelected(true);
		if (darstellung != null)
			darstellung.setzeMessmodus(true);
		if (mp.isEinzelEin() && !mp.isTimerBox()) {
			CursorTk.startWaitCursor(this.getComponent());
			if (!mp.isAnhaengenEin() || messung == null) {
				try {
					if (mp.getModus() == Messparameter.MODUSXRAY)
						messung = new XRayTimerTask(gt, gl, mp, zeitgeber, al);
					else if (hatTimerKanal) {
						messung = new ZeitTimerTask(gt, gl, mp, zeitgeber, al);
						statusAnimationLabel.setText("Timer läuft ...");
					} else
						messung = new ZeitTimerTask(gt, gl, mp, zeitgeber, al);
					messung.init();
					messung.start();
				} catch (EAdapterException e) {
					if (DEBUG)
						e.printStackTrace();
				}
			}
			messung.setWiederhoung(1);
			messung.run();
			setMessenSelected(false);
			CursorTk.stopWaitCursor(this.getComponent());
			statusAnimationLabel.setText("");
		} else {
			try {
				CursorTk.startWaitCursor(this.getComponent());
				if (zeitgeber == null)
					zeitgeber = new ScheduledThreadPoolExecutor(2);
				if (mp.getModus() == Messparameter.MODUSXRAY) {
					messung = new XRayTimerTask(gt, gl, mp, zeitgeber, al);
					messung.init();
					messung.start();
					statusAnimationLabel.setText("XRay läuft ...");
					laufendeMessung = zeitgeber.scheduleAtFixedRate(messung,
							5000, 4000, TimeUnit.MILLISECONDS);
				} else if (al.istSchnelleOszilloskopMessung(mp) && istOszilloskopCassy) {
					if (!hatOszilloskopKanal)
						throw new EAdapterException(
								"Es ist kein Oszilloskop-Kanal definiert.");
					messung = new OszilloskopTimerTask(gt, gl, mp, zeitgeber,
							al);
					messung.init();
					messung.start();
					statusAnimationLabel.setText("Oszilloskop läuft ...");
					laufendeMessung = zeitgeber.scheduleAtFixedRate(messung,
							1000, 1000, TimeUnit.MILLISECONDS);
				} else {
					if (hatTimerKanal && mp.isTimerBox()) {
						messung = new AnzahlTimerTask(gt, gl, mp, zeitgeber, al);
						statusAnimationLabel.setText("Timer läuft ...");
					} else if (al.istSchnelleOszilloskopMessung(mp)) {
						throw new EAdapterException(
								"Das Messintervall ohne Oszilloskop muss mindestens 10 ms(seriell 100 ms) betragen.");
					} else
						messung = new ZeitTimerTask(gt, gl, mp, zeitgeber, al);
					messung.init();
					messung.start();
					statusAnimationLabel.setText("Messung läuft ...");
					if (messung instanceof AnzahlTimerTask)
						laufendeMessung = zeitgeber.scheduleAtFixedRate(
								messung, 0, 1000000, TimeUnit.MICROSECONDS);
					else
						laufendeMessung = zeitgeber.scheduleAtFixedRate(
								messung, 0, mp.getIntervallMS() * 1000,
								TimeUnit.MICROSECONDS);
				}
				CursorTk.stopWaitCursor(this.getComponent());
				progressBar.setEnabled(true);
				statusMessageLabel.setText("Die "
						+ (mp.isWiederholungEin() ? " wiederholende " : "")
						+ "Messung läuft...");
				progressBarTimer = zeitgeber.scheduleWithFixedDelay(
						new FortschrittTimer(), 100, 1000,
						TimeUnit.MILLISECONDS);
			} catch (EAdapterException e1) {
				JOptionPane.showMessageDialog(GHMessen.getApplication()
						.getMainFrame(), e1.getMessage(), "Messung",
						JOptionPane.ERROR_MESSAGE, null);
				setMessenSelected(false);
				if (darstellung != null)
					darstellung.setzeMessmodus(false);
				laufendeMessung = null;
				CursorTk.stopWaitCursor(this.getComponent());
				statusAnimationLabel.setText("");
			}
		}

	}

	@Action
	public void stoppeMessung() {
		if (darstellung != null)
			darstellung.setzeMessmodus(false);
		if (laufendeMessung != null) {
			laufendeMessung.cancel(false);
			laufendeMessung = null;
		}
		if (progressBarTimer != null) {
			progressBarTimer.cancel(false);
			progressBarTimer = null;
		}
		if (messung != null)
			messung.stop();
		statusMessageLabel.setText("");
		statusAnimationLabel.setText("");
		progressBar.setEnabled(false);
		progressBar.setValue(0);
		setMessenSelected(false);
	}

	public void entferneMesstask() {
		if (messung == null)
			return;
		if (laufendeMessung != null)
			stoppeMessung();
		messung = null;
	}

	private class FortschrittTimer implements Runnable {
		@Override
		public void run() {
			if (messung == null || messung.prozentFertig() >= 100) {
				stoppeMessung();
			} else {
				int prozent = messung.prozentFertig();
				progressBar.setValue(prozent);
				if (prozent > 0)
					statusAnimationLabel.setText("");
			}
		}

	}

	@Action
	public void setzeAnzeigeAlle(ActionEvent e) {
		if (messgeraete == null || tbMessgeraete == null)
			return;
		if (e == null || e.getSource() == toolAnzeigen) {
			// Knopf gedrückt
			itemAnzeigen.setSelected(toolAnzeigen.isSelected());
		}
		if (e == null || e.getSource() == itemAnzeigen) {
			// Menü ausgewählt
			toolAnzeigen.setSelected(itemAnzeigen.isSelected());
		}
		if (toolAnzeigen.isSelected())
			tbMessgeraete.setEnabled(true);
		else
			tbMessgeraete.setEnabled(false);
	}

	private void updateRecentFilesMenu() {
		menuRecentFiles.removeAll();
		if (GHMessen.recentFiles != null) {
			for(String s : GHMessen.recentFiles) {
				menuRecentFiles.add(new JMenuItem(new FileAction(s)));
			}
		}
	}
	
	class FileAction extends AbstractAction {
	    /**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		public FileAction(String text) {
	        super(text);
	    }
	    public void actionPerformed(ActionEvent e) {
			File f = new File((String) this.getValue(NAME));
			if (!f.exists() || !f.canRead()) {
				JOptionPane
						.showMessageDialog(
								GHMessen.getApplication().getMainFrame(),
								"Die Datei existiert nicht oder kann nicht gelesen werden.",
								"Öffnen", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
			dateiLaden(f);	
	    }
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsAdded(ObservableList list, int index, int length) {
		if (list instanceof Darstellungsliste) {
			for (int i = 0; i < length; i++)
				darstellung.addTabelle(ansicht.get(index + i));
		} else if (list instanceof Groessenliste) {
			setEnableMessung(true);
			darstellung.setVisible(true);
			actionMap.get("showPrintBox").setEnabled(true);
			if (parametersBox != null
					&& (parametersBox instanceof XRayMessparameterBox
							&& !gl.hatXRayGroesse() || !(parametersBox instanceof XRayMessparameterBox && gl
							.hatXRayGroesse()))) {
				parametersBox.setVisible(false);
				parametersBox = null;
			}
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsRemoved(ObservableList list, int index,
			List oldElements) {
		if (list instanceof Darstellungsliste) {
			for (int i = 0; i < oldElements.size(); i++)
				darstellung.removeTabelle((Darstellung) oldElements.get(i));
		} else if (list instanceof Groessenliste) {
			if (gl.getSize() < 1) {
				setEnableMessung(false);
				darstellung.getTabellen().removeAll();
				darstellung.setVisible(false);
				actionMap.get("showPrintBox").setEnabled(false);
				if (parametersBox != null) {
					parametersBox.setVisible(false);
					parametersBox = null;
				}
			} else {
				if (parametersBox != null) {
					if (parametersBox instanceof XRayMessparameterBox
							&& !gl.hatXRayGroesse()
							|| !(parametersBox instanceof XRayMessparameterBox && gl
									.hatXRayGroesse())) {
						parametersBox.setVisible(false);
						parametersBox = null;
					}
				}
			}

		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementReplaced(ObservableList list, int index,
			Object oldElement) {
		if (list instanceof Darstellungsliste) {
			darstellung.removeTabelle(index);
			darstellung.addTabelle(ansicht.get(index), index);
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementPropertyChanged(ObservableList list, int index) {
		// nichts zu tun
		return;
	}

	@Action
	public void kopiereFenster() {
		if (DEBUG)
			System.out
					.println("[CanvasMenu] - kopiereFenster wurde aufgerufen...");
		Component c = getComponent().getTopLevelAncestor();
		CursorTk.startWaitCursor(c);
		try {
			Robot robot = new Robot();
			if (c == null)
				return;
			Rectangle captureSize = new Rectangle(c.getLocationOnScreen(),
					c.getSize());
			BufferedImage bufferedImage = robot
					.createScreenCapture(captureSize);
			ImageTransferHandler.copyImageToClipboard(bufferedImage);
			if (DEBUG) {
				File f = new File("GHMessen_Screenshot.png");
				ImageIO.write(bufferedImage, "png", f);
			}
		} catch (AWTException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (IOException e) {
			if (DEBUG)
				e.printStackTrace();
		} finally {
			CursorTk.stopWaitCursor(c);
		}
	}

	protected void initKeyBindings() {
		// Globale Tastaturkürzel für das Hauptfenster, diese funktionieren
		// leider nicht in Dialogfenstern.
		iMap = getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		aMap = getRootPane().getActionMap();
		iMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0, false),
				"starteMessung");
		aMap.put("starteMessung", actionMap.get("starteMessung"));
		aMap.put("stoppeMessung", actionMap.get("stoppeMessung"));
		iMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0, false),
				"toggleEinstellungenBox");
		aMap.put("toggleEinstellungenBox", actionMap.get("toggleEinstellungenBox"));
		iMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0, false),
				"toggleParameters");
		aMap.put("toggleParameters", actionMap.get("toggleParameters"));
	}

}
