/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.rmi.registry.Registry;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;

public class Konstanten {
	private static final ResourceMap resourceMap;
	public static final String DATEIVERSION;
	public static final int TABELLE_SCHRIFT_KLEIN = 12;
	public static final int TABELLE_SCHRIFT_MITTEL = 30;
	public static final int TABELLE_SCHRIFT_GROSS = 60;
	public static final int TABELLE_SCHRIFT_STANDARD = TABELLE_SCHRIFT_KLEIN;

	public static final float GRAPH_LINIE_SCHMAL = 1.0f;
	public static final float GRAPH_LINIE_MITTEL = 2.0f;
	public static final float GRAPH_LINIE_BREIT = 5.0f;
	
	public static final Color FARBE_MESSGROESSE = Color.RED;
	public static final Color FARBE_KONSTANTE = Color.MAGENTA;
	public static final Color FARBE_FORMEL = new Color(186, 85, 211);

	/* Transparenz */
	public static final AlphaComposite TRANSPARENT = AlphaComposite
			.getInstance(AlphaComposite.SRC_OVER, 0.5f);

	/* Farben der Graphen */
	public static final class FARBE_GRAPHEN {
		private static final Color[] farbe = new Color[] { Color.black,
				Color.blue, Color.red, Color.green, Color.black, Color.blue,
				Color.red, Color.green };
		public static Color get(int index) {
			return farbe[-1 < index && index < 8 ? index : 0];
		}
		
		private static final int blackRBGA = -16777216;
		private static final int blueRBGA = -65536;
		private static final int greenRBGA = -16711936;
		private static final int redRBGA = -16776961;
		
		public static int toRGBA(Color c){
			return (c.getAlpha()<<24)+(c.getBlue()<<16)+(c.getGreen()<<8)+c.getRed();
		}
		
		public static Color next(Color c){
			switch(toRGBA(c)){
			default:
				return Color.black;
			case blackRBGA:
				return Color.blue;
			case blueRBGA:
				return Color.red;
			case redRBGA:
				return Color.green;
			case greenRBGA:
				return Color.black;
			}
		}
	}

	/* Virtuelle Größe der Werte im PlotPanel */
	public static final double GRAPH_AUSWAHLBEREICH = 10.0;

	/* Breite der transparenten Markierung */
	public static final float MARKIERUNG_BREITE = 12.0f;

	/* Virtuelle Breite einer Linie in der Auswertung */
	public static final int LINIENAUSWERTUNG_BREITE = 20;

	/* Positionierung der Messgeräte */
	public static final int MESSGERAETE_OFFSET = 40;
	public static final int MESSGERAETE_VERSCHIEBUNG_HORIZONTAL = 150;
	public static final int MESSGERAETE_VERSCHIEBUNG_VERTIKAL = 100;
	public static final int MESSGERAET_HOEHE = 360;
	public static final int MESSGERAET_BREITE = 240;

	/* Datums- und Zeitformate für die Verwendung in Formeln */
	public static final DateFormat DATUMSFORMAT = new SimpleDateFormat("d.M.y");
	public static final DateFormat ZEITFORMAT = new SimpleDateFormat("H:m:s");

	/* Standardwert für die Initialisierung einer Größe */
	public static final double GROESSE_MITTELZEIT_STANDARD = 0.1;

	/* Standarport für die RMI - Kommunikation */
	public static final int RMI_PORT = Registry.REGISTRY_PORT;

	/* Maximale Eingangsanzahl eines Cassy */
	public static final int CASSY_INPUT_ANZAHL = 4;

	/* Messgeräte ausblenden, wenn die Messung in zu kurzen Intervallen erfolgt. */
	public static final double AUSBLENDEN_INTERVALL = 0.5; /* s */

	/* Bei kleineren Messintervallen ist keine Echtzeitmessung möglich */
	public static final double MIN_ECHTZEIT_INTERVALL = 0.1; /* s */

	/* Standardeinstellung der dynamischen Skalierung */
	public static final boolean SKALIERUNG_DYNAMISCH_STANDARD = false;
	
	/* Standardmesszeit in Sekunden */
	public static final double STANDARD_MESSZEIT = 120.0;

	static {
		resourceMap = Application.getInstance(ghmessen.GHMessen.class)
				.getContext().getResourceMap(ghmessen.GHMessen.class);
		DATEIVERSION = resourceMap.getString("Application.title")
				+ resourceMap.getString("Application.fileversion");
	}
}
