/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen;

import static ghmessen.Konstanten.TABELLE_SCHRIFT_STANDARD;
import ghmessen.components.ColorTableHeaderCellRenderer;
import ghmessen.components.PlotPanel;
import ghmessen.components.TabellenPane;
import ghmessen.dialoge.einstellungen.darstellung.YDialog;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Darstellungsliste;
import ghmessen.util.Util;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

/**
 * 
 * @author fschuett
 * 
 */
public class MesswertePanel extends javax.swing.JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -74595075613001592L;
	public static final boolean DEBUG = false;

	private TableModelListener autoScroller;
	private JTable scrollTable;
	private long zaehler;
	private static final int INTERVALL = 2000; /* ms */

	/** Creates new form MesswertePanel */
	public MesswertePanel() {
		scrollTable = null;
		zaehler = System.currentTimeMillis();
		autoScroller = new TableModelListener() {

			@Override
			public void tableChanged(TableModelEvent e) {
				if ((System.currentTimeMillis() - zaehler) <= INTERVALL)
					return;
				if (scrollTable != null
						&& e.getType() == TableModelEvent.INSERT
						&& e.getFirstRow() > TableModelEvent.HEADER_ROW) {
					zaehler = System.currentTimeMillis();
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (scrollTable != null)
								scrollTable.scrollRectToVisible(scrollTable
										.getCellRect(
												scrollTable.getRowCount() - 1,
												0, true));
						}
					});
				}
			}

		};

		initComponents();
	}

	private void initComponents() {

		darstellung = new javax.swing.JSplitPane();
		darstellung.setResizeWeight(0.3);
		darstellung.setOneTouchExpandable(true);
		setTabellen(new EditableTabbedPane());

		setName("Form"); // NOI18N

		darstellung.setDividerLocation(200);
		darstellung.setName("jcMessung"); // NOI18N

		getTabellen().setName("jcWerte");

		darstellung.setLeftComponent(getTabellen());

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				darstellung, javax.swing.GroupLayout.DEFAULT_SIZE, 747,
				Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				darstellung, javax.swing.GroupLayout.DEFAULT_SIZE, 524,
				Short.MAX_VALUE));

		plotPanel = new PlotPanel(tabellen);
		darstellung.setRightComponent(plotPanel);
	}// </editor-fold>//GEN-END:initComponents

	public void setzeMessmodus(boolean an) {
		if (plotPanel != null)
			plotPanel.setzeMessmodus(an);
		if (an && scrollTable != null) {
			scrollTable.getModel().removeTableModelListener(autoScroller);
			scrollTable = null;
		}
		if (an) {
			if (tabellen.getSelectedIndex() != -1
					&& tabellen.getSelectedComponent() instanceof TabellenPane) {
				TabellenPane tab = (TabellenPane) tabellen
						.getSelectedComponent();
				scrollTable = tab.getTabelle();
				scrollTable.getModel().addTableModelListener(autoScroller);

			}
		} else if (scrollTable != null) {
			scrollTable.getModel().removeTableModelListener(autoScroller);
			scrollTable = null;
		}
	}

	public void setTabellen(javax.swing.JTabbedPane tabellen) {
		this.tabellen = tabellen;
	}

	public javax.swing.JTabbedPane getTabellen() {
		return tabellen;
	}

	public void addTabelle(Darstellung tab) {
		tabellen.add(new TabellenPane(tab));
	}

	public void addTabelle(Darstellung tab, int index) {
		tabellen.add(new TabellenPane(tab), index);
	}

	public void removeTabelle(Darstellung tab) {
		Component[] clist = tabellen.getComponents();
		for (Component c : clist) {
			if (c instanceof TabellenPane) {
				TabellenPane tp = (TabellenPane) c;
				if (tp.getTabelle().getModel().equals(tab)) {
					tabellen.remove(c);
					break;
				}
			}
		}
	}

	public void removeTabelle(int index) {
		if (index < tabellen.getComponentCount()) {
			Component c = tabellen.getComponent(index);
			if (c instanceof TabellenPane) {
				tabellen.remove(c);
			}
		}
	}

	public void removeTabelle(Darstellung tab, int index) {
		Component c = tabellen.getComponent(index);
		if (c instanceof TabellenPane) {
			TabellenPane tp = (TabellenPane) c;
			if (tp.getTabelle().getModel().equals(tab))
				tabellen.remove(c);
		}
	}

	public void setDividerLocation(int pos) {
		if (darstellung != null)
			darstellung.setDividerLocation(pos);
	}

	public int getDividerLocation() {
		return (darstellung == null ? 200 : darstellung.getDividerLocation());
	}

	public JTable getSelectedTable() {
		if (tabellen == null
				|| !(tabellen.getSelectedComponent() instanceof TabellenPane))
			return null;
		JTable tab = ((TabellenPane) tabellen.getSelectedComponent())
				.getTabelle();
		JTable copy = new JTable();
		Font font = new Font("Dialog", Font.PLAIN, TABELLE_SCHRIFT_STANDARD);
		copy.setFont(font);
		copy.getTableHeader().setDefaultRenderer(
				new ColorTableHeaderCellRenderer());
		copy.setRowHeight(getFontMetrics(font).getHeight());
		copy.getTableHeader().setFont(font);
		copy.setModel(tab.getModel());
		return copy;
	}

	public String getSelectedTableName() {
		if (tabellen == null)
			return "";
		return tabellen.getSelectedComponent().getName();
	}

	private javax.swing.JSplitPane darstellung;
	private javax.swing.JTabbedPane tabellen;
	private PlotPanel plotPanel;

	class EditableTabbedPane extends JTabbedPane {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private TabMenue popup;
		
		public EditableTabbedPane(){
			super();
			TabTitleEditListener ttel = new TabTitleEditListener(this);
			addChangeListener(ttel);
			addMouseListener(ttel);
			popup = new TabMenue(this);
			MouseListener popupListener = new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					if (!e.isPopupTrigger())
						return;
					popup.show((Component) e.getSource(), e.getX(), e.getY());
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					if (!e.isPopupTrigger())
						return;
					popup.show((Component) e.getSource(), e.getX(), e.getY());
				}
			};
			addMouseListener(popupListener);
		}

		@Override
		public void setTitleAt(int index, String title) {
			super.setTitleAt(index, title);
			Component c = getComponentAt(index);
			if(c instanceof TabellenPane){
				TabellenPane tp = (TabellenPane)c;
				tp.setName(title);
			}
		}

		class TabMenue extends JPopupMenu implements ActionListener {
			/**
			 * 
			 */
			private static final long serialVersionUID = 4906247320272300488L;

			private JMenuItem darstellungNeu, darstellungEntfernen, 
						darstellungEigenschaften;
			private EditableTabbedPane darstellungen;
			private int index = -1;
			
			public TabMenue(EditableTabbedPane darstellungen) {
				super();
				this.darstellungen = darstellungen;
				darstellungNeu = new JMenuItem("Neu");
				darstellungNeu.addActionListener(this);
				add(darstellungNeu);
				darstellungEntfernen = new JMenuItem("Entfernen");
				darstellungEntfernen.addActionListener(this);
				add(darstellungEntfernen);
				darstellungEigenschaften = new JMenuItem("Eigenschaften");
				darstellungEigenschaften.addActionListener(this);
				add(darstellungEigenschaften);
			}

			@Override
			public void actionPerformed(ActionEvent e) {
				Object src = e.getSource();
				Darstellungsliste ansicht = GHMessen.getApplication().getMainView().ansicht;
				if (src == darstellungNeu) {
					String neu = "Standard";
					neu = Util.findUniqueName(ansicht, neu);
					Darstellung ds = new Darstellung(GHMessen.getApplication()
							.getMainView().gt);
					ds.setName(neu);
					ansicht.add(ds);
				} else if (src == darstellungEntfernen) {
					Component c = darstellungen.getComponentAt(index);
					if(c instanceof TabellenPane && 
							((TabellenPane)c).getTabelle().getModel() instanceof Darstellung){
						Darstellung ds = (Darstellung)((TabellenPane)c).getTabelle().getModel();
						ansicht.remove(ds);
					}
				} else if (src == darstellungEigenschaften) {
					Component c = darstellungen.getComponentAt(index);
					if(c instanceof TabellenPane &&
							((TabellenPane)c).getTabelle().getModel() instanceof Darstellung){
						Darstellung ds = (Darstellung)((TabellenPane)c).getTabelle().getModel();
						YDialog dialog = new YDialog(ds);
						dialog.setVisible(true);
					}
				}
			}
			
			@Override
			public void show(Component invoker, int x, int y) {
				index = darstellungen.indexAtLocation(x,y);
				darstellungEntfernen.setEnabled(index != -1);
				darstellungEigenschaften.setEnabled(index != -1);
				super.show(invoker,x,y);
			}
		}

		class TabTitleEditListener extends MouseAdapter implements ChangeListener {
		    private final JTextField editor = new JTextField();
		    private final JTabbedPane tabbedPane;
		    private int editingIdx = -1;
		    private int len = -1;
		    private Dimension dim;
		    private Component tabComponent;

		    public TabTitleEditListener(final JTabbedPane tabbedPane) {
		        super();
		        this.tabbedPane = tabbedPane;
		        editor.setBorder(BorderFactory.createEmptyBorder());
		        editor.addFocusListener(new FocusAdapter() {
		            @Override public void focusLost(FocusEvent e) {
		                renameTabTitle();
		            }
		        });
		        editor.addKeyListener(new KeyAdapter() {
		            @Override public void keyPressed(KeyEvent e) {
		                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
		                    renameTabTitle();
		                } else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
		                    cancelEditing();
		                } else {
		                    editor.setPreferredSize(editor.getText().length() > len ? null : dim);
		                    tabbedPane.revalidate();
		                }
		            }
		        });
		        tabbedPane.getInputMap(JComponent.WHEN_FOCUSED).put(
		            KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "start-editing");
		        tabbedPane.getActionMap().put("start-editing", new AbstractAction() {
		            /**
					 * 
					 */
					private static final long serialVersionUID = 1L;

					@Override public void actionPerformed(ActionEvent e) {
		                startEditing();
		            }
		        });
		    }
		    @Override public void stateChanged(ChangeEvent e) {
		        renameTabTitle();
		    }
		    @Override public void mouseClicked(MouseEvent me) {
		        Rectangle rect = tabbedPane.getUI().getTabBounds(tabbedPane, tabbedPane.getSelectedIndex());
		        if (rect != null && rect.contains(me.getPoint()) && me.getClickCount() == 2) {
		            startEditing();
		        } else {
		            renameTabTitle();
		        }
		    }
		    private void startEditing() {
		        editingIdx = tabbedPane.getSelectedIndex();
		        tabComponent = tabbedPane.getTabComponentAt(editingIdx);
		        tabbedPane.setTabComponentAt(editingIdx, editor);
		        editor.setVisible(true);
		        editor.setText(tabbedPane.getTitleAt(editingIdx));
		        editor.selectAll();
		        editor.requestFocusInWindow();
		        len = editor.getText().length();
		        dim = editor.getPreferredSize();
		        editor.setMinimumSize(dim);
		    }
		    private void cancelEditing() {
		        if (editingIdx >= 0) {
		            tabbedPane.setTabComponentAt(editingIdx, tabComponent);
		            editor.setVisible(false);
		            editingIdx = -1;
		            len = -1;
		            tabComponent = null;
		            editor.setPreferredSize(null);
		            tabbedPane.requestFocusInWindow();
		        }
		    }
		    private void renameTabTitle() {
		        String title = editor.getText().trim();
		        if(editingIdx >= 0 && !title.isEmpty() && 
		        		GHMessen.getApplication().getMainView().ansicht.indexOfName(title) < 0) {
					tabbedPane.setTitleAt(editingIdx, title);
		        }
		        cancelEditing();
		    }
		}
	}

}
