/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter;

import static ghmessen.groesse.Einheit.Keine;
import static ghmessen.groesse.Messbereich.Invalid;

import java.awt.image.ImageObserver;

import ghmessen.Konstanten;
import ghmessen.components.JTransparentToggleButton;
import ghmessen.groesse.Einheit;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.groesse.Messgroesse;
import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetStandard;
import ghmessen.util.Util;

import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.Vector;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;

import ld.api.TimerEdges;
import ld.api.TimerModes;
import ld.api.ios.RelaySource;

public abstract class Eingang implements IEingang, ImageObserver {

	private static final boolean DEBUG = false;

	private String symbolName = null;

	protected boolean hatTimer;
	protected boolean hatOszilloskop;
	protected boolean hatTemperaturKorrektur;
	protected boolean hatVorzeichenWechsel;
	protected IAdapter adapter;
	protected int input;
	protected Vector<Messbereich> messbereich;
	protected int auswahlB;
	protected int auswahlBStandard;
	protected IMesskanal messKanal;
	protected Vector<Messgroesse> messGroesse;
	protected int[] kanaele;

	protected String[] prefix;
	protected String[] suffix;

	// Standard-Messparameter des Eingangs
	protected double intervallStandard;
	protected double intervallMinimum;
	protected int wiederholungStandard;

	protected static BufferedImage imgSensor, imgSensor_seitlich,
			imgSchwarzerKnopf, imgRoterKnopf, imgBlauerKnopf;

	public Eingang(IAdapter adapter, int input) {
		this.hatTimer = false;
		this.hatOszilloskop = true;
		this.hatTemperaturKorrektur = false;
		this.hatVorzeichenWechsel = false;
		this.adapter = adapter;
		this.input = input;
		this.auswahlBStandard = 0;
		this.auswahlB = auswahlBStandard;
		this.messbereich = null;
		this.messKanal = null;
		this.messGroesse = null;
		this.prefix = null;
		this.suffix = null;
		this.intervallStandard = 0.1;
		this.intervallMinimum = 0.01;
		this.wiederholungStandard = 125;
	}

	@Override
	public void close() {
		return;
	}
	
	public double getIntervallStandard() {
		return this.intervallStandard;
	}

	public int getWiederholungStandard() {
		return this.wiederholungStandard;
	}

	public double getIntervallMinimum() {
		return this.intervallMinimum;
	}

	public String getName() {
		return "keine Sensorbox";
	}

	public String getEigenschaftenTitel() {
		return "Eigenschaften";
	}

	public int getNummer() {
		return 0;
	}

	public boolean hatTimer() {
		return hatTimer;
	}

	public boolean hatOszilloskop(int kanal) {
		return hatOszilloskop;
	}

	public boolean hatKorrektur(int kanal) {
		return messKanal == null ? false : messKanal.hatKorrektur();
	}

	public void korrigiereNullpunkt(int kanal) {
		if (messKanal != null)
			messKanal.korrigiereNullpunkt();
	}

	public boolean hatVorzeichenWechsel(int kanal) {
		return hatVorzeichenWechsel;
	}

	public boolean hatLED(int kanal) {
		return false;
	}

	public void schalteLED(int kanal) {
		return;
	}
	
	/*************************************************
	 * Messgroessen
	 *************************************************/

	public int getAuswahl() {
		return 0;
	}

	public void setAuswahl(int auswahl) {
		auswahlB = auswahlBStandard;
		return;
	}

	public void setAuswahl(Messgroesse auswahl) {
		return;
	}

	public int getMessgroessenAnzahl(int kanal) {
		return 1;
	}

	public int getMessgroesseIndex(int kanal) {
		return 0;
	}

	public String[] getNamen(int kanal) {
		return new String[] { 
				getMessgroesse(kanal).getName() == null ?  
						getMessgroesse(kanal).getSymbol()
						: getMessgroesse(kanal).getName()
				};
	}

	public String getSuffix(int messGroesse) {
		return "";
	}

	public Messgroesse getMessgroesse(int kanal) {
		if (messGroesse == null || messGroesse.isEmpty())
			return Messgroesse.Nichts;
		else
			return messGroesse.get(0);
	}

	public String[] getGroessenStringList(int kanal) {
		return new String[] { (prefix == null || prefix[0] == null ? ""
				: prefix[0] + " ")
				+ (getMessgroesse(kanal).getName() == null ? ""
				: getMessgroesse(kanal).getName() + " " )
				+ erzeugeSymbol(0, input)
				+ (suffix == null || suffix[0] == null ? "" : " " + suffix[0]) };
	}

	/*************************************************
	 * Messbereiche
	 *************************************************/

	public boolean hatMessbereiche() {
		return true;
	}

	public int getAuswahlB(int kanal) {
		return this.auswahlB;
	}

	public void setAuswahlB(int kanal, int auswahlB) {
		this.auswahlB = auswahlB;
	}

	public Messbereich getMessbereich(int kanal) {
		if (messbereich == null || auswahlB < 0
				|| auswahlB >= messbereich.size())
			return Invalid;
		else
			return messbereich.get(auswahlB);
	}

	public Messbereich getMessbereich(int kanal, int auswahl) {
		if (messbereich == null || auswahl < 0 || auswahl >= messbereich.size())
			return Invalid;
		else
			return messbereich.get(auswahl);
	}

	public Messbereich[] getMessbereiche(int kanal) {
		return messbereich.toArray(new Messbereich[] {});
	}

	// Bisher nur in USBox verwendet, bei setAuswahlB wird dort kanal
	// verworfen.
	public void setMessbereich(Messbereich m) {
		if (messbereich == null || !messbereich.contains(m)) {
			System.err.println("[Eingang] - setMessbereich: Messbereich(" + m
					+ ") nicht gefunden.");
			return;
		}
		setAuswahlB(-1, messbereich.indexOf(m));
	}

	public Einheit getEinheit(int kanal) {
		if (messbereich == null || messbereich.size() < 1)
			return Keine;
		else
			return messbereich.get(auswahlB).getEinheit();
	}

	public double getSensorBoxValue(double x, double y) {
		return getSensorBoxValue(x);
	}

	public double getSensorBoxValue(double x) {
		return messbereich.get(auswahlB).getFaktor() * x;
	}

	public String[] getBereicheStringList(int kanal) {
		return Messbereich.getBereicheStringList(messbereich);
	}

	public Messbereich getTemperaturBereich(int kanal) {
		return Invalid;
	}

	/*************************************************
	 * IAdapter
	 *************************************************/
	public void setAdapter(IAdapter adapter) {
		this.adapter = adapter;
	}

	public IAdapter getAdapter() {
		return adapter;
	}

	public int getInput() {
		return input;
	}

	public static int getInput(String symbol) {
		if (symbol == null || symbol.length() < 2 || symbol.indexOf("_") < 0)
			return -1;
		// Mobile- / XRay- / XRay2- / Pocket-Cassy haben nur einen Eingang mit
		// der Nummer 0.
		if (symbol.indexOf("_") == symbol.length() - 2)
			return 0;
		int input = symbol.charAt(symbol.length() - 2) - 'A';
		if (input < 0 || input > Konstanten.CASSY_INPUT_ANZAHL - 1)
			return -1;
		return input;
	}

	public void setInput(int input) {
		this.input = input;
	}

	public String getInputString() {
		return getInputString(input);
	}

	public String getInputString(int input) {
		return adapter == null ? "-1" : adapter.getEingangString(input) + adapter.getNr();
	}

	public JTransparentToggleButton createToggleButton(Rectangle bounds,
			Action a) {
		auswahlB = auswahlBStandard;
		return new JTransparentToggleButton(bounds, a);
	}

	public Messgeraet getMessgeraet(JFrame parent, Groesse g,
			ScheduledThreadPoolExecutor hauptTimer) {
		return new MessgeraetStandard(parent, getMesskanal(g, hauptTimer, true), adapter, g, hauptTimer);
	}

	public abstract IMesskanal getMesskanal(Groesse g, ScheduledThreadPoolExecutor timer, boolean autocreate);

	public void setzeStandardWerte(Groesse g) {
		auswahlB = auswahlBStandard;
		if (g == null) {
			return;
		}
		g.setMessGroesse(getMessgroesse(g.getEF()));
		if (messbereich != null && auswahlB < messbereich.size()) {
			g.setMessbereich(messbereich.get(auswahlB));
			g.getMessbereich().setzeStandardWerte(g);
			if (DEBUG)
				System.out
						.println("[Eingang] - setzeStandardWerte: Messbereich("
								+ g.getMessbereich() + ") Einheit("
								+ g.getEinheitString() + ")");
		}
		g.setIntervall(100);
		g.setMweinheit(Einheit.ms);
		g.setFaktor(1.0);
		g.setOffset(0.0);
	}

	public String erzeugeSymbol(int index, int input, int kanal) {
		return erzeugeSymbol(index, input);
	}

	public String erzeugeSymbol(int index, int input) {
		return getMessgroesse(0).getSymbol() + "_" + getInputString(input);
	}

	public int getKanaele() {
		return kanaele == null ? 1 : kanaele[0];
	}

	public String getKanalsymbol(int auswahl, int kanal) {
		return "";
	}

	public String getKanalsymbol(int kanal) {
		return "";
	}

	public int getSensorDatenAdresse() {
		return 0;
	}

	/******************************************
	 * Timer-Methoden
	 ******************************************/
	public TimerModes getTimerMode() {
		return TimerModes.Off;
	}

	public TimerEdges getTimerEdgeE() {
		return TimerEdges.Off;
	}

	public TimerEdges getTimerEdgeF() {
		return TimerEdges.Off;
	}

	public double getGateTime() {
		return 0.0;
	}

	public int getTimerCount() {
		return 0;
	}

	public boolean hatTimerArray() {
		return false;
	}

	public boolean istRelaySource() {
		return false;
	}
	
	public RelaySource getRelaySourceIO() {
		return null;
	}
	
	public boolean contains(Point p) {
		return false;
	}
	
	public JPopupMenu getPopupMenu() {
		return null;
	}
	
	/******************************************
	 * Image-Methoden
	 ******************************************/
	public int imageXOffset() {
		return 0;
	}

	public String getSymbolName() {
		if (symbolName == null) {
			symbolName = this.getClass().getName();
			if (symbolName.indexOf(".") > -1)
				symbolName = symbolName
						.substring(symbolName.lastIndexOf(".") + 1);
		}
		return symbolName;
	}

	/******************************************
	 * Interface ImageObserver
	 ******************************************/
	public boolean imageUpdate(Image img, int infoflags, int x, int y,
			int width, int height) {
		return true;
	}

	public ImageIcon getImageIcon(boolean aktiv) {
		URL url = this.getClass().getResource(
				"resources/" + getSymbolName() + (aktiv ? "_aktiv" : "")
						+ ".png");
		if (url == null)
			url = Eingang.class.getResource("resources/Eingang"
					+ (aktiv ? "_aktiv" : "") + ".png");
		if (url != null)
			return new ImageIcon(url);
		else
			return null;
	}

	static {
		imgSensor = null;
		URL url = Eingang.class.getResource("resources/Sensor.png");
		if (url != null)
			imgSensor = Util.toBufferedImage(new ImageIcon(url).getImage());
		imgSensor_seitlich = null;
		url = Eingang.class.getResource("resources/Sensor_seitlich.png");
		if (url != null)
			imgSensor_seitlich = Util.toBufferedImage(new ImageIcon(url)
					.getImage());
		imgSchwarzerKnopf = null;
		url = Eingang.class.getResource("resources/Sensor_SchwarzerKnopf.png");
		if (url != null)
			imgSchwarzerKnopf = Util.toBufferedImage(new ImageIcon(url)
					.getImage());
		imgRoterKnopf = null;
		url = Eingang.class.getResource("resources/Sensor_RoterKnopf.png");
		if (url != null)
			imgRoterKnopf = Util.toBufferedImage(new ImageIcon(url).getImage());
		imgBlauerKnopf = null;
		url = Eingang.class.getResource("resources/Sensor_BlauerKnopf.png");
		if (url != null)
			imgBlauerKnopf = Util
					.toBufferedImage(new ImageIcon(url).getImage());
	}
}
