/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter;

import ghmessen.adapter.event.AdapterAddedEvent;
import ghmessen.adapter.event.AdapterChangedEvent;
import ghmessen.adapter.event.AdapterEventListener;
import ghmessen.adapter.event.AdapterRemovedEvent;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.ObservableObjectListListener;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.List;

import org.jdesktop.observablecollections.ObservableList;

public interface IAdapter extends
		AdapterEventListener, ObservableObjectListListener {
	public void close();

	public int getVendorID();
	public HashSet<Integer> getProductID();
	public String[] getSubsystem();
	public String[] getDeviceGroup();
	public double getIntervallStandard();
	public int getWiederholungStandard();
	public double getIntervallMinimum();
	public IEingang getEingang(int ab);
	public IEingang getEingang(Groesse g);
	public String getEingangString(int aB);
	public int getEingangAnzahl();
	public boolean istSensorEingang(int ab);
	public abstract void initGroessen();
	public String getDevice();
	public String getName();
	public Groessenliste getGroessenliste();
	public int getNr();
	public void setNr(int i);
	public Component getComponent();
	public IMesskanal getMesskanal(Groesse g);

	/*********
	 * spezielle CASSY-Methoden
	 */
	public Class<?> getKlasse();
	public void reset();
	public String getSchnittstelle();

	public void deaktiviereGroesse(Groesse g);
	public void aktiviereGroesse(Groesse g);
	public void setzeStandardWerte(Groesse g, int AB, int ef, boolean neu);
	public void handleSymbolEvent(PropertyChangeEvent evt);
	public void handleArtEvent(PropertyChangeEvent evt);
	public boolean isProductID(int id);
	
	public void addPropertyChangeListener(PropertyChangeListener listener);
	public void removePropertyChangeListener(PropertyChangeListener listener);
	
	public void adapterAdded(AdapterAddedEvent evt);
	public void adapterRemoved(AdapterRemovedEvent evt);
	public void adapterChanged(AdapterChangedEvent evt);
	
	public void listElementPropertyChanged(
			@SuppressWarnings("rawtypes") ObservableList list, int index);
	public void listElementPropertyChanged(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			PropertyChangeEvent e);
	public void listElementReplaced(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			Object oldElement);
	public void listElementsAdded(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			int length);
	public void listElementsRemoved(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			@SuppressWarnings("rawtypes") List oldElements);
}
