package ghmessen.adapter;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;
import java.util.Arrays;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.FutureTask;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

public class MessInterfaceReader implements Callable<byte[]> {
	private static final boolean DEBUG = false;
	InputStream in;
	int bufferSize = 256;

	public MessInterfaceReader(FileChannel f){
		this.in = Channels.newInputStream(f);
	}
	
	public MessInterfaceReader(FileInputStream in) {
		this.in = in;
	}

	@Override
	public byte[] call() throws Exception {
		if(DEBUG)
			System.out.println("[MessInterfaceReader] - call: start");
		byte[] result = new byte[bufferSize];
		int read = in.read(result);
		if (read <= 0)
			result = null;
		else if (result.length > read)
			result = Arrays.copyOfRange(result, 0, read);
		return result;
	}
	
	public void close() throws IOException{
		if(in != null)
			in.close();
		in = null;
		THREADPOOL.shutdown();
	}

	private static final ExecutorService THREADPOOL = Executors
			.newCachedThreadPool();

	public int read(byte[] buffer, int offset, int length, long timeOut,
			TimeUnit timeUnit) throws InterruptedException, ExecutionException, TimeoutException{
		if(DEBUG)
			System.out.println("[MessInterfaceReader] - read: start");
		byte[] result = read(timeOut, timeUnit);
		if(result == null || result.length == 0 || buffer == null || buffer.length == 0 || offset >= buffer.length || offset+length > buffer.length)
			return 0;
		if(result.length > length){
			System.arraycopy(result, 0, buffer, offset, length);
			return length;
		} 
		else {
			System.arraycopy(result, 0, buffer, offset, result.length);
			return result.length;
		}
	}
	
	public byte[] read(long timeOut, TimeUnit timeUnit)
			throws InterruptedException, ExecutionException, TimeoutException {
		FutureTask<byte[]> t = new FutureTask<byte[]>(this);
		THREADPOOL.execute(t);
		return t.get(timeOut, timeUnit);
	}

}
