package ghmessen.adapter;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;
import java.util.Arrays;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.FutureTask;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

public class MessInterfaceWriter implements Callable<Integer> {
	private static final boolean DEBUG = false;
	OutputStream out;
	int bufferSize = 256;
	int length = 0;
	byte[] buffer;
	
	public MessInterfaceWriter(FileChannel f){
		out = Channels.newOutputStream(f);
		buffer = new byte[bufferSize];
	}
	
	public MessInterfaceWriter(FileOutputStream out) {
		this.out = out;
		buffer = new byte[bufferSize];
	}

	@Override
	public Integer call() throws Exception {
		try {
			out.write(buffer);
			out.flush();
			return length;
		} catch(IOException e){
			return 0;
		}
	}
	
	public void close() throws IOException{
		if(out != null)
			out.close();
		out = null;
		THREADPOOL.shutdown();
	}

	private static final ExecutorService THREADPOOL = Executors
			.newCachedThreadPool();

	public Integer write(byte[] outReport, int offset, int length, long timeOut, TimeUnit timeUnit)
			throws InterruptedException, ExecutionException, TimeoutException {
		this.length = Math.min(length, buffer.length);
		System.arraycopy(outReport, offset, buffer, 0, this.length);
		if(DEBUG)
			System.out.println(Arrays.copyOfRange(buffer, 0, this.length));
		FutureTask<Integer> t = new FutureTask<Integer>(this);
		THREADPOOL.execute(t);
		return t.get(timeOut, timeUnit);
	}

}
