/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter;


import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;

public abstract class Messkanal implements IMesskanal {
	
	private static final boolean DEBUG = false;
	
	protected Groesse g;
	protected IEingang eingang;
	protected boolean ueberschritten;
	protected boolean unterschritten;
	protected boolean oszilloskopKanal;
	private boolean timerArray;
	protected double d;
	protected boolean gueltig;
	protected boolean korrektur;
	protected double korrekturWert;
	protected double abstand;
	protected boolean gestartet;
	
	public Messkanal(IEingang eingang, Groesse g) {
		if(DEBUG)
			System.out.println("[Messkanal] - Konstruktur: Groesse("+g+")");
		this.eingang = eingang;
		this.g = g;
		eingang = null;
		ueberschritten = false;
		unterschritten = false;
		oszilloskopKanal = false;
		timerArray = false;
		korrektur = false;
		d = 0.0;
		gueltig = false;
		korrekturWert = 0.0;
		abstand = 0.1;
		gestartet = false;
	}

	public abstract double einlesen() throws MesskanalException;
	
	public double einlesenZeit() throws MesskanalException {
		throw new MesskanalException("["
				+this.getClass().getName()
				+"] Der Messkanal ist kein Zeitkanal.");
	}

	public double leseLetztenWert() throws MesskanalException {
		if(!gueltig)
			throw new MesskanalException("["
					+this.getClass().getName()
					+"] Der Messkanal enthält keinen gültigen Wert.");
		return d;
	}
	
	public int fuellePuffer(int bisher, int neu) throws InterruptedException, EAdapterException {
		throw new EAdapterException("[Messkanal] - fuellePuffer: Ungültige Messgröße("+g.getMessGroesse()+")");
	}
	
	public double lesePufferWert(int index) throws InterruptedException, EAdapterException {
		throw new EAdapterException("[Messkanal] - lesePufferWert: Ungültige Messgröße("+g.getMessGroesse()+")");
	}
	
	public double lesePufferZeit(int index) throws InterruptedException, EAdapterException {
		throw new EAdapterException("[Messkanal] - lesePufferZeit: Ungültige Messgröße("+g.getMessGroesse()+")");
	}
	
	public void finalize(){
		return;
	}
	
	public Groesse getGroesse(){
		return g;
	}
	
	public boolean istGueltig(){
		return gueltig;
	}
	
	public boolean istUeberschritten(){
		return ueberschritten;
	}
	
	public boolean istUnterschritten(){
		return unterschritten;
	}
	
	public boolean istKanalOffen(){
		return true;
	}
	
	public boolean istOszilloskopKanal(){
		return oszilloskopKanal;
	}
	
	public boolean istOszilloskopZeitKanal(){
		return false;
	}

	public boolean istLangsamerZeitKanal() {
		return false;
	}
	
	public boolean istTriggerKanal(){
		return false;
	}
	
	public void setTimerArray(boolean timerArray) {
		this.timerArray = timerArray;
	}

	public boolean hatTimerArray() {
		return timerArray;
	}
	
	public IEingang getEingang(){
		return eingang;
	}

	public boolean hatKorrektur(){
		return false;
	}
	
	@Override
	public boolean hatVorzeichenWechsel() {
		return false;
	}
	
	@Override
	public void wechseleVorzeichen() { }
	
	public Messbereich[] getMessbereiche(){
		return null;
	}
	
	@Override
	public Messbereich getMessbereich(){
		return Messbereich.Invalid;
	}
	
	@Override
	public String[] getMessbereicheStringArray(){
		return new String[]{};
	}
	
	public void korrigiereNullpunkt(){
		this.korrekturWert = 0.0;
	}
	
	public void setAbstand(double abstand) throws MesskanalException { this.abstand = abstand; }
	
	public double getAbstand() throws MesskanalException { 
		return abstand;
	}
	
}
