package ghmessen.adapter.arduino;

import ghmessen.adapter.Adapter;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.Eingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.components.JTransparentToggleButton;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.util.CursorTk;
import ghmessen.util.Util;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.HashSet;

import javax.swing.ImageIcon;
import javax.swing.JLabel;

import org.jdesktop.application.Action;

import udev.UdevDevice;

public class AdapterArduino extends Adapter implements AutoCloseable {
	private static final long serialVersionUID = -4682510751525364535L;
	private static final boolean DEBUG = false;

	private static final Rectangle[] rectA = new Rectangle[] { new Rectangle(
			5, 5, 79, 174) };

	public static final int vendorID;
	public static final HashSet<Integer> productID;
	public static final String[] subsystem;
	public static final String[] deviceGroup;
	private BufferedImage aktivImg;

	protected EingangArduino eingangArduino;
	
	public AdapterArduino(int nr, UdevDevice device, Groessenliste liste) 
			throws EAdapterException {
		super(new ImageIcon(
				AdapterArduino.class
						.getResource("resources/arduino.png")), nr,
				device, rectA, liste);
		if(DEBUG)
			System.out.println("[AdapterArduino] Konstruktor...");
		adjustImage();
	}

	@Override
	public IMesskanal getMesskanal(Groesse g) {
		return eingangArduino.getMesskanal(g, null, true);
	}

	@Override
	public void close() {
		if(eingangArduino != null)
			eingangArduino.close();
	}

	@Override
	public String getEingangString(int aB) {
		return "";
	}

	@Override
	public int getEingangAnzahl() {
		return 1;
	}

	@Override
	public boolean istSensorEingang(int ab) {
		return false;
	}

	@Override
	public String getName() {
		return "Arduino";
	}

	@Override
	public void setzeStandardWerte(Groesse g, int AB, int ef, boolean neu) {
		g.setNr(nr);
		g.setAB(AB);
		g.setEF(ef);
	}

	@Override
	protected void inputClicked(int aB, JTransparentToggleButton b) {
		if (aB >= 1)
			return;
		CursorTk.startWaitCursor(b);
		if (b.isSelected()) { // Eingang aktivieren
			eingangArduino.setAuswahl(0);
			Groesse g;
			String name = erzeugeSymbol(0, 0, 0);
			g = Util.findeGroesse(liste, name);
			if (g == null) {
				g = new Groesse(name);
				liste.add(g);
				setzeStandardWerte(g, 0, 0, true);
			} else {
				setzeStandardWerte(g, 0, 0, false);
			}
			aktiviereGroesse(g);
			aktiv[0] = true;
			setInputText(0, 0, g.getSymbol());
		} else { // Eingang deaktivieren
			deaktiviereInput(0);
		}
		adjustImage();
		CursorTk.stopWaitCursor(b);
	}
	
	@Action
	public void clickA() {
		inputClicked(0, inputButton[0]);
	}

	@Override
	protected void adjustImage() {
		super.adjustImage();
		if (aktiv[0])
			setImage(aktivImg);
	}

	protected void initComponents() {

		super.initComponents();
		setLayout(null);

		add(inputButton[0]);

		schnittLabel.setForeground(Color.RED);
		schnittLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		schnittLabel.setBounds(12, 1, 79, 15);
		add(schnittLabel);

		inputLabel[0].setForeground(Color.BLACK);
		inputLabel[0].setFont(new Font("Dialog", Font.BOLD, 16));
		inputLabel[0].setBounds(12, 102, 49, 82);
		add(inputLabel[0]);

		aktivImg = Util
				.toBufferedImage(new ImageIcon(AdapterArduino.class
						.getResource("resources/arduino_aktiv.png"))
						.getImage());

	}

	@Override
	protected void initEingaenge() throws EAdapterException {
		eingaenge = getEingangAnzahl();
		eingangArduino = new EingangArduino(this, device, 0);
		sb = new Eingang[] { eingangArduino };
		super.initEingaenge();
	}

	protected void initButtonActions() {
		buttonActions = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(this.getClass(), this);
	}

	protected void initInputLabels() {
		inputLabel[0] = new JLabel("<html>ungültig</html>");
	}

	@Override
	protected void initButtons() {
		inputButton[0] = eingangArduino.createToggleButton(rect[0],
					buttonActions.get("clickA"));
	}

	public void initGroessen() {
		if (DEBUG)
			System.out
					.println("[AdapterArduino] - initGroessen wurde aufgerufen.");
		eingangArduino.setAuswahl(0);
		aktiviereInput(0, 0, 0, false);
		if (aktiv[0])
			inputButton[0].setSelected(true);
		else
			eingangArduino.setAuswahl(0);
		this.revalidate();
		adjustImage();
	}

	public int getKanaele(int abxy) {
		if (abxy >= eingaenge)
			return 0;
		else
			return 1;
	}

	public String erzeugeSymbol(int auswahl, int abxy, int kanal) {
		if (abxy >= eingaenge)
			return "";
		else
			return eingangArduino.erzeugeSymbol(auswahl, abxy, kanal);
	}

	static {
		vendorID = 0x2341;
		productID = new HashSet<Integer>() {
			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			{
				add(0x0001);
				add(0x0043); // Arduino/SainSmart
				add(0x0243); // Arduino/Genuino
			}
		};
		subsystem = new String[] { "tty" };
		deviceGroup = new String[] { "ttyACM*" };
	}

}
