package ghmessen.adapter.arduino;

import java.util.Arrays;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.JFrame;

import ghmessen.adapter.IEingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.Messkanal;
import ghmessen.adapter.MesskanalException;
import ghmessen.adapter.seriell.SerialAdapter;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetStandard;
import ghmessen.instrumente.SensorPanel.SensorBoxPanel;
import static ghmessen.groesse.Einheit.m;

public class MesskanalArduino extends Messkanal {

	private static final boolean DEBUG = false;
	SerialAdapter arduino;
	Messbereich messbereich;
	private static Messbereich[] messbereiche;
	private static final int MAXLENGTH = 100;
	private byte[] buffer;
	private String sbuffer;
	double messwert;
	boolean gueltig;

	public MesskanalArduino(IEingang eingang, Groesse g, SerialAdapter arduino) {
		super(eingang, g);
		if(DEBUG)
			System.out.println("[MesskanalArduino] Konstruktor ...");
		this.arduino = arduino;
		if (arduino != null) {
			messbereich = messbereiche[0];
			g.setMessbereich(messbereich);
			buffer = new byte[MAXLENGTH];
			sbuffer = "";
			gueltig = false;
		}
	}

	@Override
	public int getMessbereichIndex() {
		return 0;
	}

	//Es gibt nur einen Messbereich
	@Override
	public void setMessbereich(int index) { }

	@Override
	public void setMessbereich(Messbereich messbereich) { }

	@Override
	public Messgeraet createMessgeraet(JFrame parent, IMesskanal messkanal, ScheduledThreadPoolExecutor timer) {
		return new MessgeraetStandard(parent, messkanal, eingang.getAdapter(), g, timer);
	}

	@Override
	public SensorBoxPanel getPanel(IMesskanal messkanal) {
		return new SensorBoxPanel(g, eingang.getAdapter(), messkanal);
	}

	static {
		messbereiche = new Messbereich[] { new Messbereich(1, 1.0, 0, 4.0, 0.05, 2, m, "0 .. 4m") };
	}

	@Override
	public double einlesen() throws MesskanalException {
		boolean vollstaendig;
		int gelesen;
		try {
			gelesen = arduino.read(buffer, MAXLENGTH);
			sbuffer = sbuffer + new String(Arrays.copyOfRange(buffer,0,gelesen));
			vollstaendig = sbuffer.endsWith("\r\n");
			String[] zeilen = sbuffer.split("\r\n");
			for(int index=0;index<zeilen.length-1;index++)
				werteAus(zeilen[index]);
			if(vollstaendig) {
				werteAus(zeilen[zeilen.length-1]);
				sbuffer = "";
			}
			else {
				sbuffer = zeilen[zeilen.length-1];
			}
		} catch (InterruptedException e) {
			throw new MesskanalException("[MesskanalArduino] - einlesen() ungültig.");
		}
		if(!gueltig)
			throw new MesskanalException("[MesskanalArduino] - einlesen() ungültig.");
		return messwert;
	}

	private void werteAus(String zeile) {
		try {
			messwert = Integer.parseInt(zeile.substring(0, zeile.indexOf(" cm"))) / 100.0;
			gueltig = true;
			if(DEBUG)
				System.out.println("[MesskanalArduino] werteAus("+zeile+"): "+messwert);
		} catch (NumberFormatException|StringIndexOutOfBoundsException e) {
			if(DEBUG)
				System.err.println("[MesskanalArduino] werteAus("+zeile+") ist keine gültige Zahl!");
			gueltig = false;
		}
	}

	// Es werden ständig Messwerte gelesen, daher ist start/stop nicht sinnvoll.
	@Override
	public void start() throws MesskanalException {	}

	@Override
	public void stop() throws MesskanalException { }

}
