package ghmessen.adapter.cassy;

import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IAuswahl;
import ghmessen.adapter.IEingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.event.AdapterAddedEvent;
import ghmessen.adapter.event.AdapterChangedEvent;
import ghmessen.adapter.event.AdapterRemovedEvent;
import ghmessen.groesse.Art;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.util.CursorTk;
import ghmessen.util.ProxyUtil;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import ld.api.CASSY;
import ld.api.CASSYModule;
import ld.api.IFunctionGenerator;
import ld.api.Pids;
import ld.api.UsbSerial;
import ld.api.events.CASSYQuantityClickEvent;
import ld.api.events.CASSYQuantityClickListener;
import ld.api.ios.CASSYIO;
import ld.api.ios.CASSYIOs;

import org.jdesktop.observablecollections.ObservableList;

import udev.UdevDevice;

public class AdapterCASSY implements InvocationHandler, IAdapter, Serializable, CASSYQuantityClickListener, MouseListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -1235584737045879347L;

	private static final boolean DEBUG = false;

	public static final int vendorID;
	public static final HashSet<Integer> productID;
	public static final String[] subsystem;
	public static final String[] deviceGroup;
	private CASSYComponent component;
	private JPopupMenu popup;
	protected ArrayList<IEingang> eingang;
	int nr;
	Groessenliste gl;
	UdevDevice d;

	private AdapterCASSY(CASSY cassy, int nr, UdevDevice d, Groessenliste gl)
			throws EAdapterException {
		this.nr = nr;
		this.gl = gl;
		if(this.gl != null)
			this.gl.addObservableListListener(this);
		this.d = d;
		component = new CASSYComponent(this, cassy, cassy.getCASSYs().getCASSYIOs());
		component.addCASSYQuantityClickListener(this);
		component.addMouseListener(this);
		eingang = new ArrayList<IEingang>();
		if (DEBUG)
			System.out.println("[AdapterCASSY] - Konstruktor:\n"+
					"\tcassy("+cassy+")\n"
					+"\tnr("+nr+") device("+d+")\n"
					+"\tthis("+component+")\n"
					+"\tgl("+this.gl+")\n");
	}

	public static IAdapter create(CASSY cassy, int nr, UdevDevice d,
			Groessenliste gl) throws IllegalArgumentException,
			EAdapterException {
		ArrayList<Class<?>> interfaces = new ArrayList<Class<?>>();
		Class<?> dieseKlasse = cassy.getClass();
		ProxyUtil.sammleIF(interfaces, dieseKlasse);
		ProxyUtil.addIF(interfaces, IAdapter.class);
		ProxyUtil.addIF(interfaces, Serializable.class);
		ProxyUtil.addIF(interfaces, CASSYQuantityClickListener.class);
		ProxyUtil.addIF(interfaces, MouseListener.class);
		AdapterCASSY proxyd = new AdapterCASSY(cassy, nr, d, gl);
		IAdapter proxy = (IAdapter)Proxy.newProxyInstance(cassy.getClass()
				.getClassLoader(), interfaces.toArray(new Class<?>[] {}),
				proxyd);
		for (CASSYIO io : proxyd.component.getCASSYIOs().getItemsOf(cassy)) {
			proxyd.eingang.add(EingangCASSY.create(io, proxy));
		}
		return proxy;
	}

	@Override
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {
		Object result = null;
		try {
			if (method.getDeclaringClass().equals(IAdapter.class))
				result = method.invoke(this, args);
			else
				result = method.invoke(component.getCASSY(), args);
		} catch (InvocationTargetException e) {
			throw e.getTargetException();
		} catch (Exception e) {
			throw new RuntimeException("unexpected invocation exception "
					+ e.getMessage());
		}
		return result;
	}

	@Override
	public Class<?> getKlasse() {
		return component.getCASSY().getClass();
	}

	@Override
	public void initGroessen() {
	}

	@Override
	public void close() {
		if (DEBUG)
			System.out.println("[Adapter] - close: CASSY(" + this + ")");
		for(IEingang e : eingang)
			e.close();
		component.getCASSY().close();
	}

	@Override
	public IMesskanal getMesskanal(Groesse g) {
		IMesskanal result = null;
		for(IEingang e : eingang)
			if((result = e.getMesskanal(g, null, true)) != null)
				return result;
		return null;
	}
	
	@Override
	public Component getComponent() {
		return component;
	}

	public class CASSYComponent extends CASSYModule {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		/**
		 * 
		 */
		private AdapterCASSY adapter;

		public AdapterCASSY getAdapter() {
			return adapter;
		}

		public CASSYComponent(AdapterCASSY adapter, CASSY cassy, CASSYIOs cassyios) {
			super(cassy, cassyios);
			this.adapter = adapter;
		}

	}

	@Override
	public void setzeStandardWerte(Groesse g, int AB, int ef, boolean neu) {
		g.setNr(nr);
		g.setAB(AB);
		g.setEF(ef);
	}

	static {
		vendorID = UsbSerial.getVendorId();
		subsystem = new String[] { "usb", "usbmisc", "tty" };
		deviceGroup = new String[] { "ldusb*", "ldusb*", "ttyUSB*" };
		productID = new HashSet<Integer>() {
			private static final long serialVersionUID = 1L;

			{
				for (Pids pid : Pids.values())
					add(pid.getValue());
			}
		};

	}

	@Override
	public String getEingangString(int aB) {
		if (aB < eingang.size())
			return eingang.get(aB).toString();
		else
			return "";
	}

	@Override
	public String getName() {
		return component.getCASSY().toString();
	}

	@Override
	public int getEingangAnzahl() {
		if (component.getCASSYIOs() == null)
			return 0;
		return component.getCASSYIOs().getItemsOf(component.getCASSY()).size();
	}

	@Override
	public boolean istSensorEingang(int ab) {
		return ab < 2;
	}

	@Override
	public int getVendorID() {
		return vendorID;
	}

	@Override
	public HashSet<Integer> getProductID() {
		return productID;
	}

	@Override
	public String[] getSubsystem() {
		return subsystem;
	}

	@Override
	public String[] getDeviceGroup() {
		return deviceGroup;
	}

	@Override
	public double getIntervallStandard() {
		return component.getCASSY().getMeanTime();
	}

	@Override
	public int getWiederholungStandard() {
		return 250;
	}

	@Override
	public double getIntervallMinimum() {
		return component.getCASSY().getMinIntervalTime();
	}

	@Override
	public IEingang getEingang(int ab) {
		if (ab < eingang.size())
			return eingang.get(ab);
		else if(ab == 1 && component.getCASSY() instanceof IFunctionGenerator)
			return eingang.get(0);
		else
			return null;
	}

	@Override
	public IEingang getEingang(Groesse g) {
		for(IEingang e : eingang)
			if(e.getMesskanal(g, null, false) != null)
				return e;
		return null;		
	}
	@Override
	public String getDevice() {
		return d.getName();
	}

	@Override
	public Groessenliste getGroessenliste() {
		return gl;
	}
	
	@Override
	public int getNr() {
		return nr;
	}

	@Override
	public void setNr(int i) {
		this.nr = i;
	}

	@Override
	public void reset() {
		component.getCASSY().resetCASSY();
	}

	@Override
	public String getSchnittstelle() {
		return d.getNode();
	}

	@Override
	public void deaktiviereGroesse(Groesse g) {
		g.setArt(Art.Parameter);
	}

	@Override
	public void aktiviereGroesse(Groesse g) {
		g.setArt(Art.Messgroesse);
	}

	@Override
	public void handleSymbolEvent(PropertyChangeEvent evt) {
		return;
	}

	@Override
	public void handleArtEvent(PropertyChangeEvent evt) {
		return;
	}

	@Override
	public boolean isProductID(int id) {
		return productID.contains(id);
	}

	@Override
	public void adapterAdded(AdapterAddedEvent evt) { }

	@Override
	public void adapterRemoved(AdapterRemovedEvent evt) {
		if(eingang != null)
			for(IEingang e : eingang)
				if(e instanceof IAuswahl)
					((IAuswahl)e).setAuswahl(-1);
		if(DEBUG)
			System.out.println("[AdapterCASSY] - adapterRemoved(index: "+evt.getName());
	}

	@Override
	public void adapterChanged(AdapterChangedEvent evt) {
		if(evt.nr != nr)
			return;
		CursorTk.startWaitCursor(component);
		if (evt.alt != null)
			evt.alt.close();
		eingang.set(evt.ab,evt.neu);
		CursorTk.stopWaitCursor(component);
	}

	@Override
	public void listElementPropertyChanged(@SuppressWarnings("rawtypes") ObservableList list, int index) { }

	@Override
	public void listElementPropertyChanged(@SuppressWarnings("rawtypes") ObservableList list, int index,
			PropertyChangeEvent e) {
		if(DEBUG)
			System.out.println("[AdapterCASSY] - listElementPropertyChanged(index:"+index+",p:"
					+ e.getPropertyName()+")");		
	}

	@Override
	public void listElementReplaced(@SuppressWarnings("rawtypes") ObservableList list, int index,
			Object oldElement) { }

	@Override
	public void listElementsAdded(@SuppressWarnings("rawtypes") ObservableList list, int index, int length) {
		if (!(list instanceof Groessenliste) || component == null)
			return;
		// nach dem Einlesen einer Groessenliste müssen die Interfaceobjekte
		// aktualisiert werden.
		Groesse g;
		for (int i = index; i < index + length; i++) {
			g = ((Groessenliste) list).get(i);
			if (g.getArt() != Art.Messgroesse || g.getNr() != nr)
				continue;
			if(eingang.size() <= g.getAB() || !(eingang.get(g.getAB()) instanceof IAuswahl))
				continue;
			IAuswahl auswahl = (IAuswahl)eingang.get(g.getAB());
			if(!auswahl.setAuswahl(g.getSymbol()))
				g.setArt(Art.Parameter);
		}
		if(DEBUG)
			System.out.println("[AdapterCASSY] - listElementsAdded(index:"+index+",length:"+length+")");
	}

	@Override
	public void listElementsRemoved(@SuppressWarnings("rawtypes") ObservableList list, int index,
			@SuppressWarnings("rawtypes") List oldElements) { }

	@Override
	public void addPropertyChangeListener(PropertyChangeListener listener) { }

	@Override
	public void removePropertyChangeListener(PropertyChangeListener listener) { }

	@Override
	public void cassyquantityClick(CASSYQuantityClickEvent e) {
		System.out.println("Die CASSY-Quantity "+e.getSource()+ " wurde angeklickt.");
	}

	/************************************************************
	 * MouseListener
	 ************************************************************/
	@Override
	public void mouseClicked(MouseEvent e) { }

	@Override
	public void mousePressed(MouseEvent e) {
		if (SwingUtilities.isRightMouseButton(e)) {
			if (e.isPopupTrigger())
				for(IEingang eg : eingang)
					if(eg.contains(new Point(e.getX(),e.getY()))) {
						popup = eg.getPopupMenu();
						if(popup != null)
							popup.show((Component) e.getSource(), e.getX(), e.getY());
						e.consume();
					}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (SwingUtilities.isRightMouseButton(e)) {
			if (e.isPopupTrigger())
				for(IEingang eg : eingang)
					if(eg.contains(new Point(e.getX(),e.getY()))) {
						popup = eg.getPopupMenu();
						if(popup != null)
							popup.show((Component) e.getSource(), e.getX(), e.getY());
						e.consume();
					}
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) { }

	@Override
	public void mouseExited(MouseEvent e) { }
}
