/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter.cassy;

import ghmessen.adapter.Adapter;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.Eingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.components.JTransparentToggleButton;
import ghmessen.dialoge.EinstellungenBox;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.Messgroesse;
import ghmessen.util.CursorTk;
import ghmessen.util.HTML;
import ghmessen.util.Util;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;

import javax.swing.ImageIcon;

import ld.api.CASSYException;
import ld.api.CASSYStates;
import ld.api.IXRay;
import ld.api.XRay;
import ld.api.XRay2;
import ld.api.XRayData;
import ld.api.XRayModes;
import ld.api.XRayPosition;

import org.jdesktop.application.Action;

import udev.UdevDevice;

public class AdapterXRay extends Adapter implements IXRay {

	private static final long serialVersionUID = -2821307668384192568L;

	public static final boolean DEBUG = false;

	public static final int VID_LD = 0x0F11; // USB Vendor ID of LD Didactic
	public static final int vendorID;
	public static final int PID_XRAY = 0x1100; // X-Ray Apparatus
	public static final int PID_XRAY2 = 0x1101; // X-Ray Apparatus
	public static final int PID_XRAYCT = 0x1110; // USB Product ID for CT
													// Accessory
	public static final String[] subsystem;
	public static final String[] deviceGroup;
	public static final HashSet<Integer> productID;

	private XRay xray;

	EinstellungenBox eBox = null;
	private static final Rectangle[] rectA = new Rectangle[] { new Rectangle(0,
			0, 99, 248) };
	private BufferedImage aktivImg;
	public static Messgroesse[] groessen;
	public static String[] namen;
	public static String[] symbole;
	public static HashMap<String, Double> gitterAbstaende;
	public static String STANDARDGITTER = "564,0 pm (NaCl)";
	public static HashMap<String, Double> energieUmrechnungen;
	public static final int STANDARDANODE = 3;

	public AdapterXRay(int nr, UdevDevice device, Groessenliste liste)
			throws EAdapterException {
		super(new ImageIcon(
				AdapterXRay.class.getResource("resources/AdapterXRay.png")),
				nr, device, rectA, liste);
		try {
			xray = new XRay();
			boolean opened = false;
			if (xray.open())
				opened = true;
			if (!opened) {
				xray = new XRay2();
				if (xray.open())
					opened = true;
			}
			if (!opened)
				throw new EAdapterException("device is neither XRay nor XRay2");
		} catch (IOException e1) {
			if (DEBUG)
				e1.printStackTrace();
			xray = null;
			throw new EAdapterException(e1.getLocalizedMessage());
		}
		for (Eingang e : sb)
			e.setAdapter(this);
		if (DEBUG)
			System.out.println("[" + this.getClass().toString()
					+ "] - Konstruktor aufgerufen...");
	}

	public IXRay getXRay() {
		return xray;
	}

	public void setXRay(XRay xray) {
		this.xray = xray;
	}

	@Override
	public boolean istSensorEingang(int ab) {
		return false;
	}

	@Override
	protected void initEingaenge() throws EAdapterException {
		eingaenge = 1;
		sb = new Eingang[1];
		sb[0] = new EingangXRay(this, 0);
		super.initEingaenge();
	}

	@Override
	public void initGroessen() {
		aktiviereInput(0, 0, 0, false);
		if (aktiv[0])
			inputButton[0].setSelected(true);
		this.revalidate();
		adjustImage();
	}

	@Override
	protected void initButtons() {
		inputButton[0] = new JTransparentToggleButton(rect[0],
				buttonActions.get("clickA"));
	}

	protected void initComponents() {
		super.initComponents();
		setLayout(null);

		aktivImg = Util.toBufferedImage(new ImageIcon(AdapterXRay.class
				.getResource("resources/AdapterXRay_aktiv.png")).getImage());

		add(inputButton[0]);

		schnittLabel.setForeground(Color.BLACK);
		schnittLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		schnittLabel.setBounds(12, 1, 79, 15);
		add(schnittLabel);

		inputLabel[0].setForeground(Color.BLACK);
		inputLabel[0].setFont(new Font("Dialog", Font.BOLD, 16));
		inputLabel[0].setBounds(12, 90, 49, 82);
		add(inputLabel[0]);

	}// </editor-fold>

	protected void showBox(int abxy, Eingang box, boolean show, boolean aktiv) {
		BufferedImage img = Util.toBufferedImage(getImage());
		String symbol = inputLabel[0].getText();
		symbol = HTML.enthaeltHTML(symbol) ? HTML.htmlZuSymbol(symbol) : symbol;
		BufferedImage boxImg;
		if (show)
			boxImg = Util.toBufferedImage(box.getImageIcon(aktiv).getImage());
		else
			boxImg = null;
		int boxX = 0;
		int boxY = 0;
		img.getGraphics().drawImage(boxImg, boxX, boxY, this);
		this.setImage(img);
	}

	protected void adjustImage() {
		super.adjustImage();
		if (aktiv[0]) {
			BufferedImage img = Util.toBufferedImage(getImage());
			img.getGraphics().drawImage(aktivImg, 0, 0, this);
			this.setImage(img);
		}
		showLabel(0, aktiv[0]);
	}

	@Action
	public void clickA(ActionEvent e) {
		if (e.getSource() instanceof JTransparentToggleButton) {
			JTransparentToggleButton src = (JTransparentToggleButton) e
					.getSource();
			inputClicked(0, src);
		}
	}

	protected void inputClicked(int abxy, JTransparentToggleButton b) {
		if (abxy >= eingaenge)
			return;
		CursorTk.startWaitCursor(b);
		if (b.isSelected()) { // Eingang aktivieren
			Groesse g = aktiviereInput(0, abxy, 0, true);
			if (g != null)
				g.setD_pm(gitterAbstaende.get(STANDARDGITTER));
		} else { // Eingang deaktivieren
			deaktiviereInput(abxy);
		}
		adjustImage();
		CursorTk.stopWaitCursor(b);
	}

	protected String erzeugeSymbol(int auswahl, int input, int kanal) {
		if (input >= eingaenge)
			return "";
		return sb[input].erzeugeSymbol(auswahl, input, kanal);
	}

	static {
		vendorID = VID_LD;
		subsystem = new String[] { "usb", "usbmisc" };
		deviceGroup = new String[] { "ldusb*", "ldusb*" };
		productID = new HashSet<Integer>() {
			private static final long serialVersionUID = 1L;

			{
				add(PID_XRAY);
				add(PID_XRAY2);
				add(PID_XRAYCT);
			}
		};

		groessen = new Messgroesse[] { Messgroesse.Rate, Messgroesse.Ereignisse };
		namen = new String[] { Messgroesse.Rate.getName(),
				Messgroesse.Ereignisse.getName() };
		symbole = new String[] { Messgroesse.Rate.getSymbol(),
				Messgroesse.Ereignisse.getSymbol() };
		gitterAbstaende = new HashMap<String, Double>() {
			private static final long serialVersionUID = 1L;

			{
				put("aus", 1.0);
				put(STANDARDGITTER, 564.0);
				put("402,7 pm (LiF)", 402.7);
				put("659,7 pm (KBr)", 659.7);
			}
		};

		energieUmrechnungen = new HashMap<String, Double>() {
			private static final long serialVersionUID = 1L;

			{
				put("aus", 1.0);
				put("für Fe-Anode", 0.0329896907216);
				put("für Cu-Anode", 0.052140077821);
				put("für Mo-Anode", 0.244725738397);
				put("für Ag-Anode", 0.393939393939);
				put("für W-Anode", 0.0566891891892);
			}
		};

	}

	@Override
	public IMesskanal getMesskanal(Groesse g) {
		return sb[0].getMesskanal(g, null, true);
	}

	@Override
	public void close() {
		if (xray != null)
			xray.close();
	}

	@Override
	public String getEingangString(int aB) {
		return erzeugeSymbol(0, aB, 0);
	}

	@Override
	public int getEingangAnzahl() {
		return 1;
	}

	@Override
	public String getName() {
		if(xray == null)
			return "XRay";
		else
			return xray.getName();
	}

	@Override
	public void setzeStandardWerte(Groesse g, int AB, int ef, boolean neu) {
		g.setNr(nr);
		g.setAB(AB);
		g.setEF(ef);
		if (neu && sb != null && sb[AB] != null)
			sb[AB].setzeStandardWerte(g);
	}

	/*
	 * Interface IXRay
	 */
	@Override
	public XRayData getData() {
		try {
			return xray.getData();
		} catch(CASSYException e) { 
			if(DEBUG)
				e.printStackTrace();
		}
		return null;
	}

	@Override
	public void setData(XRayData value) {
		try {
			xray.setData(value);
		} catch (CASSYException e){ 
			if(DEBUG)
				e.printStackTrace();
		}
		waitForBusyXRay();
	}

	@Override
	public XRayModes getMode() {
		return xray.getMode();
	}

	@Override
	public void setMode(XRayModes value) {
		try {
		xray.setMode(value);
		} catch(CASSYException e) { 
			if(DEBUG)
				e.printStackTrace();
		}
		waitForBusyXRay();
	}

	@Override
	public XRayPosition getPhysPosition() {
		return xray.getPhysPosition();
	}

	@Override
	public XRayPosition getPosition() {
		return xray.getPosition();
	}

	@Override
	public void setPosition(XRayPosition value) {
		try {
			xray.setPosition(value);
		} catch(CASSYException e){ 
			if(DEBUG)
				e.printStackTrace();
		}
		waitForBusyXRay();
		XRayPosition pos = xray.getPosition();
		int timeout = 10;
		while (timeout > 0 && (pos.getBetaTarget() != value.getBetaTarget() || pos.getBetaSensor() != value.getBetaSensor())) {
			try { Thread.sleep(500); } catch (InterruptedException e) { }
			pos = xray.getPosition();
			timeout--;
		}
	}

	@Override
	public String getVersion() {
		return xray.getVersion();
	}

	@Override
	public double getResolution() {
		return xray.getResolution();
	}

	@Override
	public double getMeanVoltage() {
		return xray.getMeanVoltage();
	}

	@Override
	public double getMeanCurrent() {
		return xray.getMeanCurrent();
	}

	@Override
	public XRayData getMemory(int index) {
		return xray.getMemory(index);
	}

	@Override
	public void resetCASSY() {
		try {
			xray.resetCASSY();
		} catch(CASSYException e){ 
			if(DEBUG)
				e.printStackTrace();
		}
		waitForBusyXRay();
	}

	@Override
	public CASSYStates getCASSYStatus() {
		return xray.getCASSYStatus();
	}

	public boolean waitForBusyXRay() {
		/* einige Befehle benötigen eventuell mehrere Sekunden, bis der Motor
		 * in die richtige Position gefahren ist. Diese Fahrzeit wird über
		 * StepperRunning abgefragt. Nach 5 Sekunden sollte der Motor
		 * die richtige Position erreicht haben, sonst ist etwas schief gelaufen.
		 */
		int counter = 10; // wait at most 10 secs
		try {
			XRayModes mode = null;
			Thread.sleep(50);
			try {
				mode = getMode();
			} catch(CASSYException e) {
				mode = new XRayModes(XRayModes.StepperRunning);
			}
			while (!mode.isAndEmpty(XRayModes.StepperRunning)) {
				counter--;
				if (counter <= 0)
					return false;
				Thread.sleep(100);
				try {
					mode = getMode();
				} catch(CASSYException e) {
					mode = new XRayModes(XRayModes.StepperRunning);
				}
			}
			return true;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return false;
		}
	}

}
