package ghmessen.adapter.cassy;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IAuswahl;
import ghmessen.adapter.IEingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.wertetabelle.XAchseTyp;
import ghmessen.util.ProxyUtil;
import ghmessen.util.Util;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JPopupMenu;

import ld.api.events.SelectedChangedEvent;
import ld.api.events.SelectedChangedListener;
import ld.api.events.VisibleChangedEvent;
import ld.api.events.VisibleChangedListener;
import ld.api.ios.CASSYIO;
import ld.api.ios.DefaultAxes;
import ld.api.ios.DefaultAxis;
import ld.api.ios.ICASSYQuantity;
import ld.api.ios.ICASSYTimerQuantity;
import ld.api.ios.IMirror;
import ld.api.ios.ITare;
import ld.components.MathString;

public class EingangCASSY
		implements IEingang, IAuswahl, InvocationHandler, SelectedChangedListener, VisibleChangedListener {
	private static final boolean DEBUG = false;
	private final CASSYIO cassyio;
	private final IAdapter adapter;
	private final ArrayList<IMesskanal> messkanal;
	private ArrayList<Integer> auswaehlbar;

	private EingangCASSY(CASSYIO cassyio, IAdapter adapter) {
		this.adapter = adapter;
		this.cassyio = cassyio;
		this.messkanal = new ArrayList<IMesskanal>();
		this.auswaehlbar = new ArrayList<Integer>();
		int index = 0;
		for (ICASSYQuantity q : cassyio) {
			q.addSelectedChangedListener(this);
			q.addVisibleChangedListener(this);
			if (q.getVisible())
				auswaehlbar.add(index);
			index++;
		}
		for (ICASSYQuantity q : cassyio.getSelectedQuantities()) {
			selectedChanged(new SelectedChangedEvent(q));
		}
		if (DEBUG) {
			System.out.print("[EingangCASSY] - Konstruktor:\n" + "\tadapter(" + adapter + ")\n" + "\tcassyio(" + cassyio
					+ ")\n" + "\tmesskanal(" + messkanal + ")\n" + "\tauswaehlbar(" + auswaehlbar + ")" + "\tindex("
					+ index + ")");
		}
	}

	public static IEingang create(CASSYIO cassyio, IAdapter adapter) {
		ArrayList<Class<?>> interfaces = new ArrayList<Class<?>>();
		Class<?> dieseKlasse = cassyio.getClass();
		ProxyUtil.sammleIF(interfaces, dieseKlasse);
		ProxyUtil.addIF(interfaces, IEingang.class);
		ProxyUtil.addIF(interfaces, IAuswahl.class);
		return (IEingang) Proxy.newProxyInstance(cassyio.getClass().getClassLoader(),
				interfaces.toArray(new Class<?>[] {}), new EingangCASSY(cassyio, adapter));
	}

	@Override
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		Object result = null;
		try {
			if (method.getDeclaringClass() == IEingang.class || method.getDeclaringClass() == IAuswahl.class)
				result = method.invoke(this, args);
			else
				result = method.invoke(cassyio, args);
		} catch (InvocationTargetException e) {
			throw e.getTargetException();
		} catch (Exception e) {
			throw new RuntimeException("unexpected invocation exception " + e.getMessage());
		}
		return result;
	}

	@Override
	public void close() {
		cassyio.setSelectedQuantities(0);
	}

	/**
	 * Interface Auswahl
	 * 
	 * @parameter -1 : Eingang ausschalten i : messbare quantity i
	 */
	@Override
	public boolean setAuswahl(int index) {
		if (index == -1) {
			cassyio.setSelectedQuantities(0); // Eingang Aus
			return true;
		} else if (auswaehlbar == null || auswaehlbar.get(index) >= cassyio.size())
			return false;
		if (!cassyio.get(auswaehlbar.get(index)).getSelected())
			cassyio.select(cassyio.get(auswaehlbar.get(index)), true);
		return true;
	}

	@Override
	public boolean setAuswahl(String symbol) {
		if (symbol == null || auswaehlbar == null)
			return false;
		for (int index = 0; index < auswaehlbar.size(); index++)
			if (symbol.equals(cassyio.get(auswaehlbar.get(index)).getSymbol())
					&& !cassyio.get(auswaehlbar.get(index)).getSelected()) {
				cassyio.select(cassyio.get(auswaehlbar.get(index)), true);
				return true;

			}
		return false;
	}

	@Override
	public int getAuswahl() {
		for (int index = 0; index < cassyio.size(); index++)
			if (cassyio.get(index).getSelected()) {
				if (auswaehlbar.contains(index))
					return auswaehlbar.indexOf(index);
			}
		return -1;
	}

	@Override
	public List<String> getListe() {
		ArrayList<String> resultat = new ArrayList<String>();
		for (ICASSYQuantity q : cassyio) {
			if (q.getVisible())
				resultat.add(q.getToolTipText());
		}
		return resultat;
	}

	@Override
	public int getAuswahlAnzahl() {
		return auswaehlbar == null ? 0 : auswaehlbar.size();
	}

	@Override
	public void selectedChanged(SelectedChangedEvent e) {
		if (e.getSource() instanceof ICASSYQuantity) {
			ICASSYQuantity q = (ICASSYQuantity) e.getSource();
			Groesse g = Util.findeGroesse(adapter.getGroessenliste(), MathString.replaceAllGreekLetters(q.getSymbol()));
			if (q.getSelected() && q.getVisible()) {
				if (g == null) {
					DefaultAxes axes = cassyio.getDefaultAxes();
					XAchseTyp xtyp = null;
					for (DefaultAxis kosy : axes)
						if (kosy.getYQuantity() == q) {
							xtyp = XAchseTyp.toXAchseTyp(kosy.getXMode());
							break;
						}
					g = new Groesse(adapter.getNr(), q, xtyp);
					adapter.getGroessenliste().add(g);
				}
				messkanal.add(MesskanalCASSY.create(this, g, q));
				adapter.aktiviereGroesse(g);
			} else if (g != null) {
				for (IMesskanal m : messkanal) {
					if (m.getGroesse() == g) {
						adapter.deaktiviereGroesse(g);
						messkanal.remove(m);
						adapter.getGroessenliste().remove(g);
					}
				}
			}
		}
	}

	@Override
	public IMesskanal getMesskanal(Groesse g, ScheduledThreadPoolExecutor timer, boolean autocreate) {
		if (!autocreate && (messkanal == null || messkanal.size() <= 0))
			return null;
		for (IMesskanal mk : messkanal)
			if (mk.getGroesse().equals(g))
				return mk;
		if (autocreate) {
			for (ICASSYQuantity q : cassyio)
				if (g.getSymbol().equals(q.getSymbol())) {
					messkanal.add(MesskanalCASSY.create(this, g, q));
					return getMesskanal(g, timer, false);
				}
		}
		return null;
	}

	@Override
	public String getName() {
		return cassyio.getSensorBoxName();
	}

	@Override
	public double getIntervallStandard() {
		return cassyio.getCASSY().getMeanTime();
	}

	@Override
	public int getWiederholungStandard() {
		return 250;
	}

	@Override
	public double getIntervallMinimum() {
		return cassyio.getCASSY().getMinIntervalTime();
	}

	@Override
	public boolean hatOszilloskop(int kanal) {
		return cassyio.getOscilloscopeCASSY() != null;
	}

	@Override
	public boolean hatKorrektur(int kanal) {
		if (messkanal == null || kanal >= messkanal.size())
			return false;
		return messkanal.get(kanal).hatKorrektur();
	}

	@Override
	public void korrigiereNullpunkt(int kanal) {
		if (messkanal == null || kanal >= messkanal.size())
			return;
		if (messkanal.get(kanal) instanceof ITare)
			((ITare) messkanal.get(kanal)).doTare();
	}

	@Override
	public boolean hatVorzeichenWechsel(int kanal) {
		if (messkanal == null || kanal >= messkanal.size())
			return false;
		return messkanal.get(kanal) instanceof IMirror;

	}

	@Override
	public boolean contains(Point p) {
		Rectangle r = cassyio.getDrawingRect();
		if (r == null)
			return false;
		else
			return r.contains(p);
	}

	@Override
	public JPopupMenu getPopupMenu() {
		JPopupMenu menu = new JPopupMenu();
		int index = -1;
		menu.add(new AuswahlAction("Aus", index++));
		for (ICASSYQuantity q : cassyio) {
			if (q.getVisible())
				menu.add(new AuswahlAction(q.getToolTipText(), index++));
		}
		return menu;
	}

	@Override
	public IAdapter getAdapter() {
		return adapter;
	}

	@Override
	public int getKanaele() {
		if (messkanal == null)
			return 0;
		return messkanal.size();
	}

	@Override
	public boolean hatTimer() {
		for (IMesskanal m : messkanal)
			if (m instanceof ICASSYTimerQuantity)
				return true;
		return false;
	}

	private class AuswahlAction extends AbstractAction {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private int index;

		public AuswahlAction(String name, int index) {
			this.index = index;
			putValue(Action.NAME, name);
			setEnabled(true);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			setAuswahl(index);
			adapter.getComponent().repaint();
		}

	}

	@Override
	public void visibleChanged(VisibleChangedEvent e) {
		auswaehlbar.clear();
		int index = 0;
		for (ICASSYQuantity q : cassyio) {
			if (q.getVisible())
				auswaehlbar.add(index++);
		}
		System.out.println(
				"[" + this.getClass().toString() + "] Es sind jetzt " + auswaehlbar.size() + " Größen sichtbar.");
	}
}
