package ghmessen.adapter.cassy;

import ld.api.CASSYException;
import ld.api.CASSYStates;
import ld.api.IXRay;
import ld.api.XRayData;
import ld.api.XRayModes;
import ld.api.XRayPosition;
import flanagan.analysis.Regression;
import ghmessen.adapter.EAdapterException;

public class JustiereXRay {

	private static final boolean DEBUG = false;

	public enum Status {
		INITIALISIERT, LAEUFT, ERFOLGREICH, FEHLGESCHLAGEN;
	}

	public enum Modus {
		NULLSUCHE, JUSTIERUNG;
	}

	IXRay xray = null;
	Thread justieren = null;
	Status status;
	Modus modus;
	double messZeit;
	boolean grob;
	double targetZiel;
	double gesamtZeit;
	double abgelaufeneZeit;
	double target, sensor;
	int rate;
	// Nullpunktsuche des Sensors mit verminderter Spannung 22.0 kV!
	double[] dS = new double[] { 10.0, 1.0 };
	// Justierung von Target und Sensor auf das höchste Maximum
	// bei dem Target-Winkel targetZiel
	double[] dT = new double[] { 0.2, 1.0 };
	// dB[Feinheit][Target/Sensor][links/rechts]
	double[][] dB = new double[][] { { 5.0, 10.0 }, { 0.5, 1.0 } };
	double deltaB = 0.1;
	private static final int GROB = 0;
	private static final int FEIN = 1;
	private static final int TARGET = 0;
	private static final int SENSOR = 1;
	private static final double TOLERANZ = 0.1;

	// Gauss'sche Regression
	private double[] x, y;

	public JustiereXRay(IXRay xray) {
		this(xray, 7.2);
	}

	public JustiereXRay(IXRay xray, double targetZiel) {
		super();
		this.xray = xray;
		this.targetZiel = targetZiel;
		messZeit = dT[GROB];
		grob = true;
		status = Status.INITIALISIERT;
		modus = Modus.JUSTIERUNG;
		x = null;
		y = null;
	}

	public void justierungStop() {
		if (justieren != null)
			justieren.interrupt();
		status = Status.FEHLGESCHLAGEN;
	}

	public Status getStatus() {
		return status;
	}

	public Modus getModus() {
		return modus;
	}

	public void setModus(Modus modus) {
		if (modus != this.modus) {
			if (status != Status.INITIALISIERT) {
				if (status == Status.LAEUFT)
					justierungStop();
			}
		}
		this.modus = modus;
		status = Status.INITIALISIERT;
		messZeit = dT[GROB];
		grob = true;
	}

	public int getProzentFertig() {
		return (int) (100 * abgelaufeneZeit / gesamtZeit);
	}

	public double getTargetZielWinkel() {
		return targetZiel;
	}

	public void setTargetZielWinkel(double target) {
		targetZiel = target;
	}

	public double getTarget() {
		return target;
	}

	public double getSensor() {
		return sensor;
	}

	public int getRate() {
		return rate;
	}

	private void initJustierung() throws InterruptedException {
		abgelaufeneZeit = 0;
		gesamtZeit = (2*dB[GROB][TARGET]/deltaB + 1.0)*dT[GROB]
				+ (2*dB[GROB][SENSOR]/deltaB + 1.0)*dT[GROB]
				+ (2*dB[FEIN][TARGET]/deltaB + 1.0)*dT[FEIN]
				+ (2*dB[FEIN][SENSOR]/deltaB + 1.0)*dT[FEIN];
	}

	private double sucheMaximum(double mitte, boolean bTarget,
			double nachLinks, double nachRechts, double ukV)
			throws InterruptedException {
		if(DEBUG)
			System.out.println(String.format("\n[JustiereXRay-sucheMaximum]: %s-Winkel: %3.1f[%3.1f|%3.1f]\n"
					+ "\tdT=%3.1f\tdB=%3.1f",
					bTarget?"TARGET":"SENSOR",mitte,nachLinks,nachRechts,messZeit,deltaB));
		XRayData data = new XRayData();
		XRayPosition p;
		try {
			xray.setMode(new XRayModes(bTarget ? XRayModes.Target : XRayModes.Sensor));
		} catch (CASSYException e) {
			if (!waitForXRay(xray))
				throw new CASSYException("Wait for XRay did not work.");
		}
		System.out.println();
		data.setVoltage(ukV);
		data.setCurrent(1.0);
		data.setBeta1(begrenzt(mitte - nachLinks, -10.0, 80.0));
		data.setBeta2(begrenzt(mitte + nachRechts, -10.0, 80.0));

		data.setGateTime(messZeit);
		data.setDeltaBeta(deltaB);
		xray.setData(data);
		int aktR = 0, maxR = 0, bisher = 0;
		int gesamt = anzahlWerte(data.getBeta1(), data.getBeta2(),
				data.getDeltaBeta(), -10.0, 80.0);
		double aktWinkel = 0.0, maxWinkel = 0.0;
		x = new double[gesamt];
		y = new double[gesamt];
		xray.setMode(new XRayModes(XRayModes.Scan));
		Thread.sleep(100);
		do {
			try {
				data = xray.getMemory(bisher);
				if (data != null && data.getRate() == 0) {
					abgelaufeneZeit += dT[grob ? GROB : FEIN];
					aktR = (int) (data.getCounts() / data.getGateTime());
					aktWinkel = data.getBeta();
					if (DEBUG)
						System.out
								.println(String.format("\n[justiere"
										+ (bTarget ? "TARGET" : "SENSOR")
										+ "] - Daten lesen... Winkel(%3.1f°) Rate(%4d /s) Anzahl(%d) GateTime(%f)",
										aktWinkel, aktR, data.getCounts(),
										data.getGateTime()));
					if (bTarget)
						target = data.getPosition().getBetaTarget();
					else
						sensor = data.getPosition().getBetaSensor();
					rate = aktR;
					if (aktR > maxR) {
						maxR = aktR;
						maxWinkel = aktWinkel;
					}
					x[bisher] = aktWinkel;
					y[bisher] = aktR;
					bisher++;
				}
			} catch (CASSYException e) {
				Thread.sleep(100);
			}
		} while (bisher < gesamt);
		if (modus == Modus.NULLSUCHE) {
			Regression r = new Regression(x, y);
			r.gaussian();
			double[] result = r.getBestEstimates();
			if (DEBUG)
				System.out
						.println("[JustiereXRay] - sucheMaximum Gauss-Regression("
								+ result + ")");
			maxWinkel = result[0]; // Erwartungwert
		}
		if (DEBUG)
			System.out
					.println(String.format("\n*********\n[justiere"
							+ (bTarget ? "TARGET" : "SENSOR")
							+ "] - Maximum(%4d) bei dem Winkel(%3.1f°)\n********",
							maxR, maxWinkel));
		p = xray.getPosition();
		if (bTarget) {
			p.setBetaTarget(maxWinkel);
			target = maxWinkel;
		} else {
			p.setBetaSensor(maxWinkel);
			sensor = maxWinkel;
		}
		rate = 0;
		xray.setPosition(p);
		waitForXRay(xray);
		return maxWinkel;
	}

	private void speichereJustierung() throws InterruptedException,
			EAdapterException {
		XRayPosition p;

		xray.setMode(new XRayModes(XRayModes.Coupled));
		p = xray.getPosition();
		p.setBetaTarget(p.getBetaTarget() - targetZiel);
		xray.setPosition(p);
		if(!waitForXRay(xray))
			throw new EAdapterException("[JustiereXRay] - speichereJustierung: Positionierung fehlgeschlagen.");
		speichereInXRay(xray);
	}

	private void speichereNullsuche() throws InterruptedException,
			EAdapterException {
		speichereInXRay(xray);
	}

	private void speichereInXRay(IXRay xray) throws InterruptedException,
			EAdapterException {
		xray.setMode(new XRayModes(XRayModes.SaveZero));
		waitForXRay(xray);
		xray.resetCASSY();
		waitForXRay(xray);
	}

	public void starten() throws InterruptedException, EAdapterException {
		if (status == Status.LAEUFT)
			throw new EAdapterException("[justiereXRay] error start in xray");
		justieren = new Thread() {
			@Override
			public void run() {
				try {
					status = Status.LAEUFT;
					if (modus == Modus.JUSTIERUNG)
						justierenDurchfuehren();
					else
						nullSucheDurchfuehren();
				} catch (EAdapterException e) {
					if (DEBUG)
						e.printStackTrace();
					status = Status.FEHLGESCHLAGEN;
				} catch (InterruptedException e) {
					if (DEBUG)
						e.printStackTrace();
					status = Status.FEHLGESCHLAGEN;
				} finally {
					if (status == Status.LAEUFT)
						status = Status.ERFOLGREICH;
				}

			}
		};
		justieren.start();
	}

	public void justierenDurchfuehren() throws InterruptedException,
			EAdapterException {
		double winkelNeu = 0.0, winkel = targetZiel, abstandTarget = 100.0, abstandSensor = 100.0;
		boolean useTarget = true;
		XRayData saveData = null;
		try {
			saveData = xray.getData();
		} catch (CASSYException e) {
			saveData = null;
		}
		if (saveData == null) {
			if (DEBUG)
				System.out
						.println("[justiereXRay] - Fehler beim Daten sichern.");
			throw new EAdapterException(
					"[justiereXRay] - error in accessing xray");
		}
		try {
			initJustierung();
			int zaehler = 0;
			while (grob || zaehler < 4 
					|| abstandTarget >= TOLERANZ
					|| abstandSensor >= TOLERANZ) {
				xray.setPosition(new XRayPosition(targetZiel, 2 * targetZiel));
				if (!waitForXRay(xray))
					throw new CASSYException("Wait for XRay did not work.");
				winkelNeu = sucheMaximum(
						winkel,
						useTarget,
						dB[grob ? GROB : FEIN][useTarget ? TARGET : SENSOR],
						dB[grob ? GROB : FEIN][useTarget ? TARGET : SENSOR],
						35.0);
				if (useTarget) {
					abstandTarget = Math.abs(winkelNeu - winkel);
				} else {
					abstandSensor = Math.abs(winkelNeu - winkel);
				}
				if (DEBUG)
					System.out.println(String.format("[justiereXRay] - Korrektur: "
							+ (useTarget ? "Target(" : "Sensor(") + "%3.1f)",
							winkelNeu - winkel));
				speichereJustierung();
				if (grob 
						&& abstandTarget < dB[FEIN][TARGET] 
						&& abstandSensor < dB[FEIN][SENSOR]) {
					grob = false;
					messZeit = dT[FEIN];
				} else if (!grob 
						&& messZeit == dT[FEIN] 
						&& zaehler > 3)
					messZeit = 5.0 * dT[FEIN];
				else if (zaehler > 6)
					break;
				useTarget = !useTarget;
				winkel = useTarget ? targetZiel : 2 * targetZiel;
				zaehler++;
			}
			if (abstandTarget >= TOLERANZ || abstandSensor >= TOLERANZ)
				System.out
						.println(String.format("[justiereXRay] - Die Justierung ist fehlgeschlagen: Abstand Target(%3.1f) Abstand Sensor(%3.1f)\n",
								abstandTarget, abstandSensor));
		} catch (CASSYException e) {
			e.printStackTrace();
		} finally {
			if (saveData != null) {
				xray.setData(saveData);
				if (DEBUG)
					checkError("[justiereXRay] - Daten wiederherstellen");
			}
		}
	}

	/*
	 * nullAbgleichen : solange die Position und phys. Position kleiner 0°
	 * gestellt werden kann, neue Null speichern und wieder 10° nach unten
	 * gehen. Dann den Sensor auf 10° drehen und als neue Null speichern. Jetzt
	 * stimmen logische Position 0° und phys. Position 0° des Sensors überein.
	 */
	public void nullAbgleichen() throws EAdapterException {
		xray.setMode(new XRayModes(XRayModes.Sensor));
		waitForXRay(xray);
		XRayPosition p = xray.getPosition();
		p.setBetaSensor(-10.0);
		xray.setPosition(p);
		waitForXRay(xray);
		p = xray.getPhysPosition();
		do {
			xray.setMode(new XRayModes(XRayModes.SaveZero));
			waitForXRay(xray);
			xray.setMode(new XRayModes(XRayModes.Sensor));
			waitForXRay(xray);
			p = xray.getPosition();
			p.setBetaSensor(-10.0);
			xray.setPosition(p);
			waitForXRay(xray);
			p = xray.getPhysPosition();
		} while(p.getBetaSensor() < 0.0);
		p = xray.getPosition();
		p.setBetaSensor(10.0);
		xray.setPosition(p);
		waitForXRay(xray);
		xray.setMode(new XRayModes(XRayModes.SaveZero));
		waitForXRay(xray);
	}

	public void nullSucheDurchfuehren() throws InterruptedException,
			EAdapterException {
		XRayData saveData = null;
		try {
			saveData = xray.getData();
		} catch (CASSYException e) {
			saveData = null;
		}
		if (saveData == null) {
			if (DEBUG)
				System.out
						.println("[justiereXRay] - Fehler beim Daten sichern.");
			throw new EAdapterException(
					"[justiereXRay] - error in accessing xray");
		}
		double sensorNeu = 0.0, abstandSensor = 0.0;
		boolean feinFertig = false;
		try {
			abgelaufeneZeit = 0;
			gesamtZeit = (2*dS[GROB]/deltaB + 1.0)*dT[GROB]
					+ (2*dS[FEIN]/deltaB + 1.0)*dT[FEIN];
			nullAbgleichen();
			int zaehler = 0;
			while ( abstandSensor >= TOLERANZ || grob || !feinFertig) {
				xray.setPosition(new XRayPosition(0.0,0.0));
				if(!waitForXRay(xray))
					throw new EAdapterException("[JustiereXRay] - nullSucheDurchführen: Anfangsposition anfahren fehlgeschlagen.");
				sensorNeu = sucheMaximum(0.0, false,
						dS[grob ? GROB : FEIN],
						dS[grob ? GROB : FEIN], 22.0);
				abstandSensor = Math.abs(sensorNeu);
				if (grob && abstandSensor < dB[FEIN][SENSOR]) {
					grob = false;
					messZeit = dT[FEIN];
				} else if(grob)
					zaehler++;
				else if (messZeit == dT[FEIN]){
					messZeit = 5.0 * dT[FEIN];
					feinFertig = true;
				}
				if(zaehler > 2)
					break;
				xray.setPosition(new XRayPosition(0.0,sensorNeu));
				if(!waitForXRay(xray))
					throw new EAdapterException("[JustiereXRay] - nullSucheDurchführen: Neue Position anfahren fehlgeschlagen.");
				speichereNullsuche();
			}
			if (abstandSensor < TOLERANZ) {
				if (DEBUG)
					System.out
							.println("[justiereXRay] - Die Nullsuche ist war erfolgreich.");
			} else if (DEBUG)
				System.out
						.println("[justiereXRay] - Die Nullsuche ist fehlgeschlagen.");
		} catch (CASSYException e) {
			e.printStackTrace();
		} finally {
			if (saveData != null)
				xray.setData(saveData);
			if (DEBUG)
				checkError("[justiereXRay] - Daten wiederherstellen");
		}
	}

	public boolean waitForXRay(IXRay xray) {
		/*
		 * einige Befehle benötigen eventuell mehrere Sekunden, bis der Motor in
		 * die richtige Position gefahren ist. Diese Fahrzeit wird über
		 * StepperRunning abgefragt. Nach 10 Sekunden sollte der Motor die
		 * richtige Position erreicht haben, sonst ist etwas schief gelaufen.
		 */
		int counter = 20; // wait at most 10 secs
		try {
			XRayModes mode = null;
			Thread.sleep(50);
			try {
				mode = xray.getMode();
			} catch (CASSYException e) {
				mode = new XRayModes(XRayModes.StepperRunning);
			}
			while (!mode.isAndEmpty(XRayModes.StepperRunning)) {
				if(DEBUG)
					System.out.print(mode);
				counter--;
				if (counter <= 0)
					return false;
				Thread.sleep(500);
				try {
					mode = xray.getMode();
				} catch (CASSYException e) {
					mode = new XRayModes(XRayModes.StepperRunning);
				}
			}
			if(DEBUG)
				System.out.println();
			return true;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return false;
		}
	}

	public void checkError(String s) {
		CASSYStates result = xray.getCASSYStatus();
		System.out.println(String.format("checkError: Aktion(%s) Resultat(%d)", s,
				result.getValue()));
	}

	private int anzahlWerte(double von, double bis, double intervall,
			double minimum, double maximum) {
		von = begrenzt(von, minimum, maximum);
		bis = begrenzt(bis, minimum, maximum);
		if (bis < von) {
			double tmp = bis;
			bis = von;
			von = tmp;
		}
		return (int) Math.round((bis - von) / intervall + 1.0);
	}

	private double begrenzt(double wert, double minimum, double maximum) {
		return Math.min(maximum, Math.max(minimum, wert));
	}
	
}
