/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter.cassy;

import static ghmessen.groesse.Messgroesse.Rate;

import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.JFrame;

import ld.api.CASSYException;
import ld.api.IXRay;
import ld.api.XRayData;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.IEingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.Messkanal;
import ghmessen.adapter.MesskanalException;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetXRay;
import ghmessen.instrumente.SensorPanel.SensorBoxPanel;
import ghmessen.instrumente.SensorPanel.XRayPanel;

public class MesskanalXRay extends Messkanal {

	private static final boolean DEBUG = false;

	private boolean xrayKanal;
	private boolean kanalOffen;
	protected XRayData[] puffer;
	protected int anzahl;
	private final IXRay xray;

	public MesskanalXRay(IEingang eingang, Groesse g) {
		super(eingang, g);
		xrayKanal = true;
		kanalOffen = eingang.getAdapter() != null && g.istXRayGroesse();
		puffer = new XRayData[2048];
		anzahl = 0;
		if (eingang.getAdapter() instanceof IXRay)
			xray = (IXRay) eingang.getAdapter();
		else
			xray = null;
	}

	@Override
	public double einlesen() throws MesskanalException {
		XRayData daten = xray.getData();
		ueberschritten = false;
		unterschritten = false;
		d = daten.getCounts();
		if (g.getMessGroesse() == Rate) {
			// Rate aus den Ereignissen berechnen
			if (daten.getRemainingTime() == 0)
				d /= daten.getGateTime();
			else
				d /= (daten.getGateTime() - daten.getRemainingTime());
		}
		return d;
	}

	public XRayData leseDaten() throws MesskanalException {
		return xray.getData();
	}

	@Override
	public int fuellePuffer(int bisher, int neu) throws InterruptedException,
			EAdapterException {
		anzahl = fillXRayArray(bisher, neu);
		if (DEBUG)
			System.out.println("[MesskanalXRay] - fuellePuffer: XRay bisher("
					+ bisher + ") noch(" + neu + ") neu(" + anzahl + ")");
		anzahl = Math.min(anzahl, puffer.length);
		return anzahl;
	}

	@Override
	public double lesePufferWert(int index) throws InterruptedException,
			EAdapterException {
		if (index >= 0 && index < puffer.length) {
			if (puffer[index] != null) {
				if (g.getMessGroesse() == Rate)
					return puffer[index].getGateTime() == 0 ? puffer[index]
							.getCounts() : puffer[index].getCounts()
							/ puffer[index].getGateTime();
				else
					return puffer[index].getCounts();
			} else
				throw new EAdapterException(
						"[MesskanalXRay] - lesePufferWert: Index(" + index
								+ ") ist nicht vorhanden.");
		} else
			throw new EAdapterException(
					"[MesskanalXRay] - lesePufferWert: Index(" + index
							+ ") zu groß.");
	}

	public double lesePufferWinkel(int index) throws EAdapterException {
		if (index >= 0 && index < puffer.length) {
			if (puffer[index] != null)
				return puffer[index].getBeta();
			else
				throw new EAdapterException(
						"[MesskanalXRay] - lesePufferWinkel: Index(" + index
								+ ") ist nicht vorhanden.");
		} else
			throw new EAdapterException(
					"[MesskanalXRay] - lesePufferWinkel: Index(" + index
							+ ") zu groß.");
	}

	public XRayData lesePufferDaten(int index) throws InterruptedException,
			EAdapterException {
		if (index >= 0 && index < puffer.length) {
			if (puffer[index] != null)
				return puffer[index];
			else
				throw new EAdapterException(
						"[MesskanalXRay] - lesePufferWinkel: Index(" + index
								+ ") ist nicht vorhanden.");
		} else
			throw new EAdapterException(
					"[MesskanalXRay] - lesePufferWinkel: Index(" + index
							+ ") zu groß.");
	}

	@Override
	public void finalize() {
		return;
	}

	@Override
	public boolean istUeberschritten() {
		return ueberschritten;
	}

	@Override
	public boolean istUnterschritten() {
		return unterschritten;
	}

	@Override
	public boolean istKanalOffen() {
		return kanalOffen;
	}

	@Override
	public boolean istOszilloskopKanal() {
		return oszilloskopKanal;
	}

	public boolean istXRayKanal() {
		return xrayKanal;
	}

	public void setXRayData(XRayData p) {
		if (xray != null)
			xray.setData(p);
	}

	@Override
	public int getMessbereichIndex() {
		return 0;
	}

	@Override
	public void setMessbereich(int index) { }

	@Override
	public void setMessbereich(Messbereich messbereich) { }

	@Override
	public void start() throws MesskanalException { }

	@Override
	public void stop() throws MesskanalException { }

	@Override
	public Messgeraet createMessgeraet(JFrame parent, IMesskanal messkanal,
			ScheduledThreadPoolExecutor timer) {
		return new MessgeraetXRay(parent, messkanal, eingang.getAdapter(), g, timer);
	}

	@Override
	public SensorBoxPanel getPanel(IMesskanal messkanal) {
		return new XRayPanel(g, eingang.getAdapter(), messkanal);
	}

	private int fillXRayArray(int start, int count) throws EAdapterException,
			InterruptedException {
		int next = start;
		XRayData daten = null;
		if (DEBUG)
			System.out.println("[Cassys] - fillXRayArray: start(" + start
					+ ") count(" + count + ") +++++++++++");
		try {
			daten = xray.getMemory(next);
		} catch(CASSYException e) {
			daten = null;
		}
		while ((next - start < count) && daten != null) {
			if (DEBUG)
				System.out.println("[Cassys] - fillXRayArray: lese(" + next
						+ ") schreibe(" + (next - start) + ")");
			puffer[next - start] = daten;
			next++;
			try {
				daten = xray.getMemory(next);
			} catch(CASSYException e) {
				daten = null;
			}
		}
		return next - start;
	}

}
