package ghmessen.adapter.efm;

import java.util.BitSet;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.JFrame;

import ghmessen.adapter.IEingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.Messkanal;
import ghmessen.adapter.MesskanalException;
import ghmessen.adapter.seriell.SerialAdapter;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetStandard;
import ghmessen.instrumente.SensorPanel.SensorBoxPanel;
import static ghmessen.groesse.Einheit.kV_m;

public class MesskanalEFM extends Messkanal {

	private static final boolean DEBUG = false;
	private static final double MESSWERTMAX = 255;
	SerialAdapter efm;
	boolean beenden = false;
	Schnittstelle schnittstelle;
	Messbereich messbereich;
	private static Messbereich[] messbereiche;
	int messwert;
	boolean gueltig;
	
	public MesskanalEFM(IEingang eingang, Groesse g, SerialAdapter efm) {
		super(eingang, g);
		this.efm = efm;
		if(efm != null) {
			messbereich = messbereiche[0];
			g.setMessbereich(messbereich);
			schnittstelle = new Schnittstelle();
			schnittstelle.start();
		}
	}

	@Override
	public int getMessbereichIndex() {
		return 0;
	}

	//Automatisch erkannter Messbereich nur am EFM wechselbar
	@Override
	public void setMessbereich(int index) { }

	@Override
	public void setMessbereich(Messbereich messbereich) { }

	@Override
	public void start() throws MesskanalException {
		schnittstelle = new Schnittstelle();
		schnittstelle.start();
	}

	@Override
	public void stop() throws MesskanalException {
		beenden = true;
		schnittstelle = null;
	}

	@Override
	public Messgeraet createMessgeraet(JFrame parent, IMesskanal messkanal,
			ScheduledThreadPoolExecutor timer) {
		return new MessgeraetStandard(parent, messkanal, eingang.getAdapter(),
				g, timer);
	}

	@Override
	public SensorBoxPanel getPanel(IMesskanal messkanal) {
		return new SensorBoxPanel(g, eingang.getAdapter(), 
				messkanal);
	}

	@Override
	public double einlesen() throws MesskanalException {
		if(!gueltig)
			throw new MesskanalException("[MesskanalEFM] - einlesen() ungültig.");
		return messbereich.getFaktor()*messwert/MESSWERTMAX;
	}

	static {
		messbereiche = new Messbereich[]{
				new Messbereich(-1, 1.0, 
						-1.0, 1.0, 0.1, 3, kV_m, "-1kV/m .. 1kV/m"),
				new Messbereich(-1, 10.0, 
						-10.0, 10.0, 1.0, 2, kV_m, "-10kV/m .. 10kV/m"),
				new Messbereich(-1, 100.0, 
						-100.0, 100.0,10.0, 1, kV_m, "-100kV/m .. 100kV/m")
		};
	}
	
	private class Schnittstelle extends Thread {
		private static final int MAXLENGTH = 100;
		byte[] buffer = new byte[MAXLENGTH];
		boolean restVorhanden = false;
		byte rest;
		int gelesen;
		
		public void run() {
			while(!beenden && efm != null) {
				try {
					gelesen = efm.read(buffer, MAXLENGTH);
					int index = 0;
					while(index < gelesen) {
						if(restVorhanden) {
							restVorhanden = false;
							if(werteAus(rest,buffer[index]))
								index++;
						}
						else if(index+1 < gelesen){
							if(werteAus(buffer[index], buffer[index+1]))
								index += 2;
							else
								index++;
						}
						else {
							restVorhanden = true;
							rest = buffer[index];
							index++;
						}
					}
				} catch (InterruptedException e) {
					beenden = true;
				}
			}
		}
		
		private static final int NEGATIV = 14;
		private static final int SIGNATUR0 = 0;
		private static final int SIGNATUR1 = 8;
		private static final int MESSWERTLSB = 9;
		private static final int STATUSMSG = 15;
		private static final int MESSBEREICHSTART = 10;
		
		private boolean werteAus(byte hb, byte lb) {
			BitSet bits = BitSet.valueOf(new byte[]{lb,hb});
			if(!bits.get(SIGNATUR1) || bits.get(SIGNATUR0)) {
				if(DEBUG)
					System.err.printf("[MesskanalEFM] - werteAus %x %x Bytesignatur stimmt nicht!\n", hb, lb);
				return false;
			}
			if(bits.get(STATUSMSG)){
				gueltig = false;
			}
			else {
				messwert = getMesswert(bits, lb);
				if(DEBUG)
					System.out.printf("[MesskanalEFM] - werteAus(%x %x): %d\n",hb,lb,messwert);
				//automatischer Messbereichswechsel
				if(messbereich != getMessbereich(bits.nextSetBit(MESSBEREICHSTART))){
					messbereich = getMessbereich(bits.nextSetBit(MESSBEREICHSTART));
					g.setMessbereich(messbereich);
				}
				gueltig = true;
			}
			return true;
		}
		
		private int getMesswert(BitSet bits, byte lb) {
			int messwert = (lb & 0xff) + (bits.get(MESSWERTLSB)?1:0);
			if(bits.get(NEGATIV))
				return -messwert;
			else
				return messwert;			
		}
		
		private Messbereich getMessbereich(int bereich) {
			switch(bereich) {
				case 10:
				case 11:
				case 12:
				 	return messbereiche[bereich-MESSBEREICHSTART];
				default: 
					return messbereiche[0];
			}
		}
	}
}
