/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter.event;

import ghmessen.adapter.IAdapter;

import java.io.File;
import java.util.EventObject;

import udev.UdevDevice;

@SuppressWarnings("serial")
public class AdapterAddedEvent extends EventObject {

	private static final boolean DEBUG = false;
	private UdevDevice dev;
	private int nr;
	private Class<? extends IAdapter> adapterKlasse;
	private boolean kaskadiert;
	private int kaskadeNr;

	public AdapterAddedEvent(Object source, UdevDevice dev,
			Class<? extends IAdapter> adapterKlasse) {
		this(source, dev, adapterKlasse, -1, false, -1);
	}

	public AdapterAddedEvent(Object source, UdevDevice dev,
			Class<? extends IAdapter> adapterKlasse, int nr, boolean kaskadiert,
			int kaskadeNr) {
		super(source);
		this.dev = dev;
		this.nr = nr;
		this.adapterKlasse = adapterKlasse;
		this.kaskadiert = kaskadiert;
		this.kaskadeNr = kaskadeNr;
		if(DEBUG)
			System.out.println("[AdapterAddedEvent] - Konstruktor:\n"
					+"\tdev("+dev+")\n"
					+"\tnr("+nr+") klasse("+adapterKlasse+"]\n"
					+"\tkaskadiert("+kaskadiert+") kaskadeNr("+kaskadeNr+")");
	}

	public File getFile() {
		return new File(dev.getNode());
	}

	public String getName() {
		return dev.getName();
	}

	public String getNode() {
		return dev.getNode();
	}

	public UdevDevice getDevice() {
		return dev;
	}

	public void setNummer(int nr){
		this.nr = nr;
	}
	
	public int getNummer() {
		return nr;
	}

	public void setKaskadiert(boolean kaskadiert){
		this.kaskadiert = kaskadiert;
	}
	
	public boolean istKaskadiert(){
		return kaskadiert;
	}
	
	public void setKaskadeNr(int kaskadeNr){
		this.kaskadeNr = kaskadeNr;
	}
	public int getKaskadeNr(){
		return kaskadeNr;
	}
	
	public Class<? extends IAdapter> getKlasse() {
		return adapterKlasse;
	}

	public void setKlasse(Class<? extends IAdapter> klasse) {
		adapterKlasse = klasse;
	}
	
	public String toString() {
		return "Device(" + dev.getNode() + ") Adapter(" + adapterKlasse + ") Nr(" + getNummer() + ")";
	}
}
