/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter.event;

import java.io.File;
import java.util.EventObject;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import udev.UdevDevice;

@SuppressWarnings("serial")
public class AdapterRemovedEvent extends EventObject {

	private static final boolean DEBUG = false;
	private UdevDevice dev;
	private int devNumber;
	private boolean portRemoved;
	private static Pattern p = Pattern.compile("[+-]?[0-9]+");
	
	
	public AdapterRemovedEvent(Object source, UdevDevice dev){
		this(source, dev, true);
	}
	
	public AdapterRemovedEvent(Object source, UdevDevice dev, boolean portRemoved) {
		super(source);
		this.dev = dev;
		this.devNumber = getDeviceNumber();
		this.portRemoved = portRemoved;
		if(DEBUG)
			System.out.println("[AdapterRemovedEvent] - Konstruktor:\n"
					+"\tdev("+dev+")\n"
					+"\tdevNumber("+devNumber+") portRemoved("+portRemoved+")");
	}

	public File getFile(){
		return new File(dev.getNode());
	}

	public String getName(){
		return dev.getName();
	}
	
	public UdevDevice getDevice() {
		return dev;
	}
	
	public int getDeviceNumber(){
		Matcher m = p.matcher(getName());
		if(m.find())
			return Integer.parseInt(m.group()) + 1;
		else
			return -1;
			
	}
	
	public boolean isPortRemoved() {
		return portRemoved;
	}
	public String toString() {
		return "Device("+dev.getNode()+") Nr("+devNumber+")";
	}
}
