/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter.goIO;

import static libGoIO.LibGoIO.CYCLOPS_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.LABPRO_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.LOWCOST_SPEC_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.MINI_GC_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.SKIP_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.STANDALONE_DAQ_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.USB_DIRECT_TEMP_DEFAULT_PRODUCT_ID;
import static libGoIO.LibGoIO.VERNIER_DEFAULT_VENDOR_ID;
import ghmessen.GHMessen;
import ghmessen.adapter.Adapter;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.Eingang;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.event.AdapterChangedEvent;
import ghmessen.components.JTransparentToggleButton;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.util.CursorTk;
import ghmessen.util.HTML;
import ghmessen.util.Util;

import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.Collections;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;

import libGoIO.GoIOSensor;
import libGoIO.GoIOSensorFactory;
import libGoIO.GoIOSensorCellRenderer;

import org.jdesktop.application.Action;

import udev.UdevDevice;

@SuppressWarnings("serial")
public class AdapterGoIO extends Adapter {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8132222219420464032L;
	private static final boolean DEBUG = false;

	private static final Rectangle[] rectA = new Rectangle[] { new Rectangle(
			12, 102, 79, 82) };

	public static final int vendorID;
	public static final HashSet<Integer> productID;
	public static final String[] subsystem;
	public static final String[] deviceGroup;
	public static final int ERSTER_SENSOR;
	private BufferedImage aktivImg;

	protected EingangGoIO goIO;
	protected Vector<String> sensoren;

	public AdapterGoIO(int nr, UdevDevice device, Groessenliste liste)
			throws EAdapterException {
		super(new ImageIcon(
				AdapterGoIO.class
						.getResource("resources/GoIOLinkAdapter.png")), nr,
				device, rectA, liste);
		initSensorListe();
		adjustImage();
	}

	public void initSensorListe() {
		sensoren = new Vector<String>();
		if (GHMessen.getApplication().vernierSensoren != null
				&& GHMessen.getApplication().vernierSensoren.size() != 0) {
			for (String s : GHMessen.getApplication().vernierSensoren.keySet()) {
				s = s.replaceFirst("\\|[0-9]*", "");
				if (!sensoren.contains(s))
					sensoren.addElement(s);
			}
		} else {
			for (String s : GoIOSensorFactory.createSensorMap().keySet())
				sensoren.addElement(s);
		}
		Collections.sort(sensoren);
	}

	@Override
	public void close() {
		goIO.close();
	}

	@Override
	protected void initEingaenge() throws EAdapterException {
		eingaenge = getEingangAnzahl();
		goIO = new EingangGoIO(this, device, 0);
		sb = new Eingang[] { goIO };
		super.initEingaenge();
	}

	protected void initButtonActions() {
		buttonActions = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(this.getClass(), this);
	}

	protected void initInputLabels() {
		for (int i = 0; i < eingaenge; i++)
			inputLabel[i] = new JLabel("<html>ungültig</html>");
	}

	@Override
	protected void initButtons() {
		for (int i = 0; i < eingaenge; i++)
			inputButton[i] = sb[i].createToggleButton(rect[i],
					buttonActions.get("click" + (char) ('A' + i)));
	}

	public void initGroessen() {
		if (DEBUG)
			System.out
					.println("[GoOILinkAdapter] - initGroessen wurde aufgerufen.");
		for (int AB = 0; AB < eingaenge; AB++) {
			initGroessenfuerEingang(AB);
		}

	}

	protected void initGroessenfuerEingang(int AB) {
		int auswahl = sb[AB].getAuswahl();
		for (int groesse = 0; groesse < sb[AB].getMessgroessenAnzahl(0); groesse++) {
			sb[AB].setAuswahl(groesse);
			int n = getKanaele(AB);
			for (int ef = 0; ef < n; ef++)
				aktiviereInput(groesse, AB, ef, false);
			if (aktiv[AB]) {
				inputButton[AB].setSelected(true);
				break;
			}
		}
		if (!aktiv[AB])
			sb[AB].setAuswahl(auswahl);
		this.revalidate();
		adjustImage();
	}

	public int getKanaele(int abxy) {
		if (abxy >= eingaenge)
			return 0;
		else
			return sb[abxy].getKanaele();
	}

	public String erzeugeSymbol(int auswahl, int abxy, int kanal) {
		if (abxy >= eingaenge)
			return "";
		else
			return sb[abxy].erzeugeSymbol(auswahl, abxy, kanal);
	}

	protected void initComponents() {

		super.initComponents();
		setLayout(null);

		add(inputButton[0]);

		schnittLabel.setForeground(Color.RED);
		schnittLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		schnittLabel.setBounds(12, 1, 79, 15);
		add(schnittLabel);

		inputLabel[0].setForeground(Color.BLACK);
		inputLabel[0].setFont(new Font("Dialog", Font.BOLD, 16));
		inputLabel[0].setBounds(12, 102, 49, 82);
		add(inputLabel[0]);

		aktivImg = Util
				.toBufferedImage(new ImageIcon(AdapterGoIO.class
						.getResource("resources/GoIOLinkAdapter_aktiv.png"))
						.getImage());

	}

	@Action
	public void clickA() {
		if (goIO.getSensor() == null) {
			// Sensorauswahl
			JList<String> liste = new JList<String>(sensoren);
			liste.setCellRenderer(new GoIOSensorCellRenderer());
			liste.setBorder(new BevelBorder(BevelBorder.LOWERED));
			liste.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			liste.setSelectedIndex(0);
			JScrollPane pane = new JScrollPane(liste);
			pane.setBorder(new TitledBorder("Sensoren"));
			int ret = JOptionPane.showConfirmDialog(null, pane,
					"Auswahl des Sensors", JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.PLAIN_MESSAGE);
			if (ret == JOptionPane.CANCEL_OPTION)
				return;
			String auswahl = (String) liste.getSelectedValue();
			GoIOSensor sensor = null;
			if (GHMessen.getApplication().vernierSensoren != null) {
				for (String s : GHMessen.getApplication().vernierSensoren
						.keySet())
					if (s.startsWith(auswahl.substring(0, auswahl.indexOf(")")))) {
						sensor = GHMessen.getApplication().vernierSensoren
								.get(s);
						break;
					}
			} else {
				String inhalt = GoIOSensorFactory.createSensorMap()
						.get(auswahl);
				sensor = GoIOSensor.readFromXml(inhalt);
			}
			if (sensor == null)
				return;
			goIO.setSensor(sensor);
		}
		inputClicked(0, inputButton[0]);
	}

	public void inputClicked(int abxy, JTransparentToggleButton b) {
		if (abxy >= eingaenge)
			return;
		CursorTk.startWaitCursor(b);
		if (b.isSelected()) { // Eingang aktivieren
			int index = b.getAuswahl();
			sb[abxy].setAuswahl(index);
			int kanaele = getKanaele(abxy);
			Groesse g;
			for (int ef = 0; ef < kanaele; ef++) {
				String name = erzeugeSymbol(index, abxy, ef);
				g = Util.findeGroesse(liste, name);
				if (g == null) {
					g = new Groesse(name);
					liste.add(g);
					setzeStandardWerte(g, abxy, ef, true);
				} else {
					setzeStandardWerte(g, abxy, ef, false);
				}
				aktiviereGroesse(g);
				aktiv[abxy] = true;
				setInputText(abxy, ef, g.getSymbol());
			}
		} else { // Eingang deaktivieren
			deaktiviereInput(0);
		}
		adjustImage();
		CursorTk.stopWaitCursor(b);
	}

	@Override
	protected void adjustImage() {
		super.adjustImage();
		if (aktiv[0])
			setImage(aktivImg);
		adjustSensorImage();
	}

	protected void adjustSensorImage() {
		showBox(true, aktiv[0]);
		showLabel(0, aktiv[0]);
	}

	protected void showBox(boolean show, boolean aktiv) {
		BufferedImage img = Util.toBufferedImage(getImage());
		String symbol = inputLabel[0].getText();
		symbol = HTML.enthaeltHTML(symbol) ? HTML.htmlZuSymbol(symbol) : symbol;
		BufferedImage boxImg;
		if (show)
			boxImg = Util.toBufferedImage(sb[0].getImageIcon(aktiv).getImage());
		else
			boxImg = null;
		int boxX = 49 + sb[0].imageXOffset() - boxImg.getWidth() / 2;
		int boxY = 110;
		img.getGraphics().drawImage(boxImg, boxX, boxY, this);
		this.setImage(img);
	}

	protected void deaktiviereInput(int ab, Eingang bisher) {
		Groesse g;
		for (int ef = 0; ef < bisher.getKanaele(); ef++) {
			String name = bisher.erzeugeSymbol(bisher.getAuswahl(), ab, ef);
			g = Util.findeGroesse(liste, name);
			if (g != null)
				deaktiviereGroesse(g);
		}
	}

	@Override
	public void adapterChanged(AdapterChangedEvent evt) {
		super.adapterChanged(evt);
		if (evt.adapter != this || evt.nr != nr)
			return;
		CursorTk.startWaitCursor(this);
		if (aktiv[evt.ab]) {
			deaktiviereInput(evt.ab, goIO);
			aktiv[evt.ab] = false;
			inputButton[evt.ab].setSelected(false);
		}
		goIO.close();
		try {
			goIO.open();
			if (evt.sensorNeu != 0)
				initGroessenfuerEingang(evt.ab);
			adjustImage();
		} catch (EAdapterException e) {
			if (DEBUG)
				System.out
						.println("[AdapterListe] - adapterChanged: Der geänderte Sensor konnte nicht geöffnet werden.");
		}
		CursorTk.stopWaitCursor(this);
	}

	@Override
	public void setzeStandardWerte(Groesse g, int AB, int ef, boolean neu) {
		g.setNr(nr);
		g.setAB(AB);
		g.setEF(ef);
		if (neu && sb != null && sb[AB] != null)
			sb[AB].setzeStandardWerte(g);
	}

	static {
		vendorID = VERNIER_DEFAULT_VENDOR_ID;
		productID = new HashSet<Integer>() {
			{
				add(LABPRO_DEFAULT_PRODUCT_ID);
				add(USB_DIRECT_TEMP_DEFAULT_PRODUCT_ID);
				add(SKIP_DEFAULT_PRODUCT_ID);
				add(CYCLOPS_DEFAULT_PRODUCT_ID);
				add(LOWCOST_SPEC_DEFAULT_PRODUCT_ID);
				add(MINI_GC_DEFAULT_PRODUCT_ID);
				add(STANDALONE_DAQ_DEFAULT_PRODUCT_ID);
			}
		};
		subsystem = new String[] { "usb" };
		deviceGroup = new String[] { "*-*" };
		ERSTER_SENSOR = 9;
	}

	@Override
	public String getEingangString(int aB) {
		if (DEBUG)
			System.out.println("[GoIOTyp] - getEingangString:");
		return "";
	}

	@Override
	public String getName() {
		return "GoIOLink";
	}

	@Override
	public int getEingangAnzahl(){
		return 1;
	}
	
	@Override
	public boolean istSensorEingang(int ab){
		return true;
	}
	
	@Override
	public IMesskanal getMesskanal(Groesse g) {
		return goIO.getMesskanal(g, null, true);
	}
}
