/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.adapter.goIO;

import ghmessen.GHMessen;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.EingangMehrereGroessen;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.Messkanal;
import ghmessen.adapter.MesskanalException;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.groesse.Messgroesse;
import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetGoIO;

import java.net.URL;
import java.util.HashMap;
import java.util.Vector;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import libGoIO.GoIOSensor;
import libGoIO.GoIOSensorFactory;
import libGoIO.LibGoIO;

import org.bridj.Pointer;

import udev.UdevDevice;

public class EingangGoIO extends EingangMehrereGroessen {

	private static final boolean DEBUG = false;

	private LibGoIO lib;
	private Pointer<?> handle;
	private UdevDevice device;
	private GoIOSensor sensor;
	private String sensorKey;

	public EingangGoIO(AdapterGoIO adapter, UdevDevice device, int input)
			throws EAdapterException {
		super(adapter, input);
		if (DEBUG)
			System.out.println("[EingangGoIO] - Konstruktor aufgerufen...");
		hatOszilloskop = false;
		this.device = device;
		try {
			if (GHMessen.getApplication().goIO == null) {
				if (DEBUG)
					System.out
							.println("[EingangGoIO] - Konstruktor: Globale Variable goIO wird initialisiert.");
				GHMessen.getApplication().goIO = new LibGoIO();
				lib = GHMessen.getApplication().goIO;
			} else {
				lib = GHMessen.getApplication().goIO;
			}
		} catch (IllegalStateException e) {
			if (DEBUG)
				System.out
						.println("[EingangGoIO] - Konstruktor: GHMessen ist nicht initialisiert.");
			lib = new LibGoIO();
		}
		open();
	}

	public void open() throws EAdapterException {
		if ((handle = lib.open(device.getUSBName())) == null) {
			throw new EAdapterException(
					"[EingangGoIO] - Konstruktor: Der Adapter " + device
							+ " konnte nicht geöffnet werden!");
		}
		int sensorNumber = lib.getSensorNumber(handle);
		if (sensorNumber != 0) {
			HashMap<String, GoIOSensor> vS;
			try {
				vS = GHMessen.getApplication().vernierSensoren;
				if (vS == null || vS.size() == 0) {
					if (DEBUG)
						System.out
								.println("[EingangGoIO] - open: Es wird die Gesamtsensorliste verwendet.");
					HashMap<String, String> smap = GoIOSensorFactory
							.createSensorMap();
					for (String val : smap.values()) {
						GoIOSensor sensor = GoIOSensor.readFromXml(val);
						vS.put(sensor.toString(), sensor);
						for (int i = 1; i < sensor.getNumAutoIds(); i++) {
							sensor = sensor.clone();
							sensor.setSensorNumber(sensor.getSensorNumber() + 1);
							vS.put(sensor.toString(), sensor);
						}
					}
				}
				for (GoIOSensor s : vS.values()) {
					if (s.getSensorNumber() == sensorNumber) {
						setSensor(s);
						break;
					}
				}
			} catch (IllegalStateException e) {
				if (DEBUG)
					System.out
							.println("[EingangGoIO] - Konstruktur: GHMessen ist nicht initialisiert.");
			}
			System.out.println("[EingangGoIO] - Konstruktor: Sensor-Id("
					+ sensorNumber + ") eingebunden.");
		} else {
			setSensor(null);
			if (messGroesse == null)
				messGroesse = new Vector<Messgroesse>();
		}
		hatVorzeichenWechsel = sensor == null ? false : sensor
				.isReverseSupported();
	}

	public void close() {
		if (lib != null) {
			if (DEBUG)
				System.out.println("[EingangGoIO] - close(" + handle + ")");
			lib.close(handle);
		}
	}

	@Override
	public double getIntervallStandard() {
		if (sensor == null)
			return super.getIntervallStandard();
		else if (sensor.getSensorDDSRec() == null)
			return sensor.getDelta();
		else
			return sensor.getSensorDDSRec().getTypSamplePeriod();
	}

	@Override
	public int getWiederholungStandard() {
		if (sensor == null)
			return super.getWiederholungStandard();
		else if (sensor.getSensorDDSRec() == null)
			return sensor.getNumSamples();
		else
			return sensor.getSensorDDSRec().getTypNumberOfSamples();
	}

	@Override
	public double getIntervallMinimum() {
		if (sensor == null)
			return super.getIntervallMinimum();
		else if (sensor.getSensorDDSRec() == null)
			return sensor.getMaxDelta();
		else
			return sensor.getSensorDDSRec().getMinSamplePeriod();
	}

	@Override
	public IMesskanal getMesskanal(Groesse g, ScheduledThreadPoolExecutor timer, boolean autocreate) {
		if(!autocreate && (messKanal == null || g.getEF() >= messKanal.length))
			return null;
		if (messKanal == null)
			messKanal = new Messkanal[getKanaele()];
		if (g.getEF() >= getKanaele()) {
			if (DEBUG)
				System.err
						.println("[EingangGoIO] - getMesskanal Kanalnummer zu groß EF("
								+ g.getEF() + ")");
			g.setEF(0);
		}
		if (messKanal[g.getEF()] == null)
			try {
				messKanal[g.getEF()] = new MesskanalGoIO(this, g, sensor
						.getCalibrationList().getCalibration(
								sensor.getSensorNumber(), auswahlB), lib,
						handle, sensor.isDds(), false);
			} catch (MesskanalException e) {
				if (DEBUG)
					e.printStackTrace();
			}
		return messKanal[g.getEF()];
	}

	@Override
	public Messgeraet getMessgeraet(JFrame parent, Groesse g,
			ScheduledThreadPoolExecutor hauptTimer) {
		return new MessgeraetGoIO(parent, (MesskanalGoIO) getMesskanal(g,
				hauptTimer, true), (AdapterGoIO) adapter, g, hauptTimer);
	}

	public void setSensor(GoIOSensor sensor) {
		setSensor(sensor == null ? "" : sensor.toString(), sensor);
	}

	public void setSensor(String sensorKey, GoIOSensor sensor) {
		this.sensorKey = sensorKey;
		this.sensor = sensor == null ? sensor : sensor.clone();
		try {
			if (lib.isSmartSensor(handle)) {
				sensor.setSensorDDSRec(lib.getSensorDDSRec(handle));
			} else if (sensor != null) {
				lib.setSensorDDSRec(handle, sensor.getSensorDDSRec());
			}
		} catch (EAdapterException e) {
			if (DEBUG)
				e.printStackTrace();
		}
		auswahl = 0;
		auswahlB = 0;
		if (sensor != null) {
			messGroesse = new Vector<Messgroesse>();
			try {
				messGroesse.add(Messgroesse.toMessgroesse(GoIOSensorFactory
						.getString(this.sensor.getMessgroesse())));
				aktualisiereMessbereiche();
				auswahlB = sensor.getDefaultIndex();
			} catch (IllegalArgumentException e) {
				if (DEBUG)
					e.printStackTrace();
				System.err
						.println("[EingangGoIO] - setSensor: Die Messgroesse "
								+ sensor.getMessgroesse()
								+ " wird noch nicht unterstützt.");
			}
			messKanal = null;
		}
		hatVorzeichenWechsel = sensor == null ? false : sensor
				.isReverseSupported();
	}

	public GoIOSensor getSensor() {
		return sensor;
	}

	public int getCurrentSensorNumber() throws EAdapterException {
		if (lib == null || handle == null)
			throw new EAdapterException(
					"[EingangGoIO] - getCurrentSensorNumber: lib oder handle == null.");
		return lib.getCurrentSensorNumber(handle);
	}

	public boolean isSmartSensor() {
		if (lib == null || handle == null)
			return false;
		try {
			return lib.isSmartSensor(handle);
		} catch (EAdapterException e) {
			if (DEBUG)
				e.printStackTrace();
			return false;
		}
	}

	public void setAuswahlB(int kanal, int auswahlB) {
		super.setAuswahlB(kanal, auswahlB);
		try {
			if (isSmartSensor())
				lib.setActiveCalPage(handle, auswahlB);
			else if (messKanal != null && messKanal.length > kanal
					&& messKanal[kanal] instanceof MesskanalGoIO)
				((MesskanalGoIO) messKanal[kanal]).setCalibration(sensor
						.getCalibrationList().getCalibration(
								sensor.getSensorNumber(), auswahlB));
		} catch (EAdapterException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	public void setSensorKey(String key) {
		sensorKey = key;
	}

	public String getSensorKey() {
		return sensorKey;
	}

	public UdevDevice getDevice() {
		return device;
	}

	public void setDevice(UdevDevice device) {
		this.device = device;
	}

	@Override
	protected void aktualisiereMessbereiche() {
		if (DEBUG)
			System.out.println("[EingangGoIO] - aktualisiereMessbereiche...");
		if (sensor == null)
			messbereich = new Vector<Messbereich>();
		else
			messbereich = sensor.getMessbereicheForSensorNumber();
	}

	@Override
	public ImageIcon getImageIcon(boolean aktiv) {
		URL url = null;
		if (sensor != null)
			url = sensor.getClass().getResource(
					"resources/" + "sensor_" + sensor.getSensorId() + ".png");
		else if (lib != null)
			url = lib.getClass().getResource(
					"resources/" + "sensor_UNKNOWN_ANALOG.png");
		if (url != null)
			return new ImageIcon(url);
		else
			return super.getImageIcon(aktiv);
	}

	@Override
	public boolean hatKorrektur(int kanal) {
		return sensor == null ? false : sensor.isShowZeroButton();
	}

}
