package ghmessen.components;

import static javax.swing.event.TableModelEvent.ALL_COLUMNS;
import static javax.swing.event.TableModelEvent.INSERT;
import static javax.swing.event.TableModelEvent.UPDATE;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Skalierung;
import ghmessen.groesse.darstellung.Eigenschaften;
import ghmessen.util.PlotUtil;
import ghmessen.util.Util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

public class Achse extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 199465516573052007L;
	private static final boolean DEBUG = false;

	public class SchriftPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
 * 
 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSchriftEvent(evt);
		}
	}

	public class LiniePropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
 * 
 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleLinieEvent(evt);
		}
	}

	public class IntervallHandler implements Serializable,
			PropertyChangeListener {
		/**
 * 
 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleIntervall(evt);
		}
	}

	public class SkalierungHandler implements Serializable,
			PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSkalierung(evt);
		}
	}

	public class SpalteHandler implements Serializable, PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSpalte(evt);
		}
	}

	public class TableHandler implements Serializable, TableModelListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void tableChanged(TableModelEvent evt) {
			handleTableChanged(evt);
		}
	}

	protected Skala skala;
	protected String einheit;
	protected String symbol;
	protected JLabel htmlLabel;
	protected Achsenstriche striche;
	protected Vector<JLabel> zahlen;
	protected int spalte;
	protected SchriftPropertyHandler schriftPH;
	protected LiniePropertyHandler liniePH;
	protected IntervallHandler intervallPH;
	protected SkalierungHandler skalierungPH;
	protected SpalteHandler spaltePH;
	protected TableHandler tablePH;

	// vertikal oder horizontal
	protected boolean vertikal;
	// links / unten oder rechts / oben
	protected boolean standard;

	// vertikale Achse: Abstand zwischen Strichen und Zahlen
	public static final int XSPACE = 5;
	// horizontale Achse: Abstand zwischen Strichen und Zahlen
	public static final int YSPACE = 5;

	public Achse(boolean neuVertikal, boolean standard) {
		super();
		this.vertikal = neuVertikal;
		this.standard = standard;
		spalte = -1;
		setLayout(null);
		htmlLabel = new JLabel();
		htmlLabel.setHorizontalAlignment(vertikal ? SwingConstants.LEFT
				: SwingConstants.RIGHT);
		htmlLabel.setOpaque(true);
		htmlLabel.setSize(htmlLabel.getPreferredSize());
		if (vertikal)
			htmlLabel.setLocation(
					standard ? 0 : Math.max(0,
							getWidth() - htmlLabel.getWidth()), 20);
		else
			htmlLabel.setLocation(20, 20);
		add(htmlLabel);
		striche = new Achsenstriche(vertikal, standard);
		add(striche);
		if (vertikal)
			striche.setLocation(
					standard ? Math.max(0, getWidth() - striche.getWidth()) : 0,
					0);
		else
			striche.setLocation(0,
					standard ? 0 : getHeight() - striche.getHeight());
		zahlen = new Vector<JLabel>();
		schriftPH = new SchriftPropertyHandler();
		liniePH = new LiniePropertyHandler();
		intervallPH = new IntervallHandler();
		skalierungPH = new SkalierungHandler();
		spaltePH = new SpalteHandler();
		tablePH = new TableHandler();
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				if (skala != null && zahlen != null) {
					skala.setLaenge(vertikal ? getHeight() : getWidth());
					positioniereZahlen();
					berechneParameter();
				}
			}
		});
		if (DEBUG)
			System.out.println("[Achse] - Konstruktor: "+symbol+"["+einheit+"]");
	}

	public Achse(boolean vertikal, boolean standard, String symbol,
			String einheit, double minimum, double maximum,
			double unterteilung, Skalierung skalierung) {
		this(vertikal, standard);
		this.symbol = symbol;
		this.einheit = einheit;
		this.setSkala(minimum, maximum, unterteilung, skalierung,
				vertikal ? getHeight() : getWidth());
		aktualisiereLabel();
		berechneParameter();
	}

	public void setSymbol(String symbol) {
		this.symbol = symbol;
		aktualisiereLabel();
	}

	public String getSymbol() {
		return symbol;
	}

	public void setEinheit(String einheit) {
		this.einheit = einheit;
		aktualisiereLabel();
	}

	public String getEinheit() {
		return einheit;
	}

	public BasicStroke getLinie() {
		if (striche == null)
			return null;
		return striche.getLinie();
	}

	public void setLinie(BasicStroke linie) {
		if (striche == null)
			return;
		this.striche.setLinie(linie);
	}

	public Skala getSkala() {
		return skala;
	}

	public double getMinimum() {
		return skala.getMinimum();
	}

	public double getMaximum() {
		return skala.getMaximum();
	}

	public void setSkalierung(Skalierung skalierung) {
		skala.setSkalierung(skalierung);
		aktualisiereLabel();
	}

	public Skalierung getSkalierung() {
		return skala.getSkalierung();
	}

	public void setSkala(double minimum, double maximum, double unterteilung) {
		skala = new Skala(minimum, maximum, unterteilung,
				skala == null ? Skalierung.y : skala.getSkalierung(), vertikal, vertikal? getHeight() : getWidth());
		aktualisiereLabel();
		if (striche != null)
			striche.setSkala(skala);
		if (zahlen != null) {
			aktualisiereZahlen();
			positioniereZahlen();
		}
	}

	public void setSkala(double minimum, double maximum, double unterteilung,
			Skalierung skalierung, double hoehe) {
		skala = new Skala(minimum, maximum, unterteilung, skalierung, vertikal,
				hoehe);
		aktualisiereLabel();
		if (striche != null)
			striche.setSkala(skala);
		if (zahlen != null) {
			aktualisiereZahlen();
			positioniereZahlen();
		}
	}

	public void setAchse(String symbol, String einheit, double min, double max,
			double unterteilung) {
		setAchse(symbol, einheit, min, max, unterteilung, Skalierung.y);
	}

	public void setAchse(String symbol, String einheit, double min, double max,
			double unterteilung, Skalierung skalierung) {
		this.symbol = symbol;
		this.einheit = einheit;
		this.setSkala(min, max, unterteilung, skalierung,
				vertikal ? getHeight() : getWidth());
		berechneParameter();
	}

	@Override
	public void setFont(Font font) {
		super.setFont(font);
		if (htmlLabel != null) {
			htmlLabel.setFont(font);
		}
		if (zahlen != null && zahlen.size() > 0)
			for (JLabel zahl : zahlen)
				zahl.setFont(font);
	}

	private Dimension getPreferredSizeVertikal() {
		Dimension d = new Dimension(0, 0);
		if (zahlen != null && zahlen.size() > 0)
			for (JLabel zahl : zahlen)
				d.setSize(
						Math.max(d.width, zahl.getPreferredSize().getWidth()
								+ XSPACE), 0);
		if (htmlLabel != null)
			d.setSize(d.width + htmlLabel.getPreferredSize().getWidth()
					+ XSPACE, 0);
		if (striche != null)
			d.setSize(d.width + striche.getPreferredSize().getWidth(), 0);
		d.setSize(d.width, Integer.MAX_VALUE);
		if (d.width == 0 && d.height == 0)
			d = super.getPreferredSize();
		if (DEBUG)
			System.out.println("[Achse] - getPreferredSizeVertikal: "+symbol+"(" + d + ")");
		return d;
	}

	public Dimension getPreferredSizeHorizontal() {
		Dimension d = new Dimension(0, 0);
		if (zahlen != null && zahlen.size() > 0)
			d.setSize(
					0,
					Math.max(d.height, zahlen.get(0).getPreferredSize()
							.getHeight()
							+ YSPACE));
		if (htmlLabel != null)
			d.setSize(0, d.height + htmlLabel.getPreferredSize().getHeight());
		if (striche != null)
			d.setSize(0, d.height + striche.getPreferredSize().getHeight());
		d.setSize(Integer.MAX_VALUE, d.getHeight());
		if (d.width == 0 && d.height == 0)
			d = super.getPreferredSize();
		if (DEBUG)
			System.out.println("[Achse] - getPreferredSizeHorizontal: "+symbol+"(" + d
					+ ")");
		return d;
	}

	@Override
	public Dimension getPreferredSize() {
		Dimension d = null;
		if (vertikal)
			d = getPreferredSizeVertikal();
		else
			d = getPreferredSizeHorizontal();
		if (DEBUG)
			System.out.println("[Achse] - getPreferredSize(" + d + ")");
		return d;
	}

	@Override
	public void paintComponent(Graphics g) {
		if (skala == null)
			return;
		super.paintComponent(g);
		Graphics g1 = g.create();
		if (DEBUG) {
			g1.setColor(Color.PINK);
			g1.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
			g1.setColor(Color.CYAN);
			for (Component c : this.getComponents())
				g1.drawRect(c.getX(), c.getY(), c.getWidth(), c.getHeight());
		}
		if (g1 != null)
			g1.dispose();
	}

	public void berechneParameter() {
		int size = vertikal ? getWidth() : getHeight();
		if (striche != null) {
			if (vertikal) {
				striche.setSize((int) striche.getPreferredSize().getWidth(),
						getHeight());
				if (standard)
					striche.setLocation(size - striche.getWidth(), 0);
			} else {
				striche.setSize(getWidth(), (int) striche.getPreferredSize()
						.getHeight());
				if (!standard)
					striche.setLocation(0, size - striche.getHeight());
			}
			striche.setPosKlein(skala.getStricheKlein());
			striche.setPosGross(skala.getStricheGross());
		}
		if (htmlLabel != null) {
			htmlLabel.setSize(htmlLabel.getPreferredSize());
			if (vertikal) {
				if (!standard)
					htmlLabel.setLocation(size - htmlLabel.getWidth(), 20);
			} else {
				htmlLabel.setLocation((int) getWidth() - htmlLabel.getWidth(),
						standard ? getHeight() - htmlLabel.getHeight() : 0);
			}
		}
		if (zahlen != null) {
			for (JLabel zahl : zahlen)
				zahl.setSize(zahl.getPreferredSize());
			positioniereZahlen();
		}
	}

	protected void aktualisiereLabel() {
		if (symbol == null || einheit == null || skala == null)
			return;
		String text = "<html>"
				+ skala.skaliereText(Util.htmlSub(symbol)
						.replace("_^^'", "</sub>'").replace("</html>", "")
						.replace("<html>", ""));
		htmlLabel.setText(text + (vertikal ? "<hr>" : "&nbsp;/&nbsp;")
				+ skala.skaliereText(einheit) + "</html>");
	}

	private void aktualisiereZahlen() { // Y
		String[] text = skala.getZahlen();
		int differenz = text.length - zahlen.size();
		JLabel zahl;
		for (int i = 0; i < differenz; i++) {
			zahlen.add(zahl = new JLabel());
			zahl.setFont(htmlLabel.getFont());
			add(zahl);
		}
		for (int i = 0; i > differenz; i--) {
			remove(zahlen.remove(zahlen.size() - 1));
		}
		for (int i = 0; i < text.length; i++) {
			zahlen.get(i).setText(text[i]);
		}
	}

	private void positioniereZahlen() {
		int width = getWidth();
		int height = getHeight();
		int[] posZahlen = skala.getStricheGross();
		JLabel zahl;
		if (vertikal) {
			for (int i = 0; i < zahlen.size(); i++) {
				zahl = zahlen.get(i);
				zahl.setLocation(
						standard ? width - striche.getWidth() - zahl.getWidth()
								- XSPACE : striche.getWidth() + XSPACE, Math
								.max(0, Math.min(height - zahl.getHeight(),
										posZahlen[i] - zahl.getHeight() / 2)));
			}
		} else {
			for (int i = 0; i < zahlen.size(); i++) {
				zahl = zahlen.get(i);
				zahl.setLocation(Math.max(
						0,
						Math.min(width - zahl.getWidth(),
								posZahlen[i] - zahl.getWidth() / 2)),
						standard ? striche.getHeight() + YSPACE : height
								- striche.getHeight() - zahl.getHeight()
								- YSPACE);
			}
		}

	}

	public void connectToSpalte(Darstellung ds, int spalte) {
		connectToSpalte(ds, spalte, null);
	}

	public void connectToSpalte(Darstellung ds, int spalte, String wechselPN) {
		ds.addPropertyChangeListener("yS" + spalte, skalierungPH);
		ds.addPropertyChangeListener("autoY" + spalte, skalierungPH);
		ds.addPropertyChangeListener("minimum" + spalte, intervallPH);
		ds.addPropertyChangeListener("maximum" + spalte, intervallPH);
		ds.addPropertyChangeListener("intervall" + spalte, intervallPH);
		if (wechselPN != null)
			ds.addPropertyChangeListener(wechselPN, spaltePH);
		this.spalte = spalte;
		ds.addTableModelListener(tablePH);
		setSymbol(ds.getColumnSymbol(spalte));
		setEinheit(ds.getColumnEinheit(spalte));
		handleSkalierung(new PropertyChangeEvent(ds, "yS" + spalte, null,
				ds.getYS(spalte)));
		handleIntervall(new PropertyChangeEvent(ds, "intervall" + spalte, null,
				new Intervall(ds.getMinimum(spalte), ds.getMaximum(spalte))));
	}

	public void disconnectFromSpalte(Darstellung ds, int spalte) {
		disconnectFromSpalte(ds, spalte, null);
	}

	public void disconnectFromSpalte(Darstellung ds, int spalte,
			String wechselPN) {
		ds.removePropertyChangeListener("yS" + spalte, skalierungPH);
		ds.removePropertyChangeListener("autoY" + spalte, skalierungPH);
		ds.removePropertyChangeListener("minimum" + spalte, intervallPH);
		ds.removePropertyChangeListener("maximum" + spalte, intervallPH);
		ds.removePropertyChangeListener("intervall" + spalte, intervallPH);
		if (wechselPN != null)
			ds.removePropertyChangeListener(wechselPN, spaltePH);
		this.spalte = -1;
		ds.removeTableModelListener(tablePH);
	}

	public void connectToEigenschaften(Eigenschaften ge) {
		ge.addPropertyChangeListener("schrift", schriftPH);
		ge.addPropertyChangeListener("schriftKlein", schriftPH);
		ge.addPropertyChangeListener("schriftMittel", schriftPH);
		ge.addPropertyChangeListener("schriftGross", schriftPH);
		ge.addPropertyChangeListener("linie", liniePH);
		ge.addPropertyChangeListener("linieSchmal", liniePH);
		ge.addPropertyChangeListener("linieMittel", liniePH);
		ge.addPropertyChangeListener("linieBreit", liniePH);
		setFont(ge.getSchrift());
		setLinie(ge.getLinie());
	}

	public void disconnectFromEigenschaften(Eigenschaften ge) {
		ge.removePropertyChangeListener("schrift", schriftPH);
		ge.removePropertyChangeListener("schriftKlein", schriftPH);
		ge.removePropertyChangeListener("schriftMittel", schriftPH);
		ge.removePropertyChangeListener("schriftGross", schriftPH);
		ge.removePropertyChangeListener("linie", liniePH);
		ge.removePropertyChangeListener("linieSchmal", liniePH);
		ge.removePropertyChangeListener("linieMittel", liniePH);
		ge.removePropertyChangeListener("linieBreit", liniePH);
	}

	public void handleSchriftEvent(PropertyChangeEvent evt) {
		Eigenschaften es = (Eigenschaften) evt.getSource();
		setFont(es.getSchrift());
		repaint();
	}

	public void handleLinieEvent(PropertyChangeEvent evt) {
		Eigenschaften es = (Eigenschaften) evt.getSource();
		setLinie(es.getLinie());
		repaint();
	}

	public void handleIntervall(PropertyChangeEvent evt) {
		Darstellung ds = (Darstellung) evt.getSource();
		int spalte = PlotUtil.getSpalteFromProperty(evt.getPropertyName());
		setSkala(ds.getMinimum(spalte), ds.getMaximum(spalte), -1.0);
		berechneParameter();
		repaint();
	}

	public void handleSkalierung(PropertyChangeEvent evt) {
		Darstellung ds = (Darstellung) evt.getSource();
		int i = PlotUtil.getSpalteFromProperty(evt.getPropertyName());
		setSkala(ds.getMinimum(i), ds.getMaximum(i), -1.0, ds.getYS(i),
				vertikal ? getHeight() : getWidth());
		berechneParameter();
		repaint();
	}

	public void handleSpalte(PropertyChangeEvent evt) {
		Darstellung ds = (Darstellung) evt.getSource();
		disconnectFromSpalte(ds, (Integer) evt.getOldValue(),
				evt.getPropertyName());
		int index = (Integer) evt.getNewValue();
		setAchse(ds.getColumnName(index), ds.getColumnEinheit(index),
				ds.getMinimum(index), ds.getMaximum(index), -1.0,
				ds.getYS(index));
		connectToSpalte(ds, index, evt.getPropertyName());
	}

	public void handleTableChanged(TableModelEvent evt) {
		Darstellung ds = (Darstellung) evt.getSource();
		if (spalte >= ds.getColumnCount())
			return;
		if (evt.getFirstRow() == TableModelEvent.HEADER_ROW) {
			setAchse(ds.getColumnSymbol(spalte), ds.getColumnEinheit(spalte),
					ds.getMinimum(spalte), ds.getMaximum(spalte), -1.0,
					ds.getYS(spalte));
			repaint();
		} else if (evt.getType() == INSERT && evt.getColumn() == ALL_COLUMNS
				|| evt.getType() == UPDATE) {
			if (ds.getMinimum(spalte) != skala.getMinimum()
					|| ds.getMaximum(spalte) != skala.getMaximum()) {
				setSkala(ds.getMinimum(spalte), ds.getMaximum(spalte), -1.0);
				repaint();
			}
		} else if (ds != null && ds.getRowCount() > 0) {
			setAchse(ds.getColumnSymbol(spalte), ds.getColumnEinheit(spalte),
					ds.getMinimum(spalte), ds.getMaximum(spalte), -1.0,
					ds.getYS(spalte));
			repaint();
		}
	}

	public static void main(String args[]) {
		@SuppressWarnings("serial")
		JFrame rahmen = new JFrame() {
			{
				getContentPane().add(
						new Achse(false, true, "y", "m", 0, 100, 10,
								Skalierung.y));
			}
		};
		rahmen.setSize(100, 300);
		rahmen.setVisible(true);
	}

}
