package ghmessen.components;

import ghmessen.Konstanten;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

@SuppressWarnings("serial")
public class Achsenstriche extends JPanel {

	private static final boolean DEBUG = false;
	private static final int MINKLEIN = 6;
	private static final int MINGROSS = 9;
	protected int laengeKlein;
	protected int laengeGross;
	protected BasicStroke linieKlein;
	protected BasicStroke linieGross;
	int[] posKlein;
	int[] posGross;
	protected Skala skala;
	// vertikal und (links oder rechts)
	// horizontal und (unten oder oben)
	protected boolean vertikal;
	protected boolean linksUnten;

	public Achsenstriche(boolean vertikal, boolean linksUnten) {
		super();
		this.skala = null;
		this.posKlein = null;
		this.posGross = null;
		this.vertikal = vertikal;
		this.linksUnten = linksUnten;
		linieKlein = new BasicStroke(Konstanten.GRAPH_LINIE_SCHMAL);
		linieGross = new BasicStroke(1.5f*Konstanten.GRAPH_LINIE_SCHMAL);
		laengeGross = Math.max(Math.round(MINGROSS*linieGross.getLineWidth()),MINGROSS);
		laengeKlein = Math.max(Math.round(MINKLEIN*linieKlein.getLineWidth()),MINKLEIN);
	}

	public void paintComponent(Graphics g) {
		Graphics2D g1 = (Graphics2D) g.create();
		g1.setStroke(linieKlein);
		paintStricheKlein(g1);
		g1.setStroke(linieGross);
		paintStricheGross(g1);
		if(DEBUG) {
			g1.setColor(Color.GREEN);
			g1.setStroke(((Graphics2D)g).getStroke());
			g1.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
		}
		g1.dispose();
	}

	private void paintStricheKlein(Graphics2D g) {
		Graphics g1 = g.create();
		g1.setColor(Color.BLACK);
		if (vertikal) {
			if (linksUnten)
				g1.translate(getWidth() - laengeKlein - 1, 0);
			if (posKlein.length > 0) {
				posKlein[posKlein.length - 1] = Math.max(0,
						posKlein[posKlein.length - 1]);
				posKlein[0] = Math.min(posKlein[0], getWidth() - 1);
				for (int i = 1; i < posKlein.length; i++)
					g1.drawLine(0, posKlein[i], laengeKlein, posKlein[i]);
			}
		} else {
			if (!linksUnten)
				g1.translate(0, getHeight() - laengeKlein - 1);
			if (posKlein.length > 0) {
				posKlein[0] = Math.max(0, posKlein[0]);
				posKlein[posKlein.length - 1] = (int)Math.min(
						posKlein[posKlein.length - 1], getWidth() - 1);
				for (int i = 0; i < posKlein.length; i++)
					g1.drawLine(posKlein[i], 0, posKlein[i], laengeKlein);
			}
		}
		g1.dispose();
	}

	private void paintStricheGross(Graphics2D g) {
		Graphics g1 = g.create();
		g1.setColor(Color.BLACK);
		if (vertikal) {
			if (linksUnten)
				g1.translate(getWidth() - laengeGross - 1, 0);
			if (posGross.length > 0) {
				posGross[posGross.length - 1] = Math.max(0,
						posGross[posGross.length - 1]);
				posGross[0] = Math.min(posGross[0], getHeight() - 1);
				for (int i = 0; i < posGross.length; i++)
					g1.drawLine(0, posGross[i], laengeGross, posGross[i]);
				int xAchse = linksUnten ? laengeGross : 0;
				g1.drawLine(xAchse, getHeight(), xAchse,
						0);
			}
		} else {
			if (!linksUnten)
				g1.translate(0, getHeight() - laengeGross - 1);
			if (posGross.length > 0) {
				posGross[0] = Math.max(0, posGross[0]);
				posGross[posGross.length - 1] = Math.min(
						posGross[posGross.length - 1], getWidth() - 1);
				for (int i = 0; i < posGross.length; i++)
					g1.drawLine(posGross[i], 0, posGross[i], laengeGross);
				int yAchse = linksUnten ? 0 : laengeGross;
				g1.drawLine(0, yAchse, getWidth(), yAchse);
			}
		}
	}

	@Override
	public void setSize(int width, int height) {
		super.setSize(width, height);
		if (skala != null) {
			posKlein = skala.getStricheKlein();
			posGross = skala.getStricheGross();
		}
	}

	@Override
	public void setSize(Dimension d) {
		super.setSize(d);
		if (skala != null) {
			posKlein = skala.getStricheKlein();
			posGross = skala.getStricheGross();
		}
	}

	@Override
	public Dimension getPreferredSize() {
		if(DEBUG)
			System.out.println("Achsenstriche - getPreferredSize: vertikal("+vertikal+")");
		if(vertikal)
			return new Dimension(laengeGross,Integer.MAX_VALUE);
		else
			return new Dimension(Integer.MAX_VALUE, laengeGross);
		//return super.getPreferredSize();
	}

	public BasicStroke getLinie() {
		return linieKlein;
	}

	public void setLinie(BasicStroke linie) {
		this.linieKlein = linie;
		this.linieGross = new BasicStroke(1.5f*linie.getLineWidth());
		laengeGross = Math.max(Math.round(MINGROSS*linieGross.getLineWidth()),MINGROSS);
		laengeKlein = Math.max(Math.round(MINKLEIN*linieKlein.getLineWidth()),MINKLEIN);
	}

	public int getLaengeKlein() {
		return laengeKlein;
	}

	public void setLaengeKlein(int laengeKlein) {
		this.laengeKlein = laengeKlein;
	}

	public int getLaengeGross() {
		return laengeGross;
	}

	public void setLaengeGross(int laengeGross) {
		this.laengeGross = laengeGross;
	}

	public int[] getPosKlein() {
		return posKlein;
	}

	public void setPosKlein(int[] posKlein) {
		this.posKlein = posKlein;
	}

	public int[] getPosGross() {
		return posGross;
	}

	public void setPosGross(int[] posGross) {
		this.posGross = posGross;
	}

	public Skala getSkala() {
		return skala;
	}

	public void setSkala(Skala skala) {
		this.skala = skala;
		if (skala != null) {
			posKlein = skala.getStricheKlein();
			posGross = skala.getStricheGross();
		} else {
			posKlein = null;
			posGross = null;
		}
	}
}
