/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import ghmessen.adapter.AdapterListe;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.cassy.AdapterCASSY;
import ghmessen.util.CursorTk;

import java.awt.Component;

import javax.swing.JPanel;

import org.jdesktop.observablecollections.ObservableMap;
import org.jdesktop.observablecollections.ObservableMapListener;

public class AdapterPanel extends JPanel implements ObservableMapListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -8698474615552651022L;
	@SuppressWarnings("unused")
	private static final boolean DEBUG = false;
	private AdapterListe adapter;

	public AdapterPanel(AdapterListe adapter) {
		super();
		this.adapter = adapter;
		this.adapter.addObservableMapListener(this);
		if (!adapter.isEmpty()) {
			CursorTk.startWaitCursor(this);
			for (IAdapter a : adapter.values())
				insertAdapter(a.getComponent());
			CursorTk.stopWaitCursor(this);
		}
	}

	private void insertAdapter(Component a) {
		addImpl(a, null, -1);
	}

	@Override
	protected void addImpl(Component comp, Object constraints, int index) {
		if (constraints != null
				|| index != -1
				|| !((comp instanceof IAdapter) || (comp instanceof AdapterCASSY.CASSYComponent))
				|| getComponentCount() < 1)
			super.addImpl(comp, constraints, index);
		Component[] child = getComponents();
		int nr = -1;
		if (comp instanceof IAdapter)
			nr = ((IAdapter) comp).getNr();
		else if (comp instanceof AdapterCASSY.CASSYComponent)
			nr = ((AdapterCASSY.CASSYComponent) comp).getAdapter().getNr();
		for (index = 0; index < child.length; index++) {
			boolean isAdapter = false;
			int childNr = -1;
			if (child[index] instanceof IAdapter) {
				isAdapter = true;
				childNr = ((IAdapter) child[index]).getNr();
			} else if (child[index] instanceof AdapterCASSY.CASSYComponent) {
				isAdapter = true;
				childNr = ((AdapterCASSY.CASSYComponent) child[index])
						.getAdapter().getNr();
			}
			if (!isAdapter || childNr > nr) {
				super.addImpl(comp, null, index);
				return;
			}
		}
		super.addImpl(comp, null, -1);
	}

	public void resetAll() {
		for (Component c : getComponents()) {
			if (c instanceof IAdapter)
				((IAdapter) c).reset();
			else if (c instanceof AdapterCASSY.CASSYComponent)
				((AdapterCASSY.CASSYComponent) c).getAdapter().reset();
		}
	}

	/*
	 * Observable Map Listener
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public void mapKeyValueChanged(ObservableMap map, Object index,
			Object oldValue) {
		return;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void mapKeyAdded(ObservableMap map, Object key) {
		CursorTk.startWaitCursor(this);
		if (map.get(key) instanceof IAdapter)
			insertAdapter(((IAdapter) map.get(key)).getComponent());
		CursorTk.stopWaitCursor(this);
		this.revalidate();
		this.repaint();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void mapKeyRemoved(ObservableMap list, Object key, Object oldValue) {
		CursorTk.startWaitCursor(this);
		if (oldValue instanceof IAdapter)
			remove(((IAdapter) oldValue).getComponent());
		CursorTk.stopWaitCursor(this);
		this.revalidate();
		this.repaint();
	}
}
