/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import ghmessen.parser.FormelGenerator;

import java.awt.Color;

import javax.swing.JFormattedTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.codehaus.janino.ExpressionEvaluator;

public class FormelFormattedTextField extends JFormattedTextField implements DocumentListener {

	private static final long serialVersionUID = 9181759691252704023L;
	private static final boolean DEBUG = false;
	private FormelGenerator p;
	private ExpressionEvaluator exp;
	
	public FormelFormattedTextField(AbstractFormatter format, Object value) {
		this(format);
		setValue(value);
	}
	
	public FormelFormattedTextField(AbstractFormatter format){
		super(format);
		if(DEBUG)
			System.out.println("[FormelFormattedTextField] - Konstruktor.");
		getDocument().addDocumentListener(this);
		p = new FormelGenerator();
	}

	public void setName(String s){
		p.setName(s);
	}
	
	public void setKonstanten(String[] k){
		p.setKonstanten(k);
	}
	
	public ExpressionEvaluator getEvaluator(){
		return exp;
	}
	
	private void parsen() {
		String term = getText();
		if(p.parsen(term)){
			setBackground(Color.WHITE);
			exp = p.getAusdruck(term);
		} else {
			setBackground(Color.ORANGE);
			exp = null;
		}
	}


	@Override
	public void insertUpdate(DocumentEvent e) {
		parsen();
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		parsen();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		return;
	}
}
