/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import java.awt.KeyEventDispatcher;
import static java.awt.event.KeyEvent.*;

import java.awt.event.KeyEvent;

public class GreekKeyEventDispatcher implements KeyEventDispatcher {

	private static final boolean DEBUG = false;
	private int lastVKey;
	private boolean altGraph;
	private boolean shift;

	public GreekKeyEventDispatcher() {
		super();
		lastVKey = VK_UNDEFINED;
		altGraph = false;
		shift = false;
	}

	public static String toGreek(String s) {
		int pos;
		while ((pos = s.indexOf("&")) > -1 && s.length() > pos + 1)
			s = s.substring(0, pos) + toGreek(s.charAt(pos + 1)) + s.substring(pos + 2);
		return s;
	}

	public static char toGreek(char c) {
		switch (c) {
		case 'a':
			return 'α';
		case 'b':
			return 'β';
		case 'C':
			return 'Γ';
		case 'c':
			return 'γ';
		case 'D':
			return 'Δ';
		case 'd':
			return 'δ';
		case 'F':
			return 'Φ';
		case 'f':
			return 'φ';
		case 'l':
			return 'λ';
		case 'm':
			return 'μ';
		case 'n':
			return 'ν';
		case 'O':
			return 'Ω';
		case 'o':
			return 'ω';
		case 'P':
			return 'Π';
		case 'p':
			return 'π';
		case 'r':
			return 'ρ';
		case 'S':
			return 'Σ';
		case 's':
			return 'σ';
		case 'T':
			return 'Θ';
		case 't':
			return 'ϑ';
		case 'W':
			return 'Ω';
		case 'w':
			return 'ω';
		default:
			return c;
		}
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent e) {
		if (e.isConsumed())
			return false;
		int code = e.getKeyCode();
		if (e.getID() == KEY_PRESSED) {
			switch (code) {
			case VK_ALT_GRAPH:
				altGraph = true;
				break;
			case VK_SHIFT:
				shift = true;
				break;
			default:
				lastVKey = code;
			}
			return false;
		}
		if (e.getID() == KEY_RELEASED) {
			switch (code) {
			case VK_ALT_GRAPH:
				altGraph = false;
				break;
			case VK_SHIFT:
				shift = false;
				break;
			default:
				lastVKey = VK_UNDEFINED;
			}
			return false;
		}
		if (altGraph && e.getID() == KeyEvent.KEY_TYPED) {
			if (DEBUG)
				System.out.println("[GreekKeyEventDispatcher] KeyCode(" + lastVKey + ") Event(" + e + ")");
			switch (lastVKey) {
			case VK_A:
				if (!shift)
					e.setKeyChar('α');
				break;
			case VK_B:
				if (!shift)
					e.setKeyChar('β');
				break;
			case VK_C:
				e.setKeyChar(shift ? 'Γ' : 'γ');
				break;
			case VK_D:
				e.setKeyChar(shift ? 'Δ' : 'δ');
				break;
			case VK_F:
				e.setKeyChar(shift ? 'Φ' : 'φ');
				break;
			case VK_L:
				if (!shift)
					e.setKeyChar('λ');
				break;
			case VK_M:
				if (!shift)
					e.setKeyChar('μ');
				break;
			case VK_N:
				if (!shift)
					e.setKeyChar('ν');
				break;
			case VK_O:
				e.setKeyChar(shift ? 'Ω' : 'ω');
				break;
			case VK_P:
				e.setKeyChar(shift ? 'Π' : 'π');
				break;
			case VK_R:
				if (!shift)
					e.setKeyChar('ρ');
				break;
			case VK_S:
				e.setKeyChar(shift ? 'Σ' : 'σ');
				break;
			case VK_T:
				e.setKeyChar(shift ? 'Θ' : 'ϑ');
				break;
			case VK_W:
				e.setKeyChar(shift ? 'Ω' : 'ω');
				break;
			}
			lastVKey = VK_UNDEFINED;
		}
		return false;
	}

}
