/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Action;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;

public class JTransparentToggleButton extends JToggleButton implements
		MouseListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6194627680695446042L;

	private static final boolean DEBUG = false;

	/* Popup-Menü */
	protected JPopupMenu popup;
	protected int nSymbole = 1;
	protected String[] symbole = new String[] { "U" };
	protected int ausgewaehlt = 0;

	public JTransparentToggleButton(Rectangle bounds, Action a) {
		super("");
		addMouseListener(this);
		setOpaque(false);
		setContentAreaFilled(false);
		setBorderPainted(false);
		setHideActionText(true);
		if (bounds != null)
			setBounds(bounds);
		setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		if (a != null)
			setAction(a);
		setName("");
		setBackground(Color.LIGHT_GRAY);
		initPopup();
	}

	public void setSymbole(String[] neu) {
		if (neu == null || neu.length < 1)
			return;
		symbole = neu;
		nSymbole = neu.length;
		ausgewaehlt = 0;
	}

	public void setAuswahl(int i) {
		if (i >= nSymbole)
			return;
		else
			ausgewaehlt = i;
		if (DEBUG)
			System.out
					.println("[JTransparentToggleButton] - setAuswahl: nSymbole("
							+ nSymbole + ") ausgewaehlt(" + ausgewaehlt + ")");
	}

	public int getAuswahl() {
		if (ausgewaehlt >= nSymbole)
			return 0;
		else
			return ausgewaehlt;
	}

	public String getAusgewaehltesSymbol() {
		if (ausgewaehlt >= nSymbole)
			return symbole[0];
		else
			return symbole[ausgewaehlt];
	}

	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g.create();
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
				0.5f));
		super.paint(g2);
		g2.dispose();
	}

	/************************************************************
	 * MouseListener
	 ************************************************************/
	protected void initPopup() {
		popup = null;
	}

	public JPopupMenu getPopup() {
		return popup;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		return;
	}

	@Override
	public void mousePressed(MouseEvent e) {
		if (popup != null && SwingUtilities.isRightMouseButton(e)) {
			if (e.isPopupTrigger())
				popup.show((Component) e.getSource(), e.getX(), e.getY());
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (popup != null && SwingUtilities.isRightMouseButton(e)) {
			if (e.isPopupTrigger())
				popup.show((Component) e.getSource(), e.getX(), e.getY());
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		return;
	}

	@Override
	public void mouseExited(MouseEvent e) {
		return;
	}
}
