/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import static ghmessen.Konstanten.FARBE_FORMEL;
import static ghmessen.Konstanten.FARBE_KONSTANTE;
import static ghmessen.Konstanten.FARBE_MESSGROESSE;
import ghmessen.groesse.Art;
import ghmessen.instrumente.Messgeraet;
import ghmessen.util.Util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.SoftBevelBorder;

public class MessgeraeteToggleButton extends JToggleButton {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5863114192873414319L;
	private static final boolean DEBUG = false;
	private static final int idealeBreite = 35;
	private static final int KREISDURCHMESSER = 8;
	private static final int RAND = 3;
	private String symbol;
	private Color artFarbe;
	Messgeraet m = null;
	private static final Border borderSelected = new SoftBevelBorder(
			BevelBorder.LOWERED, null, null, null, null);
	private static final Border borderUnselected = new SoftBevelBorder(
			BevelBorder.RAISED, null, null, null, null);

	public MessgeraeteToggleButton(Messgeraet m, Object symbol,
			boolean enabled, boolean selected) {
		super();
		if (DEBUG)
			System.out.println("[MessgeraeteToggleButton] - Konstruktor.");
		this.m = m;
		if (m != null) {
			setFarbe(m.getArt());
		}
		setEnabled(enabled);
		setSelected(selected);
		if (symbol instanceof String)
			this.symbol = (String) symbol;
		else if (m != null)
			this.symbol = m.getSymbol();
		else
			this.symbol = "{}";
		setMinimumSize(new Dimension(idealeBreite, idealeBreite));
		setPreferredSize(new Dimension(idealeBreite, idealeBreite));
		setMaximumSize(new Dimension(idealeBreite, idealeBreite));
		setBorder(isSelected() ? borderSelected : borderUnselected);
		setFont(new Font("Dialog", Font.BOLD, 12));
		setText(Util.htmlSub(this.symbol));
		addActionListener(new ActionListener() {
			Messgeraet m;

			ActionListener initialize(Messgeraet m) {
				this.m = m;
				return this;
			}

			@Override
			public void actionPerformed(ActionEvent evt) {
				setBorder(isSelected() ? borderSelected : borderUnselected);
				if (m != null)
					m.getGroesse().setAktiv(isSelected());
				m.setVisible(isEnabled() && isSelected());

			}

		}.initialize(m));
		addMouseListener(new MouseAdapter() {
			Messgeraet m;
			
			MouseAdapter initialize(Messgeraet m) {
				this.m = m;
				return this;
			}
			
			@Override
			public void mouseClicked(MouseEvent evt) {
				if(SwingUtilities.isRightMouseButton(evt) && m != null) {
					m.showEinstellungenBox();
				}
			}
		}.initialize(m));
	}

	public String getSymbol() {
		return symbol;
	}

	private void setFarbe(Art art) {
		artFarbe = Color.GRAY;
		if (art == null)
			return;
		switch (art) {
		default:
			break;
		case Messgroesse:
			artFarbe = FARBE_MESSGROESSE;
			break;
		case Konstante:
			artFarbe = FARBE_KONSTANTE;
			break;
		case Formel:
			artFarbe = FARBE_FORMEL;
			break;
		}
	}

	@Override
	public void setEnabled(boolean set) {
		super.setEnabled(set);
		setForeground(set ? Color.BLACK : Color.WHITE);
		if (m != null)
			m.setVisible(set && isSelected());
	}

	@Override
	public void setSelected(boolean set) {
		super.setSelected(set);
		setBorder(set ? borderSelected : borderUnselected);
		if (m != null) {
			m.setVisible(isEnabled() && set);
			m.getGroesse().setAktiv(set);
		}
	}

	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Color save = g.getColor();
		g.setColor(artFarbe);
		g.fillOval(getWidth() - KREISDURCHMESSER - RAND, RAND,
				KREISDURCHMESSER, KREISDURCHMESSER);
		g.setColor(save);
	}
}
