/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import ghmessen.instrumente.Messgeraet;
import ghmessen.instrumente.MessgeraetStatus;
import ghmessen.instrumente.MessgeraeteMap;

import java.awt.Component;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.xml.transform.sax.TransformerHandler;

import org.jdesktop.observablecollections.ObservableMap;
import org.jdesktop.observablecollections.ObservableMapListener;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class MessgeraeteToolBar extends JToolBar implements
		ObservableMapListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -1872671859613266948L;
	private static final boolean DEBUG = false;
	private MessgeraeteMap map;
	private JToggleButton alle;

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT Messgeraete (Messgeraet*)>");
		out.println("<!ELEMENT Messgeraet (MessgeraetStatus)>");
		out.println("<!ATTLIST Messgeraet name CDATA #REQUIRED>");
		MessgeraetStatus.createDTD(out);
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[MessgeraeteToolBar] Objekt ist nicht initialisiert.");
		hd.startElement("", "", "Messgeraete", null);
		Map<String, MessgeraetStatus> status = saveStatusMap();
		for (String key : status.keySet()) {
			AttributesImpl atts = new AttributesImpl();
			atts.addAttribute("", "", "name", "CDATA", key);
			hd.startElement("", "", "Messgeraet", atts);
			status.get(key).writeToXml(hd);
			hd.endElement("", "", "Messgeraet");
		}
		hd.endElement("", "", "Messgeraete");
	}

	private static transient String key;

	public static void startElement(String uri, String localName, String qName,
			Attributes attributes, Map<String, MessgeraetStatus> status)
			throws SAXException {
		if (qName.equals("Messgeraete")) {
			key = null;
			return;
		} else if (qName.equals("Messgeraet")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				String name = attributes.getQName(index);
				if (name.equals("name")) {
					key = attributes.getValue(index);
					status.put(key, new MessgeraetStatus());
				}
			}
		} else if (key != null)
			status.get(key).startElement(uri, localName, qName, attributes);
	}

	public static void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		return;
	}

	public MessgeraeteToolBar(MessgeraeteMap map, JToggleButton alle) {
		super();
		setFloatable(false);
		setRollover(true);
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		this.map = map;
		this.alle = alle;
		for (Iterator<? extends String> i = map.keySet().iterator(); i
				.hasNext();)
			mapKeyAdded(map, i.next());
	}

	public Map<String, MessgeraetStatus> saveStatusMap() {
		Map<String, MessgeraetStatus> result = new HashMap<String, MessgeraetStatus>();
		Messgeraet m;
		for (Component c : getComponents())
			if (c instanceof MessgeraeteToggleButton) {
				MessgeraeteToggleButton b = (MessgeraeteToggleButton) c;
				if ((m = map.get(b.getSymbol())) != null)
					result.put(
							b.getSymbol(),
							new MessgeraetStatus(b.isSelected(), m
									.getLocation(), m.getSize()));
			}
		return result;
	}

	public void restoreStatusMap(Map<String, MessgeraetStatus> neuerStatus) {
		Messgeraet m;
		MessgeraetStatus st;
		for (Component c : getComponents())
			if (c instanceof MessgeraeteToggleButton) {
				MessgeraeteToggleButton b = (MessgeraeteToggleButton) c;
				if (neuerStatus.containsKey(b.getSymbol())) {
					st = neuerStatus.get(b.getSymbol());
					b.setSelected(st.sichtbar);
					if ((m = map.get(b.getSymbol())) != null) {
						m.setLocation(st.position);
						m.setSize(st.ausdehnung);
					}
				}
			}
	}

	@Override
	public void setEnabled(boolean enabled) {
		Component c[] = getComponents();
		for (Component cc : c)
			cc.setEnabled(enabled);
		super.setEnabled(enabled);
	}

	public void setSelectedAlle(boolean an) {
		Component c[] = getComponents();
		for (Component jb : c)
			if (jb instanceof MessgeraeteToggleButton) {
				MessgeraeteToggleButton b = (MessgeraeteToggleButton) jb;
				b.setSelected(an);
			}
	}

	@Override
	public void mapKeyValueChanged(
			@SuppressWarnings("rawtypes") ObservableMap map, Object key,
			Object lastValue) {
		if (DEBUG)
			System.out.println("[MessgeraetetoolBar] mapKeyValueChanged");

	}

	@Override
	public void mapKeyAdded(@SuppressWarnings("rawtypes") ObservableMap map,
			Object key) {
		Messgeraet m = (Messgeraet) map.get(key);
		MessgeraeteToggleButton neu = new MessgeraeteToggleButton(m, key,
				alle.isSelected(), true);
		add(neu);
		this.revalidate();
	}

	@Override
	public void mapKeyRemoved(@SuppressWarnings("rawtypes") ObservableMap map,
			Object key, Object value) {
		Component[] c = getComponents();
		for (Component cc : c)
			if (cc instanceof MessgeraeteToggleButton) {
				MessgeraeteToggleButton b = (MessgeraeteToggleButton) cc;
				if (b.getSymbol().equals(key)) {
					remove(b);
					repaint();
					break;
				}
			}
		this.revalidate();
	}

}
