/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import static ghmessen.groesse.Darstellung.XACHSE;
import ghmessen.GHMessen;
import ghmessen.components.dialoge.Achsendialog;
import ghmessen.components.plotcanvas.PlotCanvas;
import ghmessen.dialoge.einstellungen.darstellung.YDialog;
import ghmessen.groesse.Darstellung;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import org.jdesktop.application.Action;

public class PlotPanel extends JPanel implements TableModelListener,
		ChangeListener {
	private static final long serialVersionUID = 5514881758930590919L;
	private static final boolean DEBUG = false;

	private class SkalierungAnPropertyHandler implements Serializable,
			PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			if (DEBUG)
				System.out
						.println("[PlotPanel] - SkalierungAn wurde aufgerufen...");
			boolean an = (Boolean) evt.getNewValue();
			boolean oben = ds != null
					&& ds.getGraphEigenschaften().isSkalierungOben();
			boolean achse2An = ds != null && ds.getColumnCount() > 2
					&& ds.getGraphEigenschaften().isAchse2An();
			adjustVisibility(an, oben, achse2An);
		}
	}

	private class SkalierungObenPropertyHandler implements Serializable,
			PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			if (DEBUG)
				System.out
						.println("[PlotPanel] - SkalierungAn wurde aufgerufen...");
			boolean an = ds != null
					&& ds.getGraphEigenschaften().isSkalierungAn();
			boolean oben = (Boolean) evt.getNewValue();
			boolean achse2An = ds != null && ds.getColumnCount() > 2
					&& ds.getGraphEigenschaften().isAchse2An();
			adjustVisibility(an, oben, achse2An);
		}
	}

	private class Achse2AnPropertyHandler implements Serializable,
			PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			if (DEBUG)
				System.out
						.println("[PlotPanel] - Achse2An wurde aufgerufen...");
			boolean an = ds != null
					&& ds.getGraphEigenschaften().isSkalierungAn();
			boolean achse2An = (Boolean) evt.getNewValue() && ds != null
					&& ds.getColumnCount() > 2;
			boolean oben = ds != null
					&& ds.getGraphEigenschaften().isSkalierungOben();
			adjustVisibility(an, oben, achse2An);
		}
	}

	private void adjustVisibility(boolean an, boolean oben, boolean achse2) {
		xachseu.setVisible(an && !oben);
		xachseo.setVisible(an && oben);
		leery1u.setVisible(an && !oben);
		yachse1.setVisible(an);
		leery1o.setVisible(an && oben);
		leery2u.setVisible(an && !oben && achse2);
		yachse2.setVisible(an && achse2);
		leery2o.setVisible(an && oben && achse2);
	}

	private PlotCanvas graph;
	private Achse xachseu, xachseo, yachse1, yachse2;
	private JLabel leery1u, leery2u, leery1o, leery2o;
	private JPanel toolbarPanel;
	private JToolBar yachse1Toolbar;
	private JToolBar yachse2Toolbar;
	private ButtonGroup yachse1Auswahl;
	private ButtonGroup yachse2Auswahl;
	private Achsendialog minimaxBox;
	private ActionMap actionMap;

	private Darstellung ds;
	private JTabbedPane tp;

	private SkalierungAnPropertyHandler skalierungAnPH;
	private SkalierungObenPropertyHandler skalierungObenPH;
	private Achse2AnPropertyHandler achse2AnPH;

	public PlotPanel(JTabbedPane tp) {
		this();
		if (DEBUG)
			System.out.println("[PlotPanel] - PlotPanel()");
		skalierungAnPH = new SkalierungAnPropertyHandler();
		skalierungObenPH = new SkalierungObenPropertyHandler();
		achse2AnPH = new Achse2AnPropertyHandler();
		this.tp = tp;
		this.tp.addChangeListener(this);
		this.ds = getSelectedDarstellung(tp);
		if (ds != null) {
			this.ds.addTableModelListener(this);
			xachseu.connectToSpalte(ds, XACHSE);
			xachseu.connectToEigenschaften(ds.getGraphEigenschaften());
			xachseo.connectToSpalte(ds, XACHSE);
			xachseo.connectToEigenschaften(ds.getGraphEigenschaften());
			yachse1.connectToSpalte(ds, ds.getY1(), "y1");
			yachse1.connectToEigenschaften(ds.getGraphEigenschaften());
			yachse2.connectToSpalte(ds, ds.getY2(), "y2");
			yachse2.connectToEigenschaften(ds.getGraphEigenschaften());
			this.ds.getGraphEigenschaften().addPropertyChangeListener(
					"skalierungAn", skalierungAnPH);
			this.ds.getGraphEigenschaften().addPropertyChangeListener(
					"achse2An", achse2AnPH);
			this.ds.getGraphEigenschaften().addPropertyChangeListener(
					"skalierungOben", skalierungObenPH);
			graph.setDarstellung(ds);
		}
	}

	@SuppressWarnings("serial")
	public PlotPanel() {
		super();
		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(PlotPanel.class, this);
		yachse1Auswahl = new ButtonGroup();
		yachse2Auswahl = new ButtonGroup();

		yachse1Toolbar = new JToolBar() {
			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				if (DEBUG) {
					Graphics g1 = g.create();
					g1.setColor(Color.YELLOW);
					g1.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
				}
			}
		};

		yachse1Toolbar.setFloatable(false);
		yachse1Toolbar.setLayout(new BoxLayout(yachse1Toolbar,
				BoxLayout.LINE_AXIS));
		yachse1Toolbar.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (ds != null) {
					YDialog dialog = new YDialog(ds);
					dialog.setVisible(true);
				}
			}
		});

		yachse2Toolbar = new JToolBar() {
			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				if (DEBUG) {
					Graphics g1 = g.create();
					g1.setColor(Color.MAGENTA);
					g1.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
				}
			}
		};
		yachse2Toolbar.setFloatable(false);
		yachse2Toolbar.setLayout(new BoxLayout(yachse2Toolbar,
				BoxLayout.LINE_AXIS));
		yachse2Toolbar.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (ds != null) {
					YDialog dialog = new YDialog(ds);
					dialog.setVisible(true);
				}
			}
		});
		graph = new PlotCanvas();

		ShowMiniMaxBox showMiniMaxBox = new ShowMiniMaxBox();
		xachseu = new Achse(false, true);
		xachseu.addMouseListener(showMiniMaxBox);
		xachseo = new Achse(false, false);
		xachseo.addMouseListener(showMiniMaxBox);
		xachseo.setVisible(false);

		yachse1 = new Achse(true, true);
		yachse1.addMouseListener(showMiniMaxBox);
		yachse2 = new Achse(true, false);
		yachse2.addMouseListener(showMiniMaxBox);
		yachse2.setVisible(false);

		toolbarPanel = new JPanel() {
			@Override
			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				if (DEBUG) {
					Graphics g1 = g.create();
					g1.setColor(Color.BLUE);
					g1.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
				}
			}
		};
		toolbarPanel.setLayout(new GridLayout(1, 2));
		toolbarPanel.add(yachse1Toolbar);
		toolbarPanel.add(yachse2Toolbar);

		leery1u = new JLabel("   ") {
			@Override
			public Dimension getPreferredSize() {
				if (yachse1 != null && xachseu != null)
					return new Dimension((int) yachse1.getPreferredSize()
							.getWidth(), (int) xachseu.getPreferredSize()
							.getHeight());
				else
					return super.getPreferredSize();
			}
		};
		leery2u = new JLabel("   ") {
			@Override
			public Dimension getPreferredSize() {
				if (yachse2 != null && xachseu != null)
					return new Dimension((int) yachse2.getPreferredSize()
							.getWidth(), (int) xachseu.getPreferredSize()
							.getHeight());
				else
					return super.getPreferredSize();
			}
		};
		leery2u.setVisible(false);
		leery1o = new JLabel("   ") {
			@Override
			public Dimension getPreferredSize() {
				if (yachse1 != null && xachseo != null)
					return new Dimension((int) yachse1.getPreferredSize()
							.getWidth(), (int) xachseo.getPreferredSize()
							.getHeight());
				else
					return super.getPreferredSize();
			}
		};
		leery1o.setVisible(false);
		leery2o = new JLabel("   ") {
			@Override
			public Dimension getPreferredSize() {
				if (yachse2 != null && xachseo != null)
					return new Dimension((int) yachse2.getPreferredSize()
							.getWidth(), (int) xachseo.getPreferredSize()
							.getHeight());
				else
					return super.getPreferredSize();
			}
		};
		leery2o.setVisible(false);
		setLayout(new BorderLayout());
		add(toolbarPanel, BorderLayout.NORTH);
		JPanel graphPanel = new JPanel();
		graphPanel.setLayout(new BorderLayout());
		graphPanel.setBorder(new EmptyBorder(new Insets(5, 5, 5, 5)));
		graphPanel.add(graph, BorderLayout.CENTER);
		graphPanel.add(yachse1, BorderLayout.WEST);
		graphPanel.add(yachse2, BorderLayout.EAST);
		JPanel xoPanel = new JPanel();
		xoPanel.setLayout(new BorderLayout());
		xoPanel.add(xachseo, BorderLayout.CENTER);
		xoPanel.add(leery1o, BorderLayout.WEST);
		xoPanel.add(leery2o, BorderLayout.EAST);
		graphPanel.add(xoPanel, BorderLayout.NORTH);
		JPanel xuPanel = new JPanel();
		xuPanel.setLayout(new BorderLayout());
		xuPanel.add(xachseu, BorderLayout.CENTER);
		xuPanel.add(leery1u, BorderLayout.WEST);
		xuPanel.add(leery2u, BorderLayout.EAST);
		graphPanel.add(xuPanel, BorderLayout.SOUTH);
		add(graphPanel, BorderLayout.CENTER);

	}

	public void setzeMessmodus(boolean an) {
		if (ds != null)
			ds.setzeMessmodus(an);
		if (graph != null && !an)
			graph.repaint();
	}

	/*
	 * Toolbar: showPanel Methoden
	 */
	public void removeAllButtons() {
		Enumeration<AbstractButton> btns = yachse1Auswahl.getElements();
		while (btns.hasMoreElements())
			yachse1Auswahl.remove(btns.nextElement());
		yachse1Auswahl.clearSelection();
		yachse1Toolbar.removeAll();
		btns = yachse2Auswahl.getElements();
		while (btns.hasMoreElements())
			yachse2Auswahl.remove(btns.nextElement());
		yachse2Auswahl.clearSelection();
		yachse2Toolbar.removeAll();
		yachse2Toolbar.add(Box.createHorizontalGlue());
		if (DEBUG)
			System.out.println("[PlotPanel] y1A("
					+ yachse1Auswahl.getButtonCount() + ") yachse1TB("
					+ yachse1Toolbar.getComponentCount() + ") y2A("
					+ yachse2Auswahl.getButtonCount() + ") y2TB("
					+ yachse2Toolbar.getComponentCount() + ")");
	}

	public void addButton(AchseToggleButton tb, boolean links) {
		if (links) {
			yachse1Auswahl.add(tb);
			yachse1Toolbar.add(tb);
		} else {
			yachse2Auswahl.add(tb);
			yachse2Toolbar.add(tb);
		}
		if (DEBUG)
			System.out.println("[PlotPanel] - addButton: symbol("
					+ tb.getSymbol() + ") links(" + links + ")");
	}

	public void adjustButtons() {
		removeAllButtons();
		if (ds == null)
			return;
		for (int i = 1; i < ds.getColumnCount(); i++) {
			addButton(
					new AchseToggleButton(actionMap.get("toggleAn"),
							graph.getFarbe(i), ds.getColumnSymbol(i),
							ds.getY1() == i), true);
			addButton(
					new AchseToggleButton(actionMap.get("toggleAn2"),
							graph.getFarbe(i), ds.getColumnSymbol(i),
							ds.getY2() == i), false);
		}
		boolean achse2 = ds.getColumnCount() > 2
				&& ds.getGraphEigenschaften().isSkalierungAn()
				&& ds.getGraphEigenschaften().isAchse2An();
		yachse2.setVisible(achse2);
		leery2u.setVisible(achse2);
		yachse2Toolbar.setVisible(achse2);
		toolbarPanel.revalidate();
	}

	@Action
	public void toggleAn(ActionEvent e) {
		if (ds == null)
			return;
		if (!(e.getSource() instanceof AchseToggleButton))
			return;
		AchseToggleButton tb = (AchseToggleButton) e.getSource();
		int index = ds.indexOf(tb.getSymbol());
		if(tb.isSelected())
			ds.setY1(index);
		if (DEBUG)
			System.out.println(System.currentTimeMillis()+"[PlotPanel] toggleAn: "+tb.toString());
	}

	@Action
	public void toggleAn2(ActionEvent e) {
		if (ds == null)
			return;
		if (!(e.getSource() instanceof AchseToggleButton))
			return;
		AchseToggleButton tb = (AchseToggleButton) e.getSource();
		int index = ds.indexOf(tb.getSymbol());
		if(tb.isSelected())
			ds.setY2(index);
		if (DEBUG)
			System.out.println("[PlotPanel] toggleAn2:" + tb.toString());
	}

	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (DEBUG) {
			Graphics g1 = g.create();
			g1.setColor(Color.RED);
			g1.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
		}
	}

	@Override
	public void tableChanged(TableModelEvent e) {
		if (e.getFirstRow() == TableModelEvent.HEADER_ROW) {
			if(e.getColumn() > XACHSE)
				adjustButtons();
			repaint();
		}
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		if (e.getSource() instanceof JTabbedPane) {
			Darstellung result = getSelectedDarstellung((JTabbedPane) e
					.getSource());
			if (ds != null) {
				ds.removeTableModelListener(this);
				xachseu.disconnectFromSpalte(ds, XACHSE);
				xachseu.disconnectFromEigenschaften(ds.getGraphEigenschaften());
				xachseo.disconnectFromSpalte(ds, XACHSE);
				xachseo.disconnectFromEigenschaften(ds.getGraphEigenschaften());
				yachse1.disconnectFromSpalte(ds, ds.getY1(), "y1");
				yachse1.disconnectFromEigenschaften(ds.getGraphEigenschaften());
				yachse2.disconnectFromSpalte(ds, ds.getY2(), "y2");
				yachse2.disconnectFromEigenschaften(ds.getGraphEigenschaften());
				ds.getGraphEigenschaften().removePropertyChangeListener(
						"skalierungAn", skalierungAnPH);
				ds.getGraphEigenschaften().removePropertyChangeListener(
						"achse2An", achse2AnPH);
				ds.getGraphEigenschaften().removePropertyChangeListener(
						"skalierungOben", skalierungObenPH);
				graph.setDarstellung(null);
			}
			ds = result;
			if (ds != null) {
				ds.addTableModelListener(this);
				xachseu.connectToSpalte(ds, XACHSE);
				xachseu.connectToEigenschaften(ds.getGraphEigenschaften());
				xachseo.connectToSpalte(ds, XACHSE);
				xachseo.connectToEigenschaften(ds.getGraphEigenschaften());
				yachse1.connectToSpalte(ds, ds.getY1(), "y1");
				yachse1.connectToEigenschaften(ds.getGraphEigenschaften());
				yachse2.connectToSpalte(ds, ds.getY2(), "y2");
				yachse2.connectToEigenschaften(ds.getGraphEigenschaften());
				ds.getGraphEigenschaften().addPropertyChangeListener(
						"skalierungAn", skalierungAnPH);
				ds.getGraphEigenschaften().addPropertyChangeListener(
						"achse2An", achse2AnPH);
				ds.getGraphEigenschaften().addPropertyChangeListener(
						"skalierungOben", skalierungObenPH);
				graph.setDarstellung(ds);
			}
			adjustButtons();
			repaint();
		}
	}

	public Darstellung getSelectedDarstellung(JTabbedPane tp) {
		Darstellung result = null;
		if (tp.getSelectedIndex() != -1
				&& tp.getSelectedComponent() instanceof TabellenPane) {
			TabellenPane tab = (TabellenPane) tp.getSelectedComponent();
			if (tab.getTabelle().getModel() instanceof Darstellung)
				result = (Darstellung) tab.getTabelle().getModel();
		}
		return result;
	}

	/*
	 * Dialog Minimum-Maximum-Skalierung
	 */
	private class ShowMiniMaxBox extends MouseAdapter {
		@Override
		public void mouseClicked(MouseEvent e) {
			if (!(e.getSource() instanceof Component)) {
				if (DEBUG)
					System.out
							.println("[PlotPanel] - ShowMiniMaxBox - mouseClicked: getSource ist keine Komponente!");
				return;
			}
			if (minimaxBox == null) {
				JFrame mainFrame = GHMessen.getApplication().getMainFrame();
				minimaxBox = new Achsendialog(
						mainFrame,
						true,
						ds,
						e.getSource() == xachseu || e.getSource() == xachseo ? Darstellung.XACHSE
								: (e.getSource() == yachse1 ? ds.getY1()
										: ds.getY2()));
			} else
				minimaxBox
						.setDarstellung(
								ds,
								e.getSource() == xachseu
										|| e.getSource() == xachseo ? Darstellung.XACHSE
										: (e.getSource() == yachse1 ? ds
												.getY1() : ds
												.getY2()));
			minimaxBox.setLocationRelativeTo((Component) e.getSource());
			minimaxBox.setVisible(true);
		}
	}

}
