/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import java.awt.Toolkit;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.text.ParseException;

import ghmessen.groesse.Groessenliste;

import javax.swing.JFormattedTextField;

public class SymbolFormattedTextField extends JFormattedTextField {

	private static final long serialVersionUID = 1L;

	public SymbolFormattedTextField(Groessenliste gl) {
		super();
		this.addFocusListener(new SymbolFocusAdapter(gl));
	}

	private class SymbolFocusAdapter extends FocusAdapter {
		private static final boolean DEBUG = false;
		private Groessenliste gl;

		public SymbolFocusAdapter(Groessenliste gl) {
			this.gl = gl;
		}

		@Override
		public void focusLost(FocusEvent e) {
			try {
				JFormattedTextField f = (JFormattedTextField) e.getSource();
				String alterWert = (String) f.getValue();
				String neuerWert = f.getText();
				if (neuerWert == null || neuerWert.equals("")
						|| neuerWert.contains(" ")) {
					f.setText(alterWert);
					f.commitEdit();
					Toolkit.getDefaultToolkit().beep();
				} else if (neuerWert.equals(alterWert))
					f.commitEdit();
				else {
					if (gl != null && gl.enthaeltSymbol(neuerWert)) {
						f.setText(alterWert);
						Toolkit.getDefaultToolkit().beep();
					}
					f.commitEdit();
				}
			} catch (ParseException e1) {
				if (DEBUG)
					e1.printStackTrace();
			}
		}
	}
}
