/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import static ghmessen.Konstanten.TABELLE_SCHRIFT_GROSS;
import static ghmessen.Konstanten.TABELLE_SCHRIFT_KLEIN;
import static ghmessen.Konstanten.TABELLE_SCHRIFT_MITTEL;
import static ghmessen.Konstanten.TABELLE_SCHRIFT_STANDARD;
import ghmessen.GHMessen;
import ghmessen.dialoge.einstellungen.darstellung.YDialog;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Groessenliste;
import ghmessen.util.ImageSelection;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ButtonGroup;
import javax.swing.DefaultCellEditor;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler;
import javax.swing.table.TableCellRenderer;

public class TabellenPane extends JScrollPane implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -2439947629401100375L;
	private static final boolean DEBUG = false;

	private TabellenMenue popup;
	private JDoubleTable tabelle;

	public TabellenPane(Darstellung tabelle) {
		this();
		this.tabelle.setModel(tabelle);
		popup = new TabellenMenue(tabelle);
		popup.fensterKopieren.addActionListener(this);
		MouseListener popupListener = new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				if (!e.isPopupTrigger())
					return;
				popup.show((Component) e.getSource(), e.getX(), e.getY());
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				if (!e.isPopupTrigger())
					return;
				popup.show((Component) e.getSource(), e.getX(), e.getY());
			}
		};
		this.tabelle.addMouseListener(popupListener);
		this.tabelle.getTableHeader().addMouseListener(popupListener);
		if (this.tabelle.getModel() instanceof Darstellung)
			((Darstellung) this.tabelle.getModel()).addPropertyChangeListener(
					"name", new PropertyChangeListener() {

						@Override
						public void propertyChange(PropertyChangeEvent evt) {
							if ("name".equals(evt.getPropertyName())) {
								if (getParent() != null
										&& getParent() instanceof JTabbedPane) {
									JTabbedPane tp = (JTabbedPane) getParent();
									int index = tp.indexOfTab((String) evt
											.getOldValue());
									if(index < 0)
										return;
									tp.setTitleAt(index,
											(String) evt.getNewValue());
								}
							}
						}

					});
	}

	public TabellenPane() {
		initComponents();
	}

	public String getName() {
		if (tabelle != null && tabelle.getModel() instanceof Darstellung)
			return ((Darstellung) tabelle.getModel()).getName();
		else
			return super.getName();
	}

	public void setName(String name) {
		if (tabelle != null && tabelle.getModel() instanceof Darstellung)
			((Darstellung) tabelle.getModel()).setName(name);
		else
			super.setName(name);		
	}
	
	public JTable getTabelle() {
		return tabelle;
	}

	private void initComponents() {
		tabelle = new JDoubleTable();
		setName("Größentabelle"); // NOI18N
		setViewportView(tabelle);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// kopiere das Fenster, falls das im Popup-Menü gewählt worden ist.
		// Alle anderen Aktionen werden im Menü selbst behandelt.
		Object src = e.getSource();
		if (src == popup.fensterKopieren) {
			try {
				Robot robot = new Robot();
				Rectangle captureSize = GHMessen.getApplication()
						.getMainFrame().getBounds();
				BufferedImage bufferedImage = robot
						.createScreenCapture(captureSize);
				Transferable timg = new ImageSelection(bufferedImage);
				Clipboard sysClip = Toolkit.getDefaultToolkit()
						.getSystemClipboard();
				sysClip.setContents(timg, null);
			} catch (AWTException ex) {
				if (DEBUG)
					System.out.println("[Fenster kopieren] fehlgeschlagen.");
			}
		}

	}

	class JDoubleTable extends JTable {
		private static final long serialVersionUID = 3500756505919688861L;

		JDoubleTable() {
			super();
			addKeyListener(new KeyAdapter() {

				@Override
				public void keyTyped(KeyEvent e) {
					if (e.getKeyChar() == 'l'
							&& e.getModifiersEx() == InputEvent.ALT_DOWN_MASK
							&& getModel() instanceof Darstellung) {
						Darstellung ds = (Darstellung) getModel();
						if (ds.getRowCount() > 0)
							ds.removeRow(ds.getRowCount() - 1);
					}

				}

			});
			setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			Font font = new Font("Dialog", Font.PLAIN, TABELLE_SCHRIFT_STANDARD);
			setFont(font);
			getTableHeader().setDefaultRenderer(
					new ColorTableHeaderCellRenderer());
			setRowHeight(getFontMetrics(font).getHeight());
			getTableHeader().setFont(font);
			((DefaultCellEditor) getDefaultEditor(Double.class)).getComponent()
					.setFont(font);
			setName("tabelle"); // NOI18N
		}

		@Override
		public TableCellRenderer getCellRenderer(int row, int column) {
			TableCellRenderer renderer = ((Darstellung) getModel())
					.getColumnRenderer(column);
			if (renderer == null) {
				renderer = super.getCellRenderer(row, column);
			}
			return renderer;
		}

	}

	class TabellenMenue extends JPopupMenu implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 4906247320272300488L;

		private final ButtonGroup schriftGroesse = new ButtonGroup();
		private Darstellung tabelle;

		private JMenuItem spaltenbelegung, zeileWeg, letztemessreiheWeg,
				messreihenWeg, tabKopieren;
		public JMenuItem fensterKopieren;
		private JRadioButtonMenuItem schriftKlein, schriftMittel, schriftGross;

		public TabellenMenue(Darstellung tabelle) {
			this.tabelle = tabelle;
			spaltenbelegung = new JMenuItem("Spaltenbelegung ändern");
			spaltenbelegung.addActionListener(this);
			add(spaltenbelegung);

			JMenu schriftmenu = new JMenu("Schriftgröße");
			add(schriftmenu);

			schriftKlein = new JRadioButtonMenuItem("Kleine Schrift");
			schriftKlein.addActionListener(this);
			schriftGroesse.add(schriftKlein);
			schriftKlein.setSelected(true);
			schriftmenu.add(schriftKlein);

			schriftMittel = new JRadioButtonMenuItem("Mittlere Schrift");
			schriftMittel.addActionListener(this);
			schriftGroesse.add(schriftMittel);
			schriftmenu.add(schriftMittel);

			schriftGross = new JRadioButtonMenuItem("Große Schrift");
			schriftGross.addActionListener(this);
			schriftGroesse.add(schriftGross);
			schriftmenu.add(schriftGross);

			JSeparator separator = new JSeparator();
			add(separator);

			zeileWeg = new JMenuItem("Letzte Zeile löschen");
			zeileWeg.addActionListener(this);
			zeileWeg.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L,
					InputEvent.ALT_MASK));
			add(zeileWeg);

			letztemessreiheWeg = new JMenuItem("Letzte Messreihe löschen");
			letztemessreiheWeg.addActionListener(this);
			add(letztemessreiheWeg);

			messreihenWeg = new JMenuItem("Alle Messreihen löschen");
			messreihenWeg.addActionListener(this);
			add(messreihenWeg);

			JSeparator separator_1 = new JSeparator();
			add(separator_1);

			tabKopieren = new JMenuItem("Tabelle kopieren");
			tabKopieren.addActionListener(this);
			add(tabKopieren);

			fensterKopieren = new JMenuItem("Fenster kopieren");
			fensterKopieren.addActionListener(this);
			add(fensterKopieren);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (tabelle == null)
				return;
			Object src = e.getSource();
			if (src == spaltenbelegung) {
				YDialog dialog = new YDialog(tabelle);
				dialog.setVisible(true);
			} else if (src == zeileWeg) {
				tabelle.removeRow(tabelle.getRowCount() - 1);
				GHMessen.getApplication().getMainView().entferneMesstask();
			} else if (src == letztemessreiheWeg) {
				Groessenliste gl = GHMessen.getApplication().getMainView().gl;
				if (gl.getMaxMessreihe() > 0)
					gl.removeLetzteMessreihe();
				else
					tabelle.setNumRows(0);
				GHMessen.getApplication().getMainView().entferneMesstask();
			} else if (src == messreihenWeg) {
				tabelle.setNumRows(0);
				Groessenliste gl = GHMessen.getApplication().getMainView().gl;
				gl.removeAlleMessreihen();
			} else if (src == fensterKopieren) {
				// Die Aktion wird in TabellenPane behandelt, damit das Menüfenster
				// nicht im Weg ist.
			} else {
				Component c = this.getInvoker();
				if (!(c instanceof JTable))
					return;
				JTable jt = (JTable) c;
				if (src == tabKopieren) {
					jt.selectAll();
					TransferHandler th = TableTransferHandler.defaultTransferHandler;
					Clipboard sysCb = Toolkit.getDefaultToolkit()
							.getSystemClipboard();
					th.exportToClipboard(jt, sysCb, TransferHandler.COPY);
					jt.clearSelection();
				} else if (src == schriftKlein || src == schriftMittel
						|| src == schriftGross) {
					int groesse = (src == schriftKlein ? TABELLE_SCHRIFT_KLEIN
							: (src == schriftMittel ? TABELLE_SCHRIFT_MITTEL
									: TABELLE_SCHRIFT_GROSS));
					Font font = new Font("Dialog", Font.PLAIN, groesse);
					jt.setFont(font);
					jt.setRowHeight(getFontMetrics(font).getHeight());
					jt.getTableHeader().setFont(font);
					((DefaultCellEditor) jt.getDefaultEditor(Double.class))
							.getComponent().setFont(font);
				}
			}
		}
	}

}
