/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.Transferable;
import java.text.NumberFormat;
import java.util.Locale;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.TransferHandler;

public class TableTransferHandler extends TransferHandler {

	/**
	 * 
	 */
	private static final long serialVersionUID = 194260714164586398L;

	public TableTransferHandler() {
		super();
	}

	public void exportToClipboard(JComponent comp, Clipboard clip, int action)
			throws IllegalStateException {

		if ((action == COPY || action == MOVE)
				&& (getSourceActions(comp) & action) != 0) {

			Transferable t = createTransferable(comp);
			if (t != null) {
				try {
					clip.setContents(t, null);
					exportDone(comp, t, action);
					return;
				} catch (IllegalStateException ise) {
					exportDone(comp, t, NONE);
					throw ise;
				}
			}
		}

		exportDone(comp, null, NONE);
	}

	public static final TransferHandler defaultTransferHandler = new TableTransferHandler();

	/**
	 * Create a Transferable to use as the source for a data transfer.
	 * 
	 * @param c
	 *            The component holding the data to be transfered. This argument
	 *            is provided to enable sharing of TransferHandlers by multiple
	 *            components.
	 * @return The representation of the data to be transfered.
	 * 
	 */
	public Transferable createTransferable(JComponent c) {
		if (c instanceof JTable) {
			JTable table = (JTable) c;
			int[] rows;
			int[] cols;

			if (!table.getRowSelectionAllowed()
					&& !table.getColumnSelectionAllowed()) {
				return null;
			}

			if (!table.getRowSelectionAllowed()) {
				int rowCount = table.getRowCount();

				rows = new int[rowCount];
				for (int counter = 0; counter < rowCount; counter++) {
					rows[counter] = counter;
				}
			} else {
				rows = table.getSelectedRows();
			}

			if (!table.getColumnSelectionAllowed()) {
				int colCount = table.getColumnCount();

				cols = new int[colCount];
				for (int counter = 0; counter < colCount; counter++) {
					cols[counter] = counter;
				}
			} else {
				cols = table.getSelectedColumns();
			}

			if (rows == null || cols == null || rows.length == 0
					|| cols.length == 0) {
				return null;
			}

			StringBuffer plainBuf = new StringBuffer();
			StringBuffer htmlBuf = new StringBuffer();

			htmlBuf.append("<table>\n");
			// Überschriften
			htmlBuf.append("<tr>\n");
			for (int col = 0; col < cols.length; col++) {
				String val = table.getColumnName(col).replaceAll("<html>", "")
						.replaceAll("</html>", "");
				plainBuf.append(val + "\t");
				htmlBuf.append("  <td>" + val + "</td>\n");
			}
			// we want a newline at the end of each line and not a tab
			plainBuf.deleteCharAt(plainBuf.length() - 1).append("\n");
			htmlBuf.append("</tr>\n");
			// Tabelleninhalt
			for (int row = 0; row < rows.length; row++) {
				htmlBuf.append("<tr>\n");
				for (int col = 0; col < cols.length; col++) {
					Object obj = table.getValueAt(rows[row], cols[col]);
					String val;
					if (obj instanceof Double && obj != null)
						val = NumberFormat.getInstance(Locale.getDefault())
								.format(obj);
					else
						val = ((obj == null) ? "" : obj.toString());
					plainBuf.append(val.replace(",", ".") + "\t");
					htmlBuf.append("  <td>" + val + "</td>\n");
				}
				// we want a newline at the end of each line and not a tab
				plainBuf.deleteCharAt(plainBuf.length() - 1).append("\n");
				htmlBuf.append("</tr>\n");
			}

			// remove the last newline
			plainBuf.deleteCharAt(plainBuf.length() - 1);
			htmlBuf.append("</table>\n");

			return new TableTransferable(plainBuf.toString(),
					htmlBuf.toString());
		}

		return null;
	}

	public int getSourceActions(JComponent c) {
		return COPY;
	}

}