/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.dialoge;

import ghmessen.GHMessen;
import ghmessen.components.Intervall;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Einheit;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.Skalierung;
import ghmessen.util.Util;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.ActionMap;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;

/**
 * 
 * @author fschuett
 */

/* FIXME Logarithmische Darstellung funktioniert nicht einwandfrei. */
public class Achsendialog extends javax.swing.JDialog {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3214645145114234050L;
	private static final boolean DEBUG = false;

	private Darstellung ds;
	private int index;
	private boolean firePC;
	private static ActionMap actionMap;
	private static ResourceMap resourceMap;

	public Achsendialog(java.awt.Frame parent, boolean modal, Darstellung ds,
			int index) {
		super(parent, modal);
		this.ds = ds;
		this.index = index;
		initComponents();
		firePC = true;
		adjustComponents();
		autoSkalierung();
		if (DEBUG)
			System.out.println("[Achsendialog] - init");
	}

	/**
	 * Creates new form Achsendialog
	 * 
	 * @wbp.parser.constructor
	 */
	public Achsendialog(java.awt.Frame parent, boolean modal) {
		this(parent, modal, null, -1);
		setModal(true);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {

		actionMap = Application.getInstance(GHMessen.class).getContext()
				.getActionMap(Achsendialog.class, this);
		resourceMap = Application.getInstance(GHMessen.class).getContext()
				.getResourceMap(Achsendialog.class);

		skalierung = new javax.swing.ButtonGroup();
		jPanel1 = new javax.swing.JPanel();
		y = new javax.swing.JRadioButton(actionMap.get("y"));
		y2 = new javax.swing.JRadioButton(actionMap.get("y2"));
		_y = new javax.swing.JRadioButton(actionMap.get("_y"));
		_y2 = new javax.swing.JRadioButton(actionMap.get("_y2"));
		logy = new javax.swing.JRadioButton(actionMap.get("logy"));
		sqrty = new javax.swing.JRadioButton(actionMap.get("sqrty"));
		alleAchsen = new javax.swing.JCheckBox(actionMap.get("alleAchsen"));
		schliessen = new javax.swing.JButton(actionMap.get("schliessen"));
		hilfe = new javax.swing.JButton();
		autoSkalierung = new javax.swing.JCheckBox(
				actionMap.get("autoSkalierung"));
		jPanel2 = new javax.swing.JPanel();
		minLabel = new javax.swing.JLabel();
		minimum = new javax.swing.JFormattedTextField();
		maxLabel = new javax.swing.JLabel();
		maximum = new javax.swing.JFormattedTextField();
		miniMax = new javax.swing.JButton(actionMap.get("miniMax"));

		setDefaultCloseOperation(javax.swing.WindowConstants.HIDE_ON_CLOSE);

		jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				resourceMap.getString("umrechnung.text"),
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 0, 12))); // NOI18N
		jPanel1.setName("jPanel1"); // NOI18N

		skalierung.add(y);
		y.setFont(new java.awt.Font("Dialog", 0, 10));
		y.setActionCommand("y");

		skalierung.add(y2);
		y2.setFont(new java.awt.Font("Dialog", 0, 10));
		y2.setActionCommand("y^2");

		skalierung.add(_y);
		_y.setFont(new java.awt.Font("Dialog", 0, 10));
		_y.setActionCommand("1/y");

		skalierung.add(_y2);
		_y2.setFont(new java.awt.Font("Dialog", 0, 10));
		_y2.setActionCommand("1/y^2");

		skalierung.add(logy);
		logy.setFont(new java.awt.Font("Dialog", 0, 10));
		logy.setActionCommand("log y");

		skalierung.add(sqrty);
		sqrty.setFont(new java.awt.Font("Dialog", 0, 10));
		sqrty.setActionCommand("√y");

		javax.swing.GroupLayout gl_jPanel1 = new javax.swing.GroupLayout(
				jPanel1);
		jPanel1.setLayout(gl_jPanel1);
		gl_jPanel1.setHorizontalGroup(gl_jPanel1.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				gl_jPanel1.createSequentialGroup().addContainerGap()
						.addComponent(y).addGap(18, 18, 18).addComponent(y2)
						.addGap(18, 18, 18).addComponent(_y).addGap(18, 18, 18)
						.addComponent(_y2).addGap(18, 18, 18)
						.addComponent(logy).addGap(18, 18, 18)
						.addComponent(sqrty)
						.addContainerGap(10, Short.MAX_VALUE)));
		gl_jPanel1
				.setVerticalGroup(gl_jPanel1
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jPanel1
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jPanel1
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(y)
														.addComponent(y2)
														.addComponent(_y)
														.addComponent(_y2)
														.addComponent(logy)
														.addComponent(sqrty))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		alleAchsen.setFont(new java.awt.Font("Dialog", 0, 12));

		schliessen.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N

		hilfe.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
		hilfe.setText(resourceMap.getString("hilfe.text"));
		hilfe.setEnabled(false);
		hilfe.setName("hilfe"); // NOI18N

		autoSkalierung.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N

		jPanel2.setBorder(javax.swing.BorderFactory
				.createLineBorder(new java.awt.Color(0, 0, 0)));
		jPanel2.setName("jPanel2"); // NOI18N

		minLabel.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
		minLabel.setText(resourceMap.getString("minimum.text"));
		minLabel.setName("minLabel"); // NOI18N
		minLabel.setLabelFor(minimum);

		minimum.setName("minimum"); // NOI18N
		minimum.addPropertyChangeListener("value",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent e) {
						if (ds != null && firePC) {
							if (((Number) maximum.getValue()).doubleValue() < ((Number) e
									.getNewValue()).doubleValue()) {
								ds.setIntervall(
										index,
										new Intervall(
												((Number) e.getNewValue())
														.doubleValue(),
												((Number) e.getNewValue())
														.doubleValue()));
								firePC = false;
								maximum.setValue(e.getNewValue());
								firePC = true;
							} else
								ds.setIntervall(
										index,
										new Intervall(
												((Number) e.getNewValue())
														.doubleValue(),
												((Number) maximum.getValue())
														.doubleValue()));

						}
					}
				});
		maxLabel.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
		maxLabel.setText(resourceMap.getString("maximum.text"));
		maxLabel.setName("maxLabel"); // NOI18N
		maxLabel.setLabelFor(maximum);

		maximum.setName("maximum"); // NOI18N
		maximum.addPropertyChangeListener("value",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent e) {
						if (ds != null && firePC) {
							if (((Number) minimum.getValue()).doubleValue() > ((Number) e
									.getNewValue()).doubleValue()) {
								ds.setIntervall(
										index,
										new Intervall(
												((Number) e.getNewValue())
														.doubleValue(),
												((Number) e.getNewValue())
														.doubleValue()));
								firePC = false;
								minimum.setValue(e.getNewValue());
								firePC = true;
							} else
								ds.setIntervall(
										index,
										new Intervall(((Number) minimum
												.getValue()).doubleValue(),
												((Number) e.getNewValue())
														.doubleValue()));

						}
					}
				});
		miniMax.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N

		javax.swing.GroupLayout gl_jPanel2 = new javax.swing.GroupLayout(
				jPanel2);
		jPanel2.setLayout(gl_jPanel2);
		gl_jPanel2
				.setHorizontalGroup(gl_jPanel2
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jPanel2
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jPanel2
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																miniMax,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																288,
																Short.MAX_VALUE)
														.addGroup(
																gl_jPanel2
																		.createParallelGroup(
																				javax.swing.GroupLayout.Alignment.LEADING,
																				false)
																		.addGroup(
																				gl_jPanel2
																						.createSequentialGroup()
																						.addComponent(
																								minLabel)
																						.addPreferredGap(
																								javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								minimum,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								96,
																								javax.swing.GroupLayout.PREFERRED_SIZE))
																		.addGroup(
																				gl_jPanel2
																						.createSequentialGroup()
																						.addComponent(
																								maxLabel)
																						.addPreferredGap(
																								javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																						.addComponent(
																								maximum,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								96,
																								javax.swing.GroupLayout.PREFERRED_SIZE))))
										.addContainerGap()));
		gl_jPanel2
				.setVerticalGroup(gl_jPanel2
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jPanel2
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jPanel2
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(minLabel)
														.addComponent(
																minimum,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												gl_jPanel2
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(maxLabel)
														.addComponent(
																maximum,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												9, Short.MAX_VALUE)
										.addComponent(miniMax)
										.addContainerGap()));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addGap(12, 12,
																		12)
																.addGroup(
																		layout.createParallelGroup(
																				javax.swing.GroupLayout.Alignment.LEADING)
																				.addComponent(
																						autoSkalierung,
																						javax.swing.GroupLayout.PREFERRED_SIZE,
																						227,
																						javax.swing.GroupLayout.PREFERRED_SIZE)
																				.addComponent(
																						alleAchsen)))
												.addGroup(
														layout.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING,
																false)
																.addGroup(
																		layout.createSequentialGroup()
																				.addContainerGap()
																				.addComponent(
																						jPanel2,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE))
																.addGroup(
																		layout.createSequentialGroup()
																				.addGap(12,
																						12,
																						12)
																				.addComponent(
																						jPanel1,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)))
												.addGroup(
														javax.swing.GroupLayout.Alignment.TRAILING,
														layout.createSequentialGroup()
																.addContainerGap()
																.addComponent(
																		hilfe)
																.addGap(18, 18,
																		18)
																.addComponent(
																		schliessen,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		115,
																		javax.swing.GroupLayout.PREFERRED_SIZE)))
								.addContainerGap(
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(jPanel1,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addGap(18, 18, 18)
								.addComponent(alleAchsen)
								.addGap(1, 1, 1)
								.addComponent(autoSkalierung)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(jPanel2,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addGap(18, 18, 18)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(schliessen)
												.addComponent(hilfe))
								.addContainerGap(
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)));

		pack();
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JRadioButton _y;
	private javax.swing.JRadioButton _y2;
	private javax.swing.JCheckBox alleAchsen;
	private javax.swing.JCheckBox autoSkalierung;
	private javax.swing.JButton hilfe;
	private javax.swing.JLabel minLabel;
	private javax.swing.JLabel maxLabel;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JRadioButton logy;
	private javax.swing.JButton miniMax;
	private javax.swing.JFormattedTextField minimum;
	private javax.swing.JFormattedTextField maximum;
	private javax.swing.JButton schliessen;
	private javax.swing.ButtonGroup skalierung;
	private javax.swing.JRadioButton y;
	private javax.swing.JRadioButton y2;
	private javax.swing.JRadioButton sqrty;

	// End of variables declaration//GEN-END:variables

	private void adjustComponents() {
		firePC = false;
		alleAchsen.setEnabled(this.index > Darstellung.XACHSE);
		alleAchsen.setSelected(false);
		String a = index == Darstellung.XACHSE ? "x" : "y";
		y.setText(a);
		y2.setText(a + "^2");
		_y.setText("1/" + a);
		_y2.setText("1/" + a + "^2");
		logy.setText("log " + a);
		sqrty.setText("√" + a);
		setSkalierung(ds.getYS(index));
		autoSkalierung.setSelected(ds.getAutoY(index));
		minimum.setValue(ds.getMinimum(index));
		maximum.setValue(ds.getMaximum(index));
		firePC = true;
	}

	public void setDarstellung(Darstellung ds, int index) {
		this.ds = ds;
		this.index = index;
		adjustComponents();
		autoSkalierung();
	}

	public Darstellung getDarstellung() {
		return ds;
	}

	public int getIndex() {
		return index;
	}

	private Skalierung getSkalierung() {
		String s = skalierung.getSelection().getActionCommand();
		s = s.replace("x", "y");
		return Skalierung.toSkalierung(s);
	}

	private void setSkalierung(Skalierung s) {
		String sn = s.toString();
		Enumeration<AbstractButton> m = skalierung.getElements();
		while (m.hasMoreElements()) {
			AbstractButton b = m.nextElement();
			if (b.getActionCommand().equals(sn)) {
				skalierung.setSelected(b.getModel(), true);
			}
		}
	}

	private void alleAchsenAnpassen(Einheit e, Skalierung s) {
		Groessenliste gl = GHMessen.getApplication().getMainView().gl;
		Groesse g;
		for (int i = 0; i < ds.getColumnCount(); i++) {
			g = gl.get(i);
			if (g.getEinheit() == e)
				ds.setYS(i, s);
		}
	}

	@Action
	public void schliessen() {
		dispose();
	}

	@Action
	public void miniMax() {
		firePC = false;
		Double[] minMax = new Double[2];
		ds.findeMinMax(index, minMax);
		minimum.setValue(minMax[0]);
		maximum.setValue(minMax[1]);
		ds.setIntervall(index, new Intervall((Double) minimum.getValue(),
				(Double) maximum.getValue()));
		firePC = true;
	}

	@Action
	public void autoSkalierung() {
		boolean autoAn = autoSkalierung.isSelected();
		if (firePC)
			ds.setAutoY(index, autoAn);
		minLabel.setEnabled(!autoAn);
		maxLabel.setEnabled(!autoAn);
		minimum.setEnabled(!autoAn);
		maximum.setEnabled(!autoAn);
		miniMax.setEnabled(!autoAn);
	}

	@Action
	public void alleAchsen() {
		if (!firePC)
			return;
		Groesse g = Util.findeGroesse(
				GHMessen.getApplication().getMainView().gl,
				ds.getColumnSymbol(index));
		if (g == null)
			return;
		Einheit einheit = g.getEinheit();
		Skalierung s = alleAchsen.isSelected() ? getSkalierung() : Skalierung.y;
		alleAchsenAnpassen(einheit, s);
	}

	private void skalierungAnpassen() {
		Skalierung s = getSkalierung();
		if (alleAchsen.isSelected()) {
			Groesse g = Util.findeGroesse(GHMessen.getApplication()
					.getMainView().gl, ds.getColumnSymbol(index));
			if (g == null)
				return;
			Einheit einheit = g.getEinheit();
			alleAchsenAnpassen(einheit, s);
		} else
			ds.setYS(index, s);
	}

	@Action
	public void y() {
		if (firePC)
			skalierungAnpassen();
	}

	@Action
	public void y2() {
		if (firePC)
			skalierungAnpassen();
	}

	@Action
	public void _y() {
		if (firePC)
			skalierungAnpassen();
	}

	@Action
	public void _y2() {
		if (firePC)
			skalierungAnpassen();
	}

	@Action
	public void logy() {
		if (firePC)
			skalierungAnpassen();
	}

	@Action
	public void sqrty() {
		if (firePC)
			skalierungAnpassen();
	}
}
