/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.dialoge;

import static ghmessen.groesse.darstellung.Eigenschaften.AKIMAINTERPOLATION;
import static ghmessen.groesse.darstellung.Eigenschaften.KEINEINTERPOLATION;
import static ghmessen.groesse.darstellung.Eigenschaften.LINIEBREIT;
import static ghmessen.groesse.darstellung.Eigenschaften.LINIEMITTEL;
import static ghmessen.groesse.darstellung.Eigenschaften.LINIESCHMAL;
import static ghmessen.groesse.darstellung.Eigenschaften.SCHRIFTGROSS;
import static ghmessen.groesse.darstellung.Eigenschaften.SCHRIFTKLEIN;
import static ghmessen.groesse.darstellung.Eigenschaften.SCHRIFTMITTEL;
import static ghmessen.groesse.darstellung.Eigenschaften.SINCINTERPOLATION;
import ghmessen.GHMessen;
import ghmessen.groesse.WerteTabelle;
import ghmessen.groesse.darstellung.Eigenschaften;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.TitledBorder;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;

/**
 * 
 * @author fschuett
 */
public class CanvasEigenschaften extends javax.swing.JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8146320513982187286L;
	private Eigenschaften es;
	private WerteTabelle gt;
	private ActionMap actionMap;
	private ResourceMap resourceMap;
	private boolean firePC;

	public CanvasEigenschaften(java.awt.Frame parent, Eigenschaften es,
			boolean modal) {
		super(parent, modal);
		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(CanvasEigenschaften.class, this);
		resourceMap = Application.getInstance(ghmessen.GHMessen.class)
				.getContext().getResourceMap(CanvasEigenschaften.class);
		setResizable(false);
		setTitle(resourceMap.getString("eigenschaften.title"));
		this.es = es;
		firePC = false;
		initComponents();
		firePC = true;
	}

	private void initComponents() {
		schriftGroesse = new javax.swing.ButtonGroup();
		linienBreite = new javax.swing.ButtonGroup();
		interpolation = new javax.swing.ButtonGroup();
		skalierung = new javax.swing.ButtonGroup();
		schriftPanel = new javax.swing.JPanel();
		schriftKlein = new javax.swing.JRadioButton();
		schriftMittel = new javax.swing.JRadioButton();
		schriftGross = new javax.swing.JRadioButton();
		liniePanel = new javax.swing.JPanel();
		linieSchmal = new javax.swing.JRadioButton();
		linieMittel = new javax.swing.JRadioButton();
		linieBreit = new javax.swing.JRadioButton();
		anzeigePanel = new javax.swing.JPanel();
		anzeigeRaster = new javax.swing.JCheckBox();
		anzeigeKoordinaten = new javax.swing.JCheckBox();
		skalierungPanel = new javax.swing.JPanel();
		skalierungAus = new javax.swing.JRadioButton();
		skalierungUnten = new javax.swing.JRadioButton();
		skalierungOben = new javax.swing.JRadioButton();
		skalierungDynamisch = new javax.swing.JCheckBox();
		schliessen = new javax.swing.JButton();

		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		schriftPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(resourceMap
						.getString("schriftgroesse.text")));
		schriftGroesse.add(schriftKlein);
		schriftKlein.setText(resourceMap.getString("schriftklein.text"));
		schriftKlein.setSelected(es.getSchrift().equals(SCHRIFTKLEIN));
		schriftKlein.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (schriftKlein.isSelected())
					es.setSchrift(SCHRIFTKLEIN);
			}
		});
		schriftGroesse.add(schriftMittel);
		schriftMittel.setText(resourceMap.getString("schriftmittel.text"));
		schriftMittel.setSelected(es.getSchrift().equals(SCHRIFTMITTEL));
		schriftMittel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (schriftMittel.isSelected())
					es.setSchrift(SCHRIFTMITTEL);
			}
		});
		schriftGroesse.add(schriftGross);
		schriftGross.setText(resourceMap.getString("schriftgross.text"));
		schriftGross.setSelected(es.getSchrift().equals(SCHRIFTGROSS));
		schriftGross.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (schriftGross.isSelected())
					es.setSchrift(SCHRIFTGROSS);
			}
		});

		javax.swing.GroupLayout gl_schriftPanel = new javax.swing.GroupLayout(
				schriftPanel);
		schriftPanel.setLayout(gl_schriftPanel);
		gl_schriftPanel
				.setHorizontalGroup(gl_schriftPanel
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_schriftPanel
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_schriftPanel
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																schriftKlein)
														.addComponent(
																schriftMittel)
														.addComponent(
																schriftGross))
										.addContainerGap(44, Short.MAX_VALUE)));
		gl_schriftPanel
				.setVerticalGroup(gl_schriftPanel
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_schriftPanel
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(schriftKlein)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(schriftMittel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(schriftGross)
										.addContainerGap(15, Short.MAX_VALUE)));

		liniePanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(resourceMap.getString("liniePanel.text")));

		linienBreite.add(linieSchmal);
		linieSchmal.setText(resourceMap.getString("linieschmal.text"));
		linieSchmal.setSelected(es.getLinie() == LINIESCHMAL);
		linieSchmal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (linieSchmal.isSelected())
					es.setLinie(LINIESCHMAL);
			}
		});
		linienBreite.add(linieMittel);
		linieMittel.setText(resourceMap.getString("liniemittel.text"));
		linieMittel.setSelected(es.getLinie() == LINIEMITTEL);
		linieMittel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (linieMittel.isSelected())
					es.setLinie(LINIEMITTEL);
			}
		});
		linienBreite.add(linieBreit);
		linieBreit.setText(resourceMap.getString("liniebreit.text"));
		linieBreit.setSelected(es.getLinie() == LINIEBREIT);
		linieBreit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (linieBreit.isSelected())
					es.setLinie(LINIEBREIT);
			}
		});

		javax.swing.GroupLayout gl_liniePanel = new javax.swing.GroupLayout(
				liniePanel);
		liniePanel.setLayout(gl_liniePanel);
		gl_liniePanel
				.setHorizontalGroup(gl_liniePanel
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_liniePanel
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_liniePanel
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																linieSchmal)
														.addComponent(
																linieMittel)
														.addComponent(
																linieBreit))
										.addContainerGap(31, Short.MAX_VALUE)));
		gl_liniePanel
				.setVerticalGroup(gl_liniePanel
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_liniePanel
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(linieSchmal)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(linieMittel)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addComponent(linieBreit)
										.addContainerGap(15, Short.MAX_VALUE)));

		anzeigePanel.setBorder(new TitledBorder(null, resourceMap
				.getString("anzeige.text"), TitledBorder.LEADING,
				TitledBorder.TOP, null, null));
		anzeigeRaster.setSelected(es.isRasterAn());
		anzeigeRaster.setText(resourceMap.getString("anzeigeraster.text"));
		anzeigeRaster.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				es.setRasterAn(anzeigeRaster.isSelected());
			}
		});

		anzeigeKoordinaten.setText(resourceMap
				.getString("anzeigekoordinaten.text"));
		anzeigeAchse2 = new javax.swing.JCheckBox();
		anzeigeAchse2.setSelected(es.isAchse2An());
		anzeigeAchse2.setText(resourceMap.getString("anzeigeachse2.text"));
		anzeigeAchse2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				es.setAchse2An(anzeigeAchse2.isSelected());
			}
		});

		javax.swing.GroupLayout gl_anzeigePanel = new javax.swing.GroupLayout(
				anzeigePanel);
		gl_anzeigePanel.setHorizontalGroup(gl_anzeigePanel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_anzeigePanel
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								gl_anzeigePanel
										.createParallelGroup(Alignment.LEADING)
										.addComponent(anzeigeRaster)
										.addComponent(anzeigeKoordinaten)
										.addComponent(anzeigeAchse2))
						.addContainerGap(GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)));
		gl_anzeigePanel.setVerticalGroup(gl_anzeigePanel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_anzeigePanel.createSequentialGroup()
						.addComponent(anzeigeRaster)
						.addPreferredGap(ComponentPlacement.RELATED)
						.addComponent(anzeigeAchse2)
						.addPreferredGap(ComponentPlacement.RELATED)
						.addComponent(anzeigeKoordinaten)
						.addContainerGap(125, Short.MAX_VALUE)));
		anzeigePanel.setLayout(gl_anzeigePanel);

		skalierungPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(resourceMap.getString("skalierung.text")));
		skalierung.add(skalierungAus);
		skalierungAus.setSelected(!es.isSkalierungAn());
		skalierungAus.setText(resourceMap.getString("skalierungaus.text"));
		skalierungAus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				es.setSkalierungAn(!skalierungAus.isSelected());
			}
		});
		skalierung.add(skalierungUnten);
		skalierungUnten.setText(resourceMap.getString("skalierunglu.text"));
		skalierungUnten.setSelected(es.isSkalierungAn()
				&& !es.isSkalierungOben());
		skalierungUnten.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (skalierungUnten.isSelected()) {
					es.setSkalierungAn(true);
					es.setSkalierungOben(false);
				}
			}
		});
		skalierung.add(skalierungOben);
		skalierungOben.setText(resourceMap.getString("skalierungro.text"));
		skalierungOben
				.setSelected(es.isSkalierungAn() && es.isSkalierungOben());
		skalierungOben.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (skalierungOben.isSelected()) {
					es.setSkalierungAn(true);
					es.setSkalierungOben(true);
				}
			}
		});
		skalierungDynamisch.setText(resourceMap
				.getString("skalierungdynamisch.text"));
		skalierungDynamisch.setToolTipText(resourceMap
				.getString("skalierungdynamisch.tip"));
		gt = GHMessen.getApplication().getMainView().gt;
		if (gt != null) {
			skalierungDynamisch.setSelected(gt.isDynamisch());
		}
		skalierungDynamisch.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (gt != null && firePC)
					gt.setDynamisch(skalierungDynamisch.isSelected());
			}
		});

		javax.swing.GroupLayout gl_skalierungPanel = new javax.swing.GroupLayout(
				skalierungPanel);
		skalierungPanel.setLayout(gl_skalierungPanel);
		gl_skalierungPanel
				.setHorizontalGroup(gl_skalierungPanel
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_skalierungPanel
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_skalierungPanel
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																skalierungOben)
														.addComponent(
																skalierungUnten)
														.addComponent(
																skalierungAus))
										.addContainerGap(27, Short.MAX_VALUE)));
		gl_skalierungPanel
				.setVerticalGroup(gl_skalierungPanel
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_skalierungPanel
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(skalierungAus)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierungUnten)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierungOben)
										.addContainerGap(15, Short.MAX_VALUE)));

		schliessen.setAction(actionMap.get("dialogSchliessen"));
		interpolationPanel = new javax.swing.JPanel();
		keineInterpolation = new javax.swing.JRadioButton();
		akimaInterpolation = new javax.swing.JRadioButton();
		sincInterpolation = new javax.swing.JRadioButton();
		interpolationPanel
				.setBorder(javax.swing.BorderFactory
						.createTitledBorder(resourceMap
								.getString("interpolation.text")));
		interpolation.add(keineInterpolation);
		keineInterpolation
				.setSelected(es.getInterpolation() == KEINEINTERPOLATION);
		keineInterpolation.setText(resourceMap
				.getString("interpolationaus.text"));
		interpolation.add(akimaInterpolation);
		keineInterpolation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (keineInterpolation.isSelected())
					es.setInterpolation(KEINEINTERPOLATION);
			}
		});
		akimaInterpolation.setText(resourceMap
				.getString("interpolationakima.text"));
		akimaInterpolation
				.setSelected(es.getInterpolation() == AKIMAINTERPOLATION);
		akimaInterpolation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (akimaInterpolation.isSelected())
					es.setInterpolation(AKIMAINTERPOLATION);
			}
		});
		interpolation.add(sincInterpolation);
		sincInterpolation.setText(resourceMap
				.getString("interpolationsinc.text"));
		sincInterpolation
				.setSelected(es.getInterpolation() == SINCINTERPOLATION);
		sincInterpolation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				if (sincInterpolation.isSelected())
					es.setInterpolation(SINCINTERPOLATION);
			}
		});

		javax.swing.GroupLayout gl_interpolationPanel = new javax.swing.GroupLayout(
				interpolationPanel);
		gl_interpolationPanel
				.setHorizontalGroup(gl_interpolationPanel
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_interpolationPanel
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_interpolationPanel
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																keineInterpolation)
														.addComponent(
																akimaInterpolation)
														.addComponent(
																sincInterpolation))
										.addContainerGap(61, Short.MAX_VALUE)));
		gl_interpolationPanel.setVerticalGroup(gl_interpolationPanel
				.createParallelGroup(Alignment.LEADING).addGroup(
						gl_interpolationPanel.createSequentialGroup()
								.addComponent(keineInterpolation)
								.addPreferredGap(ComponentPlacement.RELATED)
								.addComponent(akimaInterpolation)
								.addPreferredGap(ComponentPlacement.RELATED)
								.addComponent(sincInterpolation).addGap(20)));
		interpolationPanel.setLayout(gl_interpolationPanel);

		JPanel panel = new JPanel();
		panel.setBorder(new TitledBorder(null, resourceMap
				.getString("graphen.text"), TitledBorder.LEADING,
				TitledBorder.TOP, null, null));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		layout.setHorizontalGroup(layout
				.createParallelGroup(Alignment.TRAILING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.TRAILING)
												.addComponent(
														skalierungDynamisch)
												.addComponent(
														schliessen,
														GroupLayout.PREFERRED_SIZE,
														169,
														GroupLayout.PREFERRED_SIZE)
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING)
																				.addComponent(
																						anzeigePanel,
																						GroupLayout.PREFERRED_SIZE,
																						124,
																						GroupLayout.PREFERRED_SIZE)
																				.addComponent(
																						schriftPanel,
																						GroupLayout.PREFERRED_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.PREFERRED_SIZE))
																.addPreferredGap(
																		ComponentPlacement.RELATED)
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING)
																				.addComponent(
																						liniePanel,
																						GroupLayout.PREFERRED_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.PREFERRED_SIZE)
																				.addComponent(
																						panel,
																						GroupLayout.PREFERRED_SIZE,
																						118,
																						GroupLayout.PREFERRED_SIZE))
																.addPreferredGap(
																		ComponentPlacement.UNRELATED)
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING)
																				.addComponent(
																						interpolationPanel,
																						GroupLayout.PREFERRED_SIZE,
																						144,
																						GroupLayout.PREFERRED_SIZE)
																				.addComponent(
																						skalierungPanel,
																						GroupLayout.DEFAULT_SIZE,
																						144,
																						Short.MAX_VALUE))))
								.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addComponent(
														skalierungPanel,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.DEFAULT_SIZE,
														Short.MAX_VALUE)
												.addComponent(
														liniePanel,
														GroupLayout.DEFAULT_SIZE,
														141, Short.MAX_VALUE)
												.addComponent(
														schriftPanel,
														GroupLayout.DEFAULT_SIZE,
														141, Short.MAX_VALUE))
								.addPreferredGap(ComponentPlacement.RELATED)
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addGroup(
														layout.createParallelGroup(
																Alignment.BASELINE)
																.addComponent(
																		anzeigePanel,
																		GroupLayout.PREFERRED_SIZE,
																		102,
																		GroupLayout.PREFERRED_SIZE)
																.addComponent(
																		interpolationPanel,
																		GroupLayout.PREFERRED_SIZE,
																		102,
																		GroupLayout.PREFERRED_SIZE))
												.addComponent(
														panel,
														GroupLayout.PREFERRED_SIZE,
														102,
														GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(ComponentPlacement.RELATED)
								.addComponent(skalierungDynamisch)
								.addPreferredGap(ComponentPlacement.RELATED)
								.addComponent(schliessen,
										GroupLayout.PREFERRED_SIZE, 26,
										GroupLayout.PREFERRED_SIZE)
								.addContainerGap()));
		anzeigeWerte = new javax.swing.JCheckBox();
		anzeigeWerte.setText(resourceMap.getString("anzeigewerte.text"));
		anzeigeWerte.setSelected(es.isWerteAn());
		anzeigeWerte.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				es.setWerteAn(anzeigeWerte.isSelected());
			}
		});

		anzeigeLinien = new javax.swing.JCheckBox();
		anzeigeLinien.setSelected(es.isLinieAn());
		anzeigeLinien.setText(resourceMap.getString("anzeigelinien.text"));
		anzeigeLinien.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				es.setLinieAn(anzeigeLinien.isSelected());
			}
		});
		interpolationPanel.setEnabled(anzeigeLinien.isSelected());
		keineInterpolation.setEnabled(anzeigeLinien.isSelected());
		akimaInterpolation.setEnabled(anzeigeLinien.isSelected());
		sincInterpolation.setEnabled(anzeigeLinien.isSelected());
		anzeigeLinien.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				interpolationPanel.setEnabled(anzeigeLinien.isSelected());
				keineInterpolation.setEnabled(anzeigeLinien.isSelected());
				akimaInterpolation.setEnabled(anzeigeLinien.isSelected());
				sincInterpolation.setEnabled(anzeigeLinien.isSelected());
			}
		});

		anzeigeBalken = new javax.swing.JCheckBox();
		anzeigeBalken.setText(resourceMap.getString("anzeigebalken.text"));
		anzeigeBalken.setSelected(es.isBalkenAn());
		anzeigeBalken.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!firePC)
					return;
				es.setBalkenAn(anzeigeBalken.isSelected());
			}
		});
		GroupLayout gl_panel = new GroupLayout(panel);
		gl_panel.setHorizontalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								gl_panel.createParallelGroup(Alignment.LEADING)
										.addComponent(anzeigeWerte)
										.addComponent(anzeigeLinien)
										.addComponent(anzeigeBalken))
						.addContainerGap(33, Short.MAX_VALUE)));
		gl_panel.setVerticalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup().addComponent(anzeigeWerte)
						.addPreferredGap(ComponentPlacement.RELATED)
						.addComponent(anzeigeLinien)
						.addPreferredGap(ComponentPlacement.RELATED)
						.addComponent(anzeigeBalken)
						.addContainerGap(20, Short.MAX_VALUE)));
		panel.setLayout(gl_panel);
		getContentPane().setLayout(layout);

		pack();
	}

	private javax.swing.JRadioButton akimaInterpolation;
	private javax.swing.JCheckBox anzeigeAchse2;
	private javax.swing.JCheckBox anzeigeBalken;
	private javax.swing.JCheckBox anzeigeKoordinaten;
	private javax.swing.JCheckBox anzeigeLinien;
	private javax.swing.JCheckBox anzeigeRaster;
	private javax.swing.JCheckBox anzeigeWerte;
	private javax.swing.ButtonGroup interpolation;
	private javax.swing.JButton schliessen;
	private javax.swing.JPanel schriftPanel;
	private javax.swing.JPanel liniePanel;
	private javax.swing.JPanel anzeigePanel;
	private javax.swing.JPanel interpolationPanel;
	private javax.swing.JPanel skalierungPanel;
	private javax.swing.JRadioButton skalierungAus;
	private javax.swing.JRadioButton skalierungUnten;
	private javax.swing.JRadioButton skalierungOben;
	private javax.swing.JCheckBox skalierungDynamisch;
	private javax.swing.JRadioButton keineInterpolation;
	private javax.swing.JRadioButton linieBreit;
	private javax.swing.JRadioButton linieMittel;
	private javax.swing.JRadioButton linieSchmal;
	private javax.swing.ButtonGroup linienBreite;
	private javax.swing.ButtonGroup schriftGroesse;
	private javax.swing.JRadioButton schriftGross;
	private javax.swing.JRadioButton schriftKlein;
	private javax.swing.JRadioButton schriftMittel;
	private javax.swing.JRadioButton sincInterpolation;
	private javax.swing.ButtonGroup skalierung;

	@Action
	public void dialogSchliessen(ActionEvent e) {
		dispose();
	}
}
