/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.plotcanvas;

import static ghmessen.components.plotcanvas.MouseModus.PositioniereLinieSenkrecht;
import static ghmessen.components.plotcanvas.MouseModus.PositioniereLinieWaagerecht;
import static ghmessen.components.plotcanvas.MouseModus.PositioniereNeuenText;
import static ghmessen.groesse.Darstellung.XACHSE;
import ghmessen.GHMessen;
import ghmessen.components.ImageTransferHandler;
import ghmessen.components.PlotPanel;
import ghmessen.components.dialoge.CanvasEigenschaften;
import ghmessen.dialoge.einstellungen.darstellung.YDialog;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.darstellung.Anpassung;
import ghmessen.groesse.darstellung.AnpassungEnum;
import ghmessen.groesse.darstellung.AnpassungFrei;
import ghmessen.groesse.darstellung.AnpassungMittelwert;
import ghmessen.groesse.darstellung.AnpassungRegression;
import ghmessen.groesse.darstellung.Auswertung;
import ghmessen.groesse.darstellung.Eigenschaften;
import ghmessen.groesse.wertetabelle.StellenRenderer;
import ghmessen.parser.FormelGenerator;
import ghmessen.util.CursorTk;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.ActionMap;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;

import org.codehaus.janino.ExpressionEvaluator;
import org.jdesktop.application.Action;

@SuppressWarnings("serial")
public class CanvasMenue extends JPopupMenu {

	private static final boolean DEBUG = false;

	public CanvasMenue() {

		ffDlg = null;
		beiAuswahlAktiv = new Vector<JMenuItem>();
		beiAuswertungAktiv = new Vector<JMenuItem>();

		ActionMap actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(CanvasMenue.class, this);
		auswahl = false;
		auswertung = false;

		add(new JMenuItem(actionMap.get("eigenschaftenDlg")));

		JMenuItem achsenbelegung = new JMenuItem("Achsenbelegung ändern");
		achsenbelegung.setAction(actionMap.get("achsenbelegungDlg"));
		achsenbelegung.setText("Achsenbelegung ändern");
		add(achsenbelegung);

		JSeparator separator = new JSeparator();
		add(separator);

		JMenuItem zoom = new JMenuItem("Zoomen");
		zoom.setEnabled(false);
		add(zoom);

		JMenuItem zoom_aus = new JMenuItem("Zoom ausschalten");
		zoom_aus.setEnabled(false);
		add(zoom_aus);

		JSeparator separator_1 = new JSeparator();
		add(separator_1);

		JMenu markierungMenu = new JMenu("Markierung setzen");
		add(markierungMenu);

		JMenuItem markierungText = new JMenuItem("Text");
		markierungText.setAction(actionMap.get("textPlatzieren"));
		markierungMenu.add(markierungText);

		JMenuItem markierungLinieS = new JMenuItem("Senkrechte Linie");
		markierungLinieS.setAction(actionMap.get("linieSenkrecht"));
		markierungMenu.add(markierungLinieS);

		JMenuItem markierungLinieW = new JMenuItem("Waagerechte Linie");
		markierungLinieW.setAction(actionMap.get("linieWaagerecht"));
		markierungMenu.add(markierungLinieW);

		JMenuItem differenz = new JMenuItem("Differenz messen");
		differenz.setEnabled(false);
		markierungMenu.add(differenz);

		JMenuItem mittelwert = new JMenuItem(
				actionMap.get("anpassungMittelwert"));
		add(mittelwert);

		anpassungMenu = new JMenu("Anpassung durchführen");
		anpassungMenu
				.setToolTipText("Vor der Verwendung muss zunächst ein Graph oder Graphabschnitt ausgewählt werden.");
		add(anpassungMenu);

		JMenuItem anpassungUGerade = new JMenuItem("Ursprungsgerade");
		anpassungUGerade.setAction(actionMap.get("anpassung"));
		anpassungUGerade.setText("Ursprungsgerade");
		anpassungMenu.add(anpassungUGerade);

		JMenuItem anpassungAGerade = new JMenuItem("Ausgleichsgerade");
		anpassungAGerade.setAction(actionMap.get("anpassung"));
		anpassungAGerade.setText("Ausgleichsgerade");
		anpassungMenu.add(anpassungAGerade);

		JMenuItem anpassungNParabel = new JMenuItem("Normalparabel");
		anpassungNParabel.setAction(actionMap.get("anpassung"));
		anpassungNParabel.setText("Normalparabel");
		anpassungMenu.add(anpassungNParabel);

		JMenuItem anpassungParabel = new JMenuItem("Parabel");
		anpassungParabel.setAction(actionMap.get("anpassung"));
		anpassungParabel.setText("Parabel");
		anpassungMenu.add(anpassungParabel);

		anpassungHyperbel = new JMenuItem("Hyperbel 1/x");
		anpassungHyperbel.setAction(actionMap.get("anpassung"));
		anpassungHyperbel.setText("Hyperbel 1/x");
		anpassungMenu.add(anpassungHyperbel);

		anpassungHyperbel2 = new JMenuItem("Hyperbel 1/x^2");
		anpassungHyperbel2.setAction(actionMap.get("anpassung"));
		anpassungHyperbel2.setText("Hyperbel 1/x^2");
		anpassungMenu.add(anpassungHyperbel2);

		anpassungEFunktion = new JMenuItem("Exponentialfunktion #e^x");
		anpassungEFunktion.setAction(actionMap.get("anpassung"));
		anpassungEFunktion.setText("Exponentialfuntion #e^x");
		anpassungMenu.add(anpassungEFunktion);

		anpassungEinhuellende = new JMenuItem("Einhüllende #e^-x");
		anpassungEinhuellende.setAction(actionMap.get("anpassung"));
		anpassungEinhuellende.setText("Einhüllende #e^-x");
		anpassungMenu.add(anpassungEinhuellende);

		anpassungFrei = new JMenuItem("freie Anpassung f(x,A,B,C,D)");
		anpassungFrei.setAction(actionMap.get("anpassung"));
		anpassungFrei.setText("freie Anpassung f(x,A,B,C,D)");
		anpassungMenu.add(anpassungFrei);

		JMenu integralMenu = new JMenu("Integral berechnen");
		add(integralMenu);

		JMenuItem integralFlaeche = new JMenuItem("Fläche zur x-Achse");
		integralFlaeche.setEnabled(false);
		integralMenu.add(integralFlaeche);

		JMenuItem integralPeak = new JMenuItem("Peakfläche");
		integralPeak.setEnabled(false);
		integralMenu.add(integralPeak);

		JMenu auswertungMenu = new JMenu("Weitere Auswertungen");
		add(auswertungMenu);

		JMenuItem auswertungPoisson = new JMenuItem(
				"Poissonverteilung berechnen");
		auswertungPoisson.setEnabled(false);
		auswertungMenu.add(auswertungPoisson);

		JMenuItem auswertungGauss = new JMenuItem("Gaußverteilung berechnen");
		auswertungGauss.setEnabled(false);
		auswertungMenu.add(auswertungGauss);

		JMenuItem auswertungSchwerpunkt = new JMenuItem(
				"Peakschwerpunkt bestimmen");
		auswertungSchwerpunkt.setEnabled(false);
		auswertungMenu.add(auswertungSchwerpunkt);

		JMenuItem auswertungFFaktor = new JMenuItem("Formfaktor bestimmen");
		auswertungFFaktor.setEnabled(false);
		auswertungMenu.add(auswertungFFaktor);

		JMenuItem auswertungWelligkeit = new JMenuItem("Welligkeit bestimmen");
		auswertungWelligkeit.setEnabled(false);
		auswertungMenu.add(auswertungWelligkeit);

		JSeparator separator_4 = new JSeparator();
		auswertungMenu.add(separator_4);

		JMenuItem auswertungAequivalenz = new JMenuItem(
				"Äquivalenzpunkt bestimmen");
		auswertungAequivalenz.setEnabled(false);
		auswertungMenu.add(auswertungAequivalenz);

		JMenuItem auswertungSystole = new JMenuItem(
				"Systole und Diastole bestimmen");
		auswertungSystole.setEnabled(false);
		auswertungMenu.add(auswertungSystole);

		JSeparator separator_2 = new JSeparator();
		add(separator_2);

		JMenuItem auswertungLoeschen = new JMenuItem(
				"Letzte Auswertung löschen");
		auswertungLoeschen.setAction(actionMap.get("auswertungLoeschen"));
		auswertungLoeschen.setText("Letzte Auswertung löschen");
		add(auswertungLoeschen);
		beiAuswertungAktiv.add(auswertungLoeschen);

		JMenuItem auswertungAlleLoeschen = new JMenuItem(
				"Alle Auswertungen löschen");
		auswertungAlleLoeschen.setAction(actionMap
				.get("auswertungLoeschenAlle"));
		auswertungAlleLoeschen.setText("Alle Auswertungen löschen");
		add(auswertungAlleLoeschen);
		beiAuswertungAktiv.add(auswertungAlleLoeschen);

		JMenuItem bereichLoeschen = new JMenuItem("Bereich löschen");
		bereichLoeschen.setAction(actionMap.get("bereichLoeschen"));
		add(bereichLoeschen);
		beiAuswahlAktiv.add(bereichLoeschen);

		JSeparator separator_3 = new JSeparator();
		add(separator_3);

		add(actionMap.get("kopierenBitmap"));

		add(actionMap.get("kopierenFenster"));
	}

	private boolean auswahl;
	private boolean auswertung;
	private Vector<JMenuItem> beiAuswahlAktiv;
	private Vector<JMenuItem> beiAuswertungAktiv;
	private JMenu anpassungMenu;
	private FreieFunktionDlg ffDlg;
	private JMenuItem anpassungEFunktion;
	private JMenuItem anpassungEinhuellende;
	private JMenuItem anpassungFrei;
	private JMenuItem anpassungHyperbel;
	private JMenuItem anpassungHyperbel2;

	@Action
	public void eigenschaftenDlg(ActionEvent e) {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			PlotCanvas pc = (PlotCanvas) c;
			Eigenschaften es = pc.getDarstellung().getGraphEigenschaften();
			JFrame mainFrame = GHMessen.getApplication().getMainFrame();
			CanvasEigenschaften ceDlg = new CanvasEigenschaften(mainFrame, es,
					true);
			ceDlg.setLocationRelativeTo(c);
			ceDlg.setVisible(true);

		}
	}

	@Override
	public void show(Component c, int x, int y) {
		auswahl = auswahlVorhanden(c);
		auswertung = auswertungVorhanden(c);
		anpassungMenu.setEnabled(auswahl);
		for (JMenuItem m : beiAuswahlAktiv)
			m.setEnabled(auswahl);
		for (JMenuItem m : beiAuswertungAktiv)
			m.setEnabled(auswertung);
		super.show(c, x, y);
	}

	@Action
	public void achsenbelegungDlg() {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			PlotCanvas pc = (PlotCanvas) c;
			YDialog dialog = new YDialog(pc.getDarstellung());
			dialog.setVisible(true);
		}
	}

	private boolean auswahlVorhanden(Component c) {
		if (c instanceof PlotCanvas) {
			GraphHighlight[] auswahlen = ((PlotCanvas) c).getAuswahl() == null ? null
					: ((PlotCanvas) c).getAuswahl().getHighlights();
			return auswahlen != null && auswahlen.length > 0;
		} else
			return false;
	}

	private GraphHighlight[] getAuswahlen(Component c) {
		if (c instanceof PlotCanvas) {
			GraphHighlight[] auswahlen = ((PlotCanvas) c).getAuswahl() == null ? null
					: ((PlotCanvas) c).getAuswahl().getHighlights();
			if (auswahlen == null || auswahlen.length <= 0)
				return null;
			else
				return auswahlen;
		} else
			return null;
	}

	private boolean auswertungVorhanden(Component c) {
		if (c instanceof PlotCanvas) {
			Vector<Auswertung> auswertung;
			PlotCanvas p = (PlotCanvas) c;
			if (p.getDarstellung() != null) {
				auswertung = p.getDarstellung().getAuswertung();
				return auswertung != null && auswertung.size() > 0;
			} else
				return false;
		} else
			return false;
	}

	@Action
	public void anpassungMittelwert(ActionEvent e) {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			PlotCanvas pc = (PlotCanvas) c;
			Darstellung ds = pc.getDarstellung();
			GraphHighlight[] aw = getAuswahlen(c);
			if (ds == null || aw == null)
				return;
			Anpassung neu;
			String symbolY, einheitY;
			int stellenY;
			for (GraphHighlight gh : aw) {
				Double[] x = ds.getColumnArray(0, gh.getRow1(), gh.getRow2());
				Double[] y = ds.getColumnArray(gh.getCol(), gh.getRow1(),
						gh.getRow2());
				if (x == null || y == null)
					continue;
				symbolY = ds.getColumnSymbol(gh.getCol());
				einheitY = ds.getColumnEinheit(gh.getCol());
				if (ds.getColumnRenderer(gh.getCol()) instanceof StellenRenderer)
					stellenY = ((StellenRenderer) ds.getColumnRenderer(gh
							.getCol())).getStellen();
				else
					stellenY = 3;
				neu = new AnpassungMittelwert(x, y, gh.getCol(), gh.getRow1(),
						gh.getRow2(), symbolY, stellenY, einheitY,
						pc.getFarbe(gh.getCol()+ds.getAbschnitt(gh.getRow1())));
				ds.neueAuswertung(neu);
			}
			pc.getAuswahl().removeAllHighlights();
		}
	}

	@Action
	public void anpassung(ActionEvent e) {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			PlotCanvas pc = (PlotCanvas) c;
			Darstellung ds = pc.getDarstellung();
			GraphHighlight[] aw = getAuswahlen(c);
			if (ds == null || aw == null)
				return;
			AnpassungEnum typ;
			if (e.getSource() == anpassungEFunktion) {
				typ = AnpassungEnum.eFunktion;
			} else if (e.getSource() == anpassungEinhuellende) {
				typ = AnpassungEnum.Einhuellende;
			} else if (e.getSource() == anpassungFrei) {
				typ = AnpassungEnum.freieFunktion;
			} else if (e.getSource() == anpassungHyperbel) {
				typ = AnpassungEnum.Hyperbel;
			} else if (e.getSource() == anpassungHyperbel2) {
				typ = AnpassungEnum.QuadHyperbel;
			} else {
				typ = AnpassungEnum.valueOf(e.getActionCommand());
			}
			Anpassung neu;
			String symbolX = ds.getColumnSymbol(XACHSE), symbolY;
			String einheitX = ds.getColumnEinheit(XACHSE), einheitY;
			int stellenX, stellenY;
			if (ds.getColumnRenderer(XACHSE) instanceof StellenRenderer)
				stellenX = ((StellenRenderer) ds.getColumnRenderer(XACHSE))
						.getStellen();
			else
				stellenX = 3;
			for (GraphHighlight gh : aw) {
				Double[] x = ds.getColumnArray(XACHSE, gh.getRow1(),
						gh.getRow2());
				Double[] y = ds.getColumnArray(gh.getCol(), gh.getRow1(),
						gh.getRow2());
				symbolY = ds.getColumnSymbol(gh.getCol());
				einheitY = ds.getColumnEinheit(gh.getCol());
				if (ds.getColumnRenderer(gh.getCol()) instanceof StellenRenderer)
					stellenY = ((StellenRenderer) ds.getColumnRenderer(gh
							.getCol())).getStellen();
				else
					stellenY = 3;
				if (x == null || y == null)
					continue;
				if (typ == AnpassungEnum.freieFunktion) {
					ffDlg = new FreieFunktionDlg(GHMessen.getApplication()
							.getMainFrame());
					ffDlg.pack();
					ffDlg.setLocationRelativeTo(GHMessen.getApplication()
							.getMainFrame());
					ffDlg.setVisible(true);
					ExpressionEvaluator term = ffDlg.getValidatedExpression();
					double[] start = ffDlg.getParameters();
					String s = ffDlg.getTerm();
					if (term == null || term == FormelGenerator.NULLAUSDRUCK)
						continue;
					neu = new AnpassungFrei(typ, x, y, term, start, pc
							.getXSkala().getMinimum(), pc.getXSkala()
							.getMaximum(), gh.getCol(), gh.getRow1(),
							gh.getRow2(), symbolX, symbolY, stellenX, stellenY,
							einheitX, einheitY, s, pc.getFarbe(gh.getCol()+ds.getAbschnitt(gh.getRow1())));
				} else
					neu = new AnpassungRegression(typ, x, y, pc.getXSkala()
							.getMinimum(), pc.getXSkala().getMaximum(),
							gh.getCol(), gh.getRow1(), gh.getRow2(), symbolX,
							symbolY, stellenX, stellenY, einheitX, einheitY,
							pc.getFarbe(gh.getCol()+ds.getAbschnitt(gh.getRow1())));
				ds.neueAuswertung(neu);
				neu.setFont(ds.getGraphEigenschaften().getSchrift());
			}
			pc.getAuswahl().removeAllHighlights();
		}
	}

	@Action
	public void auswertungLoeschen() {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			Darstellung ds = ((PlotCanvas) c).getDarstellung();
			if (ds == null || ds.getAuswertung() == null)
				return;
			ds.entferneAuswertung(ds.getAuswertung().size() - 1);
		}
	}

	@Action
	public void auswertungLoeschenAlle() {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			Darstellung ds = ((PlotCanvas) c).getDarstellung();
			if (ds == null || ds.getAuswertung() == null)
				return;
			ds.entferneAlleAuswertungen();
		}
	}

	@Action
	public void textPlatzieren() {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			((PlotCanvas) c).setMausModus(PositioniereNeuenText);
		}
	}

	@Action
	public void linieSenkrecht() {
		liniePlatzieren(true);
	}

	@Action
	public void linieWaagerecht() {
		liniePlatzieren(false);
	}

	public void liniePlatzieren(boolean vertikal) {
		Component c = this.getInvoker();
		if (c instanceof PlotCanvas) {
			((PlotCanvas) c).setMausModus(vertikal ? PositioniereLinieSenkrecht
					: PositioniereLinieWaagerecht);
		}
	}

	@Action
	public void bereichLoeschen() {
		Component c = getInvoker();
		GraphHighlight[] auswahlen = getAuswahlen(c);
		if (auswahlen == null)
			return;
		if (c instanceof PlotCanvas) {
			Darstellung ds = ((PlotCanvas) c).getDarstellung();
			if (ds == null)
				return;
			for (GraphHighlight gh : auswahlen) {
				ds.entferneBereich(gh.getCol(), gh.getRow1(), gh.getRow2());
			}
			((PlotCanvas) c).getAuswahl().removeAllHighlights();
		}
	}

	@Action
	public void kopierenBitmap() {
		Component c = getInvoker();
		while (c != null && !(c instanceof PlotPanel))
			c = c.getParent();
		if (c != null) {
			try {
				CursorTk.startWaitCursor(c);
				BufferedImage image = new BufferedImage(c.getWidth(),
						c.getHeight(), BufferedImage.TYPE_INT_ARGB);

				c.paintAll(image.getGraphics());

				ImageTransferHandler.copyImageToClipboard(image);
				if (DEBUG) {
					ImageIO.write(image, "png", new File(
							"GHMessen_PlotPanelBitmap.png"));
				}
			} catch (IOException e) {
				if (DEBUG)
					e.printStackTrace();
			} finally {
				CursorTk.stopWaitCursor(c);
			}
		} else {
			if (DEBUG)
				System.out
						.println("[CanvasMenu] - kopierenBitmap: PlotPanel nicht gefunden!");
		}

	}

	@Action
	public void kopierenFenster() {
		Component c = getInvoker();
		// enthaltenden JFrame suchen
		if (c instanceof PlotCanvas)
			c = ((PlotCanvas) c).getTopLevelAncestor();
		// JFrame unterstützt kein paintAll, daher die JRootPane verwenden
		if (c instanceof JFrame)
			c = ((JFrame) c).getRootPane();
		if (c != null) {
			try {
				CursorTk.startWaitCursor(c);
				BufferedImage image = new BufferedImage(c.getWidth(),
						c.getHeight(), BufferedImage.TYPE_INT_ARGB);

				c.paintAll(image.getGraphics());

				ImageTransferHandler.copyImageToClipboard(image);
				if (DEBUG) {
					ImageIO.write(image, "png",
							new File("GHMessen_Fenster.png"));
				}
			} catch (IOException e) {
				if (DEBUG)
					e.printStackTrace();
			} finally {
				CursorTk.stopWaitCursor(c);
			}
		} else {
			if (DEBUG)
				System.out
						.println("[CanvasMenu] - kopierenBitmap: JRootPane nicht gefunden!");
		}

	}

	public void setAuswahl(boolean auswahl) {
		this.auswahl = auswahl;
	}

	public boolean isAuswahl() {
		return auswahl;
	}
}
