/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.plotcanvas;

import ghmessen.parser.FormelGenerator;
import ghmessen.parser.FreieFunktionParser;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.codehaus.janino.ExpressionEvaluator;


class FreieFunktionDlg extends JDialog
                   implements ActionListener,
                              PropertyChangeListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7922393811023685395L;
	private static boolean DEBUG = false;
    private ExpressionEvaluator term = null;
    private JTextField eingabe;
    private FreieFunktionParameter start;

    private String text;
    private JOptionPane optionPane;

    private String btnString1 = "Berechnen";
    private String btnString2 = "Abbrechen";
    private FreieFunktionParser p;

    public ExpressionEvaluator getValidatedExpression() {
        return term;
    }

    public double[] getParameters(){
    	return start.getParameters();
    }
    
    public String getTerm(){
    	return text;
    }
    
    /** Creates the reusable dialog. */
    public FreieFunktionDlg(Frame aFrame) {
        super(aFrame, true);
        if(DEBUG)
        	System.out.println("[FreieFunktionDlg] - Konstruktor.");
        p = new FreieFunktionParser("f");
        setTitle("Freie Anpassung");

        eingabe = new JTextField(10);
        eingabe.setText("1");
        String msgString1 = "Bitte tragen Sie die gewünschte Funktion";
        String msgString2 = "f(t,A,B,C,D) zur Anpassung ein.";
        start = new FreieFunktionParameter();
        Object[] array = {msgString1, msgString2, eingabe, start};

        Object[] options = {btnString1, btnString2};

        optionPane = new JOptionPane(array,
                                    JOptionPane.QUESTION_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options,
                                    options[0]);

        setContentPane(optionPane);

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
            }
        });

        addComponentListener(new ComponentAdapter() {
            public void componentShown(ComponentEvent ce) {
                eingabe.requestFocusInWindow();
            }
        });

        eingabe.addActionListener(this);
        optionPane.addPropertyChangeListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        optionPane.setValue(btnString1);
    }

    public void propertyChange(PropertyChangeEvent e) {
        String prop = e.getPropertyName();

        if (isVisible()
         && (e.getSource() == optionPane)
         && (JOptionPane.VALUE_PROPERTY.equals(prop) ||
             JOptionPane.INPUT_VALUE_PROPERTY.equals(prop))) {
            Object value = optionPane.getValue();

            if (value == JOptionPane.UNINITIALIZED_VALUE) {
                return;
            }

            optionPane.setValue(
                    JOptionPane.UNINITIALIZED_VALUE);

            if (btnString1.equals(value)) {
                    text = eingabe.getText();
                    boolean korrekt;
					korrekt = p.parsen(text);
					if(korrekt){
						term = p.getAusdruck(text);
						if(term == FormelGenerator.NULLAUSDRUCK)
							korrekt = false;
					} else {
						korrekt = false;
					}
                if (korrekt) {
                    clearAndHide();
                } else {
                    eingabe.selectAll();
                    JOptionPane.showMessageDialog(
                                    FreieFunktionDlg.this,
                                    "Die Funktion \n"
                                    +"f(t,A,B,C,D)=" + text + "\n "
                                    + "ist ungültig.",
                                    "Bitte Eingabe korrigieren.",
                                    JOptionPane.ERROR_MESSAGE);
                    term = null;
                    eingabe.requestFocusInWindow();
                }
            } else { //user closed dialog or clicked cancel
                term = null;
                clearAndHide();
            }
        }
    }

    /** This method clears the dialog and hides it. */
    public void clearAndHide() {
        eingabe.setText(null);
        setVisible(false);
    }
}
