/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.plotcanvas;

import ghmessen.groesse.darstellung.Auswertung;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Rectangle;

public class FreiePositionenLayoutManager implements LayoutManager {

	@Override
	public void addLayoutComponent(String name, Component comp) {
	}

	@Override
	public void removeLayoutComponent(Component comp) {
	}

	public void replaceWithEdit(Container target, Component alt, Component neu){
		int index = -1;
		for(int i=0;i<target.getComponentCount();i++)
			if(target.getComponent(i) == alt){
				index = i;
				break;
			}
		if(index > -1){
			neu.setPreferredSize(new Dimension(2*alt.getWidth(), 2*alt.getHeight()));
			neu.setBounds(new Rectangle(alt.getX(), alt.getY(), 2*alt.getWidth(), 2*alt.getHeight()));
			neu.setFont(alt.getFont().deriveFont(2*alt.getFont().getSize()));
			target.remove(index);
			target.add(neu,index);
			target.validate();
		}
		
	}
	
	public void restoreToView(Container target, Component alt, Component neu){
		int index = -1;
		for(int i=0;i<target.getComponentCount();i++)
			if(target.getComponent(i) == alt){
				index = i;
				break;
			}
		if(index > -1){
			target.remove(index);
			neu.setSize(neu.getPreferredSize());
			target.add(neu,index);
			target.validate();
		}
		
	}
	
	@Override
	public Dimension preferredLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			Dimension dim = new Dimension(0,0);
	        int nComps = target.getComponentCount();
	        int x = 0, y = 0;
	        for (int i = 0 ; i < nComps ; i++) {
	            Component c = target.getComponent(i);
	            if (c.isVisible()) {
	                Dimension d = c.getPreferredSize();
	                x = c.getX();
	                y = c.getY();
	                Auswertung a = null;
	                if(c instanceof Auswertung){
	                	a = (Auswertung) c;
	                	a.updateLocation();
	                	x = c.getX();
	                	y = c.getY();
	                }
	                dim.height = Math.max(dim.height, y + d.height);
	                dim.width = Math.max(dim.width, x + d.width);
	            }
	        }

	        Insets insets = target.getInsets();
			dim.width += insets.left + insets.right;
			dim.height += insets.top + insets.bottom;
			return dim;

		}
	}

	@Override
	public Dimension minimumLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			Dimension dim = new Dimension(0, 0);
			int nmembers = target.getComponentCount();

			for (int i = 0; i < nmembers; i++) {
				Component m = target.getComponent(i);
				if (m.isVisible()) {
					Dimension d = m.getMinimumSize();
					dim.height = Math.max(dim.height, d.height);
					dim.width = Math.max(dim.width, d.width);
				}
			}

			Insets insets = target.getInsets();
			dim.width += insets.left + insets.right;
			dim.height += insets.top + insets.bottom;
			return dim;

		}
	}

	@Override
    public void layoutContainer(Container parent) {
        int nComps = parent.getComponentCount();
        int x = 0, y = 0;

        for (int i = 0 ; i < nComps ; i++) {
            Component c = parent.getComponent(i);
            if (c.isVisible()) {
                Dimension d = c.getPreferredSize();
                x = c.getX();
                y = c.getY();
                Auswertung a = null;
                if(c instanceof Auswertung){
                	a = (Auswertung) c;
                	a.updateLocation();
                	x = c.getX();
                	y = c.getY();
                }
                // Set the component's size and position.
                c.setBounds(x, y, d.width, d.height);

            }
        }
    }

    public String toString() {
        return getClass().getName() + "[]";
    }
}
