package ghmessen.components.plotcanvas;

import static java.awt.geom.PathIterator.*;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

import static ghmessen.components.plotcanvas.GRAPH2D.*;

//TODO: neue Segmente BALKEN, WERT, LINIE implementieren als einzelnes Segment
public class Graph2D implements Shape, Cloneable {

	private static final boolean DEBUG = false;

	public static final int SEG_BALKEN = 100;
	public static final int SEG_WERT = 101;
	public static final int SEG_LINIE = 102;

	private GRAPH2D typ;
	private Vector<Path2D> pfad;
	private Vector<Integer> laenge;
	private Path2D connected;
	private Path2D closed;

	public Graph2D() {
		this(new Path2D.Double(), LEER);
	}

	public Graph2D(GRAPH2D typ) {
		this(new Path2D.Double(), typ);
	}

	public Graph2D(Path2D pfad, GRAPH2D typ) {
		this.typ = typ;
		this.pfad = new Vector<Path2D>();
		this.pfad.add(pfad);
		this.connected = pfad;
		if (typ == LINIE)
			this.closed = createClosedFromLine(this.pfad);
		else
			this.closed = null;
		this.laenge = calculateLength();
		if (DEBUG)
			System.out.println("[Graph2D] - Konstruktor...");
	}

	public Graph2D(Vector<Path2D> pfad, GRAPH2D typ) {
		this.typ = typ;
		this.pfad = pfad;
		this.connected = createConnectedFromPaths(pfad);
		if (typ == LINIE)
			this.closed = createClosedFromLine(pfad);
		else
			this.closed = null;
		this.laenge = calculateLength();
		if (DEBUG)
			System.out.println("[Graph2D] - Konstruktor...");
	}

	public void setTyp(GRAPH2D typ) {
		this.typ = typ;
		if (typ == LINIE)
			this.closed = createClosedFromLine(pfad);
		else
			this.closed = null;
		this.laenge = calculateLength();
	}

	public GRAPH2D getTyp() {
		return typ;
	}

	public void setPath2D(Vector<Path2D> pfad) {
		this.pfad = pfad;
		this.connected = createConnectedFromPaths(pfad);
		if (typ == LINIE)
			this.closed = createClosedFromLine(pfad);
		else
			this.closed = null;
		this.laenge = calculateLength();
	}

	public Vector<Path2D> getPath2D() {
		return pfad;
	}

	public void setPath2D(Vector<Path2D> pfad, GRAPH2D typ) {
		this.typ = typ;
		this.pfad = pfad;
		this.connected = createConnectedFromPaths(pfad);
		if (typ == LINIE)
			this.closed = createClosedFromLine(pfad);
		else
			this.closed = null;
		this.laenge = calculateLength();
	}

	@Override
	public Rectangle getBounds() {
		if (typ != LINIE)
			return connected.getBounds();
		else
			return closed.getBounds();
	}

	@Override
	public Rectangle2D getBounds2D() {
		if (typ != LINIE)
			return connected.getBounds2D();
		else
			return closed.getBounds2D();
	}

	@Override
	public boolean contains(double x, double y) {
		if (typ != LINIE)
			return connected.contains(x, y);
		else
			return closed.contains(x, y);
	}

	@Override
	public boolean contains(Point2D p) {
		if (typ != LINIE)
			return connected.contains(p);
		else
			return closed.contains(p);
	}

	@Override
	public boolean intersects(double x, double y, double w, double h) {
		if (typ != LINIE)
			return connected.intersects(x, y, w, h);
		else
			return closed.intersects(x, y, w, h);
	}

	@Override
	public boolean intersects(Rectangle2D r) {
		if (typ != LINIE)
			return connected.intersects(r);
		else
			return closed.intersects(r);
	}

	@Override
	public boolean contains(double x, double y, double w, double h) {
		if (typ != LINIE)
			return connected.contains(x, y, w, h);
		else
			return closed.contains(x, y, w, h);
	}

	@Override
	public boolean contains(Rectangle2D r) {
		if (typ != LINIE)
			return connected.contains(r);
		else
			return closed.contains(r);
	}

	public Relation getNeighbourRelation(Point p, int presetIdx) {
		return getNeighbourRelation(new Point2D.Double(p.x, p.y), presetIdx);
	}

	public Relation getNeighbourRelation(double px, double py, int presetIdx) {
		return getNeighbourRelation(new Point2D.Double(px, py), presetIdx);
	}

	public Relation getNeighbourRelation(Point2D p, int presetIdx) {
		Vector<Relation> rv = new Vector<Relation>();
		Relation r = null;
		Vector<PathIterator> pi = getPointsIterators(null);
		if (presetIdx == -1) {
			for (int index = 0; index < pi.size(); index++) {
				rv.add(calculateRelation(pi.get(index), p));
				if (index > 0)
					rv.lastElement().nummer += laenge.get(index - 1) + 1;
				if (r == null)
					r = rv.lastElement();
				else if (r.abstand > rv.lastElement().abstand)
					r = rv.lastElement();
			}
		} else {
			int index = calculateSegment(presetIdx);
			r = calculateRelation(pi.get(index), p);
			if(index > 0)
				r.nummer += laenge.get(index-1) + 1;
		}
		return r;
	}

	private Relation calculateRelation(PathIterator pi, Point2D p) {
		Relation r = new Relation(-1, Double.POSITIVE_INFINITY, 1.0);
		int index = 0;
		double abstand = Double.POSITIVE_INFINITY;
		double[] coords = new double[6];
		while (!pi.isDone()) {
			if (pi.currentSegment(coords) != PathIterator.SEG_CLOSE) {
				abstand = p.distance(coords[0], coords[1]);
				if (abstand < r.abstand) {
					r.abstand = abstand;
					r.nummer = index;
					r.position = Math.signum(p.getX() - coords[0]);
				}
			}
			index++;
			pi.next();
		}
		return r;
	}

	public class Relation {
		public Relation(int nummer, double abstand, double position) {
			this.nummer = nummer;
			this.abstand = abstand;
			this.position = position;
		}

		// Nummer des Punktes im Pfad
		public int nummer;
		// Abstand zum indizierten Punkt
		public double abstand;
		// Position zum indizierten Punkt -1.0 links vom indizierten Punkt
		public double position;
	}

	@Override
	public PathIterator getPathIterator(AffineTransform at) {
		return connected.getPathIterator(at);
	}

	@Override
	public PathIterator getPathIterator(AffineTransform at, double flatness) {
		return connected.getPathIterator(at, flatness);
	}

	public PathIterator getPointsIterator(AffineTransform at) {
		return getPointsIterator(at, 10);
	}

	public Vector<PathIterator> getPointsIterators(AffineTransform at) {
		return getPointsIterators(at, 10);
	}

	public PathIterator getPointsIterator(AffineTransform at, double flatness) {
		return new PointsIterator(at, flatness, typ, connected);
	}

	public Vector<PathIterator> getPointsIterators(AffineTransform at,
			double flatness) {
		Vector<PathIterator> result = new Vector<PathIterator>();
		for (Path2D p : pfad)
			result.add(new PointsIterator(at, flatness, typ, p));
		return result;
	}

	private Vector<Integer> calculateLength() {
		Vector<Integer> result = new Vector<Integer>();
		for (int index = 0; index < pfad.size(); index++) {
			if (index == 0)
				result.add(calculatePointsCount(new PointsIterator(null, 10,
						typ, pfad.get(index))));
			else
				result.add(result.get(index - 1)
						+ calculatePointsCount(new PointsIterator(null, 10,
								typ, pfad.get(index))) + 1);
		}
		return result;
	}

	private int calculatePointsCount(PathIterator pi) {
		int laenge = 0;
		while (!pi.isDone()) {
			laenge++;
			pi.next();
		}
		return laenge;
	}

	private int calculateSegment(int index){
		if(laenge == null || laenge.isEmpty())
			return 0;
		for(int segment = 0;segment < laenge.size();segment++)
			if(laenge.get(segment) > index)
				return segment;
		return -1;
	}
	
	private class PointsIterator implements PathIterator {

		private GRAPH2D typ;
		private PathIterator pi;

		public PointsIterator(AffineTransform at, double flatness, GRAPH2D typ,
				Path2D pfad) {
			this.typ = typ;
			pi = pfad.getPathIterator(at, flatness);
		}

		@Override
		public int getWindingRule() {
			return pi.getWindingRule();
		}

		@Override
		public boolean isDone() {
			return pi.isDone();
		}

		@Override
		public void next() {
			switch (typ) {
			default:
				pi.next();
				break;
			case BALKEN:
			case WERT:
				pi.next();
				pi.next();
				pi.next();
				pi.next();
				pi.next();
				pi.next();
				break;
			}
		}

		@Override
		public int currentSegment(float[] coords) {
			return pi.currentSegment(coords);
		}

		@Override
		public int currentSegment(double[] coords) {
			return pi.currentSegment(coords);
		}

	}

	private double[] umkehren(double[] von, double[] coords, int typ) {
		double[] rueck = new double[6];
		switch (typ) {
		case SEG_CUBICTO:
			rueck[5] = von[1];
			rueck[4] = von[0];
			rueck[3] = coords[1];
			rueck[2] = coords[0];
			rueck[1] = coords[3];
			rueck[0] = coords[2];
			break;
		case SEG_QUADTO:
			rueck[3] = von[1];
			rueck[2] = von[0];
			rueck[1] = coords[1];
			rueck[0] = coords[0];
			break;
		case SEG_LINETO:
		case SEG_MOVETO:
			rueck[1] = von[1];
			rueck[0] = von[0];
			break;
		default:
			break;
		}
		return rueck;
	}

	private Path2D anhaengen(Path2D pfad, int typ, double[] coords) {
		switch (typ) {
		case SEG_MOVETO:
			pfad.moveTo(coords[0], coords[1]);
			break;
		case SEG_LINETO:
			pfad.lineTo(coords[0], coords[1]);
			break;
		case SEG_QUADTO:
			pfad.quadTo(coords[0], coords[1], coords[2], coords[3]);
			break;
		case SEG_CUBICTO:
			pfad.curveTo(coords[0], coords[1], coords[2], coords[3], coords[4],
					coords[5]);
			break;
		default:
		case SEG_CLOSE:
			pfad.closePath();
			break;
		}
		return pfad;
	}

	private Path2D createConnectedFromPaths(Vector<Path2D> pfad) {
		if (pfad == null || pfad.isEmpty())
			return null;
		Path2D result = (Path2D) pfad.get(0).clone();
		for (int index = 1; index < pfad.size(); index++)
			result.append(pfad.get(index), false);
		return result;
	}

	private Path2D createClosedFromLine(Vector<Path2D> pfad) {
		if (pfad == null || pfad.isEmpty())
			return null;
		Iterator<Path2D> it = pfad.iterator();
		Path2D result = createClosedFromLine(it.next());
		while (it.hasNext())
			result.append(it.next(), false);
		return result;
	}

	private Path2D createClosedFromLine(Path2D pfad) {
		// Der Typ muss LINIE sein.
		Path2D closed = new Path2D.Double();
		Stack<double[]> coordStack = new Stack<double[]>();
		Stack<Integer> typStack = new Stack<Integer>();
		double[] von = new double[6];
		int typ;
		double[] coords = new double[6];
		PathIterator pi = pfad.getPathIterator(null);
		if (!pi.isDone()) {
			typ = pi.currentSegment(coords);
			closed = anhaengen(closed, typ, coords);
			von[0] = coords[0];
			von[1] = coords[1];
			pi.next();
		}
		while (!pi.isDone()) {
			typ = pi.currentSegment(coords);
			if (typ == PathIterator.SEG_MOVETO) {
				while (!typStack.isEmpty())
					closed = anhaengen(closed, typStack.pop(), coordStack.pop());
			} else if (typ == PathIterator.SEG_CLOSE) {
				coordStack.empty();
				typStack.empty();
			} else {
				coordStack.push(umkehren(von, coords, typ));
				typStack.push(typ);
			}
			closed = anhaengen(closed, typ, coords);
			von[0] = coords[0];
			von[1] = coords[1];
			pi.next();
		}
		while (!typStack.isEmpty())
			closed = anhaengen(closed, typStack.pop(), coordStack.pop());
		return closed;
	}
}
