/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.plotcanvas;

import ghmessen.Konstanten;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Path2D;

public class GraphHighlightPainter {
	private static final boolean DEBUG = false;
	private Color color;
	private boolean gefuellt;
	private BasicStroke stroke = new BasicStroke(Konstanten.MARKIERUNG_BREITE,
			BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);

	public GraphHighlightPainter(Color c, boolean gefuellt) {
		color = c;
		this.gefuellt = gefuellt;
	}

	public Color getColor() {
		return color;
	}

	public boolean istGefuellt() {
		return gefuellt;
	}

	public static float getStrokeWidth() {
		return Konstanten.MARKIERUNG_BREITE;
	}

	public void paint(Graphics g, int col, int offs0, int offs1, Shape bounds,
			PlotCanvas c) {
		Graphics2D g2d = (Graphics2D) g.create();
		try {
			if (DEBUG)
				System.out
						.println("[GraphHighlightPainter] - paint: col,row1,row2("
								+ col + "," + offs0 + "," + offs1 + ")");
			Path2D p = c.modelToView(col, offs0, offs1);
			Color color = getColor();
			if (color == null) {
				g2d.setColor(c.getSelectionColor());
			} else {
				g2d.setColor(color);
			}
			g2d.setStroke(stroke);
			if (gefuellt)
				g2d.fill(p);
			else
				g2d.draw(p);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}

}
