/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.plotcanvas;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.Vector;

public class GraphHighlighter {
	private static final boolean DEBUG = false;

	private class HighlightVector extends Vector<GraphHighlight> {
		/**
		 * 
		 */
		private static final long serialVersionUID = -8363191564300298141L;

		public HighlightVector() {
			super();
		}

		public GraphHighlight ueberlapptMit(GraphHighlight neu) {
			if (size() <= 0)
				return null;
			for (GraphHighlight gh : this)
				if (gh != neu && gh.ueberlapptMit(neu))
					return gh;
			return null;
		}
	}

	private HighlightVector auswahlen = new HighlightVector();
	private PlotCanvas component;

	public GraphHighlightPainter defaultOutline = new GraphHighlightPainter(
			null, false);
	public GraphHighlightPainter defaultFilled = new GraphHighlightPainter(
			null, true);

	public GraphHighlighter() {
		super();
		if (DEBUG)
			System.out.println("[GraphHighlighter] - Konstruktor.");
	}

	public GraphHighlighter(Color farbe) {
		this();
		defaultOutline = new GraphHighlightPainter(farbe, false);
		defaultFilled = new GraphHighlightPainter(farbe, true);
	}

	public void install(PlotCanvas c) {
		component = c;
		removeAllHighlights();
	}

	public void install(PlotCanvas c, Color farbe) {
		install(c);
		defaultOutline = new GraphHighlightPainter(farbe, false);
		defaultFilled = new GraphHighlightPainter(farbe, true);
	}

	public void deinstall(PlotCanvas c) {
		component = null;
	}

	public void paint(Graphics g) {
		Rectangle a = new Rectangle(component.getSize());
		Insets insets = component.getInsets();
		a.x += insets.left;
		a.y += insets.top;
		a.width -= insets.left + insets.right;
		a.height -= insets.top + insets.bottom;
		int len = auswahlen.size();
		for (int i = 0; i < len; i++) {
			GraphHighlight info = (GraphHighlight) auswahlen.elementAt(i);
			GraphHighlightPainter p = info.getPainter();
			p.paint(g, info.getCol(), info.getRow1(), info.getRow2(), a,
					component);

		}
	}

	public GraphHighlight addHighlight(int col, int p0, int p1,
			GraphHighlightPainter p) throws BadLocationException {
		GraphHighlight i = new GraphHighlight(p, col, p0, p1);
		GraphHighlight alt = auswahlen.ueberlapptMit(i);
		if (alt != null) {
			auswahlen.remove(alt);
			i = i.vereinigungMit(alt);
		}
		auswahlen.addElement(i);
		component.damageRange(component, col, p0, p1);
		return i;
	}

	public Object addHighlight(GraphHighlight neu, boolean gefuellt) {
		auswahlen.addElement(neu);
		neu.painter = gefuellt ? defaultFilled : defaultOutline;
		component.damageRange(component, neu.getCol(), neu.getRow1(),
				neu.getRow2());
		return neu;
	}

	public GraphHighlight removeHighlight(Object tag) {
		GraphHighlight info = (GraphHighlight) tag;
		component.damageRange(component, info.getCol(), info.getRow1(),
				info.getRow2());
		int index = auswahlen.indexOf(tag);
		return index > -1 ? auswahlen.remove(index) : null;
	}

	public void removeAllHighlights() {
		if (component != null) {
			int len = auswahlen.size();
			if (len != 0) {
				for (int i = 0; i < len; i++) {
					GraphHighlight info = (GraphHighlight) auswahlen
							.elementAt(i);
					component.damageRange(component, info.getCol(),
							info.getRow1(), info.getRow2());
				}
				auswahlen.removeAllElements();
			}
		}
	}

	public void changeHighlight(Object tag, int col, int p0, int p1)
			throws BadLocationException {
		GraphHighlight info = (GraphHighlight) tag;
		int oldCol = info.getCol();
		int oldP0 = info.getRow1();
		int oldP1 = info.getRow2();
		if (DEBUG)
			System.out.println("[GraphHighlighter] - changeHighlight: ("
					+ oldCol + "," + oldP0 + "," + oldP1 + ") -> (" + col + ","
					+ p0 + "," + p1 + ")");
		if (oldCol == col) {
			component.damageRange(component, col, oldP0, oldP1);
			component.damageRange(component, col, p0, p1);
			/*
			 * if (p0 == oldP0) { component.damageRange(component, col,
			 * Math.min(oldP1, p1), Math.max(oldP1, p1)); } else if (p1 ==
			 * oldP1) { component.damageRange(component, col, Math.min(p0,
			 * oldP0), Math.max(p0, oldP0)); } else {
			 * component.damageRange(component, col, oldP0, oldP1);
			 * component.damageRange(component, col, p0, p1); }
			 */
		} else {
			component.damageRange(component, oldCol, oldP0, oldP1);
			component.damageRange(component, col, p0, p1);
		}
		info.setBezug(col, p0, p1);
		GraphHighlight alt = auswahlen.ueberlapptMit(info);
		if (alt != null) {
			auswahlen.remove(alt);
			info = info.vereinigungMit(alt);
		}
	}

	public GraphHighlight[] getHighlights() {
		GraphHighlight[] h = new GraphHighlight[auswahlen.size()];
		auswahlen.copyInto(h);
		return h;
	}

}
