/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.components.plotcanvas;

import static ghmessen.Konstanten.GRAPH_AUSWAHLBEREICH;
import static ghmessen.components.plotcanvas.GRAPH2D.BALKEN;
import static ghmessen.components.plotcanvas.GRAPH2D.LINIE;
import static ghmessen.components.plotcanvas.GRAPH2D.WERT;
import static ghmessen.components.plotcanvas.MouseModus.Normal;
import static javax.swing.event.TableModelEvent.ALL_COLUMNS;
import static javax.swing.event.TableModelEvent.INSERT;
import static javax.swing.event.TableModelEvent.UPDATE;
import ghmessen.GHMessen;
import ghmessen.GHMessenView;
import ghmessen.Konstanten;
import ghmessen.Konstanten.FARBE_GRAPHEN;
import ghmessen.components.Intervall;
import ghmessen.components.Skala;
import ghmessen.components.plotcanvas.Graph2D.Relation;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Skalierung;
import ghmessen.groesse.darstellung.Anpassung;
import ghmessen.groesse.darstellung.Auswertung;
import ghmessen.groesse.darstellung.Eigenschaften;
import ghmessen.groesse.darstellung.LinienAuswertung;
import ghmessen.groesse.darstellung.TextAuswertung;
import ghmessen.util.AkimaInterpolation;
import ghmessen.util.PlotUtil;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

public class PlotCanvas extends JLabel implements MouseInputListener,
		PropertyChangeListener {
	private static final long serialVersionUID = -5237146451256861140L;
	private static final boolean DEBUG = false;
	/* Popup-Menü */
	private JPopupMenu popup;
	// Achsen & Werte
	private Skala xSkala, ySkala;

	public Graph2D[] graph;
	private AffineTransform[] yAT;

	// TODO Path2D-Cache für das Raster
	private int width, height;
	private Darstellung ds;
	private BasicStroke normalBreite = new BasicStroke(
			Konstanten.GRAPH_LINIE_SCHMAL);
	private BasicStroke hauptBreite = new BasicStroke(
			1.5f * Konstanten.GRAPH_LINIE_SCHMAL);
	/* Auswahl von Graphenstücken */
	private static final Color auswahlFarbe = new Color(Color.blue.getRed(),
			Color.blue.getGreen(), Color.blue.getBlue(), 122);
	private static final Color hoverFarbe = new Color(Color.yellow.getRed(),
			Color.yellow.getGreen(), Color.yellow.getBlue(), 150);
	private Composite original;
	private GraphHighlighter auswahl;
	private GraphHighlighter hover;
	private GraphHighlight hoverTag;
	private boolean hoverVisible = false;
	private boolean editable;
	private double auswahlAbstand = GRAPH_AUSWAHLBEREICH;

	private GHMessenView mainView;

	public PlotCanvas() {
		super();
		modus = Normal;
		mark = null;
		selectionTag = null;
		setOpaque(true);
		setBackground(Color.WHITE);
		xSkala = new Skala(-5, 5, 1.0, Skalierung.y, false,
				getWidth() <= 0 ? 100 : getWidth());
		ySkala = new Skala(-5, 5, 1.0, Skalierung.y, true,
				getHeight() <= 0 ? 100 : getHeight());
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				if (xSkala != null && ySkala != null) {
					xSkala.setLaenge(getWidth());
					ySkala.setLaenge(getHeight());
					for (Component c : getComponents())
						if (c instanceof Auswertung)
							((Auswertung) c).updateLocation();
					repaint();
				}

			}
		});
		auswahl = new GraphHighlighter(auswahlFarbe);
		auswahl.install(this);
		hover = new GraphHighlighter(hoverFarbe);
		hover.install(this);
		hoverTag = null;
		editable = true;
		setInheritsPopupMenu(false);
		this.ds = null;
		popup = new CanvasMenue();
		addMouseListener(this);
		addMouseMotionListener(this);
		mainView = null;
		auswertungPH = new AuswertungHandler();
		intervallPH = new IntervallHandler();
		skalierungPH = new SkalierungHandler();
		schriftPH = new SchriftPropertyHandler();
		liniePH = new LiniePropertyHandler();
		spaltePH = new SpalteHandler();
		tablePH = new TableHandler();

		setLayout(new FreiePositionenLayoutManager());
	}

	public PlotCanvas(Darstellung ds) {
		this();
		this.ds = ds;
		if (ds != null) {
			if (ds.getAuswertung() != null)
				for (Auswertung a : ds.getAuswertung())
					if (a.hatKomponente())
						add(a.getKomponente());
			connectToSpalte(ds, Darstellung.XACHSE);
			if (ds.getColumnCount() > 1)
				connectToSpalte(ds, ds.getY1(), "y1");
			connectToDarstellung(ds);
		}
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(Short.MAX_VALUE, Short.MAX_VALUE);
	}

	public boolean isBackgroundSet() {
		return true;
	}

	public void setXBounds(Double min, Double max) {
		xSkala.setIntervall(new Intervall(min, max), -1.0);
		for (Component c : getComponents())
			if (c instanceof Auswertung)
				((Auswertung) c).updateLocation();
	}

	public void setYBounds(Double min, Double max) {
		ySkala.setIntervall(new Intervall(min, max), -1.0);
		for (Component c : getComponents())
			if (c instanceof Auswertung)
				((Auswertung) c).updateLocation();
	}

	public void setDarstellung(Darstellung ds) {
		Darstellung alt = this.ds;
		if (alt != null) {
			disconnectFromSpalte(alt, Darstellung.XACHSE);
			disconnectFromSpalte(alt, alt.getY1(), "y1");
			disconnectFromDarstellung(alt);
		}
		this.ds = ds;
		removeAllHighlights();
		if (ds != null) {
			removeAll();
			if (ds.getAuswertung() != null)
				for (Auswertung a : ds.getAuswertung())
					if (a.hatKomponente())
						add(a.getKomponente());
			connectToSpalte(ds, Darstellung.XACHSE);
			if (ds.getColumnCount() > 1)
				connectToSpalte(ds, ds.getY1(), "y1");
			connectToDarstellung(ds);
		}
	}

	public Darstellung getDarstellung() {
		return ds;
	}

	public Color getFarbe(int i) {
		return FARBE_GRAPHEN.get(i - 1);
	}

	public Color getSelectionColor() {
		return PlotCanvas.auswahlFarbe;
	}

	public GraphHighlighter getHighlighter() {
		return auswahl;
	}

	public GraphHighlightPainter getSelectionPainter(boolean gefuellt) {
		return gefuellt ? auswahl.defaultFilled : auswahl.defaultOutline;
	}

	public Color getHoverColor() {
		return PlotCanvas.hoverFarbe;
	}

	public GraphHighlightPainter getHoverPainter(boolean gefuellt) {
		return gefuellt ? hover.defaultFilled : hover.defaultOutline;
	}

	public boolean isEditable() {
		return editable;
	}

	public void setEditable(boolean set) {
		this.editable = set;
	}

	public void setAuswahl(GraphHighlighter auswahl) {
		this.auswahl = auswahl;
	}

	public GraphHighlighter getAuswahl() {
		return auswahl;
	}

	public Skala getXSkala() {
		return xSkala;
	}

	public Skala getYSkala() {
		return ySkala;
	}

	public void setMausModus(MouseModus modus) {
		this.modus = modus;
		switch (modus) {
		case Normal:
		default:
			this.setCursor(null);
			if (DEBUG)
				System.out.println("[PlotCanvas] - setMausModus(Normal)");
			break;
		case PositioniereNeuenText:
			this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
			if (DEBUG)
				System.out
						.println("[PlotCanas] - setMausModus(PositioniereText)");
			break;
		case PositioniereLinieSenkrecht:
		case PositioniereLinieWaagerecht:
			this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
			if (DEBUG)
				System.out
						.println("[PlotCanvas] - setMausModus(PositioniereLinie...)");
			break;
		}
	}

	public MouseModus getMausModus() {
		return modus;
	}

	private Vector<Path2D> paintPolygon(Graphics2D g2, AffineTransform af,
			Double[] x, Double[] y) {
		Vector<Path2D> pfade = new Vector<Path2D>();
		if (x == null || x.length < 1)
			return pfade;
		Path2D pfad = new Path2D.Double();
		boolean fortsetzung = false;
		for (int i = 0; i < x.length; i++) {
			if (x[i] == null || y[i] == null) {
				fortsetzung = false;
				if (x[i] == null) {
					pfade.add(pfad);
					pfad = new Path2D.Double();
				}
				continue;
			}
			if (fortsetzung)
				pfad.lineTo(x[i], y[i]);
			else
				pfad.moveTo(x[i], y[i]);
			fortsetzung = true;
		}
		pfade.add(pfad);
		for (Path2D p : pfade) {
			p.transform(af);
			g2.draw(p);
			g2.setColor(FARBE_GRAPHEN.next(g2.getColor()));
		}
		return pfade;
	}

	private Vector<Path2D> paintWerte(Graphics2D g2, AffineTransform af,
			Double[] x, Double[] y) {
		Vector<Path2D> pfade = new Vector<Path2D>();
		if (x == null || x.length < 1)
			return pfade;
		double width = Math.abs(5.0/af.getScaleX());
		double height = Math.abs(5.0/af.getScaleY());
		Path2D pfad = new Path2D.Double();
		Shape punkt;
		for (int i = 0; i < x.length; i++) {
			if (x[i] != null && y[i] != null) {
				punkt = new Rectangle2D.Double(x[i] - width/2.0, y[i] - height/2.0, width,height);
				pfad.append(punkt, false);
			} else if (x[i] == null) {
				pfade.add(pfad);
				pfad = new Path2D.Double();
			}
		}
		pfade.add(pfad);
		for (Path2D p : pfade) {
			p.transform(af);
			g2.draw(p);
			g2.setColor(FARBE_GRAPHEN.next(g2.getColor()));
		}
		return pfade;
	}

	private Vector<Path2D> paintBalken(Graphics2D g2, AffineTransform af,
			Double[] x, Double[] y) {
		Vector<Path2D> pfade = new Vector<Path2D>();
		if (x == null || x.length < 2)
			return pfade;
		Path2D pfad = new Path2D.Double();
		Shape balken;
		g2.setComposite(Konstanten.TRANSPARENT);
		Double xl = null, yl, breite = null;
		int last;
		for (int i = 0; i < x.length - 1; i++) {
			if (x[i] != null && y[i] != null && x[i + 1] != null) {
				if (xl != null)
					xl += breite;
				else
					xl = x[i] - 0.5 * Math.abs(x[i + 1] - x[i]);
				yl = Math.min(0.0, y[i]);
				breite = 0.5 * (x[i + 1] + x[i]) - xl;
				g2.fill(balken = af
						.createTransformedShape(new Rectangle2D.Double(xl, yl,
								breite, Math.abs(y[i]))));
				pfad.append(balken, false);
			} else if (x[i] == null) {
				last = i - 1;
				if (last > 0 && x[last] != null && y[last] != null
						&& x[last - 1] != null) {
					xl = x[last] - 0.5 * Math.abs(x[last] - x[last - 1]);
					yl = Math.min(0.0, y[last]);
					breite = x[last] - x[last - 1];
					g2.fill(balken = af
							.createTransformedShape(new Rectangle2D.Double(xl,
									yl, breite, Math.abs(y[last]))));
					pfad.append(balken, false);
				}
				pfade.add(pfad);
				pfad = new Path2D.Double();
				g2.setColor(FARBE_GRAPHEN.next(g2.getColor()));
			}

		}
		last = x.length - 1;
		if (last > 0 && x[last] != null && y[last] != null
				&& x[last - 1] != null) {
			xl = x[last] - 0.5 * Math.abs(x[last] - x[last - 1]);
			yl = Math.min(0.0, y[last]);
			breite = x[last] - x[last - 1];
			g2.fill(balken = af.createTransformedShape(new Rectangle2D.Double(
					xl, yl, breite, Math.abs(y[last]))));
			pfad.append(balken, false);
		}
		pfade.add(pfad);
		g2.setComposite(original);
		return pfade;
	}

	private Vector<Path2D> paintAkimaInterpolation(Graphics2D g2,
			AffineTransform af, Double[] x, Double[] y) {
		Vector<Path2D> pfade = new Vector<Path2D>();
		int index = 0;
		int laenge;
		AkimaInterpolation cs;
		while (index < x.length) {
			while(index < x.length && y[index] == null)
				index++;
			laenge = 0;
			cs = null;
			try {
				while (index + laenge < x.length && x[index + laenge] != null)
					laenge++;
				if (laenge < 5) {
					// Die Interpolation benötigt mindestens 5 Punkte.
					if(DEBUG)
						System.out.println("[PlotCanvas] paintAkimaInterpolation: weniger als 5 Punkte - verwende Polygon");
					Path2D pfad = new Path2D.Double();
					pfad.moveTo(x[index], y[index]);
					for(int i=1;i<laenge;i++)
						if(y[index+i] != null)
							pfad.lineTo(x[index+i], y[index+i]);
					pfad.transform(af);
					g2.draw(pfad);
					pfade.add(pfad);
				} else if (index == 0 && laenge == x.length) {
					cs = new AkimaInterpolation(x, y);
					if (cs != null) {
						cs.getPfad().transform(af);
						g2.draw(cs.getPfad());
						pfade.add(cs.getPfad());
					}
				} else {
					Double[] xx = Arrays.copyOfRange(x, index, index + laenge);
					Double[] yy = Arrays.copyOfRange(y, index, index + laenge);
					cs = new AkimaInterpolation(xx, yy);
					if (cs != null) {
						cs.getPfad().transform(af);
						g2.draw(cs.getPfad());
						pfade.add(cs.getPfad());
						g2.setColor(FARBE_GRAPHEN.next(g2.getColor()));
					}
				}
			} catch (Exception e) {
				if (DEBUG)
					System.err.println("[PlotCanvas] - paintInterpolation: x("
							+ x + ") y(" + y + ") " + e.getLocalizedMessage());
			}
			index += laenge + 1;
		}
		return pfade;
	}

	private void paintRaster(Graphics2D g2) {
		int tfx = xSkala.transform(xSkala.getMinimum()), tfy = ySkala
				.transform(ySkala.getMinimum()), tfX = xSkala.transform(xSkala
				.getMaximum()), tfY = ySkala.transform(ySkala.getMaximum());
		// Nebengitter
		if (ds.getGraphEigenschaften().isRasterAn()) {
			g2.setStroke(normalBreite);
			g2.setColor(Color.LIGHT_GRAY);
			Path2D xNeben = new Path2D.Double();
			int[] x = xSkala.getStricheKlein();
			for (int i = 0; i < x.length; i++) {
				xNeben.moveTo(x[i], tfy);
				xNeben.lineTo(x[i], tfY);
			}
			Path2D yNeben = new Path2D.Double();
			int[] y = ySkala.getStricheKlein();
			for (int i = 0; i < y.length; i++) {
				yNeben.moveTo(tfx, y[i]);
				yNeben.lineTo(tfX, y[i]);
			}
			g2.draw(xNeben);
			g2.draw(yNeben);

			// Hauptgitter
			g2.setStroke(hauptBreite);
			g2.setColor(Color.LIGHT_GRAY);
			Path2D xHaupt = new Path2D.Double();
			x = xSkala.getStricheGross();
			for (int i = 0; i < x.length; i++) {
				xHaupt.moveTo(x[i], tfy);
				xHaupt.lineTo(x[i], tfY);
			}
			Path2D yHaupt = new Path2D.Double();
			y = ySkala.getStricheGross();
			for (int i = 0; i < y.length; i++) {
				yHaupt.moveTo(tfx, y[i]);
				yHaupt.lineTo(tfX, y[i]);
			}
			g2.draw(xHaupt);
			g2.draw(yHaupt);
		}

	}

	private void paintAnpassungen(Graphics2D g) {
		if (ds == null || ds.getAuswertung() == null)
			return;
		Graphics2D g2 = (Graphics2D) g.create();
		int col;
		int stdCol = ds.getY1();
		try {
			Vector<Auswertung> anpassung = ds.getAuswertung();
			for (Auswertung aus : anpassung) {
				if (aus instanceof Anpassung) {
					Anpassung a = (Anpassung) aus;
					col = a.getSpalte();
					if (col > 0)
						g.setColor(a.getFarbe());
					Path2D p2d = null;
					if (a.hatZeichnungVonBis())
						p2d = new Path2D.Double((Shape) a.getZeichnungVonBis()
								.clone(), col < 0 ? yAT[stdCol] : yAT[col]);
					else
						p2d = new Path2D.Double((Shape) a.getZeichnungVonBis()
								.clone(), col < 0 ? yAT[stdCol] : yAT[col]);
					g.draw(p2d);
				}
			}
		} finally {
			g2.dispose();
		}
	}

	@SuppressWarnings("unused")
	@Override
	public void paintComponent(Graphics g) {

		original = ((Graphics2D) g).getComposite();
		width = this.getWidth();
		height = this.getHeight();
		Rectangle cr = g.getClipBounds();
		boolean allesZeichnen = cr == null || cr.x == 0 && cr.y == 0
				&& cr.width == width && cr.height == height;
		if (DEBUG)
			System.out.println("[PlotCanvas] - paintComponent: (" + cr.x + ","
					+ cr.y + "|w:" + cr.width + ",h:" + cr.height + ") alles("
					+ allesZeichnen + ")");
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		// Alte Werte sichern
		Color altColor = g2.getColor();
		Stroke altStroke = g2.getStroke();
		if (DEBUG && !allesZeichnen) {
			g2.setColor(Color.BLUE.brighter());
			g2.drawRect(cr.x, cr.y, cr.width - 1, cr.height - 1);
		}
		// Transformation
		g2.setClip(0, 0, width, height);
		// Hintergrundfarbe
		g2.setColor(Color.WHITE);
		if (ds == null)
			return;
		paintRaster(g2);
		if (allesZeichnen) {
			graph = new Graph2D[ds.getColumnCount()];
			yAT = new AffineTransform[ds.getColumnCount()];
		}
		// gibt es eine skalierte Größe ?
		boolean skaliert = false;
		for (int i = 0; i < ds.getColumnCount(); i++) {
			if (ds.getYS(i) != Skalierung.y) {
				skaliert = true;
				break;
			}
		}
		g2.setStroke(ds.getGraphEigenschaften().getLinie());
		int rowEnde = ds == null ? -1 : ds.getRowCount() - 1;
		if (rowEnde > -1) {
			int rowStart = 0;
			Vector<Integer> abschnitt = new Vector<Integer>();
			abschnitt.add(rowStart);
			abschnitt.add(rowEnde);
			Double x0, y0, x1, y1;
			if (!allesZeichnen && !ds.emptyRowsExist()) {
				double paintXmin = xSkala.inversTransform(cr.x);
				int anfang = rowStart;
				do {
					x0 = skaliert || ds.getColumnFaktor(0) != 1.0 ? 
							ds.getScaledDoubleValueAt(anfang, 0, ds.getColumnFaktor(0)) :
								ds.getDoubleValueAt(anfang, 0);
					anfang++;
				} while (x0 < paintXmin && anfang <= rowEnde);
				rowStart = Math.max(0, anfang - 2);
				double paintXmax = xSkala.inversTransform(cr.x + cr.width);
				int ende = rowEnde;
				while (ende > rowStart) {
					x1 = skaliert || ds.getColumnFaktor(0) != 1.0 ? 
							ds.getScaledDoubleValueAt(ende, 0, ds.getColumnFaktor(0)) : 
								ds.getDoubleValueAt(ende, 0);
					try {
						if (paintXmax > x1)
							break;
					} catch (NullPointerException e) {
						if (DEBUG)
							e.printStackTrace();
					}
					ende--;
				}
				rowEnde = Math.min(ende + 1, rowEnde);
				abschnitt.set(0, rowStart);
				abschnitt.set(1, rowEnde);
				if (DEBUG)
					System.out
							.println("[PlotCanvas] - paintComponent: !allesZeichnen row("
									+ rowStart
									+ ") rowEnde("
									+ rowEnde
									+ ") xmin("
									+ paintXmin
									+ ") xmax("
									+ paintXmax + ")");
			}
			Double[] x, y;
			for (int index = 0; index < abschnitt.size(); index += 2) {
				rowStart = abschnitt.get(index);
				rowEnde = abschnitt.get(index + 1);
				if (!skaliert && ds.getColumnFaktor(0) == 1.0)
					x = ds.getColumnArray(0, rowStart, rowEnde);
				else
					x = ds.getScaledColumnArray(0, rowStart, rowEnde, ds.getColumnFaktor(0));
				AffineTransform xaf = new AffineTransform(
						xSkala.getAffineTransform());
				for (int col = 1; col < ds.getColumnCount(); col++) {
					yAT[col] = AffineTransform.getTranslateInstance(0,
							ySkala.getLaenge());
					yAT[col].scale(
							1.0,
							-ySkala.getLaenge()
									/ (ds.getMaximum(col) - ds.getMinimum(col)));
					yAT[col].translate(0, -ds.getMinimum(col));
					yAT[col].concatenate(xaf);
					if (!skaliert)
						y = ds.getColumnArray(col, rowStart, rowEnde);
					else
						y = ds.getScaledColumnArray(col, rowStart, rowEnde);
					g.setColor(FARBE_GRAPHEN.get(index / 2 + col - 1));
					if (ds.getGraphEigenschaften().isWerteAn()) {
						if (allesZeichnen)
							graph[col] = new Graph2D(paintWerte(g2, yAT[col],
									x, y), WERT);
						else
							paintWerte(g2, yAT[col], x, y);
						g.setColor(FARBE_GRAPHEN.get(index / 2 + col - 1));
					}
					if (ds.getGraphEigenschaften().isBalkenAn()) {
						if (allesZeichnen)
							graph[col] = new Graph2D(paintBalken(g2, yAT[col],
									x, y), BALKEN);
						else
							paintBalken(g2, yAT[col], x, y);
						g.setColor(FARBE_GRAPHEN.get(index / 2 + col - 1));
					}
					if (ds.getGraphEigenschaften().isLinieAn()) {
						if (allesZeichnen) {
							if (ds.getGraphEigenschaften()
									.getKeineInterpolation())
								graph[col] = new Graph2D(paintPolygon(g2,
										yAT[col], x, y), LINIE);
							else
								graph[col] = new Graph2D(
										paintAkimaInterpolation(g2, yAT[col],
												x, y), LINIE);
						} else {
							if (ds.getGraphEigenschaften()
									.getKeineInterpolation())
								paintPolygon(g2, yAT[col], x, y);
							else
								paintAkimaInterpolation(g2, yAT[col], x, y);
						}
						g.setColor(FARBE_GRAPHEN.get(index / 2 + col - 1));
					}
				}
			}
		}
		// Anpassungskurven
		paintAnpassungen(g2);

		if (auswahl != null)
			auswahl.paint(g2);
		if (hover != null)
			hover.paint(g2);
		// Gesicherte Werte zurückschreiben
		if (DEBUG) {
			g2.setColor(Color.CYAN);
			g2.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
		}
		g2.setColor(altColor);
		g2.setStroke(altStroke);
	}

	/*************************************************************
	 * Highlighting
	 ************************************************************/

	private Path2D path_append(Path2D p, int typ, double[] coords) {
		switch (typ) {
		default:
			break;
		case PathIterator.SEG_CLOSE:
			p.closePath();
			break;
		case PathIterator.SEG_LINETO:
			p.lineTo(coords[0], coords[1]);
			break;
		case PathIterator.SEG_MOVETO:
			p.moveTo(coords[0], coords[1]);
			break;
		case PathIterator.SEG_CUBICTO:
			p.curveTo(coords[0], coords[1], coords[2], coords[3], coords[4],
					coords[5]);
			break;
		}
		return p;
	}

	public Path2D modelToView(int col, int rowA, int rowE)
			throws BadLocationException {
		if (DEBUG)
			System.out.println("[PlotCanvas] - modelToView: (" + col + ","
					+ rowA + "," + rowE + ")");
		if (col < 0 || rowA < 0 || rowE < 0)
			throw new BadLocationException("PlotCanvas: Spalte(" + col
					+ ") Zeile1(" + rowA + ") und Zeile2(" + rowE
					+ " ist eine unmögliche Position.");
		if (getHeight() <= 0 || getWidth() <= 0)
			return null;
		Darstellung ds = getDarstellung();
		if (ds == null || col > ds.getColumnCount() - 1)
			throw new BadLocationException("PlotCanvas-Darstellung(" + ds
					+ ") ist keine gültige Darstellung.");
		if (graph == null || col >= graph.length || graph[col] == null)
			throw new BadLocationException("PlotCanvas: Graph nicht vorhanden.");
		Path2D result = new Path2D.Double();
		boolean include;
		double[] coords = new double[6];
		int typ;
		int index = 0;
		boolean initial = true;
		Iterator<Integer> it = ds.emptyRowsExist() ? ds.getEmptyRowIterator()
				: null;
		int nextEmptyRow = it == null ? -1 : it.next();
		for (PathIterator p = graph[col].getPathIterator(null); !p.isDone()
				&& index <= rowE; index++) {
			if (nextEmptyRow != -1 && index == nextEmptyRow) {
				index++;
				nextEmptyRow = it.hasNext() ? it.next() : -1;
			}
			include = rowA <= index && index <= rowE;
			switch (graph[col].getTyp()) {
			case WERT:
			case BALKEN:
				for (int i = 0; i < 6; i++) {
					if (include) {
						typ = p.currentSegment(coords);
						path_append(result, typ, coords);
					}
					p.next();
				}
				break;
			case LINIE:
				if (include) {
					typ = p.currentSegment(coords);
					path_append(result,
							initial ? PathIterator.SEG_MOVETO : typ, coords);
					initial = false;
				}
				p.next();
				break;
			case LEER:
				p.next();
				break;
			}
		}
		return result;
	}

	/*
	 * presetCol : -1 - x und y prüfen presetCol : [1,maxCol] nur x prüfen
	 * presetIdx: -1 alle Abschnitte prüfen, Abschnitt mit presetIdx prüfen
	 */
	public ZellAdresse viewToModel(PlotCanvas c, Point p, int presetCol,
			int presetIdx) {
		if (DEBUG)
			System.out.println("[PlotCanvas] - viewToModel: p(" + p.x + "|"
					+ p.y + ")");
		try {
			ZellAdresse mPt = new ZellAdresse();
			Rectangle2D auswahl = new Rectangle2D.Double(p.x - auswahlAbstand
					/ 2.0, p.y - auswahlAbstand / 2.0, auswahlAbstand,
					auswahlAbstand);
			for (int col = 1; col < graph.length; col++) {
				if (presetCol != -1 && presetCol != col)
					continue;
				if (presetCol != -1 || graph[col].intersects(auswahl)) {
					mPt.setSpalte(col);
					Relation r = graph[col].getNeighbourRelation(p, presetIdx);
					if (r.abstand > auswahlAbstand / 2.0) {
						if (r.position < 0)
							r.nummer = Math.max(0, r.nummer - 1);
						mPt.setBreite(1);
					}
					mPt.setZeile(r.nummer);
					if (DEBUG)
						System.out.println("[PlotCanvas] - viewToModel -> ("
								+ mPt.getSpalte() + "," + mPt.getZeile() + ","
								+ mPt.getBreite() + ")");
					return mPt;
				}
			}
			return null;
		} catch (NullPointerException e) {
			if (DEBUG)
				System.err.println("[PlotCanvas] - viewToModel: "
						+ e.getLocalizedMessage());
			return null;
		}
	}

	public void damageRange(PlotCanvas component, int col, int rowA, int rowE) {
		try {
			if (DEBUG)
				System.out.println("[PlotCanvas] - damageRange: (" + col + ","
						+ rowA + "," + rowE + ")");
			Path2D pfad = component.modelToView(col, rowA, rowE);
			Rectangle2D result = pfad.getBounds2D();
			float increaseBy = GraphHighlightPainter.getStrokeWidth();
			result.setRect(result.getX() - increaseBy, result.getY()
					- increaseBy, result.getWidth() + 2 * increaseBy,
					result.getHeight() + 2 * increaseBy);
			component.repaint(result.getBounds());
		} catch (BadLocationException ex) {
			if (DEBUG)
				ex.printStackTrace();
		} catch (NullPointerException ex) {
			if (DEBUG)
				ex.printStackTrace();
		}
	}

	public void removeAllHighlights() {
		auswahl.removeAllHighlights();
		hover.removeAllHighlights();
		selectionTag = null;
		hoverTag = null;
	}

	public Point2D viewToXY(Point pt) {
		return new Point2D.Double(xSkala.inversTransform(pt.x),
				ySkala.inversTransform(pt.y));
	}

	public Point xYToView(Point2D p) {
		return new Point(xSkala.transform(p.getX()), ySkala.transform(p.getY()));
	}

	/************************************************************
	 * MouseInputListener, FocusListener
	 ************************************************************/

	private MouseModus modus;
	private ZellAdresse mark;
	private GraphHighlight selectionTag;

	private void setHoverVisible(boolean set) {
		hoverVisible = set;
	}

	public boolean isHoverVisible() {
		return hoverVisible;
	}

	private ZellAdresse getMark() {
		return mark;
	}

	public void mouseClicked(MouseEvent e) {
		int nclicks = e.getClickCount();
		if (DEBUG)
			System.out.println("[PlotCanvas] - mouseClicked: nclicks("
					+ nclicks + ")");
		if (!e.isConsumed()) {
			if (SwingUtilities.isLeftMouseButton(e)) {
				// mouse 1 Verhalten
				if (nclicks == 1) {
					boolean vertikal = false;
					switch (modus) {
					default:
					case Normal:
						if (hoverTag != null) {
							if (!e.isControlDown()) {
								auswahl.removeAllHighlights();
								selectionTag = null;
							}
							auswahl.addHighlight(hover
									.removeHighlight(hoverTag), hoverTag
									.getPainter().istGefuellt());
							hoverTag = null;
							selectionTag = null;
						} else {
							auswahl.removeAllHighlights();
							selectionTag = null;
						}
						break;
					case PositioniereNeuenText:
						if (ds != null && ds.getGraphEigenschaften() != null) {
							TextAuswertung ta = new TextAuswertung(ds
									.getGraphEigenschaften().getSchrift(),
									"neuer Text", null, viewToXY(e.getPoint()),
									ds.getY1());
							ds.neueAuswertung(ta);
							ta.updateLocation();
							ta.starteEingabe();
						}
						setMausModus(Normal);
						break;
					case PositioniereLinieSenkrecht:
						vertikal = true;
					case PositioniereLinieWaagerecht:
						if (ds != null && ds.getGraphEigenschaften() != null) {
							LinienAuswertung la = new LinienAuswertung(
									vertikal, ds.getGraphEigenschaften()
											.getLinie(),
									viewToXY(e.getPoint()), ds.getY1());
							ds.neueAuswertung(la);
							la.updateLocation();
						}
						setMausModus(Normal);
						break;
					}
				}
			} else if (SwingUtilities.isMiddleMouseButton(e)) {
				// mouse 2 Verhalten
				if (nclicks == 1 && isEditable() && isEnabled()) {
					// paste system selection, if it exists
				}
			}
			// mouse 3 Verhalten: Popup-Menü in MousePressed, MouseReleased
		}
	}

	public void mousePressed(MouseEvent e) {
		if (SwingUtilities.isRightMouseButton(e)) {
			if (e.isPopupTrigger())
				popup.show((Component) e.getSource(), e.getX(), e.getY());
		} else if (SwingUtilities.isLeftMouseButton(e) && isEnabled())
			adjustCaret(e);
	}

	private void adjustCaret(MouseEvent e) {
		if (getMark() != null) {
			moveCaret(e, getMark().getSpalte()); // drag läuft
		} else {
			positionCaret(e); // drag beginnt
		}
	}

	protected void positionCaret(MouseEvent e) {
		Point pt = new Point(e.getX(), e.getY());
		ZellAdresse pos = viewToModel(this, pt, -1, -1);
		if (pos != null) {
			setDot(pos);
		}
	}

	protected void moveCaret(MouseEvent e, int spalte) {
		Point pt = new Point(e.getX(), e.getY());
		ZellAdresse pos = viewToModel(this, pt, spalte, mark.getZeile());
		if (pos != null) {
			moveDot(pos);
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (SwingUtilities.isRightMouseButton(e)) {
			if (e.isPopupTrigger())
				popup.show((Component) e.getSource(), e.getX(), e.getY());
		}
	}

	public void mouseEntered(MouseEvent e) {
		if (isEnabled())
			setHoverVisible(true);
		// mark = null;
	}

	public void mouseExited(MouseEvent e) {
		setHoverVisible(false);
		if (hoverTag != null && hover != null) {
			hover.removeAllHighlights();
			hoverTag = null;
		}
		// mark = null;
		if (mainView == null)
			mainView = GHMessen.getApplication().getMainView();
		mainView.statusMessageLabel.setText(" ");
	}

	public void mouseDragged(MouseEvent e) {
		if ((!e.isConsumed()) && SwingUtilities.isLeftMouseButton(e)) {
			hover.removeAllHighlights();
			hoverTag = null;
			if (getMark() == null && e.isControlDown()) {
				selectionTag = null;
			} else if (getMark() == null) {
				auswahl.removeAllHighlights();
				selectionTag = null;
			}
			adjustCaret(e);
		}
	}

	public void mouseMoved(MouseEvent e) {
		if (isEnabled()) {
			if (mark != null)
				mark = null;
			if (ds == null)
				return;
			if (mainView == null)
				mainView = GHMessen.getApplication().getMainView();
			Point pt = new Point(e.getX(), e.getY());
			ZellAdresse pos = viewToModel(this, pt, -1, -1);
			if (pos != null) {
				if (hover != null && getDarstellung() != null) {
					try {
						int von, bis;
						if (ds.emptyRowsExist()) {
							Iterator<Integer> it = ds.getEmptyRowIterator();
							von = 0;
							bis = it.next() - 1;
							while (it.hasNext() && pos.getZeile() > bis) {
								von = bis + 2;
								bis = it.next() - 1;
							}
							if (pos.getZeile() > bis) {
								von = bis + 2;
								bis = ds.getRowCount() - 1;
							}
						} else {
							von = 0;
							bis = ds.getRowCount() - 1;
						}
						if (hoverTag != null) {
							if (hoverTag.getCol() != pos.getSpalte()
									|| hoverTag.getRow1() != von
									|| hoverTag.getRow2() != bis) {
								hover.changeHighlight(hoverTag,
										pos.getSpalte(), von, bis);
							}
						} else {
							GraphHighlightPainter p = getHoverPainter(graph[pos
									.getSpalte()].getTyp() == BALKEN);
							hoverTag = hover.addHighlight(pos.getSpalte(), von,
									bis, p);
						}
					} catch (BadLocationException ex) {
						throw new Error("Bad caret position");
					}
				}
			} else {
				hoverTag = null;
				hover.removeAllHighlights();
			}
			Point2D p2d = new Point2D.Double(xSkala.inversTransform(pt.x),
					ySkala.inversTransform(pt.y));
			mainView.statusMessageLabel.setText(String.format(
					"Koordinaten(%2.2f|%2.2f)",
					xSkala.getSkalierung().istLogarithmisch() ? Math.pow(10.0,
							p2d.getX()) : p2d.getX(),
					ySkala.getSkalierung().istLogarithmisch() ? Math.pow(10.0,
							p2d.getY()) : p2d.getY()));
		} else if (selectionTag != null || hoverTag != null)
			removeAllHighlights();
	}

	public void moveDot(ZellAdresse dot) {
		if (!isEnabled()) {
			setDot(dot);
			return;
		}
		handleMoveDot(dot);
	}

	void handleMoveDot(ZellAdresse dot) {
		if (ds != null) {
			int col = mark.getSpalte();
			int p0 = Math.min(dot.getZeile(), mark.getZeile());
			int p1 = Math.max(dot.getZeile(), mark.getZeile());
			if (p1 == dot.getZeile() && dot.getBreite() > 0
					|| p1 == mark.getZeile() && mark.getZeile() > 0)
				p1++;
			if(DEBUG)
				System.out.println("[PlotCanvas] - handleMoveDot: (" + col + ","
					+ p0 + "," + p1 + ")");
			// bei p0 == p1 mindestens zwei Punkte auswählen
			// if (p0 == p1) {
			// p1 = Math.min(p1 + 1, ds.getRowCount() - 1);
			// }
			try {
				if (selectionTag != null) {
					if (col != selectionTag.getCol()
							|| p0 != selectionTag.getRow1()
							|| p1 != selectionTag.getRow2())
						auswahl.changeHighlight(selectionTag, col, p0, p1);
				} else {
					GraphHighlightPainter p = getSelectionPainter(graph[col]
							.getTyp() == BALKEN);
					selectionTag = auswahl.addHighlight(col, p0, p1, p);
				}
			} catch (BadLocationException e) {
				throw new Error("Bad caret position");
			}
		}
	}

	public void setDot(ZellAdresse dot) {
		handleSetDot(dot);
	}

	void handleSetDot(ZellAdresse dot) {
		// move dot, if it changed
		if (dot != null) {
			if (ds != null) {
				dot.setZeile(Math.min(dot.getZeile(), ds.getRowCount() - 1));
				dot.setZeile(Math.max(dot.getZeile(), 0));
				dot.setSpalte(Math.max(dot.getSpalte(), 1));
				dot.setSpalte(Math.min(dot.getSpalte(), ds.getColumnCount() - 1));
			} else {
				dot = null;
			}
		}
		mark = dot;
		if (selectionTag != null) {
			auswahl.removeHighlight(selectionTag);
			selectionTag = null;
		}
	}

	/************************************************************
	 * PropertyChangeListener: DARSTELLUNG, GRAPHEIGENSCHAFTEN
	 ************************************************************/
	private SchriftPropertyHandler schriftPH;
	private LiniePropertyHandler liniePH;
	private AuswertungHandler auswertungPH;
	private IntervallHandler intervallPH;
	private SkalierungHandler skalierungPH;
	private SpalteHandler spaltePH;
	private TableHandler tablePH;

	public void connectToDarstellung(Darstellung ds) {
		ds.getGraphEigenschaften().addPropertyChangeListener(this);
		ds.getGraphEigenschaften().addPropertyChangeListener("schrift",
				schriftPH);
		ds.getGraphEigenschaften().addPropertyChangeListener("linie", liniePH);
		handleSchriftEvent(new PropertyChangeEvent(ds.getGraphEigenschaften(),
				"schrift", null, ds.getGraphEigenschaften().getSchrift()));
		handleLinieEvent(new PropertyChangeEvent(ds.getGraphEigenschaften(),
				"linie", null, ds.getGraphEigenschaften().getLinie()));
		ds.addPropertyChangeListener("entferneAuswertung", auswertungPH);
		ds.addPropertyChangeListener("neueAuswertung", auswertungPH);
		ds.addTableModelListener(tablePH);
	}

	public void disconnectFromDarstellung(Darstellung ds) {
		ds.getGraphEigenschaften().removePropertyChangeListener(this);
		ds.getGraphEigenschaften().removePropertyChangeListener("schrift",
				schriftPH);
		ds.getGraphEigenschaften().removePropertyChangeListener("linie",
				liniePH);
		ds.removePropertyChangeListener("entferneAuswertung", auswertungPH);
		ds.removePropertyChangeListener("neueAuswertung", auswertungPH);
		ds.removeTableModelListener(tablePH);
	}

	public void connectToSpalte(Darstellung ds, int spalte) {
		connectToSpalte(ds, spalte, null);
	}

	public void connectToSpalte(Darstellung ds, int spalte, String wechselPN) {
		ds.addPropertyChangeListener("yS" + spalte, skalierungPH);
		ds.addPropertyChangeListener("autoY" + spalte, skalierungPH);
		ds.addPropertyChangeListener("minimum" + spalte, intervallPH);
		ds.addPropertyChangeListener("maximum" + spalte, intervallPH);
		ds.addPropertyChangeListener("intervall" + spalte, intervallPH);
		handleIntervall(new PropertyChangeEvent(ds, "intervall" + spalte, null,
				new Intervall(ds.getMinimum(spalte), ds.getMaximum(spalte))));
		if (wechselPN != null)
			ds.addPropertyChangeListener(wechselPN, spaltePH);
	}

	public void disconnectFromSpalte(Darstellung ds, int spalte) {
		disconnectFromSpalte(ds, spalte, null);
	}

	public void disconnectFromSpalte(Darstellung ds, int spalte,
			String wechselPN) {
		ds.removePropertyChangeListener("yS" + spalte, skalierungPH);
		ds.removePropertyChangeListener("autoY" + spalte, skalierungPH);
		ds.removePropertyChangeListener("minimum" + spalte, intervallPH);
		ds.removePropertyChangeListener("maximum" + spalte, intervallPH);
		ds.removePropertyChangeListener("intervall" + spalte, intervallPH);
		if (wechselPN != null)
			ds.removePropertyChangeListener(wechselPN, spaltePH);
	}

	public class SchriftPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSchriftEvent(evt);
		}
	}

	public class LiniePropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleLinieEvent(evt);
		}
	}

	public class IntervallHandler implements Serializable,
			PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleIntervall(evt);
		}
	}

	public class SkalierungHandler implements Serializable,
			PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSkalierung(evt);
		}
	}

	public class AuswertungHandler implements Serializable,
			PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleAuswertung(evt);
		}
	}

	public class SpalteHandler implements Serializable, PropertyChangeListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSpalte(evt);
		}
	}

	public class TableHandler implements Serializable, TableModelListener {
		/**
* 
*/
		private static final long serialVersionUID = 1L;

		@Override
		public void tableChanged(TableModelEvent evt) {
			handleTableChanged(evt);
		}
	}

	public void handleAuswertung(PropertyChangeEvent evt) {
		String s = evt.getPropertyName();
		if (s.equals("entferneAuswertung")) {
			Auswertung a = (Auswertung) evt.getOldValue();
			if (a.hatKomponente())
				remove(a.getKomponente());
		} else if (s.equals("neueAuswertung")) {
			Auswertung a = (Auswertung) evt.getNewValue();
			if (a.hatKomponente()) {
				add(a.getKomponente());
			}
		}
		repaint();
	}

	public void handleSchriftEvent(PropertyChangeEvent evt) {
		Eigenschaften es = (Eigenschaften) evt.getSource();
		for (Component c : getComponents())
			c.setFont(es.getSchrift());
		repaint();
	}

	public void handleLinieEvent(PropertyChangeEvent evt) {
		Eigenschaften es = (Eigenschaften) evt.getSource();
		normalBreite = es.getLinie();
		hauptBreite = new BasicStroke(1.5f * es.getLinie().getLineWidth());
		for (Component c : getComponents())
			if (c instanceof Auswertung)
				((Auswertung) c).setLinienbreite(es.getLinie());
		revalidate();
		repaint();
	}

	public void handleIntervall(PropertyChangeEvent evt) {
		int index = PlotUtil.getSpalteFromProperty(evt.getPropertyName());
		if (index < 0)
			return;
		if (index == Darstellung.XACHSE) {
			setXBounds(ds.getMinimum(index), ds.getMaximum(index));
		} else {
			setYBounds(ds.getMinimum(index), ds.getMaximum(index));
		}
		repaint();
	}

	public void handleSkalierung(PropertyChangeEvent evt) {
		int index = PlotUtil.getSpalteFromProperty(evt.getPropertyName());
		if (index < 0)
			return;
		if (index == Darstellung.XACHSE) {
			xSkala.setSkalierung(ds.getYS(index));
			setXBounds(ds.getMinimum(index), ds.getMaximum(index));
		} else {
			ySkala.setSkalierung(ds.getYS(index));
			setYBounds(ds.getMinimum(index), ds.getMaximum(index));
		}
		revalidate();
		repaint();
	}

	public void handleSpalte(PropertyChangeEvent evt) {
		Darstellung ds = (Darstellung) evt.getSource();
		disconnectFromSpalte(ds, (Integer) evt.getOldValue(),
				evt.getPropertyName());
		int index = (Integer) evt.getNewValue();
		connectToSpalte(ds, index, evt.getPropertyName());
	}

	public void handleTableChanged(TableModelEvent evt) {
		Darstellung ds = (Darstellung) evt.getSource();
		if (evt.getFirstRow() == TableModelEvent.HEADER_ROW) {
			removeAllHighlights();
			setXBounds(ds.getMinimum(Darstellung.XACHSE),
					ds.getMaximum(Darstellung.XACHSE));
			int spalte = ds.getY1();
			setYBounds(ds.getMinimum(spalte), ds.getMaximum(spalte));
			repaint();
		} else if (evt.getType() == INSERT && evt.getColumn() == ALL_COLUMNS
				|| evt.getType() == UPDATE) {
			boolean minMax = ds.getMinimum(Darstellung.XACHSE) != xSkala
					.getMinimum();
			minMax = minMax
					|| ds.getMaximum(Darstellung.XACHSE) != xSkala.getMaximum();
			int spalte = ds.getY1();
			minMax = minMax || ds.getMinimum(spalte) != ySkala.getMinimum();
			minMax = minMax || ds.getMaximum(spalte) != ySkala.getMaximum();
			if (minMax) {
				setXBounds(ds.getMinimum(Darstellung.XACHSE),
						ds.getMaximum(Darstellung.XACHSE));
				setYBounds(ds.getMinimum(spalte), ds.getMaximum(spalte));
				repaint();
			} else {
				int first = evt.getFirstRow();
				first = Math.max(0, first - 1);
				int last = evt.getLastRow();
				try {
					double min = ds.getDoubleValueAt(first, Darstellung.XACHSE, ds.getColumnFaktor(Darstellung.XACHSE));
					double max = ds.getDoubleValueAt(last, Darstellung.XACHSE, ds.getColumnFaktor(Darstellung.XACHSE));
					int x, w;
					x = xSkala.transform(min);
					w = xSkala.transform(max) - x;
					if (DEBUG)
						System.out
								.println("[PlotPanel] - graph.repaint: first("
										+ first + ") last(" + last + ") min("
										+ min + ") max(" + max + ")");
					repaint(x, 0, w, getHeight());
				} catch (NullPointerException ex) {
					if (DEBUG)
						ex.printStackTrace();
				}
			}
		} else if (ds != null && ds.getRowCount() > 0) {
			setXBounds(ds.getMinimum(Darstellung.XACHSE),
					ds.getMaximum(Darstellung.XACHSE));
			int spalte = ds.getY1();
			setYBounds(ds.getMinimum(spalte), ds.getMaximum(spalte));
			repaint();
		} else if (ds != null) {
			repaint();
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		// es wurde eine Eigenschaft des Graphen geändert.
		repaint();
	}
}
