/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge;

import ghmessen.GHMessen;
import ghmessen.dialoge.eigenschaften.BereichInterface;
import ghmessen.dialoge.eigenschaften.BereichSensoren;
import ghmessen.util.CursorTk;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.BevelBorder;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;

public class EigenschaftenBox extends JDialog implements BereichInterface {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8737205330456089740L;

	private static final boolean DEBUG = false;
	private static final String[] bereiche = new String[] { "BEREICH_SENSOREN" };
	private static final int BEREICH_SENSOREN = 0;
	private JPanel bereichAuswahl;
	private JPanel bereich;
	private CardLayout bereich_cl;
	private static ResourceMap resourceMap;
	private static ActionMap actionMap;

	public EigenschaftenBox(Frame parent, boolean modal, GHMessen app) {
		super(parent, modal);

		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(EigenschaftenBox.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(EigenschaftenBox.class);

		setSize(900, 500);
		setTitle(resourceMap.getString("eigenschaften.title"));
		/* Schließen des Fensters */
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				if (DEBUG)
					System.out
							.println("[EigenschaftenBox] - windowClosing aufgerufen.");
				int antwort = JOptionPane.showConfirmDialog(e.getComponent(),
						resourceMap.getString("speichernDialog.frage"),
						resourceMap.getString("speichernDialog.title"),
						JOptionPane.YES_NO_CANCEL_OPTION);
				switch (antwort) {
				case JOptionPane.YES_OPTION:
					schreiben();
				case JOptionPane.NO_OPTION:
					dispose();
					break;
				default:
					break;
				}
			}
		});

		JPanel inhalt = new JPanel();
		getContentPane().add(inhalt, BorderLayout.CENTER);

		bereichAuswahl = new JPanel();
		bereichAuswahl.setBorder(new BevelBorder(BevelBorder.LOWERED, null,
				null, null, null));
		bereichAuswahl.setBackground(Color.WHITE);

		/* Bereich einrichten */
		bereich = new JPanel();
		bereich_cl = new CardLayout();
		bereich.setLayout(bereich_cl);
		bereich.add(new BereichSensoren(), bereiche[BEREICH_SENSOREN]);
		bereich_cl.show(bereich, bereiche[BEREICH_SENSOREN]);

		GroupLayout gl_inhalt = new GroupLayout(inhalt);
		gl_inhalt.setHorizontalGroup(gl_inhalt.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_inhalt
						.createSequentialGroup()
						.addContainerGap()
						.addComponent(bereichAuswahl,
								GroupLayout.PREFERRED_SIZE, 160,
								GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(ComponentPlacement.RELATED)
						.addComponent(bereich, GroupLayout.DEFAULT_SIZE, 709,
								Short.MAX_VALUE).addContainerGap()));
		gl_inhalt
				.setVerticalGroup(gl_inhalt
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								Alignment.TRAILING,
								gl_inhalt
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_inhalt
														.createParallelGroup(
																Alignment.TRAILING)
														.addComponent(
																bereich,
																Alignment.LEADING,
																GroupLayout.DEFAULT_SIZE,
																446,
																Short.MAX_VALUE)
														.addComponent(
																bereichAuswahl,
																Alignment.LEADING,
																GroupLayout.DEFAULT_SIZE,
																446,
																Short.MAX_VALUE))
										.addContainerGap()));

		/* Bereichsauswahl einrichten */
		ButtonGroup symbole = new ButtonGroup();
		JToggleButton auswahlSensoren = new JToggleButton(
				resourceMap.getString("auswahlSensoren.Action.text"));
		auswahlSensoren.setSelected(true);
		auswahlSensoren.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				bereich_cl.show(bereich, bereiche[BEREICH_SENSOREN]);
			}

		});
		symbole.add(auswahlSensoren);
		bereichAuswahl.add(auswahlSensoren);
		inhalt.setLayout(gl_inhalt);

		/* Knöpfe unten einrichten */
		JPanel knoepfe = new JPanel();
		getContentPane().add(knoepfe, BorderLayout.SOUTH);

		JButton hilfe = new JButton(actionMap.get("hilfe"));
		hilfe.setEnabled(GHMessen.hilfe != null);
		if (GHMessen.hilfe != null && GHMessen.hSet != null)
			GHMessen.hilfe.enableHelpKey(getRootPane(), "eigenschaften",
					GHMessen.hSet);
		JButton btnAbbrechen = new JButton(actionMap.get("abbrechen"));
		JButton btnOk = new JButton(actionMap.get("ok"));

		GroupLayout gl_knoepfe = new GroupLayout(knoepfe);
		gl_knoepfe.setHorizontalGroup(gl_knoepfe.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_knoepfe
						.createSequentialGroup()
						.addContainerGap()
						.addComponent(hilfe)
						.addPreferredGap(ComponentPlacement.RELATED, 565,
								Short.MAX_VALUE).addComponent(btnOk).addGap(18)
						.addComponent(btnAbbrechen).addContainerGap()));
		gl_knoepfe
				.setVerticalGroup(gl_knoepfe
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_knoepfe
										.createSequentialGroup()
										.addGroup(
												gl_knoepfe
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																hilfe,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																btnAbbrechen)
														.addComponent(btnOk))
										.addContainerGap()));
		knoepfe.setLayout(gl_knoepfe);
	}

	public void initialisieren() {
		if (DEBUG)
			System.out
					.println("[EigenschaftenBox] - initialisieren aufgerufen...");
		for (Component c : bereich.getComponents()) {
			if (c instanceof BereichInterface)
				((BereichInterface) c).initialisieren();
		}
	}

	public void schreiben() {
		if (DEBUG)
			System.out.println("[EigenschaftenBox] - schreiben aufgerufen...");
		for (Component c : bereich.getComponents()) {
			if (c instanceof BereichInterface)
				((BereichInterface) c).schreiben();
		}
	}

	/* Aktionen */
	@Action
	public void hilfe() {
		if (GHMessen.hilfe != null)
			GHMessen.hilfe.setDisplayed(true);
		GHMessen.hilfe.setCurrentID("eigenschaften");
	}

	@Action
	public void abbrechen() {
		dispose();
	}

	@Action
	public void ok() {
		CursorTk.startWaitCursor(this);
		schreiben();
		CursorTk.stopWaitCursor(this);
		dispose();
	}

}
