/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge;

import ghmessen.GHMessen;
import ghmessen.GHMessenView;
import ghmessen.adapter.AdapterListe;
import ghmessen.components.AdapterPanel;
import ghmessen.dialoge.einstellungen.EinstellungenAllgemeinPanel;
import ghmessen.dialoge.einstellungen.EinstellungenGroessePanel;
import ghmessen.dialoge.einstellungen.EinstellungenKommentarPanel;
import ghmessen.groesse.Darstellungsliste;
import ghmessen.groesse.Groessenliste;
import ghmessen.util.CursorTk;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.KeyStroke;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.observablecollections.ObservableList;
import org.jdesktop.observablecollections.ObservableListListener;

public class EinstellungenBox extends javax.swing.JDialog implements
		ObservableListListener {
	private static final long serialVersionUID = 2185885235762337449L;
	private static final boolean DEBUG = false;

	public static final int TABINTERFACE = 0, TABFORMEL = 1,
			TABMODELL = 2, TABKOMMENTAR = 3, TABALLGEMEIN = 4;
	private AdapterListe al;
	public Groessenliste abhaengigeGroessen;
	private Darstellungsliste ansicht;
	private GHMessenView mainView;
	private ActionMap actionMap;
	private ResourceMap resourceMap;

	private void writeObject(ObjectOutputStream out) throws IOException {
		throw new NotSerializableException(
				"Einstellungen-Box darf nicht gespeichert werden.");
	}

	private void readObject(ObjectInputStream in) throws IOException {
		throw new NotSerializableException(
				"Einstellungen-Box kann nicht wiederhergestellt werden.");
	}

	/** Creates new form EinstellungenBox */
	public EinstellungenBox(java.awt.Frame parent, AdapterListe al,
			Darstellungsliste ansicht, Groessenliste abhaengigeGroessen) {
		super(parent, true);
		mainView = GHMessen.getApplication().getMainView();
		CursorTk.startWaitCursor(this);
		setModal(false);
		this.abhaengigeGroessen = abhaengigeGroessen;
		this.abhaengigeGroessen.addObservableListListener(this);
		this.ansicht = ansicht;
		this.al = al;
		initComponents();
		setSize(740, 480);
		jtpEigenschaften.setSelectedIndex(0);
		CursorTk.stopWaitCursor(this);
		jtpEigenschaften.setSelectedIndex(TABINTERFACE);
		if (DEBUG)
			System.out.println("[EinstellungenBox] - init:");
	}

	private void initComponents() {
		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(ghmessen.dialoge.EinstellungenBox.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(EinstellungenBox.class);
		initKeyBindings();
		jtpEigenschaften = new javax.swing.JTabbedPane();
		jDE_Interface = new javax.swing.JPanel();
		jInterfaceliste = new AdapterPanel(al);
		ifLabel = new javax.swing.JLabel();
		ifLabel.setHorizontalTextPosition(SwingConstants.RIGHT);
		jDE_Formel = new EinstellungenGroessePanel(abhaengigeGroessen);
		jDE_Modell = new javax.swing.JPanel();
		jDE_Kommentar = new EinstellungenKommentarPanel();
		jDE_Allgemein = new EinstellungenAllgemeinPanel(al);
		jPanel1 = new javax.swing.JPanel();
		schliessen = new javax.swing.JButton(actionMap.get("schliessen"));
		parameter = new javax.swing.JButton(actionMap.get("showParameters"));
		parameter.setEnabled(false);
		beispiel = new javax.swing.JButton();
		beispiel.setText(resourceMap.getString("beispiel.text"));
		hilfe = new javax.swing.JButton(actionMap.get("hilfe"));
		hilfe.setEnabled(GHMessen.hilfe != null);
		if (GHMessen.hilfe != null && GHMessen.hSet != null)
			GHMessen.hilfe.enableHelpKey(getRootPane(), "einstellungen",
					GHMessen.hSet);
		setTitle(resourceMap.getString("Form.title"));
		setIconImage(null);

		jtpEigenschaften.setBorder(javax.swing.BorderFactory
				.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
		jtpEigenschaften
				.setTabLayoutPolicy(javax.swing.JTabbedPane.SCROLL_TAB_LAYOUT);

		ifLabel.setText(resourceMap.getString("ifLabel.text"));

		// Interface-Karte
		javax.swing.GroupLayout gl_jDE_Interface = new javax.swing.GroupLayout(
				jDE_Interface);
		gl_jDE_Interface
				.setHorizontalGroup(gl_jDE_Interface
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_jDE_Interface
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jDE_Interface
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																jInterfaceliste,
																GroupLayout.DEFAULT_SIZE,
																620,
																Short.MAX_VALUE)
														.addComponent(ifLabel))
										.addContainerGap()));
		gl_jDE_Interface.setVerticalGroup(gl_jDE_Interface.createParallelGroup(
				Alignment.LEADING).addGroup(
				Alignment.TRAILING,
				gl_jDE_Interface
						.createSequentialGroup()
						.addContainerGap()
						.addComponent(jInterfaceliste,
								GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE)
						.addContainerGap().addComponent(ifLabel)
						.addContainerGap()));
		jInterfaceliste.setLayout(new BoxLayout(jInterfaceliste,
				BoxLayout.LINE_AXIS));
		jDE_Interface.setLayout(gl_jDE_Interface);

		jtpEigenschaften
				.addTab(resourceMap
						.getString("jDE_Interface.TabConstraints.tabTitle"), jDE_Interface); //$NON-NLS-1$

		jtpEigenschaften
				.addTab(resourceMap
						.getString("jDE_Formel.TabConstraints.tabTitle"), jDE_Formel); //$NON-NLS-1$

		jDE_Modell.setEnabled(false);

		// Modell-Karte
		javax.swing.GroupLayout gl_jDE_Modell = new javax.swing.GroupLayout(
				jDE_Modell);
		jDE_Modell.setLayout(gl_jDE_Modell);
		gl_jDE_Modell.setHorizontalGroup(gl_jDE_Modell.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 600,
				Short.MAX_VALUE));
		gl_jDE_Modell.setVerticalGroup(gl_jDE_Modell.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 320,
				Short.MAX_VALUE));

		jtpEigenschaften
				.addTab(resourceMap
						.getString("jDE_Modell.TabConstraints.tabTitle"), jDE_Modell); //$NON-NLS-1$

		jtpEigenschaften
				.addTab(resourceMap
						.getString("jDE_Kommentar.TabConstraints.tabTitle"), jDE_Kommentar); //$NON-NLS-1$

		jtpEigenschaften
				.addTab(resourceMap
						.getString("jDE_Allgemein.TabConstraints.tabTitle"), jDE_Allgemein); // NOI18N //$NON-NLS-1$

		beispiel.setEnabled(false);

		javax.swing.GroupLayout gl_jPanel1 = new javax.swing.GroupLayout(
				jPanel1);
		jPanel1.setLayout(gl_jPanel1);
		gl_jPanel1
				.setHorizontalGroup(gl_jPanel1
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jPanel1
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(schliessen)
										.addGap(18, 18, 18)
										.addComponent(parameter)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												168, Short.MAX_VALUE)
										.addComponent(beispiel)
										.addGap(18, 18, 18).addComponent(hilfe)
										.addContainerGap()));
		gl_jPanel1
				.setVerticalGroup(gl_jPanel1
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								gl_jPanel1
										.createSequentialGroup()
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGroup(
												gl_jPanel1
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																schliessen)
														.addComponent(parameter)
														.addComponent(hilfe)
														.addComponent(beispiel))
										.addContainerGap()));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		layout.setHorizontalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.TRAILING)
												.addComponent(
														jPanel1,
														Alignment.LEADING,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.DEFAULT_SIZE,
														Short.MAX_VALUE)
												.addComponent(
														jtpEigenschaften,
														Alignment.LEADING,
														GroupLayout.DEFAULT_SIZE,
														654, Short.MAX_VALUE))
								.addContainerGap()));
		layout.setVerticalGroup(layout.createParallelGroup(Alignment.LEADING)
				.addGroup(
						Alignment.TRAILING,
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(jtpEigenschaften,
										GroupLayout.DEFAULT_SIZE, 350,
										Short.MAX_VALUE)
								.addPreferredGap(ComponentPlacement.RELATED)
								.addComponent(jPanel1,
										GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addContainerGap()));
		getContentPane().setLayout(layout);

		jtpEigenschaften.setEnabledAt(TABMODELL, false);
		jtpEigenschaften.setSelectedIndex(TABINTERFACE);
		pack();
		initKeyBindings();
		Application.getInstance().getContext().getResourceMap(getClass())
				.injectComponents(getContentPane());
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton schliessen;
	private javax.swing.JButton parameter;
	private javax.swing.JButton beispiel;
	private javax.swing.JButton hilfe;
	private AdapterPanel jInterfaceliste;
	private EinstellungenAllgemeinPanel jDE_Allgemein;
	private javax.swing.JPanel jDE_Interface;
	private EinstellungenGroessePanel jDE_Formel;
	private EinstellungenKommentarPanel jDE_Kommentar;
	private javax.swing.JPanel jDE_Modell;
	private javax.swing.JLabel ifLabel;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JTabbedPane jtpEigenschaften;

	// End of variables declaration//GEN-END:variables

	public void waehleKarteireiter(int reiter) {
		if (reiter < 0 || reiter > TABALLGEMEIN)
			return;
		jtpEigenschaften.setSelectedIndex(reiter);
	}

	public void setKommentar(String text) {
		if (jDE_Kommentar != null)
			jDE_Kommentar.setText(text);
	}

	public String getKommentar() {
		if (jDE_Kommentar != null)
			return jDE_Kommentar.getText();
		else
			return "";
	}

	@Action
	public void schliessen() {
		setVisible(false);
	}

	public AdapterPanel getCassyPanel() {
		return jInterfaceliste;
	}

	public void initKeyBindings() {
		InputMap inputMap = getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionMap = getRootPane().getActionMap();
		if(inputMap != null && actionMap != null) {
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0, false), "startStoppMessung");
			actionMap.put("startStoppMessung", startStoppMessung);
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0, false), "showParameters");
			actionMap.put("showParameters", showParameters);
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0, false), "showEinstellungenBox");
			actionMap.put("showEinstellungenBox", showEinstellungenBox);
		}

	}

	static javax.swing.Action startStoppMessung  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			JFrame f = GHMessen.getApplication().getMainFrame();
			f.dispatchEvent(new KeyEvent(f, KeyEvent.KEY_PRESSED, e.getWhen(), 
					0, KeyEvent.VK_F9, KeyEvent.CHAR_UNDEFINED));
			f.dispatchEvent(new KeyEvent(f, KeyEvent.KEY_RELEASED, e.getWhen(), 
					0, KeyEvent.VK_F9, KeyEvent.CHAR_UNDEFINED));
		}
		
	};
	
	static javax.swing.Action showEinstellungenBox  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GHMessenView mainView = GHMessen.getApplication().getMainView();
			mainView.toggleEinstellungenBox(null);
		}
		
	};
	
	static javax.swing.Action showParameters  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GHMessenView mainView = GHMessen.getApplication().getMainView();
			mainView.toggleParameters(null);
		}
		
	};
	
	@Action
	public void showParameters(javax.swing.Action a) {
		mainView.showParameters(a);
	}
	@Action
	public void hilfe() {
		if (GHMessen.hilfe != null)
			GHMessen.hilfe.setDisplayed(true);
		GHMessen.hilfe.setCurrentID("einstellungen");
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsAdded(ObservableList list, int index, int length) {
		parameter.setEnabled(true);
		if (ansicht.size() == 0)
			ansicht.neueDarstellung();
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsRemoved(ObservableList list, int index,
			List oldElements) {
		if (abhaengigeGroessen.getSize() < 1) {
			ansicht.clear();
			parameter.setEnabled(false);
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementReplaced(ObservableList list, int index,
			Object oldElement) {
		// nichts zu tun, da sich die Anzahl nicht geändert hat

	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementPropertyChanged(ObservableList list, int index) {
		// nichts zu tun, da sich die Anzahl nicht geändert hat

	}

}
