/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge;

import ghmessen.GHMessen;
import ghmessen.util.CursorTk;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import org.jdesktop.application.ResourceMap;

public class GHMessenAboutBox extends JDialog {
	private static final long serialVersionUID = 5695613061230783782L;
	private final JPanel contentPanel = new JPanel();
	private final Action schliessen = new SwingAction();
	private JLabel label;
	private JLabel lblghMessen;
	private ResourceMap resourceMap;
	private JLabel copyright;
	private JButton okButton;

	public static void main(String[] args) {
		try {
			GHMessenAboutBox dialog = new GHMessenAboutBox(null);
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public GHMessenAboutBox(Frame parent) {
		super(parent);
		CursorTk.startWaitCursor(this);
		initComponents();
		CursorTk.stopWaitCursor(this);
	}

	private void initComponents() {
		resourceMap = GHMessen.getInstance(ghmessen.GHMessen.class)
				.getContext().getResourceMap(GHMessenAboutBox.class);
		setBounds(100, 100, 700, 460);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		{
			label = new JLabel("");
			label.setIcon(new ImageIcon(GHMessenAboutBox.class
					.getResource("/ghmessen/resources/ghmessen_About.png")));
		}
		{
			lblghMessen = new JLabel(resourceMap.getString("Application.title"));
			lblghMessen.setFont(new Font("Dialog", Font.BOLD, 18));
		}

		JLabel lblVersion = new JLabel(
				resourceMap.getString("Application.version"));

		JLabel txtrCassylabIstEin = new JLabel();
		txtrCassylabIstEin.setFont(new Font("Dialog", Font.PLAIN, 12));
		txtrCassylabIstEin.setText("<html>"
				+ resourceMap.getString("aboutGHMessen")
				+ resourceMap.getString("usesGHMessen")
				+ resourceMap.getString("symboleText") + "</html>");
		txtrCassylabIstEin.setBackground(UIManager
				.getColor("Button.background"));
		GroupLayout gl_contentPanel = new GroupLayout(contentPanel);
		gl_contentPanel
				.setHorizontalGroup(gl_contentPanel
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_contentPanel
										.createSequentialGroup()
										.addComponent(label)
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_contentPanel
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_contentPanel
																		.createSequentialGroup()
																		.addComponent(
																				lblghMessen)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				lblVersion))
														.addComponent(
																txtrCassylabIstEin,
																GroupLayout.DEFAULT_SIZE,
																314,
																Short.MAX_VALUE))
										.addGap(31)));
		gl_contentPanel
				.setVerticalGroup(gl_contentPanel
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_contentPanel
										.createSequentialGroup()
										.addGap(0)
										.addGroup(
												gl_contentPanel
														.createParallelGroup(
																Alignment.TRAILING,
																false)
														.addComponent(
																label,
																Alignment.LEADING,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addGroup(
																Alignment.LEADING,
																gl_contentPanel
																		.createSequentialGroup()
																		.addGroup(
																				gl_contentPanel
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								lblghMessen)
																						.addComponent(
																								lblVersion))
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				txtrCassylabIstEin,
																				GroupLayout.PREFERRED_SIZE,
																				373,
																				GroupLayout.PREFERRED_SIZE)))
										.addGap(20)));
		contentPanel.setLayout(gl_contentPanel);
		{
			JPanel buttonPane = new JPanel();
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				okButton = new JButton("Schließen");
				okButton.setToolTipText("Das Dialogfenster schließen");
				okButton.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				{
					copyright = new JLabel("<html><p>"
							+ resourceMap.getString("Application.lizenz")
							+ "<BR>"
							+ resourceMap.getString("Application.copyright")
							+ "</p></html>");
				}
				okButton.setAction(schliessen);
				okButton.setActionCommand("OK");
				getRootPane().setDefaultButton(okButton);
			}
			GroupLayout gl_buttonPane = new GroupLayout(buttonPane);
			gl_buttonPane.setHorizontalGroup(gl_buttonPane.createParallelGroup(
					Alignment.LEADING).addGroup(
					Alignment.TRAILING,
					gl_buttonPane
							.createSequentialGroup()
							.addContainerGap()
							.addComponent(copyright, GroupLayout.DEFAULT_SIZE,
									383, Short.MAX_VALUE)
							.addPreferredGap(ComponentPlacement.UNRELATED)
							.addComponent(okButton).addContainerGap()));
			gl_buttonPane.setVerticalGroup(gl_buttonPane.createParallelGroup(
					Alignment.LEADING).addGroup(
					gl_buttonPane
							.createSequentialGroup()
							.addGap(5)
							.addGroup(
									gl_buttonPane
											.createParallelGroup(
													Alignment.BASELINE)
											.addComponent(copyright,
													GroupLayout.PREFERRED_SIZE,
													49,
													GroupLayout.PREFERRED_SIZE)
											.addComponent(okButton))));
			buttonPane.setLayout(gl_buttonPane);
		}
	}

	private class SwingAction extends AbstractAction {

		/**
		 * 
		 */
		private static final long serialVersionUID = 6834367161653788072L;

		public SwingAction() {
			putValue(ACCELERATOR_KEY,
					KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.ALT_MASK));
			putValue(MNEMONIC_KEY, KeyEvent.VK_ESCAPE);
			putValue(NAME, "Schließen");
			putValue(SHORT_DESCRIPTION, "Schließen des Fensters");
		}

		public void actionPerformed(ActionEvent e) {
			SwingUtilities.getWindowAncestor((Component) e.getSource())
					.dispose();
		}
	}
}
