/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge;

import static javax.swing.LayoutStyle.ComponentPlacement.RELATED;
import static javax.swing.LayoutStyle.ComponentPlacement.UNRELATED;

import java.awt.CardLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.NumberFormat;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.AbstractSpinnerModel;
import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.InputMap;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.KeyStroke;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.MutableComboBoxModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultFormatter;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;

import ghmessen.GHMessen;
import ghmessen.GHMessenView;
import ghmessen.Konstanten;
import ghmessen.Messparameter;
import ghmessen.adapter.AdapterListe;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.event.AdapterAddedEvent;
import ghmessen.adapter.event.AdapterChangedEvent;
import ghmessen.adapter.event.AdapterEventListener;
import ghmessen.adapter.event.AdapterRemovedEvent;
import ghmessen.components.FormelFormattedTextField;
import ghmessen.groesse.Einheit;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.WerteTabelle;
import ghmessen.util.Util;
import ghmessen.util.ZeitUtil;
import ld.api.ICASSY;
import ld.api.TriggerBits;
import ld.api.ios.CASSYIO;
import ld.api.ios.CASSYIOs;
import ld.api.ios.CASSYTimerQuantity;
import ld.api.ios.CASSYTimerTime;
import ld.api.ios.ICASSYQuantity;

public class MessparameterBox extends javax.swing.JDialog implements
		PropertyChangeListener, AdapterEventListener {
	private static final long serialVersionUID = 1814885794031351663L;
	private static final boolean DEBUG = false;
	private boolean firePC;
	protected Messparameter mp;
	protected Groessenliste gl;
	protected AdapterListe al;
	private boolean useTimer;
	private final String[] zeitIntervallStandard = new String[] { "10 μs",
			"20 μs", "50 μs", "100 μs", "200 μs", "500 μs", "1 ms", "2 ms",
			"5 ms", "10 ms", "20 ms", "50 ms", "100 ms", "200 ms", "500 ms",
			"1 s", "2 s", "5 s", "10 s", "15 s", "30 s", "1 min", "2 min",
			"5 min", "10 min", "15 min", "30 min", "1 h", "2 h", "3 h", "5 h",
			"6 h", "12 h", "24 h" };
	private final String[] zeitEinheit = Einheit.getZeitEinheitenStringArray();
	private final String[] osziValues = { "", "125", "250", "500", "1000",
			"2000", "4000", "8000" };
	private String[] timerValues = { "", "125", "250", "500", "1000", "1500",
			"2000", "2500" };
	private Vector<String> triggerOsziModel;

	/** Creates new form MessparameterBox */
	public MessparameterBox(java.awt.Frame parent, boolean modal,
			Messparameter mp, AdapterListe al, Groessenliste gl) {
		super(parent, modal);
		firePC = true;
		if (DEBUG)
			System.out.println("[MessparameterBox] - Konstruktor.");
		this.mp = mp;
		if (mp != null) {
			mp.setModus(Messparameter.MODUSCASSY);
			mp.addPropertyChangeListener(this);
		}
		this.al = al;
		if (al != null)
			al.getMonitor().addAdapterEventListener(this);
		this.gl = gl;
		useTimer = istTimerBenutzt();
		triggerOsziModel = new Vector<String>();
		firePC = false;
		initComponents();
		firePC = true;
		adjustComponents();
	}

	protected void initComponents() {
		ResourceMap resourceMap = Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(MessparameterBox.class);

		ActionMap actionMap = Application.getInstance().getContext()
				.getActionMap(MessparameterBox.class, this);

		initKeyBindings();
		
		setTitle("MessparameterBox"); // NOI18N

		/* Aufnahme */
		art = new javax.swing.ButtonGroup();
		automatischeAufnahme = new javax.swing.JRadioButton(
				actionMap.get("automatischeMessung"));
		automatischeAufnahme.setFont(resourceMap.getFont("Dialog-Plain-10")); // NOI18N
		art.add(automatischeAufnahme);

		manuelleAufnahme = new javax.swing.JRadioButton(
				actionMap.get("manuelleMessung"));
		manuelleAufnahme.setFont(resourceMap.getFont("Dialog 10 Normal")); // NOI18N
		art.add(manuelleAufnahme);

		signalEin = new javax.swing.JCheckBox(
				actionMap.get("akustischesSignal"));
		signalEin.setFont(resourceMap.getFont("Dialog-Plain-10")); // NOI18N

		if (mp.isEinzelEin())
			art.setSelected(manuelleAufnahme.getModel(), true);
		else
			art.setSelected(automatischeAufnahme.getModel(), true);

		JPanel pAufnahme = new JPanel();
		pAufnahme.setBorder(new TitledBorder(null, resourceMap
				.getString("aufnahme.title"), TitledBorder.LEADING,
				TitledBorder.TOP, null, null));

		GroupLayout gl_pAufnahme = new GroupLayout(pAufnahme);
		gl_pAufnahme.setHorizontalGroup(gl_pAufnahme
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_pAufnahme.createParallelGroup(Alignment.LEADING)
								.addComponent(automatischeAufnahme)
								.addComponent(manuelleAufnahme)
								.addComponent(signalEin)).addContainerGap());
		gl_pAufnahme.setVerticalGroup(gl_pAufnahme.createSequentialGroup()
				.addContainerGap().addComponent(automatischeAufnahme)
				.addPreferredGap(RELATED).addComponent(manuelleAufnahme)
				.addPreferredGap(UNRELATED).addComponent(signalEin)
				.addContainerGap());
		pAufnahme.setLayout(gl_pAufnahme);

		/* Messwerte */
		messwerte = new javax.swing.ButtonGroup();
		JRadioButton neueMessreihe = new JRadioButton(
				actionMap.get("neueMessreihe"));
		neueMessreihe.setFont(resourceMap.getFont("dialog.font"));
		messwerte.add(neueMessreihe);

		anhaengen = new JRadioButton(actionMap.get("anhaengen"));
		anhaengen.setFont(resourceMap.getFont("dialog.font"));
		anhaengen.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if (fortsetzen != null)
					fortsetzen.setEnabled(anhaengen.isSelected());
			}
		});

		messwerte.add(anhaengen);

		fortsetzen = new JCheckBox(actionMap.get("fortsetzen"));
		fortsetzen.setEnabled(false);
		fortsetzen.setFont(resourceMap.getFont("dialog.font"));

		ersetzen = new JRadioButton(actionMap.get("ersetzen"));
		ersetzen.setFont(resourceMap.getFont("dialog.font"));
		ersetzen.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if (wiederholungEin != null)
					wiederholungEin.setEnabled(ersetzen.isSelected());
			}
		});
		messwerte.add(ersetzen);
		if (mp != null) {
			if (!mp.isUeberschreibenEin())
				neueMessreihe.setSelected(true);
			else if (mp.isAnhaengenEin())
				anhaengen.setSelected(true);
			else
				ersetzen.setSelected(true);
			fortsetzen.setSelected(mp.isFortsetzenEin());
		}

		wiederholungEin = new javax.swing.JCheckBox(
				actionMap.get("wiederholungEin"));
		wiederholungEin.setFont(resourceMap.getFont("dialog.font"));
		if (mp != null)
			wiederholungEin.setSelected(mp.isWiederholungEin());

		JPanel pMesswerte = new JPanel();
		pMesswerte.setBorder(new TitledBorder(null, resourceMap
				.getString("messwerte.title"), TitledBorder.LEADING,
				TitledBorder.TOP, null, null));

		GroupLayout gl_pMesswerte = new GroupLayout(pMesswerte);
		gl_pMesswerte
				.setHorizontalGroup(gl_pMesswerte
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								gl_pMesswerte
										.createParallelGroup(Alignment.LEADING)
										.addComponent(neueMessreihe)
										.addComponent(anhaengen)
										.addComponent(fortsetzen)
										.addComponent(ersetzen)
										.addGroup(
												gl_pMesswerte
														.createSequentialGroup()
														.addContainerGap()
														.addComponent(
																wiederholungEin)))
						.addContainerGap());

		gl_pMesswerte.setVerticalGroup(gl_pMesswerte.createSequentialGroup()
				.addContainerGap().addComponent(neueMessreihe)
				.addPreferredGap(RELATED).addComponent(anhaengen)
				.addPreferredGap(RELATED).addComponent(fortsetzen)
				.addPreferredGap(RELATED).addComponent(ersetzen)
				.addPreferredGap(RELATED).addComponent(wiederholungEin)
				.addContainerGap());
		pMesswerte.setLayout(gl_pMesswerte);

		/* Zeiten */
		intervallLabel = new javax.swing.JLabel(
				resourceMap.getString("intervallLabel.text"));
		intervallLabel.setFont(resourceMap.getFont("dialog.font")); // NOI18N

		intervall = new javax.swing.JSpinner();
		intervall.setFont(resourceMap.getFont("dialog.font")); // NOI18N
		intervall.setModel(new javax.swing.SpinnerListModel(
				zeitIntervallStandard));
		intervall.setName("intervall"); // NOI18N
		if (mp != null) {
			String s = ZeitUtil.getZeitString(mp.getIntervall());
			if (Util.stringIndexOf(zeitIntervallStandard, s) < 0) {
				s = "100 ms";
				mp.setIntervall(0.1);
			}
			((javax.swing.SpinnerListModel) intervall.getModel()).setValue(s);
		}
		intervall.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if (firePC && mp != null) {
					firePC = false;
					mp.setManuell(true);
					String s = (String) intervall.getModel().getValue();
					mp.setIntervall(ZeitUtil.getZeitWert(s));
					if (DEBUG)
						System.out.println("[MessparameterBox] - intervall("
								+ s + ") Event(" + e + ")");
					if (mp.getWiederholung() != 0) {
						mp.setMesszeit(mp.getWiederholung()
								* mp.getIntervall());
						messZeit.setValue("s".equals(messEinheit.getSelectedItem())? 
								mp.getMesszeit() :
								mp.getMesszeit() / Einheit.toEinheit(
										(String)messEinheit.getSelectedItem()).getFaktor());
					}
					firePC = true;
				}
				CardLayout cl = (CardLayout) triggerPanel.getLayout();
				if (firePC && mp != null && mp.getIntervallMS() < 10)
					cl.show(triggerPanel, "2");
				else if (firePC)
					cl.show(triggerPanel, "1");
			}
		});
		intervallLabel.setLabelFor(intervall);

		anzahlLabel = new javax.swing.JLabel(
				resourceMap.getString("anzahlLabel.text"));
		anzahlLabel.setFont(resourceMap.getFont("dialog.font"));

		wiederholung = new javax.swing.JSpinner();
		Util.spinnerSetCommitsOnValidEdit(wiederholung, true);
		wiederholung.setFont(resourceMap.getFont("dialog.font")); // NOI18N
		if (mp != null) {
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					useTimer ? timerValues : osziValues));
			String s = Long.toString(mp.getWiederholung());
			if (Util.stringIndexOf(useTimer ? timerValues : osziValues, s) < 0)
				s = "250";
			wiederholung.getModel().setValue(s);
		}
		wiederholung.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if (firePC && mp != null) {
					try {
						firePC = false;
						mp.setManuell(true);
						String s = (String) wiederholung.getModel().getValue();
						if (s == null || "".equals(s)) {
							mp.setWiederholung(0);
							mp.setMesszeit(120.0);
						}
						else {
							mp.setWiederholung(Long.parseLong(s));
							mp.setMesszeit(mp.getWiederholung()
									* mp.getIntervall());
						}
						messZeit.setValue("s".equals(messEinheit.getSelectedItem())? 
								mp.getMesszeit() :
								mp.getMesszeit() / Einheit.toEinheit(
										(String)messEinheit.getSelectedItem()).getFaktor());
						if (DEBUG)
							System.out
									.println("[MessparameterBox] - wiederholung("
											+ s + ") evt(" + evt + ")");
					} catch (IllegalArgumentException e) {
						if (DEBUG)
							System.out
									.println("[MessparameterBox] - wiederholung.stateChanged("
											+ e.getLocalizedMessage() + ")");
					} finally {
						firePC = true;
					}
				}
			}
		});
		anzahlLabel.setLabelFor(wiederholung);

		messzeitLabel = new javax.swing.JLabel(
				resourceMap.getString("messzeitLabel.text"));
		messzeitLabel.setFont(resourceMap.getFont("dialog.font"));
		messZeit = new JFormattedTextField(NumberFormat.getNumberInstance());
		messZeit.setColumns(7);
		messZeit.setValue(mp == null ? 120.0 : ZeitUtil.getZeitWert(mp
				.getMesszeit()));
		messZeit.addPropertyChangeListener("value",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						if (firePC && mp != null) {
							try {
								mp.setManuell(true);
								Number value = (Number) messZeit.getValue();
								Einheit e = Einheit.s;
								if (messEinheit != null)
									e = Einheit.toEinheit((String) messEinheit
											.getModel().getSelectedItem());
								mp.setMesszeit(value.doubleValue()
										* e.getFaktor());
							} catch (NumberFormatException e) {
								if (DEBUG)
									System.out
											.println("[MessparameterBox] - messZeit: "
													+ e.getLocalizedMessage());
							} catch (IllegalArgumentException e) {
								if (DEBUG)
									System.out
											.println("[MessparameterBox] - messZeit: "
													+ e.getLocalizedMessage());
							}
						}
					}
				});
		messzeitLabel.setLabelFor(messZeit);
		messEinheit = new javax.swing.JComboBox<String>();
		messEinheit.setFont(resourceMap.getFont("dialog.font"));
		messEinheit.setModel(new javax.swing.DefaultComboBoxModel<String>(zeitEinheit));
		if (mp != null) {
			messEinheit.getModel().setSelectedItem(
					ZeitUtil.getZeitEinheit(mp.getMesszeit()).toString());
		} else
			messEinheit.getModel().setSelectedItem("s");
		messEinheit.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent evt) {
				if (firePC && mp != null
						&& evt.getStateChange() == ItemEvent.SELECTED) {
					try {
						//firePC = false;
						Einheit e = Einheit.toEinheit((String) evt.getItem());
						messZeit.setValue(mp.getMesszeit() / e.getFaktor());
						WerteTabelle gt = GHMessen.getApplication()
								.getMainView().gt;
						if (gt != null)
							gt.setNewMesszeitEinheit(e);
						//firePC = true;
					} catch (Exception e) {
						if (DEBUG)
							System.out
									.println("[MessparameterBox] - messEinheit: "
											+ e.getLocalizedMessage());
					}
				}
			}
		});

		JPanel pZeiten = new JPanel();
		pZeiten.setBorder(new TitledBorder(resourceMap
				.getString("zeiten.title")));
		GroupLayout gl_pZeiten = new GroupLayout(pZeiten);
		gl_pZeiten.setHorizontalGroup(gl_pZeiten
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_pZeiten.createParallelGroup(Alignment.LEADING)
								.addComponent(intervallLabel)
								.addComponent(anzahlLabel)
								.addComponent(messzeitLabel))
				.addPreferredGap(RELATED)
				.addGroup(
						gl_pZeiten
								.createParallelGroup(Alignment.LEADING)
								.addComponent(intervall)
								.addComponent(wiederholung)
								.addGroup(
										gl_pZeiten.createSequentialGroup()
												.addComponent(messZeit)
												.addPreferredGap(RELATED)
												.addComponent(messEinheit)))
				.addContainerGap());

		gl_pZeiten.setVerticalGroup(gl_pZeiten
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_pZeiten.createParallelGroup(Alignment.BASELINE)
								.addComponent(intervallLabel)
								.addComponent(intervall))
				.addPreferredGap(RELATED)
				.addGroup(
						gl_pZeiten.createParallelGroup(Alignment.BASELINE)
								.addComponent(anzahlLabel)
								.addComponent(wiederholung))
				.addPreferredGap(UNRELATED)
				.addGroup(
						gl_pZeiten.createParallelGroup(Alignment.BASELINE)
								.addComponent(messzeitLabel)
								.addComponent(messZeit)
								.addComponent(messEinheit)).addContainerGap());
		pZeiten.setLayout(gl_pZeiten);

		/* Knöpfe */
		schliessen = new javax.swing.JButton(actionMap.get("schliesseFenster"));
		schliessen.setFont(resourceMap.getFont("Dialog-Plain-10")); // NOI18N

		hilfe = new javax.swing.JButton(actionMap.get("hilfe"));
		hilfe.setFont(resourceMap.getFont("Dialog-Plain-10")); // NOI18N
		hilfe.setEnabled(GHMessen.hilfe != null);
		if (GHMessen.hilfe != null && GHMessen.hSet != null)
			GHMessen.hilfe.enableHelpKey(getRootPane(), "messparameter",
					GHMessen.hSet);

		JPanel pKnoepfe = new JPanel();
		GroupLayout gl_pKnoepfe = new GroupLayout(pKnoepfe);
		gl_pKnoepfe.setHorizontalGroup(gl_pKnoepfe
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_pKnoepfe.createParallelGroup(Alignment.CENTER)
								.addComponent(schliessen).addComponent(hilfe))
				.addContainerGap());
		gl_pKnoepfe
				.setVerticalGroup(gl_pKnoepfe.createSequentialGroup()
						.addContainerGap().addComponent(schliessen)
						.addPreferredGap(RELATED).addComponent(hilfe)
						.addContainerGap());
		pKnoepfe.setLayout(gl_pKnoepfe);

		/* Bedingungen */
		triggerEin = new javax.swing.JCheckBox(actionMap.get("triggerEin"));
		triggerEin.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				boolean selected = triggerEin.isSelected();
				triggerTerm.setEnabled(selected);
				triggerOsziSymbol.setEnabled(selected);
				triggerOsziFlanke.setEnabled(selected);
				triggerOsziWert.setEnabled(selected);
				triggerOsziEinheit.setEnabled(selected);
			}
		});
		triggerEin.setFont(resourceMap.getFont("dialog.font"));
		triggerEin.setSelected(mp == null ? false : mp.isTriggerEin());

		DefaultFormatter formelFormatter = new DefaultFormatter();
		formelFormatter.setOverwriteMode(false);
		triggerTerm = new FormelFormattedTextField(formelFormatter);
		triggerTerm.setColumns(25);
		triggerTerm.setValue(mp == null ? "" : mp.getTriggerTerm());
		triggerTerm.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (firePC && "value".equals(e.getPropertyName())) {
					mp.setTriggerTerm((String) e.getNewValue());
					mp.setTriggerEE(triggerTerm.getEvaluator());
				}
			}
		});
		triggerTerm.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				if (gl == null)
					triggerTerm.setKonstanten(new String[] {});
				else
					triggerTerm.setKonstanten(gl.getSymbole());
			}
		});
		triggerTerm.setFont(resourceMap.getFont("dialog.font"));

		messBedingungEin = new javax.swing.JCheckBox(
				actionMap.get("messBedingungEin"));
		messBedingungEin.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				messBedingungTerm.setEnabled(messBedingungEin.isSelected());
			}
		});
		messBedingungEin.setFont(resourceMap.getFont("dialog.font"));
		messBedingungEin.setSelected(mp == null ? false : mp.isBedingungEin());

		DefaultFormatter messBedFormatter = new DefaultFormatter();
		messBedFormatter.setOverwriteMode(false);
		messBedingungTerm = new FormelFormattedTextField(messBedFormatter, mp.getBedingungTerm());

		messBedingungTerm.setFont(resourceMap.getFont("Dialog 10 Normal")); // NOI18N
		messBedingungTerm.setName("Messbedingung");
		messBedingungTerm
				.addPropertyChangeListener(new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent e) {
						if (firePC && "value".equals(e.getPropertyName())) {
							mp.setBedingungTerm((String) e.getNewValue());
							mp.setBedingungEE(messBedingungTerm.getEvaluator());
						}
					}
				});
		messBedingungTerm.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				if (gl == null)
					messBedingungTerm.setKonstanten(new String[] {});
				else
					messBedingungTerm.setKonstanten(gl.getSymbole());
			}
		});

		stoppBedingungEin = new JCheckBox(actionMap.get("stoppBedingungEin"));
		stoppBedingungEin.setFont(resourceMap.getFont("dialog.font"));
		stoppBedingungEin.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				stoppBedingungTerm.setEnabled(stoppBedingungEin.isSelected());
			}
		});
		stoppBedingungEin.setSelected(mp == null ? false : mp.isStoppEin());

		DefaultFormatter stoppBedFormatter = new DefaultFormatter();
		stoppBedFormatter.setOverwriteMode(false);
		stoppBedingungTerm = new FormelFormattedTextField(stoppBedFormatter, mp.getStoppTerm());
		stoppBedingungTerm.setFont(resourceMap.getFont("dialog.font"));
		stoppBedingungTerm
				.addPropertyChangeListener(new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent e) {
						if (firePC && "value".equals(e.getPropertyName())) {
							mp.setStoppTerm((String) e.getNewValue());
							mp.setStoppEE(stoppBedingungTerm.getEvaluator());
						}
					}
				});
		stoppBedingungTerm.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				if (gl == null)
					stoppBedingungTerm.setKonstanten(new String[] {});
				else
					stoppBedingungTerm.setKonstanten(gl.getSymbole());
			}
		});

		JPanel triggerNormal = new JPanel();
		GroupLayout gl_triggerNormal = new GroupLayout(triggerNormal);
		gl_triggerNormal.setHorizontalGroup(gl_triggerNormal
				.createParallelGroup(Alignment.LEADING).addComponent(
						triggerTerm));
		gl_triggerNormal.setVerticalGroup(gl_triggerNormal.createParallelGroup(
				Alignment.BASELINE).addComponent(triggerTerm));
		triggerNormal.setLayout(gl_triggerNormal);

		JPanel triggerOszi = new JPanel();
		Groesse g = null;
		if (mp != null && mp.getTriggerSymbol() != null
				&& !"".equals(mp.getTriggerSymbol()))
			g = Util.findeGroesse(gl, mp.getTriggerSymbol());
		triggerOsziSymbol = new JComboBox<String>(new DefaultComboBoxModel<String>(
				triggerOsziModel));
		triggerOsziSymbol.setFont(resourceMap.getFont("dialog.font"));
		triggerOsziSymbol.addFocusListener(new FocusAdapter() {
			@Override
			public void focusGained(FocusEvent e) {
				if (gl == null)
					return;
				String[] namen = gl.getOszilloskopSymbole();
				MutableComboBoxModel<String> mcbm = (MutableComboBoxModel<String>) triggerOsziSymbol
						.getModel();
				if (DEBUG)
					System.out
							.println("[MessparameterBox] - triggerOsziSymbol.focusGained: alt("
									+ triggerOsziModel + ") neu(" + namen + ")");
				int alt = triggerOsziModel.size();
				int neu = namen.length;
				int set = Math.min(alt, neu);
				for (int i = 0; i < set; i++) {
					if (!namen[i].equals(triggerOsziModel.get(i))) {
						mcbm.removeElementAt(i);
						mcbm.insertElementAt(namen[i], i);
					}
				}
				for (int i = neu; i < alt; i++)
					mcbm.removeElementAt(neu);
				for (int i = alt; i < neu; i++)
					mcbm.addElement(namen[i]);
			}
		});
		if (g != null)
			triggerOsziSymbol.setSelectedItem(g.getSymbol());
		triggerOsziSymbol.setAction(actionMap.get("triggerOsziSymbol"));

		triggerOsziFlanke = new JComboBox<String>(new DefaultComboBoxModel<String>(
				new String[] { resourceMap.getString("steigend.text"),
						resourceMap.getString("fallend.text") }));
		triggerOsziFlanke.setFont(resourceMap.getFont("dialog.font"));
		triggerOsziFlanke.setSelectedIndex(mp == null ? 1 : (mp
				.getTriggerFlanke() == TriggerBits.None ? 1 : 0));
		triggerOsziFlanke.setAction(actionMap.get("triggerOsziFlanke"));

		triggerOsziWert = new JFormattedTextField(mp == null ? 0.0
				: mp.getTriggerWert());
		triggerOsziWert.setFont(resourceMap.getFont("dialog.font"));
		triggerOsziWert.setAction(actionMap.get("triggerOsziWert"));

		triggerOsziEinheit = new JLabel(g == null ? "V" : g.getEinheitString());
		triggerOsziEinheit.setFont(resourceMap.getFont("dialog.font"));

		GroupLayout gl_triggerOszi = new GroupLayout(triggerOszi);
		gl_triggerOszi.setHorizontalGroup(gl_triggerOszi
				.createSequentialGroup().addComponent(triggerOsziSymbol)
				.addPreferredGap(RELATED).addComponent(triggerOsziFlanke)
				.addPreferredGap(RELATED).addComponent(triggerOsziWert)
				.addComponent(triggerOsziEinheit));
		gl_triggerOszi.setVerticalGroup(gl_triggerOszi
				.createParallelGroup(Alignment.BASELINE)
				.addComponent(triggerOsziSymbol)
				.addComponent(triggerOsziFlanke).addComponent(triggerOsziWert)
				.addComponent(triggerOsziEinheit));
		triggerOszi.setLayout(gl_triggerOszi);

		triggerPanel = new JPanel(new CardLayout());
		triggerPanel.add(triggerNormal, "1");
		triggerPanel.add(triggerOszi, "2");

		if (mp != null) {
			CardLayout cl = (CardLayout) triggerPanel.getLayout();
			if (mp.getIntervallMS() < 10)
				cl.show(triggerPanel, "2");
			else
				cl.show(triggerPanel, "1");
		}

		JPanel pBedingungen = new JPanel();
		pBedingungen.setBorder(new TitledBorder(null, resourceMap
				.getString("bedingungen.title"), TitledBorder.LEADING,
				TitledBorder.TOP, null, null));

		GroupLayout gl_pBedingungen = new GroupLayout(pBedingungen);
		gl_pBedingungen.setHorizontalGroup(gl_pBedingungen
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_pBedingungen.createParallelGroup(Alignment.LEADING)
								.addComponent(triggerEin)
								.addComponent(messBedingungEin)
								.addComponent(stoppBedingungEin))
				.addPreferredGap(RELATED)
				.addGroup(
						gl_pBedingungen.createParallelGroup(Alignment.LEADING)
								.addComponent(triggerPanel)
								.addComponent(messBedingungTerm)
								.addComponent(stoppBedingungTerm))
				.addContainerGap());
		gl_pBedingungen.setVerticalGroup(gl_pBedingungen
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						gl_pBedingungen.createParallelGroup(Alignment.BASELINE)
								.addComponent(triggerEin)
								.addComponent(triggerPanel))
				.addPreferredGap(RELATED)
				.addGroup(
						gl_pBedingungen.createParallelGroup(Alignment.BASELINE)
								.addComponent(messBedingungEin)
								.addComponent(messBedingungTerm))
				.addPreferredGap(RELATED)
				.addGroup(
						gl_pBedingungen.createParallelGroup(Alignment.BASELINE)
								.addComponent(stoppBedingungEin)
								.addComponent(stoppBedingungTerm))
				.addContainerGap());

		pBedingungen.setLayout(gl_pBedingungen);
		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		layout.setHorizontalGroup(
			layout.createParallelGroup(Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
					.addContainerGap()
					.addGroup(layout.createParallelGroup(Alignment.LEADING)
						.addComponent(pAufnahme, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addComponent(pMesswerte, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(layout.createParallelGroup(Alignment.LEADING)
						.addGroup(layout.createSequentialGroup()
							.addComponent(pZeiten, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
							.addPreferredGap(ComponentPlacement.RELATED)
							.addComponent(pKnoepfe, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
						.addComponent(pBedingungen, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addContainerGap())
		);
		layout.setVerticalGroup(
			layout.createParallelGroup(Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
					.addContainerGap()
					.addGroup(layout.createParallelGroup(Alignment.TRAILING)
						.addComponent(pMesswerte, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addGroup(layout.createSequentialGroup()
							.addGroup(layout.createParallelGroup(Alignment.LEADING)
								.addComponent(pAufnahme, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addComponent(pZeiten, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addComponent(pKnoepfe, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
							.addGap(18)
							.addComponent(pBedingungen, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)))
					.addContainerGap())
		);
		getContentPane().setLayout(layout);

		pack();
	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.ButtonGroup art;
	private javax.swing.ButtonGroup messwerte;
	private javax.swing.JButton schliessen;
	private javax.swing.JButton hilfe;
	private javax.swing.JCheckBox wiederholungEin;
	private javax.swing.JCheckBox signalEin;
	private javax.swing.JComboBox<String> messEinheit;
	private javax.swing.JLabel intervallLabel;
	private javax.swing.JLabel anzahlLabel;
	private javax.swing.JLabel messzeitLabel;
	private javax.swing.JRadioButton automatischeAufnahme;
	private javax.swing.JRadioButton manuelleAufnahme;
	private javax.swing.JSpinner intervall;
	private javax.swing.JSpinner wiederholung;
	private JFormattedTextField messZeit;
	private javax.swing.JCheckBox triggerEin;
	/* Panel mit CardLayout */
	private JPanel triggerPanel;
	/* Card - Layout 1. Karte */
	private FormelFormattedTextField triggerTerm;
	/* Card - Layout 2. Karte */
	private JComboBox<String> triggerOsziSymbol;
	private JComboBox<String> triggerOsziFlanke;
	private JFormattedTextField triggerOsziWert;
	private JLabel triggerOsziEinheit;
	private javax.swing.JCheckBox messBedingungEin;
	private FormelFormattedTextField messBedingungTerm;
	private JCheckBox stoppBedingungEin;
	private FormelFormattedTextField stoppBedingungTerm;
	private JRadioButton ersetzen;
	private JRadioButton anhaengen;
	private JCheckBox fortsetzen;

	@Action
	public void hilfe() {
		if (GHMessen.hilfe != null)
			GHMessen.hilfe.setDisplayed(true);
		GHMessen.hilfe.setCurrentID("messparameter");
	}

	@Action
	public void manuelleMessung() {
		if (firePC && mp != null)
			mp.setEinzelEin(true);
	}

	@Action
	public void automatischeMessung() {
		if (firePC && mp != null)
			mp.setEinzelEin(false);
	}

	@Action
	public void akustischesSignal() {
		if (firePC && mp != null)
			mp.setSignalEin(signalEin.isSelected());
	}

	@Action
	public void anhaengen() {
		if (firePC && mp != null) {
			mp.setAnhaengenEin(true);
			mp.setUeberschreibenEin(false);
		}
	}

	@Action
	public void fortsetzen() {
		if (firePC && mp != null) {
			mp.setFortsetzenEin(fortsetzen.isSelected());
		}
	}

	@Action
	public void ersetzen() {
		if (firePC && mp != null) {
			mp.setAnhaengenEin(false);
			mp.setUeberschreibenEin(true);
		}
	}

	@Action
	public void neueMessreihe() {
		if (firePC && mp != null) {
			mp.setAnhaengenEin(false);
			mp.setUeberschreibenEin(false);
		}
	}

	@Action
	public void wiederholungEin() {
		if (firePC && mp != null)
			mp.setWiederholungEin(wiederholungEin.isSelected());
	}

	@Action
	public void triggerEin() {
		if (firePC && mp != null)
			mp.setTriggerEin(triggerEin.isSelected());
	}

	@Action
	public void messBedingungEin() {
		if (firePC && mp != null)
			mp.setBedingungEin(messBedingungEin.isSelected());
	}

	@Action
	public void stoppBedingungEin() {
		if (firePC && mp != null)
			mp.setStoppEin(stoppBedingungEin.isSelected());
	}

	@Action
	public void triggerOsziSymbol() {
		if (!firePC || gl == null)
			return;
		Groesse g = Util.findeGroesse(gl,
				(String) triggerOsziSymbol.getSelectedItem());
		if (g != null && mp != null) {
			mp.setTriggerSymbol(g.getSymbol());
			triggerOsziEinheit.setText(g.getEinheitString());
		}
	}

	@Action
	public void triggerOsziFlanke() {
		if (firePC && mp != null)
			mp.setTriggerFlanke(triggerOsziFlanke.getSelectedIndex() == 0 ? TriggerBits.Rising
					: TriggerBits.None);
	}

	@Action
	public void triggerOsziWert() {
		if (firePC && mp != null)
			mp.setTriggerWert(((Number) triggerOsziWert.getValue())
					.doubleValue());
	}

	@Action
	public void schliesseFenster() {
		WindowEvent wev = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
		Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
	}

	protected void adjustComponents() {
		boolean aktiv = mp != null ? automatischeAufnahme.isSelected()
				&& !mp.isTimerBox() : automatischeAufnahme.isSelected();
		signalEin.setEnabled(aktiv);
		messEinheit.setEnabled(automatischeAufnahme.isSelected());
		intervallLabel.setEnabled(aktiv);
		intervall.setEnabled(aktiv);
		anzahlLabel.setEnabled(aktiv);
		wiederholung.setEnabled(aktiv);
		messzeitLabel.setEnabled(automatischeAufnahme.isSelected());
		messZeit.setEnabled(automatischeAufnahme.isSelected());
		triggerEin.setEnabled(aktiv);
		messBedingungEin.setEnabled(aktiv);
		stoppBedingungEin.setEnabled(aktiv);
	}

	protected boolean istTimerBenutzt() {
		if (al == null)
			return false;
		for (IAdapter a : al.values()) {
			if (!(a instanceof ICASSY))
				continue;
			CASSYIOs cassyios = ((ICASSY) a).getCASSYs().getCASSYIOs();
			for (CASSYIO io : cassyios)
				for (ICASSYQuantity q : io.getSelectedQuantities())
					if (q instanceof CASSYTimerQuantity
							|| q instanceof CASSYTimerTime)
						return true;
		}
		return false;
	}

	@SuppressWarnings("serial")
	private class StepNumberSpinnerModel extends AbstractSpinnerModel {
		private Object[] values;

		private Object value;
		private boolean mitNull = false;

		public StepNumberSpinnerModel(boolean mitNull, Object[] values) {
			super();
			this.values = values != null && values.length > 1
					&& values[0].equals("") ? values
					: new Object[] { "", "125" };
			this.mitNull = mitNull;
			this.value = values[mitNull ? 0 : 1];
		}

		public void setMitNull(boolean mitNull) {
			this.mitNull = mitNull;
			if (value.equals("") && !mitNull)
				value = values[1];
		}

		@Override
		public Object getValue() {
			return value;
		}

		@Override
		public void setValue(Object value) {
			int min = mitNull ? 0 : 1;
			for (int i = min; i < values.length; i++)
				if (value.toString().equals(values[i])) {
					this.value = values[i];
					break;
				}
			fireStateChanged();
		}

		@Override
		public Object getNextValue() {
			int min = mitNull ? 0 : 1;
			for (int i = min; i < values.length - 1; i++)
				if (values[i] == value)
					return values[i + 1];
			return null;
		}

		@Override
		public Object getPreviousValue() {
			int min = mitNull ? 0 : 1;
			for (int i = values.length - 1; i >= min + 1; i--)
				if (value == values[i])
					return values[i - 1];
			return null;
		}

	}

	@Override
	public void adapterAdded(AdapterAddedEvent evt) {
		if (mp == null)
			return;
		useTimer = istTimerBenutzt();
		if (useTimer) {
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					timerValues));
		} else {
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					osziValues));
		}
	}

	@Override
	public void adapterRemoved(AdapterRemovedEvent evt) {
		if (mp == null)
			return;
		useTimer = istTimerBenutzt();
		wiederholung.setModel(new StepNumberSpinnerModel(
				mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
				osziValues));
		if (useTimer) {
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					timerValues));
		} else {
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					osziValues));
		}
		mp.reset();
	}

	@Override
	public void adapterChanged(AdapterChangedEvent evt) {
		if (mp == null || evt.neu == null)
			return;
		if (useTimer && evt.neu.hatTimer()
				|| (!useTimer && !evt.neu.hatTimer()))
			return;
		if (!useTimer && evt.neu.hatTimer()) {
			useTimer = true;
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					timerValues));
		} else if (!istTimerBenutzt()) {
			useTimer = false;
			wiederholung.setModel(new StepNumberSpinnerModel(
					mp.getIntervall() >= Konstanten.MIN_ECHTZEIT_INTERVALL,
					osziValues));
			mp.reset();
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (firePC)
			try {
				firePC = false;
				Object neu = evt.getNewValue();
				switch(evt.getPropertyName()) {
				case "einzelEin":
				case "timerBox":
					adjustComponents();
					break;
				case "intervall":
					double intervall = (Double) evt.getNewValue();
					if (wiederholung.getModel() instanceof StepNumberSpinnerModel)
						((StepNumberSpinnerModel) wiederholung.getModel())
								.setMitNull(intervall >= Konstanten.MIN_ECHTZEIT_INTERVALL);
					break;
				case "triggerTerm":
						if(neu != null && !neu.equals(triggerTerm.getValue()))
							triggerTerm.setValue(neu);
						break;
				case "bedingungTerm":
						if(neu != null && !neu.equals(messBedingungTerm.getValue()))
							messBedingungTerm.setValue(neu);
						break;
				case "stoppTerm":
						if( neu != null	&& !neu.equals(stoppBedingungTerm.getValue()))
							stoppBedingungTerm.setValue(neu);
						break;
				case "triggerSymbol":
						if(neu != null && !neu.equals(triggerOsziSymbol))
							triggerOsziSymbol.setSelectedItem(neu);
						break;
				}
			} finally {
				firePC = true;
			}
	}

	protected void initKeyBindings() {
		InputMap inputMap = getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionMap = getRootPane().getActionMap();
		if(inputMap != null && actionMap != null) {
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0, false), "startStoppMessung");
			actionMap.put("startStoppMessung", startStoppMessung);
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0, false), "showParameters");
			actionMap.put("showParameters", showParameters);
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0, false), "showEinstellungenBox");
			actionMap.put("showEinstellungenBox", showEinstellungenBox);
		}

	}

	static javax.swing.Action startStoppMessung  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			JFrame f = GHMessen.getApplication().getMainFrame();
			f.dispatchEvent(new KeyEvent(f, KeyEvent.KEY_PRESSED, e.getWhen(), 
					0, KeyEvent.VK_F9, KeyEvent.CHAR_UNDEFINED));
			f.dispatchEvent(new KeyEvent(f, KeyEvent.KEY_RELEASED, e.getWhen(), 
					0, KeyEvent.VK_F9, KeyEvent.CHAR_UNDEFINED));
		}
		
	};
	
	static javax.swing.Action showEinstellungenBox  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GHMessenView mainView = GHMessen.getApplication().getMainView();
			mainView.toggleEinstellungenBox(null);
		}
		
	};
	
	static javax.swing.Action showParameters  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GHMessenView mainView = GHMessen.getApplication().getMainView();
			mainView.toggleParameters(null);
		}
		
	};
	
}
