/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge;

import ghmessen.GHMessen;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.cassy.JustiereXRay;
import ghmessen.util.Util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JProgressBar;
import javax.swing.LayoutStyle.ComponentPlacement;

import ld.api.ICASSY;
import ld.api.IXRay;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;

public class XRayKristallEinmessen extends javax.swing.JDialog {

	private static final long serialVersionUID = -7580927517585877141L;
	private static final boolean DEBUG = false;

	public static final int KRISTALL_NACL = 0;
	public static final int KRISTALL_LIF = 1;
	public static final int KRISTALL_KBR = 2;

	public static final int ANODE_FE = 0;
	public static final int ANODE_CU = 1;
	public static final int ANODE_MO = 2;
	public static final int ANODE_AG = 3;
	public static final int ANODE_W = 4;

	private static final double[][] MAXIMUMTARGET = // Kristall, Anode
	{ { 20.1, 15.9, 7.2, 5.7, 15.2 }, { 28.8, 22.5, 10.2, 8.0, 21.5 },
			{ 17.1, 13.5, 6.2, 4.9, 12.9 }, };
	private ActionMap actionMap;
	private ResourceMap resourceMap;
	private ScheduledFuture<?> fortschritt;
	private boolean statusMessung;
	private IXRay xray;
	private JustiereXRay justieren;

	/** Creates new form XRayKristallEinmessen */
	public XRayKristallEinmessen(java.awt.Frame parent, IXRay xray,
			boolean modal) {
		super(parent, modal);
		justieren = new JustiereXRay(xray);
		setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
		setModal(true);
		setAlwaysOnTop(true);
		fortschritt = null;
		this.xray = xray;
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentHidden(ComponentEvent e) {
				if (justieren != null
						&& justieren.getStatus() == JustiereXRay.Status.LAEUFT)
					justieren.justierungStop();
				beendeJustierung();
			}
		});
		initComponents();
	}

	private void initComponents() {

		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(XRayKristallEinmessen.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(XRayKristallEinmessen.class);
		starteSuche = new javax.swing.JButton();
		starteSuche.setAction(actionMap.get("suchen"));
		abbrechen = new javax.swing.JButton(actionMap.get("schliessen"));
		jPanel1 = new javax.swing.JPanel();
		nullGrad = new javax.swing.JCheckBox();
		jLabel1 = new javax.swing.JLabel();
		jLabel2 = new javax.swing.JLabel();
		jLabel3 = new javax.swing.JLabel();
		kristall = new javax.swing.JComboBox<String>();
		anode = new javax.swing.JComboBox<String>();
		messWinkel = new javax.swing.JFormattedTextField(Util.doubleFormat);
		jPanel2 = new javax.swing.JPanel();
		jLabel4 = new javax.swing.JLabel();
		jLabel5 = new javax.swing.JLabel();
		jLabel6 = new javax.swing.JLabel();
		targetWinkel = new javax.swing.JFormattedTextField(Util.doubleFormat);
		targetWinkel.setEnabled(false);
		targetWinkel.setEditable(false);
		sensorWinkel = new javax.swing.JFormattedTextField(Util.doubleFormat);
		sensorWinkel.setEditable(false);
		sensorWinkel.setEnabled(false);
		rate = new javax.swing.JFormattedTextField(Util.doubleFormat);
		rate.setEditable(false);
		rate.setEnabled(false);
		jScrollPane1 = new javax.swing.JScrollPane();
		jTextArea1 = new javax.swing.JTextArea();

		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		starteSuche.setFont(resourceMap.getFont("standard.font"));

		abbrechen.setFont(resourceMap.getFont("standard.font"));

		jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				"Kristall und Anode",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				resourceMap.getFont("standard.font"))); // NOI18N

		nullGrad.setFont(resourceMap.getFont("standard.font"));
		nullGrad.setText(resourceMap.getString("nullGrad.text"));
		nullGrad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				boolean markiert = nullGrad.isSelected();
				if (justieren != null)
					justieren.setModus(markiert ? JustiereXRay.Modus.NULLSUCHE
							: JustiereXRay.Modus.JUSTIERUNG);
				anode.setEnabled(!markiert);
				kristall.setEnabled(!markiert);
				messWinkel.setEnabled(!markiert);
			}
		});
		jLabel1.setFont(resourceMap.getFont("standard.font"));
		jLabel1.setText(resourceMap.getString("kristall.text"));

		jLabel2.setFont(resourceMap.getFont("standard.font"));
		jLabel2.setText(resourceMap.getString("anode.text"));

		jLabel3.setFont(resourceMap.getFont("standard.font"));
		jLabel3.setText(resourceMap.getString("targetwinkel.text"));

		kristall.setFont(resourceMap.getFont("standard.font"));
		kristall.setModel(new javax.swing.DefaultComboBoxModel<String>(
				new String[] { "NaCl (2d = 564,0 pm)", "LiF (2d = 402,7 pm)",
						"KBr (2d = 659,7 pm)" }));
		kristall.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (kristall.getSelectedIndex() > -1 && anode != null
						&& anode.getSelectedIndex() > -1) {
					messWinkel.setText(String.format("%3.1f°",
							MAXIMUMTARGET[kristall.getSelectedIndex()][anode
									.getSelectedIndex()]));
					if (justieren != null)
						try {
							if (justieren.getStatus() == JustiereXRay.Status.LAEUFT)
								throw new EAdapterException(
										"XRay adjustment angle error in xray "
												+ ((ICASSY) xray).getIndex());
							justieren
									.setTargetZielWinkel(MAXIMUMTARGET[kristall
											.getSelectedIndex()][anode
											.getSelectedIndex()]);
						} catch (EAdapterException e1) {
							if (DEBUG)
								e1.printStackTrace();
						}
				}
			}
		});
		kristall.setSelectedIndex(KRISTALL_NACL);

		anode.setFont(resourceMap.getFont("standard.font"));
		anode.setModel(new javax.swing.DefaultComboBoxModel<String>(
				new String[] { "Fe (Kα = 6,40 keV)", "Cu (Kα = 8,04 keV)",
						"Mo (Kα = 17,44 keV)", "Ag (Kα = 22,1 keV)",
						"W (Lα = 8,39 keV)" }));
		anode.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (anode.getSelectedIndex() > -1 && kristall != null
						&& kristall.getSelectedIndex() > -1) {
					messWinkel.setText(String.format("%3.1f°",
							MAXIMUMTARGET[kristall.getSelectedIndex()][anode
									.getSelectedIndex()]));
					if (justieren != null)
						try {
							if (justieren.getStatus() == JustiereXRay.Status.LAEUFT)
								throw new EAdapterException(
										"XRay adjustment angle error in xray "
												+ ((ICASSY) xray).getIndex());
							justieren
									.setTargetZielWinkel(MAXIMUMTARGET[kristall
											.getSelectedIndex()][anode
											.getSelectedIndex()]);
						} catch (EAdapterException e1) {
							if (DEBUG)
								e1.printStackTrace();
						}
				}
			}
		});
		anode.setSelectedIndex(ANODE_MO);

		messWinkel.setBackground(new java.awt.Color(238, 238, 238));
		messWinkel.setBorder(null);

		javax.swing.GroupLayout gl_jPanel1 = new javax.swing.GroupLayout(
				jPanel1);
		jPanel1.setLayout(gl_jPanel1);
		gl_jPanel1
				.setHorizontalGroup(gl_jPanel1
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jPanel1
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jPanel1
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(nullGrad)
														.addGroup(
																gl_jPanel1
																		.createSequentialGroup()
																		.addGroup(
																				gl_jPanel1
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								jLabel1)
																						.addComponent(
																								jLabel2)
																						.addComponent(
																								jLabel3))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																		.addGroup(
																				gl_jPanel1
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING,
																								false)
																						.addComponent(
																								messWinkel)
																						.addComponent(
																								anode,
																								0,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								kristall,
																								javax.swing.GroupLayout.PREFERRED_SIZE,
																								142,
																								javax.swing.GroupLayout.PREFERRED_SIZE))))
										.addContainerGap(34, Short.MAX_VALUE)));
		gl_jPanel1
				.setVerticalGroup(gl_jPanel1
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jPanel1
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(nullGrad)
										.addGap(18, 18, 18)
										.addGroup(
												gl_jPanel1
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.CENTER)
														.addComponent(jLabel1)
														.addComponent(
																kristall,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												gl_jPanel1
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.CENTER)
														.addComponent(
																anode,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel2))
										.addGap(18, 18, 18)
										.addGroup(
												gl_jPanel1
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(jLabel3)
														.addComponent(
																messWinkel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				"Fortschritt",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				resourceMap.getFont("standard.font")));

		jLabel4.setFont(resourceMap.getFont("standard.font"));
		jLabel4.setText(resourceMap.getString("targetwinkel.text"));

		jLabel5.setFont(resourceMap.getFont("standard.font"));
		jLabel5.setText(resourceMap.getString("sensorwinkel.text"));

		jLabel6.setFont(resourceMap.getFont("standard.font"));
		jLabel6.setText(resourceMap.getString("rate.text"));

		targetWinkel.setBackground(new java.awt.Color(238, 238, 238));
		targetWinkel.setBorder(null);
		targetWinkel.setText("0 °");

		sensorWinkel.setBackground(new java.awt.Color(238, 238, 238));
		sensorWinkel.setBorder(null);
		sensorWinkel.setText("0 °");

		rate.setBackground(new java.awt.Color(238, 238, 238));
		rate.setBorder(null);
		rate.setText("0 /s");

		progressBar = new JProgressBar();
		progressBar.setEnabled(false);

		javax.swing.GroupLayout gl_jPanel2 = new javax.swing.GroupLayout(
				jPanel2);
		gl_jPanel2
				.setHorizontalGroup(gl_jPanel2
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_jPanel2
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jPanel2
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_jPanel2
																		.createSequentialGroup()
																		.addComponent(
																				progressBar,
																				GroupLayout.DEFAULT_SIZE,
																				271,
																				Short.MAX_VALUE)
																		.addContainerGap())
														.addGroup(
																gl_jPanel2
																		.createSequentialGroup()
																		.addGroup(
																				gl_jPanel2
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addComponent(
																								jLabel4)
																						.addComponent(
																								jLabel5)
																						.addComponent(
																								jLabel6))
																		.addPreferredGap(
																				ComponentPlacement.UNRELATED)
																		.addGroup(
																				gl_jPanel2
																						.createParallelGroup(
																								Alignment.LEADING,
																								false)
																						.addComponent(
																								rate)
																						.addComponent(
																								sensorWinkel)
																						.addComponent(
																								targetWinkel,
																								GroupLayout.PREFERRED_SIZE,
																								141,
																								GroupLayout.PREFERRED_SIZE))
																		.addGap(46)))));
		gl_jPanel2
				.setVerticalGroup(gl_jPanel2
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_jPanel2
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jPanel2
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_jPanel2
																		.createSequentialGroup()
																		.addComponent(
																				targetWinkel,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(18)
																		.addComponent(
																				sensorWinkel,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(18)
																		.addComponent(
																				rate,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE))
														.addGroup(
																gl_jPanel2
																		.createSequentialGroup()
																		.addComponent(
																				jLabel4)
																		.addGap(18)
																		.addComponent(
																				jLabel5)
																		.addGap(18)
																		.addComponent(
																				jLabel6)))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addComponent(progressBar,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addGap(0, 0, Short.MAX_VALUE)));
		jPanel2.setLayout(gl_jPanel2);

		jTextArea1.setBackground(new java.awt.Color(238, 238, 238));
		jTextArea1.setColumns(20);
		jTextArea1.setEditable(false);
		jTextArea1.setRows(5);
		jTextArea1.setText(resourceMap.getString("anleitung.text"));
		jTextArea1.setBorder(null);
		jScrollPane1.setViewportView(jTextArea1);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		layout.setHorizontalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		starteSuche,
																		GroupLayout.PREFERRED_SIZE,
																		211,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(42)
																.addComponent(
																		abbrechen,
																		GroupLayout.PREFERRED_SIZE,
																		150,
																		GroupLayout.PREFERRED_SIZE))
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING,
																				false)
																				.addComponent(
																						jPanel1,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)
																				.addComponent(
																						jPanel2,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE))
																.addGap(18)
																.addComponent(
																		jScrollPane1,
																		GroupLayout.DEFAULT_SIZE,
																		300,
																		Short.MAX_VALUE)))
								.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(Alignment.TRAILING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		jPanel1,
																		GroupLayout.PREFERRED_SIZE,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(18)
																.addComponent(
																		jPanel2,
																		GroupLayout.PREFERRED_SIZE,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(0,
																		0,
																		Short.MAX_VALUE))
												.addComponent(
														jScrollPane1,
														GroupLayout.DEFAULT_SIZE,
														335, Short.MAX_VALUE))
								.addGap(18)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(starteSuche)
												.addComponent(abbrechen))
								.addContainerGap()));
		getContentPane().setLayout(layout);

		pack();
		initDataBindings();
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton abbrechen;
	private javax.swing.JComboBox<String> anode;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JLabel jLabel5;
	private javax.swing.JLabel jLabel6;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JTextArea jTextArea1;
	private javax.swing.JComboBox<String> kristall;
	private javax.swing.JFormattedTextField messWinkel;
	private javax.swing.JCheckBox nullGrad;
	private javax.swing.JFormattedTextField rate;
	private javax.swing.JFormattedTextField sensorWinkel;
	private javax.swing.JButton starteSuche;
	private javax.swing.JFormattedTextField targetWinkel;
	private javax.swing.JProgressBar progressBar;

	// End of variables declaration//GEN-END:variables

	private void beendeJustierung() {
		// Oberfläche wiederherstellen, Messung aktivieren, Variable
		// justierung leeren
		GHMessen.getApplication().getMainView().setEnableMessung(statusMessung);
		if (progressBar != null) {
			progressBar.setEnabled(false);
			progressBar.setValue(0);
		}
		sensorWinkel.setEnabled(false);
		targetWinkel.setEnabled(false);
		rate.setEnabled(false);
		if (starteSuche != null)
			starteSuche.setEnabled(true);
	}

	private class FortschrittTimer implements Runnable {
		@Override
		public void run() {
			if (justieren == null
					|| justieren.getStatus() != JustiereXRay.Status.LAEUFT) {
				if (fortschritt != null) {
					fortschritt.cancel(false);
					fortschritt = null;
				}
				beendeJustierung();
				if (justieren.getStatus() == JustiereXRay.Status.ERFOLGREICH)
					abbrechen.setText(resourceMap.getString("schliessen.text"));
			} else if (justieren != null) {
				progressBar.setValue(justieren.getProzentFertig());
				sensorWinkel.setText(String.format("%3.1f°",
						justieren.getSensor()));
				if (justieren.getModus() == JustiereXRay.Modus.JUSTIERUNG)
					targetWinkel.setText(String.format("%3.1f°",
							justieren.getTarget()));
				rate.setText(String.format("%d /s", justieren.getRate()));
			}
		}

	}

	@Action
	public void suchen() {
		// Oberfläche anpassen, Messung deaktivieren, Progressbar starten
		try {
			justieren.starten();
			statusMessung = GHMessen.getApplication().getMainView()
					.isMessungEnabled();
			GHMessen.getApplication().getMainView().setEnableMessung(false);
			starteSuche.setEnabled(false);
			progressBar.setEnabled(true);
			sensorWinkel.setEnabled(true);
			rate.setEnabled(true);
			targetWinkel.setEnabled(!nullGrad.isSelected());
			fortschritt = GHMessen.getApplication().getMainView().zeitgeber
					.scheduleWithFixedDelay(new FortschrittTimer(), 100, 1000,
							TimeUnit.MILLISECONDS);
		} catch (InterruptedException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (EAdapterException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	@Action
	public void schliessen() {
		// Abbrechen einer eventuell laufenden Messung und schließen des
		// Dialogs.
		if (justieren != null
				&& justieren.getStatus() == JustiereXRay.Status.LAEUFT) {
			justieren.justierungStop();
			beendeJustierung();
		}
		setVisible(false);
	}

	protected void initDataBindings() {
	}
}
