/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge;

import ghmessen.GHMessen;
import ghmessen.Messparameter;
import ghmessen.adapter.AdapterListe;
import ghmessen.adapter.IAdapter;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.util.Util;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.text.ParseException;

import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ld.api.IXRay;
import ld.api.XRayModes;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;

public class XRayMessparameterBox extends MessparameterBox {

	private static final long serialVersionUID = -1047478272234877378L;
	private static final boolean DEBUG = false;
	private ActionMap actionMap;
	private ResourceMap resourceMap;
	private FocusAdapter txraydataAdapter;
	private JDialog einmessenBox;

	private enum EingabeFeld {
		U_kV, I_mA, beta_min, beta_max, db, dt, beta_sensor, beta_target, beta_coupled;
	}

	public XRayMessparameterBox(java.awt.Frame parent, boolean modal,
			Messparameter mp, AdapterListe al, Groessenliste gl) {
		super(parent, modal, mp, al, gl);
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentHidden(ComponentEvent e) {
				if (einmessenBox != null && einmessenBox.isVisible())
					einmessenBox.setVisible(false);
			}
		});
		mp.setModus(Messparameter.MODUSXRAY);
		XRayModes xrmode = mp.getGoniometerMode();
		sensorWinkel.setValue(mp.getBetaSensor());
		targetWinkel.setValue(mp.getBetaTarget());
		if (!xrmode.isAndEmpty(XRayModes.Sensor)) {
			sensor.setSelected(true);
		} else if (!xrmode.isAndEmpty(XRayModes.Target)) {
			target.setSelected(true);
		} else if (!xrmode.isAndEmpty(XRayModes.Coupled)) {
			coupled.setSelected(true);
		}
		u_kV.setValue(mp.getU());
		i_mA.setValue(mp.getI());
		beta_min.setValue(mp.getBuG());
		beta_max.setValue(mp.getBoG());
		db.setValue(mp.getDB());
		dt.setValue(mp.getDT());
		txraydataAdapter = new FocusAdapter() {
			Object prop;

			public void focusGained(FocusEvent e) {
				Messparameter mp = null;
				prop = e.getSource();
				if (prop instanceof JFormattedTextField) {
					JFormattedTextField feld = (JFormattedTextField) (prop instanceof JFormattedTextField ? prop
							: null);
					if (feld != null) {
						prop = feld.getDocument().getProperty("parameter");
						mp = (Messparameter) (prop instanceof Messparameter ? prop
								: null);
					}
				} else if (prop instanceof JRadioButton) {
					JRadioButton feld = (JRadioButton) prop;
					prop = feld.getClientProperty("parameter");
					mp = (Messparameter) (prop instanceof Messparameter ? prop : null);
				}
				if (mp == null)
					return;
				XRayModes xrmode = mp.getGoniometerMode();
				sensorWinkel.setValue(mp.getBetaSensor());
				targetWinkel.setValue(mp.getBetaTarget());
				if (!xrmode.isAndEmpty(XRayModes.Sensor)) {
					sensor.setSelected(true);
				} else if (!xrmode.isAndEmpty(XRayModes.Target)) {
					target.setSelected(true);
				} else if (!xrmode.isAndEmpty(XRayModes.Coupled)) {
					coupled.setSelected(true);
				}
				u_kV.setValue(mp.getU());
				i_mA.setValue(mp.getI());
				beta_min.setValue(mp.getBuG());
				beta_max.setValue(mp.getBoG());
				db.setValue(mp.getDB());
				dt.setValue(mp.getDT());
			}

			public void focusLost(FocusEvent e) {
				prop = e.getSource();
				JFormattedTextField feld = (JFormattedTextField) (prop instanceof JFormattedTextField ? prop
						: null);
				if (feld != null) {
					prop = feld.getDocument().getProperty("name");
					EingabeFeld name = (EingabeFeld) (prop instanceof EingabeFeld ? prop
							: null);
					prop = feld.getDocument().getProperty("parameter");
					Messparameter mp = (Messparameter) (prop instanceof Messparameter ? prop
							: null);
					parameterSchreiben(feld, name, mp);
				}
			}

			public void parameterSchreiben(JFormattedTextField feld,
					EingabeFeld name, Messparameter mp) {
				if (feld == null)
					return;
				double alt = (Double) feld.getValue();
				try {
					feld.commitEdit();
					double wert = (Double) feld.getValue();
					prop = feld.getDocument().getProperty("minimum");
					double min = prop instanceof Double ? (Double) prop : 0.0;
					prop = feld.getDocument().getProperty("maximum");
					double max = prop instanceof Double ? (Double) prop
							: 1000.0;
					if (wert < min) {
						feld.setText(Double.toString(min));
						feld.commitEdit();
					} else if (wert > max) {
						feld.setText(Double.toString(max));
						feld.commitEdit();
					}
				} catch (ParseException e) {
					if (DEBUG)
						e.printStackTrace();
				}
				Double d = (Double) feld.getValue();
				if (mp == null || d == null || d == alt)
					return;
				if (DEBUG)
					System.out
							.println("[XRayMessparameterBox] - parameterSchreiben: feld("
									+ feld
									+ ") name("
									+ name
									+ ") mp("
									+ mp
									+ ")");
				switch (name) {
				case U_kV:
					mp.setU(d);
					break;
				case I_mA:
					mp.setI(d);
					break;
				case beta_min:
					mp.setBuG(d);
					break;
				case beta_max:
					mp.setBoG(d);
					break;
				case db:
					mp.setDB(d);
					break;
				case dt:
					mp.setDT(d);
					break;
				case beta_sensor:
				case beta_target:
				case beta_coupled:
					mp.setB(d);
					break;
				}
			}

		};
		sensorWinkel.addFocusListener(txraydataAdapter);
		targetWinkel.addFocusListener(txraydataAdapter);
		u_kV.addFocusListener(txraydataAdapter);
		i_mA.addFocusListener(txraydataAdapter);
		beta_min.addFocusListener(txraydataAdapter);
		beta_max.addFocusListener(txraydataAdapter);
		db.addFocusListener(txraydataAdapter);
		dt.addFocusListener(txraydataAdapter);
	}

	protected void initComponents() {

		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(XRayMessparameterBox.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(XRayMessparameterBox.class);
		goniometer = new javax.swing.ButtonGroup();
		messModus = new javax.swing.ButtonGroup();
		messwerte = new javax.swing.ButtonGroup();
		schliessen = new javax.swing.JButton(actionMap.get("schliessen"));
		hilfe = new javax.swing.JButton(actionMap.get("hilfe"));
		hilfe.setEnabled(GHMessen.hilfe != null);
		if (GHMessen.hilfe != null && GHMessen.hSet != null)
			GHMessen.hilfe.enableHelpKey(getRootPane(), "xraymessparameter",
					GHMessen.hSet);
		einmessen = new javax.swing.JButton(actionMap.get("einmessen"));

		jTabbedPane1 = new javax.swing.JTabbedPane();
		geraet = new javax.swing.JPanel();
		pGoniometer = new javax.swing.JPanel();
		sensorLabel1 = new javax.swing.JLabel();
		sensorLabel2 = new javax.swing.JLabel();
		sensorWinkel = new javax.swing.JFormattedTextField(Util.doubleFormat);
		sensorWinkel.setToolTipText(resourceMap
				.getString("sensorWinkel.toolTipText"));
		sensorWinkel.getDocument().putProperty("name", EingabeFeld.beta_sensor);
		sensorWinkel.getDocument().putProperty("feld", sensorWinkel);
		sensorWinkel.getDocument().putProperty("parameter", mp);
		sensorWinkel.getDocument().putProperty("minimum", new Double(-10.0));
		sensorWinkel.getDocument().putProperty("maximum", new Double(170.0));
		sensor = new javax.swing.JRadioButton(actionMap.get("goniometer"));
		sensor.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if (targetLabel1 != null)
					targetLabel1.setEnabled(sensor.isSelected());
				if (targetLabel2 != null)
					targetLabel2.setEnabled(sensor.isSelected());
				if (targetWinkel != null)
					targetWinkel.setEnabled(sensor.isSelected());
			}
		});
		sensor.setActionCommand("1");
		sensor.putClientProperty("parameter", mp);

		targetLabel1 = new javax.swing.JLabel();
		targetLabel2 = new javax.swing.JLabel();
		targetWinkel = new javax.swing.JFormattedTextField(Util.doubleFormat);
		targetWinkel.setToolTipText(resourceMap
				.getString("targetWinkel.toolTipText"));
		targetWinkel.getDocument().putProperty("name", EingabeFeld.beta_target);
		targetWinkel.getDocument().putProperty("feld", targetWinkel);
		targetWinkel.getDocument().putProperty("parameter", mp);
		targetWinkel.getDocument().putProperty("minimum", new Double(-180.0));
		targetWinkel.getDocument().putProperty("maximum", new Double(180.0));
		target = new javax.swing.JRadioButton(actionMap.get("goniometer"));
		target.addChangeListener(
				new ChangeListener() {
					public void stateChanged(ChangeEvent evt) {
						if (sensorLabel1 != null)
							sensorLabel1.setEnabled(target.isSelected());
						if (sensorLabel2 != null)
							sensorLabel2.setEnabled(target.isSelected());
						if (sensorWinkel != null)
							sensorWinkel.setEnabled(target.isSelected());
					}
				});
		target.setActionCommand("2");
		target.putClientProperty("parameter", mp);

		coupled = new javax.swing.JRadioButton(actionMap.get("goniometer"));
		coupled.setActionCommand("4");
		coupled.putClientProperty("parameter", mp);

		pParameter = new javax.swing.JPanel();
		jLabel7 = new javax.swing.JLabel();
		u_kV = new javax.swing.JFormattedTextField(Util.doubleFormat);
		u_kV.setToolTipText(resourceMap.getString("u_kV.toolTipText"));
		u_kV.getDocument().putProperty("feld", u_kV);
		u_kV.getDocument().putProperty("name", EingabeFeld.U_kV);
		u_kV.getDocument().putProperty("parameter", mp);
		u_kV.getDocument().putProperty("minimum", new Double(0.0));
		u_kV.getDocument().putProperty("maximum", new Double(35.0));
		jLabel8 = new javax.swing.JLabel();
		jLabel9 = new javax.swing.JLabel();
		i_mA = new javax.swing.JFormattedTextField(Util.doubleFormat);
		i_mA.setToolTipText(resourceMap.getString("i_mA.toolTipText"));
		i_mA.getDocument().putProperty("name", EingabeFeld.I_mA);
		i_mA.getDocument().putProperty("feld", i_mA);
		i_mA.getDocument().putProperty("parameter", mp);
		i_mA.getDocument().putProperty("minimum", new Double(0.0));
		i_mA.getDocument().putProperty("maximum", new Double(1.0));
		jLabel10 = new javax.swing.JLabel();
		jLabel11 = new javax.swing.JLabel();
		beta_min = new javax.swing.JFormattedTextField(Util.doubleFormat);
		beta_min.setToolTipText(resourceMap.getString("beta_min.toolTipText"));
		beta_min.getDocument().putProperty("name", EingabeFeld.beta_min);
		beta_min.getDocument().putProperty("feld", beta_min);
		beta_min.getDocument().putProperty("parameter", mp);
		beta_min.getDocument().putProperty("minimum", new Double(-180.0));
		beta_min.getDocument().putProperty("maximum", new Double(180.0));
		jLabel12 = new javax.swing.JLabel();
		jLabel13 = new javax.swing.JLabel();
		beta_max = new javax.swing.JFormattedTextField(Util.doubleFormat);
		beta_max.setToolTipText(resourceMap.getString("beta_max.toolTipText"));
		beta_max.getDocument().putProperty("name", EingabeFeld.beta_max);
		beta_max.getDocument().putProperty("feld", beta_max);
		beta_max.getDocument().putProperty("parameter", mp);
		beta_max.getDocument().putProperty("minimum", new Double(-180.0));
		beta_max.getDocument().putProperty("maximum", new Double(180.0));
		jLabel14 = new javax.swing.JLabel();
		jLabel15 = new javax.swing.JLabel();
		db = new javax.swing.JFormattedTextField(Util.doubleFormat);
		db.setToolTipText(resourceMap.getString("db.toolTipText"));
		db.getDocument().putProperty("name", EingabeFeld.db);
		db.getDocument().putProperty("feld", db);
		db.getDocument().putProperty("parameter", mp);
		db.getDocument().putProperty("minimum", new Double(0.1));
		db.getDocument().putProperty("maximum", new Double(20.0));
		jLabel16 = new javax.swing.JLabel();
		jLabel17 = new javax.swing.JLabel();
		dt = new javax.swing.JFormattedTextField(Util.doubleFormat);
		dt.setToolTipText(resourceMap.getString("dt.toolTipText"));
		dt.getDocument().putProperty("name", EingabeFeld.dt);
		dt.getDocument().putProperty("feld", dt);
		dt.getDocument().putProperty("parameter", mp);
		dt.getDocument().putProperty("minimum", new Double(0.1));
		dt.getDocument().putProperty("maximum", new Double(9999.0));
		jLabel18 = new javax.swing.JLabel();
		pAktionen = new javax.swing.JPanel();
		automatisch = new javax.swing.JRadioButton(actionMap.get("modusAuto"));
		manuell = new javax.swing.JRadioButton(actionMap.get("modusManuell"));

		setTitle(resourceMap.getString("Form.title"));

		pGoniometer.setBorder(javax.swing.BorderFactory.createTitledBorder(
				null, resourceMap.getString("goniometer.text"),
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 0, 12))); // NOI18N

		goniometer.add(sensor);
		sensor.setFont(new java.awt.Font("Dialog", 0, 12));
		sensor.setText(resourceMap.getString("sensor.text"));
		sensor.setName("sensor"); // NOI18N

		sensorLabel1.setFont(new java.awt.Font("Dialog", 0, 12));
		sensorLabel1.setText("β=");
		sensorLabel1.setName("jLabel1"); // NOI18N

		sensorWinkel.setValue(new Double(0.0));
		sensorWinkel.setName("sensorWinkel"); // NOI18N

		sensorLabel2.setFont(new java.awt.Font("Dialog", 0, 12));
		sensorLabel2.setText("°");
		sensorLabel2.setName("jLabel2"); // NOI18N

		goniometer.add(target);
		target.setFont(new java.awt.Font("Dialog", 0, 12));
		target.setText("Target");
		target.setName("target"); // NOI18N

		targetLabel1.setFont(new java.awt.Font("Dialog", 0, 12));
		targetLabel1.setText("β=");
		targetLabel1.setName("jLabel3"); // NOI18N

		targetWinkel.setValue(new Double(0.0));
		targetWinkel.setName("targetWinkel"); // NOI18N

		targetLabel2.setFont(new java.awt.Font("Dialog", 0, 12));
		targetLabel2.setText("°");
		targetLabel2.setName("jLabel4"); // NOI18N

		goniometer.add(coupled);
		coupled.setFont(new java.awt.Font("Dialog", 0, 12));
		coupled.setSelected(true);
		coupled.setText("Coupled");
		coupled.setName("coupled"); // NOI18N

		javax.swing.GroupLayout gl_pGoniometer = new javax.swing.GroupLayout(
				pGoniometer);
		gl_pGoniometer
				.setHorizontalGroup(gl_pGoniometer
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_pGoniometer
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_pGoniometer
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(sensor)
														.addComponent(target)
														.addComponent(coupled))
										.addGap(18)
										.addGroup(
												gl_pGoniometer
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																targetLabel1)
														.addComponent(
																sensorLabel1))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_pGoniometer
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_pGoniometer
																		.createSequentialGroup()
																		.addComponent(
																				sensorWinkel,
																				GroupLayout.PREFERRED_SIZE,
																				45,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(1)
																		.addComponent(
																				sensorLabel2,
																				GroupLayout.PREFERRED_SIZE,
																				16,
																				GroupLayout.PREFERRED_SIZE))
														.addGroup(
																gl_pGoniometer
																		.createSequentialGroup()
																		.addComponent(
																				targetWinkel,
																				GroupLayout.PREFERRED_SIZE,
																				45,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(1)
																		.addComponent(
																				targetLabel2,
																				GroupLayout.PREFERRED_SIZE,
																				16,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(0,
																				0,
																				Short.MAX_VALUE)))
										.addContainerGap()));
		gl_pGoniometer
				.setVerticalGroup(gl_pGoniometer
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_pGoniometer
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_pGoniometer
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(sensor)
														.addComponent(
																sensorLabel1)
														.addComponent(
																sensorWinkel,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																sensorLabel2))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_pGoniometer
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(target)
														.addComponent(
																targetLabel1)
														.addComponent(
																targetWinkel,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																targetLabel2))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addComponent(coupled)
										.addContainerGap(15, Short.MAX_VALUE)));
		pGoniometer.setLayout(gl_pGoniometer);

		pParameter.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				resourceMap.getString("parameter.text"),
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 0, 12))); // NOI18N

		jLabel7.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel7.setText("U=");

		u_kV.setValue(new Double(5.0));

		jLabel8.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel8.setText("kV");

		jLabel9.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel9.setText("mA");

		i_mA.setValue(0.00);

		jLabel10.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel10.setText("I=");

		jLabel11.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel11.setText("°");

		beta_min.setValue(3.1);

		jLabel12.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel12.setText("<html>β<sub>min</sub>=</html>");

		jLabel13.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel13.setText("°");

		beta_max.setValue(30.0);

		jLabel14.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel14.setText("<html>β<sub>max</sub>=</html>");

		jLabel15.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel15.setText("°");

		db.setValue(0.1);

		jLabel16.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel16.setText("Δβ=");

		jLabel17.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel17.setText("s");

		dt.setValue(1.0);

		jLabel18.setFont(new java.awt.Font("Dialog", 0, 12));
		jLabel18.setText("Δt=");

		javax.swing.GroupLayout gl_pParameter = new javax.swing.GroupLayout(
				pParameter);
		gl_pParameter
				.setHorizontalGroup(gl_pParameter
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_pParameter
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(jLabel10)
														.addComponent(jLabel7))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																u_kV,
																GroupLayout.PREFERRED_SIZE,
																42,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																i_mA,
																GroupLayout.PREFERRED_SIZE,
																42,
																GroupLayout.PREFERRED_SIZE))
										.addGap(1)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(jLabel8)
														.addComponent(jLabel9))
										.addGap(18)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(jLabel14)
														.addComponent(jLabel12))
										.addGap(5)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																beta_max,
																GroupLayout.PREFERRED_SIZE,
																42,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																beta_min,
																GroupLayout.PREFERRED_SIZE,
																42,
																GroupLayout.PREFERRED_SIZE))
										.addGap(1)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(jLabel13)
														.addComponent(jLabel11))
										.addGap(12)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(jLabel16)
														.addComponent(jLabel18))
										.addGap(5)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																dt,
																GroupLayout.PREFERRED_SIZE,
																42,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																db,
																GroupLayout.PREFERRED_SIZE,
																42,
																GroupLayout.PREFERRED_SIZE))
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_pParameter
																		.createSequentialGroup()
																		.addGap(1)
																		.addComponent(
																				jLabel15))
														.addGroup(
																gl_pParameter
																		.createSequentialGroup()
																		.addGap(1)
																		.addComponent(
																				jLabel17)))
										.addContainerGap(61, Short.MAX_VALUE)));
		gl_pParameter
				.setVerticalGroup(gl_pParameter
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_pParameter
										.createSequentialGroup()
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(jLabel7)
														.addComponent(
																u_kV,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel8)
														.addComponent(jLabel12)
														.addComponent(
																beta_min,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel11)
														.addComponent(jLabel16)
														.addComponent(
																db,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel15))
										.addGap(18)
										.addGroup(
												gl_pParameter
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(jLabel10)
														.addComponent(
																i_mA,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel9)
														.addComponent(jLabel14)
														.addComponent(
																beta_max,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel13)
														.addComponent(jLabel18)
														.addComponent(
																dt,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel17))
										.addContainerGap(36, Short.MAX_VALUE)));
		pParameter.setLayout(gl_pParameter);

		pAktionen.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				resourceMap.getString("actions.text"),
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 0, 12))); // NOI18N
		pAktionen.setName("jPanel3"); // NOI18N

		messModus.add(automatisch);
		automatisch.setSelected(true);

		messModus.add(manuell);

		javax.swing.GroupLayout gl_pAktionen = new javax.swing.GroupLayout(
				pAktionen);
		gl_pAktionen.setHorizontalGroup(gl_pAktionen.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_pAktionen
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								gl_pAktionen
										.createParallelGroup(Alignment.LEADING)
										.addComponent(automatisch)
										.addComponent(manuell))
						.addContainerGap(226, Short.MAX_VALUE)));
		gl_pAktionen.setVerticalGroup(gl_pAktionen.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_pAktionen.createSequentialGroup().addContainerGap()
						.addComponent(automatisch)
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addComponent(manuell)
						.addContainerGap(40, Short.MAX_VALUE)));
		pAktionen.setLayout(gl_pAktionen);

		JPanel panel = new JPanel();
		panel.setBorder(new TitledBorder(null, "Messwerte",
				TitledBorder.LEADING, TitledBorder.TOP, null, null));

		javax.swing.GroupLayout gl_geraet = new javax.swing.GroupLayout(geraet);
		gl_geraet
				.setHorizontalGroup(gl_geraet
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_geraet
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_geraet
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_geraet
																		.createSequentialGroup()
																		.addComponent(
																				pAktionen,
																				GroupLayout.PREFERRED_SIZE,
																				265,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.UNRELATED)
																		.addComponent(
																				panel,
																				GroupLayout.DEFAULT_SIZE,
																				363,
																				Short.MAX_VALUE))
														.addGroup(
																gl_geraet
																		.createSequentialGroup()
																		.addComponent(
																				pGoniometer,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.UNRELATED)
																		.addComponent(
																				pParameter,
																				GroupLayout.DEFAULT_SIZE,
																				428,
																				Short.MAX_VALUE)))
										.addContainerGap()));
		gl_geraet.setVerticalGroup(gl_geraet.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_geraet
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								gl_geraet
										.createParallelGroup(Alignment.LEADING)
										.addComponent(pParameter,
												GroupLayout.DEFAULT_SIZE, 100,
												Short.MAX_VALUE)
										.addComponent(pGoniometer,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE))
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addGroup(
								gl_geraet
										.createParallelGroup(Alignment.LEADING)
										.addComponent(panel,
												GroupLayout.DEFAULT_SIZE, 116,
												Short.MAX_VALUE)
										.addComponent(pAktionen,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)).addGap(10)));

		JRadioButton anhaengen = new JRadioButton(actionMap.get("anhaengen"));
		anhaengen.setFont(resourceMap.getFont("dialog.font"));
		messwerte.add(anhaengen);

		JRadioButton ersetzen = new JRadioButton(actionMap.get("ersetzen"));
		ersetzen.setFont(resourceMap.getFont("dialog.font"));
		messwerte.add(ersetzen);

		JRadioButton neueMessreihe = new JRadioButton(
				actionMap.get("neueMessreihe"));
		neueMessreihe.setFont(resourceMap.getFont("dialog.font"));
		messwerte.add(neueMessreihe);

		if (mp != null) {
			if (!mp.isUeberschreibenEin())
				neueMessreihe.setSelected(true);
			else if (mp.isAnhaengenEin())
				anhaengen.setSelected(true);
			else
				ersetzen.setSelected(true);
		}

		GroupLayout gl_panel = new GroupLayout(panel);
		gl_panel.setHorizontalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup()
						.addGroup(
								gl_panel.createParallelGroup(Alignment.LEADING)
										.addComponent(neueMessreihe)
										.addComponent(ersetzen)
										.addComponent(anhaengen))
						.addContainerGap(172, Short.MAX_VALUE)));
		gl_panel.setVerticalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup().addGap(5)
						.addComponent(anhaengen)
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addComponent(ersetzen).addGap(4)
						.addComponent(neueMessreihe)
						.addContainerGap(12, Short.MAX_VALUE)));
		panel.setLayout(gl_panel);
		geraet.setLayout(gl_geraet);

		jTabbedPane1.addTab(resourceMap.getString("geraet.title"), geraet);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		layout.setHorizontalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addComponent(
														jTabbedPane1,
														GroupLayout.DEFAULT_SIZE,
														669, Short.MAX_VALUE)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		schliessen,
																		GroupLayout.PREFERRED_SIZE,
																		152,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(88)
																.addComponent(
																		einmessen,
																		GroupLayout.PREFERRED_SIZE,
																		154,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(95)
																.addComponent(
																		hilfe,
																		GroupLayout.PREFERRED_SIZE,
																		112,
																		GroupLayout.PREFERRED_SIZE)))
								.addContainerGap()));
		layout.setVerticalGroup(layout.createParallelGroup(Alignment.TRAILING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(jTabbedPane1,
										GroupLayout.DEFAULT_SIZE, 277,
										Short.MAX_VALUE)
								.addGap(18)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(schliessen)
												.addComponent(einmessen)
												.addComponent(hilfe))
								.addContainerGap()));
		getContentPane().setLayout(layout);

		pack();
	}// </editor-fold>//GEN-END:initComponents

	private javax.swing.JRadioButton automatisch;
	private javax.swing.JRadioButton coupled;
	private javax.swing.JButton einmessen;
	private javax.swing.JPanel geraet;
	private javax.swing.ButtonGroup goniometer;
	private javax.swing.JButton hilfe;
	private javax.swing.JFormattedTextField u_kV;
	private javax.swing.JFormattedTextField i_mA;
	private javax.swing.JFormattedTextField beta_min;
	private javax.swing.JFormattedTextField beta_max;
	private javax.swing.JFormattedTextField db;
	private javax.swing.JFormattedTextField dt;
	private javax.swing.JLabel sensorLabel1;
	private javax.swing.JLabel jLabel10;
	private javax.swing.JLabel jLabel11;
	private javax.swing.JLabel jLabel12;
	private javax.swing.JLabel jLabel13;
	private javax.swing.JLabel jLabel14;
	private javax.swing.JLabel jLabel15;
	private javax.swing.JLabel jLabel16;
	private javax.swing.JLabel jLabel17;
	private javax.swing.JLabel jLabel18;
	private javax.swing.JLabel sensorLabel2;
	private javax.swing.JLabel targetLabel1;
	private javax.swing.JLabel targetLabel2;
	private javax.swing.JLabel jLabel7;
	private javax.swing.JLabel jLabel8;
	private javax.swing.JLabel jLabel9;
	private javax.swing.JPanel pGoniometer;
	private javax.swing.JPanel pParameter;
	private javax.swing.JPanel pAktionen;
	private javax.swing.JTabbedPane jTabbedPane1;
	private javax.swing.JRadioButton manuell;
	private javax.swing.ButtonGroup messModus;
	private javax.swing.JButton schliessen;
	private javax.swing.JRadioButton sensor;
	private javax.swing.JFormattedTextField sensorWinkel;
	private javax.swing.JRadioButton target;
	private javax.swing.JFormattedTextField targetWinkel;
	private javax.swing.ButtonGroup messwerte;

	// End of variables declaration//GEN-END:variables

	@Override
	protected void adjustComponents() {
		beta_min.setEnabled(!mp.isEinzelEin());
		beta_max.setEnabled(!mp.isEinzelEin());
		db.setEnabled(!mp.isEinzelEin());
	}

	@Action
	public void schliessen() {
		if (einmessenBox != null && einmessenBox.isVisible()) {
			einmessenBox.setVisible(false);
		}
		setVisible(false);
	}

	@Action
	public void goniometer() {
		String s = goniometer.getSelection().getActionCommand();
		int mode = Integer.parseInt(s);
		mp.setGoniometerMode(mode);
	}

	@Action
	public void modusManuell() {
		if (manuell.isSelected() && mp != null) {
			mp.setEinzelEin(true);
			adjustComponents();
		}
	}

	@Action
	public void modusAuto() {
		if (automatisch.isSelected() && mp != null) {
			mp.setEinzelEin(false);
			adjustComponents();
		}
	}

	@Action
	public void anhaengen() {
		if (mp != null) {
			mp.setAnhaengenEin(true);
			mp.setUeberschreibenEin(true);
		}
	}

	@Action
	public void ersetzen() {
		if (mp != null) {
			mp.setAnhaengenEin(false);
			mp.setUeberschreibenEin(true);
		}
	}

	@Action
	public void neueMessreihe() {
		if (mp != null) {
			mp.setAnhaengenEin(false);
			mp.setUeberschreibenEin(false);
		}
	}

	@Action
	public void hilfe() {
		if (GHMessen.hilfe != null)
			GHMessen.hilfe.setDisplayed(true);
		GHMessen.hilfe.setCurrentID("xraymessparameter");
	}

	@Action
	public void einmessen() {
		if (einmessenBox == null) {
			JFrame mainFrame = GHMessen.getApplication().getMainFrame();
			Groesse g = gl.getXRayGroesse();
			IAdapter a = al.get(g.getNr());
			if (a instanceof IXRay) {
				einmessenBox = new XRayKristallEinmessen(mainFrame, (IXRay) a,
						false);
				einmessenBox.setLocationRelativeTo(mainFrame);
			} else {
				if (DEBUG)
					System.out.println("XRay was not found!");
			}
		}
		einmessenBox.setVisible(true);
	}

	public void parameterSchreiben(JFormattedTextField feld, EingabeFeld name) {
		Double d = (Double) feld.getValue();
		if (mp == null || d == null)
			return;
		switch (name) {
		case U_kV:
			mp.setU(d);
			break;
		case I_mA:
			mp.setI(d);
			break;
		case beta_min:
			mp.setBuG(d);
			break;
		case beta_max:
			mp.setBoG(d);
			break;
		case db:
			mp.setDB(d);
			break;
		case dt:
			mp.setDT(d);
			break;
		case beta_sensor:
		case beta_target:
		case beta_coupled:
			mp.setB(d);
			break;
		}
	}

}
