/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge.eigenschaften.sensoren;

import ghmessen.GHMessen;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.goIO.AdapterGoIO;
import ghmessen.dialoge.eigenschaften.BereichInterface;

import java.awt.Font;
import java.awt.GridLayout;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import libGoIO.GoIOSensor;
import libGoIO.GoIOSensorCellRenderer;
import libGoIO.GoIOSensorFactory;
import libGoIO.GoIOSensorInfo;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;

@SuppressWarnings("serial")
public class SensorenVernier extends JPanel implements BereichInterface {
	private static final boolean DEBUG = false;

	private JTree alleSensoren, vorhandeneSensoren;

	private JButton hinzufuegen;
	private JButton entfernen;
	private JButton alleHinzufuegen;
	private JButton alleEntfernen;

	private static ActionMap actionMap;
	private static ResourceMap resourceMap;

	public SensorenVernier() {
		this(new HashMap<String, GoIOSensor>());
	}

	public SensorenVernier(HashMap<String, GoIOSensor> vernierSensoren) {

		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(SensorenVernier.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(SensorenVernier.class);

		DefaultTreeModel alleListe;
		DefaultTreeModel vorhandeneListe;
		alleListe = GoIOSensorFactory
				.createSensorTreeModel(new DefaultMutableTreeNode(resourceMap
						.getString("alleSensoren.title")));
		vorhandeneListe = initVorhandeneListe(new DefaultMutableTreeNode(
				resourceMap.getString("vorhandeneSensoren.title")), alleListe,
				vernierSensoren);

		JLabel lblVerniersensoren = new JLabel(
				resourceMap.getString("dialog.title"));
		lblVerniersensoren.setFont(new Font("Dialog", Font.BOLD, 14));
		lblVerniersensoren.setHorizontalAlignment(SwingConstants.CENTER);

		JLabel lblVerfgbareSensoren = new JLabel(
				resourceMap.getString("alleSensoren.title"));

		JLabel lblVorhandeneSensoren = new JLabel(
				resourceMap.getString("vorhandeneSensoren.title"));

		hinzufuegen = new JButton(actionMap.get("hinzufuegen"));
		hinzufuegen.setEnabled(false);
		alleHinzufuegen = new JButton(actionMap.get("alleHinzufuegen"));
		entfernen = new JButton(actionMap.get("entfernen"));
		entfernen.setEnabled(false);
		alleEntfernen = new JButton(actionMap.get("alleEntfernen"));

		alleSensoren = new JTree(alleListe);
		alleSensoren.setCellRenderer(new GoIOSensorCellRenderer());
		alleSensoren.setRootVisible(false);
		alleSensoren.addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent e) {
				TreePath tp = e.getNewLeadSelectionPath();
				hinzufuegen.setEnabled(tp != null);
			}
		});
		alleSensoren.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null,
				null, null));
		JScrollPane alleRollen = new JScrollPane(alleSensoren);

		vorhandeneSensoren = new JTree(vorhandeneListe);
		vorhandeneSensoren.setCellRenderer(new GoIOSensorCellRenderer());
		vorhandeneSensoren.setRootVisible(false);
		vorhandeneSensoren.setBorder(new BevelBorder(BevelBorder.LOWERED, null,
				null, null, null));
		vorhandeneSensoren
				.addTreeSelectionListener(new TreeSelectionListener() {
					public void valueChanged(TreeSelectionEvent e) {
						TreePath tp = e.getNewLeadSelectionPath();
						entfernen.setEnabled(tp != null);
					}
				});
		JScrollPane vorhandeneRollen = new JScrollPane(vorhandeneSensoren);

		/* Layout */
		JPanel knoepfe = new JPanel();
		GroupLayout groupLayout = new GroupLayout(this);
		groupLayout
				.setHorizontalGroup(groupLayout
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								groupLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												groupLayout
														.createParallelGroup(
																Alignment.TRAILING)
														.addComponent(
																lblVerniersensoren,
																GroupLayout.DEFAULT_SIZE,
																623,
																Short.MAX_VALUE)
														.addGroup(
																groupLayout
																		.createSequentialGroup()
																		.addComponent(
																				lblVerfgbareSensoren)
																		.addPreferredGap(
																				ComponentPlacement.RELATED,
																				417,
																				Short.MAX_VALUE)
																		.addComponent(
																				lblVorhandeneSensoren))
														.addGroup(
																groupLayout
																		.createSequentialGroup()
																		.addComponent(
																				alleRollen,
																				GroupLayout.PREFERRED_SIZE,
																				217,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.UNRELATED)
																		.addComponent(
																				knoepfe,
																				GroupLayout.PREFERRED_SIZE,
																				164,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				vorhandeneRollen,
																				GroupLayout.DEFAULT_SIZE,
																				224,
																				Short.MAX_VALUE)))
										.addContainerGap()));
		groupLayout
				.setVerticalGroup(groupLayout
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								groupLayout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(lblVerniersensoren)
										.addGap(18)
										.addGroup(
												groupLayout
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																lblVerfgbareSensoren,
																GroupLayout.PREFERRED_SIZE,
																15,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																lblVorhandeneSensoren))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												groupLayout
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																groupLayout
																		.createParallelGroup(
																				Alignment.BASELINE)
																		.addComponent(
																				alleRollen,
																				GroupLayout.DEFAULT_SIZE,
																				321,
																				Short.MAX_VALUE)
																		.addComponent(
																				vorhandeneRollen,
																				GroupLayout.DEFAULT_SIZE,
																				321,
																				Short.MAX_VALUE))
														.addComponent(
																knoepfe,
																GroupLayout.PREFERRED_SIZE,
																132,
																GroupLayout.PREFERRED_SIZE))
										.addContainerGap()));
		knoepfe.setLayout(new GridLayout(4, 1, 0, 0));
		knoepfe.add(hinzufuegen);
		knoepfe.add(alleHinzufuegen);
		knoepfe.add(entfernen);
		knoepfe.add(alleEntfernen);
		setLayout(groupLayout);
	}

	public void initialisieren() {
		if (DEBUG)
			System.out
					.println("[SensorenVernier] - initialisieren aufgerufen...");
		if (GHMessen.getApplication() != null) {
			setVorhandeneListe(GHMessen.getApplication().vernierSensoren);
		} else
			setVorhandeneListe(new HashMap<String, GoIOSensor>());
	}

	public void schreiben() {
		if (DEBUG)
			System.out.println("[SensorenVernier] - schreiben aufgerufen...");
		try {
			if (GHMessen.getApplication().vernierSensoren != null) {
				GHMessen.getApplication().vernierSensoren.clear();
				GHMessen.getApplication().vernierSensoren
						.putAll(getVorhandeneListe());
			} else
				GHMessen.getApplication().vernierSensoren = getVorhandeneListe();
			for (IAdapter a : GHMessen.getApplication().getMainView().al
					.values())
				if (a instanceof AdapterGoIO)
					((AdapterGoIO) a).initSensorListe();
		} catch (IllegalStateException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	public void setVorhandeneListe(HashMap<String, GoIOSensor> vernierSensoren) {
		DefaultTreeModel alleListe;
		alleListe = GoIOSensorFactory
				.createSensorTreeModel(new DefaultMutableTreeNode(resourceMap
						.getString("alleSensoren.title")));
		vorhandeneSensoren.setModel(initVorhandeneListe(
				new DefaultMutableTreeNode(resourceMap
						.getString("vorhandeneSensoren.title")), alleListe,
				vernierSensoren));
		alleSensoren.setModel(alleListe);
	}

	public HashMap<String, GoIOSensor> getVorhandeneListe() {
		HashMap<String, GoIOSensor> sensoren = new HashMap<String, GoIOSensor>();
		HashMap<String, String> sensorMap = GoIOSensorFactory.createSensorMap();
		TreeNode root = (TreeNode) vorhandeneSensoren.getModel().getRoot();
		for (Enumeration<? extends TreeNode> e = root.children(); e.hasMoreElements();) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) e
					.nextElement();
			for (Enumeration<? extends TreeNode> ee = node.children(); ee.hasMoreElements();) {
				DefaultMutableTreeNode leaf = (DefaultMutableTreeNode) ee
						.nextElement();
				if (!(leaf.getUserObject() instanceof GoIOSensorInfo))
					continue;
				GoIOSensorInfo sensor = (GoIOSensorInfo) leaf.getUserObject();
				GoIOSensor srec = GoIOSensor.readFromXml(sensorMap.get(sensor
						.toString()));
				sensoren.put(srec.toString(), srec);
				for (int i = 1; i < srec.getNumAutoIds(); i++) {
					srec = srec.clone();
					srec.setSensorNumber(srec.getSensorNumber() + 1);
					sensoren.put(srec.toString(), srec);
				}
			}
		}
		return sensoren;

	}

	private DefaultTreeModel initVorhandeneListe(DefaultMutableTreeNode root,
			DefaultTreeModel quelle, HashMap<String, GoIOSensor> sensoren) {
		// alleListe muss vorher vollständig initialisiert sein
		if (quelle == null || sensoren == null || sensoren.isEmpty())
			return new DefaultTreeModel(root);
		DefaultMutableTreeNode qroot = (DefaultMutableTreeNode) (quelle
				.getRoot());
		for (String key : sensoren.keySet()) {
			String rubrik = sensoren.get(key).getRubrik();
			key = key.replaceFirst("\\|[0-9]*", "");
			DefaultMutableTreeNode knoten = null;
			knoten = findeSensorKnoten(qroot, rubrik, key);
			if (knoten != null) {
				DefaultMutableTreeNode rubrikNode = (DefaultMutableTreeNode) knoten
						.getParent();
				knoten.removeFromParent();
				if (rubrikNode.getChildCount() == 0)
					rubrikNode.removeFromParent();
				rubrikNode = findeRubrikKnoten(root, rubrik, true);
				// Blatt hinzufügen
				rubrikNode.add(knoten);
			}
		}
		return new DefaultTreeModel(root);
	}

	private DefaultMutableTreeNode findeRubrikKnoten(
			DefaultMutableTreeNode root, String rubrik, boolean autocreate) {
		DefaultMutableTreeNode knoten;
		for (@SuppressWarnings("rawtypes")
		Enumeration e = root.children(); e.hasMoreElements();) {
			knoten = (DefaultMutableTreeNode) e.nextElement();
			if (rubrik.equals(knoten.getUserObject()))
				return knoten;
		}
		if (autocreate) {
			knoten = new DefaultMutableTreeNode(rubrik);
			root.add(knoten);
			return knoten;
		} else
			return null;
	}

	private DefaultMutableTreeNode findeSensorKnoten(
			DefaultMutableTreeNode baum, String rubrik, String name) {
		DefaultMutableTreeNode knoten;
		knoten = findeRubrikKnoten(baum, rubrik, false);
		if (knoten != null) {
			for (@SuppressWarnings("rawtypes")
			Enumeration e = knoten.children(); e.hasMoreElements();) {
				knoten = (DefaultMutableTreeNode) e.nextElement();
				if (knoten.getUserObject() instanceof GoIOSensorInfo
						&& name.equals(knoten.getUserObject().toString())) {
					return knoten;
				}
			}
		}
		return null;
	}

	private void updateTrees() {
		((DefaultTreeModel) alleSensoren.getModel()).reload();
		((DefaultTreeModel) vorhandeneSensoren.getModel()).reload();
	}

	/*
	 * =================== Aktionen ================================
	 */
	@Action
	public void hinzufuegen() {
		Vector<TreePath> pfad = new Vector<TreePath>(Arrays.asList(alleSensoren
				.getSelectionPaths()));
		if (DEBUG)
			System.out.println("[SensorenVernier] - hinzufügen aufgerufen...");
		if (pfad == null || pfad.size() == 0)
			return;
		// Auswahl normalisieren, Sensoren statt Rubriken einsetzen
		Vector<TreePath> rubrik = new Vector<TreePath>();
		for (TreePath t : pfad) {
			if (!((DefaultMutableTreeNode) t.getLastPathComponent()).isLeaf()) {
				rubrik.add(t);
			}
		}
		if (rubrik.size() > 0) {
			for (TreePath einzelRubrik : rubrik) {
				for (@SuppressWarnings("rawtypes")
				Enumeration e = ((DefaultMutableTreeNode) einzelRubrik
						.getLastPathComponent()).children(); e
						.hasMoreElements();) {
					pfad.add(einzelRubrik.pathByAddingChild(e.nextElement()));
				}
			}
			pfad.removeAll(rubrik);
		}
		// Knoten zunächst aus dem Baum alleSensoren entfernen und ggf. leere
		// Rubriken entfernen
		Vector<DefaultMutableTreeNode> transfer = new Vector<DefaultMutableTreeNode>();
		for (TreePath t : pfad) {
			DefaultMutableTreeNode knoten = (DefaultMutableTreeNode) t
					.getLastPathComponent();
			DefaultMutableTreeNode parent = (DefaultMutableTreeNode) knoten
					.getParent();
			knoten.removeFromParent();
			if (parent.getChildCount() == 0)
				parent.removeFromParent();
			transfer.add(knoten);
		}
		// Knoten in den Baum vorhandeneSensoren einfügen und ggf. Rubriken
		// erstellen
		for (DefaultMutableTreeNode knoten : transfer) {
			String rubrikName = ((GoIOSensorInfo) knoten.getUserObject())
					.getRubrik();
			DefaultMutableTreeNode rubrikKnoten = findeRubrikKnoten(
					(DefaultMutableTreeNode) vorhandeneSensoren.getModel()
							.getRoot(), rubrikName, true);
			rubrikKnoten.add(knoten);
		}
		updateTrees();
	}

	@Action
	public void entfernen() {
		Vector<TreePath> pfad = new Vector<TreePath>(
				Arrays.asList(vorhandeneSensoren.getSelectionPaths()));
		if (DEBUG)
			System.out.println("[SensorenVernier] - entfernen aufgerufen...");
		if (pfad == null || pfad.size() == 0)
			return;
		// Auswahl normalisieren, Sensoren statt Rubriken einsetzen
		Vector<TreePath> rubrik = new Vector<TreePath>();
		for (TreePath t : pfad) {
			if (!((DefaultMutableTreeNode) t.getLastPathComponent()).isLeaf()) {
				rubrik.add(t);
			}
		}
		if (rubrik.size() > 0) {
			for (TreePath einzelRubrik : rubrik) {
				for (@SuppressWarnings("rawtypes")
				Enumeration e = ((DefaultMutableTreeNode) einzelRubrik
						.getLastPathComponent()).children(); e
						.hasMoreElements();) {
					pfad.add(einzelRubrik.pathByAddingChild(e.nextElement()));
				}
			}
			pfad.removeAll(rubrik);
		}
		// Knoten zunächst aus dem Baum alleSensoren entfernen und ggf. leere
		// Rubriken entfernen
		Vector<DefaultMutableTreeNode> transfer = new Vector<DefaultMutableTreeNode>();
		for (TreePath t : pfad) {
			DefaultMutableTreeNode knoten = (DefaultMutableTreeNode) t
					.getLastPathComponent();
			DefaultMutableTreeNode parent = (DefaultMutableTreeNode) knoten
					.getParent();
			knoten.removeFromParent();
			if (parent.getChildCount() == 0)
				parent.removeFromParent();
			transfer.add(knoten);
		}
		// Knoten in den Baum vorhandeneSensoren einfügen und ggf. Rubriken
		// erstellen
		for (DefaultMutableTreeNode knoten : transfer) {
			String rubrikName = ((GoIOSensorInfo) knoten.getUserObject())
					.getRubrik();
			DefaultMutableTreeNode rubrikKnoten = findeRubrikKnoten(
					(DefaultMutableTreeNode) alleSensoren.getModel().getRoot(),
					rubrikName, true);
			rubrikKnoten.add(knoten);
		}
		updateTrees();
	}

	@Action
	void alleHinzufuegen() {
		if (DEBUG)
			System.out
					.println("[SensorenVernier] - alle hinzufügen aufgerufen...");
		vorhandeneSensoren.setModel(GoIOSensorFactory
				.createSensorTreeModel(new DefaultMutableTreeNode(resourceMap
						.getString("vorhandeneSensoren.title"))));
		alleSensoren.setModel(new DefaultTreeModel(new DefaultMutableTreeNode(
				resourceMap.getString("alleSensoren.title"))));
	}

	@Action
	public void alleEntfernen() {
		if (DEBUG)
			System.out
					.println("[SensorenVernier] - alle entfernen aufgerufen...");
		alleSensoren.setModel(GoIOSensorFactory
				.createSensorTreeModel(new DefaultMutableTreeNode(resourceMap
						.getString("vorhandeneSensoren.title"))));
		vorhandeneSensoren.setModel(new DefaultTreeModel(
				new DefaultMutableTreeNode(resourceMap
						.getString("alleSensoren.title"))));
	}

}
