package ghmessen.dialoge.einstellungen;

import ghmessen.adapter.AdapterListe;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.efm.AdapterEFM;
import ghmessen.adapter.event.AdapterAddedEvent;
import ghmessen.adapter.event.AdapterChangedEvent;
import ghmessen.adapter.event.AdapterEventListener;
import ghmessen.adapter.event.AdapterRemovedEvent;
import ghmessen.util.CursorTk;

import java.awt.event.ActionEvent;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.ListSelectionModel;

import ld.api.ICASSY;
import ld.api.Pids;
import ld.api.UsbSerial;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.observablecollections.ObservableMap;
import org.jdesktop.observablecollections.ObservableMapListener;

public class EinstellungenAllgemeinPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5573560033306047998L;
	private static final boolean DEBUG = false;
	private boolean firePC = true;
	private static ActionMap actionMap;
	private static ResourceMap resourceMap;

	private static String efm = "Elektrofeldmeter PHYWE (1150010)";
	private static String[] messInterfaces = new String[] { "aus", "CASSY",
			"ASCII (einstellbar)", "Waage", "VideoCom (337 47)",
			"IR Position Detector (332 11)", "MFA 2001 (727 230)",
			"MetraHit (531 28 - 531 - 20)",
			"Temperaturmessgerät (666 209/666 454)",
			"Digitales Spektralphotometer (667 3491)",
			"Digitales Spektralphotometer (667 3493)", "Data Logger (666 252)",
			"pH-Meter (666 221)", "Conductivity-Meter (666 222)",
			"Lux-Meter (666 223 / 666 230)", "O2-Meter (666 224)",
			"Photo-Meter (666 225)", "Sound-Level-Meter (666 231)",
			"CO2-CO-CH4-Meter (666 232)", "Optical Power-Meter (736 435)",
			"Antennendrehtisch (737 405)",
			efm};

	private AdapterListe al;
	private DefaultListModel<String> interfaceListe;
	private PortModel[] portListe;
	
	private JButton testComAuswahl;

	private class PortModel extends DefaultComboBoxModel<String> {
		private static final long serialVersionUID = 1L;

		public PortModel() {
			super();
			if (al != null && al.getMonitor() != null) {
				for (String s : al.getMonitor().getPortListe())
					this.addElement(s);
			}
		}
	}

	public EinstellungenAllgemeinPanel(AdapterListe al) {
		super();
		this.al = al;
		interfaceListe = new DefaultListModel<String>();
		if (this.al != null) {
			refreshListe(null);
			this.al.addObservableMapListener(new ObservableMapListener() {
				@SuppressWarnings("rawtypes")
				@Override
				public void mapKeyAdded(ObservableMap map, Object key) {
					refreshListe(null);
				}

				@SuppressWarnings("rawtypes")
				@Override
				public void mapKeyRemoved(ObservableMap map, Object key,
						Object value) {
					refreshListe(null);
				}

				@SuppressWarnings("rawtypes")
				@Override
				public void mapKeyValueChanged(ObservableMap map, Object key,
						Object lastValue) {
					refreshListe(null);
				}

			});
			if (this.al.getMonitor() != null)
				this.al.getMonitor().addAdapterEventListener(
						new AdapterEventListener() {
							@Override
							public void adapterAdded(AdapterAddedEvent evt) {
								if (!evt.getName().startsWith("ttyUSB"))
									return;
								for (PortModel p : portListe)
									if (p.getIndexOf(evt.getName()) < 0)
										p.addElement(evt.getName());
							}

							@Override
							public void adapterRemoved(AdapterRemovedEvent evt) {
								if (!evt.isPortRemoved()
										|| !evt.getName().startsWith("ttyUSB"))
									return;
								String portName = evt.getDevice().getName();
								for (int index = 0; index < port.length; index++) {
									if (portName.equals(portListe[index]
											.getSelectedItem())
											&& !"aus".equals(adapter[index]
													.getSelectedItem()))
										adapter[index].setSelectedItem("aus");
									portListe[index].removeElement(portName);
								}
							}

							@Override
							public void adapterChanged(AdapterChangedEvent evt) {
								return;
							}
						});
		}
		initComponents();
	}

	@SuppressWarnings("unchecked")
	private void initComponents() {
		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(EinstellungenAllgemeinPanel.class, this);
		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(EinstellungenAllgemeinPanel.class);
		testComAuswahl = new JButton(actionMap.get("comAuswahl"));
		testComAuswahl.setText("Test Port 0");
		jScrollPane2 = new javax.swing.JScrollPane();
		refreshListe = new javax.swing.JButton(actionMap.get("refreshListe"));
		refreshListe.setEnabled(false);
		aktualisiere = new javax.swing.JButton();
		aktualisiere.setText(resourceMap.getString("aktualisiere.text"));
		aktualisiere.setEnabled(false);
		portListe = new PortModel[4];
		for (int i = 0; i < adapter.length; i++) {
			adapter[i] = new javax.swing.JComboBox<String>();
			adapter[i].setAction(actionMap.get("comAuswahl"));
			adapter[i].setFont(resourceMap.getFont("Dialog-Plain-10")); // NOI18N
			adapter[i]
					.setModel(new DefaultComboBoxModel<String>(messInterfaces));
			port[i] = new JComboBox<String>(portListe[i] = new PortModel());
			port[i].setFont(resourceMap.getFont("Dialog-Plain-10")); // NOI18N
		}
		comLabel = new javax.swing.JLabel(
				resourceMap.getString("comLabel.text"));
		spracheLabel = new javax.swing.JLabel(
				resourceMap.getString("spracheLabel.text"));
		sprache = new javax.swing.JComboBox<String>(
				new javax.swing.DefaultComboBoxModel<String>(
						new String[] { "Deutsch" }));
		modulFeldLabel = new javax.swing.JLabel(
				resourceMap.getString("modulFeldLabel.text"));
		modulFeld = new JList<String>(interfaceListe);
		modulFeld.setEnabled(false);
		modulFeld.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		modulFeld.setBorder(javax.swing.BorderFactory
				.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
		modulFeld.setName("jtCASSY_Module"); // NOI18N
		jScrollPane2.setViewportView(modulFeld);

		javax.swing.GroupLayout gl_jDE_Allgemein = new javax.swing.GroupLayout(
				this);
		gl_jDE_Allgemein
				.setHorizontalGroup(gl_jDE_Allgemein
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_jDE_Allgemein
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jDE_Allgemein
														.createParallelGroup(
																Alignment.LEADING,
																false)
														.addGroup(
																gl_jDE_Allgemein
																		.createSequentialGroup()
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.LEADING,
																								false)
																						.addComponent(
																								port[3],
																								0,
																								GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								port[2],
																								0,
																								GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addGroup(
																								gl_jDE_Allgemein
																										.createSequentialGroup()
																										.addGap(31)
																										.addComponent(
																												spracheLabel,
																												GroupLayout.PREFERRED_SIZE,
																												92,
																												GroupLayout.PREFERRED_SIZE))
																						.addComponent(
																								port[0],
																								0,
																								129,
																								Short.MAX_VALUE)
																						.addComponent(
																								port[1],
																								0,
																								GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE))
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addComponent(
																								sprache,
																								GroupLayout.PREFERRED_SIZE,
																								188,
																								GroupLayout.PREFERRED_SIZE)
																						.addGroup(
																								gl_jDE_Allgemein
																										.createParallelGroup(
																												Alignment.LEADING,
																												false)
																										.addComponent(
																												adapter[3],
																												0,
																												234,
																												Short.MAX_VALUE)
																										.addComponent(
																												adapter[2],
																												0,
																												234,
																												Short.MAX_VALUE)
																										.addComponent(
																												adapter[1],
																												0,
																												234,
																												Short.MAX_VALUE)
																										.addComponent(
																												adapter[0],
																												0,
																												0,
																												Short.MAX_VALUE))))
														.addComponent(
																comLabel,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))
										.addGroup(
												gl_jDE_Allgemein
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_jDE_Allgemein
																		.createSequentialGroup()
																		.addGap(58)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.TRAILING,
																								false)
																						.addComponent(
																								refreshListe,
																								Alignment.LEADING,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)
																						.addComponent(
																								aktualisiere,
																								Alignment.LEADING,
																								GroupLayout.DEFAULT_SIZE,
																								252,
																								Short.MAX_VALUE)
																						.addComponent(
																								testComAuswahl,
																								Alignment.LEADING,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								Short.MAX_VALUE)))
														.addGroup(
																gl_jDE_Allgemein
																		.createSequentialGroup()
																		.addGap(26)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addComponent(
																								modulFeldLabel,
																								GroupLayout.DEFAULT_SIZE,
																								286,
																								Short.MAX_VALUE)
																						.addComponent(
																								jScrollPane2,
																								GroupLayout.DEFAULT_SIZE,
																								286,
																								Short.MAX_VALUE))))
										.addContainerGap()));
		gl_jDE_Allgemein
				.setVerticalGroup(gl_jDE_Allgemein
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_jDE_Allgemein
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jDE_Allgemein
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(comLabel)
														.addComponent(
																modulFeldLabel))
										.addGroup(
												gl_jDE_Allgemein
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_jDE_Allgemein
																		.createSequentialGroup()
																		.addGap(22)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								adapter[0],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								port[0],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE))
																		.addGap(18)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								adapter[1],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								port[1],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE))
																		.addGap(18)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								adapter[2],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								port[2],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE))
																		.addGap(18)
																		.addGroup(
																				gl_jDE_Allgemein
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								adapter[3],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								port[3],
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)))
														.addGroup(
																gl_jDE_Allgemein
																		.createSequentialGroup()
																		.addGap(18)
																		.addComponent(
																				jScrollPane2,
																				GroupLayout.PREFERRED_SIZE,
																				149,
																				GroupLayout.PREFERRED_SIZE)))
										.addGap(13)
										.addComponent(refreshListe)
										.addGap(18)
										.addGroup(
												gl_jDE_Allgemein
														.createParallelGroup(
																Alignment.BASELINE)
														.addComponent(
																spracheLabel)
														.addComponent(
																sprache,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																aktualisiere))
										.addComponent(testComAuswahl)
										.addContainerGap(31, Short.MAX_VALUE)));
		setLayout(gl_jDE_Allgemein);

	}

	private javax.swing.JButton refreshListe;
	private javax.swing.JButton aktualisiere;
	@SuppressWarnings("rawtypes")
	private JComboBox[] adapter = new JComboBox[4], port = new JComboBox[4];
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JComboBox<String> sprache;
	private javax.swing.JLabel comLabel;
	private javax.swing.JLabel spracheLabel;
	private javax.swing.JLabel modulFeldLabel;
	private JList<String> modulFeld;

	@Action
	public void refreshListe(ActionEvent e) {
		interfaceListe.clear();
		for (IAdapter a : al.values())
			if (a instanceof ICASSY)
				interfaceListe.addElement(a.getName() + "(" + a.getNr() + "|"
						+ a.getDevice() + ")");
	}

	@SuppressWarnings("unchecked")
	private String leseAuswahl(ActionEvent e) {
		if (e.getSource() instanceof JComboBox) {
			return (String) ((JComboBox<String>) e.getSource()).getSelectedItem();
		} else
			return "aus";
	}

	private int indexOf(Object source) {
		try {
			for (int i = 0; i < port.length; i++)
				if (adapter[i] == source || port[i] == source)
					return i;
			return -1;
		} catch (NullPointerException e) {
			return -1;
		}
	}

	private void adjustLists(int index, boolean gewaehlt) {
		String portName = (String) port[index].getSelectedItem();
		for (int portNr = 0; portNr < port.length; portNr++) {
			if (index == portNr)
				continue;
			if (gewaehlt)
				portListe[portNr].removeElement(portName);
			else
				portListe[portNr].addElement(portName);
		}
	}

	@Action
	public void comAuswahl(ActionEvent e) {
		if (!firePC)
			return;
		if(e.getSource() == testComAuswahl)
			e.setSource(adapter[0]);
		int index = indexOf(e.getSource());
		String auswahl = leseAuswahl(e);
		String portName = (String) port[index].getSelectedItem();
		DefaultListModel<String> cl;
		if (modulFeld.getModel() instanceof DefaultListModel)
			cl = (DefaultListModel<String>) modulFeld.getModel();
		else {
			cl = new DefaultListModel<String>();
			modulFeld.setModel(cl);
		}
		if (DEBUG)
			System.err
					.println("[EinstellungenAllgemeinPanel] - comAuswahl: Auswahl("
							+ auswahl + ") Port(" + portName + ")");
		CursorTk.startWaitCursor(this);
		if (auswahl.equals("CASSY")) {
			int vendor = UsbSerial.getVendorId();
			int product = Pids.CASSY.getValue();
			this.al.getMonitor().deviceAdapterChosen(portName, vendor, product);
		} else if (auswahl.equals(efm)) {
			int vendor = AdapterEFM.vendorID;
			//FIXME: read productID
			int product = 1150010;
			this.al.getMonitor().deviceAdapterChosen(portName, vendor, product);
		} else if (auswahl.equals("aus")) {
			this.al.getMonitor().deviceAdapterChosen(portName, -1, -1);
		}
		adjustLists(index, !auswahl.equals("aus"));
		port[index].setEnabled(auswahl.equals("aus"));
		CursorTk.stopWaitCursor(this);
	}
}
