/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge.einstellungen;

import ghmessen.dialoge.einstellungen.darstellung.AchsenPanel;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Darstellungsliste;

import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.List;
import java.util.Vector;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.MutableComboBoxModel;

import org.jdesktop.application.Action;
import org.jdesktop.observablecollections.ObservableList;
import org.jdesktop.observablecollections.ObservableListListener;

public class EinstellungenDarstellungPanel extends javax.swing.JPanel implements
		ObservableListListener {
	private static final long serialVersionUID = -4131673090668302308L;
	private static final boolean DEBUG = false;
	private Darstellungsliste ansicht;
	private Darstellung aktuelleTabelle;
	private Vector<String> ansichtModel;
	private ActionMap actionMap;

	public EinstellungenDarstellungPanel(Darstellungsliste ansicht) {
		super();
		if (DEBUG)
			System.out
					.println("[EinstellungenDarstellungPanel] - Konstruktor.");
		this.ansicht = ansicht;
		this.aktuelleTabelle = null;
		this.ansichtModel = new Vector<String>();
		if (ansicht != null) {
			ansicht.addObservableListListener(this);
			if (ansicht.size() > 0)
				aktuelleTabelle = this.ansicht.get(0);
		}
		initComponents();
		achsen.setDarstellung(aktuelleTabelle);
		adjustComponents();
	}

	private void initComponents() {

		actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(EinstellungenDarstellungPanel.class, this);

		jLabel10 = new javax.swing.JLabel();
		darstellung = new javax.swing.JComboBox<String>(new DefaultComboBoxModel<String>(
				ansichtModel));
		darstellung.setEditable(true);
		darstellungFocusAdapter = new FocusAdapter() {
			@Override
			public void focusGained(FocusEvent e) {
				String[] namen = ansicht.getNamen();
				MutableComboBoxModel<String> mcbm = (MutableComboBoxModel<String>) darstellung
						.getModel();
				if (DEBUG)
					System.out
							.println("[EinstellungenDarstellungPanel] - darstellung.focusGained: alt("
									+ ansichtModel + ") neu(" + namen + ")");
				int alt = ansichtModel.size();
				int neu = namen.length;
				int set = Math.min(alt, neu);
				for (int i = 0; i < set; i++) {
					if (!namen[i].equals(ansichtModel.get(i))) {
						mcbm.removeElementAt(i);
						mcbm.insertElementAt(namen[i], i);
					}
				}
				for (int i = neu; i < alt; i++)
					mcbm.removeElementAt(neu);
				for (int i = alt; i < neu; i++)
					mcbm.addElement(namen[i]);
			}
		};
		darstellung.addFocusListener(darstellungFocusAdapter);
		darstellung.getEditor().getEditorComponent()
				.addFocusListener(darstellungFocusAdapter);

		darstellung_neu = new javax.swing.JButton();
		darstellung_neu.setAction(actionMap.get("neueDarstellung"));
		darstellung_weg = new javax.swing.JButton();
		darstellung_weg.setAction(actionMap.get("entferneDarstellung"));
		achsen = new AchsenPanel(aktuelleTabelle);

		setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
		setName("jDE_Darstellung"); // NOI18N

		jLabel10.setText("Darstellung auswählen"); // NOI18N
		jLabel10.setName("jLabel10"); // NOI18N

		darstellung.setName("darstellung"); // NOI18N
		darstellung.setEditable(true);
		darstellung.setAction(actionMap.get("setzeDarstellung"));

		darstellung_neu.setText("neue Darstellung"); // NOI18N
		darstellung_neu.setName("darstellung_neu"); // NOI18N

		darstellung_weg.setText("Darstellung löschen"); // NOI18N
		darstellung_weg.setName("darstellung_weg"); // NOI18N

		javax.swing.GroupLayout gl_jDE_Darstellung = new javax.swing.GroupLayout(
				this);
		setLayout(gl_jDE_Darstellung);
		gl_jDE_Darstellung
				.setHorizontalGroup(gl_jDE_Darstellung
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jDE_Darstellung
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jDE_Darstellung
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																achsen,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addGroup(
																gl_jDE_Darstellung
																		.createSequentialGroup()
																		.addComponent(
																				jLabel10)
																		.addGap(16,
																				16,
																				16)
																		.addComponent(
																				darstellung,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				154,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addGap(18,
																				18,
																				18)
																		.addComponent(
																				darstellung_neu)
																		.addGap(18,
																				18,
																				18)
																		.addComponent(
																				darstellung_weg)))
										.addContainerGap()));
		gl_jDE_Darstellung
				.setVerticalGroup(gl_jDE_Darstellung
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_jDE_Darstellung
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_jDE_Darstellung
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																darstellung,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(jLabel10)
														.addComponent(
																darstellung_neu)
														.addComponent(
																darstellung_weg))
										.addGap(18, 18, 18)
										.addComponent(
												achsen,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												340, Short.MAX_VALUE)
										.addContainerGap()));
	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JLabel jLabel10;
	private AchsenPanel achsen;
	public javax.swing.JButton darstellung_neu;
	private javax.swing.JButton darstellung_weg;
	private javax.swing.JComboBox<String> darstellung;
	private FocusAdapter darstellungFocusAdapter;

	@Action
	public void neueDarstellung() {
		aktuelleTabelle = ansicht.neueDarstellung();
		((MutableComboBoxModel<String>) darstellung.getModel())
				.addElement(aktuelleTabelle.getName());
		darstellung.getModel().setSelectedItem(aktuelleTabelle.getName());
	}

	@Action
	public void entferneDarstellung() {
		String alt = (String) darstellung.getModel().getSelectedItem();
		if (alt == null)
			return;
		ansicht.remove(aktuelleTabelle);
		aktuelleTabelle = null;
		((MutableComboBoxModel<String>) darstellung.getModel()).removeElement(alt);
	}

	public void waehleDarstellung(Darstellung ds) {
		int index = ansicht.indexOfName(ds.getName());
		if (index > -1) {
			aktuelleTabelle = ds;
			darstellung.getModel().setSelectedItem(ds.getName());
			achsen.setDarstellung(aktuelleTabelle);
			adjustComponents();
		}
	}

	@Action
	public void setzeDarstellung(ActionEvent e) {
		if (aktuelleTabelle != null && e != null
				&& e.getActionCommand() == "comboBoxEdited") {
			String neu = (String) darstellung.getSelectedItem();
			aktuelleTabelle.setName(neu);
			int index = ansicht.indexOf(aktuelleTabelle);
			if (DEBUG)
				System.out
						.println("[EinstellungenDarstellungPanel] - setzeDarstellung: comboBoxEdited neu("
								+ neu + ") index(" + index + ")");
			((MutableComboBoxModel<String>) darstellung.getModel())
					.removeElementAt(index);
			((MutableComboBoxModel<String>) darstellung.getModel()).insertElementAt(
					neu, index);
			darstellung.setSelectedIndex(index);
		} else {
			int index = ansicht.indexOfName((String) darstellung.getModel()
					.getSelectedItem());
			if (index >= 0 && index < ansicht.getSize()) {
				aktuelleTabelle = ansicht.get(index);
				achsen.setDarstellung(aktuelleTabelle);
			}
			adjustComponents();
		}
	}

	private void adjustComponents() {
		darstellung_neu.setEnabled(ansicht != null && ansicht.size() < 10);
		darstellung_weg.setEnabled(ansicht.size() > 1);
		darstellung.setEnabled(ansicht.size() > 0);
	}

	/********************************************************************
	 * ObservableListListener: DARSTELLUNGSLISTE
	 ********************************************************************/
	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsAdded(ObservableList list, int index, int length) {
		if (list instanceof Darstellungsliste) {
			Darstellungsliste dl = (Darstellungsliste) list;
			int aktuell = dl.indexOf(aktuelleTabelle);
			if (aktuell > -1) {
				aktuelleTabelle = dl.get(aktuell);
				achsen.setDarstellung(aktuelleTabelle);
			} else
				setzeDarstellung(null);
			adjustComponents();
		}
	}

	@Override
	public void listElementsRemoved(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			@SuppressWarnings("rawtypes") List oldElements) {
		if (list instanceof Darstellungsliste) {
			Darstellungsliste dl = (Darstellungsliste) list;
			int aktuell = dl.indexOf(aktuelleTabelle);
			if (aktuell > -1) {
				achsen.setDarstellung(aktuelleTabelle);
			} else
				setzeDarstellung(null);
			adjustComponents();
		}
	}

	@Override
	public void listElementReplaced(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			Object oldElement) {
		if (list instanceof Darstellungsliste) {
			Darstellungsliste dl = (Darstellungsliste) list;
			int aktuell = dl.indexOf(aktuelleTabelle);
			if (aktuell > -1) {
				achsen.setDarstellung(aktuelleTabelle);
			} else
				setzeDarstellung(null);
			adjustComponents();
		}

	}

	@Override
	public void listElementPropertyChanged(
			@SuppressWarnings("rawtypes") ObservableList list, int index) {
		if (list instanceof Darstellungsliste) {
			Darstellungsliste dl = (Darstellungsliste) list;
			if (aktuelleTabelle == dl.get(index)) {
				achsen.setDarstellung(aktuelleTabelle);
				adjustComponents();
			}
		}

	}
}
