/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge.einstellungen;

import static ghmessen.groesse.Art.Ableitung;
import static ghmessen.groesse.Art.FFT;
import static ghmessen.groesse.Art.Formel;
import static ghmessen.groesse.Art.Histogramm;
import static ghmessen.groesse.Art.Integral;
import static ghmessen.groesse.Art.Konstante;
import static ghmessen.groesse.Art.Mittelwert;
import static ghmessen.groesse.Art.Parameter;
import ghmessen.GHMessen;
import ghmessen.components.EinheitFormattedTextField;
import ghmessen.components.FormelFormattedTextField;
import ghmessen.components.SymbolFormattedTextField;
import ghmessen.groesse.Art;
import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Einheit;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.ObservableObjectListListener;
import ghmessen.util.Util;

import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.MutableComboBoxModel;
import javax.swing.text.DefaultFormatter;

import org.jdesktop.application.Action;
import org.jdesktop.observablecollections.ObservableList;

public class EinstellungenGroessePanel extends JPanel implements
		ObservableObjectListListener {
	private static final long serialVersionUID = -2885996641521502884L;
	private static final boolean DEBUG = false;
	public Groessenliste gl;
	private Groesse aktuelleGroesse;
	private Vector<String> bezugModel;
	private Vector<String> groesseModel;
	private boolean firePC;

	private void writeObject(ObjectOutputStream out) throws IOException {
		throw new NotSerializableException(
				"EinstellungenGroessePanel kann nicht gespeichert werden.");
	}

	public EinstellungenGroessePanel(Groessenliste messGroessen) {
		super();
		firePC = true;
		bezugModel = new Vector<String>();
		groesseModel = new Vector<String>();
		this.gl = messGroessen;
		aktuelleGroesse = null;
		if (gl != null) {
			gl.addObservableListListener(this);
			if (gl.size() > 0)
				aktuelleGroesse = gl.get(0);
		}
		initComponents();
		aktualisiereGUI();
		adjustComponents();
	}

	private void initComponents() {

		ActionMap actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(EinstellungenGroessePanel.class, this);
		art = new ButtonGroup();
		groesseFocusAdapter = new FocusAdapter() {
			@Override
			public void focusGained(FocusEvent e) {
				String[] namen = gl.getNamen();
				MutableComboBoxModel<String> mcbm = (MutableComboBoxModel<String>) groesse
						.getModel();
				if (DEBUG)
					System.out
							.println("[EinstellungenGroessePanel] - groesse.focusGained: alt("
									+ groesseModel + ") neu(" + namen + ")");
				int alt = groesseModel.size();
				int neu = namen.length;
				int set = Math.min(alt, neu);
				for (int i = 0; i < set; i++) {
					if (!namen[i].equals(groesseModel.get(i))) {
						mcbm.removeElementAt(i);
						mcbm.insertElementAt(namen[i], i);
					}
				}
				for (int i = neu; i < alt; i++)
					mcbm.removeElementAt(neu);
				for (int i = alt; i < neu; i++)
					mcbm.addElement(namen[i]);
			}
		};
		groesse = new javax.swing.JComboBox<String>(new DefaultComboBoxModel<String>(
				groesseModel));
		groesse.setEditable(true);
		groesse.setAction(actionMap.get("setzeGroesse"));
		// abhängig von Editable wird der eine bzw. der andere Listener
		// ausgelöst.
		groesse.addFocusListener(groesseFocusAdapter);
		groesse.getEditor().getEditorComponent()
				.addFocusListener(groesseFocusAdapter);
		lblgroesse = new javax.swing.JLabel();
		groesse_neu = new javax.swing.JButton();
		groesse_neu.setAction(actionMap.get("neueGroesse"));
		groesse_weg = new javax.swing.JButton();
		groesse_weg.setAction(actionMap.get("entferneGroesse"));
		eigenschaften = new javax.swing.JPanel();
		konstante = new javax.swing.JRadioButton();
		konstante.setEnabled(false);
		konstante.setAction(actionMap.get("art_konstante"));
		konstante.putClientProperty("art", Konstante);
		art.add(konstante);
		parameter = new javax.swing.JRadioButton();
		parameter.setEnabled(false);
		parameter.setAction(actionMap.get("art_parameter"));
		parameter.putClientProperty("art", Parameter);
		art.add(parameter);
		parameter.setSelected(true);
		formel = new javax.swing.JRadioButton();
		formel.setEnabled(false);
		formel.setAction(actionMap.get("art_formel"));
		formel.putClientProperty("art", Formel);
		art.add(formel);
		DefaultFormatter formeltermFormat = new DefaultFormatter();
		formeltermFormat.setOverwriteMode(false);
		formelterm = new FormelFormattedTextField(formeltermFormat);
		if (gl != null && firePC) {
			int aktuell = gl.indexOf(aktuelleGroesse);
			if (aktuell > -1) {
				String[] symbole = Arrays.copyOfRange(gl.getTextSymbole(), 0,
						aktuell);
				formelterm.setKonstanten(symbole);
			}
		}
		formelterm.setName(aktuelleGroesse != null ? aktuelleGroesse
				.getSymbol() : "");
		formelterm.setValue(aktuelleGroesse != null ? aktuelleGroesse
				.getFormel() : "0");
		formelterm.setEnabled(false);
		formelterm.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC) {
					aktuelleGroesse.setFormel((String) formelterm.getValue());
					aktuelleGroesse.setFormelterm(formelterm.getEvaluator());
				}
			}
		});
		konstante_wert = new JFormattedTextField(Util.doubleFormat);
		konstante_wert.setValue(aktuelleGroesse != null ? aktuelleGroesse
				.getWert() : 0.0);
		konstante_wert.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					aktuelleGroesse.setWert(((Number) e.getNewValue())
							.doubleValue());
			}
		});
		ableitung = new javax.swing.JRadioButton();
		ableitung.setEnabled(false);
		ableitung.setAction(actionMap.get("art_ableitung"));
		ableitung.putClientProperty("art", Ableitung);
		art.add(ableitung);
		integral = new javax.swing.JRadioButton();
		integral.setEnabled(false);
		integral.setAction(actionMap.get("art_integral"));
		integral.putClientProperty("art", Integral);
		art.add(integral);
		fft = new javax.swing.JRadioButton();
		fft.setEnabled(false);
		fft.setAction(actionMap.get("art_fft"));
		fft.putClientProperty("art", FFT);
		art.add(fft);
		mittelwert = new javax.swing.JRadioButton();
		mittelwert.setEnabled(false);
		mittelwert.setAction(actionMap.get("art_mittelwert"));
		mittelwert.putClientProperty("art", Mittelwert);
		art.add(mittelwert);
		mittelwert_intervall = new JFormattedTextField(Util.doubleFormat);
		mittelwert_intervall.setEnabled(false);
		mittelwert_intervall.setValue(aktuelleGroesse != null ? aktuelleGroesse
				.getIntervall() : 10.0);
		mittelwert_intervall
				.addPropertyChangeListener("value", new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent e) {
						if (aktuelleGroesse != null && firePC)
							aktuelleGroesse.setIntervall(((Number) e
									.getNewValue()).doubleValue());
					}
				});

		mittelwert_einheit = new javax.swing.JComboBox<String>();
		mittelwert_einheit.setEnabled(false);
		mittelwert_einheit.setAction(actionMap.get("setzeMWEinhei"));
		histogramm = new javax.swing.JRadioButton();
		histogramm.setEnabled(false);
		histogramm.setAction(actionMap.get("art_histogramm"));
		histogramm.putClientProperty("art", Histogramm);
		art.add(histogramm);
		histogramm_d = new JFormattedTextField(Util.doubleFormat);
		histogramm_d.setEnabled(false);
		histogramm_d.setValue(aktuelleGroesse != null ? aktuelleGroesse
				.getDelta() : 1.0);
		histogramm_d.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					aktuelleGroesse.setDelta(((Number) e.getNewValue())
							.doubleValue());
			}
		});

		lblhistogramm_d = new javax.swing.JLabel();
		lblhistogramm_d.setEnabled(false);
		bezug = new javax.swing.JComboBox<String>(new DefaultComboBoxModel<String>(bezugModel));
		bezug.setEnabled(false);
		bezug.setToolTipText("Setze den Bezug für verschiedene Größenarten (Ableitung, ...)");
		bezug.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				if (DEBUG)
					System.out
							.println("[EinstellungenGroessePanel] - bezug.focusGained.");
				String auswahl = (String) bezug.getSelectedItem();
				int aktuell = gl.indexOf(aktuelleGroesse);
				if (aktuell > -1) {
					String[] symbole = Arrays.copyOfRange(gl.getSymbole(), 0,
							aktuell);
					MutableComboBoxModel<String> mcbm = (MutableComboBoxModel<String>) bezug
							.getModel();
					int alt = bezugModel.size();
					int neu = symbole.length;
					int set = Math.min(alt, neu);
					for (int i = 0; i < set; i++) {
						if (!symbole[i].equals(bezugModel.get(i))) {
							mcbm.removeElementAt(i);
							mcbm.insertElementAt(symbole[i], i);
						}
					}
					for (int i = neu; i < alt; i++)
						mcbm.removeElementAt(neu);
					for (int i = alt; i < neu; i++)
						mcbm.addElement(symbole[i]);
					bezug.setSelectedItem(auswahl);
				}
			}

			public void focusLost(FocusEvent e) {
				if (DEBUG)
					System.out
							.println("[EinstellungenGroessePanel] - bezug.focusLost.");
				if (aktuelleGroesse != null) {
					String symbol = (String) bezug.getSelectedItem();
					Groesse g = Util.findeGroesse(gl, symbol);
					aktuelleGroesse.setBezug(g);
					if (aktuelleGroesse.getArt() != Art.Histogramm
							|| GHMessen.getApplication().getMainView().ansicht == null
							|| GHMessen.getApplication().getMainView().ansicht
									.getHistogramm() == null)
						return;
					GHMessen.getApplication().getMainView().ansicht
							.getHistogramm().setY(Darstellung.XACHSE,
									aktuelleGroesse.getBezug2());
				}
			}
		});
		lblbezug = new javax.swing.JLabel();
		lblbezug.setEnabled(false);
		symbol = new SymbolFormattedTextField(gl);
		symbol.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					aktuelleGroesse.setSymbol((String) e.getNewValue());
			}
		});
		symbol.setEnabled(false);
		symbol.setToolTipText("Eindeutiges Symbol ohne Leerzeichen.");

		lblsymbol = new javax.swing.JLabel();
		lblsymbol.setEnabled(false);
		lblsymbol.setLabelFor(symbol);
		lbleinheit = new javax.swing.JLabel();
		lbleinheit.setEnabled(false);
		einheit = new EinheitFormattedTextField();
		einheit.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					try {
						aktuelleGroesse.setEinheitString((String) e
								.getNewValue());
					} catch (IllegalArgumentException ee) {
						if (DEBUG)
							System.out
									.println("Einstellungen - Größe - einheit: Fehler propertyChange("
											+ e.getOldValue()
											+ " -> "
											+ e.getNewValue() + ")");
					}
			}
		});
		einheit.setEnabled(false);
		einheit.setToolTipText("Es muss eine der vordefinierten Einheiten verwendet werden.");
		lblvon = new javax.swing.JLabel();
		lblvon.setEnabled(false);
		von = new JFormattedTextField(Util.doubleFormat);
		von.setEnabled(false);
		von.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					aktuelleGroesse.setVon(((Number) e.getNewValue())
							.doubleValue());
			}
		});

		lblbis = new javax.swing.JLabel();
		lblbis.setEnabled(false);
		bis = new JFormattedTextField(Util.doubleFormat);
		bis.setEnabled(false);
		bis.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					aktuelleGroesse.setBis(((Number) e.getNewValue())
							.doubleValue());
			}
		});

		lblstellen = new javax.swing.JLabel();
		lblstellen.setEnabled(false);
		stellen = new JFormattedTextField(Util.createIntegerMinMaxFormatter(0,
				20));
		stellen.setEnabled(false);
		stellen.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (aktuelleGroesse != null && firePC)
					aktuelleGroesse.setStellen(((Number) e.getNewValue())
							.intValue());
			}
		});

		setName("jDE_Formel"); // NOI18N

		lblgroesse.setText("G\u00F6\u00DFe ausw\u00E4hlen"); // NOI18N
		lblgroesse.setName("jLabel2"); // NOI18N

		groesse_neu.setText("neue Gr\u00F6\u00DFe"); // NOI18N
		groesse_neu.setName("groesse_neu"); // NOI18N

		groesse_weg.setText("Gr\u00F6\u00DFe l\u00F6schen"); // NOI18N
		groesse_weg.setName("groesse_weg"); // NOI18N
		groesse_weg.setEnabled(false);

		eigenschaften.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Eigenschaften")); // NOI18N
		eigenschaften.setName("eigenschaften"); // NOI18N

		mittelwert_einheit.setModel(new DefaultComboBoxModel<String>(new String[] {
				"s", "ms", "min", "h" }));
		mittelwert_einheit.setName("mittelwert_einheit"); // NOI18N

		lblhistogramm_d.setText(")"); // NOI18N
		lblhistogramm_d.setName("jLabel3"); // NOI18N

		lblbezug.setText("von"); // NOI18N
		lblbezug.setName("jLabel4"); // NOI18N

		lblsymbol.setText("Symbol:"); // NOI18N
		lblsymbol.setName("jLabel5"); // NOI18N

		lbleinheit.setText("Einheit:"); // NOI18N
		lbleinheit.setName("jLabel6"); // NOI18N

		einheit.setText("m"); // NOI18N
		einheit.setName("einheit"); // NOI18N

		lblvon.setText("von:"); // NOI18N
		lblvon.setName("jLabel7"); // NOI18N

		von.setText("0"); // NOI18N
		von.setName("von"); // NOI18N

		lblbis.setText("bis:"); // NOI18N
		lblbis.setName("jLabel8"); // NOI18N

		bis.setText("100"); // NOI18N
		bis.setName("bis"); // NOI18N

		lblstellen.setText("Dezimalstellen:"); // NOI18N
		lblstellen.setName("jLabel9"); // NOI18N

		stellen.setText("1"); // NOI18N
		stellen.setName("stellen"); // NOI18N

		javax.swing.GroupLayout gl_eigenschaften = new javax.swing.GroupLayout(
				eigenschaften);
		gl_eigenschaften
				.setHorizontalGroup(gl_eigenschaften
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_eigenschaften
										.createSequentialGroup()
										.addGroup(
												gl_eigenschaften
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_eigenschaften
																		.createSequentialGroup()
																		.addContainerGap()
																		.addGroup(
																				gl_eigenschaften
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addGroup(
																								gl_eigenschaften
																										.createSequentialGroup()
																										.addComponent(
																												konstante)
																										.addPreferredGap(
																												ComponentPlacement.UNRELATED)
																										.addComponent(
																												konstante_wert,
																												GroupLayout.PREFERRED_SIZE,
																												62,
																												GroupLayout.PREFERRED_SIZE))
																						.addComponent(
																								parameter)
																						.addComponent(
																								formel)))
														.addGroup(
																gl_eigenschaften
																		.createSequentialGroup()
																		.addGap(29)
																		.addComponent(
																				formelterm,
																				GroupLayout.DEFAULT_SIZE,
																				619,
																				Short.MAX_VALUE))
														.addGroup(
																gl_eigenschaften
																		.createSequentialGroup()
																		.addContainerGap()
																		.addGroup(
																				gl_eigenschaften
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addGroup(
																								gl_eigenschaften
																										.createSequentialGroup()
																										.addComponent(
																												ableitung)
																										.addGap(18)
																										.addComponent(
																												integral))
																						.addGroup(
																								gl_eigenschaften
																										.createSequentialGroup()
																										.addComponent(
																												mittelwert)
																										.addPreferredGap(
																												ComponentPlacement.RELATED)
																										.addComponent(
																												mittelwert_intervall,
																												GroupLayout.PREFERRED_SIZE,
																												42,
																												GroupLayout.PREFERRED_SIZE)
																										.addPreferredGap(
																												ComponentPlacement.RELATED)
																										.addComponent(
																												mittelwert_einheit,
																												GroupLayout.PREFERRED_SIZE,
																												GroupLayout.DEFAULT_SIZE,
																												GroupLayout.PREFERRED_SIZE)))
																		.addGap(18)
																		.addGroup(
																				gl_eigenschaften
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addGroup(
																								gl_eigenschaften
																										.createSequentialGroup()
																										.addComponent(
																												histogramm)
																										.addPreferredGap(
																												ComponentPlacement.RELATED)
																										.addComponent(
																												histogramm_d,
																												GroupLayout.PREFERRED_SIZE,
																												32,
																												GroupLayout.PREFERRED_SIZE)
																										.addGap(3)
																										.addComponent(
																												lblhistogramm_d))
																						.addComponent(
																								fft))
																		.addPreferredGap(
																				ComponentPlacement.UNRELATED)
																		.addComponent(
																				lblbezug)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				bezug,
																				0,
																				77,
																				Short.MAX_VALUE))
														.addGroup(
																gl_eigenschaften
																		.createSequentialGroup()
																		.addContainerGap()
																		.addComponent(
																				lblsymbol)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				symbol,
																				GroupLayout.PREFERRED_SIZE,
																				36,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.UNRELATED)
																		.addComponent(
																				lbleinheit)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				einheit,
																				GroupLayout.PREFERRED_SIZE,
																				36,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				lblvon)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				von,
																				GroupLayout.PREFERRED_SIZE,
																				56,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				lblbis)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				bis,
																				GroupLayout.PREFERRED_SIZE,
																				58,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.RELATED,
																				73,
																				Short.MAX_VALUE)
																		.addComponent(
																				lblstellen)
																		.addGap(2)
																		.addComponent(
																				stellen,
																				GroupLayout.PREFERRED_SIZE,
																				46,
																				GroupLayout.PREFERRED_SIZE)))
										.addContainerGap()));
		gl_eigenschaften
				.setVerticalGroup(gl_eigenschaften
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_eigenschaften
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_eigenschaften
														.createParallelGroup(
																Alignment.TRAILING)
														.addComponent(konstante)
														.addComponent(
																konstante_wert,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addComponent(parameter)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addComponent(formel)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addComponent(formelterm,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												ComponentPlacement.UNRELATED)
										.addGroup(
												gl_eigenschaften
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_eigenschaften
																		.createSequentialGroup()
																		.addGroup(
																				gl_eigenschaften
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								ableitung)
																						.addComponent(
																								integral)
																						.addComponent(
																								fft))
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addGroup(
																				gl_eigenschaften
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								mittelwert)
																						.addComponent(
																								mittelwert_intervall,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								mittelwert_einheit,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								histogramm)
																						.addComponent(
																								histogramm_d,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								lblhistogramm_d)))
														.addGroup(
																gl_eigenschaften
																		.createParallelGroup(
																				Alignment.BASELINE)
																		.addComponent(
																				lblbezug,
																				GroupLayout.PREFERRED_SIZE,
																				45,
																				GroupLayout.PREFERRED_SIZE)
																		.addComponent(
																				bezug,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)))
										.addGap(18)
										.addGroup(
												gl_eigenschaften
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_eigenschaften
																		.createParallelGroup(
																				Alignment.BASELINE)
																		.addComponent(
																				lblsymbol)
																		.addComponent(
																				symbol,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addComponent(
																				lbleinheit)
																		.addComponent(
																				einheit,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addComponent(
																				lblvon)
																		.addComponent(
																				von,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addComponent(
																				lblbis)
																		.addComponent(
																				bis,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE))
														.addGroup(
																gl_eigenschaften
																		.createParallelGroup(
																				Alignment.BASELINE)
																		.addComponent(
																				lblstellen)
																		.addComponent(
																				stellen,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)))
										.addContainerGap()));
		eigenschaften.setLayout(gl_eigenschaften);

		javax.swing.GroupLayout gl_Groesse = new javax.swing.GroupLayout(
				this);
		setLayout(gl_Groesse);
		gl_Groesse
				.setHorizontalGroup(gl_Groesse
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_Groesse
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_Groesse
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																eigenschaften,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addGroup(
																gl_Groesse
																		.createSequentialGroup()
																		.addComponent(
																				lblgroesse)
																		.addGap(16,
																				16,
																				16)
																		.addComponent(
																				groesse,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				154,
																				Short.MAX_VALUE)
																		.addGap(18,
																				18,
																				18)
																		.addComponent(
																				groesse_neu)
																		.addGap(18,
																				18,
																				18)
																		.addComponent(
																				groesse_weg)))
										.addContainerGap()));
		gl_Groesse
				.setVerticalGroup(gl_Groesse
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_Groesse
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_Groesse
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																groesse,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																lblgroesse)
														.addComponent(
																groesse_neu)
														.addComponent(
																groesse_weg))
										.addGap(18, 18, 18)
										.addComponent(
												eigenschaften,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));
	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JComboBox<String> mittelwert_einheit;
	private javax.swing.JComboBox<String> bezug;
	private javax.swing.JLabel lblgroesse;
	private javax.swing.JLabel lblhistogramm_d;
	private javax.swing.JLabel lblbezug;
	private javax.swing.JLabel lblsymbol;
	private javax.swing.JLabel lbleinheit;
	private javax.swing.JLabel lblvon;
	private javax.swing.JLabel lblbis;
	private javax.swing.JLabel lblstellen;
	private javax.swing.JRadioButton konstante;
	private javax.swing.JRadioButton parameter;
	private javax.swing.JRadioButton formel;
	private javax.swing.JRadioButton ableitung;
	private javax.swing.JRadioButton integral;
	private javax.swing.JRadioButton fft;
	private javax.swing.JRadioButton mittelwert;
	private javax.swing.JRadioButton histogramm;
	private FormelFormattedTextField formelterm;
	private JFormattedTextField konstante_wert;
	private JFormattedTextField mittelwert_intervall;
	private JFormattedTextField histogramm_d;
	private SymbolFormattedTextField symbol;
	private EinheitFormattedTextField einheit;
	private JFormattedTextField von;
	private JFormattedTextField bis;
	private JFormattedTextField stellen;
	private javax.swing.JButton groesse_neu;
	private javax.swing.JButton groesse_weg;
	private ButtonGroup art;
	private javax.swing.JComboBox<String> groesse;
	private FocusAdapter groesseFocusAdapter;
	private javax.swing.JPanel eigenschaften;

	private void aktualisiereGUI() {
		if (aktuelleGroesse == null)
			return;
		firePC = false;
		symbol.setValue(aktuelleGroesse.getSymbol());
		einheit.setValue(aktuelleGroesse.getEinheit().toString());
		von.setValue(aktuelleGroesse.getVon());
		bis.setValue(aktuelleGroesse.getBis());
		stellen.setValue(aktuelleGroesse.getStellen());
		konstante_wert.setValue(aktuelleGroesse.getWert());
		int aktuell = gl.indexOf(aktuelleGroesse);
		if (aktuell > -1) {
			String[] symbole = Arrays.copyOfRange(gl.getTextSymbole(), 0, aktuell);
			formelterm.setKonstanten(symbole);
			String k = "";
			for (String s : symbole)
				k = k + "," + s;
			formel.setText("Formel(t,n,f,β,λ,E,date,time" + k + ")");
			if (bezug != null && bezugModel != null) {
				if(aktuelleGroesse.getBezug() != null && bezugModel.indexOf(aktuelleGroesse.getBezug().getSymbol()) <= -1)
					bezugModel.addElement(aktuelleGroesse.getBezug().getSymbol());
			}
			bezug.setSelectedItem(aktuelleGroesse.getBezug() != null ? aktuelleGroesse
					.getBezug().getSymbol() : null);
		}
		formelterm.setName(aktuelleGroesse.getSymbol());
		formelterm.setValue(aktuelleGroesse.getFormel());
		mittelwert_intervall.setValue(aktuelleGroesse.getIntervall());
		mittelwert_einheit.setSelectedItem(aktuelleGroesse.getMweinheit());
		histogramm_d.setValue(aktuelleGroesse.getDelta());
		Util.selectButton(art, "art", aktuelleGroesse.getArt());
		firePC = true;
	}

	private void adjustComponents() {
		groesse_neu.setEnabled(gl != null && gl.getSize() < 100);
		if (aktuelleGroesse != null && aktuelleGroesse.getArt() != Art.Klasse) {
			if(DEBUG)
				System.out.println("EinstellungenGroessePanel] - adjustComponents: aktivieren");
			groesse_weg.setEnabled(gl != null && gl.getSize() > 0
					&& aktuelleGroesse.getArt() != Art.Messgroesse);
			eigenschaften.setEnabled(true);
			boolean keineMessGroesse = aktuelleGroesse.getArt() != Art.Messgroesse;
			Art aArt = aktuelleGroesse.getArt();
			konstante.setEnabled(keineMessGroesse);
			konstante_wert.setEnabled(aktuelleGroesse.getArt() == Art.Konstante
					|| aktuelleGroesse.getArt() == Art.Parameter);
			parameter.setEnabled(keineMessGroesse);
			formel.setEnabled(keineMessGroesse);
			formelterm.setEnabled(aktuelleGroesse.getArt() == Art.Formel);
			ableitung.setEnabled(keineMessGroesse);
			integral.setEnabled(keineMessGroesse);
			fft.setEnabled(keineMessGroesse);
			mittelwert.setEnabled(keineMessGroesse);
			mittelwert_intervall
					.setEnabled(aktuelleGroesse.getArt() == Art.Mittelwert);
			mittelwert_einheit
					.setEnabled(aktuelleGroesse.getArt() == Art.Mittelwert);
			histogramm.setEnabled(keineMessGroesse);
			lblhistogramm_d.setEnabled(histogramm.isEnabled());
			histogramm_d.setEnabled(aktuelleGroesse.getArt() == Art.Histogramm);
			bezug.setEnabled(aArt == Art.Ableitung || aArt == Art.Integral
					|| aArt == Art.FFT || aArt == Art.Histogramm
					|| aArt == Art.Mittelwert);
			lblbezug.setEnabled(bezug.isEnabled());
			groesse.setEditable(keineMessGroesse);
			symbol.setEnabled(keineMessGroesse);
			lblsymbol.setEnabled(symbol.isEnabled());
			einheit.setEnabled(keineMessGroesse);
			lbleinheit.setEnabled(einheit.isEnabled());
			von.setEnabled(keineMessGroesse);
			lblvon.setEnabled(von.isEnabled());
			bis.setEnabled(keineMessGroesse);
			lblbis.setEnabled(bis.isEnabled());
			stellen.setEnabled(keineMessGroesse);
			lblstellen.setEnabled(stellen.isEnabled());
		} else {
			if(DEBUG)
				System.out.println("[EinstellungenGroessePanel] - adjustComponents: deaktivieren");
			groesse_weg.setEnabled(false);
			eigenschaften.setEnabled(false);
			konstante.setEnabled(false);
			konstante_wert.setEnabled(false);
			parameter.setEnabled(false);
			formel.setEnabled(false);
			formelterm.setEnabled(false);
			ableitung.setEnabled(false);
			integral.setEnabled(false);
			fft.setEnabled(false);
			mittelwert.setEnabled(false);
			mittelwert_intervall.setEnabled(false);
			mittelwert_einheit.setEnabled(false);
			histogramm.setEnabled(false);
			histogramm_d.setEnabled(false);
			lblhistogramm_d.setEnabled(false);
			bezug.setEnabled(false);
			lblbezug.setEnabled(false);
			groesse_weg.setEnabled(false);
			symbol.setEnabled(false);
			lblsymbol.setEnabled(false);
			einheit.setEnabled(false);
			lbleinheit.setEnabled(false);
			von.setEnabled(false);
			lblvon.setEnabled(false);
			bis.setEnabled(false);
			lblbis.setEnabled(false);
			stellen.setEnabled(false);
			lblstellen.setEnabled(false);
		}
	}

	/********************************************************************
	 * Actions
	 ********************************************************************/
	@Action
	public void neueGroesse() {
		aktuelleGroesse = new Groesse(Util.findUniqueName(gl,
				"neue Größe"),Util.findUniqueSymbol(gl, "f_1", false));
		gl.add(aktuelleGroesse);
		((MutableComboBoxModel<String>) groesse.getModel()).addElement(aktuelleGroesse
				.getName());
		groesse.getModel().setSelectedItem(aktuelleGroesse.getName());
		aktualisiereGUI();
		adjustComponents();
	}

	@Action
	public void entferneGroesse() {
		String alt = (String) groesse.getModel().getSelectedItem();
		if (alt == null)
			return;
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			art_parameter();
		gl.remove(aktuelleGroesse);
		aktuelleGroesse = null;
		((MutableComboBoxModel<String>) groesse.getModel()).removeElement(alt);
		setzeGroesse(null);
	}

	@Action
	public void setzeGroesse(ActionEvent e) {
		if (aktuelleGroesse != null && e != null
				&& e.getActionCommand() == "comboBoxEdited") {
			String neu = (String) groesse.getSelectedItem();
			aktuelleGroesse.setName(neu);
			aktuelleGroesse.setManuellerName(true);
			int index = gl.indexOf(aktuelleGroesse);
			if (DEBUG)
				System.out
						.println("[EinstellungenGroessePanel] - setzeGroesse: comboBoxEdited neu("
								+ neu + ") index(" + index + ")");
			((MutableComboBoxModel<String>) groesse.getModel()).removeElementAt(index);
			((MutableComboBoxModel<String>) groesse.getModel()).insertElementAt(neu,
					index);
			groesse.setSelectedIndex(index);
		} else {
			int index = gl.indexOfName((String) groesse.getModel()
					.getSelectedItem());
			if (index >= 0 && index < gl.getSize()) {
				aktuelleGroesse = gl.get(index);
				aktualisiereGUI();
			}
			adjustComponents();
		}
	}

	@Action
	public void art_konstante() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(Konstante);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void art_parameter() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(Parameter);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void art_formel() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(Formel);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void art_ableitung() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(Ableitung);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void art_integral() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(Integral);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void art_fft() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(FFT);
		aktuelleGroesse.setBezug(null);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void art_mittelwert() {
		if (aktuelleGroesse.getArt() == Art.Histogramm)
			histogrammKlasse_weg();
		aktuelleGroesse.setArt(Mittelwert);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	private void histogrammKlasse_weg() {
		gl.remove(aktuelleGroesse.getBezug2());
		aktuelleGroesse.setBezug2(null);
	}

	@Action
	public void art_histogramm() {
		if (aktuelleGroesse.getArt() != Histogramm) {
			Groesse g = new Groesse(Util.findUniqueName(gl,	"Klasse"),
					Util.findUniqueSymbol(gl, "k_1", false));
			g.setArt(Art.Klasse);
			g.setBezug(aktuelleGroesse.getBezug());
			g.setDelta(aktuelleGroesse.getDelta());
			gl.add(g);
			aktuelleGroesse.setBezug2(g);
		}
		aktuelleGroesse.setArt(Histogramm);
		groesse.setSelectedItem(aktuelleGroesse.getName());
		adjustComponents();
	}

	@Action
	public void setzeMWEinheit() {
		aktuelleGroesse.setMweinheit(Einheit
				.toEinheit((String) mittelwert_einheit.getSelectedItem()));
	}

	/********************************************************************
	 * GROESSENLISTE
	 ********************************************************************/
	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsAdded(ObservableList list, int index, int length) {
		if (list instanceof Groessenliste) {
			Groessenliste gl = (Groessenliste) list;
			int aktuell = gl.indexOf(aktuelleGroesse);
			if (aktuell > -1) {
				String[] symbole = Arrays.copyOfRange(gl.getTextSymbole(), 0,
						aktuell);
				if (formelterm != null)
					formelterm.setKonstanten(symbole);
				String k = "";
				for (String s : symbole)
					k = k + "," + s;
				formel.setText("Formel(t,n,f,β,λ,E,date,time" + k + ")");
				if (DEBUG)
					System.out
							.println("Formel(t,n,f,β,λ,E,date,time" + k + ")");
			} else
				setzeGroesse(null);
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsRemoved(ObservableList list, int index,
			List oldElements) {
		if (list instanceof Groessenliste) {
			Groessenliste gl = (Groessenliste) list;
			int aktuell = gl.indexOf(aktuelleGroesse);
			if (aktuell > -1) {
				String[] symbole = Arrays.copyOfRange(gl.getTextSymbole(), 0,
						aktuell);
				if (formelterm != null)
					formelterm.setKonstanten(symbole);
				String k = "";
				for (String s : symbole)
					k = k + "," + s;
				formel.setText("Formel(t,n,f,β,λ,E,date,time" + k + ")");
			} else
				setzeGroesse(null);
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementReplaced(ObservableList list, int index,
			Object oldElement) {
		if (list instanceof Groessenliste) {
			Groessenliste gl = (Groessenliste) list;
			int aktuell = gl.indexOf(aktuelleGroesse);
			if (aktuell > -1) {
				String[] symbole = Arrays.copyOfRange(gl.getTextSymbole(), 0,
						aktuell);
				if (formelterm != null)
					formelterm.setKonstanten(symbole);
				String k = "";
				for (String s : symbole)
					k = k + "," + s;
				formel.setText("Formel(t,n,f,β,λ,E,date,time" + k + ")");
			} else
				setzeGroesse(null);
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementPropertyChanged(ObservableList list, int index) {
	}

	@Override
	public void listElementPropertyChanged(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			PropertyChangeEvent e) {
		if (aktuelleGroesse == list.get(index)) {
			String s = e.getPropertyName();
			if("wert".equals(s))
				konstante_wert.setValue(aktuelleGroesse.getWert());
			else if("name".equals(s))
				groesse.setSelectedItem(aktuelleGroesse.getName());
			else if("symbol".equals(s)){
				symbol.setValue(aktuelleGroesse.getSymbol());				
				formelterm.setName(aktuelleGroesse.getSymbol());
			}
			else if("von".equals(s))
				von.setValue(aktuelleGroesse.getVon());
			else if("bis".equals(s))
				bis.setValue(aktuelleGroesse.getBis());
			else if("einheit".equals(s))
				einheit.setValue(aktuelleGroesse.getEinheit().toString());
			else if("stellen".equals(s))
				stellen.setValue(aktuelleGroesse.getStellen());
			else if("formelterm".equals(s))
				formelterm.setValue(aktuelleGroesse.getFormel());
			else if("art".equals(s))
				Util.selectButton(art, "art", aktuelleGroesse.getArt());
			else if("bezug".equals(s))
				bezug.setSelectedItem(aktuelleGroesse.getBezug() != null ? aktuelleGroesse
						.getBezug().getSymbol() : null);
//			int aktuell = gl.indexOf(aktuelleGroesse);
//			if (aktuell > -1) {
//				String[] symbole = Arrays.copyOfRange(gl.getSymbole(), 0,
//						aktuell);
//				formelterm.setKonstanten(symbole);
//				String k = "";
//				for (String s1 : symbole)
//					k = k + "," + s1;
//				formel.setText("Formel(t,n,f,β,λ,E,date,time" + k + ")");
//			}
		}
	}
}
