/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge.einstellungen.darstellung;

import static ghmessen.groesse.wertetabelle.XAchseTyp.UNABHAENGIG;
import static javax.swing.event.TableModelEvent.DELETE;
import static javax.swing.event.TableModelEvent.HEADER_ROW;
import static javax.swing.event.TableModelEvent.INSERT;
import static javax.swing.event.TableModelEvent.UPDATE;
import ghmessen.groesse.Skalierung;
import ghmessen.groesse.WerteTabelle;

import java.util.Arrays;
import java.util.Vector;

import javax.swing.MutableComboBoxModel;
import javax.swing.event.TableModelEvent;

public class AbhaengigeAchsePanel extends AchsePanel {

	private static final long serialVersionUID = -5628098658157026639L;
	private static final boolean DEBUG = false;

	public AbhaengigeAchsePanel(WerteTabelle wt, String name, String symbol,
			Skalierung skalierung, String achse, boolean balken) {
		super(wt, name, symbol, skalierung, achse, balken);
		if (DEBUG)
			System.out.println("[AbhaengigeAchsePanel] - Konstruktor.");
	}

	protected void initListe() {
		if (wt == null)
			return;
		alteListe = new Vector<String>() {
			private static final long serialVersionUID = 1L;

			{
				add("aus");
				if (wt.getColumnCount() > UNABHAENGIG)
					addAll(Arrays.asList(wt.getColumnSymbols(UNABHAENGIG,
							wt.getColumnCount())));
			}
		};
		if (!alteListe.contains(alteAchse))
			alteAchse = alteListe.get(0);
	}

	/**********************************************************
	 * Änderungen der Wertetabelle
	 **********************************************************/
	@Override
	public void tableChanged(TableModelEvent e) {
		if (e.getSource() instanceof WerteTabelle
				&& e.getFirstRow() == HEADER_ROW) {
			// Bei INSERT, DELETE sind die Parameter brauchbar, bei UPDATE
			// wurde eventuell eine Spalte hinzugefügt.
			// Es werden nur abhängige Größen dargestellt.
			// Vorsicht: alteListe(0) = "aus"!
			int index = e.getColumn();
			if (index < UNABHAENGIG)
				return;
			int indexIntern = index + 1 - UNABHAENGIG;
			switch (e.getType()) {
			case INSERT:
				((MutableComboBoxModel<String>) achse.getModel()).insertElementAt(
						((WerteTabelle) e.getSource()).getColumnSymbol(index),
						indexIntern);
				break;
			case DELETE:
				if (alteAchse.equals(alteListe.get(indexIntern)))
					achse.setSelectedIndex(0);
				((MutableComboBoxModel<String>) achse.getModel())
						.removeElementAt(indexIntern);
				break;
			case UPDATE:
				MutableComboBoxModel<String> mm = (MutableComboBoxModel<String>) achse
						.getModel();
				boolean updateSelected = alteAchse.equals(alteListe
						.get(indexIntern));
				String neueAchse = ((WerteTabelle) e.getSource())
						.getColumnSymbol(index);
				if (alteAchse.equals(neueAchse))
					return;
				mm.insertElementAt(neueAchse, indexIntern + 1);
				if (updateSelected)
					mm.setSelectedItem(alteListe.get(indexIntern + 1));
				mm.removeElementAt(indexIntern);
				break;
			}
		}
	}

}
