/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge.einstellungen.darstellung;

import static ghmessen.groesse.Skalierung._y;
import static ghmessen.groesse.Skalierung._y2;
import static ghmessen.groesse.Skalierung.logy;
import static ghmessen.groesse.Skalierung.sqrty;
import static ghmessen.groesse.Skalierung.y;
import static ghmessen.groesse.Skalierung.y2;
import static javax.swing.event.TableModelEvent.DELETE;
import static javax.swing.event.TableModelEvent.HEADER_ROW;
import static javax.swing.event.TableModelEvent.INSERT;
import static javax.swing.event.TableModelEvent.UPDATE;

import java.awt.Dimension;
import java.util.Arrays;
import java.util.Vector;

import ghmessen.groesse.Skalierung;
import ghmessen.groesse.WerteTabelle;

import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.MutableComboBoxModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;

public class AchsePanel extends JPanel implements TableModelListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5232408621075259585L;
	private static final int idealeBreite = 90;
	private static final boolean DEBUG = false;
	protected JComboBox<String> achse;
	protected JRadioButton[] skalierung;
	protected ButtonGroup achseBG;
	protected JCheckBox balken;
	private ActionMap actionMap;
	protected boolean listenerAn = true;

	protected WerteTabelle wt;
	protected String name;
	protected String symbol;

	protected Skalierung alteSkalierung;
	protected String alteAchse;
	protected boolean alterBalken;
	protected Vector<String> alteListe;

	public AchsePanel(WerteTabelle wt, String name, String symbol,
			Skalierung skalierung, String achse, boolean balken) {
		super();
		if (DEBUG)
			System.out.println("[AchsePanel] - Konstruktor.");
		this.wt = wt;
		this.name = name;
		this.symbol = symbol;
		alteSkalierung = skalierung;
		alteAchse = achse;
		alterBalken = balken;
		if (wt != null) {
			wt.addTableModelListener(this);
			initListe();
		} else
			alteListe = new Vector<String>() {
				private static final long serialVersionUID = 1L;

				{
					add("aus");
				}
			};
		initComponents();
	}

	/**
	 * @wbp.parser.constructor
	 */
	public AchsePanel(WerteTabelle wt, String name, String symbol) {
		this(wt, name, symbol, y, "aus", false);
	}

	protected void initListe() {
		if (wt == null) {
			alteListe = new Vector<String>() {
				private static final long serialVersionUID = 1L;
				{
					add("aus");
				}
			};
			alteAchse = "aus";
			return;
		}
		alteListe = new Vector<String>() {
			private static final long serialVersionUID = 1L;

			{
				add("aus");
				addAll(Arrays.asList(wt.getColumnSymbols()));
			}
		};
		if (!alteListe.contains(alteAchse))
			alteAchse = alteListe.get(0);
	}

	public void setWerte(WerteTabelle wt, Skalierung skalierung, String achse,
			boolean balken) {
		if (this.wt != wt) {
			if (this.wt != null)
				wt.removeTableModelListener(this);
			this.wt = wt;
			initListe();
			this.wt.addTableModelListener(this);
			this.achse.setModel(new DefaultComboBoxModel<String>(alteListe));
		}
		alteSkalierung = skalierung;
		for (JRadioButton b : this.skalierung)
			if (b.getClientProperty("Skalierung") == skalierung) {
				b.setSelected(true);
				break;
			}
		alteAchse = achse;
		this.achse.getModel().setSelectedItem(alteAchse);
		alterBalken = balken;
		this.balken.setSelected(balken);
	}

	protected void initComponents() {
		actionMap = Application.getInstance().getContext()
				.getActionMap(AchsePanel.class, this);

		setMaximumSize(new Dimension(idealeBreite, Integer.MAX_VALUE));
		achse = new JComboBox<String>();
		skalierung = new JRadioButton[6];
		for (int i = 0; i < skalierung.length; i++)
			skalierung[i] = new JRadioButton();
		balken = new JCheckBox();
		achseBG = new ButtonGroup();

		setBorder(javax.swing.BorderFactory.createTitledBorder(name)); // NOI18N
		setName("name"); // NOI18N

		achse.setModel(new DefaultComboBoxModel<String>(alteListe));
		achse.getModel().setSelectedItem(alteAchse);
		achse.setName("achse"); // NOI18N
		achse.addActionListener(actionMap.get("waehleGroesse"));

		achseBG.add(skalierung[0]);
		if (alteSkalierung == y)
			skalierung[0].setSelected(true);
		skalierung[0].setText(symbol); // NOI18N
		skalierung[0].addActionListener(actionMap.get("skalierung_linear"));

		achseBG.add(skalierung[1]);
		if (alteSkalierung == y2)
			skalierung[1].setSelected(true);
		skalierung[1].setText("<html>" + symbol + "<sup>2</sup></html>"); // NOI18N
		skalierung[1]
				.addActionListener(actionMap.get("skalierung_quadratisch"));

		achseBG.add(skalierung[2]);
		if (alteSkalierung == _y)
			skalierung[2].setSelected(true);
		skalierung[2].setText("1/" + symbol); // NOI18N
		skalierung[2].addActionListener(actionMap.get("skalierung_reziprok"));

		achseBG.add(skalierung[3]);
		if (alteSkalierung == _y2)
			skalierung[3].setSelected(true);
		skalierung[3].setText("<html>1/" + symbol + "<sup>2</sup></html>"); // NOI18N
		skalierung[3].addActionListener(actionMap
				.get("skalierung_reziprokquad"));

		achseBG.add(skalierung[4]);
		if (alteSkalierung == logy)
			skalierung[4].setSelected(true);
		skalierung[4].setText("log " + symbol); // NOI18N
		skalierung[4].addActionListener(actionMap
				.get("skalierung_logarithmisch"));

		achseBG.add(skalierung[5]);
		if(alteSkalierung == sqrty)
			skalierung[5].setSelected(true);
		skalierung[5].setText("<html>" + sqrty.toString(symbol) + "</html>");
		skalierung[5].addActionListener(actionMap.get("skalierung_wurzel"));
		
		javax.swing.GroupLayout gl_Panel0 = new javax.swing.GroupLayout(this);
		setLayout(gl_Panel0);
		gl_Panel0.setHorizontalGroup(gl_Panel0
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(balken).addComponent(skalierung[0])
				.addComponent(skalierung[1]).addComponent(skalierung[2])
				.addComponent(skalierung[3]).addComponent(skalierung[4])
				.addComponent(skalierung[5])
				.addComponent(achse, 0, 63, Short.MAX_VALUE));
		gl_Panel0
				.setVerticalGroup(gl_Panel0
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								gl_Panel0
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												achse,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addGap(18, 18, 18)
										.addComponent(skalierung[0])
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierung[1])
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierung[2])
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierung[3])
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierung[4])
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(skalierung[5])
										.addGap(18, 18, 18)
										.addComponent(balken)
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		balken.setAction(actionMap.get("balken_an"));
		if ("x".equals(symbol)) {
			balken.setText("Polar"); // NOI18N balken.setName("Polar"); //
										// NOI18N
		} else {
			balken.setText("Balken");
			balken.setName("Balken");
		}
		
	}

	@Action
	public void skalierung_linear() {
		firePropertyChange("skalierung", alteSkalierung, y);
		alteSkalierung = y;
	}

	@Action
	public void skalierung_quadratisch() {
		firePropertyChange("skalierung", alteSkalierung, y2);
		alteSkalierung = y2;
	}

	@Action
	public void skalierung_reziprok() {
		firePropertyChange("skalierung", alteSkalierung, _y);
		alteSkalierung = _y;
	}

	@Action
	public void skalierung_reziprokquad() {
		firePropertyChange("skalierung", alteSkalierung, _y2);
		alteSkalierung = _y2;
	}

	@Action
	public void skalierung_logarithmisch() {
		firePropertyChange("skalierung", alteSkalierung, logy);
		alteSkalierung = logy;
	}

	@Action
	public void skalierung_wurzel(){
		firePropertyChange("skalierung", alteSkalierung, sqrty);
		alteSkalierung = sqrty;
	}
	
	@Action
	public void waehleGroesse() {
		String neueAchse = (String) achse.getSelectedItem();
		firePropertyChange("achse", alteAchse, neueAchse);
		alteAchse = neueAchse;

	}

	@Action
	public void balken_an() {
		boolean neuerBalken = balken.isSelected();
		firePropertyChange("balken", alterBalken, neuerBalken);
		alterBalken = neuerBalken;
	}

	/**********************************************************
	 * Änderungen der Wertetabelle
	 **********************************************************/
	@Override
	public void tableChanged(TableModelEvent e) {
		if (e.getSource() instanceof WerteTabelle
				&& e.getFirstRow() == HEADER_ROW) {
			// Bei INSERT, DELETE sind die Parameter brauchbar, bei UPDATE
			// wurde eventuell eine Spalte hinzugefügt.
			// Vorsicht: alteListe(0) = "aus"!
			int index = e.getColumn();
			switch (e.getType()) {
			case INSERT:
				((MutableComboBoxModel<String>) achse.getModel()).insertElementAt(
						((WerteTabelle) e.getSource()).getColumnSymbol(index),
						index + 1);
				break;
			case DELETE:
				if (alteAchse.equals(alteListe.get(index + 1)))
					achse.setSelectedIndex(0);
				((MutableComboBoxModel<String>) achse.getModel())
						.removeElementAt(index + 1);
				break;
			case UPDATE:
				MutableComboBoxModel<String> mm = (MutableComboBoxModel<String>) achse
						.getModel();
				boolean updateSelected = alteAchse.equals(alteListe
						.get(index + 1));
				String neueAchse = ((WerteTabelle) e.getSource())
						.getColumnSymbol(index);
				if(alteAchse.equals(neueAchse))
					return;
				mm.insertElementAt(neueAchse, index + 2);
				if (updateSelected)
					mm.setSelectedItem(alteListe.get(index + 2));
				mm.removeElementAt(index + 1);
				break;
			}
		}
	}
}
