/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.dialoge.einstellungen.darstellung;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;

import ghmessen.groesse.Darstellung;
import ghmessen.groesse.Skalierung;

import javax.swing.BoxLayout;
import javax.swing.JPanel;

public class AchsenPanel extends JPanel {

	private static final long serialVersionUID = 1562875023004140553L;
	private static final boolean DEBUG = false;

	private class SkalierungAchsePropertyHandler implements Serializable, PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSkalierungAchseEvent(evt);
		}
	}
	
	private class AchseAchsePropertyHandler implements Serializable, PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleAchseAchseEvent(evt);
		}
	}
	
	private class BalkenAchsePropertyHandler implements Serializable, PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleBalkenAchseEvent(evt);
		}
	}
	
	private Darstellung ds;
	private boolean achseListenerAn = true;
	protected SkalierungAchsePropertyHandler skalierungAPH;
	protected AchseAchsePropertyHandler achseAPH;
	protected BalkenAchsePropertyHandler balkenAPH;
	
	public AchsenPanel(Darstellung ds) {
		super();
		skalierungAPH = new SkalierungAchsePropertyHandler();
		achseAPH = new AchseAchsePropertyHandler();
		balkenAPH = new BalkenAchsePropertyHandler();
		if (DEBUG)
			System.out.println("[AchsenPanel] - Konstruktor.");
		this.ds = ds;
		initComponents();
		if (ds != null) {
			aktualisiereAchsen();
		}
	}

	public void setDarstellung(Darstellung neu) {
		ds = neu;
		if (ds != null) {
			aktualisiereAchsen();
		} else
			removeAll();
		revalidate();
		repaint();
	}

	private void aktualisiereAchsen() {
		if (ds == null)
			return;
		//achseListenerAn = false;
		// eine zusätzliche Achse wird zur Auswahl hinzugefügt
		int dsAchsen = ds.getColumnCount() + 1;
		int pAchsen = this.getComponentCount();
		int vorhanden = Math.min(dsAchsen, pAchsen);
		// vorhandene Achsen aktualisieren
		for (int i = 0; i < vorhanden; i++) {
			aktualisiereAchse(i, dsAchsen-1);
		}
		// überflüssige Panele entfernen
		for (int i = vorhanden; i < pAchsen; i++) {
			removeLast();
		}
		// zusätzliche Panele hinzufügen
		for (int i = vorhanden; i < dsAchsen; i++) {
			neueAchse(i, dsAchsen-1);
		}
		//achseListenerAn = true;
	}

	private void aktualisiereAchse(int i, int last) {
		if (i == last)
			((AchsePanel) getComponent(i)).setWerte(ds.getWerteTabelle(),
					Skalierung.y, "aus", false);
		else
			((AchsePanel) getComponent(i)).setWerte(ds.getWerteTabelle(),
					ds.getYS(i), ds.getColumnSymbol(i), ds.isBalken(i));
	}

	private void neueAchse(int i, int last) {
		AchsePanel c;
		if (i == 0)
			c = new AchsePanel(ds.getWerteTabelle(), "X-Achse", "x",
					ds.getYS(i), ds.getColumnSymbol(i), ds.isBalken(i));
		else if(i == last){
			c = new AbhaengigeAchsePanel(ds.getWerteTabelle(), "Y-Achse", "y",
					Skalierung.y, "aus", false);
		}
		else
			c = new AbhaengigeAchsePanel(ds.getWerteTabelle(), "Y-Achse", "y",
					ds.getYS(i), ds.getColumnSymbol(i), ds.isBalken(i));
		add(c);
	}

	@Override
	public Component add(Component c) {
		c.addPropertyChangeListener("skalierung", skalierungAPH);
		c.addPropertyChangeListener("achse", achseAPH);
		c.addPropertyChangeListener("balken", balkenAPH);
		return super.add(c);
	}

	@Override
	public void remove(int index){
		getComponents()[index].removePropertyChangeListener("skalierung",  skalierungAPH);
		getComponents()[index].removePropertyChangeListener("achse",  achseAPH);
		getComponents()[index].removePropertyChangeListener("balken",  balkenAPH);
		super.remove(index);
	}
	
	public void removeLast() {
		Component c = getComponents()[getComponentCount() - 1];
		c.removePropertyChangeListener("skalierung", skalierungAPH);
		c.removePropertyChangeListener("achse", achseAPH);
		c.removePropertyChangeListener("balken", balkenAPH);
		remove(c);
	}

	@Override
	public void removeAll() {
		for (Component c : this.getComponents()){
			c.removePropertyChangeListener("skalierung", skalierungAPH);
			c.removePropertyChangeListener("achse", achseAPH);
			c.removePropertyChangeListener("balken", balkenAPH);
		}
		super.removeAll();
	}

	public int indexOf(Component c) {
		Component[] achsen = getComponents();
		if (achsen == null)
			return -1;
		for (int i = 0; i < achsen.length; i++)
			if (achsen[i] == c)
				return i;
		return -1;
	}

	private void initComponents() {
		BoxLayout gl_achsen = new BoxLayout(this, BoxLayout.LINE_AXIS);
		setLayout(gl_achsen);
	}

	protected void handleSkalierungAchseEvent(PropertyChangeEvent evt) {
		if(!achseListenerAn)
			return;
		int index = indexOf((Component) evt.getSource());
		ds.setYS(index, (Skalierung) evt.getNewValue());
	}

	protected void handleAchseAchseEvent(PropertyChangeEvent evt) {
		if(!achseListenerAn)
			return;
		int index = indexOf((Component) evt.getSource());
		ds.setY(index, (String) evt.getNewValue());
		if("aus".equals(evt.getOldValue()) && getComponentCount() < Darstellung.MAX_Y){
			neueAchse(1,1);
			revalidate();
			repaint();
		}
		else if("aus".equals(evt.getNewValue())){
			remove(index);
			revalidate();
			repaint();
		}
	}

	protected void handleBalkenAchseEvent(PropertyChangeEvent evt) {
		if(!achseListenerAn)
			return;
		int index = indexOf((Component) evt.getSource());
		ds.setBalken(index, (Boolean) evt.getNewValue());
	}

}
