/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import java.io.PrintWriter;


public enum Art {
	Parameter, Formel, Ableitung, Integral, FFT, Mittelwert, Histogramm, Konstante, Messgroesse, Klasse, Unbekannt;

    public static void createDTD(PrintWriter out){
    	out.print("<!ENTITY Art \"");
    	String s = "";
    	for(Art e : values())
    		s += e.toString() + "|";
    	out.println(s.substring(0,s.length()-1)+"\">");
    }
    
	public static Art toArt(String s) {
		switch (s.charAt(0)) {
		case 'K':
			if (s.charAt(1) == 'o')
				return Konstante;
			else
				return Klasse;
		case 'P':
			return Parameter;
		case 'F':
			if (s.charAt(1) == 'o')
				return Formel;
			else
				return FFT;
		case 'A':
			return Ableitung;
		case 'I':
			return Integral;
		case 'M':
			if (s.charAt(1) == 'i')
				return Mittelwert;
			else
				return Messgroesse;
		case 'H':
			return Histogramm;
		default:
			return Unbekannt;
		}
	}

	public static Art fromCL4(int cl4, double typ) {
		if (cl4 >= 524030) {
			// Versuche, bekannte Sensoren zu erkennen.
//			Eingang eingang = EingangArt.toEingang(Sensor, 1, 0, cl4);
			return Messgroesse;
		}
		switch (cl4) {
		default:
			return Unbekannt;
		case 0:
		case 5:
			return Messgroesse;
		case -1:
			return Parameter;
		case 10:
			int i = (int) typ;
			return Art.toArt(i);
		case 15:
			return Klasse;
		}
	}

	public static Art toArt(int i) {
		switch (i) {
		case 0:
			return Art.Parameter;
		case 1:
			return Art.Formel;
		case 2:
			return Art.Ableitung;
		case 3:
			return Art.Integral;
		case 4:
			return Art.FFT;
		case 5:
			return Art.Mittelwert;
		case 6:
			return Art.Histogramm;
		case 7:
			return Art.Konstante;
		case 8:
			return Art.Messgroesse;
		case 9:
			return Art.Klasse;
		default:
			return Art.Unbekannt;
		}
	}

}
