/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import static ghmessen.groesse.wertetabelle.XAchseTyp.UNABHAENGIG;
import static javax.swing.event.TableModelEvent.DELETE;
import static javax.swing.event.TableModelEvent.HEADER_ROW;
import static javax.swing.event.TableModelEvent.INSERT;
import static javax.swing.event.TableModelEvent.UPDATE;
import ghmessen.GHMessen;
import ghmessen.Messparameter;
import ghmessen.components.Intervall;
import ghmessen.filefilter.FileFormatException;
import ghmessen.groesse.darstellung.Anpassung;
import ghmessen.groesse.darstellung.AnpassungFrei;
import ghmessen.groesse.darstellung.AnpassungMittelwert;
import ghmessen.groesse.darstellung.AnpassungRegression;
import ghmessen.groesse.darstellung.Auswertung;
import ghmessen.groesse.darstellung.Eigenschaften;
import ghmessen.groesse.darstellung.LinienAuswertung;
import ghmessen.groesse.darstellung.TextAuswertung;
import ghmessen.groesse.wertetabelle.XAchseTyp;
import ghmessen.util.Util;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.EventListener;
import java.util.Iterator;
import java.util.Locale;
import java.util.Scanner;
import java.util.Vector;

import javax.swing.event.EventListenerList;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class Darstellung implements TableModel, Serializable,
		TableModelListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8530533245487684853L;
	private static final boolean DEBUG = false;

	/** List of listeners */
	protected transient EventListenerList listenerList = new EventListenerList();

	private transient PropertyChangeSupport support = new PropertyChangeSupport(
			this);

	public static final int XACHSE = 0;

	private String name; // Name der Darstellung
	private WerteTabelle werte; // Wertetabelle (Spalte 0,1,2 unabhängige
								// Variable
								// t,n,f,...)
	private Vector<Integer> spalten; // sichtbare Spalten der Wertetabelle
	private int y1; // linke y-Achse und skalierende Spalte
	private int y2; // rechte y-Achse
	private boolean autox; // Die x-Achse wird automatisch mit der y-Achse
							// gewechselt.
	private Vector<Double> minimum, maximum; // manuelle Minima, Maxima
	private Eigenschaften graph; // Eigenschaften der graphischen Darstellung
	private Vector<Auswertung> auswertung; // Anpassungs- und Integralkurven mit
											// Benennungen und alle anderen
											// Auswertungen
	public static final int MAX_Y = 7;
	private boolean balken[] = new boolean[MAX_Y + 1]; // XACHSE = Index 0
	private boolean autoY[] = new boolean[MAX_Y + 1];
	private Skalierung[] yS = { Skalierung.y, Skalierung.y, Skalierung.y,
			Skalierung.y, Skalierung.y, Skalierung.y, Skalierung.y,
			Skalierung.y };

	public static void createDTD(PrintWriter out) {
		Eigenschaften.createDTD(out);
		Skalierung.createDTD(out);
		out.println("<!ELEMENT Auswertung ((Anpassung|AnpassungFrei|ApassungMittelwert|AnpassungRegression|LinienAuswertung|TextAuswertung)*)>");
		Anpassung.createDTD(out);
		AnpassungFrei.createDTD(out);
		AnpassungMittelwert.createDTD(out);
		AnpassungRegression.createDTD(out);
		LinienAuswertung.createDTD(out);
		TextAuswertung.createDTD(out);
		out.println("<!ELEMENT Darstellung (DSpalten,(DAn|DY1),(DAn2|DY2),DMinimum,DMaximum,Eigenschaften,Auswertung,DBalken,DAutoY,DSkalierung)>");
		out.println("<!ATTLIST Darstellung name CDATA #REQUIRED autox (BOOLEAN) #REQUIRED>");
		out.println("<!ELEMENT DSpalten (INT*)>");
		out.println("<!ELEMENT DAn (BOOLEAN*)>");
		out.println("<!ELEMENT DY1 (INT)>");
		out.println("<!ELEMENT DAn2 (BOOLEAN*)>");
		out.println("<!ELEMENT DY2 (INT)>");
		out.println("<!ELEMENT DMinimum (DOUBLE*)>");
		out.println("<!ELEMENT DMaximum (DOUBLE*)>");
		out.println("<!ELEMENT DBalken (BOOLEAN*)>");
		out.println("<!ELEMENT DAutoY (BOOLEAN*)>");
		out.println("<!ELEMENT DSkalierung (Skalierung*)>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[Darstellung] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "name", "CDATA", name);
		atts.addAttribute("", "", "autox", "BOOLEAN", Boolean.toString(autox));
		hd.startElement("", "", "Darstellung", atts);
		String inhalt;
		hd.startElement("", "", "DSpalten", null);
		inhalt = Util.writeIntegersToXml(spalten);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DSpalten");
		hd.startElement("", "", "DY1", null);
		inhalt = Integer.toString(y1);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DY1");
		hd.startElement("", "", "DY2", null);
		inhalt = Integer.toString(y2);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DY2");
		hd.startElement("", "", "DMinimum", null);
		inhalt = Util.writeDoublesToXml(minimum);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DMinimum");
		hd.startElement("", "", "DMaximum", null);
		inhalt = Util.writeDoublesToXml(maximum);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DMaximum");
		graph.writeToXml(hd);
		hd.startElement("", "", "Auswertung", null);
		for (Auswertung a : auswertung)
			a.writeToXml(hd);
		hd.endElement("", "", "Auswertung");
		hd.startElement("", "", "DBalken", null);
		inhalt = Util.writeBooleansToXml(balken);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DBalken");
		hd.startElement("", "", "DAutoY", null);
		inhalt = Util.writeBooleansToXml(autoY);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DAutoY");
		hd.startElement("", "", "DSkalierung", null);
		inhalt = Util.writeSkalierungsToXml(yS);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "DSkalierung");
		hd.endElement("", "", "Darstellung");
	}

	private static enum BEREICH {
		DARSTELLUNG, AUSWERTUNG, EIGENSCHAFTEN, ANPASSUNG
	};

	private BEREICH bereich;

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		switch (bereich) {
		case DARSTELLUNG:
			if (qName.equals("Darstellung")) {
				for (int index = 0; index < attributes.getLength(); index++) {
					try {
						String name = attributes.getLocalName(index);
						String inhalt = attributes.getValue(index);
						Field field = getClass().getDeclaredField(name);
						if (DEBUG)
							System.out
									.println("[Darstellung] - startElement: Feld("
											+ name
											+ ") Typ("
											+ field.getType()
											+ ") Inhalt(" + inhalt + ")");
						Util.setFieldValue(this, field, inhalt);
					} catch (SecurityException e) {
						if (DEBUG)
							e.printStackTrace();
						throw new SAXException(e.getMessage());
					} catch (NoSuchFieldException e) {
						if (DEBUG)
							e.printStackTrace();
						throw new SAXException(e.getMessage());
					} catch (IllegalArgumentException e) {
						if (DEBUG)
							e.printStackTrace();
						throw new SAXException(e.getMessage());
					}
				}
			} else if (qName.equals("DSpalten") 
					|| qName.equals("DAn") || qName.equals("DAn2") 
					|| qName.equals("DY1") || qName.equals("DY2") 
					|| qName.equals("DBalken")
					|| qName.equals("DAutoY") || qName.equals("DMinimum")
					|| qName.equals("DMaximum") || qName.equals("DSkalierung"))
				return;
			else if (qName.equals("Eigenschaften")) {
				bereich = BEREICH.EIGENSCHAFTEN;
				graph = new Eigenschaften();
				graph.startElement(uri, localName, qName, attributes);
			} else if (qName.equals("Auswertung")) {
				bereich = BEREICH.AUSWERTUNG;
				auswertung = new Vector<Auswertung>();
			}
			break;
		case EIGENSCHAFTEN:
			graph.startElement(uri, localName, qName, attributes);
			break;
		case AUSWERTUNG:
			if (qName.equals("AnpassungFrei")) {
				auswertung.add(new AnpassungFrei());
				bereich = BEREICH.ANPASSUNG;
			} else if (qName.equals("ApassungMittelwert")) {
				auswertung.add(new AnpassungMittelwert());
				bereich = BEREICH.ANPASSUNG;
			} else if (qName.equals("AnpassungRegression")) {
				auswertung.add(new AnpassungRegression());
				bereich = BEREICH.ANPASSUNG;
			} else if (qName.equals("LinienAuswertung"))
				auswertung.add(new LinienAuswertung());
			else if (qName.equals("TextAuswertung"))
				auswertung.add(new TextAuswertung());
			auswertung.get(auswertung.size() - 1).startElement(uri, localName,
					qName, attributes);
			break;
		case ANPASSUNG:
			auswertung.get(auswertung.size() - 1).startElement(uri, localName,
					qName, attributes);
			break;
		}
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		switch (bereich) {
		case DARSTELLUNG:
			if (qName.equals("Darstellung")) {
				return;
			}
			else if (qName.equals("DSpalten"))
				spalten = Util.intVectorFromXml(inhalt);
			else if (qName.equals("DY1")) {
				y1 = Integer.parseInt(inhalt);
			}
			else if (qName.equals("DY2")) {
				y2 = Integer.parseInt(inhalt);
			}
			else if (qName.equals("DAn")) {
				Vector<Boolean> tmp = Util.booleanVectorFromXml(inhalt);
				y1 = -1;
				for(int i=0;i<tmp.size();i++)
					if(tmp.get(i))
						y1 = i;
			}
			else if (qName.equals("DAn2")) {
				Vector<Boolean> tmp = Util.booleanVectorFromXml(inhalt);
				y2 = -1;
				for(int i=0;i<tmp.size();i++)
					if(tmp.get(i))
						y2 = i;
			}
			else if (qName.equals("DBalken"))
				balken = Util.prbooleanAFromXml(inhalt);
			else if (qName.equals("DAutoY"))
				autoY = Util.prbooleanAFromXml(inhalt);
			else if (qName.equals("DMinimum"))
				minimum = Util.doubleVectorFromXml(inhalt);
			else if (qName.equals("DMaximum"))
				maximum = Util.doubleVectorFromXml(inhalt);
			else if (qName.equals("DSkalierung"))
				yS = Util.skalierungAFromXml(inhalt);
			else
				throw new SAXException("[Darstellung] - Ende(" + qName + ")");
			break;
		case EIGENSCHAFTEN:
			graph.endElement(uri, localName, qName, inhalt);
			if (qName.equals("Eigenschaften"))
				bereich = BEREICH.DARSTELLUNG;
			break;
		case AUSWERTUNG:
			if (qName.equals("Auswertung"))
				bereich = BEREICH.DARSTELLUNG;
			else if (auswertung.size() > 0)
				auswertung.get(auswertung.size() - 1).endElement(uri,
						localName, qName, inhalt);
			break;
		case ANPASSUNG:
			if (qName.equals("AnpassungRegression")
					|| qName.equals("AnpassungMittelwert")
					|| qName.equals("AnpassungFrei"))
				bereich = BEREICH.AUSWERTUNG;
			if (auswertung.size() > 0)
				auswertung.get(auswertung.size() - 1).endElement(uri,
						localName, qName, inhalt);
			break;
		}
	}

	/*
	 * Konstruktor ohne Parameter für das Einlesen aus einer Xml-Datei. Die
	 * WerteTabelle muss mittels "setWerteTabelle" unbedingt später zugeordnet
	 * werden.
	 */
	public Darstellung() {
		bereich = BEREICH.DARSTELLUNG;
		autox = true;
	}

	public Darstellung(WerteTabelle werte, String name, String x, String y1) {
		this.werte = werte;
		this.spalten = new Vector<Integer>(werte.getColumnCount());
		Arrays.fill(autoY, true);
		Arrays.fill(balken, false);
		Arrays.fill(yS, Skalierung.y);
		this.minimum = new Vector<Double>(werte.getColumnCount());
		this.maximum = new Vector<Double>(werte.getColumnCount());
		this.y1 = -1;
		this.y2 = -1;
		this.graph = new Eigenschaften();
		this.auswertung = new Vector<Auswertung>();
		setName(name);
		addColumn(x);
		addColumn(y1);
		setY1(1);
		werte.addTableModelListener(this);
	}

	public Darstellung(WerteTabelle werte, String name, Groesse x, Groesse y1) {
		this(werte, name, x.getSymbol(), y1.getSymbol());
	}

	public Darstellung(WerteTabelle werte) {
		this(werte, true);
	}

	public Darstellung(WerteTabelle werte, boolean autoSpalten) {
		this.werte = werte;
		setName("Standard");
		this.spalten = new Vector<Integer>(werte.getColumnCount());
		Arrays.fill(autoY, true);
		Arrays.fill(balken, false);
		Arrays.fill(yS, Skalierung.y);
		this.minimum = new Vector<Double>(werte.getColumnCount());
		this.maximum = new Vector<Double>(werte.getColumnCount());
		this.y1 = -1;
		this.y2 = -1;
		this.graph = new Eigenschaften();
		this.auswertung = new Vector<Auswertung>();
		if (autoSpalten) {
			if (werte.getColumnCount() > 0) {
				addColumn(0);
				if (werte.getColumnCount() > UNABHAENGIG) {
					addColumn(UNABHAENGIG);
					setY1(1);
				}
			}
		}
		werte.addTableModelListener(this);
	}

	private String getCL4Symbol(int n, boolean IS_CL4)
			throws FileFormatException {
		switch (n) {
		case 0:
			return "n";
		case 1:
			return IS_CL4 ? "t" : "E";
		case 2:
			return "f";
		default:
			try {
				return werte.getColumnSymbol(n + UNABHAENGIG - 3);
			} catch (IndexOutOfBoundsException e) {
				if (DEBUG)
					System.out
							.println("Die CL4-Darstellung hat eine falsche Größennummer("
									+ n + ")Histogramm?");
				return werte.getColumnCount() > UNABHAENGIG ? werte
						.getColumnName(UNABHAENGIG) : "t";
				// Histogramm ignorieren: throw new
				// FileFormatException("line "+nr+": Falsche Größennummer("+n+")");
			}
		}
	}

	public void setFromCL4(String[] s, boolean IS_CL4)
			throws FileFormatException {
		if (s == null || s.length != 2)
			throw new FileFormatException("CL4-Darstellung ist fehlerhaft.");
		Scanner s2i = new Scanner(s[1]);
		s2i.useLocale(Locale.US);
		String[] ySymbol = new String[8];
		int[] yS = new int[8];
		double[] yMin = new double[8], yMax = new double[8];
		int groesse;
		int ii = 0;
		while (ii < 8) {
			groesse = s2i.nextInt();
			if (groesse != -1) {
				ySymbol[ii] = getCL4Symbol(groesse, IS_CL4);
				yS[ii] = s2i.nextInt();
				yMin[ii] = s2i.nextDouble();
				yMax[ii] = s2i.nextDouble();
				if (ii != 0 && !IS_CL4) {
					int reihe = 1;
					while (Util.findeGroesse(GHMessen.getApplication()
							.getMainView().gl, ySymbol[ii] + "#" + reihe) != null) {
						if (ii + 1 >= 8)
							break;
						ySymbol[ii + 1] = ySymbol[ii] + "#" + reihe;
						yS[ii + 1] = yS[ii];
						yMin[ii + 1] = yMin[ii];
						yMax[ii + 1] = yMax[ii];
						ii++;
						reihe++;
					}
				}
			}
			ii++;
		}
		s2i.nextInt(); // 1 Zahl (-1) ignoriert
		if (ySymbol[1] == null) {
			s2i.close();
			throw new FileFormatException(
					"Die CL4-Darstellung enthält kein y-Achsensymbol.");
		}
		setName(s[0]);
		autox = "Standard".equals(name);
		s2i.nextInt();
		s2i.nextInt();
		s2i.nextInt();
		s2i.nextInt();
		s2i.nextInt(); // 5 Zahlen (10101) ignoriert
		graph.setLinieAn(true);
		int yBits = 0;
		ii = 0;
		while (ii < 8 && ySymbol[ii] != null) {
			setY(ii, ySymbol[ii]);
			setAutoY(ii, false);
			switch (yS[ii]) {
			case 0:
				setYS(ii, Skalierung.y);
				break;
			case 1:
				setYS(ii, Skalierung.y2);
				break;
			case 2:
				setYS(ii, Skalierung._y);
				break;
			case 3:
				setYS(ii, Skalierung._y2);
				break;
			default:
				setYS(ii, Skalierung.logy);
				break;
			}
			setIntervall(ii, new Intervall(yMin[ii], yMax[ii]));
			if (s2i.hasNextInt()) {
				yBits = s2i.nextInt();
				setBalken(ii, (yBits & 4) != 0);
				graph.setLinieAn((yBits & 2) != 0);
				graph.setWerteAn((yBits & 1) != 0);
			}
			ii++;
		}
		if (s2i.hasNextInt())
			s2i.nextInt();
		if (s2i.hasNextInt())
			s2i.nextInt();
		if (s2i.hasNextInt())
			s2i.nextInt(); // 3 Zahlen (010) ignoriert
		if(spalten.size() > 1)
			setY1(1);
		s2i.close();
	}

	/******************************************************************
	 * Messmodus y1- / ausschalten
	 ******************************************************************/
	private transient boolean autoscaleX;
	private transient boolean autoscaleY1;
	private transient boolean messModusAn = false;

	public void setzeMessmodus(boolean an) {
		Groesse g;
		if (an && !messModusAn) {
			autoscaleX = autoY[0];
			autoscaleY1 = autoY[1];
			if (werte.isDynamisch()) {
				setAutoY(0, false);
				Messparameter mp = GHMessen.getApplication().getMainView().mp;
				if (autoY[0]) {
					if (mp != null) {
						if (mp.getModus() == Messparameter.MODUSXRAY
								&& mp.parameter != null) {
							setIntervall(0,
									new Intervall(mp.parameter.getBeta1(),
											mp.parameter.getBeta2()));
						} else if (mp.getMesszeit() > 0) {
							setIntervall(0,
									new Intervall(0.0, mp.getMesszeit()));
						} else if (mp.getIntervall() > 0
								&& mp.getWiederholung() > 0) {
							setIntervall(
									0,
									new Intervall(0.0, mp.getIntervall()
											* mp.getWiederholung()));
						}
					} else {
						setIntervall(0, new Intervall(0.0, 10.0));
					}
				}
				setAutoY(1, false);
				if (autoscaleY1) {
					g = Util.findeGroesse(GHMessen.getApplication()
							.getMainView().gl, getY(1));
					if (g != null) {
						if (mp.getModus() == Messparameter.MODUSXRAY) {
							setIntervall(1, new Intervall(0.0, g.getBis()));
						} else {
							setIntervall(1, new Intervall(g.getMessbereich()
									.getMinimum(), g.getMessbereich()
									.getMaximum()));
						}
					} else {
						setIntervall(1, new Intervall(-5.0, 5.0));
					}
				}
			}
			messModusAn = an;
		} else if (messModusAn && !an) {
			werte.aktualisiereMinMax();
			if (autoscaleX != autoY[1])
				setAutoY(0, autoscaleX);
			if (autoscaleY1 != autoY[1])
				setAutoY(1, autoscaleY1);
			messModusAn = an;
		}
	}

	/********************************************************************
	 * Methoden zum speichern und laden
	 ********************************************************************/
	private void readObject(ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		in.defaultReadObject();
		restoreListenings();
		listenerList = new EventListenerList();
		support = new PropertyChangeSupport(this);
	}

	public void restoreListenings() {
		if (GHMessen.getApplication() != null
				&& GHMessen.getApplication().getMainView() != null) {
			werte = GHMessen.getApplication().getMainView().gt;
			if (werte != null)
				werte.addTableModelListener(this);
		}
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		this.support.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		this.support.removePropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String property,
			PropertyChangeListener listener) {
		this.support.addPropertyChangeListener(property, listener);
	}

	public void removePropertyChangeListener(String property,
			PropertyChangeListener listener) {
		this.support.removePropertyChangeListener(property, listener);
	}

	public void setName(String s) {
		String alt = name;
		name = s;
		support.firePropertyChange("name", alt, s);
		autox = "Standard".equals(name);
	}

	public String getName() {
		return name;
	}

	public String toString() {
		return getName();
	}

	public void setAutox(boolean set) {
		boolean alt = autox;
		autox = set;
		support.firePropertyChange("autox", alt, set);
	}
	
	public boolean getAutox() {
		return autox;
	}
	
	public void setY1(int i) {
		int alt = y1;
		this.y1 = i;
		support.firePropertyChange("y1", alt, i);
		if (DEBUG)
			System.out.println(System.currentTimeMillis()+"[Darstellung] setY1("+i+"): " + (i == -1 ? "null" : getColumnSymbol(i)));
		if (autox && i != -1) {
			Groessenliste gl = GHMessen.getApplication().getMainView().gl;
			Groesse g = Util.findeGroesse(gl, getColumnSymbol(i));
			if (g != null) {
				XAchseTyp xtyp = g.getXAchse();
				if (xtyp.getIndex() < UNABHAENGIG)
					setX(xtyp.getIndex());
				if (DEBUG)
					System.out.println(System.currentTimeMillis()+"[Darstellung] setY1: autox(" + autox + "|" + xtyp
							+ ")");
			}
		}
	}

	public void setY2(int i) {
		int alt = y2;
		this.y2 = i;
		support.firePropertyChange("y2", alt, i);
	}

	public int getY1() {
		return y1;
	}

	public int getY2() {
		return y2;
	}

	public int indexOf(String s) {
		s = Util.htmlSubRemove(s);
		for (int i = 0; i < spalten.size(); i++)
			if (getColumnSymbol(i).equals(s))
				return i;
		return -1;

	}

	/*************************************************************
	 * ACHSEN
	 *************************************************************/
	public void setPolar(boolean polar) {
		boolean alt = balken[XACHSE];
		this.balken[XACHSE] = polar;
		support.firePropertyChange("polar", alt, polar);
	}

	public boolean isPolar() {
		return balken[XACHSE];
	}

	public void setX(Groesse x) {
		setX(x.getSymbol());
	}

	public void setX(String x) {
		setX(convertToColumn(x));
	}

	public void setZeitmessung() {
		setX(0);
	}
	
	public void setManuelleMessung() {
		setX(1);
	}
	
	private void setX(int col) {
		setColumn(XACHSE, col);
		entferneAlleAuswertungen();
		fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW,
				XACHSE, UPDATE));
	}

	public String getX() {
		return werte.getColumnSymbol(spalten.elementAt(XACHSE).intValue());
	}

	public void setBalken(int i, boolean balken) {
		boolean alt = this.balken[i];
		this.balken[i] = balken;
		support.firePropertyChange("balken" + i, alt, balken);
	}

	public boolean isBalken(int i) {
		return balken[i];
	}

	public void setAutoY(int i, boolean an) {
		boolean alt = autoY[i];
		autoY[i] = an;
		if (i == 0)
			autoscaleX = an;
		else if (i == 1)
			autoscaleY1 = an;
		if (!an) {
			if (minimum.get(i) == null)
				minimum.set(i, werte.getMinimum(spalten.get(i)));
			if (maximum.get(i) == null)
				maximum.set(i, werte.getMaximum(spalten.get(i)));
		}
		support.firePropertyChange("autoY" + i, alt, an);
	}

	public boolean getAutoY(int i) {
		return autoY[i];
	}

	public void setYS(int i, Skalierung s) {
		Skalierung alt = yS[i];
		this.yS[i] = s;
		support.firePropertyChange("yS" + i, alt, s);
	}

	public Skalierung getYS(int i) {
		return yS[i];
	}

	public void setY(int i, Groesse y) {
		setY(i, y.getSymbol());
	}

	public void setY(int i, String y) {
		int type;
		if (y.equals("aus")) {
			removeColumn(i);
			type = DELETE;
		} else if (i < spalten.size()) {
			setColumn(i, convertToColumn(y));
			type = UPDATE;
		} else {
			addColumn(convertToColumn(y));
			type = INSERT;
		}
		entferneAlleAuswertungenFuerSpalte(i);
		fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW, i,
				type));
	}

	public String getY(int i) {
		if (i > spalten.size())
			return "aus";
		else
			return werte.getColumnSymbol(spalten.elementAt(i).intValue());
	}

	/*************************************************************
	 * Spalten
	 *************************************************************/
	private void addColumn(int werteCol) {
		int col = spalten.size();
		if (DEBUG)
			System.out.println("[Darstellung] - addColumn(" + col + " -> "
					+ werteCol + ")");
		if (col < 0 || col > MAX_Y) {
			if (DEBUG)
				System.out.println("[Darstellung] - addColumn(" + col
						+ ") - FEHLER(Der Index ist zu groß.)");
			return;
		}
		spalten.addElement(werteCol);
		minimum.addElement(null);
		maximum.addElement(null);
		if(y1 < 1 || col == 1)
			setY1(col);
		if(y2 < 1 || col <= 2)
			setY2(col);
		yS[col] = Skalierung.y;
		autoY[col] = true;
		balken[col] = false;
	}

	private void addColumn(String x) {
		addColumn(convertToColumn(x));
	}

	private void setColumn(int col, int werteCol) {
		if (DEBUG)
			System.out.println("[Darstellung] - setColumn(" + col + " -> "
					+ werteCol + ") maxCol(" + spalten.size() + ")");
		if (col >= spalten.size())
			return;
		if (werteCol == spalten.get(col))
			return;
		spalten.set(col, werteCol);
		minimum.set(col, null);
		maximum.set(col, null);
		if(y1 < 1)
			setY1(col);
		if(y2 < 1)
			setY2(col);
		yS[col] = Skalierung.y;
		autoY[col] = true;
		balken[col] = false;
	}

	private void removeColumn(int col) {
		if (col >= spalten.size())
			return;
		spalten.remove(col);
		minimum.remove(col);
		maximum.remove(col);
		if(y1 > col){
			setY1(y1-1);
		} else if(y1 == col){
			if(spalten.size() > 1)
				setY1(1);
			else
				setY1(-1);
		}
		if(y2 > col)
			setY2(y2-1);
		else if(y2 == col){
			if(spalten.size() > 1)
				setY2(spalten.size()-1);
			else
				setY2(-1);
		}
	}

	/*************************************************************
	 * Skalierung
	 *************************************************************/

	public void setWerteTabelle(WerteTabelle werte) {
		if (this.werte != null) {
			this.werte.removeTableModelListener(this);
		}
		this.werte = werte;
		this.werte.addTableModelListener(this);
	}

	public WerteTabelle getWerteTabelle() {
		return werte;
	}

	/*************************************************************
	 * Erweiterung Nullzeilen
	 *************************************************************/
	public boolean emptyRowsExist() {
		if (werte == null)
			return false;
		return werte.emptyRowsExist();
	}

	public int getEmptyRowCount() {
		if (werte == null)
			return 0;
		return werte.getEmptyRowCount();
	}

	public Iterator<Integer> getEmptyRowIterator() {
		if (werte == null)
			return new Vector<Integer>().iterator();
		return werte.getEmptyRowIterator();
	}

	public int getAbschnitt(int index) {
		if (werte == null)
			return 0;
		return werte.getAbschnitt(index);
	}

	/*************************************************************
	 * TableModel
	 *************************************************************/

	@Override
	public int getRowCount() {
		return werte.getRowCount();
	}

	@Override
	public int getColumnCount() {
		return spalten.size();
	}

	public Double[] getColumnArray(int col, int zeile1, int zeile2) {
		if (col < 0 || col >= spalten.size() || werte == null)
			return null;
		return werte.getColumnArray(spalten.get(col), zeile1, zeile2);
	}

	public Double[] getColumnArray(int col, int zeile1, int zeile2, double eFaktor) {
		return getScaledColumnArray(col, zeile1, zeile2, eFaktor);
	}
	
	public Double[] getScaledColumnArray(int col, int zeile1, int zeile2) {
		return getScaledColumnArray(col, zeile1, zeile2, 1.0);
	}

	public Double[] getScaledColumnArray(int col, int zeile1, int zeile2, double eFaktor) {
		if (col < 0 || col >= spalten.size() || werte == null || eFaktor == 0)
			return null;
		Double[] dd = werte.getColumnArray(spalten.get(col), zeile1, zeile2);
		if (dd == null)
			return null;
		for (int index = 0; index < dd.length; index++)
			dd[index] = yS[col].skaliere(dd[index]/eFaktor);
		return dd;		
	}
	@Override
	public String getColumnName(int columnIndex) {
		if (columnIndex >= spalten.size())
			throw new IndexOutOfBoundsException();
		return werte.getColumnName(spalten.elementAt(columnIndex));
	}

	public String getColumnSymbol(int columnIndex) {
		if (columnIndex >= spalten.size())
			throw new IndexOutOfBoundsException();
		return werte.getColumnSymbol(spalten.elementAt(columnIndex));
	}

	public String getColumnEinheit(int columnIndex) {
		if (columnIndex >= spalten.size())
			throw new IndexOutOfBoundsException();
		return werte.getColumnEinheit(spalten.elementAt(columnIndex));
	}

	public double getColumnFaktor(int columnIndex) {
		if (columnIndex >= spalten.size())
			throw new IndexOutOfBoundsException();
		return werte.getColumnFaktor(spalten.elementAt(columnIndex));
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		if (columnIndex >= spalten.size())
			throw new IndexOutOfBoundsException();
		else
			return Double.class;
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}

	public Double getDoubleValueAt(int rowIndex, int columnIndex) {
		return getValueAt(rowIndex, columnIndex);
	}

	public Double getDoubleValueAt(int rowIndex, int columnIndex, double eFaktor) {
		return getScaledDoubleValueAt(rowIndex, columnIndex, 1.0);
	}
	
	public Double getScaledDoubleValueAt(int rowIndex, int colIndex) {
		return getScaledDoubleValueAt(rowIndex, colIndex, 1.0);
	}

	public Double getScaledDoubleValueAt(int rowIndex, int colIndex, double eFaktor) {
		Double val = getDoubleValueAt(rowIndex, colIndex);
		return eFaktor == 0 ? yS[colIndex].skaliere(val) 
				: yS[colIndex].skaliere(val / eFaktor);		
	}
	
	@Override
	public Double getValueAt(int rowIndex, int columnIndex) {
		if (columnIndex >= spalten.size() || rowIndex >= werte.getRowCount())
			return null;
		else
			return (Double) werte.getValueAt(rowIndex,
					spalten.elementAt(columnIndex));
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (columnIndex >= spalten.size())
			throw new IndexOutOfBoundsException();
		else {
			columnIndex = spalten.elementAt(columnIndex);
			werte.setValueAt(aValue, rowIndex, columnIndex);
			fireTableCellUpdated(rowIndex, columnIndex);
		}
	}

	public void entferneBereich(int col, int row1, int row2) {
		if (row1 > row2) {
			int tmp = row1;
			row1 = row2;
			row2 = tmp;
		}
		for (int row = row1; row <= row2; row++)
			setValueAt(null, row, col);
	}

	public int getRowIndexOfNearest(Double aValue, int col)
			throws IndexOutOfBoundsException {
		if (col < 0 || col >= spalten.size())
			throw new IndexOutOfBoundsException();
		else {
			col = spalten.elementAt(col);
			return werte.getRowIndexOfNearest(aValue, col);
		}
	}

	public int getScaledRowIndexOfNearest(Double aValue, int col)
			throws IndexOutOfBoundsException {
		if (col < 0 || col >= spalten.size())
			throw new IndexOutOfBoundsException();
		else {
			return getRowIndexOfNearest(yS[col].unSkaliere(aValue), col);
		}
	}

	public int getColIndexOfNearest(Double aValue, int row)
			throws IndexOutOfBoundsException {
		if (row < 0 || row >= getRowCount())
			throw new IndexOutOfBoundsException();
		else
			return werte.getColIndexOfNearest(aValue, row);
	}

	public int getScaledColIndexOfNearest(Double aValue, int row)
			throws IndexOutOfBoundsException {
		if (row < 0 || row >= getRowCount())
			throw new IndexOutOfBoundsException();
		else {
			Vector<Double> rowData = new Vector<Double>(spalten.size());
			for (int i = 0; i < spalten.size(); i++)
				rowData.add(getScaledDoubleValueAt(row, i));
			int result = 1;
			Double value = rowData.get(result);
			double diff = Math.abs(aValue - (value == null ? 0 : value));
			double diffNeu;
			int index = 2;
			while (index < getColumnCount()) {
				value = rowData.get(index);
				diffNeu = Math.abs(aValue - (value == null ? 0 : value));
				if (diffNeu < diff) {
					result = index;
					diff = diffNeu;
				}
				index++;
			}
			return result;
		}
	}

	public void addRow(double[] values) {
		Vector<Double> data = new Vector<Double>(werte.getColumnCount());
		for (int i = 0; i < werte.getColumnCount(); i++)
			data.add(new Double(0));
		for (int i = 0; i < spalten.size(); i++)
			data.setElementAt(values[i], spalten.elementAt(i));
		werte.addRow(data);
	}

	public void removeRow(int row) {
		werte.removeRow(row);
		entferneAuswertungenMitZeile(row);
	}

	public void setNumRows(int rows) {
		werte.setNumRows(rows);
		if (rows == 0)
			entferneAlleAuswertungen();
		entferneAuswertungenHinterZeile(rows - 1);
	}

	public void setIntervall(int spalte, Intervall intervall) {
		if (spalte >= spalten.size())
			return;
		Intervall alt = new Intervall(minimum.get(spalte), maximum.get(spalte));
		minimum.setElementAt(intervall.getMin(), spalte);
		maximum.setElementAt(intervall.getMax(), spalte);
		support.firePropertyChange("intervall" + spalte, alt, intervall);
	}

	public void setMinimum(int spalte, Double min) {
		if (spalte >= spalten.size())
			return;
		Double alt = minimum.get(spalte);
		minimum.setElementAt(min, spalte);
		support.firePropertyChange("minimum" + spalte, alt, min);
	}

	private Double getScaledMinOrMax(int spalte, Skalierung s, boolean maximum) {
		if (spalten == null || spalte >= spalten.size())
			return null;
		Double result = null;
		spalte = spalten.get(spalte);
		int rows = getRowCount();
		if (spalte < 0 || spalte >= werte.getColumnCount()) {
			return null;
		}
		@SuppressWarnings("rawtypes")
		Vector<Vector> w = werte.getDataVector();
		Double wert;
		for (int row = 0; row < rows; row++) {
			wert = s.skaliere((Double) w.get(row).get(spalte));
			result = maximum ? WerteTabelle.findeMaximum(wert, result)
					: WerteTabelle.findeMinimum(wert, result);
		}
		if(result == null) {
			SymbolNameEinheit symbol = werte.getColumnSymbolNameEinheit(spalte);
			result = maximum ? 
						WerteTabelle.findeMaximum(s.skaliere(symbol.getMinimum()), s.skaliere(symbol.getMaximum()))
						: WerteTabelle.findeMinimum(s.skaliere(symbol.getMinimum()), s.skaliere(symbol.getMaximum()));
		}
		return result;
	}

	public Double getMinimum(int spalte) {
		if (spalte >= spalten.size())
			return null;
		if (!autoY[spalte])
			return minimum.elementAt(spalte);
		if (yS[spalte] == Skalierung.y)
			return werte.getMinimum(spalten.get(spalte));
		else
			return getScaledMinOrMax(spalte, yS[spalte], false);
	}

	public void setMaximum(int spalte, Double max) {
		if (spalte >= spalten.size())
			return;
		Double alt = maximum.get(spalte);
		maximum.setElementAt(max, spalte);
		support.firePropertyChange("maximum" + spalte, alt, max);
	}

	public Double getMaximum(int spalte) {
		if (spalte >= spalten.size())
			return null;
		if (!autoY[spalte])
			return maximum.elementAt(spalte);
		if (yS[spalte] == Skalierung.y)
			return werte.getMaximum(spalten.get(spalte));
		else
			return getScaledMinOrMax(spalte, yS[spalte], true);
	}

	public void findeMinMax(int spalte, Double[] minMax) {
		if (spalte >= spalten.size())
			return;
		if (yS[spalte] == Skalierung.y)
			werte.findeMinMax(spalten.get(spalte), minMax);
		else {
			minMax[0] = getScaledMinOrMax(spalte, yS[spalte], false);
			minMax[1] = getScaledMinOrMax(spalte, yS[spalte], true);
		}

	}

	/*****************************************************************
	 * TableColumn Methoden
	 *****************************************************************/
	public TableCellRenderer getColumnRenderer(int col) {
		if (col < 0 || col >= spalten.size())
			return null;
		else {
			return werte.getColumnRenderer(spalten.get(col));
		}
	}

	/*****************************************************************
	 * TableModel Events
	 *****************************************************************/
	@Override
	public void addTableModelListener(TableModelListener l) {
		listenerList.add(TableModelListener.class, l);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
		listenerList.remove(TableModelListener.class, l);
	}

	public TableModelListener[] getTableModelListeners() {
		return (TableModelListener[]) listenerList
				.getListeners(TableModelListener.class);
	}

	public void fireTableDataChanged() {
		fireTableChanged(new TableModelEvent(this));
	}

	public void fireTableStructureChanged() {
		fireTableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
	}

	public void fireTableRowsInserted(int firstRow, int lastRow) {
		fireTableChanged(new TableModelEvent(this, firstRow, lastRow,
				TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
	}

	public void fireTableRowsUpdated(int firstRow, int lastRow) {
		fireTableChanged(new TableModelEvent(this, firstRow, lastRow,
				TableModelEvent.ALL_COLUMNS, TableModelEvent.UPDATE));
	}

	public void fireTableRowsDeleted(int firstRow, int lastRow) {
		fireTableChanged(new TableModelEvent(this, firstRow, lastRow,
				TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
	}

	public void fireTableCellUpdated(int row, int column) {
		fireTableChanged(new TableModelEvent(this, row, row, column));
	}

	public void fireTableChanged(TableModelEvent e) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == TableModelListener.class) {
				((TableModelListener) listeners[i + 1]).tableChanged(e);
			}
		}
	}

	public <T extends EventListener> T[] getListeners(Class<T> listenerType) {
		return listenerList.getListeners(listenerType);
	}

	protected int convertToColumn(String s) {
		for (int i = 0; i < werte.getColumnCount(); i++)
			if (werte.getColumnSymbol(i).equals(s))
				return i;
		throw new IndexOutOfBoundsException();
	}

	private void refireTableChanged(TableModelEvent e) {
		Integer column = e.getColumn();
		if (column != TableModelEvent.ALL_COLUMNS)
			column = spalten.indexOf(column);
		TableModelEvent evt = new TableModelEvent(this, e.getFirstRow(),
				e.getLastRow(), column, e.getType());
		fireTableChanged(evt);
	}

	@Override
	public void tableChanged(TableModelEvent e) {
		boolean refire = false;
		if (e.getFirstRow() == TableModelEvent.HEADER_ROW) {
			switch (e.getType()) {
			case DELETE:
				for (int i = 0; i < getColumnCount(); i++)
					if (spalten.get(i) == e.getColumn()) {
						removeColumn(i);
						refire = true;
					}
				break;
			case UPDATE:
				// Eine Spaltenbezeichnung hat sich geändert.
				if(auswertung != null && e.getColumn() == XACHSE) {
					for(Auswertung a : auswertung) {
						if(a instanceof Anpassung)
							((Anpassung)a).setVonBis(minimum.get(XACHSE),
									maximum.get(XACHSE));
					}
				}
				refire = true;
				break;
			case INSERT:
				if (e.getColumn() > UNABHAENGIG) {
					String s = werte.getColumnSymbol(e.getColumn());
					if ("Standard".equals(name)) {
						// alle Größen zur Standard-Darstellung hinzufügen
						addColumn(e.getColumn());
						refire = true;
					} else if (s.contains("#")) {
						// abhängige Spalten hinzufügen
						s = s.substring(0, s.indexOf("#"));
						for (int i = 1; i < getColumnCount(); i++) {
							if (s.equals(werte.getColumnSymbol(spalten.get(i)))) {
								addColumn(e.getColumn());
								refire = true;
								break;
							}
						}
					}
				}
				break;
			}
		} else if (e.getType() == UPDATE) {
			// Editieren der Tabelle
			refire = true;
		} else
			refire = true;
		if (refire)
			refireTableChanged(e);
	}

	public void setGraphEigenschaften(Eigenschaften graph) {
		Eigenschaften alt = this.graph;
		this.graph = graph;
		support.firePropertyChange("graph", alt, graph);
	}

	public Eigenschaften getGraphEigenschaften() {
		return graph;
	}

	/*************************************************************
	 * Auswertung
	 * 
	 *************************************************************/

	public Vector<Auswertung> getAuswertung() {
		return auswertung;
	}

	public void neueAuswertung(Auswertung neu) {
		if (this.auswertung == null)
			auswertung = new Vector<Auswertung>();
		auswertung.add(neu);
		support.firePropertyChange("neueAuswertung", null, neu);
	}

	public void neueAuswertungen(Vector<Auswertung> neu) {
		if (this.auswertung == null)
			auswertung = new Vector<Auswertung>();
		auswertung.addAll(neu);
		support.firePropertyChange("neueAuswertungen", null, neu);
	}

	public void aendereAuswertung(Auswertung geaendert) {
		if (auswertung == null)
			neueAuswertung(geaendert);
		int index = auswertung.indexOf(geaendert);
		if (index < -1) {
			Auswertung alt = auswertung.get(index);
			auswertung.set(index, geaendert);
			support.firePropertyChange("aendereAuswertung", alt, geaendert);
		} else
			neueAuswertung(geaendert);
	}

	public Auswertung entferneAuswertung(int index) {
		if (auswertung == null || index < 0 || index >= auswertung.size())
			return null;
		Auswertung alt = auswertung.remove(index);
		support.firePropertyChange("entferneAuswertung", alt, null);
		return alt;
	}

	public Auswertung entferneAuswertung(Auswertung alt) {
		if (auswertung == null)
			return alt;
		int index = auswertung.indexOf(alt);
		if (index < 0)
			return alt;
		else
			return entferneAuswertung(index);
	}

	public void entferneAuswertungen(Vector<Auswertung> weg) {
		if (weg == null || auswertung == null)
			return;
		for (int i = 0; i < weg.size(); i++)
			entferneAuswertung(weg.get(i));
	}

	public void entferneAlleAuswertungen() {
		if (auswertung == null)
			return;
		while (auswertung.size() > 0)
			entferneAuswertung(auswertung.firstElement());
	}

	private void entferneAuswertungenMitZeile(int zeile) {
		if (auswertung == null)
			return;
		Vector<Auswertung> weg = new Vector<Auswertung>();
		for (Auswertung a : auswertung) {
			if (a.enthaeltZeile(zeile))
				weg.add(a);
		}
		entferneAuswertungen(weg);
	}

	private void entferneAuswertungenHinterZeile(int zeile) {
		if (auswertung == null || zeile < 0)
			return;
		Vector<Auswertung> weg = new Vector<Auswertung>();
		for (Auswertung a : auswertung)
			if (a.getEndeZeile() > zeile)
				weg.add(a);
		entferneAuswertungen(weg);
	}

	private void entferneAlleAuswertungenFuerSpalte(int spalte) {
		if (auswertung == null || spalte < 1)
			return;
		Vector<Auswertung> weg = new Vector<Auswertung>();
		for (Auswertung a : auswertung)
			if (a.getSpalte() == spalte)
				weg.add(a);
		entferneAuswertungen(weg);
	}

}
