/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import static ghmessen.groesse.Grundeinheit.AMPERE;
import static ghmessen.groesse.Grundeinheit.ATM;
import static ghmessen.groesse.Grundeinheit.BAR;
import static ghmessen.groesse.Grundeinheit.CELSIUS;
import static ghmessen.groesse.Grundeinheit.COULOMB;
import static ghmessen.groesse.Grundeinheit.DEZIBEL;
import static ghmessen.groesse.Grundeinheit.ELEKTRONENVOLT;
import static ghmessen.groesse.Grundeinheit.FAHRENHEIT;
import static ghmessen.groesse.Grundeinheit.G;
import static ghmessen.groesse.Grundeinheit.GRAD;
import static ghmessen.groesse.Grundeinheit.HERTZ;
import static ghmessen.groesse.Grundeinheit.INHG;
import static ghmessen.groesse.Grundeinheit.KELVIN;
import static ghmessen.groesse.Grundeinheit.KILOGRAMM;
import static ghmessen.groesse.Grundeinheit.LITER_SEKUNDE;
import static ghmessen.groesse.Grundeinheit.METER;
import static ghmessen.groesse.Grundeinheit.METER_SEKUNDE;
import static ghmessen.groesse.Grundeinheit.METER_SEKUNDE2;
import static ghmessen.groesse.Grundeinheit.MMHG;
import static ghmessen.groesse.Grundeinheit.NEWTON;
import static ghmessen.groesse.Grundeinheit.NEWTON_KILOGRAMM;
import static ghmessen.groesse.Grundeinheit.NEWTON_METER;
import static ghmessen.groesse.Grundeinheit.PASCAL;
import static ghmessen.groesse.Grundeinheit.PFUND;
import static ghmessen.groesse.Grundeinheit.PSI;
import static ghmessen.groesse.Grundeinheit.SEKUNDE;
import static ghmessen.groesse.Grundeinheit.SIEMENS_METER;
import static ghmessen.groesse.Grundeinheit.TESLA;
import static ghmessen.groesse.Grundeinheit.TORR;
import static ghmessen.groesse.Grundeinheit.VOLT;
import static ghmessen.groesse.Grundeinheit.VOLT_METER;
import static ghmessen.groesse.Grundeinheit.WATT;
import static ghmessen.groesse.Grundeinheit._SEKUNDE;
import static ghmessen.groesse.Vielfachheit.DREISECHSH;
import static ghmessen.groesse.Vielfachheit.EINS;
import static ghmessen.groesse.Vielfachheit.HEKTO;
import static ghmessen.groesse.Vielfachheit.KILO;
import static ghmessen.groesse.Vielfachheit.MEGA;
import static ghmessen.groesse.Vielfachheit.MIKRO;
import static ghmessen.groesse.Vielfachheit.MILLI;
import static ghmessen.groesse.Vielfachheit.NANO;
import static ghmessen.groesse.Vielfachheit.PICO;
import static ghmessen.groesse.Vielfachheit.SECHZIG;
import static ghmessen.groesse.Vielfachheit.SECHZIGSTEL;
import static ghmessen.groesse.Vielfachheit.ZEHNTAUSENDSTEL;
import static ghmessen.groesse.Vielfachheit.ZEHNTEL;
import static ghmessen.groesse.Vielfachheit.ZENTI;

import java.io.Serializable;

public class Einheit implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -6453811549802661598L;
	public static final Einheit nm = new Einheit(NANO, METER);
	public static final Einheit μm = new Einheit(MIKRO, METER);
	public static final Einheit mm = new Einheit(MILLI, METER);
	public static final Einheit cm = new Einheit(ZENTI, METER);
	public static final Einheit m = new Einheit(EINS, METER);
	public static final Einheit ns = new Einheit(NANO, SEKUNDE);
	public static final Einheit μs = new Einheit(MIKRO, SEKUNDE);
	public static final Einheit ms = new Einheit(MILLI, SEKUNDE);
	public static final Einheit s = new Einheit(SEKUNDE);
	public static final Einheit min = new Einheit("min", SECHZIG, SEKUNDE);
	public static final Einheit h = new Einheit("h", DREISECHSH, SEKUNDE);
	public static final Einheit mV = new Einheit(MILLI, VOLT);
	public static final Einheit V = new Einheit(VOLT);
	public static final Einheit kV = new Einheit(KILO, VOLT);
	public static final Einheit pA = new Einheit(PICO, AMPERE);
	public static final Einheit nA = new Einheit(NANO, AMPERE);
	public static final Einheit μA = new Einheit(MIKRO, AMPERE);
	public static final Einheit mA = new Einheit(MILLI, AMPERE);
	public static final Einheit A = new Einheit(AMPERE);
	public static final Einheit _C = new Einheit("°C", EINS, CELSIUS);
	public static final Einheit K = new Einheit(KELVIN);
	public static final Einheit F = new Einheit(FAHRENHEIT);
	public static final Einheit μN = new Einheit(MIKRO, NEWTON);
	public static final Einheit mN = new Einheit(MILLI, NEWTON);
	public static final Einheit cN = new Einheit(ZENTI, NEWTON);
	public static final Einheit N = new Einheit(NEWTON);
	public static final Einheit kN = new Einheit(KILO, NEWTON);
	public static final Einheit Keine = new Einheit(EINS, Grundeinheit.Keine);
	public static final Einheit Hz = new Einheit(HERTZ);
	public static final Einheit kHz = new Einheit(KILO, HERTZ);
	public static final Einheit _s = new Einheit("1/s", EINS, _SEKUNDE);
	public static final Einheit _min = new Einheit("1/min", SECHZIGSTEL,
			_SEKUNDE);
	public static final Einheit m_s = new Einheit("m/s", EINS, METER_SEKUNDE);
	public static final Einheit cm_s = new Einheit("cm/s", ZENTI, METER_SEKUNDE);
	public static final Einheit mm_s = new Einheit("mm/s", MILLI, METER_SEKUNDE);
	public static final Einheit m_s2 = new Einheit("m/s^2", EINS,
			METER_SEKUNDE2);
	public static final Einheit cm_s2 = new Einheit("cm/s^2", ZENTI,
			METER_SEKUNDE2);
	public static final Einheit mm_s2 = new Einheit("mm/s^2", MILLI,
			METER_SEKUNDE2);
	public static final Einheit _g = new Einheit("°", EINS, GRAD);
	public static final Einheit mS_cm = new Einheit("mS/cm", ZEHNTEL,
			SIEMENS_METER);
	public static final Einheit μS_cm = new Einheit("μS/cm", ZEHNTAUSENDSTEL,
			SIEMENS_METER);
	public static final Einheit W = new Einheit(WATT);
	public static final Einheit mW = new Einheit(MILLI, WATT);
	public static final Einheit kW = new Einheit(KILO, WATT);
	public static final Einheit eV = new Einheit(ELEKTRONENVOLT);
	public static final Einheit keV = new Einheit(KILO, ELEKTRONENVOLT);
	public static final Einheit MeV = new Einheit(MEGA, ELEKTRONENVOLT);
	public static final Einheit C = new Einheit(EINS, COULOMB);
	public static final Einheit mC = new Einheit(EINS, COULOMB);
	public static final Einheit μC = new Einheit(MIKRO, COULOMB);
	public static final Einheit nC = new Einheit(NANO, COULOMB);
	public static final Einheit pC = new Einheit(PICO, COULOMB);
	public static final Einheit g = new Einheit(EINS, G, "G");
	public static final Einheit dB = new Einheit(EINS, DEZIBEL);
	public static final Einheit T = new Einheit(EINS, TESLA);
	public static final Einheit mT = new Einheit(MILLI, TESLA);
	public static final Einheit hPa = new Einheit(HEKTO, PASCAL);
	public static final Einheit l_s = new Einheit("l/s", EINS, LITER_SEKUNDE);
	public static final Einheit N_m = new Einheit("N/m", EINS, NEWTON_METER);
	public static final Einheit kg = new Einheit(EINS, KILOGRAMM);
	public static final Einheit N_kg = new Einheit("N/kg", EINS,
			NEWTON_KILOGRAMM);
	public static final Einheit lb = new Einheit("lb", EINS, PFUND);
	public static final Einheit Torr = new Einheit(EINS, TORR, "torr");
	public static final Einheit mmHg = new Einheit("mm Hg", EINS, MMHG);
	public static final Einheit bar = new Einheit(EINS, BAR);
	public static final Einheit mbar = new Einheit(MILLI, BAR);
	public static final Einheit psi = new Einheit(EINS, PSI);
	public static final Einheit InHG = new Einheit("in Hg", EINS, INHG);
	public static final Einheit Atm = new Einheit("atm", EINS, ATM);
	public static final Einheit kPa = new Einheit(KILO, PASCAL);
	public static final Einheit kV_m = new Einheit(KILO, VOLT_METER);

	Vielfachheit vielfachheit;
	Grundeinheit grundeinheit;
	private String name;
	private String[] alias;

	private Einheit() {
		this(null, EINS, Grundeinheit.Keine);
	}

	private Einheit(String name) {
		this(name, EINS, Grundeinheit.Keine);
	}

	private Einheit(Grundeinheit grundeinheit) {
		this(grundeinheit.toString(), EINS, grundeinheit);
	}

	private Einheit(Vielfachheit vielfachheit, Grundeinheit grundeinheit) {
		this(vielfachheit.toString()+grundeinheit.toString(), vielfachheit, grundeinheit);
	}

	private Einheit(Vielfachheit vielfachheit, Grundeinheit grundeinheit,
			String... alias) {
		this(vielfachheit.toString()+grundeinheit.toString(), vielfachheit, grundeinheit, alias);
	}

	private Einheit(String name, Vielfachheit vielfachheit,
			Grundeinheit grundeinheit, String... alias) {
		if(name == null)
			this.name = vielfachheit.toString()+grundeinheit.toString();
		else
			this.name = name;
		this.vielfachheit = vielfachheit;
		this.grundeinheit = grundeinheit;
		this.alias = alias;
		synchronized (Einheit.class) {
			getMappings().put(name, this);
			if (this.alias != null)
				for (String s : alias)
					getMappings().put(s, this);
		}
	}

	private static java.util.HashMap<String, Einheit> mappings;

	private static java.util.HashMap<String, Einheit> getMappings() {
		if (mappings == null) {
			synchronized (Einheit.class) {
				if (mappings == null) {
					mappings = new java.util.HashMap<String, Einheit>();
				}
			}
		}
		return mappings;
	}

	@Override
	public String toString() {
		if (name != null)
			return name;
		else
			return vielfachheit.toString() + grundeinheit.toString();
	}

	public static Einheit toEinheit(String value) {
		synchronized (Einheit.class) {
			Einheit enumObj = getMappings().get(value);
			if (enumObj == null) {
				return new Einheit(value);
			} else {
				return enumObj;
			}
		}
	}

	public double getFaktor() {
		return vielfachheit.getFaktor();
	}

	public static String[] getZeitEinheitenStringArray() {
		return new String[] { "ns", "μs", "ms", "s", "min", "h" };
	}
}
