/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import static ghmessen.groesse.Einheit.A;
import static ghmessen.groesse.Einheit.C;
import static ghmessen.groesse.Einheit.Hz;
import static ghmessen.groesse.Einheit.K;
import static ghmessen.groesse.Einheit.Keine;
import static ghmessen.groesse.Einheit.N;
import static ghmessen.groesse.Einheit.T;
import static ghmessen.groesse.Einheit.V;
import static ghmessen.groesse.Einheit.W;
import static ghmessen.groesse.Einheit._C;
import static ghmessen.groesse.Einheit._g;
import static ghmessen.groesse.Einheit._s;
import static ghmessen.groesse.Einheit.cm;
import static ghmessen.groesse.Einheit.dB;
import static ghmessen.groesse.Einheit.g;
import static ghmessen.groesse.Einheit.hPa;
import static ghmessen.groesse.Einheit.l_s;
import static ghmessen.groesse.Einheit.m;
import static ghmessen.groesse.Einheit.mS_cm;
import static ghmessen.groesse.Einheit.m_s;
import static ghmessen.groesse.Einheit.m_s2;
import static ghmessen.groesse.Einheit.ms;
import static ghmessen.groesse.Einheit.s;
import static ghmessen.groesse.Messbereich.PHI;
import static ghmessen.groesse.Einheit.kV_m;

import ghmessen.util.Util;

import java.io.PrintWriter;
import java.lang.reflect.Field;

import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class Messgroesse {
	private static final boolean DEBUG = false;
	public static final Messgroesse Nichts = new Messgroesse("Nichts", Keine);
	public static final Messgroesse Weg = new Messgroesse("Weg", m, "s");
	public static final Messgroesse Zeit = new Messgroesse("Zeit", s, "t");
	public static final Messgroesse Spannung = new Messgroesse("Spannung", V, "U");
	public static final Messgroesse Stromstärke = new Messgroesse("Stromstärke", A, "I",
			"Stromstärke", "Strom");
	public static final Messgroesse Leistung = new Messgroesse("Leistung", W, "P");
	public static final Messgroesse Ladung = new Messgroesse("Ladung", C, "Q");
	public static final Messgroesse Phasenwinkel = new Messgroesse("Phasenwinkel", _g, "φ");
	public static final Messgroesse CosPhi = new Messgroesse("CosPhi", Keine, "φ");
	public static final Messgroesse CelsiusTemperatur = new Messgroesse("CelsiusTemperatur", _C,
			"ϑ", "Temperatur");
	public static final Messgroesse KelvinTemperatur = new Messgroesse("KelvinTemperatur", K, "T",
			"Temperatur");
	public static final Messgroesse Temperaturdifferenz = new Messgroesse("Temperaturdifferenz", K,
			"ΔT", "Differenz");
	public static final Messgroesse Kraft = new Messgroesse("Kraft", N, "F");
	public static final Messgroesse Frequenz = new Messgroesse("Frequenz", Hz, "f");
	public static final Messgroesse Periodendauer = new Messgroesse("Periodendauer", s, "T");
	public static final Messgroesse Impulse = new Messgroesse("Impulse", Keine, "I");
	public static final Messgroesse Geschwindigkeit = new Messgroesse("Geschwindigkeit", m_s, "v");
	public static final Messgroesse Beschleunigung = new Messgroesse("Beschleunigung", m_s2, "a",
			"Beschleunigung");
	public static final Messgroesse GBeschleunigung = new Messgroesse("GBeschleunigung", g, "a",
			"G-Beschleunigung");
	public static final Messgroesse Pegel = new Messgroesse("Pegel", Keine);
	public static final Messgroesse Ereignisse = new Messgroesse("Ereignisse", Keine, "N");
	public static final Messgroesse Rate = new Messgroesse("Rate", _s, "R");
	public static final Messgroesse Laufzeit = new Messgroesse("Laufzeit", s, "Δt");
	public static final Messgroesse Verdunkelung = new Messgroesse("Verdunkelung", Keine, "n");
	public static final Messgroesse Dunkelzeit = new Messgroesse("Dunkelzeit", ms, "t");
	public static final Messgroesse Winkel = new Messgroesse("Winkel", _g, "β");
	public static final Messgroesse RadiantWinkel = new Messgroesse("RadiantWinkel", Keine, "b",
			"Winkel");
	public static final Messgroesse LinearerStoß = new Messgroesse("LinearerStoß", m_s, "v");
	public static final Messgroesse Drehstoß = new Messgroesse("Drehstoß", _s, "ω");
	public static final Messgroesse Relais = new Messgroesse("Relais", Keine, "R");
	public static final Messgroesse Source = new Messgroesse("Source", Keine, "S");
	public static final Messgroesse Leitfähigkeit = new Messgroesse("Leitfähigkeit", mS_cm, "C");
	public static final Messgroesse pHWert = new Messgroesse("pHWert", Keine, "pH");
	public static final Messgroesse Schallpegel = new Messgroesse("Schallpegel", dB, "L");
	public static final Messgroesse Magnetfeldstärke = new Messgroesse("Magnetfeldstärke", T, "B",
			"Magnetfeldstärke", "Magn. Flussdichte", "Magn. Feldstärke");
	public static final Messgroesse Relativdruck = new Messgroesse("Relativdruck", hPa, "p");
	public static final Messgroesse Absolutdruck = new Messgroesse("Absolutdruck", hPa, "p");
	public static final Messgroesse Druck = new Messgroesse("Druck", hPa, "p");
	public static final Messgroesse Volumenfluss = new Messgroesse("Volumenfluss", l_s, "dV");
	public static final Messgroesse Gradamplitude = new Messgroesse("Gradamplitude", _g, "A",
			"Amplitude");
	public static final Messgroesse Radiantamplitude = new Messgroesse("Radiantamplitude", Keine,
			"Ab", "Amplitude");
	public static final Messgroesse Wegamplitude = new Messgroesse("Wegamplitude", cm, "As",
			"Amplitude");
	public static final Messgroesse Schalldruck = new Messgroesse("Schalldruck", Keine, "p");
	public static final Messgroesse Lichtpegel = new Messgroesse("Lichtpegel", Keine, "P");
	public static final Messgroesse ElektrischeFeldstärke = new Messgroesse("Elektrische Feldstärke", kV_m, "E",
			"E-Feldstärke");
	public Messgroesse() { }
	
	private Messgroesse(String name) {
		this(name, Einheit.Keine);
	}
	private Messgroesse(String name, Einheit einheit) {
		this(name, einheit, null);
	}

	private Messgroesse(String name, Einheit einheit, String symbol,
			String... alias) {
		this.name = name;
		this.einheit = einheit;
		this.symbol = symbol;
		this.alias = alias;
		synchronized (Messgroesse.class) {
			getMappings().put(name, this);
			if (this.alias != null)
				for (String s : alias)
					if(!getMappings().containsKey(s))
						getMappings().put(s, this);
			if(!getSymMappings().containsKey(symbol))
				getSymMappings().put(symbol, this);
		}
		if(DEBUG){
			System.out.println("[Messgroesse] erzeugt("+name+"|"+symbol+"|"+einheit+"|"+alias+")");
		}
	}

	private Einheit einheit;
	private String symbol;
	private String name;
	private String[] alias;
	private static java.util.HashMap<String, Messgroesse> mappings;
	private static java.util.HashMap<String, Messgroesse> symMappings;

	private static java.util.HashMap<String, Messgroesse> getMappings() {
		if (mappings == null) {
			synchronized (Messgroesse.class) {
				if (mappings == null) {
					mappings = new java.util.HashMap<String, Messgroesse>();
				}
			}
		}
		return mappings;
	}

	private static java.util.HashMap<String, Messgroesse> getSymMappings() {
		if (symMappings == null) {
			synchronized (Messgroesse.class) {
				if (symMappings == null) {
					symMappings = new java.util.HashMap<String, Messgroesse>();
				}
			}
		}
		return symMappings;
	}


	public Einheit getEinheit() {
		return einheit;
	}

	public String getSymbol() {
		return symbol;
	}

	public String getName() {
		return name;
	}

	@Override
	public String toString() {
		if(name != null)
			return name + " " + symbol;
		else
			return symbol;
	}
	
	public void setzeStandardWerte(Groesse g) {
		if (!g.isManuellerName())
			g.setName(name);
		g.setEinheit(einheit);
		if(this == Phasenwinkel)
			g.setMessbereich(PHI);
		g.getMessbereich().setzeStandardWerte(g);
	}

	public static String[] getNamen(Messgroesse[] m) {
		String[] result = new String[m.length];
		for (int i = 0; i < result.length; i++)
			result[i] = (m[i].getName() == null ? m[i].getSymbol()
								: m[i].getName());
		return result;
	}

	public static Messgroesse toMessgroesse(String value) {
		// value kann ein Name oder eine Kombination aus Name und Symbol oder ein Symbol sein.
		String sym = null;
		if(value != null && value.contains(" ")) {
			sym = value.substring(value.indexOf(" ")+1);
			if("".equals(sym))
					sym = null;
			value = value.substring(0, value.indexOf(" "));
		}
		synchronized (Messgroesse.class) {
			Messgroesse enumObj = getMappings().get(value);
			if (enumObj != null) {
				return enumObj;
			}
			if (sym != null) {
				enumObj = getSymMappings().get(sym);
				if (enumObj != null) {
					return enumObj;
				}
			} else {
				enumObj = getSymMappings().get(value);
				if (enumObj != null) {
					return enumObj;
				}
			}
			return new Messgroesse(value);
		}
	}

	public static Messgroesse toMessgroesse(String value, String unit, String symbol) {
		return toMessgroesse(value, Einheit.toEinheit(unit), symbol);
	}
	
	public static Messgroesse toMessgroesse(String value, Einheit einheit, String symbol) {
		synchronized (Messgroesse.class) {
			Messgroesse enumObj = getMappings().get(value);
			if (enumObj == null) {
				return new Messgroesse(value, einheit, symbol);
			} else if(enumObj.einheit != einheit){
				return new Messgroesse(value+"_"+einheit.toString(), einheit, symbol);
			} else {
				return enumObj;
			}
		}
	}

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT Messgroesse (Alias?)>");
		out.println("<!ATTLIST Messgroesse ");
		out.println("\tname CDATA #REQUIRED");
		out.println("\tsymbol CDATA #REQUIRED");
		out.println("\teinheit CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT Alias (#PCDATA)* >");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[Messgroesse] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "name","CDATA",name);
		atts.addAttribute("", "", "symbol","CDATA",symbol);
		atts.addAttribute("", "", "einheit","CDATA",einheit.toString());
		hd.startElement("", "", "Messgroesse", atts);
		hd.startElement("", "", "Alias", null);
		String out = "";
		for (String s : alias) {
			out = out + "\"" + s + "\",";
		}
		if (!"".equals(out))
			out = out.substring(0, out.length() - 1);
		hd.characters(out.toCharArray(), 0, out.length());
		hd.endElement("", "", "Alias");
		hd.endElement("", "", "Messgroesse");
	}

	public Messgroesse startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("Messgroesse")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					Field field = getClass().getDeclaredField(name);
					Util.setFieldValue(this, field, inhalt);
				} catch (SecurityException e) {
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					throw new SAXException(e.getMessage());
				}
			}
			if (name == null || symbol == null || einheit == null)
				return this;
			else
				try {
					return toMessgroesse(name, einheit, symbol);
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (SecurityException e) {
					e.printStackTrace();
				}
			return this;
		} else if (qName.equals("Alias")) {
			return this;
		} else
			throw new SAXException("[Messgroesse] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		if (qName.equals("Alias")) {
			alias = Util.aStringFromXml(inhalt, true);
			for(String s : alias)
				if(!getMappings().containsKey(s))
					getMappings().put(s, this);
		} else if (qName.equals("Messgroesse"))
			return;
		else
			throw new SAXException("[Messgroesse] - endElement(" + qName + ")");
	}

}
