/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import java.io.PrintWriter;

public enum Skalierung {
	x,x2,_x,_x2,logx,sqrtx, y,y2,_y,_y2,logy, sqrty;

	public static void createDTD(PrintWriter out){
		out.print("<!ENTITY Skalierung \"");
		String s = "";
		for(Skalierung e : values())
			s += e.toString() + "|";
		out.println(s.substring(0,s.length()-1)+"\">");
	}

	public String toString(){
		switch(this){
			case x: return "x";
			case y: return "y";
			case x2: return "x^2";
			case y2: return "y^2";
			case _x: return "1/x";
			case _y: return "1/y";
			case _x2: return "1/x^2";
			case _y2: return "1/y^2";
			case logx: return "log x";
			case logy: return "log y";
			case sqrtx: return "√x";
			case sqrty: return "√y";
			default: return "y";
		}
	}
	
	public String toString(String var){
		switch(this){
			case x:
			case y: 
				return var;
			case x2:
			case y2:
				return var + "<sup>2</sup>";
			case _x:
			case _y: 
				return "1/" + var;
			case _x2:
			case _y2: 
				return "1/" + var + "<sup>2</sup>";
			case logx:
			case logy: 
				return "log(" + var + ")";
			case sqrtx:
			case sqrty: 
				return "√" + var;
			default: 
				return var;
		}
	}
	
	public static Skalierung toSkalierung(String s){
		switch(s.charAt(0)){
			case 'x':
				return s.length()==1 ? x : x2;
			case 'y':
				return s.length()==1 ? y : y2;
			case 'l':
				return s.charAt(3)=='x' ? logx : logy;
			case '√':
				return s.charAt(1)=='x' ? sqrtx : sqrty;
			case '1':
				return s.length()==3 ? (s.charAt(2)=='x' ? _x : _y) : (s.charAt(2)=='x' ? _x2 : _y2);
			default: return x;
		}
	}
	
	public Double skaliere(Double x){
		if(x == null)
			return x;
		switch(this){
		default:
			return x;
		case x2:
		case y2:
			return x*x;
		case _x:
		case _y:
			return 1.0/x;
		case _x2:
		case _y2:
			return 1.0/(x*x);
		case logx:
		case logy:
			return Math.log10(x);
		case sqrtx:
		case sqrty:
			return Math.sqrt(x);
		}
	}

	public Double unSkaliere(Double x){
		if(x == null)
			return x;
		switch(this){
		default:
			return x;
		case x2:
		case y2:
			return Math.sqrt(x);
		case _x:
		case _y:
			return 1.0/x;
		case _x2:
		case _y2:
			return 1.0/Math.sqrt(x);
		case logx:
		case logy:
			return Math.exp(x);
		case sqrtx:
		case sqrty:
			return x*x;
		}
	}
	
	public boolean istLogarithmisch() {
		switch(this){
		default:
			return false;
		case logx:
		case logy:
			return true;
		}
	}
}
