/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import java.io.Serializable;

import ghmessen.components.Intervall;
import ghmessen.util.Util;

public class SymbolNameEinheit implements Serializable {
	private static final long serialVersionUID = -7108663521302948870L;
	String symbol;
	String einheit;
	int stellen;
	double minimum;
	double maximum;
	String html;
	
	public SymbolNameEinheit(String symbol){
		this(symbol,"", 1, -10.0, 10.0);
	}
	
	public SymbolNameEinheit(String symbol, String einheit, int stellen, double minimum, double maximum){
		this.symbol = symbol;
		this.einheit = einheit;
		this.stellen = stellen;
		this.minimum = minimum;
		this.maximum = maximum;
		createHtml();
	}
	
	public SymbolNameEinheit(String symbol, Einheit einheit, int stellen, double minimum, double maximum){
		this(symbol, einheit != null ? einheit.toString() : "", stellen, minimum, maximum);
	}
	
	@Override
	public String toString(){
		return html;
	}
	
	public void setSymbol(String symbol){
		this.symbol = symbol;
		createHtml();
	}
	
	public String getSymbol(){
		return symbol;
	}
	
	public void setEinheit(String einheit){
		this.einheit = einheit;
		createHtml();
	}
	
	public void setEinheit(Einheit einheit){
		setEinheit(einheit.toString());
	}
	
	public String getEinheit(){
		return einheit;
	}
	
	public int getStellen() {
		return stellen;
	}
	
	public void setStellen(int stellen) {
		this.stellen = stellen;
	}
	
	public double getMinimum() {
		return minimum;
	}
	
	public void setMinimum(double minimum) {
		this.minimum = minimum;
	}
	
	public double getMaximum() {
		return maximum;
	}
	
	public void setMaximum(double maximum) {
		this.maximum = maximum;
	}
	
	public Intervall getIntervall() {
		return new Intervall(minimum, maximum);
	}
	
	public void setIntervall(Intervall intervall) {
		this.minimum = intervall.getMin();
		this.maximum = intervall.getMax();
	}
	
	private void createHtml(){
		html = Util.htmlSymbolUndEinheit(symbol, einheit);
	}
	
}
