/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import static javax.swing.event.TableModelEvent.DELETE;
import static javax.swing.event.TableModelEvent.HEADER_ROW;
import static javax.swing.event.TableModelEvent.INSERT;
import static javax.swing.event.TableModelEvent.UPDATE;
import static ghmessen.groesse.wertetabelle.XAchseTyp.*;
import flanagan.analysis.Stat;
import flanagan.math.ArrayMaths;
import flanagan.math.FourierTransform;
import ghmessen.GHMessen;
import ghmessen.Konstanten;
import ghmessen.Messparameter;
import ghmessen.components.Intervall;
import ghmessen.groesse.wertetabelle.StellenRenderer;
import ghmessen.groesse.wertetabelle.TimeRenderer;
import ghmessen.groesse.wertetabelle.XAchseTyp;
import ghmessen.util.Util;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.xml.transform.sax.TransformerHandler;

import org.codehaus.janino.ExpressionEvaluator;
import org.jdesktop.observablecollections.ObservableList;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class WerteTabelle extends DefaultTableModel implements
		ObservableObjectListListener, Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7471753669036938809L;
	private static final boolean DEBUG = false;

	private Double[] minimum, maximum;
	private boolean autoBerechnen;
	private boolean dynamisch;
	private Vector<StellenRenderer> renderer;

	protected transient IntervallPropertyHandler intervallPH;
	protected transient MesszeitPropertyHandler messzeitPH;
	protected transient ModusPropertyHandler modusPH;

	private void writeObject(ObjectOutputStream out) throws IOException {
		// Die listenerList aus AbstractListModel ist nicht transient, daher
		// muss sie vor dem Speichern geleert werden.
		out.defaultWriteObject();
	}

	private void readObject(ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		// Die listenerList aus AbstractListModel ist nicht transient, daher
		// muss sie vor dem Speichern geleert werden.
		in.defaultReadObject();
	}

	public boolean isAutoBerechnen() {
		return autoBerechnen;
	}

	public void setAutoBerechnen(boolean an) {
		this.autoBerechnen = an;
	}

	public boolean isDynamisch() {
		return this.dynamisch;
	}

	public void setDynamisch(boolean dynamisch) {
		this.dynamisch = dynamisch;
	}

	public TableModelListener[] leereListenerListe() {
		TableModelListener[] sichereListe = getTableModelListeners().clone();
		for (TableModelListener l : getTableModelListeners())
			removeTableModelListener(l);
		return sichereListe;
	}

	public void fuelleListenerListe(TableModelListener[] list) {
		for (TableModelListener l : list)
			addTableModelListener(l);
	}

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT WerteTabelle (WTMinimum,WTMaximum,WTZeile*)>");
		out.println("<!ATTLIST WerteTabelle autoBerechnen (BOOLEAN) #REQUIRED dynamisch (BOOLEAN) #IMPLIED>");
		out.println("<!ELEMENT WTMinimum (DOUBLE*)>");
		out.println("<!ELEMENT WTMaximum (DOUBLE*)>");
		out.println("<!ELEMENT WTZeile (DOUBLE*)>");
	}

	@SuppressWarnings("unchecked")
	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[WerteTabelle] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "autoBerechnen", "BOOLEAN",
				Boolean.toString(autoBerechnen));
		atts.addAttribute("", "", "dynamisch", "BOOLEAN",
				Boolean.toString(dynamisch));
		String inhalt;
		hd.startElement("", "", "WerteTabelle", atts);
		hd.startElement("", "", "WTMinimum", null);
		inhalt = Util.writeDoublesToXml(minimum);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "WTMinimum");
		hd.startElement("", "", "WTMaximum", null);
		inhalt = Util.writeDoublesToXml(maximum);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "WTMaximum");
		for (@SuppressWarnings("rawtypes") Vector v : getDataVector()) {
			hd.startElement("", "", "WTZeile", null);
			inhalt = Util.writeDoublesToXml(v);
			hd.characters(inhalt.toCharArray(), 0, inhalt.length());
			hd.endElement("", "", "WTZeile");
		}
		hd.endElement("", "", "WerteTabelle");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("WerteTabelle")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					Field field = getClass().getDeclaredField(name);
					if (DEBUG)
						System.out
								.println("[WerteTabelle] - startElement: Feld("
										+ name + ") Typ(" + field.getType()
										+ ") Inhalt(" + inhalt + ")");
					Util.setFieldValue(this, field, inhalt);
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		} else if (qName.equals("WTMinimum") || qName.equals("WTMaximum")
				|| qName.equals("WTZeile")) {
			return;
		} else
			throw new SAXException("[WerteTabelle] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		if (qName.equals("WerteTabelle")) {
			return;
		} else if (qName.equals("WTMinimum")) {
			minimum = Util.doubleAFromXml(inhalt);
		} else if (qName.equals("WTMaximum")) {
			maximum = Util.doubleAFromXml(inhalt);
		} else if (qName.equals("WTZeile")) {
			addRow(Util.doubleVectorFromXml(inhalt));
		} else
			throw new SAXException("[WerteTabelle] - BEnde(" + qName + ")");
	}

	public WerteTabelle(Groessenliste gl, Messparameter mp) {
		super(0, 0);
		autoBerechnen = true;
		dynamisch = Konstanten.SKALIERUNG_DYNAMISCH_STANDARD;
		intervallPH = new IntervallPropertyHandler();
		messzeitPH = new MesszeitPropertyHandler();
		modusPH = new ModusPropertyHandler();
		renderer = new Vector<StellenRenderer>();
		minimum = new Double[0];
		maximum = new Double[0];
		assert_tnfblE_Columns(mp);
		if (dynamisch) {
			Double[] min = new Double[getColumnCount()], max = new Double[getColumnCount()];
			findeAlleMinMax(min, max);
			minimum = min;
			maximum = max;
		}
		if (gl != null) {
			Groesse g;
			for (int index = 0; index < gl.size(); index++) {
				g = gl.get(index);
				insertColumn(index + UNABHAENGIG, g.getSymbolNameEinheit(),
						g.getStellen(), g.getVon(), g.getBis());
			}
			gl.addObservableListListener(this);
		}
		if (mp != null) {
			mp.addPropertyChangeListener("intervall", this.intervallPH);
			mp.addPropertyChangeListener("messzeit", this.messzeitPH);
			mp.addPropertyChangeListener("anhaengenEin", this.modusPH);
		}
	}

	@SuppressWarnings("rawtypes")
	public Vector<Vector> getDataVector() {
		return dataVector;
	}

	public Double[] getColumnArray(int col, int zeile1, int zeile2) {
		@SuppressWarnings("rawtypes")
		Vector<Vector> dataVector = getDataVector();
		if (col < 0 || zeile1 < 0 || zeile1 > zeile2 || dataVector == null
				|| dataVector.size() == 0 || zeile2 >= dataVector.size()
				|| col >= dataVector.get(0).size())
			return null;
		Double[] ar = new Double[zeile2 - zeile1 + 1];
		for (int i = zeile1; i <= zeile2; i++)
			ar[i - zeile1] = (Double) dataVector.get(i).get(col);
		return ar;
	}

	public Object[] getColumnNames() {
		return columnIdentifiers.toArray();
	}

	public Object[] getColumnNames(int from, int to) {
		return columnIdentifiers.subList(from, to).toArray();
	}

	public SymbolNameEinheit[] getColumnSymbolNameEinheiten() {
		return (SymbolNameEinheit[]) columnIdentifiers.toArray();
	}

	public SymbolNameEinheit getColumnSymbolNameEinheit(int col) {
		if (columnIdentifiers == null || col >= columnIdentifiers.size())
			return null;
		else
			return (SymbolNameEinheit) columnIdentifiers.elementAt(col);	
	}
	
	public String getColumnSymbol(int col) {
		if (col >= getColumnCount())
			return null;
		Object result = columnIdentifiers.get(col);
		if (result instanceof SymbolNameEinheit)
			return ((SymbolNameEinheit) result).getSymbol();
		else
			return result.toString();
	}

	public String getColumnEinheit(int col) {
		if (col >= getColumnCount())
			return null;
		Object result = columnIdentifiers.get(col);
		if (result instanceof SymbolNameEinheit)
			return ((SymbolNameEinheit) result).getEinheit().toString();
		else
			return result.toString();
	}

	public double getColumnFaktor(int col) {
		if (col >= getColumnCount())
			return 1.0;
		Object result = columnIdentifiers.get(col);
		if (result instanceof SymbolNameEinheit)
			return Einheit.toEinheit(((SymbolNameEinheit) result).getEinheit()).getFaktor();
		else
			return 1.0;
	}

	public String[] getColumnSymbols() {
		return getColumnSymbols(0, getColumnCount());
	}

	public String[] getColumnSymbols(int from, int to) {
		if (to - from <= 0)
			return null;
		String[] result = new String[to - from];
		for (int i = from; i < to; i++)
			result[i - from] = ((SymbolNameEinheit) columnIdentifiers.get(i))
					.getSymbol();
		return result;
	}

	@SuppressWarnings("unchecked")
	public void setColumnIdentifier(int col, Object identifier) {
		if (col >= getColumnCount() || col >= columnIdentifiers.size())
			return;
		columnIdentifiers.set(col, identifier);
		if (!dynamisch && identifier instanceof SymbolNameEinheit) {
			minimum[col] = ((SymbolNameEinheit) identifier).getMinimum();
			maximum[col] = ((SymbolNameEinheit) identifier).getMaximum();
		}
		fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW, col,
				UPDATE));
	}

	public void setDataVector(Double[][] dataVector, String[] columnIdentifiers) {
		super.setDataVector(dataVector, columnIdentifiers);
	}

	protected void assert_tnfblE_Columns(Messparameter mp) {
		double zeitMax = mp != null ? Math.max(
				mp.getIntervall() * mp.getWiederholung(), mp.getMesszeit())
				: 120.0;
		if (getColumnCount() > 0) {
			insertColumn(T.ordinal(), new SymbolNameEinheit("t", "s", 3, 0.0,
					zeitMax), 3, 0.0, zeitMax);
			insertColumn(N.ordinal(), new SymbolNameEinheit("n", "", 0, 0.0,
					20.0), 0, 0.0, 20.0);
			insertColumn(F.ordinal(), new SymbolNameEinheit("f", "Hz", 0, 0.0,
					1000.0), 0, 0.0, 1000.0);
			insertColumn(BETA.ordinal(), new SymbolNameEinheit("β", "°", 1,
					0.0, 30.0), 1, 0.0, 30.0);
			insertColumn(LAMBDA.ordinal(), new SymbolNameEinheit("λ", "pm", 0,
					0.0, 1000.0), 0, 0.0, 1000.0);
			insertColumn(ENERGIE.ordinal(), new SymbolNameEinheit("E", "keV",
					1, 0.0, 200.0), 1, 0.0, 200.0);
		} else {
			addColumn(new SymbolNameEinheit("t", "s", 3, 0.0, zeitMax), 3, 0.0,
					zeitMax);
			addColumn(new SymbolNameEinheit("n", "", 0, 0.0, 20.0), 0, 0.0,
					20.0);
			addColumn(new SymbolNameEinheit("f", "Hz", 0, 0.0, 1000.0), 0, 0.0,
					1000.0);
			addColumn(new SymbolNameEinheit("β", "°", 1, 0.0, 30.0), 1, 0.0,
					30.0);
			addColumn(new SymbolNameEinheit("λ", "pm", 0, 0.0, 1000.0), 0, 0.0,
					1000.0);
			addColumn(new SymbolNameEinheit("E", "keV", 0, 0.0, 200.0), 1, 0.0,
					200.0);
		}
		for (int i = UNABHAENGIG; i < getColumnCount(); i++) {
			if (getColumnSymbol(i).equals("t")
					|| getColumnSymbol(i).equals("n")
					|| getColumnSymbol(i).equals("f")
					|| getColumnSymbol(i).equals("β")
					|| getColumnSymbol(i).equals("λ")
					|| getColumnSymbol(i).equals("E"))
				removeColumn(i);
		}
	}

	@SuppressWarnings("rawtypes")
	protected static Vector<Vector> convertToVector(Double[][] anArray) {
		if (anArray == null) {
			return null;
		}
		Vector<Vector> v = new Vector<Vector>(anArray.length);
		for (int i = 0; i < anArray.length; i++) {
			v.addElement(convertToVector(anArray[i]));
		}
		return v;
	}

	protected static Vector<Double> convertToVector(Double[] anArray) {
		if (anArray == null) {
			return null;
		}
		Vector<Double> v = new Vector<Double>(anArray.length);
		for (int i = 0; i < anArray.length; i++) {
			v.addElement(anArray[i]);
		}
		return v;
	}

	/*************************************************************
	 * Erweiterung Nullzeilen
	 *************************************************************/
	public boolean emptyRowsExist() {
		@SuppressWarnings("rawtypes")
		Vector<Vector> d = getDataVector();
		for (@SuppressWarnings("unchecked") Vector<Double> row : d)
			if (row.get(N.ordinal()) == null)
				return true;
		return false;
	}

	public int getEmptyRowCount() {
		int result = 0;
		@SuppressWarnings("rawtypes")
		Vector<Vector> d = getDataVector();
		for (@SuppressWarnings("unchecked") Vector<Double> row : d)
			if (row.get(N.ordinal()) == null)
				result++;
		return result;
	}

	public Iterator<Integer> getEmptyRowIterator() {
		Vector<Integer> v = new Vector<Integer>();
		@SuppressWarnings("rawtypes")
		Vector<Vector> d = getDataVector();
		int rowCount = getRowCount();
		for (int index = 0; index < rowCount; index++)
			if (d.get(index).get(N.ordinal()) == null)
				v.add(index);
		return v.iterator();

	}

	public int getAbschnitt(int index) {
		Iterator<Integer> it = getEmptyRowIterator();
		int ret = 0;
		while (it.hasNext() && it.next() < index)
			ret++;
		return ret;
	}

	public void addEmptyRow() {
		Double[] dobj = new Double[getColumnCount()];
		super.addRow(dobj);
	}

	public void addRow(double[] rowData) {
		Double[] dobj = new Double[rowData.length];
		for (int i = 0; i < rowData.length; i++)
			dobj[i] = rowData[i];
		addRow(dobj);
	}

	public void addRow(Double[] rowData) {
		for (int i = 0; i < rowData.length; i++) {
			minimum[i] = findeMinimum(minimum[i], rowData[i]);
			maximum[i] = findeMaximum(maximum[i], rowData[i]);
		}
		super.addRow(rowData);
	}

	public void addRows(@SuppressWarnings("rawtypes") Vector<Vector> data) {
		if (data == null || data.size() < 1)
			return;
		@SuppressWarnings("rawtypes")
		Vector<Vector> dataVec = getDataVector();
		int zeilenAlt = dataVec.size();
		int zeilenNeu = data.size() + zeilenAlt - 1;
		dataVec.addAll(data);
		if (dynamisch)
			for (@SuppressWarnings("rawtypes") Vector row : data) {
				for (int i = 0; i < minimum.length; i++) {
					minimum[i] = findeMinimum(minimum[i], (Double) row.get(i));
					maximum[i] = findeMaximum(maximum[i], (Double) row.get(i));
				}
			}
		fireTableRowsInserted(zeilenAlt, zeilenNeu);
	}

	public void addRows(Double[][] data) {
		addRows(convertToVector(data));
	}

	public void insertRow(int row, Double[] rowData) {
		insertRow(row, convertToVector(rowData));
		if (dynamisch && rowData != null && rowData.length >= minimum.length) {
			for (int i = 0; i < minimum.length; i++) {
				minimum[i] = findeMinimum(minimum[i], rowData[i]);
				maximum[i] = findeMaximum(maximum[i], rowData[i]);
			}
		}
	}

	@SuppressWarnings("unchecked")
	private void _replaceRows(int firstRow, @SuppressWarnings("rawtypes") Vector<Vector> data,
			boolean ergaenzen) {
		for (int row = 0; row < data.size(); row++)
			replaceRow(firstRow + row, data.get(row), false, ergaenzen);
	}

	public void replaceRows(int firstRow, @SuppressWarnings("rawtypes") Vector<Vector> data,
			boolean minMaxNeu, boolean ergaenzen) {
		if (data == null || data.size() < 1)
			return;
		if (firstRow > getRowCount() - 1) {
			addRows(data);
			return;
		}
		@SuppressWarnings("rawtypes")
		Vector<Vector> replaceVec = new Vector<Vector>();
		int replaceRows = Math.min(firstRow + data.size(), getRowCount());
		for (int row = firstRow; row < replaceRows; row++)
			replaceVec.add(data.get(row - firstRow));
		_replaceRows(firstRow, replaceVec, ergaenzen);
		data.removeAll(replaceVec);
		if (data.size() > 0)
			addRows(data);
		if (dynamisch && minMaxNeu)
			findeAlleMinMax(minimum, maximum);
	}

	public void addColumn(Object columnName, int stellen, double minimum,
			double maximum) {
		int index = getColumnCount();
		addColumn(columnName);
		renderer.addElement(new StellenRenderer(stellen));
		this.minimum = Arrays.copyOf(this.minimum, index + 1);
		this.maximum = Arrays.copyOf(this.maximum, index + 1);
		if (!dynamisch) {
			this.minimum[index] = minimum;
			this.maximum[index] = maximum;
		}
	}

	public void addColumn(Object columnName, int stellen, double minimum,
			double maximum, Double[] columnData) {
		int index = getColumnCount();
		addColumn(columnName, convertToVector(columnData));
		renderer.addElement(new StellenRenderer(stellen));
		this.minimum = Arrays.copyOf(this.minimum, index + 1);
		this.maximum = Arrays.copyOf(this.maximum, index + 1);
		if (dynamisch) {
			Double[] minmax = new Double[2];
			findeMinMax(index, minmax);
			this.minimum[index] = minmax[0];
			this.maximum[index] = minmax[1];
		} else {
			this.minimum[index] = minimum;
			this.maximum[index] = maximum;
		}
	}

	public void insertColumn(int columnIndex, Object columnName, int stellen,
			double minimum, double maximum) {
		insertColumn(columnIndex, columnName, stellen, minimum, maximum,
				new Vector<Double>(getRowCount()));
	}

	public void insertColumn(int columnIndex, Object columnName, int stellen,
			double minimum, double maximum, Double[] columnData) {
		insertColumn(columnIndex, columnName, stellen, minimum, maximum,
				convertToVector(columnData));
	}

	@SuppressWarnings("unchecked")
	public void insertColumn(int columnIndex, Object columnName, int stellen,
			double minimum, double maximum, Vector<Double> columnData) {
		columnIdentifiers.insertElementAt(columnName, columnIndex);
		renderer.insertElementAt(new StellenRenderer(stellen), columnIndex);
		if (columnData != null) {
			int columnSize = columnData.size();
			if (columnSize > getRowCount()) {
				dataVector.setSize(columnSize);
			}
			justifyRows(0, getRowCount());
			for (int i = 0; i < columnSize; i++) {
				Vector<Double> row = (Vector<Double>) dataVector.elementAt(i);
				row.insertElementAt(columnData.elementAt(i), columnIndex);
			}
		} else {
			justifyRows(0, getRowCount());
		}
		Double[] min = new Double[getColumnCount()];
		Double[] max = new Double[getColumnCount()];
		System.arraycopy(this.minimum, 0, min, 0, columnIndex);
		System.arraycopy(this.maximum, 0, max, 0, columnIndex);
		if (min.length > columnIndex + 1) {
			System.arraycopy(this.minimum, columnIndex + 1, min,
					columnIndex + 1, min.length - columnIndex);
			System.arraycopy(this.maximum, columnIndex, max, columnIndex,
					max.length - columnIndex);
		}
		if (dynamisch) {
			Double[] minmax = new Double[2];
			findeMinMax(columnIndex, minmax);
			min[columnIndex] = minmax[0];
			max[columnIndex] = minmax[1];
		} else {
			min[columnIndex] = minimum;
			max[columnIndex] = maximum;
		}
		this.minimum = min;
		this.maximum = max;
		fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW,
				columnIndex, INSERT));
	}

	@SuppressWarnings("unchecked")
	public void removeColumn(int columnIndex) {
		columnIdentifiers.removeElementAt(columnIndex);
		renderer.removeElementAt(columnIndex);
		int rows = getRowCount();
		for (int i = 0; i < rows; i++) {
			Vector<Double> row = (Vector<Double>) dataVector.elementAt(i);
			row.removeElementAt(columnIndex);
		}
		fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW,
				columnIndex, DELETE));
	}

	public void aktualisiereMinMax() {
		if (minimum == null || minimum.length != getColumnCount())
			minimum = new Double[getColumnCount()];
		if (maximum == null || maximum.length != getColumnCount())
			maximum = new Double[getColumnCount()];
		if (dynamisch)
			findeAlleMinMax(minimum, maximum);
	}

	@SuppressWarnings("unchecked")
	private void findeAlleMinMax(Double[] min, Double[] max) {
		if (min == null || max == null)
			return;
		Arrays.fill(min, null);
		Arrays.fill(max, null);
		int rows = getRowCount();
		int cols = getColumnCount();
		if (rows < 1 || cols < 1 || min.length < cols || max.length < cols) {
			return;
		}
		@SuppressWarnings("rawtypes")
		Vector<Vector> werte = this.getDataVector();
		Vector<Double> zeile;
		Double wert;
		for (int row = 0; row < rows; row++) {
			zeile = werte.get(row);
			for (int col = 0; col < cols; col++) {
				wert = zeile.get(col);
				min[col] = findeMinimum(min[col], wert);
				max[col] = findeMaximum(max[col], wert);
			}
		}
	}

	private static Double checkForNaN(Double d) {
		if (d == null || d == Double.NaN || d == Double.NEGATIVE_INFINITY
				|| d == Double.POSITIVE_INFINITY)
			return null;
		else
			return d;
	}

	public static Double findeMinimum(Double d1, Double d2) {
		d1 = checkForNaN(d1);
		d2 = checkForNaN(d2);
		if (d1 != null && d2 != null)
			return Math.min(d1, d2);
		return d1 != null ? d1 : d2;
	}

	public static Double findeMaximum(Double d1, Double d2) {
		d1 = checkForNaN(d1);
		d2 = checkForNaN(d2);
		if (d1 != null && d2 != null)
			return Math.max(d1, d2);
		return d1 != null ? d1 : d2;
	}

	public void findeMinMax(int index, Double[] minmax) {
		boolean auchMax = minmax.length == 2;
		findeMinMax(-1, index, true, auchMax, minmax);
	}

	private void findeMinMax(int exRow, int col, boolean min, boolean max,
			Double[] minmax) {
		//TODO IndexOutOfBoundsException
		if (minmax == null)
			return;
		Arrays.fill(minmax, null);
		int rows = getRowCount();
		if (col < 0 || col >= getColumnCount()) {
			return;
		}
		@SuppressWarnings("rawtypes")
		Vector<Vector> werte = this.getDataVector();
		Double wert;
		for (int row = 0; row < rows; row++) {
			if (row == exRow)
				continue;
			wert = (Double) werte.get(row).get(col);
			if (min)
				minmax[0] = findeMinimum(minmax[0], wert);
			if (max)
				minmax[1] = findeMaximum(minmax[1], wert);
		}
		if (minmax[0] == null || minmax[1] == null) {
			SymbolNameEinheit symbol = getColumnSymbolNameEinheit(col);
			if(minmax[0] == null)
				minmax[0] = symbol.getMinimum();
			if(minmax[1] == null)
				minmax[1] = symbol.getMaximum();
		}
	}

	public boolean isCellEditable(int row, int column) {
		return true;
	}

	public double getDoubleValueAt(int row, XAchseTyp column) {
		return getDoubleValueAt(row, column.ordinal());
	}

	public double getDoubleValueAt(int row, int column) {
		return ((Double) super.getValueAt(row, column)).doubleValue();
	}

	public void setValueAt(Object aValue, int row, XAchseTyp column) {
		setValueAt(aValue, row, column.ordinal());
	}

	@Override
	public void setValueAt(Object aValue, int row, int column) {
		Double newValue = (Double) aValue;
		if (dynamisch) {
			Double oldValue = (Double) getValueAt(row, column);
			if (minimum[column] == null || maximum[column] == null) {
				minimum[column] = newValue;
				maximum[column] = newValue;
			} else if (oldValue == null && newValue != null) {
				minimum[column] = Math.min(minimum[column], newValue);
				maximum[column] = Math.max(maximum[column], newValue);
			} else if (newValue != null) {
				boolean neuesMinimum = false, neuesMaximum = false;
				if (oldValue == null)
					oldValue = newValue;
				if (newValue < minimum[column] || oldValue <= minimum[column])
					neuesMinimum = true;
				if (newValue > maximum[column] || oldValue >= maximum[column])
					neuesMaximum = true;
				if (neuesMinimum || neuesMaximum) {
					Double[] minmax = new Double[] { newValue, newValue };
					findeMinMax(row, column, neuesMinimum, neuesMaximum, minmax);
					if (neuesMinimum)
						minimum[column] = minmax[0];
					if (neuesMaximum)
						maximum[column] = minmax[1];
				}
			}
		}
		super.setValueAt(newValue, row, column);
	}

	@SuppressWarnings("unchecked")
	private void spalteBerechnen(int col, ExpressionEvaluator ee) {
		int rows = getRowCount();
		if (rows <= 0)
			return;
		Double intervall;
		Double[] d = new Double[] {};
		Calendar jetzt = Calendar.getInstance();
		int datum = ghmessen.util.Math.datum(jetzt);
		int uhrzeit = ghmessen.util.Math.uhrzeit(jetzt);
		if (rows < 2)
			intervall = Konstanten.GROESSE_MITTELZEIT_STANDARD;
		else
			intervall = getDoubleValueAt(1, T) - getDoubleValueAt(0, T);
		for (int row = 0; row < rows; row++) {
			d = (Double[]) getDataVector().get(row).toArray(d);
			if (d[N.ordinal()] == null)
				continue;
			try {
				d[col] = (Double) ee.evaluate(new Object[] {
						getDoubleValueAt(row, T), getDoubleValueAt(row, N),
						getDoubleValueAt(row, F), getDoubleValueAt(row, BETA),
						getDoubleValueAt(row, LAMBDA),
						getDoubleValueAt(row, ENERGIE), intervall, datum,
						uhrzeit, Arrays.copyOfRange(d, UNABHAENGIG, d.length),
						this });
				setValueAt(d[col], row, col);
			} catch (InvocationTargetException e) {
				if (DEBUG)
					e.printStackTrace();
			} catch (NullPointerException e) {
				if (DEBUG)
					e.printStackTrace();
			}

		}
	}

	public void spalteBerechnen(int col, Groesse g) {
		switch (g.getArt()) {
		case Formel:
			spalteBerechnen(col, g.getFormelterm());
			break;
		case Histogramm: {
			Groessenliste gl = GHMessen.getApplication().getMainView().gl;
			if (gl == null || g.getBezug2() == null
					|| g.getBezug2().getArt() != Art.Klasse
					|| g.getBezug2().getBezug() != g.getBezug())
				break;
			int colS = gl.indexOf(g.getBezug()) + UNABHAENGIG;
			int colK = gl.indexOf(g.getBezug2()) + UNABHAENGIG;
			ArrayMaths am = new ArrayMaths(getColumnArray(colS, 0,
					getRowCount() - 1));
			double[][] result = Stat.histogramBins(am.array(), g.getBezug2()
					.getDelta());
			if (result != null && result[0] != null)
				for (int row = 0; row < getRowCount(); row++) {
					if (row < result[0].length) {
						setValueAt(result[0][row], row, colK);
						setValueAt(result[1][row], row, col);
					} else {
						setValueAt(null, row, colK);
						setValueAt(null, row, col);
					}
				}

			break;
		}
		case FFT:
			if (g.getBezug() == null) {
				for (int i = 0; i < getRowCount(); i++)
					setValueAt(null, i, col);
			} else {
				int colB = GHMessen.getApplication().getMainView().gl.indexOf(g
						.getBezug()) + UNABHAENGIG;
				ArrayMaths am = new ArrayMaths(getColumnArray(colB, 0,
						getRowCount() - 1));
				FourierTransform ft = new FourierTransform(am.array());
				ft.setDeltaT(GHMessen.getApplication().getMainView().mp
						.getIntervall());
				double[][] result = ft.powerSpectrum();
				int length = ft.getUsedDataLength() / 2;
				setNumRows(Math.max(length, getRowCount()));
				for (int ii = 0; ii < length; ii++) {
					setValueAt(result[0][ii], ii, F);
					setValueAt(result[1][ii], ii, col);
				}
			}
			break;
		default:
			break;
		}
	}

	public void entferneBereich(int col, int row1, int row2) {
		if (row2 < row1) {
			for (int row = row2; row <= row1; row++)
				super.setValueAt(null, row, col);
		} else {
			for (int row = row1; row <= row2; row++)
				super.setValueAt(null, row, col);
		}
	}

	@SuppressWarnings("unchecked")
	private void justifyRows(int from, int to) {
		// Sometimes the DefaultTableModel is subclassed
		// instead of the AbstractTableModel by mistake.
		// Set the number of rows for the case when getRowCount
		// is overridden.
		dataVector.setSize(getRowCount());

		for (int i = from; i < to; i++) {
			if (dataVector.elementAt(i) == null) {
				dataVector.setElementAt(new Vector<Double>(), i);
			}
			((Vector<Double>) dataVector.elementAt(i))
					.setSize(getColumnCount());
		}
	}

	public Double getMinimum(int spalte) {
		if (spalte < 0 || spalte >= getColumnCount() || minimum == null || spalte >= minimum.length)
			return -10.0;
		else
			return minimum[spalte] == null ? -10.0 : minimum[spalte];
	}

	public Double getMaximum(int spalte) {
		if (spalte < 0 || spalte >= getColumnCount() || maximum == null || spalte >= maximum.length)
			return 10.0;
		else
			return maximum[spalte] == null ? 10.0 : maximum[spalte];
	}

	public void replaceRow(int row, Double[] data, boolean minMaxNeu,
			boolean ergaenzen) {
		replaceRow(row, convertToVector(data), minMaxNeu, ergaenzen);
	}

	public void replaceRow(int row, Vector<Double> data, boolean minMaxNeu,
			boolean ergaenzen) {
		if (row < getRowCount()) {
			if (ergaenzen) {
				for (int col = 0; col < data.size(); col++)
					if (data.get(col) != null && data.get(col) != Double.NaN)
						setValueAt(data.get(col), row, col);
			} else {
				insertRow(row, data);
				removeRow(row + 1);
			}
		} else
			addRow(data);
		if (dynamisch && minMaxNeu)
			findeAlleMinMax(minimum, maximum);
	}

	public void removeRow(int row) {
		super.removeRow(row);
		if (dynamisch)
			findeAlleMinMax(minimum, maximum);
	}

	public void setNumRows(int rows) {
		super.setNumRows(rows);
		if (dynamisch)
			findeAlleMinMax(minimum, maximum);
	}

	@SuppressWarnings("unchecked")
	public Double[] getRow(int row) {
		Double[] result = new Double[1];
		result = (Double[]) Array.newInstance(result.getClass()
				.getComponentType(), getColumnCount());
		result = (Double[]) getDataVector().get(row).toArray(result);
		return result;
	}

	public int getRowIndexOfNearest(Double aValue, int col) {
		if (col < 0 || col >= getColumnCount() || getRowCount() <= 0
				|| aValue == null)
			return -1;
		else {
			int result = 0;
			Double value = getDoubleValueAt(0, col);
			double diff = Math.abs(aValue - (value == null ? 0 : value));
			double diffNeu;
			int index = 1;
			while (index < getRowCount()) {
				value = getDoubleValueAt(index, col);
				diffNeu = Math.abs(aValue - (value == null ? 0 : value));
				if (diffNeu < diff) {
					result = index;
					diff = diffNeu;
				}
				index++;
			}
			return result;
		}
	}

	public int getColIndexOfNearest(Double aValue, int row) {
		if (row < 0 || row >= getRowCount() || getColumnCount() < UNABHAENGIG
				|| aValue == null)
			return -1;
		else {
			@SuppressWarnings("unchecked")
			Vector<Double> rowData = getDataVector().get(row);
			int result = 1;
			Double value = rowData.get(result);
			double diff = Math.abs(aValue - (value == null ? 0 : value));
			double diffNeu;
			int index = UNABHAENGIG;
			while (index < getColumnCount()) {
				value = rowData.get(index);
				diffNeu = Math.abs(aValue - (value == null ? 0 : value));
				if (diffNeu < diff) {
					result = index;
					diff = diffNeu;
				}
				index++;
			}
			return result;
		}
	}

	/***********************************************************
	 * Default Renderer
	 ***********************************************************/
	public TableCellRenderer getColumnRenderer(int col) {
		if (col < renderer.size())
			return renderer.get(col);
		else
			return null;
	}

	/**********************************************************
	 * Methoden zur Synchronisation mit der Größenliste und deren Größen
	 **********************************************************/
	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsAdded(ObservableList list, int index, int length) {
		if (list instanceof Groessenliste) {
			Groessenliste gl = (Groessenliste) list;
			Groesse g;
			for (int i = 0; i < length; i++) {
				g = gl.get(index + i);
				insertColumn(index + i + UNABHAENGIG, g.getSymbolNameEinheit(),
						g.getStellen(), g.getVon(), g.getBis());
			}
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementsRemoved(ObservableList list, int index,
			List oldElements) {
		if (list instanceof Groessenliste) {
			for (int i = index + oldElements.size() - 1; i >= index; i--) {
				removeColumn(i + UNABHAENGIG);
			}
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementReplaced(ObservableList list, int index,
			Object oldElement) {
		if (list instanceof Groessenliste) {
			Groessenliste gl = (Groessenliste) list;
			Groesse g = gl.get(index);
			removeColumn(index + UNABHAENGIG);
			insertColumn(index + UNABHAENGIG, g.getSymbolNameEinheit(),
					g.getStellen(), g.getVon(), g.getBis());
		}
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void listElementPropertyChanged(ObservableList list, int index) {
		return;
	}

	@Override
	public void listElementPropertyChanged(
			@SuppressWarnings("rawtypes") ObservableList list, int index,
			PropertyChangeEvent evt) {
		Groesse g = (Groesse) evt.getSource();
		if ("symbol".equals(evt.getPropertyName())
				|| "einheit".equals(evt.getPropertyName())
				|| "von".equals(evt.getPropertyName())
				|| "bis".equals(evt.getPropertyName())) {
			setColumnIdentifier(index + UNABHAENGIG, g.getSymbolNameEinheit());
		} else if ("stellen".equals(evt.getPropertyName())) {
			renderer.get(index + UNABHAENGIG).setStellen(g.getStellen());
			fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW,
					index + UNABHAENGIG, UPDATE));
		} else if (g.getArt() == Art.Formel
				&& "formelterm".equals(evt.getPropertyName())
				&& g.getFormelterm() != null && autoBerechnen) {
			spalteBerechnen(index + UNABHAENGIG, g.getFormelterm());
		} else if ("bezug".equals(evt.getPropertyName()) && autoBerechnen) {
			switch (g.getArt()) {
			case Histogramm:
			case FFT:
				spalteBerechnen(index + UNABHAENGIG, g);
				break;
			default:
				break;
			}
		}

	}

	/**************************************************************
	 * PropertyChangeListener Messparameter
	 **************************************************************/
	private class IntervallPropertyHandler implements Serializable,
			PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleIntervallEvent(evt);
		}
	}

	private class MesszeitPropertyHandler implements Serializable,
			PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleMesszeitEvent(evt);
		}
	}

	private class ModusPropertyHandler implements Serializable,
			PropertyChangeListener {
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleModusEvent(evt);
		}
	}

	private void handleIntervallEvent(PropertyChangeEvent evt) {
		if (renderer != null && renderer.size() > T.ordinal()) {
			double intervall = (Double) evt.getNewValue();
			Object cid = columnIdentifiers.get(T.ordinal());
			double faktor = 1.0;
			if (cid != null && cid instanceof SymbolNameEinheit) {
				try {
					faktor = Einheit.toEinheit(
							((SymbolNameEinheit) cid).getEinheit()).getFaktor();
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
				}
			}
			renderer.get(T.ordinal()).setStellen(
					(int)Math.ceil(Math.max(0, -Math.log10(intervall / faktor))));
			fireTableChanged(new TableModelEvent(this, HEADER_ROW, HEADER_ROW,
					T.ordinal(), UPDATE));
			if (DEBUG)
				System.out
						.println("[WerteTabelle] - handleIntervallEvent: Intervall("
								+ evt.getNewValue()
								+ ") Stellen("
								+ renderer.get(T.ordinal()).getStellen() + ")");
		}
	}

	public void handleModusEvent(PropertyChangeEvent evt) {
		// anhaengenEin:true -> Messreihen #1,... entfernen
		// anhaengenEin:false -> mit null angehaengte Reihen entfernen
		if ((Boolean) evt.getNewValue()) {
			Groessenliste gl = GHMessen.getApplication().getMainView().gl;
			if (gl == null)
				return;
			while (gl.getMaxMessreihe() > 0)
				gl.removeLetzteMessreihe();
		} else {
			if (emptyRowsExist()) {
				int firstER = getEmptyRowIterator().next();
				setNumRows(firstER);
			}
		}
	}

	public void setNewMesszeitEinheit(Einheit zeitE) {
		try {
			SymbolNameEinheit t = (SymbolNameEinheit) columnIdentifiers.get(T
					.ordinal());
			Einheit alt = Einheit.toEinheit(t.getEinheit());
			if (alt == Einheit.Keine) {
				if (DEBUG)
					System.out
							.println("[WerteTabelle] - setNewMesszeitEinheit: alte Einheit nicht vorhanden. Das sollte nicht passieren.");
				return;
			}
			t.setEinheit(zeitE);
			t.setMaximum(t.getMaximum() * alt.getFaktor() / zeitE.getFaktor());
			updateTimeRenderer(alt, zeitE, t.getMaximum());
			setColumnIdentifier(T.ordinal(), t);
		} catch (NullPointerException e) {
			if (DEBUG)
				System.out
						.println("[WerteTabelle] - setNewMesszeitEinheit: zeitE("
								+ zeitE
								+ ") columnIdentifiers("
								+ columnIdentifiers + ")");
		}
	}

	private void updateTimeRenderer(Einheit alt, Einheit neu, double maximum) {
		if (renderer != null && renderer.size() > T.ordinal()) {
			double newFaktor = neu.getFaktor();
			double faktor = renderer.get(T.ordinal()).getStellen();
			if (newFaktor > 1.0
					&& !(renderer.get(T.ordinal()) instanceof TimeRenderer))
				renderer.set(T.ordinal(), new TimeRenderer(maximum));
			else if (newFaktor <= 1.0
					&& renderer.get(T.ordinal()) instanceof TimeRenderer)
				renderer.set(T.ordinal(), new StellenRenderer());
			faktor = Math.pow(10.0, -faktor);
			faktor *= alt.getFaktor();
			faktor /= neu.getFaktor();
			renderer.get(T.ordinal()).setStellen(
					(int) Math.max(0, -Math.log10(faktor)));
		}
	}

	private void handleMesszeitEvent(PropertyChangeEvent evt) {
		double messzeit = (Double) evt.getNewValue();
		SymbolNameEinheit t = (SymbolNameEinheit) columnIdentifiers.get(T
				.ordinal());
		if (messzeit == 0.0)
			messzeit = Konstanten.STANDARD_MESSZEIT;
		Einheit e = Einheit.toEinheit(t.getEinheit());
		t.setIntervall(new Intervall(0.0, messzeit / e.getFaktor()));
		setColumnIdentifier(T.ordinal(), t);
	}

}
