/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse.darstellung;

import flanagan.analysis.Regression;
import flanagan.analysis.RegressionFunction;
import ghmessen.components.plotcanvas.GraphBereich;
import ghmessen.parser.FreieFunktionParser;
import ghmessen.util.Util;

import java.awt.Color;
import java.awt.geom.Path2D;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;

import javax.xml.transform.sax.TransformerHandler;

import org.codehaus.janino.ExpressionEvaluator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/*
 * Anpassungsfunktion f(t,A,B,C,D) wird als Regressiosfunktion verwendet.
 */
public class AnpassungFrei extends Anpassung implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6695656079370920706L;
	private static final boolean DEBUG = false;
	private transient ExpressionEvaluator f;
	private double[] param;
	private String funktionsTerm;

	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		in.defaultReadObject();
		FreieFunktionParser p = new FreieFunktionParser();
		f = p.getAusdruck(funktionsTerm);
		aktualisieren();
	}

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT AnpassungFrei (APX,APY,APY1,GraphBereich,TextAuswertung,AFTerm,AFParam)>");
		out.println("<!ATTLIST AnpassungFrei typ (AnpassungEnum) #REQUIRED von (DOUBLE) #REQUIRED bis (DOUBLE) #REQUIRED n (INT) #REQUIRED");
		out.println("\tterm CDATA #REQUIRED htmlTerm CDATA #REQUIRED farbe CDATA #REQUIRED rsqr (DOUBLE) #REQUIRED autoZeichnung (BOOLEAN) #REQUIRED");
		out.println("\tautoAnpassung (BOOLEAN) #REQUIRED autoZeichnungVonBis (BOOLEAN) #REQUIRED manuell (BOOLEAN) #IMPLIED");
		out.println("\tsymbolX CDATA #REQUIRED symbolY CDATA #REQUIRED");
		out.println("\tstellenX (INT) #REQUIRED stellenY (INT) #REQUIRED einheitX CDATA #REQUIRED einheitY CDATA #REQUIRED>");
		out.println("<!ELEMENT AFTerm (CDATA)>");
		out.println("<!ELEMENT AFParam (DOUBLE*)>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[AnpassungFrei] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "typ", "AnpassungEnum", typ.toString());
		atts.addAttribute("", "", "von", "DOUBLE", Double.toString(von));
		atts.addAttribute("", "", "bis", "DOUBLE", Double.toString(bis));
		atts.addAttribute("", "", "symbolX", "CDATA", symbolX);
		atts.addAttribute("", "", "symbolY", "CDATA", symbolY);
		atts.addAttribute("", "", "stellenX", "INT", Integer.toString(stellenX));
		atts.addAttribute("", "", "stellenY", "INT", Integer.toString(stellenY));
		atts.addAttribute("", "", "einheitX", "CDATA", einheitX);
		atts.addAttribute("", "", "einheitY", "CDATA", einheitY);
		atts.addAttribute("", "", "n", "INT", Integer.toString(n));

		atts.addAttribute("", "", "term", "CDATA", term);
		atts.addAttribute("", "", "htmlTerm", "CDATA", htmlTerm);
		atts.addAttribute("", "", "farbe", "CDATA", Integer.toString(farbe.getRGB()));
		atts.addAttribute("", "", "rsqr", "DOUBLE", Double.toString(rsqr));
		atts.addAttribute("", "", "autoZeichnung", "BOOLEAN",
				Boolean.toString(autoZeichnung));
		atts.addAttribute("", "", "autoAnpassung", "BOOLEAN",
				Boolean.toString(autoAnpassung));
		atts.addAttribute("", "", "autoZeichnungVonBis", "BOOLEAN",
				Boolean.toString(autoZeichnungVonBis));
		atts.addAttribute("", "", "manuell", "BOOLEAN",
				Boolean.toString(manuell));
		hd.startElement("", "", "AnpassungFrei", atts);
		String inhalt;
		hd.startElement("", "", "APX", null);
		inhalt = Util.writeDoublesToXml(x);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "APX");
		hd.startElement("", "", "APY", null);
		inhalt = Util.writeDoublesToXml(y);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "APY");
		hd.startElement("", "", "APY1", null);
		inhalt = Util.writeDoublesToXml(y1);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "APY1");
		bezug.writeToXml(hd);
		if (benennung != null)
			benennung.writeToXml(hd);
		hd.startElement("", "", "AFTerm", null);
		hd.characters(funktionsTerm.toCharArray(), 0, funktionsTerm.length());
		hd.endElement("", "", "AFTerm");
		hd.startElement("", "", "AFParam", null);
		inhalt = Util.writeDoublesToXml(param);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "AFParam");
		hd.endElement("", "", "AnpassungFrei");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("AnpassungFrei")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					Field field = null;
					Class<?> c = this.getClass();
					while (field == null && c != null)
						try {
							field = c.getDeclaredField(name);
						} catch (NoSuchFieldException e) {
							if (c.getSuperclass() != null)
								c = c.getSuperclass();
							else
								c = null;
						}
					if (DEBUG)
						System.out
								.println("[AnpassungFrei] - startElement: Feld("
										+ name
										+ ") Typ("
										+ field.getType()
										+ ") Inhalt(" + inhalt + ")");
					if ("farbe".equals(name)){
						try {
							farbe = new Color(Integer.parseInt(inhalt));
						} catch(NumberFormatException e) {
							farbe = Color.black;
						}
					}
					else if (c != null)
						Util.setFieldValue(c.cast(this), field, inhalt);
					else
						throw new NoSuchFieldException(
								"[AnpassungFrei] - startElement: Feld("
										+ field.getName()
										+ ") existiert in keiner Superklasse.");
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		} else if (qName.equals("APX") || qName.equals("APY")
				|| qName.equals("APY1")) {
			return;
		} else if (qName.equals("GraphBereich")) {
			bezug = new GraphBereich();
			bezug.startElement(uri, localName, qName, attributes);
		} else if (qName.equals("TextAuswertung")) {
			benennung = new TextAuswertung();
			benennung.startElement(uri, localName, qName, attributes);
		} else if (qName.equals("AFParam") || qName.equals("AFTerm"))
			return;
		else
			throw new SAXException("[AnpassungFrei] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		if (qName.equals("AnpassungFrei")) {
			FreieFunktionParser p = new FreieFunktionParser();
			f = p.getAusdruck(funktionsTerm);
			aktualisieren();
		} else if (qName.equals("APX")) {
			x = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("APY")) {
			y = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("APY1")) {
			y1 = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("AFParam")) {
			param = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("AFTerm")) {
			funktionsTerm = inhalt;
		} else if (qName.equals("GraphBereich")
				|| qName.equals("TextAuswertung")) {
			return;
		} else
			throw new SAXException("[AnpassungFrei] - Start(" + qName + ")");
	}

	public AnpassungFrei() {
		super();
	}

	public AnpassungFrei(AnpassungEnum typ, Double[] x, Double[] y,
			ExpressionEvaluator f, double[] param, double von, double bis,
			GraphBereich bezug, String symbolX, String symbolY, int stellenX,
			int stellenY, String einheitX, String einheitY, String term,
			String htmlTerm, double rsqr, Path2D zeichnung,
			boolean autoZeichnung, boolean autoAnpassung, Color farbe) {
		super(typ, x, y, von, bis, bezug, symbolX, symbolY, stellenX, stellenY,
				einheitX, einheitY, term, htmlTerm, rsqr, zeichnung,
				autoZeichnung, autoAnpassung, false, farbe);
		this.f = f;
		this.param = param;
		funktionsTerm = term;
		this.autoZeichnung = true;
		this.autoAnpassung = true;
		this.autoZeichnungVonBis = true;
		aktualisieren();
	}

	public AnpassungFrei(AnpassungEnum typ, Double[] x, Double[] y,
			ExpressionEvaluator f, double[] param, double von, double bis,
			int spalte, int zeile1, int zeile2, String symbolX, String symbolY,
			int stellenX, int stellenY, String einheitX, String einheitY,
			String term, Color farbe) {
		this(typ, x, y, f, param, von, bis, new GraphBereich(spalte, zeile1,
				zeile2), symbolX, symbolY, stellenX, stellenY, einheitX,
				einheitY, term, "", 0.0, null, false, false, farbe);
	}

	@Override
	protected void anpassen() throws AnpassungException {
		if (f == null)
			throw new AnpassungException("[Freie Anpassung] Ausdruck ungültig.");
		if (x == null || y == null || x.length != y.length)
			throw new AnpassungException("Anpassung nicht durchführbar: x(" + x
					+ ") y(" + y + ")");
		Regression r = new Regression(x, y);
		RF funktion = new RF();
		r.simplex(funktion, param);
		param = r.getCoeff();
		if (param == null || param.length != 4)
			throw new AnpassungException(
					"[Freie Funktion] Parameteranzahl der Regression falsch("
							+ param + ")");
		rsqr = r.getSumOfSquares();
		setTerm(param);
		htmlAusTermErzeugen();
		try {
			y1 = new double[x.length];
			for (int i = 0; i < x.length; i++) {
				y1[i] = (Double) f.evaluate(new Object[] { new Double(x[i]),
						new Double(param[0]), new Double(param[1]),
						new Double(param[2]), new Double(param[3]) });
			}
		} catch (InvocationTargetException e) {
			if (DEBUG)
				e.printStackTrace();
		}

	}

	private void setTerm(double[] werte) {
		if (werte == null || werte.length != 4)
			return;
		String s = term;
		s = s.replaceAll("A",
				String.format("%2." + (stellenX + stellenY) + "f", werte[0]));
		s = s.replaceAll("B",
				String.format("%2." + (stellenX + stellenY) + "f", werte[1]));
		s = s.replaceAll("C",
				String.format("%2." + (stellenX + stellenY) + "f", werte[2]));
		s = s.replaceAll("D",
				String.format("%2." + (stellenX + stellenY) + "f", werte[3]));
		s = s.replaceAll("y", symbolY);
		s = s.replaceAll("t", symbolX);
		s = s.concat(String.format(" (R<sup>2</sup>=%"
				+ (stellenX + stellenY + 2) + "f)", rsqr));
		super.setTerm(s);
	}

	@Override
	protected void zeichnen() throws AnpassungException {
		if (x == null || y1 == null || x.length != y1.length)
			throw new AnpassungException(
					"Zeichnung der Anpassung nicht durchführbar: x(" + x
							+ ") y(" + y1 + ")");
		zeichnung = new Path2D.Double();
		if (y1.length < 2)
			throw new AnpassungException(
					"[Freie Funktion] Zeichnung hat zu wenige Punkte("
							+ y1.length + ")");
		zeichnung.moveTo(x[0], y1[0]);
		for (int i = 1; i < n; i++)
			zeichnung.lineTo(x[i], y1[i]);
	}

	@Override
	protected double value(double x) throws AnpassungException {
		try {
			return (Double) f.evaluate(new Object[] { new Double(x),
					new Double(param[0]), new Double(param[1]),
					new Double(param[2]), new Double(param[3]) });
		} catch (InvocationTargetException e) {
			if (DEBUG)
				e.printStackTrace();
			throw new AnpassungException("[Freie Funktion] - value(" + x + ")");
		}

	}

	@Override
	protected void zeichnen(double von, double bis) throws AnpassungException {
		if (von > bis) {
			double tmp = von;
			von = bis;
			bis = tmp;
		}
		zeichnungVonBis = new Path2D.Double();
		double dx = (bis - von) / 100;
		zeichnungVonBis.moveTo(von, value(von));
		for (int i = 1; i <= 100; i++)
			zeichnungVonBis.lineTo(von + i * dx, value(von + i * dx));
	}

	private class RF implements RegressionFunction {

		@Override
		public double function(double[] param, double[] x) {
			try {
				return (Double) f.evaluate(new Object[] { new Double(x[0]),
						new Double(param[0]), new Double(param[1]),
						new Double(param[2]), new Double(param[3]) });
			} catch (InvocationTargetException e) {
				if (DEBUG)
					e.printStackTrace();
				return 0.0;
			}
		}

	}

}
