/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse.darstellung;

import static ghmessen.groesse.darstellung.AnpassungEnum.Ausgleichsgerade;
import flanagan.analysis.Regression;
import flanagan.analysis.RegressionFunction;
import flanagan.analysis.Stat;
import ghmessen.components.plotcanvas.GraphBereich;
import ghmessen.util.Util;
import ghmessen.util.HTML;

import java.awt.Color;
import java.awt.geom.Path2D;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;

import javax.xml.transform.sax.TransformerHandler;

import org.codehaus.janino.CompileException;
import org.codehaus.janino.ExpressionEvaluator;
import org.codehaus.janino.Parser.ParseException;
import org.codehaus.janino.Scanner.ScanException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class AnpassungRegression extends Anpassung {

	private static final long serialVersionUID = -8015574954668672587L;
	private static final boolean DEBUG = false;
	protected transient ExpressionEvaluator ausdruck;
	protected transient Double[] parameter;

	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		in.defaultReadObject();
		aktualisieren();
	}

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT AnpassungRegression (APX,APY,APY1,GraphBereich,TextAuswertung)>");
		out.println("<!ATTLIST AnpassungRegression typ (AnpassungEnum) #REQUIRED von (DOUBLE) #REQUIRED bis (DOUBLE) #REQUIRED n (INT) #REQUIRED");
		out.println("\tterm CDATA #REQUIRED htmlTerm CDATA #REQUIRED farbe CDATA #REQUIRED rsqr (DOUBLE) #REQUIRED autoZeichnung (BOOLEAN) #REQUIRED");
		out.println("\tautoAnpassung (BOOLEAN) #REQUIRED autoZeichnungVonBis (BOOLEAN) #REQUIRED manuell (BOOLEAN) #IMPLIED");
		out.println("\tsymbolX CDATA #REQUIRED symbolY CDATA #REQUIRED");
		out.println("\tstellenX (INT) #REQUIRED stellenY (INT) #REQUIRED einheitX CDATA #REQUIRED einheitY CDATA #REQUIRED>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[Anpassung] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "typ", "AnpassungEnum", typ.toString());
		atts.addAttribute("", "", "von", "DOUBLE", Double.toString(von));
		atts.addAttribute("", "", "bis", "DOUBLE", Double.toString(bis));
		atts.addAttribute("", "", "symbolX", "CDATA", symbolX);
		atts.addAttribute("", "", "symbolY", "CDATA", symbolY);
		atts.addAttribute("", "", "stellenX", "INT", Integer.toString(stellenX));
		atts.addAttribute("", "", "stellenY", "INT", Integer.toString(stellenY));
		atts.addAttribute("", "", "einheitX", "CDATA", einheitX);
		atts.addAttribute("", "", "einheitY", "CDATA", einheitY);
		atts.addAttribute("", "", "n", "INT", Integer.toString(n));

		atts.addAttribute("", "", "term", "CDATA", term);
		atts.addAttribute("", "", "htmlTerm", "CDATA", htmlTerm);
		atts.addAttribute("", "", "farbe", "CDATA", Integer.toString(farbe.getRGB()));
		atts.addAttribute("", "", "rsqr", "DOUBLE", Double.toString(rsqr));
		atts.addAttribute("", "", "autoZeichnung", "BOOLEAN",
				Boolean.toString(autoZeichnung));
		atts.addAttribute("", "", "autoAnpassung", "BOOLEAN",
				Boolean.toString(autoAnpassung));
		atts.addAttribute("", "", "autoZeichnungVonBis", "BOOLEAN",
				Boolean.toString(autoZeichnungVonBis));
		atts.addAttribute("", "", "manuell", "BOOLEAN",
				Boolean.toString(manuell));
		hd.startElement("", "", "AnpassungRegression", atts);
		String inhalt;
		hd.startElement("", "", "APX", null);
		inhalt = Util.writeDoublesToXml(x);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "APX");
		hd.startElement("", "", "APY", null);
		inhalt = Util.writeDoublesToXml(y);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "APY");
		hd.startElement("", "", "APY1", null);
		inhalt = Util.writeDoublesToXml(y1);
		hd.characters(inhalt.toCharArray(), 0, inhalt.length());
		hd.endElement("", "", "APY1");
		bezug.writeToXml(hd);
		if (benennung != null)
			benennung.writeToXml(hd);
		hd.endElement("", "", "AnpassungRegression");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("AnpassungRegression")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					Field field = null;
					Class<?> c = this.getClass();
					while (field == null && c != null)
						try {
							field = c.getDeclaredField(name);
						} catch (NoSuchFieldException e) {
							if (c.getSuperclass() != null)
								c = c.getSuperclass();
							else
								c = null;
						}
					if (DEBUG)
						System.out
								.println("[AnpassungRegression] - startElement: Feld("
										+ name
										+ ") Typ("
										+ field.getType()
										+ ") Inhalt(" + inhalt + ")");
					if ("farbe".equals(name)){
						try {
							farbe = new Color(Integer.parseInt(inhalt));
						} catch(NumberFormatException e) {
							farbe = Color.black;
						}
					}
					else if (c != null)
						Util.setFieldValue(c.cast(this), field, inhalt);
					else
						throw new NoSuchFieldException(
								"[AnpassungRegression] - startElement: Feld("
										+ field.getName()
										+ ") existiert in keiner Superklasse.");
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		} else if (qName.equals("APX") || qName.equals("APY")
				|| qName.equals("APY1")) {
			return;
		} else if (qName.equals("GraphBereich")) {
			bezug = new GraphBereich();
			bezug.startElement(uri, localName, qName, attributes);
		} else if (qName.equals("TextAuswertung")) {
			benennung.startElement(uri, localName, qName, attributes);
		} else
			throw new SAXException("[AnpassungRegression] - Start(" + qName
					+ ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		if (qName.equals("AnpassungRegression")) {
			aktualisieren();
		} else if (qName.equals("APX")) {
			x = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("APY")) {
			y = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("APY1")) {
			y1 = Util.prdoubleAFromXml(inhalt);
		} else if (qName.equals("GraphBereich")) {
			return;
		} else if(qName.equals("TextAuswertung")) {
			benennung.endElement(uri, localName, qName, inhalt);
		} else
			throw new SAXException("[AnpassungRegression] - Start(" + qName
					+ ")");
	}

	public AnpassungRegression(AnpassungEnum typ, Double[] x, Double[] y,
			double von, double bis, GraphBereich bezug, String symbolX,
			String symbolY, int stellenX, int stellenY, String einheitX,
			String einheitY, String term, String htmlTerm, double rsqr,
			Path2D zeichnung, boolean autoZeichnung, boolean autoAnpassung,
			Color farbe) {
		super(typ, x, y, von, bis, bezug, symbolX, symbolY, stellenX, stellenY,
				einheitX, einheitY, term, htmlTerm, rsqr, zeichnung,
				autoZeichnung, autoAnpassung, true, farbe);
	}

	public AnpassungRegression(AnpassungEnum typ, Double[] x, Double[] y,
			GraphBereich bezug, String symbolX, String symbolY, int stellenX,
			int stellenY, String einheitX, String einheitY, String term,
			String htmlTerm, double rsqr, Path2D zeichnung,
			boolean autoZeichnung, boolean autoAnpassung, Color farbe) {
		super(typ, x, y, 0, 0, bezug, symbolX, symbolY, stellenX, stellenY,
				einheitX, einheitY, term, htmlTerm, rsqr, zeichnung,
				autoZeichnung, autoAnpassung, false, farbe);
		if (x != null && y != null && x.length > 0 && x.length == y.length) {
			setVonBis(this.x[0], this.x[this.x.length - 1]);
		}
	}

	public AnpassungRegression(AnpassungEnum typ, Double[] x, Double[] y,
			double von, double bis, int spalte, int zeile1, int zeile2,
			String symbolX, String symbolY, int stellenX, int stellenY,
			String einheitX, String einheitY, Color farbe) {
		this(typ, x, y, von, bis, new GraphBereich(spalte, zeile1, zeile2),
				symbolX, symbolY, stellenX, stellenY, einheitX, einheitY, "",
				"", 0.0, null, true, true, farbe);
	}

	public AnpassungRegression(AnpassungEnum typ, Double[] x, Double[] y,
			Color farbe) {
		this(typ, x, y, null, "t", "y", 3, 3, null, null, "", "", 0.0, null,
				true, true, farbe);
	}

	public AnpassungRegression() {
		this(Ausgleichsgerade, null, null, null, "t", "y", 3, 3, null, null,
				"", "", 0.0, null, false, false, null);
	}

	/********************************************************
	 * Berechnungen
	 * 
	 ********************************************************/
	@Override
	protected void anpassen() throws AnpassungException {
		if (x == null || y == null || x.length != y.length)
			throw new AnpassungException("Anpassung nicht durchführbar: x(" + x
					+ ") y(" + y + ")");
		Regression r = new Regression(x, y);
		parameter = null;
		String ausdruckTerm = null;
		ausdruck = new ExpressionEvaluator();
		ausdruck.setReturnType(Double.class);
		switch (typ) {
		default:
			break;
		case Ursprungsgerade:
			if (x.length < 1)
				throw new AnpassungException(
						"[Ursprungsgerade] Die Punktanzahl(" + x.length
								+ ") ist zu klein.");
			parameter = erzeugeParamValues(new double[] { Stat.mean(y)
					/ Stat.mean(x) });
			if (parameter == null || parameter.length != 2)
				throw new AnpassungException(
						"[Ursprungsgerade] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "m", "x" }, new Class[] {
					Double.class, Double.class });
			ausdruckTerm = "m*x";
			term = String.format("<i><b>"
							+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
							+ " = %2."
							+ (stellenX + stellenY)
							+ "f "
							+ (einheitX == null || einheitY == null ? ""
									: einheitY + "&frasl;" + einheitX) + " &sdot; "
							+ symbolX + " <font size=\"-1\" color=\"#A4A4A4\">"
							+ ";(R<sup>2</sup>=%2."
							+ (stellenX + stellenY + 2) + "f)</font>", parameter[0],
					rsqr);
			break;
		case Ausgleichsgerade:
			if (x.length < 2)
				throw new AnpassungException(
						"[Ausgleichsgerade] Die Punkteanzahl(" + x.length
								+ ") ist zu klein.");
			else if (x.length == 2) {
				if (x[0] == x[1])
					throw new AnpassungException(
							"[Ausgleichsgerade] Die 2 Punkte A(" + x[0] + "|"
									+ y[0] + ") und B(" + x[1] + "|" + y[1]
									+ ") erfüllen nicht die Voraussetzungen.");
				parameter = new Double[] {
						y[0] - (y[1] - y[0]) / (x[1] - x[0]) * x[0],
						(y[1] - y[0]) / (x[1] - x[0]), 0.0 };
				rsqr = 0.0;
			} else {
				r.linear();
				parameter = erzeugeParamValues(r.getCoeff());
				if (parameter == null || parameter.length != 3)
					throw new AnpassungException(
							"[Ausgleichsgerade] Parameteranzahl der Regression falsch("
									+ parameter + ")");
				rsqr = r.getSumOfSquares();
			}
			ausdruck.setParameters(new String[] { "b", "m", "x" }, new Class[] {
					Double.class, Double.class, Double.class });
			ausdruckTerm = "b+m*x";
			term = String.format("<i><b>"
							+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
							+ " = %2."
							+ (stellenX + stellenY)
							+ "f "
							+ (einheitX == null || einheitY == null ? ""
									: einheitY + "&frasl;" + einheitX) + " &sdot; "
							+ symbolX + " %+2." + stellenY + "f "
							+ (einheitY == null ? "" : einheitY)
							+ " <font size=\"-1\" color=\"#A4A4A4\">;(R<sup>2</sup>=%2."
							+ (stellenX + stellenY + 2) + "f)</font>", parameter[1],
					parameter[0], rsqr);
			break;
		case Normalparabel: {
			NormalParabel funktion = new NormalParabel();
			double[] start = new double[] { 1.0 };
			try {
				r.simplex(funktion, start);
			} catch (IllegalArgumentException e) {
				throw new AnpassungException(
						"[Normalparabel] Es wurde eine IllegalArgumentException("
								+ e.getLocalizedMessage() + ") verursacht.");
			}
			parameter = erzeugeParamValues(r.getCoeff());
			if (parameter == null || parameter.length != 2)
				throw new AnpassungException(
						"[Normalparabel] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "a", "x" }, new Class[] {
					Double.class, Double.class });
			ausdruckTerm = "a*x*x";
			term = String.format("<i><b>"
					+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
					+ " = %2."
					+ (stellenX + 2 * stellenY)
					+ "f "
					+ (einheitY != null && einheitX != null ? einheitY + "&frasl;"
							+ einheitX + "<sup>2</sup>" : "") + " &sdot; " + symbolX
					+ "<sup>2</sup> <font size=\"-1\" color=\"#A4A4A4\">"
					+ ";(R<sup>2</sup>=%2." + (stellenX + stellenY + 2)
					+ "f)</font>", parameter[0], rsqr);
			break;
		}
		case Parabel:
			try {
				r.polynomial(2);
			} catch (IllegalArgumentException e) {
				throw new AnpassungException(
						"[Parabel] Die Anpassung hat eine IllegalArgumentException("
								+ e.getLocalizedMessage() + ") ausgelöst.");
			}
			parameter = erzeugeParamValues(r.getCoeff());
			if (parameter == null || parameter.length != 4)
				throw new AnpassungException(
						"[Parabel] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "c", "b", "a", "x" },
					new Class[] { Double.class, Double.class, Double.class,
							Double.class });
			ausdruckTerm = "a*x*x+b*x+c";
			term = String.format("<i><b>"
							+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
							+ " = %2."
							+ (stellenX + 2 * stellenY)
							+ "f "
							+ (einheitY != null && einheitX != null ? einheitY
									+ "&frasl;" + einheitX + "<sup>2</sup>" : "")
							+ " &sdot; "
							+ symbolX
							+ "<sup>2</sup> %+2."
							+ (stellenX + stellenY)
							+ "f "
							+ (einheitX == null || einheitY == null ? ""
									: einheitY + "/" + einheitX) + " * "
							+ symbolX + "%+2." + stellenY + "f "
							+ (einheitY == null ? "" : einheitY)
							+ " <font size=\"-1\" color=\"#A4A4A4\">;(R<sup>2</sup>=%2."
							+ (stellenX + stellenY + 2) + "f)</font>", parameter[2],
					parameter[1], parameter[0], rsqr);
			break;
		case eFunktion:
			try {
				r.exponentialSimple();
			} catch (IllegalArgumentException e) {
				throw new AnpassungException(
						"[e-Funktion] Die Anpassung hat eine IllegalArgumentException("
								+ e.getLocalizedMessage() + ") ausgelöst.");
			}
			parameter = erzeugeParamValues(r.getCoeff());
			if (parameter == null || parameter.length != 3)
				throw new AnpassungException(
						"[Exponentialfunktion] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "a", "k", "x" }, new Class[] {
					Double.class, Double.class, Double.class });
			ausdruckTerm = "a*Math.exp(k*x)";
			term = String.format("<i><b>" 
					+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
					+ " = %2." + stellenY + "f "
					+ (einheitY == null ? "" : einheitY) + " &sdot; &#x2107;<sup>%2."
					+ (stellenY + stellenX) + "f "
					+ (einheitX == null ? "": "1&frasl;" + einheitX) + " &sdot; " + symbolX
					+ "</sup> <font size=\"-1\" color=\"#A4A4A4\">"
					+ ";(R<sup>2</sup>=%2." + (stellenX + stellenY + 2)
					+ "f)</font>", parameter[1], parameter[0], rsqr);
			break;
		case Einhuellende: {
			Einhuellende f = new Einhuellende();
			double[] param = new double[4];
			Arrays.fill(param, 1.0);
			try {
				r.simplex(f, param);
			} catch (IllegalArgumentException e) {
				throw new AnpassungException(
						"[Einhüllende] Die Anpassung hat eine IllegalArgumentException("
								+ e.getLocalizedMessage() + ") ausgelöst.");
			}
			parameter = erzeugeParamValues(r.getCoeff());
			if (parameter == null || parameter.length != 5)
				throw new AnpassungException(
						"[Einhüllende] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			parameter = Arrays.copyOf(parameter, 3);
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "a", "k", "x" }, new Class[] {
					Double.class, Double.class, Double.class });
			ausdruckTerm = "a*Math.exp(k*x)";
			term = String.format("<i><b>"
					+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
					+ " = %2." + stellenY + "f "
					+ (einheitY == null ? "" : einheitY) + " &sdot; &#x2107;<sup>%2."
					+ (stellenY + stellenX) + "f "
					+ (einheitX == null ? "1&frasl;" : einheitX) + " &sdot; " + symbolX
					+ "</sup> <font size=\"-1\" color=\"#A4A4A4\">"
					+ ";(R<sup>2</sup>=%2." + (stellenX + stellenY + 2)
					+ "f)</font>", parameter[0], parameter[1], rsqr);
			break;
		}
		case Hyperbel: {
			Hyperbel funktion = new Hyperbel();
			double[] start = new double[] { 1.0 };
			try {
				r.simplex(funktion, start);
			} catch (IllegalArgumentException e) {
				throw new AnpassungException(
						"[Hyperbel] Die Anpassung hat eine IllegalArgumentException("
								+ e.getLocalizedMessage() + ") ausgelöst.");
			}
			parameter = erzeugeParamValues(r.getCoeff());
			if (parameter == null || parameter.length != 2)
				throw new AnpassungException(
						"[Hyperbel] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "a", "x" }, new Class[] {
					Double.class, Double.class });
			ausdruckTerm = "a/x";
			term = String.format("<i><b>"
							+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
							+ " = %2."
							+ (stellenX + stellenY)
							+ "f "
							+ (einheitX != null && einheitY != null ? einheitY
									+ "&sdot;" + einheitX : "") + "&frasl;" + symbolX
							+ " <font size=\"-1\" color=\"#A4A4A4\">;(R<sup>2</sup>=%2."
							+ (stellenX + stellenY + 2) + "f)</font>", parameter[0],
					rsqr);
			break;
		}
		case QuadHyperbel: {
			QuadHyperbel funktion = new QuadHyperbel();
			double[] start = new double[] { 1.0 };
			try {
				r.simplex(funktion, start);
			} catch (IllegalArgumentException e) {
				throw new AnpassungException(
						"[Quadratische Hyperbel] Die Anpassung hat eine IllegalArgumentException("
								+ e.getLocalizedMessage() + ") ausgelöst.");
			}
			parameter = erzeugeParamValues(r.getCoeff());
			if (parameter == null || parameter.length != 2)
				throw new AnpassungException(
						"[quadratische Hyperbel] Parameteranzahl der Regression falsch("
								+ parameter + ")");
			rsqr = r.getSumOfSquares();
			ausdruck.setParameters(new String[] { "a", "x" }, new Class[] {
					Double.class, Double.class });
			ausdruckTerm = "a/x^2";
			term = String.format("<i><b>"
					+ HTML.symbolZuHTML(symbolY) + "</b>(" + symbolX + ")</i>"
					+ " = %2."
					+ (stellenX + stellenY)
					+ "f "
					+ (einheitX != null && einheitY != null ? einheitY + "&sdot;"
							+ einheitX + "<sup>2</sup>" : "") + "&frasl;" + symbolX
					+ "<sup>2</sup> <font size=\"-1\" color=\"#A4A4A4\">;(R<sup>2</sup>=%2."
					+ (stellenX + stellenY + 2) + "f)</font>", parameter[0], rsqr);
			break;
		}
		}
		htmlAusTermErzeugen();
		try {
			ausdruck.cook(ausdruckTerm);
			y1 = new double[x.length];
			for (int i = 0; i < x.length; i++) {
				parameter[parameter.length - 1] = new Double(x[i]);
				y1[i] = (Double) ausdruck.evaluate(parameter);
			}
		} catch (CompileException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (ParseException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (ScanException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (InvocationTargetException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	private Double[] erzeugeParamValues(double[] p) {
		// Der erzeugte Array ist um eins größer für den x-Wert
		Double[] result;
		if (p == null)
			return new Double[1];
		else {
			result = new Double[p.length + 1];
			for (int i = 0; i < p.length; i++)
				result[i] = new Double(p[i]);
			return result;
		}
	}

	@Override
	protected void zeichnen() throws AnpassungException {
		if (x == null || y1 == null || x.length != y1.length)
			throw new AnpassungException(
					"Zeichnung der Anpassung nicht durchführbar: x(" + x
							+ ") y(" + y1 + ")");
		zeichnung = new Path2D.Double();
		switch (typ) {
		case Ursprungsgerade:
			if (y1.length < 2)
				throw new AnpassungException(
						"Ursprungsgerade Zeichnung hat zu wenige Punkte("
								+ y1.length + ")");
			zeichnung.moveTo(0, 0);
			zeichnung.lineTo(x[n - 1], y1[n - 1]);
			break;
		case Ausgleichsgerade:
			if (y1.length < 2)
				throw new AnpassungException(
						"Ausgleichsgerade Zeichnung hat zu wenige Punkte("
								+ y1.length + ")");
			zeichnung.moveTo(x[0], y1[0]);
			zeichnung.lineTo(x[n - 1], y1[n - 1]);
			break;
		case Einhuellende:
			if (y1.length < 2)
				throw new AnpassungException(
						"[Einhüllende] Zeichnung hat zu wenige Punkte("
								+ y1.length + ")");
			zeichnung.moveTo(x[0], y1[0]);
			Path2D zeichnung1 = new Path2D.Double();
			zeichnung1.moveTo(x[0], y[0]);
			for (int i = 1; i < n; i++) {
				zeichnung.lineTo(x[i], y1[i]);
				zeichnung1.lineTo(x[i], y1[i]);
			}
			zeichnung.append(zeichnung1, false);
			break;
		default:
			if (y1.length < 2)
				throw new AnpassungException(
						"Parabel/eFunktion Zeichnung hat zu wenige Punkte("
								+ y1.length + ")");
			zeichnung.moveTo(x[0], y1[0]);
			for (int i = 1; i < n; i++)
				zeichnung.lineTo(x[i], y1[i]);
			break;
		}
	}

	@Override
	protected double value(double x) throws AnpassungException {
		if (ausdruck == null || parameter == null)
			return 0;
		else {
			parameter[parameter.length - 1] = new Double(x);
			try {
				return (Double) ausdruck.evaluate(parameter);
			} catch (InvocationTargetException e) {
				if (DEBUG)
					e.printStackTrace();
				throw new AnpassungException(
						"[AnpassungRegressein] - value konnte nicht berechnet werden ausdruck("
								+ ausdruck + ") Parameter(" + parameter
								+ ") x(" + x + ")");
			}
		}
	}

	@Override
	protected void zeichnen(double von, double bis) throws AnpassungException {
		if (von > bis) {
			double tmp = von;
			von = bis;
			bis = tmp;
		}
		zeichnungVonBis = new Path2D.Double();
		switch (typ) {
		case Ursprungsgerade:
			zeichnungVonBis.moveTo(von, value(von));
			zeichnungVonBis.lineTo(bis, value(bis));
			break;
		case Ausgleichsgerade:
			zeichnungVonBis.moveTo(von, value(von));
			zeichnungVonBis.lineTo(bis, value(bis));
			break;
		case Einhuellende: {
			double dx = (bis - von) / 100;
			zeichnungVonBis.moveTo(von, value(von));
			Path2D zeichnung1 = new Path2D.Double();
			zeichnung1.moveTo(von, -value(von));
			for (int i = 1; i <= 100; i++) {
				zeichnungVonBis.lineTo(von + i * dx, value(von + i * dx));
				zeichnung1.lineTo(von + i * dx, -value(von + i * dx));
			}
			zeichnungVonBis.append(zeichnung1, false);
			break;
		}
		default: {
			double dx = (bis - von) / 100;
			zeichnungVonBis.moveTo(von, value(von));
			for (int i = 1; i <= 100; i++)
				zeichnungVonBis.lineTo(von + i * dx, value(von + i * dx));
			break;
		}
		}
	}

	private class Hyperbel implements RegressionFunction {

		@Override
		public double function(double[] param, double[] x) {
			return param[0] / x[0];
		}

	}

	private class QuadHyperbel implements RegressionFunction {

		@Override
		public double function(double[] param, double[] x) {
			return param[0] / (x[0] * x[0]);
		}

	}

	private class NormalParabel implements RegressionFunction {

		@Override
		public double function(double[] param, double[] x) {
			return param[0] * x[0] * x[0];
		}

	}

	private class Einhuellende implements RegressionFunction {

		@Override
		public double function(double[] param, double[] x) {
			return param[0] * Math.exp(param[1] * x[0])
					* Math.sin(param[2] * x[0] + param[3]);
		}

	}

}
