/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse.darstellung;

import ghmessen.util.Util;

import java.awt.BasicStroke;
import java.awt.Font;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Field;

import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import static ghmessen.Konstanten.*;

public class Eigenschaften implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 819884490570014294L;
	private static final boolean DEBUG = false;
	private transient PropertyChangeSupport support = new PropertyChangeSupport(
			this);

	private void readObject(ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		in.defaultReadObject();
		linie = float2Linie(in.readFloat());
		support = new PropertyChangeSupport(this);
	}

	private void writeObject(ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
		out.writeFloat(linie2Float(linie));
	}

	private float linie2Float(BasicStroke linie) {
		if (linie == LINIESCHMAL)
			return 1.5f;
		else if (linie == LINIEMITTEL)
			return 2.5f;
		else
			return 5.0f;
	}

	private BasicStroke float2Linie(float stroke) {
		if (stroke == 1.5f)
			return LINIESCHMAL;
		else if (stroke == 2.5f)
			return LINIEMITTEL;
		else
			return LINIEBREIT;
	}

	private int schrift2Int(Font schrift) {
		return schrift.getSize();
	}

	private Font int2Schrift(int size) {
		if (SCHRIFTKLEIN.getSize() == size)
			return SCHRIFTKLEIN;
		else if (SCHRIFTMITTEL.getSize() == size)
			return SCHRIFTMITTEL;
		else
			return SCHRIFTGROSS;
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		this.support.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		this.support.removePropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String property,
			PropertyChangeListener listener) {
		this.support.addPropertyChangeListener(property, listener);
	}

	public void removePropertyChangeListener(String property,
			PropertyChangeListener listener) {
		this.support.removePropertyChangeListener(property, listener);
	}

	public static final Font SCHRIFTKLEIN = new Font("Dialog", Font.PLAIN, TABELLE_SCHRIFT_KLEIN);
	public static final Font SCHRIFTMITTEL = new Font("Dialog", Font.PLAIN, TABELLE_SCHRIFT_MITTEL);
	public static final Font SCHRIFTGROSS = new Font("Dialog", Font.PLAIN, TABELLE_SCHRIFT_GROSS);
	private Font schrift;
	public static final BasicStroke LINIESCHMAL = new BasicStroke(GRAPH_LINIE_SCHMAL);
	public static final BasicStroke LINIEMITTEL = new BasicStroke(GRAPH_LINIE_MITTEL);
	public static final BasicStroke LINIEBREIT = new BasicStroke(GRAPH_LINIE_BREIT);
	private transient BasicStroke linie;
	private boolean skalierungAn;
	private boolean skalierungOben;
	private boolean rasterAn;
	private boolean linieAn;
	private boolean achse2An;
	private boolean werteAn;
	private boolean balkenAn;
	public static final int KEINEINTERPOLATION = 0;
	public static final int AKIMAINTERPOLATION = 1;
	public static final int SINCINTERPOLATION = 2;
	private int interpolation;

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT Font (CDATA)>");
		out.println("<!ELEMENT Eigenschaften EMPTY>");
		out.println("<!ATTLIST Eigenschaften schrift (Font) #REQUIRED linie CDATA #REQUIRED skalierungAn (BOOLEAN) #REQUIRED ");
		out.println("\tskalierungOben (BOOLEAN) #REQUIRED ");
		out.println("\trasterAn (BOOLEAN) #REQUIRED linieAn (BOOLEAN) #REQUIRED achse2An (BOOLEAN) #REQUIRED werteAn (BOOLEAN) #REQUIRED balkenAn (BOOLEAN) #REQUIRED ");
		out.println("\tinterpolation CDATA #REQUIRED>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[Eigenschaften] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "schrift", "Font",
				Integer.toString(schrift2Int(schrift)));
		atts.addAttribute("", "", "linie", "CDATA",
				Float.toString(linie2Float(linie)));
		atts.addAttribute("", "", "skalierungAn", "BOOLEAN",
				Boolean.toString(skalierungAn));
		atts.addAttribute("", "", "skalierungOben", "BOOLEAN",
				Boolean.toString(skalierungOben));
		atts.addAttribute("", "", "rasterAn", "BOOLEAN",
				Boolean.toString(rasterAn));
		atts.addAttribute("", "", "linieAn", "BOOLEAN",
				Boolean.toString(linieAn));
		atts.addAttribute("", "", "achse2An", "BOOLEAN",
				Boolean.toString(achse2An));
		atts.addAttribute("", "", "werteAn", "BOOLEAN",
				Boolean.toString(werteAn));
		atts.addAttribute("", "", "balkenAn", "BOOLEAN",
				Boolean.toString(balkenAn));
		atts.addAttribute("", "", "interpolation", "INT",
				Integer.toString(interpolation));
		hd.startElement("", "", "Eigenschaften", atts);
		hd.endElement("", "", "Eigenschaften");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("Eigenschaften")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					/* Kompatibilität: achsenAn -> achse2An */
					if("achsenAn".equals(name))
						name = "achse2An";
					String inhalt = attributes.getValue(index);
					Field field = getClass().getDeclaredField(name);
					if (DEBUG)
						System.out
								.println("[Eigenschaften] - startElement: Feld("
										+ name
										+ ") Typ("
										+ field.getType()
										+ ") Inhalt(" + inhalt + ")");
					if (field.getName().equals("linie")) {
						linie = float2Linie(Float.parseFloat(inhalt));
					} else if (field.getName().equals("schrift")) {
						schrift = int2Schrift(Integer.parseInt(inhalt));
					} else
						Util.setFieldValue(this, field, inhalt);
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		} else
			throw new SAXException("[Eigenschaften] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		if (qName.equals("Eigenschaften"))
			return;
		else
			throw new SAXException("[Eigenschaften] - Ende(" + qName + ")");
	}

	public Eigenschaften() {
		schrift = SCHRIFTKLEIN;
		linie = LINIESCHMAL;
		skalierungAn = true;
		skalierungOben = false;
		rasterAn = true;
		linieAn = true;
		achse2An = true;
		werteAn = false;
		balkenAn = false;
		interpolation = AKIMAINTERPOLATION;
	}

	public void setSchrift(Font schrift) {
		Font alt = this.schrift;
		this.schrift = schrift;
		support.firePropertyChange("schrift", alt, schrift);
	}

	public Font getSchrift() {
		return schrift;
	}

	public void setSchriftKlein(boolean an) {
		Font alt = this.schrift;
		if (an) {
			this.schrift = SCHRIFTKLEIN;
			support.firePropertyChange("schriftKlein",
					alt.equals(SCHRIFTKLEIN), an);
		}
	}

	public boolean getSchriftKlein() {
		return schrift.equals(SCHRIFTKLEIN);
	}

	public void setSchriftMittel(boolean an) {
		Font alt = this.schrift;
		if (an) {
			this.schrift = SCHRIFTMITTEL;
			support.firePropertyChange("schriftMittel",
					alt.equals(SCHRIFTMITTEL), an);
		}
	}

	public boolean getSchriftMittel() {
		return schrift.equals(SCHRIFTMITTEL);
	}

	public void setSchriftGross(boolean an) {
		Font alt = this.schrift;
		if (an) {
			this.schrift = SCHRIFTGROSS;
			support.firePropertyChange("schriftGross",
					alt.equals(SCHRIFTGROSS), an);
		}
	}

	public boolean getSchriftGross() {
		return schrift.equals(SCHRIFTGROSS);
	}

	public void setLinie(BasicStroke linie) {
		BasicStroke alt = this.linie;
		this.linie = linie;
		support.firePropertyChange("linie", alt, linie);
	}

	public BasicStroke getLinie() {
		return linie;
	}

	public void setLinieSchmal(boolean an) {
		BasicStroke alt = this.linie;
		this.linie = an ? LINIESCHMAL : LINIEMITTEL;
		support.firePropertyChange("linieSchmal", alt == LINIESCHMAL, an);
	}

	public boolean getLinieSchmal() {
		return linie == LINIESCHMAL;
	}

	public void setLinieMittel(boolean an) {
		BasicStroke alt = this.linie;
		this.linie = an ? LINIEMITTEL : LINIESCHMAL;
		support.firePropertyChange("linieMittel", alt == LINIEMITTEL, an);
	}

	public boolean getLinieMittel() {
		return linie == LINIEMITTEL;
	}

	public void setLinieBreit(boolean an) {
		BasicStroke alt = this.linie;
		this.linie = an ? LINIEBREIT : LINIEMITTEL;
		support.firePropertyChange("linieBreit", alt == LINIEBREIT, an);
	}

	public boolean getLinieBreit() {
		return linie == LINIEBREIT;
	}

	public void setSkalierungAn(boolean skalierungAn) {
		boolean alt = this.skalierungAn;
		this.skalierungAn = skalierungAn;
		support.firePropertyChange("skalierungAn", alt, skalierungAn);
	}

	public boolean isSkalierungAn() {
		return skalierungAn;
	}

	public void setSkalierungOben(boolean skalierungOben) {
		boolean alt = this.skalierungOben;
		this.skalierungOben = skalierungOben;
		support.firePropertyChange("skalierungOben", alt, skalierungOben);
	}

	public boolean isSkalierungOben() {
		return skalierungOben;
	}

	public void setSkalierungAus(boolean an) {
		setSkalierungAn(!an);
		setSkalierungOben(false);
	}

	public boolean getSkalierungAus() {
		return !isSkalierungAn();
	}

	public void setSkalierungLO(boolean an) {
		setSkalierungAn(an);
		setSkalierungOben(an);
	}

	public boolean getSkalierungLO() {
		return skalierungAn && skalierungOben;
	}

	public void setSkalierungLU(boolean an) {
		setSkalierungAn(an);
		setSkalierungOben(!an);
	}

	public boolean getSkalierungLU() {
		return skalierungAn && !skalierungOben;
	}

	public void setRasterAn(boolean rasterAn) {
		boolean alt = this.rasterAn;
		this.rasterAn = rasterAn;
		support.firePropertyChange("rasterAn", alt, rasterAn);
	}

	public boolean isRasterAn() {
		return rasterAn;
	}

	public void setLinieAn(boolean linieAn) {
		boolean alt = this.linieAn;
		this.linieAn = linieAn;
		support.firePropertyChange("linieAn", alt, linieAn);
	}

	public boolean isLinieAn() {
		return linieAn;
	}

	public void setAchse2An(boolean achse2An) {
		boolean alt = this.achse2An;
		this.achse2An = achse2An;
		support.firePropertyChange("achse2An", alt, achse2An);
	}

	public boolean isAchse2An() {
		return achse2An;
	}

	public void setWerteAn(boolean werteAn) {
		boolean alt = this.werteAn;
		this.werteAn = werteAn;
		support.firePropertyChange("werteAn", alt, werteAn);
	}

	public boolean isWerteAn() {
		return werteAn;
	}

	public void setBalkenAn(boolean balkenAn) {
		boolean alt = this.balkenAn;
		this.balkenAn = balkenAn;
		support.firePropertyChange("balkenAn", alt, balkenAn);
	}

	public boolean isBalkenAn() {
		return balkenAn;
	}

	public void setInterpolation(int interpolation) {
		int alt = this.interpolation;
		this.interpolation = interpolation;
		support.firePropertyChange("interpolation", alt, interpolation);
	}

	public int getInterpolation() {
		return interpolation;
	}

	public void setKeineInterpolation(boolean an) {
		int alt = this.interpolation;
		this.interpolation = an ? KEINEINTERPOLATION : AKIMAINTERPOLATION;
		support.firePropertyChange("keineInterpolation",
				alt == KEINEINTERPOLATION, an);
	}

	public boolean getKeineInterpolation() {
		return interpolation == KEINEINTERPOLATION;
	}

	public void setAkimaInterpolation(boolean an) {
		int alt = this.interpolation;
		this.interpolation = an ? AKIMAINTERPOLATION : KEINEINTERPOLATION;
		support.firePropertyChange("akimaInterpolation",
				alt == AKIMAINTERPOLATION, an);
	}

	public boolean getAkimaInterpolation() {
		return interpolation == AKIMAINTERPOLATION;
	}

	public void setSincInterpolation(boolean an) {
		int alt = this.interpolation;
		this.interpolation = an ? SINCINTERPOLATION : KEINEINTERPOLATION;
		support.firePropertyChange("sincInterpolation",
				alt == SINCINTERPOLATION, an);
	}

	public boolean getSincInterpolation() {
		return interpolation == SINCINTERPOLATION;
	}
}
