/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse.darstellung;

import static ghmessen.Konstanten.LINIENAUSWERTUNG_BREITE;
import static ghmessen.groesse.darstellung.Eigenschaften.LINIEBREIT;
import static ghmessen.groesse.darstellung.Eigenschaften.LINIEMITTEL;
import static ghmessen.groesse.darstellung.Eigenschaften.LINIESCHMAL;
import ghmessen.components.plotcanvas.PlotCanvas;
import ghmessen.util.Util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Field;

import javax.swing.JLabel;
import javax.swing.border.LineBorder;
import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class LinienAuswertung extends JLabel implements Auswertung {

	private static final long serialVersionUID = 7595559484948326615L;
	private static final boolean DEBUG = false;
	private Point2D pos;
	private int col;
	private int oX, oY;
	private boolean vertikal;
	private transient boolean isDrag = false;
	private transient BasicStroke breite;

	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		in.defaultReadObject();
		breite = float2Breite(in.readFloat());
		restoreListenings();
	}

	private void restoreListenings() {
		addComponentListener(new ComponentAdapter() {
			public void componentMoved(ComponentEvent e) {
				updateXY();
			}
		});
		addMouseMotionListener(new MouseMotionListener() {
			@Override
			public void mouseDragged(MouseEvent e) {
				if (!isDrag) {
					getParent().setCursor(
							Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
					isDrag = true;
				}
				Component c = e.getComponent();
				boolean vertikal = ((LinienAuswertung) c).vertikal;
				if (vertikal)
					c.setLocation(c.getX() - oX + e.getX(), c.getY() - oY);
				else
					c.setLocation(c.getX() - oX, c.getY() - oY + e.getY());
				c.getParent().repaint();
			}

			@Override
			public void mouseMoved(MouseEvent e) {
				if (isDrag) {
					getParent().setCursor(Cursor.getDefaultCursor());
					isDrag = false;
				}
			}

		});
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
		out.writeFloat(breite2Float(breite));
	}

	private float breite2Float(BasicStroke breite) {
		if (breite == LINIESCHMAL)
			return 1.5f;
		else if (breite == LINIEMITTEL)
			return 2.5f;
		else
			return 5.0f;

	}

	private BasicStroke float2Breite(float stroke) {
		if (stroke == 1.5f)
			return LINIESCHMAL;
		else if (stroke == 2.5f)
			return LINIEMITTEL;
		else
			return LINIEBREIT;
	}

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT LinienAuswertung EMPTY>");
		out.println("<!ATTLIST LinienAuswertung col (INT) #REQUIRED posx (DOUBLE) #REQUIRED");
		out.println("\tposy (DOUBLE) #REQUIRED oX (INT) #REQUIRED oY (INT) #REQUIRED vertikal (BOOLEAN) #REQUIRED");
		out.println("\tbreite CDATA #REQUIRED>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[LinienAuswertung] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "col", "INT", Integer.toString(col));
		atts.addAttribute("", "", "posx", "DOUBLE", Double.toString(pos.getX()));
		atts.addAttribute("", "", "posy", "DOUBLE", Double.toString(pos.getY()));
		atts.addAttribute("", "", "oX", "INT", Integer.toString(oX));
		atts.addAttribute("", "", "oY", "INT", Integer.toString(oY));
		atts.addAttribute("", "", "vertikal", "BOOLEAN",
				Boolean.toString(vertikal));
		atts.addAttribute("", "", "breite", "CDATA",
				Float.toString(breite2Float(breite)));
		hd.startElement("", "", "LinienAuswertung", atts);
		hd.endElement("", "", "LinienAuswertung");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("LinienAuswertung")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					Field field = null;
					if("col".equals(name))
						col = Integer.parseInt(inhalt);
					else if ("posx".equals(name))
						pos.setLocation(Double.parseDouble(inhalt), pos.getY());
					else if ("posy".equals(name))
						pos.setLocation(pos.getX(), Double.parseDouble(inhalt));
					else if ("breite".equals(name))
						breite = float2Breite(Float.parseFloat(inhalt));
					else {
						field = getClass().getDeclaredField(name);
						Util.setFieldValue(this, field, inhalt);
					}
					if (DEBUG)
						System.out
								.println("[LinienAuswertung] - startElement: Feld("
										+ name
										+ ") Typ("
										+ (field == null ? "null" : field
												.getType())
										+ ") Inhalt("
										+ inhalt + ")");
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		} else
			throw new SAXException("[LinienAuswertung] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		return;
	}

	public LinienAuswertung() {
		this(true, LINIEMITTEL, new Point2D.Double(),-1);
	}

	public LinienAuswertung(boolean vertikal, BasicStroke breite, Point2D pos, int col) {
		super();
		this.vertikal = vertikal;
		oX = vertikal ? LINIENAUSWERTUNG_BREITE / 2 : 0;
		oY = vertikal ? 0 : LINIENAUSWERTUNG_BREITE / 2;
		this.breite = breite;
		if (vertikal) {
			setPreferredSize(new Dimension(LINIENAUSWERTUNG_BREITE,
					Short.MAX_VALUE));
		} else {
			setPreferredSize(new Dimension(Short.MAX_VALUE,
					LINIENAUSWERTUNG_BREITE));
		}
		setEnabled(true);
		setVisible(true);
		setToolTipText("Durch Ziehen mit der Maus kann die Linie positioniert werden.");
		setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		if (DEBUG)
			setBorder(new LineBorder(Color.MAGENTA, 1));
		setPos(pos);
		this.col = col;
		addComponentListener(new ComponentAdapter() {
			public void componentMoved(ComponentEvent e) {
				updateXY();
			}
		});
		addMouseMotionListener(new MouseMotionListener() {
			@Override
			public void mouseDragged(MouseEvent e) {
				Component c = e.getComponent();
				boolean vertikal = ((LinienAuswertung) c).vertikal;
				if (vertikal)
					c.setLocation(c.getX() + e.getX() - oX, c.getY() - oY);
				else
					c.setLocation(c.getX() - oX, c.getY() + e.getY() - oY);
				c.getParent().repaint();
			}

			@Override
			public void mouseMoved(MouseEvent e) {
				return;
			}

		});
	}

	@Override
	public boolean isManuell(){
		return true;
	}
	
	@Override
	public void setManuell(boolean an){
		return;
	}
	
	public void updateXY() {
		if (getParent() instanceof PlotCanvas) {
			PlotCanvas c = (PlotCanvas) getParent();
			Point p = getLocation();
			p.translate(oX, oY);
			pos = c.viewToXY(p);
		}
	}

	public void updateLocation() {
		if (pos != null && getParent() instanceof PlotCanvas) {
			PlotCanvas c = (PlotCanvas) getParent();
			Point p = c.xYToView(pos);
			p.translate(-oX, -oY);
			super.setBounds(p.x, p.y, getWidth(), getHeight());
		}
	}

	@Override
	public void paint(Graphics g) {
		// super.paint(g);
		if (getWidth() <= 0 || getHeight() <= 0)
			return;

		Graphics2D g2 = null;
		try {
			g2 = (Graphics2D) g.create();
			g2.setStroke(breite);
			if (vertikal)
				g2.drawLine(LINIENAUSWERTUNG_BREITE / 2, 0,
						LINIENAUSWERTUNG_BREITE / 2, getHeight());
			else
				g2.drawLine(0, LINIENAUSWERTUNG_BREITE / 2, getWidth(),
						LINIENAUSWERTUNG_BREITE / 2);
		} finally {
			if (g2 != null)
				g2.dispose();
		}
	}

	@Override
	public void setBounds(int x, int y, int width, int height) {
		if (vertikal)
			super.setBounds(x, 0, LINIENAUSWERTUNG_BREITE, height);
		else
			super.setBounds(0, y, width, LINIENAUSWERTUNG_BREITE);
		updateXY();
	}

	@Override
	public Component getKomponente() {
		return this;
	}

	@Override
	public void setKomponente(Component linie) {
		return;
	}

	@Override
	public void setLinienbreite(BasicStroke breite) {
		this.breite = breite;
	}

	@Override
	public BasicStroke getLinienbreite() {
		return this.breite;
	}

	public void setBreite(float breite) {
		this.breite = float2Breite(breite);
	}

	@Override
	public boolean hatKomponente() {
		return true;
	}

	@Override
	public Point2D getPos() {
		return pos;
	}

	@Override
	public void setPos(Point2D pos) {
		this.pos = pos;
		updateLocation();
	}

	public void setOX(int x) {
		oX = x;
	}

	public void setOY(int y) {
		oY = y;
	}

	public void setVertikal(boolean vertikal) {
		this.vertikal = vertikal;
	}

	@Override
	public int getEndeZeile() {
		return -1;
	}

	@Override
	public int getSpalte() {
		return -1;
	}

	@Override
	public boolean enthaeltZeile(int zeile) {
		return false;
	}

	@Override
	public void setTextgroesse(Font font) {
		return;
	}

	@Override
	public Font getTextgroesse() {
		return null;
	}

	@Override
	public void setFarbe(Color farbe) {
		setForeground(farbe);
	}

	@Override
	public Color getFarbe() {
		return getForeground();
	}

}
