/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse.darstellung;

import ghmessen.components.plotcanvas.FreiePositionenLayoutManager;
import ghmessen.components.plotcanvas.PlotCanvas;
import ghmessen.util.Util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Field;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.border.LineBorder;
import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class TextAuswertung extends JLabel implements Auswertung {

	private static final long serialVersionUID = 7595559484948326615L;
	private static final boolean DEBUG = false;
	private Point2D pos;
	private int col;
	private transient boolean isDrag = false;
	private transient TextEingabe eingabe;

	private void readObject(java.io.ObjectInputStream in) throws IOException,
			ClassNotFoundException {
		in.defaultReadObject();
		String text = in.readUTF();
		setText(text);
		pos = (Point2D) in.readObject();
		col = in.readInt();
		setLinienbreite((BasicStroke) in.readObject());
		Font font = (Font) in.readObject();
		setFont(font);
		Color color = (Color) in.readObject();
		setBorder(BorderFactory.createLineBorder(color));
		isDrag = false;
		eingabe = null;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
		out.writeUTF(getText());
		out.writeObject(getPos());
		out.writeInt(col);
		out.writeObject(getLinienbreite());
		out.writeObject(getFont());
		out.writeObject(((LineBorder) getBorder()).getLineColor());
	}

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT TextAuswertung EMPTY>");
		out.println("<!ATTLIST TextAuswertung col (INT) #REQUIRED posx (DOUBLE) #REQUIRED posy (DOUBLE) #REQUIRED");
		out.println("\ttext CDATA #REQUIRED font (Font) #REQUIRED color CDATA #REQUIRED >");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[TextAuswertung] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "col", "INT", Integer.toString(col));
		atts.addAttribute("", "", "posx", "DOUBLE", Double.toString(pos.getX()));
		atts.addAttribute("", "", "posy", "DOUBLE", Double.toString(pos.getY()));
		atts.addAttribute("", "", "text", "CDATA", getText());
		String inhalt = getFont().getFamily();
		inhalt += "-"
				+ (getFont().isPlain() ? "plain" : (getFont().isBold() ? "bold"
						: "") + (getFont().isItalic() ? "italic" : ""));
		inhalt += "-" + getFont().getSize();
		atts.addAttribute("", "", "font", "CDATA", inhalt);
		if (getBorder() instanceof LineBorder) {
			LineBorder lb = (LineBorder) getBorder();
			Color c = lb.getLineColor();
			if (c == null)
				c = Color.BLACK;
			atts.addAttribute("", "", "color", "CDATA",
					Integer.toString(c.getRGB()));

		}
		hd.startElement("", "", "TextAuswertung", atts);
		hd.endElement("", "", "TextAuswertung");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("TextAuswertung")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					Field field = null;
					if("col".equals(name))
						col = Integer.parseInt(inhalt);
					else if ("posx".equals(name))
						pos.setLocation(Double.parseDouble(inhalt), pos.getY());
					else if ("posy".equals(name))
						pos.setLocation(pos.getX(), Double.parseDouble(inhalt));
					else if ("text".equals(name))
						setText(inhalt);
					else if ("font".equals(name))
						setFont(Font.decode(inhalt));
					else if ("color".equals(name))
						setBorder(BorderFactory.createLineBorder(Color
								.decode(inhalt)));
					else {
						field = getClass().getDeclaredField(name);
						Util.setFieldValue(this, field, inhalt);
					}
					if (DEBUG)
						System.out
								.println("[TextAuswertung] - startElement: Feld("
										+ name
										+ ") Typ("
										+ (field == null ? "null" : field
												.getType())
										+ ") Inhalt("
										+ inhalt + ")");
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (NoSuchFieldException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		} else
			throw new SAXException("[TextAuswertung] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		repaint();
	}

	public TextAuswertung() {
		this(Font.getFont(Font.DIALOG), "", Color.BLACK, new Point2D.Double(), -1);
	}

	public TextAuswertung(Font font, String text, Color color, Point2D pos, int col) {
		super(text);
		setOpaque(true);
		setBackground(Color.WHITE);
		eingabe = null;
		setToolTipText("Durch Ziehen mit der Maus kann das Feld positioniert werden.");
		setFont(font);
		setBorder(BorderFactory.createLineBorder(color));
		setPos(pos);
		this.col = col;
		addComponentListener(new ComponentAdapter() {
			public void componentMoved(ComponentEvent e) {
				updateXY();
			}
		});
		addMouseMotionListener(new MouseMotionListener() {
			@Override
			public void mouseDragged(MouseEvent e) {
				if (!isDrag) {
					getParent().setCursor(
							Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
					isDrag = true;
				}
				Component c = e.getComponent();
				c.setLocation(c.getX() + e.getX() - getWidth() / 2, c.getY()
						+ e.getY() - getHeight() / 2);
				c.getParent().repaint();
			}

			@Override
			public void mouseMoved(MouseEvent e) {
				if (isDrag) {
					getParent().setCursor(Cursor.getDefaultCursor());
					isDrag = false;
				}
			}

		});
		addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1
						&& e.getClickCount() == 2) {
					if (e.getSource() instanceof TextAuswertung) {
						TextAuswertung ta = (TextAuswertung) e.getSource();
						if (eingabe == null)
							eingabe = new TextEingabe(ta);
						if (ta.getParent().getLayout() instanceof FreiePositionenLayoutManager) {
							FreiePositionenLayoutManager fpl = (FreiePositionenLayoutManager) ta
									.getParent().getLayout();
							fpl.replaceWithEdit(ta.getParent(), ta, eingabe);
						}
						eingabe.requestFocusInWindow();
					}
				}

			}
		});
	}

	@Override
	public Component getKomponente() {
		return this;
	}

	public void starteEingabe() {
		if (eingabe == null)
			eingabe = new TextEingabe(this);
		if (getParent().getLayout() instanceof FreiePositionenLayoutManager) {
			FreiePositionenLayoutManager fpl = (FreiePositionenLayoutManager) getParent()
					.getLayout();
			fpl.replaceWithEdit(getParent(), this, eingabe);
		}
		eingabe.requestFocusInWindow();
		eingabe.selectAll();
	}

	@Override
	public void setKomponente(Component feld) {
		return;
	}

	@Override
	public boolean hatKomponente() {
		return true;
	}

	@Override
	public boolean isManuell() {
		return true;
	}
	
	@Override
	public void setManuell(boolean an) {
		return;
	}
	
	@Override
	public Point2D getPos() {
		return pos;
	}

	@Override
	public void setPos(Point2D pos) {
		this.pos = pos;
		updateLocation();
	}

	@Override
	public void setLinienbreite(BasicStroke breite) {
		return;
	}

	@Override
	public BasicStroke getLinienbreite() {
		return null;
	}

	@Override
	public int getEndeZeile() {
		return -1;
	}

	@Override
	public int getSpalte() {
		return col;
	}

	@Override
	public boolean enthaeltZeile(int zeile) {
		return false;
	}

	@Override
	public void setTextgroesse(Font font) {
		setFont(font);
	}

	@Override
	public Font getTextgroesse() {
		return getFont();
	}

	@Override
	public void setFarbe(Color farbe) {
		setBorder(BorderFactory.createLineBorder(farbe));
	}

	@Override
	public Color getFarbe() {
		return ((LineBorder) getBorder()).getLineColor();
	}

	@Override
	public void updateXY() {
		if (getParent() instanceof PlotCanvas) {
			PlotCanvas c = (PlotCanvas) getParent();
			pos = c.viewToXY(getLocation());
		}
	}

	@Override
	public void updateLocation() {
		if (pos != null && getParent() instanceof PlotCanvas) {
			PlotCanvas c = (PlotCanvas) getParent();
			setLocation(c.xYToView(pos));
		}
	}

	@Override
	public void setFont(Font f) {
		super.setFont(f);
		try {
			// setColumns(getText().length());
			setSize(getPreferredSize());
		} catch (NullPointerException e) {
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		g.setColor(getBackground());
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setColor(getForeground());
		super.paintComponent(g);
	}

	/*******************************************
	 * Editieren der Textkomponente
	 * *****************************************/
	private class TextEingabe extends JTextField implements ActionListener {
		private static final long serialVersionUID = 1L;
		private TextAuswertung anzeige;

		public TextEingabe(TextAuswertung anzeige) {
			super(anzeige.getText());
			this.anzeige = anzeige;
			this.setBorder(new BevelBorder(BevelBorder.LOWERED, Color.YELLOW,
					Color.GRAY));
			addActionListener(this);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			anzeige.setText(getText());
			if (this.getParent().getLayout() instanceof FreiePositionenLayoutManager) {
				FreiePositionenLayoutManager fpl = (FreiePositionenLayoutManager) this
						.getParent().getLayout();
				fpl.restoreToView(this.getParent(), this, anzeige);
				anzeige.getParent().repaint();
			}
		}

	}

}
