/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import static ghmessen.Konstanten.FARBE_MESSGROESSE;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import ghmessen.GHMessen;
import ghmessen.GHMessenView;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.event.AdapterAddedEvent;
import ghmessen.adapter.event.AdapterChangedEvent;
import ghmessen.adapter.event.AdapterEventListener;
import ghmessen.adapter.event.AdapterRemovedEvent;
import ghmessen.groesse.Art;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.instrumente.komponenten.Werteanzeige;
import ghmessen.instrumente.komponenten.Zeigeranzeige;

public abstract class Messgeraet extends JDialog implements
		AdapterEventListener, ComponentListener {
	private static final long serialVersionUID = -3501529264432959901L;
	private static final boolean DEBUG = false;

	private class NullpunktPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleNullpunktEvent(evt);
		}
	}

	private class MessbereichPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleMessbereichEvent(evt);
		}
	}

	private class SymbolPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleSymbolEvent(evt);
		}
	}

	private class AufnahmePropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleAufnahmeEvent(evt);
		}
	}

	private class EinheitPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleEinheitEvent(evt);
		}
	}

	private class WertPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleWertEvent(evt);
		}
	}

	private class VonPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleVonEvent(evt);
		}
	}

	private class BisPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleBisEvent(evt);
		}
	}

	private class StellenPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleStellenEvent(evt);
		}
	}

	protected Zeigeranzeige skala;
	protected Werteanzeige wert;
	protected Color wertHintergrund;
	protected Groesse g;
	protected IAdapter a;
	protected boolean einlesen;
	protected transient MessgeraetEigenschaften eigenschaftenBox;
	protected int delay;
	protected transient ScheduledThreadPoolExecutor hauptTimer;
	protected transient ScheduledFuture<?> runningTask;
	protected transient Runnable task;
	protected transient IMesskanal messkanal;
	protected JFrame parent;
	
	protected NullpunktPropertyHandler nullpunktPH;
	protected MessbereichPropertyHandler messbereichPH;
	protected SymbolPropertyHandler symbolPH;
	protected AufnahmePropertyHandler aufnahmePH;
	protected EinheitPropertyHandler einheitPH;
	protected WertPropertyHandler wertPH;
	protected VonPropertyHandler vonPH;
	protected BisPropertyHandler bisPH;
	protected StellenPropertyHandler stellenPH;

	public Messgeraet(JFrame parent, IMesskanal messkanal, IAdapter a,
			Groesse g, ScheduledThreadPoolExecutor hauptTimer) {
		super(parent);
		this.messkanal = messkanal;
		this.delay = 250;
		this.einlesen = true;
		this.hauptTimer = hauptTimer;
		this.runningTask = null;
		this.task = null;
		nullpunktPH = new NullpunktPropertyHandler();
		messbereichPH = new MessbereichPropertyHandler();
		symbolPH = new SymbolPropertyHandler();
		aufnahmePH = new AufnahmePropertyHandler();
		einheitPH = new EinheitPropertyHandler();
		wertPH = new WertPropertyHandler();
		vonPH = new VonPropertyHandler();
		bisPH = new BisPropertyHandler();
		stellenPH = new StellenPropertyHandler();
		// setFocusable(false);
		// setFocusableWindowState(false); Eingaben bei Konstanten sind nicht
		// möglich
		addComponentListener(this);
		this.parent = parent;
		this.g = g;
		this.a = a;
		if (g != null) {
			this.g.addPropertyChangeListener("nullpunkt", nullpunktPH);
			this.g.addPropertyChangeListener("messbereich", messbereichPH);
			this.g.addPropertyChangeListener("symbol", symbolPH);
			this.g.addPropertyChangeListener("aufnahme", aufnahmePH);
			this.g.addPropertyChangeListener("einheit", einheitPH);
			this.g.addPropertyChangeListener("wert", wertPH);
			this.g.addPropertyChangeListener("von", vonPH);
			this.g.addPropertyChangeListener("bis", bisPH);
			this.g.addPropertyChangeListener("stellen", stellenPH);
		}

		String titel = g == null ? "Messgerät " : "Messgerät " + g.getSymbol();
		setTitle(titel);
		this.hauptTimer = hauptTimer;
		initComponents();
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		getContentPane().addMouseListener(showEinstellungen);
	}

	public String getSymbol() {
		if (g != null)
			return g.getSymbol();
		else
			return "ø";
	}

	public Art getArt() {
		if (g != null)
			return g.getArt();
		else
			return null;
	}

	public IAdapter getAdapter() {
		return a;
	}

	protected void initSkala() {
		if (g != null) {
			Messbereich mb = g.getMessbereich();
			skala = new Zeigeranzeige(g.getWert(), g.getNullpunkt(), mb,
					FARBE_MESSGROESSE);
		} else
			skala = new Zeigeranzeige();
	}

	protected void initWert() {
		if (g != null) {
			wert = new Werteanzeige(g.getEinheit().toString(), "%2.2f",
					g.getSymbol(), g.getWert());
			wert.setDezimalStellen(g.getStellen());
		} else
			wert = new Werteanzeige();
	}

	protected void initComponents() {
		initKeyBindings();
		setSize(360, 260);
		initSkala();
		skala.setMinimumSize(new Dimension(335, 100));
		skala.setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
		// skala.setAlignmentY(Component.TOP_ALIGNMENT);
		initWert();
		wert.setMinimumSize(new Dimension(330, 100));
		wert.setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
		wertHintergrund = wert.getBackground();
		GroupLayout gl = new GroupLayout(getContentPane());
		gl.setVerticalGroup(gl.createSequentialGroup().addGap(10)
				.addComponent(skala).addGap(10).addComponent(wert).addGap(10));
		gl.setHorizontalGroup(gl
				.createSequentialGroup()
				.addGap(10)
				.addGroup(
						gl.createParallelGroup().addComponent(skala)
								.addComponent(wert)).addGap(10));
		getContentPane().setLayout(gl);
	}

	public void entferneMessgeraet() {
		if (DEBUG)
			System.out.println("[Messgeraet] entferneMessgeraet("
					+ g.getSymbol() + ")");
		if (eigenschaftenBox != null) {
			eigenschaftenBox.schliesseFenster();
			eigenschaftenBox = null;
		}
		if (hauptTimer != null && runningTask != null)
			runningTask.cancel(true);
		setVisible(false);
		if (messkanal != null)
			messkanal = null;
		schliesseFenster();
	}

	public void schliesseFenster() {
		WindowEvent wev = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
		Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
	}

	@Override
	public String getName() {
		if (g == null)
			return null;
		return g.getSymbol();
	}

	public Groesse getGroesse() {
		return g;
	}

	public IMesskanal getMesskanal() {
		return messkanal;
	}

	public void setEinlesen(boolean set) {
		einlesen = set;
	}

	public boolean isEinlesen() {
		return einlesen;
	}

	private final MouseAdapter showEinstellungen = new ShowEinstellungenBox();

	private class ShowEinstellungenBox extends MouseAdapter {
		@Override
		public void mouseClicked(MouseEvent e) {
			showEinstellungenBox();
		}
	}

	public void showEinstellungenBox() {
		if (eigenschaftenBox == null) {
			eigenschaftenBox = new MessgeraetEigenschaften(parent, true, a,
					messkanal, g);
			eigenschaftenBox.setLocationRelativeTo(parent);
			Point pos = eigenschaftenBox.getLocation();
			eigenschaftenBox.setLocation(pos.x + 100, pos.y + 50);
		}
		eigenschaftenBox.setVisible(!eigenschaftenBox.isVisible());
	}

	protected void startTask() {
		if (hauptTimer != null) {
			if (task != null)
				return;
			task = new MessgeraetTask();
		}
	}

	protected void stopTask() {
		if (hauptTimer != null && runningTask != null) {
			runningTask.cancel(true);
		}
		task = null;
	}

	protected void restartTask() {
		stopTask();
		startTask();
	}

	protected boolean executeTask() {
		return messkanal == null ? false : messkanal.istKanalOffen();
	}

	protected class MessgeraetTask implements Runnable {
		@Override
		public void run() {
			if (task == null)
				return;
			executeTask();
		}
	}

	protected abstract void setWert(Double wert, boolean overRange,
			boolean underRange) throws EAdapterException;

	/*********************************************************
	 * ComponentListener
	 *********************************************************/
	@Override
	public void componentResized(ComponentEvent e) {
		return;

	}

	@Override
	public void componentMoved(ComponentEvent e) {
		return;

	}

	@Override
	public void componentHidden(ComponentEvent e) {
		stopTask();
	}

	@Override
	public void componentShown(ComponentEvent e) {
		startTask();
	}

	/******************************************
	 * PropertyChangeListener
	 ******************************************/
	public void handleNullpunktEvent(PropertyChangeEvent evt) {
	}

	public void handleMessbereichEvent(PropertyChangeEvent evt) {
	}

	public void handleSymbolEvent(PropertyChangeEvent evt) {
	}

	public void handleAufnahmeEvent(PropertyChangeEvent evt) {
	}

	public void handleEinheitEvent(PropertyChangeEvent evt) {
	}

	public void handleWertEvent(PropertyChangeEvent evt) {
	}

	public void handleVonEvent(PropertyChangeEvent evt) {
	}

	public void handleBisEvent(PropertyChangeEvent evt) {
	}

	public void handleStellenEvent(PropertyChangeEvent evt) {
	}

	@Override
	public void adapterAdded(AdapterAddedEvent evt) {
		return;

	}

	@Override
	public void adapterRemoved(AdapterRemovedEvent evt) {
		entferneMessgeraet();
	}

	@Override
	public void adapterChanged(AdapterChangedEvent evt) {
		if (g != null && g.getNr() == evt.nr && g.getAB() == evt.ab) {
			if (evt.alt != evt.neu)
				entferneMessgeraet();
		}
	}
	
	protected void initKeyBindings() {
		InputMap inputMap = getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionMap = getRootPane().getActionMap();
		if(inputMap != null && actionMap != null) {
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0, false), "startStoppMessung");
			actionMap.put("startStoppMessung", startStoppMessung);
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0, false), "showParameters");
			actionMap.put("showParameters", showParameters);
			inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0, false), "showEinstellungenBox");
			actionMap.put("showEinstellungenBox", showEinstellungenBox);
		}

	}
	static Action startStoppMessung  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			JFrame f = GHMessen.getApplication().getMainFrame();
			f.dispatchEvent(new KeyEvent(f, KeyEvent.KEY_PRESSED, e.getWhen(), 
					0, KeyEvent.VK_F9, KeyEvent.CHAR_UNDEFINED));
			f.dispatchEvent(new KeyEvent(f, KeyEvent.KEY_RELEASED, e.getWhen(), 
					0, KeyEvent.VK_F9, KeyEvent.CHAR_UNDEFINED));
		}
		
	};
	
	static Action showEinstellungenBox  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GHMessenView mainView = GHMessen.getApplication().getMainView();
			mainView.toggleEinstellungenBox(null);
		}
		
	};
	
	static Action showParameters  = new AbstractAction() {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent e) {
			GHMessenView mainView = GHMessen.getApplication().getMainView();
			mainView.toggleParameters(null);
		}
		
	};
	
}
