/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IMesskanal;
import ghmessen.groesse.Groesse;
import ghmessen.instrumente.SensorPanel.SensorBoxPanel;
import ghmessen.instrumente.komponenten.KorrekturDlg;
import ghmessen.util.CursorTk;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.MissingResourceException;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.WindowConstants;

import ld.api.ios.IBoxLed;
import ld.api.ios.IMirror;
import ld.components.MathString;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;

public class MessgeraetEigenschaften extends javax.swing.JDialog {
	private static final long serialVersionUID = 4557125886203851007L;

	private static boolean DEBUG = false;

	protected Groesse g;
	protected IAdapter a;
	protected IMesskanal messkanal;
	private KorrekturDlg korrekturDlg;
	private Frame parent;

	/** Creates new form MessgeraetEigenschaften */
	public MessgeraetEigenschaften(java.awt.Frame parent, boolean modal,
			IAdapter a, IMesskanal messkanal, Groesse g) {
		super(parent, modal);
		if (DEBUG)
			System.out.println("[MessgeraetEigenschaften] - Konstruktor.");
		this.g = g;
		this.a = a;
		this.messkanal = messkanal;
		korrekturDlg = null;
		this.parent = parent;
		setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		initComponents();
	}

	protected void initComponents() {

		ActionMap actionMap = Application.getInstance().getContext()
				.getActionMap(MessgeraetEigenschaften.class, this);
		ResourceMap resourceMap = Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(MessgeraetEigenschaften.class);

		setSize(new Dimension(600, 500));
		setPreferredSize(new Dimension(600, 500));
		setMaximumSize(new Dimension(32000, 32000));
		schliessen = new javax.swing.JButton(
				resourceMap.getString("schliessen.text"));
		schliessen.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});
		loeschen = new javax.swing.JButton(
				resourceMap.getString("loeschen.text"));
		hilfe = new javax.swing.JButton(resourceMap.getString("hilfe.text"));
		hilfe.setEnabled(false);
		korrigieren = new javax.swing.JButton(actionMap.get("korrigieren"));
		korrigieren.setVisible(messkanal.hatKorrektur());
		setModal(true);

		offset = new JButton(" \u2192 0 \u2190 ");
		offset.setVisible(messkanal.hatKorrektur());
		offset.addActionListener(new AbstractAction() {
			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {
				Component c = null;
				if (e.getSource() instanceof Component)
					c = (Component) e.getSource();
				if (c != null)
					CursorTk.startWaitCursor(c);
				messkanal.korrigiereNullpunkt();
				if (c != null)
					CursorTk.stopWaitCursor(c);
			}
		});

		vorzeichenWechsel = new JButton("s \u2194 -s");
		vorzeichenWechsel.setVisible(messkanal.hatVorzeichenWechsel());
		vorzeichenWechsel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (messkanal instanceof IMirror)
					((IMirror) messkanal).doMirror();
			}
		});

		led = new JButton("LED an/aus");
		led.setVisible(messkanal instanceof IBoxLed);
		led.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				messkanal.wechseleVorzeichen();
			}
		});

		panel = new JPanel();
		panel.setAlignmentY(Component.TOP_ALIGNMENT);
		panel.setAlignmentX(Component.LEFT_ALIGNMENT);

		boxPanel = messkanal.getPanel(messkanal);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		layout.setHorizontalGroup(layout
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						layout.createParallelGroup(Alignment.LEADING)
								.addComponent(panel,
										GroupLayout.PREFERRED_SIZE, 408,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(boxPanel,
										GroupLayout.PREFERRED_SIZE, 408,
										GroupLayout.PREFERRED_SIZE))
				.addGap(18)
				.addGroup(
						layout.createParallelGroup(Alignment.LEADING)
								.addComponent(schliessen,
										GroupLayout.PREFERRED_SIZE, 134,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(hilfe, GroupLayout.DEFAULT_SIZE,
										GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)
								.addComponent(loeschen,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)
								.addComponent(korrigieren,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)
								.addComponent(offset,
										GroupLayout.PREFERRED_SIZE, 134,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(vorzeichenWechsel,
										GroupLayout.PREFERRED_SIZE, 134,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(led, GroupLayout.PREFERRED_SIZE,
										134, GroupLayout.PREFERRED_SIZE))
				.addGap(378));
		layout.setVerticalGroup(layout
				.createSequentialGroup()
				.addContainerGap()
				.addGroup(
						layout.createParallelGroup(Alignment.LEADING)
								.addGroup(
										layout.createSequentialGroup()
												.addComponent(
														panel,
														GroupLayout.PREFERRED_SIZE,
														44,
														GroupLayout.PREFERRED_SIZE)
												.addPreferredGap(
														ComponentPlacement.RELATED)
												.addComponent(
														boxPanel,
														GroupLayout.DEFAULT_SIZE,
														296, Short.MAX_VALUE))
								.addGroup(
										layout.createSequentialGroup()
												.addComponent(schliessen)
												.addGap(17)
												.addComponent(loeschen)
												.addGap(18)
												.addComponent(hilfe)
												.addGap(18)
												.addComponent(korrigieren)
												.addGap(18)
												.addComponent(offset)
												.addGap(18)
												.addComponent(vorzeichenWechsel)
												.addGap(18).addComponent(led)))
				.addContainerGap());
		eingangLabel = new javax.swing.JLabel("Eingang "
				+ g.getSymbol().substring(g.getSymbol().indexOf("_") + 1));
		try {
			eingang = new javax.swing.JTextField(messkanal.getEingang().getName());
		} catch (MissingResourceException e) {
			eingang = new javax.swing.JTextField("Box524092");
		}
		eingang.setEditable(false);

		GroupLayout gl_panel = new GroupLayout(panel);
		gl_panel.setHorizontalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup()
						.addComponent(eingangLabel)
						.addGap(117)
						.addComponent(eingang, GroupLayout.PREFERRED_SIZE, 208,
								GroupLayout.PREFERRED_SIZE)
						.addContainerGap(GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)));
		gl_panel.setVerticalGroup(gl_panel
				.createSequentialGroup()
				.addGroup(
						gl_panel.createParallelGroup(Alignment.BASELINE)
								.addComponent(eingangLabel)
								.addComponent(eingang,
										GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE))
				.addContainerGap(25, Short.MAX_VALUE));
		panel.setLayout(gl_panel);
		getContentPane().setLayout(layout);
		setTitle(MathString.convertToToolTipText("Eigenschaften "
				+ g.getSymbol()));
		this.validate();
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JTextField eingang;
	private javax.swing.JLabel eingangLabel;
	private javax.swing.JButton hilfe;
	private javax.swing.JButton korrigieren;
	private javax.swing.JButton loeschen;
	protected javax.swing.JButton offset;
	protected javax.swing.JButton led;
	protected javax.swing.JButton schliessen;
	private javax.swing.JButton vorzeichenWechsel;
	private JPanel panel;
	private SensorBoxPanel boxPanel;

	// End of variables declaration//GEN-END:variables

	public void schliesseFenster() {
		WindowEvent wev = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
		Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
	}

	@Action
	public void korrigieren() {
		if (korrekturDlg == null) {
			korrekturDlg = new KorrekturDlg(parent, g, g.getEinheitString());
			korrekturDlg.setLocationRelativeTo(parent);
			Point pos = korrekturDlg.getLocation();
			korrekturDlg.setLocation(pos.x + 100, pos.y + 50);
		}
		korrekturDlg.setVisible(true);
	}

}
