/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import static ghmessen.Konstanten.FARBE_KONSTANTE;
import ghmessen.adapter.EAdapterException;
import ghmessen.components.Intervall;
import ghmessen.groesse.Einheit;
import ghmessen.groesse.Groesse;
import ghmessen.instrumente.komponenten.DragZeigeranzeige;
import ghmessen.instrumente.komponenten.EingabeWerteanzeige;

import java.beans.PropertyChangeEvent;

import javax.swing.JFrame;

public class MessgeraetKonstante extends Messgeraet {

	public MessgeraetKonstante(JFrame parent, Groesse g) {
		super(parent, null, null, g, null);
	}

	private static final long serialVersionUID = 6951008540962828185L;
	private static final boolean DEBUG = false;

	@Override
	protected void setWert(Double wert, boolean overRange, boolean underRange)
			throws EAdapterException {

	}

	protected void initSkala() {
		if (g != null) {
			skala = new DragZeigeranzeige(g.getWert(), g.getNullpunkt(),
					g.getVon(), g.getBis(), -1.0, FARBE_KONSTANTE);
		} else
			skala = new DragZeigeranzeige();
		skala.addPropertyChangeListener("wert", wertPH);
	}

	protected void initWert() {
		if (g != null) {
			wert = new EingabeWerteanzeige(g.getEinheit().toString(), "%2.2f",
					g.getSymbol(), g.getWert());
			wert.setDezimalStellen(g.getStellen());
		} else
			wert = new EingabeWerteanzeige();
		wert.addPropertyChangeListener("wert", wertPH);
	}

	public void showEinstellungenBox() {
		return;
	}

	@Override
	public void handleWertEvent(PropertyChangeEvent evt) {
		if (evt.getSource() == skala)
			g.setWert(skala.getWert());
		else if (evt.getSource() == wert)
			g.setWert(wert.getWert());
		else {
			skala.setWert((Double) evt.getNewValue());
			wert.setWert((Double) evt.getNewValue());
		}
	}

	@Override
	public void handleSymbolEvent(PropertyChangeEvent evt) {
		setTitle("Messgerät " + evt.getNewValue());
		wert.setGroesse((String) evt.getNewValue());
	}

	@Override
	public void handleEinheitEvent(PropertyChangeEvent evt) {
		Einheit e = (Einheit) evt.getNewValue();
		wert.setEinheit(e.toString());
	}

	@Override
	public void handleVonEvent(PropertyChangeEvent evt) {
		if (DEBUG)
			System.out
					.println("[MessgeraetKonstante] - propertyChange: Messbereich von - bis geändert.");
		skala.setIntervall(new Intervall(g.getVon(), g.getBis()), -1.0);
	}

	@Override
	public void handleBisEvent(PropertyChangeEvent evt) {
		if (DEBUG)
			System.out
					.println("[MessgeraetKonstante] - propertyChange: Messbereich von - bis geändert.");
		skala.setIntervall(new Intervall(g.getVon(), g.getBis()), -1.0);
	}

	@Override
	public void handleStellenEvent(PropertyChangeEvent evt) {
		wert.setDezimalStellen((Integer) evt.getNewValue());
	}
}
