/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.MesskanalException;
import ghmessen.groesse.Groesse;
import ghmessen.instrumente.komponenten.Logikanzeige;

import java.awt.BorderLayout;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.swing.JFrame;

public class MessgeraetLogik extends Messgeraet {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3501529264432959901L;
	private static final boolean DEBUG = false;
	protected Logikanzeige anzeige;

	public MessgeraetLogik(JFrame parent, IMesskanal messkanal, IAdapter a, Groesse g,
			ScheduledThreadPoolExecutor hauptTimer) {
		super(parent, messkanal, a, g, hauptTimer);
	}

	@Override
	protected void initSkala() {
		skala = null;
	}

	@Override
	protected void initWert() {
		wert = null;
	}

	protected void initAnzeige() {
		anzeige = new Logikanzeige(g.getSymbol(), 0.0);
	}

	@Override
	protected void initComponents() {
		setSize(360, 140);
		initAnzeige();
		getContentPane().add(anzeige, BorderLayout.CENTER);
	}

	@Override
	protected void startTask() {
		super.startTask();
		try {
			scheduleTask();
		} catch (MesskanalException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	protected void scheduleTask() throws MesskanalException {
		if (task != null) {
			runningTask = (ScheduledFuture<?>) hauptTimer
					.scheduleWithFixedDelay(task, 0, delay,
							TimeUnit.MILLISECONDS);
		}
	}

	@Override
	protected boolean executeTask() {
		if (!super.executeTask())
			return false;
		Double d = 0.0;
		try {
			d = messkanal.einlesen();
			setWert(d, false, false);

		} catch (MesskanalException e1) {
			if (DEBUG)
				e1.printStackTrace();
			d = Double.NaN;
		}
		return true;
	}

	@Override
	protected void setWert(Double zahl, boolean overRange, boolean underRange) {
		anzeige.setWert(zahl);
	}
}
