/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import static ghmessen.groesse.Messgroesse.Relais;
import static ghmessen.groesse.Messgroesse.Source;
import ghmessen.GHMessen;
import ghmessen.adapter.AdapterListe;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IMesskanal;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messgroesse;
import ghmessen.parser.RSParser;
import ghmessen.util.CursorTk;
import ghmessen.util.HTML;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import ld.api.CASSY;
import ld.api.IRelay;
import ld.api.ISource;
import ld.api.ios.IToggle;

public class MessgeraetRSEigenschaften extends MessgeraetEigenschaften
		implements DocumentListener {

	private static final long serialVersionUID = 6804956345942468851L;

	private static final boolean DEBUG = false;
	protected boolean hatToggle;
	
	public MessgeraetRSEigenschaften(Frame parent, boolean modal, IAdapter a, IMesskanal messkanal, Groesse g) {
		super(parent, modal, a, messkanal, g);
		p = null;
		if(DEBUG)
			System.out.println("["+this.getClass().toString()+"] Konstruktur aufgerufen...");
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	@Override
	protected void initComponents() {
		hatToggle = messkanal instanceof IToggle;
		
		name = new javax.swing.JLabel();
		formel = new javax.swing.JFormattedTextField();
		wert = new javax.swing.JLabel();
		schliessen = new javax.swing.JButton();
		schliessen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});
		umschalten = new javax.swing.JCheckBox();
		umschalten.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (g != null && e.getSource() instanceof JCheckBox) {
					JCheckBox cb = (JCheckBox) e.getSource();
					if(hatToggle)
						((IToggle)messkanal).setToggle(cb.isSelected());
					g.setSynchron(cb.isSelected());
					if (g.isSynchron()) {
						String formel1 = formel.getText();
						if (!formel1.equals("0") && !formel1.equals("1"))
							formel.setText(g.getMessGroesse() == Relais ? "0"
									: "1");
					}
				}
			}
		});
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		name.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
		name.setText("<html>" + erzeugeName() + "(time,date,n,t,old)=</html>");
		name.setName("name"); // NOI18N

		if (g == null)
			formel.setText("0");
		else if (g.getMessGroesse() == Messgroesse.Relais)
			formel.setText(g.getWert() < 0.5 ? "0" : "1");
		else
			formel.setText(g.getWert() < 0.05 ? "0" : g.getWert() > 0.95 ? "1"
					: Double.toString(g.getWert()));
		formel.setName("formel"); // NOI18N
		formel.getDocument().addDocumentListener(this);

		wert.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
		wert.setText(erzeugeWert());
		wert.setName("wert"); // NOI18N

		schliessen.setText("Schließen");
		schliessen.setName("schliessen"); // NOI18N

		umschalten.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
		umschalten.setText("Umschalten bei automatischer Messung");
		umschalten.setName("umschalten"); // NOI18N
		umschalten.setVisible(hatToggle);
		
		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(
														name,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														249,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addGroup(
														layout.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING)
																.addComponent(
																		schliessen,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		102,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addGroup(
																		layout.createSequentialGroup()
																				.addComponent(
																						formel,
																						javax.swing.GroupLayout.PREFERRED_SIZE,
																						257,
																						javax.swing.GroupLayout.PREFERRED_SIZE)
																				.addPreferredGap(
																						javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																				.addComponent(
																						wert)))
												.addComponent(umschalten))
								.addContainerGap(
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(name,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(
														formel,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(wert))
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
								.addComponent(umschalten)
								.addGap(18, 18, 18)
								.addComponent(schliessen)
								.addContainerGap(
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)));

		pack();
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JFormattedTextField formel;
	private javax.swing.JLabel name;
	private javax.swing.JCheckBox umschalten;
	private javax.swing.JLabel wert;

	// End of variables declaration//GEN-END:variables

	private String erzeugeName() {
		if (g == null) {
			return "Null";
		} else {
			String s = g.getName();
			String symbol = g.getSymbol().replaceAll("_","");
			return s.substring(0, s.indexOf(symbol)) + " "
					+ HTML.symbolZuHTML(g.getSymbol(), false);
		}
	}

	private String erzeugeWert() {
		if (g == null)
			return "=AUS";
		else if (g.getWert() < 0.5)
			return "=AUS";
		else
			return "=AN";
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		if (g != null)
			auswerten();

	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		if (g != null)
			auswerten();

	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		if (g != null)
			auswerten();

	}

	private transient RSParser p;
	private transient AdapterListe al;
	private transient CASSY c;

	private void auswerten() {
		CursorTk.startWaitCursor(this);
		String formel1 = formel.getText();
		if (formel1.equals("0") || formel1.equals("1")) {
			g.setWert(Double.parseDouble(formel1));
			if (c == null) {
				al = GHMessen.getApplication().getMainView().al;
				c = (CASSY) al.get(g.getNr());
			}
			if (c != null) {
				if (g.getMessGroesse() == Relais && c instanceof IRelay)
					((IRelay) c).setRelay(formel1.equals("1"));
				else if (g.getMessGroesse() == Source && c instanceof ISource)
					((ISource) c).setSource(Double.parseDouble(formel1));
			}
			wert.setText(erzeugeWert());
		} else
			wert.setText(" ");
		g.setFormel(formel1);
		if (p == null) {
			p = new RSParser();
		}
		p.setName(g.getSymbol());
		if (p.parsen(formel1)) {
			formel.setBackground(Color.white);
			g.setFormelterm(p.getAusdruck(formel1));
		} else {
			formel.setBackground(Color.ORANGE);
			g.setFormelterm(null);
		}
		CursorTk.stopWaitCursor(this);
	}
}
