/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import java.awt.Dimension;
import java.awt.Point;
import java.io.PrintWriter;
import java.io.Serializable;

import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class MessgeraetStatus implements Serializable {

	private static final long serialVersionUID = 230909808264281271L;
	private static final boolean DEBUG = false;
	public Boolean sichtbar;
	public Point position;
	public Dimension ausdehnung;

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT MessgeraetStatus EMPTY>");
		out.println("<!ATTLIST MessgeraetStatus sichtbar (BOOLEAN) #REQUIRED posX (INT) #REQUIRED");
		out.println("\tposY (INT) #REQUIRED width (INT) #REQUIRED height (INT) #REQUIRED>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[MessgeraetStatus] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("", "", "sichtbar", "BOOLEAN",
				Boolean.toString(sichtbar));
		atts.addAttribute("", "", "posX", "INT", Integer.toString(position.x));
		atts.addAttribute("", "", "posY", "INT", Integer.toString(position.y));
		atts.addAttribute("", "", "width", "INT",
				Integer.toString(ausdehnung.width));
		atts.addAttribute("", "", "height", "INT",
				Integer.toString(ausdehnung.height));
		hd.startElement("", "", "MessgeraetStatus", atts);
		hd.endElement("", "", "MessgeraetStatus");
	}

	public void startElement(String uri, String localName, String qName,
			Attributes attributes)
			throws SAXException {
		if (qName.equals("MessgeraetStatus")) {
			for(int index = 0; index < attributes.getLength();index++){
				try {
					String name = attributes.getLocalName(index);
					String inhalt = attributes.getValue(index);
					if("sichtbar".equals(name))
						sichtbar = Boolean.parseBoolean(inhalt);
					else if("posX".equals(name))
						position.setLocation(Double.parseDouble(inhalt),position.getY());
					else if("posY".equals(name))
						position.setLocation(position.getX(),Double.parseDouble(inhalt));
					else if("width".equals(name))
						ausdehnung.setSize(Integer.parseInt(inhalt), ausdehnung.getHeight());
					else if("height".equals(name))
						ausdehnung.setSize(ausdehnung.getWidth(), Integer.parseInt(inhalt));
					if (DEBUG)
						System.out
								.println("[MessgeraetStatus] - startElement: Feld("
										+ name
										+ ") Inhalt(" + inhalt + ")");
				} catch (SecurityException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				} catch (IllegalArgumentException e) {
					if (DEBUG)
						e.printStackTrace();
					throw new SAXException(e.getMessage());
				}
			}
		}
	}
	
	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		return;
	}

	public MessgeraetStatus(){
		super();
		sichtbar = false;
		position = new Point();
		ausdehnung = new Dimension();
	}

	public MessgeraetStatus(boolean sichtbar, Point position,
			Dimension ausdehnung) {
		this.sichtbar = sichtbar;
		this.position = position;
		this.ausdehnung = ausdehnung;
	}
}
