/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IMesskanal;
import ghmessen.groesse.Groesse;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.JFrame;

import ld.api.ios.ICASSYQuantity;
import ld.api.ios.IWaveform;
import ld.components.MathString;

import org.codehaus.janino.ExpressionEvaluator;

public class MessgeraetStellgroesse extends MessgeraetCASSY {

	private static final long serialVersionUID = -5220536090987520141L;
	private static final boolean DEBUG = false;
	private IWaveform f;
	
	private class FormeltermPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleFormeltermEvent(evt);
		}
	}

	private class ParameterPropertyHandler implements Serializable,
			PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			setzeGeneratorWerte();
		}
	}

	private static final HashSet<String> attrs = new HashSet<String>() {
		private static final long serialVersionUID = 1L;
		{
			add("form");
			add("formSynchron");
			add("amplitude");
			add("intoffset");
			add("frequenz");
			add("anstieg");
			add("formelterm");
		}
	};

	private boolean neueWellenform;

	private FormeltermPropertyHandler formeltermPH;
	private ParameterPropertyHandler parameterPH;

	public MessgeraetStellgroesse(JFrame parent, IMesskanal messkanal, IAdapter a, Groesse g,
			ScheduledThreadPoolExecutor hauptTimer) {
		super(parent, messkanal, a, g, hauptTimer);
		if(messkanal instanceof IWaveform)
			this.f = (IWaveform)messkanal;
		formeltermPH = new FormeltermPropertyHandler();
		parameterPH = new ParameterPropertyHandler();
		g.addPropertyChangeListener("formelterm", formeltermPH);
		for (String s : attrs)
			if (!("formelterm".equals(s)))
				g.addPropertyChangeListener(s, parameterPH);
		neueWellenform = false;
		setTitle(MathString.convertToToolTipText("Stellgröße " + "$"
				+ ((ICASSYQuantity) messkanal).getSymbol() + "$"));
		setzeGeneratorWerte();
		setTitle("Stellgröße "+g.getSymbol());
	}

	private void setzeWellenForm() {
		if (g == null || f == null)
			return;
		if (neueWellenform) {
			ExpressionEvaluator ex = g.getFormelterm();
			if (ex != null) {
				int anzahl = 4000;
				double[] waveForm = new double[anzahl];
				double t = 0.0;
				double delta = 1.0/anzahl;
				if (DEBUG)
					System.out
							.println("[MessgeraetStellgroesse] - setzeWellenForm: anzahl("
									+ anzahl + ") delta(" + delta + ")");
				Object[] parameterValues = new Object[] {
						t, 0.0, 0.0, 0.0, 0.0, 0.0,
						0.0, 0, 0, null, null};
				for (int i = 0; i < anzahl; i++) {
					try {
						waveForm[i] = (Double) ex
								.evaluate(parameterValues);
					} catch (InvocationTargetException e) {
						if (DEBUG)
							e.printStackTrace();
						waveForm[i] = 0;
					}
					t += delta;
					parameterValues[0] = t;
				}
				if (DEBUG) {
					System.out.print(waveForm[0]);
					for (int i = 0; i < anzahl; i++) {
						System.out.print("," + waveForm[i]);
						if (i % 10 == 0)
							System.out.println();
					}
				}
				f.getWaveformParser().setWaveform(waveForm);
				if (DEBUG)
					System.out
							.println("[MessgeraetStellgroesse] - setzeWellenForm: Übertragung beendet.");
				neueWellenform = false;
			}
		}
	}

	private void setzeGeneratorWerte() {
		if (g == null || f == null)
			return;
		if (DEBUG)
			System.out
					.println("[MessgeraetStellgroesse] - setzeGeneratorWerte");
		
		if (neueWellenform)
			setzeWellenForm();
		f.setSingleShot(false);
        f.setWaveform(g.getForm());
        f.getFrequencyParser().setValue(g.getFrequenz());
        f.getAmplitudeParser().setValue(g.getAmplitude());
        f.getOffsetParser().setValue(g.getFormOffset());
        f.getRatioParser().setValue(g.getAnstieg());
	}

	public void handleFormeltermEvent(PropertyChangeEvent evt) {
		if (evt.getNewValue() == null)
			return;
		else
			neueWellenform = true;
		setzeGeneratorWerte();
	}

}
