/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente;

import static ghmessen.Konstanten.FARBE_MESSGROESSE;
import ghmessen.adapter.EAdapterException;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IMesskanal;
import ghmessen.components.Intervall;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messgroesse;
import ghmessen.instrumente.komponenten.Werteanzeige;
import ghmessen.instrumente.komponenten.Zeigeranzeige;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.JFrame;

public class MessgeraetXRay extends Messgeraet {

	private static final long serialVersionUID = 2717511154454775038L;

	/** Creates new form MessgeraetXRay */
	public MessgeraetXRay(JFrame parent, IMesskanal messkanal, IAdapter a, Groesse g,
			ScheduledThreadPoolExecutor hauptTimer) {
		super(parent, messkanal, a, g, hauptTimer);
		delay = 1000; // einmal pro Sekunde ablesen reicht
	}

	@Override
	protected void initSkala() {
		if(g.getMessGroesse() == Messgroesse.Rate || g.getMessGroesse() == Messgroesse.Ereignisse) {
			skala = new Zeigeranzeige(g.getWert(), g.getNullpunkt(),
					g.getVon(), g.getBis(), -1.0, FARBE_MESSGROESSE);
		} else {
			super.initSkala();
		}
	}

	@Override
	protected void initWert() {
		wert = new Werteanzeige(g.getEinheit().toString(), "%2.2f",
				g.getSymbol(), g.getWert());
		wert.setDezimalStellen(g.getStellen());
	}

	@Override
	public void handleBisEvent(PropertyChangeEvent evt) {
		skala.setIntervall(new Intervall(0.0, (Double) evt.getNewValue()), -1.0);
	}

	@Override
	protected void setWert(Double w, boolean overRange, boolean underRange)
			throws EAdapterException {
		// Hier ist nichts zu tun, da kein interaktives Messgerät vorhanden ist.
		wert.setBackground(Color.ORANGE);
	}
}
