/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.SensorPanel;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IAuswahl;
import ghmessen.adapter.IMesskanal;
import ghmessen.groesse.Aufnahme;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messbereich;
import ghmessen.instrumente.komponenten.Nullpunkt;
import ld.api.ios.ICASSYQuantity;
import ld.api.ios.IGateTime;
import ld.api.ios.IMeasurement;
import ld.api.ios.IOrigin;
import ld.api.ios.IRadius;
import ld.api.ios.IRange;
import ld.api.ios.IRangeMax;
import ld.api.ios.IRmsMeasurement;
import ld.api.ios.IUltrasonic;
import ld.api.ios.Measurement;

public class CASSYBoxPanel extends SensorBoxPanel implements DocumentListener {

	private static final long serialVersionUID = -5939377804182781080L;
	private static final boolean DEBUG = false;
	private static ActionMap actionMap;
	private static ResourceMap resourceMap;
	protected boolean firePC;

	/*
	 * This variables can not have a default value, because they are initialized
	 * after SensorBoxPanel.initComponents
	 */
	protected boolean hatGroessen = false;
	protected boolean hatBereiche = false;
	protected boolean hatNullpunkt = false;
	protected boolean hatMittelwert = false;
	protected boolean hatVariablenMittelwert = false;
	protected boolean hatEffektivwert = false;
	protected boolean hatRadius = false;
	protected boolean hatTorzeit = false;
	protected boolean hatMaximum = false;
	protected boolean hatUltraschall = false;
	protected boolean hatTrigger = false;

	private enum Eigenschaft {
		messMaximum, torzeit;
	}

	@Override
	public boolean hatGroessen() {
		return hatGroessen;
	}

	@Override
	public boolean hatBereiche() {
		return hatBereiche;
	}

	@Override
	public boolean hatNullpunkt() {
		return hatNullpunkt;
	}

	@Override
	public boolean hatMesswertAufnahme() {
		return hatMittelwert;
	}

	@Override
	public boolean hatEffektivWert() {
		return hatEffektivwert;
	}

	public boolean hatRadius() {
		return hatRadius;
	}

	public boolean hatTorzeit() {
		return hatTorzeit;
	}

	public boolean hatMaximum() {
		return hatMaximum;
	}

	public boolean hatUltraschall() {
		return hatUltraschall;
	}

	public boolean hatTrigger() {
		return hatTrigger;
	}

	public CASSYBoxPanel(Groesse g, IAdapter a, IMesskanal messkanal) {
		super(g, a, messkanal);
		initCASSYBoxValues();
		firePC = false;
		if (hatGroessen())
			setzeMessgroessenListe();
		if (hatBereiche() && messkanal != null)
			setzeMessbereichListe(messkanal.getMessbereicheStringArray());
		if (hatNullpunkt() && g != null)
			setNullpunkt(g.getNullpunkt());
		if (hatGroessen())
			setMessgroesse();
		if (hatBereiche() && messkanal != null)
			setMessbereich(messkanal.getMessbereichIndex());
		if (hatRadius() && messkanal != null)
			setRadius(((IRadius) messkanal).getRadius());
		if (hatMesswertAufnahme() && g != null)
			setzeMesswertAufnahme();
		if (hatUltraschall() && g != null) {
			setSchmalesSichtfeld(g.isSchmalesSichtfeld());
			setNichtImmerAktiv(g.isNichtImmerAktiv());
		}
		if (hatTrigger() && g != null)
			setGetriggert(g.isAusloeserTriggert());
		firePC = true;
		if (DEBUG)
			System.out.println("[" + this.getClass().toString() + "] Konstruktor aufgerufen");
	}

	private void initCASSYBoxValues() {
		// set properties
		if (messkanal != null) {
			if (!(messkanal.getEingang() instanceof IAuswahl))
				hatGroessen = false;
			else
				hatGroessen = ((IAuswahl) messkanal.getEingang()).getAuswahlAnzahl() > 1;
			hatBereiche = messkanal instanceof IRange && !(messkanal instanceof IRangeMax);
			hatNullpunkt = messkanal instanceof IOrigin;
			hatMittelwert = messkanal instanceof IMeasurement;
			hatVariablenMittelwert = hatMittelwert && ((IMeasurement)messkanal).getVariableMeanTime();
			hatEffektivwert = messkanal instanceof IRmsMeasurement;
			hatRadius = messkanal instanceof IRadius;
			hatTorzeit = messkanal instanceof IGateTime;
			hatMaximum = messkanal instanceof IRangeMax;
			hatUltraschall = messkanal instanceof IUltrasonic;
			hatTrigger = messkanal.istTriggerKanal();
			// modify components
			if(hatVariablenMittelwert)
				intervall.setValue(((IMeasurement)messkanal).getMeanTime());
		}
		if(this.getClass() != CASSYBoxPanel.class)
			return;
		if (!hatNullpunkt)

		{
			links.setVisible(false);
			mittig.setVisible(false);
			rechts.setVisible(false);
			nullpunktPanel.setVisible(false);
		}
		if (!hatGroessen) {
			messgroesse.setVisible(false);
			messgroesseLabel.setVisible(false);
		}
		if (!hatBereiche) {
			messbereichLabel.setVisible(false);
			messbereich.setVisible(false);
		}
		if (!hatMittelwert) {
			aufnahmePanel.setVisible(false);
			momentanwerte.setVisible(false);
			effektivwerte.setVisible(false);
			mittelwerte.setVisible(false);
			intervall.setVisible(false);
			intervallEinheit.setVisible(false);
		} else if (!hatEffektivwert) {
			effektivwerte.setVisible(false);
		}
		if (hatMittelwert && !hatVariablenMittelwert) {
			intervall.setVisible(false);
			intervallEinheit.setVisible(false);
		}
		radius.setVisible(hatRadius);
		lblRadius.setVisible(hatRadius);
		lblRadiusEinheit.setVisible(hatRadius);
		if (!hatMaximum) {
			lblMessMaximum.setVisible(false);
			messMaximum.setVisible(false);
		}
		if (!hatTorzeit) {
			lblTorzeit.setVisible(false);
			torzeit.setVisible(false);
			lblS.setVisible(false);
		}
		if (!hatMaximum && !hatTorzeit)
			counterPanel.setVisible(false);
		if (!hatUltraschall) {
			nichtImmerAktiv.setVisible(false);
			schmalesSichtfeld.setVisible(false);
			if (!hatTrigger)
				ausloeserTriggert.setVisible(false);
			ultraPanel.setVisible(false);
		}
		if (g != null) {
			messMaximum.setValue(new Double(g.getBis()));
			torzeit.setValue(new Double(g.getZeitraum()));
		}
	}

	@Override
	protected void initComponents() {
		actionMap = org.jdesktop.application.Application.getInstance().getContext().getActionMap(CASSYBoxPanel.class,
				this);

		resourceMap = org.jdesktop.application.Application.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(CASSYBoxPanel.class);

		links = new javax.swing.JRadioButton();
		mittig = new javax.swing.JRadioButton();
		rechts = new javax.swing.JRadioButton();
		nullpunktPanel = new javax.swing.JPanel();
		messgroesseLabel = new javax.swing.JLabel();
		messgroesse = new javax.swing.JComboBox<String>();
		messbereichLabel = new javax.swing.JLabel();
		messbereich = new javax.swing.JComboBox<String>();
		nullpunkt = new javax.swing.ButtonGroup();
		radius = new JFormattedTextField();
		aufnahmePanel = new javax.swing.JPanel();
		momentanwerte = new javax.swing.JRadioButton();
		mittelwerte = new javax.swing.JRadioButton();
		effektivwerte = new javax.swing.JRadioButton();
		intervall = new javax.swing.JFormattedTextField();
		intervallEinheit = new javax.swing.JLabel("s");
		aufnahme = new javax.swing.ButtonGroup();

		nullpunktPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("nullpunkt.text")));

		links.setAction(actionMap.get("nullpunkt_links"));
		nullpunkt.add(links);
		mittig.setAction(actionMap.get("nullpunkt_mittig"));
		nullpunkt.add(mittig);
		rechts.setAction(actionMap.get("nullpunkt_rechts"));
		nullpunkt.add(rechts);
		javax.swing.GroupLayout gl_nullpunktPanel = new javax.swing.GroupLayout(nullpunktPanel);
		nullpunktPanel.setLayout(gl_nullpunktPanel);
		gl_nullpunktPanel.setHorizontalGroup(gl_nullpunktPanel.createSequentialGroup().addContainerGap()
				.addComponent(links).addComponent(mittig).addComponent(rechts).addContainerGap());
		gl_nullpunktPanel.setVerticalGroup(gl_nullpunktPanel.createSequentialGroup()
				.addGroup(gl_nullpunktPanel.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
						.addComponent(links).addComponent(mittig).addComponent(rechts)));

		messgroesseLabel.setText(resourceMap.getString("messgroesse.text"));
		messgroesse.setAction(actionMap.get("messgroesse_setzen"));

		messbereichLabel.setText(resourceMap.getString("messbereich.text"));
		messbereich.setAction(actionMap.get("messbereich_setzen"));

		aufnahmePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Messwertaufnahme"));
		aufnahmePanel.setName("jPanel1"); // NOI18N

		momentanwerte.setAction(actionMap.get("aufnahme_momentan")); // NOI18N
		momentanwerte.setSelected(true);
		momentanwerte.setName("momentanwerte"); // NOI18N
		aufnahme.add(momentanwerte);

		mittelwerte.setAction(actionMap.get("aufnahme_mittelwerte")); // NOI18N
		mittelwerte.setName("mittelwerte"); // NOI18N
		aufnahme.add(mittelwerte);

		effektivwerte.setAction(actionMap.get("aufnahme_effektivwerte"));
		effektivwerte.setName("effektivwerte"); // NOI18N
		aufnahme.add(effektivwerte);

		intervall.setName("intervall"); // NOI18N
		intervall.setColumns(5);
		intervall.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				((IMeasurement)messkanal).setMeanTime(((Number) e.getNewValue()).doubleValue());
			}
		});

		javax.swing.GroupLayout gl_aufnamePanel = new javax.swing.GroupLayout(aufnahmePanel);
		aufnahmePanel.setLayout(gl_aufnamePanel);
		gl_aufnamePanel.setHorizontalGroup(gl_aufnamePanel
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(gl_aufnamePanel.createSequentialGroup().addContainerGap().addGroup(gl_aufnamePanel
						.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addComponent(momentanwerte)
						.addGroup(gl_aufnamePanel.createSequentialGroup().addComponent(mittelwerte).addGap(18, 18, 18)
								.addComponent(intervall, javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
								.addGap(1, 1, 1).addComponent(intervallEinheit, javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addComponent(effektivwerte)).addContainerGap(31, Short.MAX_VALUE)));
		gl_aufnamePanel.setVerticalGroup(gl_aufnamePanel.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(gl_aufnamePanel.createSequentialGroup().addContainerGap().addComponent(momentanwerte)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(gl_aufnamePanel.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(mittelwerte)
								.addComponent(intervall, javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
								.addComponent(intervallEinheit, javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED).addComponent(effektivwerte)
						.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)));

		radius.addPropertyChangeListener("value", new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent e) {
				if (!firePC)
					return;
				((IRadius) messkanal).setRadius(((Number) e.getNewValue()).doubleValue());
				g.setRadius(((Number) e.getNewValue()).doubleValue());
			}

		});
		lblRadius = new JLabel(resourceMap.getString("radius.text"));
		lblRadiusEinheit = new JLabel("m");

		counterPanel = new JPanel();
		counterPanel
				.setBorder(new TitledBorder(null, "Z\u00E4hler", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		lblMessMaximum = new JLabel("Messmaximum:");

		messMaximum = new JFormattedTextField();
		messMaximum.getDocument().addDocumentListener(this);
		messMaximum.getDocument().putProperty("name", Eigenschaft.messMaximum);
		lblTorzeit = new JLabel("Torzeit:");

		torzeit = new JFormattedTextField();
		torzeit.getDocument().addDocumentListener(this);
		torzeit.getDocument().putProperty("name", Eigenschaft.torzeit);

		lblS = new JLabel("s");

		GroupLayout gl_counterPanel = new GroupLayout(counterPanel);
		gl_counterPanel.setHorizontalGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_counterPanel.createSequentialGroup().addContainerGap()
						.addGroup(gl_counterPanel.createParallelGroup(Alignment.TRAILING, false)
								.addComponent(lblTorzeit, Alignment.LEADING, GroupLayout.DEFAULT_SIZE,
										GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addComponent(lblMessMaximum, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 109,
										Short.MAX_VALUE))
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING, false)
								.addComponent(messMaximum, GroupLayout.PREFERRED_SIZE, 153, GroupLayout.PREFERRED_SIZE)
								.addGroup(gl_counterPanel.createSequentialGroup()
										.addComponent(torzeit, GroupLayout.PREFERRED_SIZE, 107,
												GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(ComponentPlacement.RELATED).addComponent(lblS,
												GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
						.addContainerGap(97, Short.MAX_VALUE)));
		gl_counterPanel.setVerticalGroup(
				gl_counterPanel.createParallelGroup(Alignment.LEADING).addGroup(gl_counterPanel.createSequentialGroup()
						.addGroup(gl_counterPanel.createParallelGroup(Alignment.BASELINE).addComponent(lblMessMaximum)
								.addComponent(messMaximum, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE))
						.addGap(18)
						.addGroup(gl_counterPanel.createParallelGroup(Alignment.BASELINE).addComponent(lblTorzeit)
								.addComponent(torzeit, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(lblS))
						.addContainerGap(121, Short.MAX_VALUE)));
		counterPanel.setLayout(gl_counterPanel);

		ultraPanel = new JPanel();
		ultraPanel
				.setBorder(new TitledBorder(null, "Messparameter", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		ultraPanel.setName("ultraPanel");

		nichtImmerAktiv = new JCheckBox("nur während einer Messung aktiv");
		nichtImmerAktiv.setAction(actionMap.get("immer_aktiv"));
		ausloeserTriggert = new JCheckBox("Auslöser triggert neue Messung");
		ausloeserTriggert.setAction(actionMap.get("getriggert"));

		schmalesSichtfeld = new JCheckBox("schmales Sichtfeld");
		schmalesSichtfeld.setAction(actionMap.get("schmales_Sichtfeld"));

		javax.swing.GroupLayout gl_ultraPanel = new javax.swing.GroupLayout(ultraPanel);
		gl_ultraPanel.setHorizontalGroup(gl_ultraPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_ultraPanel.createSequentialGroup().addContainerGap()
						.addGroup(gl_ultraPanel.createParallelGroup(Alignment.LEADING)
								.addComponent(nichtImmerAktiv, GroupLayout.DEFAULT_SIZE, 273, Short.MAX_VALUE)
								.addComponent(ausloeserTriggert, GroupLayout.DEFAULT_SIZE, 273, Short.MAX_VALUE)
								.addComponent(schmalesSichtfeld, GroupLayout.DEFAULT_SIZE, 273, Short.MAX_VALUE))
						.addContainerGap()));
		gl_ultraPanel.setVerticalGroup(gl_ultraPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_ultraPanel.createSequentialGroup().addContainerGap().addComponent(nichtImmerAktiv)
						.addPreferredGap(ComponentPlacement.UNRELATED).addComponent(ausloeserTriggert)
						.addPreferredGap(ComponentPlacement.UNRELATED).addComponent(schmalesSichtfeld)
						.addContainerGap(10, Short.MAX_VALUE)));
		ultraPanel.setLayout(gl_ultraPanel);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING)
				.addGroup(layout.createSequentialGroup().addContainerGap().addGroup(layout
						.createParallelGroup(Alignment.LEADING).addComponent(nullpunktPanel, 0, 426, Short.MAX_VALUE)
						.addGroup(layout.createSequentialGroup()
								.addGroup(layout.createParallelGroup(Alignment.LEADING).addComponent(messbereichLabel)
										.addComponent(messgroesseLabel, GroupLayout.PREFERRED_SIZE, 81,
												GroupLayout.PREFERRED_SIZE)
										.addComponent(lblRadius))
								.addPreferredGap(ComponentPlacement.RELATED)
								.addGroup(layout.createParallelGroup(Alignment.LEADING)
										.addComponent(messbereich, 0, 333, Short.MAX_VALUE)
										.addComponent(messgroesse, 0, 333, Short.MAX_VALUE)
										.addGroup(layout.createSequentialGroup()
												.addComponent(radius, GroupLayout.PREFERRED_SIZE, 104,
														GroupLayout.PREFERRED_SIZE)
												.addPreferredGap(ComponentPlacement.RELATED)
												.addComponent(lblRadiusEinheit))))
						.addComponent(aufnahmePanel, javax.swing.GroupLayout.Alignment.LEADING,
								javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addComponent(ultraPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(counterPanel, GroupLayout.DEFAULT_SIZE, 426, Short.MAX_VALUE))
						.addContainerGap()));
		layout.setVerticalGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup()
				.addContainerGap()
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(messgroesseLabel).addComponent(
						messgroesse, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
				.addPreferredGap(ComponentPlacement.UNRELATED)
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(messbereichLabel).addComponent(
						messbereich, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
				.addGap(18)
				.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(lblRadius)
						.addComponent(radius, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addComponent(lblRadiusEinheit))
				.addGap(103)
				.addComponent(aufnahmePanel, javax.swing.GroupLayout.PREFERRED_SIZE,
						javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
				.addComponent(ultraPanel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
						GroupLayout.PREFERRED_SIZE)
				.addPreferredGap(ComponentPlacement.RELATED)
				.addComponent(nullpunktPanel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
						GroupLayout.PREFERRED_SIZE)
				.addContainerGap().addComponent(counterPanel, GroupLayout.DEFAULT_SIZE, 198, Short.MAX_VALUE)
				.addContainerGap()));
		this.setLayout(layout);
		this.validate();
	}

	protected javax.swing.JLabel messgroesseLabel;
	protected javax.swing.JLabel messbereichLabel;
	protected javax.swing.JPanel nullpunktPanel;
	protected JLabel lblRadius;
	protected JLabel lblRadiusEinheit;
	protected JLabel lblMessMaximum;
	protected JPanel counterPanel;
	protected JLabel lblTorzeit;
	protected JLabel lblS;
	protected JPanel ultraPanel;
	protected javax.swing.JRadioButton links;
	protected javax.swing.JComboBox<String> messbereich;
	protected javax.swing.JComboBox<String> messgroesse;
	protected javax.swing.JRadioButton mittig;
	protected javax.swing.JRadioButton rechts;
	protected javax.swing.ButtonGroup nullpunkt;
	protected javax.swing.JFormattedTextField radius;
	protected javax.swing.JRadioButton effektivwerte;
	protected javax.swing.JPanel aufnahmePanel;
	protected javax.swing.JRadioButton mittelwerte;
	protected javax.swing.JRadioButton momentanwerte;
	protected javax.swing.ButtonGroup aufnahme;
	protected javax.swing.JFormattedTextField intervall;
	protected javax.swing.JLabel intervallEinheit;
	protected javax.swing.JFormattedTextField torzeit;
	protected javax.swing.JFormattedTextField messMaximum;
	protected javax.swing.JCheckBox nichtImmerAktiv;
	protected javax.swing.JCheckBox ausloeserTriggert;
	protected javax.swing.JCheckBox schmalesSichtfeld;

	@Action
	public void nullpunkt_links() {
		if (firePC)
			g.setNullpunkt(Nullpunkt.links);
	}

	@Action
	public void nullpunkt_mittig() {
		if (firePC)
			g.setNullpunkt(Nullpunkt.mittig);
	}

	@Action
	public void nullpunkt_rechts() {
		if (firePC)
			g.setNullpunkt(Nullpunkt.rechts);
	}

	@Action
	public void messbereich_setzen() {
		if (!firePC)
			return;
		int index = messbereich.getSelectedIndex();
		messkanal.setMessbereich(index);
		if (DEBUG)
			System.out.println("[StandardBoxPanel] - messbereich_setzen: Name("
					+ messkanal.getMessbereich().getBereichName() + ") Wert(" + index + ")");
		g.setMessbereich(messkanal.getMessbereich());
	}

	@Action
	public void messgroesse_setzen() {
		if (!firePC)
			return;
		int index = messgroesse.getSelectedIndex();
		waehleMessgroesse(index);
		setzeMessbereichListe(messkanal.getMessbereicheStringArray());
		setMessbereich(messkanal.getMessbereichIndex());
	}

	private void waehleMessgroesse(int index) {
		IAuswahl auswahl = (IAuswahl) messkanal.getEingang();
		auswahl.setAuswahl(index);
	}

	protected void setzeMessgroessenListe() {
		messgroesse.setModel(new DefaultComboBoxModel<String>(
				((IAuswahl) messkanal.getEingang()).getListe().toArray(new String[] {})));
	}

	protected void setzeMessbereichListe(String[] eineListe) {
		String[] bereiche = eineListe == null ? messkanal.getMessbereicheStringArray() : eineListe;
		if (DEBUG) {
			System.out.print("[StandardBoxPanel] - setzeMessbereichListe(");
			for (String s : bereiche) {
				System.out.print(s + " ");
			}
			System.out.println(")");
		}
		messbereich.setModel(new DefaultComboBoxModel<String>(bereiche));
	}

	public void setNullpunktEnabled(boolean set) {
		if (links != null && mittig != null && rechts != null && nullpunktPanel != null) {
			((TitledBorder) nullpunktPanel.getBorder()).setTitleColor(set ? Color.BLACK : Color.GRAY);
			links.setEnabled(set);
			mittig.setEnabled(set);
			rechts.setEnabled(set);
		}
	}

	public void setNullpunkt(Nullpunkt nullpunkt) {
		switch (nullpunkt) {
		default:
		case links:
			links.setSelected(true);
			break;
		case mittig:
			mittig.setSelected(true);
			break;
		case rechts:
			rechts.setSelected(true);
			break;
		}
	}

	@Action
	public void aufnahme_momentan() {
		if (firePC) {
			g.setAufnahme(Aufnahme.momentan);
			setAufnahme(Aufnahme.momentan);
		}
		adjustComponents();
	}

	@Action
	public void aufnahme_mittelwerte() {
		if (firePC) {
			g.setAufnahme(Aufnahme.gemittelt);
			setAufnahme(Aufnahme.gemittelt);
		}
		adjustComponents();
	}

	@Action
	public void aufnahme_effektivwerte() {
		if (firePC) {
			g.setAufnahme(Aufnahme.effektiv);
			setAufnahme(Aufnahme.effektiv);
		}
		adjustComponents();
	}

	protected void setzeMesswertAufnahme() {
		switch (g.getAufnahme()) {
		case momentan:
			aufnahme.setSelected(momentanwerte.getModel(), true);
			break;
		case gemittelt:
			aufnahme.setSelected(mittelwerte.getModel(), true);
			break;
		case effektiv:
			aufnahme.setSelected(effektivwerte.getModel(), true);
			break;
		}
	}

	public void setMessgroesse() {
		messgroesse.setSelectedItem(((ICASSYQuantity) messkanal).getToolTipText());
		if (DEBUG)
			System.out.println("[CASSYBoxPanel] - setMessgroesse: " + messgroesse.getSelectedItem());
	}

	public void setMessbereich(int index) {
		messbereich.setSelectedIndex(index);
		if (DEBUG)
			System.out.println("[CASSYBoxPanel] - setMessbereich: index(" + index + ")");
		Messbereich m = g.getMessbereich();
		setNullpunktEnabled(m.getMaximum() == -m.getMinimum());
	}

	public void setRadius(double aRadius) {
		radius.setValue(aRadius);
		if (DEBUG)
			System.out.println("[CASSYBoxPanel] - setRadius: radius(" + aRadius + ")");
	}

	private void wertSchreiben(DocumentEvent e) {
		if (!firePC)
			return;
		switch ((Eigenschaft) e.getDocument().getProperty("name")) {
		case messMaximum:
			((IRangeMax) messkanal).setRangeMax((Double) messMaximum.getValue());
			g.setBis((Double) messMaximum.getValue());
			break;
		case torzeit:
			((IGateTime) messkanal).setGateTime((Double) torzeit.getValue());
			g.setZeitraum((Double) torzeit.getValue());
			break;
		}
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		wertSchreiben(e);

	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		wertSchreiben(e);

	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		return;
	}

	@Action
	public void schmales_Sichtfeld() {
		if (!firePC)
			return;
		boolean an = schmalesSichtfeld.isSelected();
		g.setSchmalesSichtfeld(an);
	}

	public void setSchmalesSichtfeld(boolean an) {
		schmalesSichtfeld.setSelected(an);
	}

	@Action
	public void immer_aktiv() {
		if (!firePC)
			return;
		g.setNichtImmerAktiv(nichtImmerAktiv.isSelected());
	}

	public void setNichtImmerAktiv(boolean an) {
		nichtImmerAktiv.setSelected(an);
	}

	@Action
	public void getriggert() {
		if (!firePC)
			return;
		g.setAusloeserTriggert(ausloeserTriggert.isSelected());
	}

	public void setGetriggert(boolean an) {
		ausloeserTriggert.setSelected(an);
	}

	protected void adjustComponents() {
		intervall.setEnabled(hatVariablenMittelwert && (mittelwerte.isSelected() || effektivwerte.isSelected()));
		intervallEinheit.setEnabled(hatVariablenMittelwert && (mittelwerte.isSelected() || effektivwerte.isSelected()));
	}

	private void setAufnahme(Aufnahme aufnahme) {
		if(messkanal instanceof IMeasurement) {
			IMeasurement im = (IMeasurement)messkanal;
			switch(aufnahme) {
			case momentan:
				im.setMeasurement(Measurement.Normal);
				break;
			case gemittelt:
				im.setMeasurement(Measurement.Mean);
				break;
			case effektiv:
				im.setMeasurement(Measurement.Rms);
				break;
			}
		}
	}
	
}
