/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.SensorPanel;

import ghmessen.GHMessen;
import ghmessen.adapter.Eingang;
import ghmessen.adapter.goIO.AdapterGoIO;
import ghmessen.adapter.goIO.EingangGoIO;
import ghmessen.adapter.goIO.MesskanalGoIO;
import ghmessen.groesse.Art;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.Messbereich;
import ghmessen.groesse.Messgroesse;
import ghmessen.instrumente.komponenten.Nullpunkt;
import ghmessen.util.Util;

import java.awt.Color;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.TitledBorder;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;

public class GoIOBoxPanel extends SensorBoxPanel {

	private static final long serialVersionUID = -5939377804182781080L;
	private static final boolean DEBUG = false;
	private static ActionMap actionMap;
	private static ResourceMap resourceMap;
	protected boolean firePC;

	@Override
	public boolean hatGroessen() {
		return true;
	}

	@Override
	public boolean hatBereiche() {
		return true;
	}

	@Override
	public boolean hatNullpunkt() {
		return true;
	}

	/** Creates new form StandardBoxPanel */
	public GoIOBoxPanel(Groesse g, AdapterGoIO a, MesskanalGoIO messkanal) {
		super(g, a, messkanal);
		initGoIOBoxValues();
		firePC = false;
		setzeMessgroessenListe();
		if (messkanal != null)
			setzeMessbereichListe(messkanal.getMessbereicheStringArray());
		setNullpunkt(g.getNullpunkt());
		setMessgroesse(((EingangGoIO) messkanal.getEingang()).getAuswahl());
		setMessbereich(messkanal.getMessbereichIndex());
		firePC = true;
		if (DEBUG)
			System.out.println("[GoIOBoxPanel] - Konstruktor aufgerufen...");
	}

	private void initGoIOBoxValues() {
		if (g.getMessGroesse() == Messgroesse.Leistung) {
			messgroesse.setVisible(false);
			messgroesseLabel.setVisible(false);
		}
		
	}
	
	protected void initComponents() {

		actionMap = org.jdesktop.application.Application.getInstance()
				.getContext().getActionMap(GoIOBoxPanel.class, this);

		resourceMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getResourceMap(GoIOBoxPanel.class);

		links = new javax.swing.JRadioButton();
		mittig = new javax.swing.JRadioButton();
		rechts = new javax.swing.JRadioButton();
		nullpunktPanel = new javax.swing.JPanel();
		messgroesseLabel = new javax.swing.JLabel();
		messgroesse = new javax.swing.JComboBox<String>();
		messbereichLabel = new javax.swing.JLabel();
		messbereich = new javax.swing.JComboBox<String>();
		nullpunkt = new javax.swing.ButtonGroup();

		nullpunktPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(resourceMap.getString("nullpunkt.text")));

		links.setAction(actionMap.get("nullpunkt_links"));
		nullpunkt.add(links);
		mittig.setAction(actionMap.get("nullpunkt_mittig"));
		nullpunkt.add(mittig);
		rechts.setAction(actionMap.get("nullpunkt_rechts"));
		nullpunkt.add(rechts);
		javax.swing.GroupLayout gl_nullpunktPanel = new javax.swing.GroupLayout(
				nullpunktPanel);
		nullpunktPanel.setLayout(gl_nullpunktPanel);
		gl_nullpunktPanel.setHorizontalGroup(gl_nullpunktPanel
				.createSequentialGroup().addContainerGap().addComponent(links)
				.addComponent(mittig).addComponent(rechts).addContainerGap());
		gl_nullpunktPanel
				.setVerticalGroup(gl_nullpunktPanel
						.createSequentialGroup()
						.addGroup(
								gl_nullpunktPanel
										.createParallelGroup(
												javax.swing.GroupLayout.Alignment.BASELINE)
										.addComponent(links)
										.addComponent(mittig)
										.addComponent(rechts)));

		messgroesseLabel.setText(resourceMap.getString("messgroesse.text"));
		messgroesse.setAction(actionMap.get("messgroesse_setzen"));
		messbereichLabel.setText(resourceMap.getString("messbereich.text"));
		messbereich.setAction(actionMap.get("messbereich_setzen"));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		layout.setHorizontalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addComponent(nullpunktPanel,
														0, 426, Short.MAX_VALUE)
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING)
																				.addComponent(
																						messbereichLabel)
																				.addComponent(
																						messgroesseLabel,
																						GroupLayout.PREFERRED_SIZE,
																						81,
																						GroupLayout.PREFERRED_SIZE))
																.addPreferredGap(
																		ComponentPlacement.RELATED)
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING)
																				.addComponent(
																						messbereich,
																						0,
																						333,
																						Short.MAX_VALUE)
																				.addComponent(
																						messgroesse,
																						0,
																						333,
																						Short.MAX_VALUE))))
								.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(messgroesseLabel)
												.addComponent(
														messgroesse,
														GroupLayout.PREFERRED_SIZE,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(ComponentPlacement.UNRELATED)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(messbereichLabel)
												.addComponent(
														messbereich,
														GroupLayout.PREFERRED_SIZE,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.PREFERRED_SIZE))
								.addGap(18)
								.addGap(103)
								.addComponent(nullpunktPanel,
										GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addContainerGap()));
		this.setLayout(layout);
	}

	protected javax.swing.JLabel messgroesseLabel;
	protected javax.swing.JLabel messbereichLabel;
	protected javax.swing.JPanel nullpunktPanel;
	protected javax.swing.JRadioButton links;
	protected javax.swing.JComboBox<String> messbereich;
	protected javax.swing.JComboBox<String> messgroesse;
	protected javax.swing.JRadioButton mittig;
	protected javax.swing.JRadioButton rechts;
	protected javax.swing.ButtonGroup nullpunkt;

	@Action
	public void nullpunkt_links() {
		if (firePC)
			g.setNullpunkt(Nullpunkt.links);
	}

	@Action
	public void nullpunkt_mittig() {
		if (firePC)
			g.setNullpunkt(Nullpunkt.mittig);
	}

	@Action
	public void nullpunkt_rechts() {
		if (firePC)
			g.setNullpunkt(Nullpunkt.rechts);
	}

	@Action
	public void messbereich_setzen() {
		if (!firePC)
			return;
		int index = messbereich.getSelectedIndex();
		messkanal.setMessbereich(index);
		if (DEBUG)
			System.out.println("[GoIOBoxPanel] - messbereich_setzen: Name("
					+ messkanal.getMessbereich().getBereichName() + ") Wert("
					+ index + ")");
		g.setMessbereich(messkanal.getMessbereich());
	}

	@Action
	public void messgroesse_setzen() {
		if (!firePC)
			return;
		int index = messgroesse.getSelectedIndex();
		waehleMessgroesse(index);
		setzeMessbereichListe(messkanal.getMessbereicheStringArray());
		setMessbereich(messkanal.getMessbereichIndex());
	}

	private void waehleMessgroesse(int index) {
		// Kanäle berücksichtigen
		EingangGoIO sb = (EingangGoIO) messkanal.getEingang();
		int indexAlt = sb.getAuswahl();
		if (index == indexAlt)
			return;
		int kanaeleAlt = sb.getKanaele();
		((Eingang) sb).setAuswahl(index);
		int kanaeleNeu = sb.getKanaele();
		if (DEBUG)
			System.out.println("[GoIOBoxPanel] - waehleMessgroesse: indexAlt("
					+ indexAlt + ") kanaeleAlt(" + kanaeleAlt + ") index("
					+ index + ") kanaeleNeu(" + kanaeleNeu + ")");
		Groessenliste gl = GHMessen.getApplication().getMainView().gl;
		if (kanaeleAlt > kanaeleNeu) {
			for (int ef = kanaeleNeu; ef < kanaeleAlt; ef++) {
				Groesse gWeg = null;
				if (gl != null) {
					if ((gWeg = Util.findeCassyGroesse(gl, g.getNr(),
							g.getAB(), ef)) != null) {
						gWeg.setMessGroesse(Messgroesse.Nichts);
						gWeg.setArt(Art.Parameter);
						if (DEBUG)
							System.out
									.println("[GoIOBoxPanel] - waehleMessgroesse: deaktiviert: "
											+ gWeg.getSymbol()
											+ " Nr("
											+ gWeg.getNr()
											+ ") AB("
											+ gWeg.getAB()
											+ ") EF("
											+ gWeg.getEF() + ")");
					}

				}
			}
		} else if (kanaeleNeu > kanaeleAlt) {
			for (int ef = kanaeleAlt; ef < kanaeleNeu; ef++) {
				Groesse gNeu = null;
				if (gl != null) {
					gNeu = Util.findeCassyGroesse(gl, g.getNr(), g.getAB(), ef);
					if (gNeu == null) {
						gNeu = new Groesse(g.getNr(), g.getAB(), ef);
						gl.add(gNeu);
					}
					if (DEBUG)
						System.out
								.println("[GoIOBoxPanel] - waehleMessgroesse: aktiviert: "
										+ gNeu.getSymbol()
										+ " Nr("
										+ gNeu.getNr()
										+ ") AB("
										+ gNeu.getAB()
										+ ") EF(" + gNeu.getEF() + ")");
				}
			}
		}
		for (int ef = 0; ef < kanaeleNeu; ef++) {
			if (gl != null) {
				Groesse gg = Util.findeCassyGroesse(gl, g.getNr(), g.getAB(),
						ef);
				if (gg.getArt() != Art.Messgroesse)
					gg.setArt(Art.Messgroesse);
				gg.setSymbol(sb.erzeugeSymbol(index, gg.getAB(), gg.getEF()));
				((EingangGoIO) messkanal.getEingang()).setzeStandardWerte(gg);
				if (DEBUG)
					System.out
							.println("[GoIOBoxPanel] - waehleMessgroesse: geändert: "
									+ gg.getSymbol()
									+ " Nr("
									+ gg.getNr()
									+ ") AB("
									+ gg.getAB()
									+ ") EF("
									+ gg.getEF() + ")");
			}
		}
	}

	protected void setzeMessgroessenListe() {
		messgroesse.setModel(new DefaultComboBoxModel<String>(
				((EingangGoIO) messkanal.getEingang()).getGroessenStringList(g
						.getEF())));
	}

	protected void setzeMessbereichListe(String[] eineListe) {
		String[] bereiche = eineListe == null ? messkanal
				.getMessbereicheStringArray() : eineListe;
		if (DEBUG) {
			System.out.print("[GoIOBoxPanel] - setzeMessbereichListe(");
			for (String s : bereiche) {
				System.out.print(s + " ");
			}
			System.out.println(")");
		}
		messbereich.setModel(new DefaultComboBoxModel<String>(bereiche));
	}

	public void setNullpunktEnabled(boolean set) {
		if (links != null && mittig != null && rechts != null
				&& nullpunktPanel != null) {
			((TitledBorder) nullpunktPanel.getBorder())
					.setTitleColor(set ? Color.BLACK : Color.GRAY);
			links.setEnabled(set);
			mittig.setEnabled(set);
			rechts.setEnabled(set);
		}
	}

	public void setNullpunkt(Nullpunkt nullpunkt) {
		switch (nullpunkt) {
		default:
		case links:
			links.setSelected(true);
			break;
		case mittig:
			mittig.setSelected(true);
			break;
		case rechts:
			rechts.setSelected(true);
			break;
		}
	}

	public void setMessgroesse(int index) {
		messgroesse.setSelectedIndex(index);
		if (DEBUG)
			System.out.println("[StandardBoxPanel] - setMessgroesse: index("
					+ index + ")");
	}

	public void setMessbereich(int index) {
		messbereich.setSelectedIndex(index);
		if (DEBUG)
			System.out.println("[StandardBoxPanel] - setMessbereich: index("
					+ index + ")");
		Messbereich m = g.getMessbereich();
		setNullpunktEnabled(m.getMaximum() == -m.getMinimum());
	}

}
