/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.SensorPanel;

import java.awt.ComponentOrientation;
import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultFormatter;
import javax.swing.text.DefaultFormatterFactory;

import org.jdesktop.application.Action;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IAuswahl;
import ghmessen.adapter.IMesskanal;
import ghmessen.components.FormelFormattedTextField;
import ghmessen.components.LogarithmicJSlider;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Messgroesse;
import ghmessen.util.Util;
import ld.api.Waveforms;

public class StellgroessePanel extends CASSYBoxPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -2330516828390634013L;
	private static final boolean DEBUG = false;
	private static String[] namen = new String[] { "Spannung U",
			"Stromstärke I" };
	private ActionMap actionMap;
	private boolean istStrom = false;

	@Override
	public boolean hatGroessen() {
		return true;
	}

	@Override
	public boolean hatBereiche() {
		return true;
	}

	@Override
	public boolean hatNullpunkt() {
		return true;
	}

	@Override
	public boolean hatMesswertAufnahme() {
		return true;
	}

	public StellgroessePanel(Groesse g, IAdapter a, IMesskanal messkanal) {
		super(g, a, messkanal);
		initStellgroesseValues();
	}

	private void initStellgroesseValues() {
		// only init for own class, not for subclass
		if(this.getClass() != StellgroessePanel.class)
			return;
		if(g == null)
			return;
		if (g.getMessGroesse() == Messgroesse.Stromstärke)
			istStrom = true;
		messbereich.setText(g.getMessbereich().getBereichName());
		form.setModel(new DefaultComboBoxModel<String>(new String[] {
				(istStrom ? "Gleichstrom" : "Gleichspannung"), "Sinus (+/-)",
				"Rechteck (+/-)", "Rechteck (0/+)", "Dreieck (+/-)",
				"Dreieck (0/+)", "Formel" }));
		form.setSelectedIndex(g.getForm().ordinal());
		formel.setValue(g.getFormel());
		formel.setName(g.getSymbol());
		starteSynchron.setSelected(g.isFormSynchron());
		frequenzS.setValue((int) (100 * g.getFrequenz()));
		frequenz.setValue(g.getFrequenz());
		DefaultFormatterFactory format = new DefaultFormatterFactory(
				istStrom ? Util.createMinMaxFormatter(-1.0, 1.0, 3) :
					Util.createMinMaxFormatter(-10.0, 10.0, 3));
		amplitudeS.setValue((int)(istStrom?1000*g.getAmplitude():100*g.getAmplitude()));	
		amplitude.setFormatterFactory(format);
		amplitude.setValue(g.getAmplitude());
		amplitudeEinheit.setText(istStrom? "A" : "V");
		offsetS.setValue((int)(istStrom? g.getFormOffset()*1000.0	: g.getFormOffset()*100.0));
		offset.setFormatterFactory(format);
		offset.setValue(g.getFormOffset());
		offsetEinheit.setText(istStrom ? "A" : "V");
		anstiegS.setValue((int)g.getAnstieg());
		anstieg.setValue(new Double(g.getAnstieg()));
	}
	
	@Override
	protected void initComponents() {

		actionMap = org.jdesktop.application.Application.getInstance()
				.getContext().getActionMap(StellgroessePanel.class, this);

		if (a == null)
			System.err.println("[StellgroessePanel] - initComponents: g(" + g
					+ ") CASSY(" + a + ")");

		links = new javax.swing.JRadioButton();
		links.setFont(new Font("Dialog", Font.PLAIN, 11));
		mittig = new javax.swing.JRadioButton();
		mittig.setFont(new Font("Dialog", Font.PLAIN, 11));
		rechts = new javax.swing.JRadioButton();
		rechts.setFont(new Font("Dialog", Font.PLAIN, 11));
		jPanel2 = new javax.swing.JPanel();
		jPanel2.setFont(new Font("Dialog", Font.PLAIN, 11));
		jLabel2 = new javax.swing.JLabel();
		jLabel2.setFont(new Font("Dialog", Font.PLAIN, 11));
		messgroesse = new javax.swing.JComboBox<String>();
		messgroesse.setFont(new Font("Dialog", Font.PLAIN, 11));
		jLabel3 = new javax.swing.JLabel();
		jLabel3.setFont(new Font("Dialog", Font.PLAIN, 11));
		messbereich = new JLabel();
		messbereich.setFont(new Font("Dialog", Font.PLAIN, 11));
		nullpunkt = new javax.swing.ButtonGroup();
		momentanwerte = new JRadioButton();
		mittelwerte = new JRadioButton();
		effektivwerte = new JRadioButton();
		aufnahme = new javax.swing.ButtonGroup();
		form = new javax.swing.JComboBox<String>();

		jPanel2.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Nullpunkt"));
		jPanel2.setName("jPanel2"); // NOI18N

		links.setAction(actionMap.get("nullpunkt_links")); // NOI18N
		links.setName("links"); // NOI18N
		nullpunkt.add(links);

		mittig.setAction(actionMap.get("nullpunkt_mittig")); // NOI18N
		mittig.setName("mittig"); // NOI18N
		nullpunkt.add(mittig);

		rechts.setAction(actionMap.get("nullpunkt_rechts")); // NOI18N
		rechts.setName("rechts"); // NOI18N
		nullpunkt.add(rechts);

		javax.swing.GroupLayout gl_jPanel2 = new javax.swing.GroupLayout(
				jPanel2);
		gl_jPanel2.setHorizontalGroup(gl_jPanel2.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_jPanel2
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								gl_jPanel2
										.createParallelGroup(Alignment.LEADING)
										.addComponent(links)
										.addComponent(mittig)
										.addComponent(rechts))
						.addContainerGap(158, Short.MAX_VALUE)));
		gl_jPanel2.setVerticalGroup(gl_jPanel2.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_jPanel2.createSequentialGroup().addComponent(links)
						.addPreferredGap(ComponentPlacement.RELATED)
						.addComponent(mittig)
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addComponent(rechts)
						.addContainerGap(12, Short.MAX_VALUE)));
		jPanel2.setLayout(gl_jPanel2);

		jLabel2.setText("Stellgröße:");
		jLabel2.setName("jLabel2"); // NOI18N

		messgroesse.setName("messgroesse"); // NOI18N
		messgroesse.setAction(actionMap.get("messgroesse_setzen"));

		jLabel3.setText("Stellbereich:");
		jLabel3.setName("jLabel3"); // NOI18N

		messbereich.setName("messbereich");

		JLabel lblNewLabel = new JLabel("Signalform:");
		lblNewLabel.setFont(new Font("Dialog", Font.PLAIN, 11));

		form.setName("form");
		form.setAction(actionMap.get("signalform"));
		form.setModel(new DefaultComboBoxModel<String>(new String[] {
				"Gleichspannung", "Sinus (+/-)",
				"Rechteck (+/-)", "Rechteck (0/+)", "Dreieck (+/-)",
				"Dreieck (0/+)", "Formel" }));
		form.setFont(new Font("Dialog", Font.PLAIN, 11));

		JPanel panel = new JPanel();
		panel.setFont(new Font("Dialog", Font.PLAIN, 11));
		panel.setBorder(new TitledBorder(null, "Messwertaufnahme",
				TitledBorder.LEADING, TitledBorder.TOP, null, null));

		JPanel panel_1 = new JPanel();
		panel_1.setFont(new Font("Dialog", Font.PLAIN, 11));

		JLabel lFormel = new JLabel("Formel: f(t) =");
		lFormel.setFont(new Font("Dialog", Font.PLAIN, 11));

		DefaultFormatter formelFormatter = new DefaultFormatter();
		formelFormatter.setOverwriteMode(false);
		formel = new FormelFormattedTextField(formelFormatter);
		formel.setColumns(10);
		formel.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(g != null) {
					g.setFormel((String) e.getNewValue());
					g.setFormelterm(formel.getEvaluator());
				}
			}
		});

		starteSynchron = new JCheckBox();
		starteSynchron.setAction(actionMap.get("starte_synchron"));
		starteSynchron.setText("nur während einer Messung aktiv");
		starteSynchron.setFont(new Font("Dialog", Font.PLAIN, 11));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		layout.setHorizontalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.LEADING)
																				.addGroup(
																						layout.createSequentialGroup()
																								.addGroup(
																										layout.createParallelGroup(
																												Alignment.LEADING)
																												.addComponent(
																														jLabel2,
																														GroupLayout.PREFERRED_SIZE,
																														81,
																														GroupLayout.PREFERRED_SIZE)
																												.addComponent(
																														jLabel3)
																												.addComponent(
																														lblNewLabel))
																								.addPreferredGap(
																										ComponentPlacement.RELATED)
																								.addGroup(
																										layout.createParallelGroup(
																												Alignment.LEADING)
																												.addComponent(
																														messbereich,
																														0,
																														219,
																														Short.MAX_VALUE)
																												.addComponent(
																														form,
																														0,
																														219,
																														Short.MAX_VALUE)
																												.addComponent(
																														messgroesse,
																														0,
																														219,
																														Short.MAX_VALUE)))
																				.addGroup(
																						layout.createSequentialGroup()
																								.addComponent(
																										panel,
																										GroupLayout.PREFERRED_SIZE,
																										164,
																										GroupLayout.PREFERRED_SIZE)
																								.addPreferredGap(
																										ComponentPlacement.RELATED)
																								.addComponent(
																										jPanel2,
																										0,
																										0,
																										Short.MAX_VALUE)))
																.addContainerGap())
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		starteSynchron)
																.addContainerGap(
																		122,
																		Short.MAX_VALUE))
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				Alignment.TRAILING,
																				false)
																				.addGroup(
																						layout.createSequentialGroup()
																								.addComponent(
																										lFormel)
																								.addPreferredGap(
																										ComponentPlacement.RELATED)
																								.addComponent(
																										formel))
																				.addComponent(
																						panel_1,
																						Alignment.LEADING,
																						GroupLayout.PREFERRED_SIZE,
																						310,
																						GroupLayout.PREFERRED_SIZE))
																.addContainerGap(
																		14,
																		Short.MAX_VALUE)))));
		layout.setVerticalGroup(layout
				.createParallelGroup(Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(jLabel2)
												.addComponent(
														messgroesse,
														GroupLayout.PREFERRED_SIZE,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.PREFERRED_SIZE))
								.addGap(7)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(messbereich)
												.addComponent(jLabel3))
								.addGap(7)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(
														form,
														GroupLayout.PREFERRED_SIZE,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.PREFERRED_SIZE)
												.addComponent(lblNewLabel))
								.addPreferredGap(ComponentPlacement.UNRELATED)
								.addComponent(starteSynchron)
								.addPreferredGap(ComponentPlacement.UNRELATED)
								.addComponent(panel_1,
										GroupLayout.PREFERRED_SIZE, 111,
										GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(ComponentPlacement.RELATED)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(lFormel)
												.addComponent(
														formel,
														GroupLayout.PREFERRED_SIZE,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(ComponentPlacement.UNRELATED)
								.addGroup(
										layout.createParallelGroup(
												Alignment.BASELINE)
												.addComponent(
														panel,
														GroupLayout.PREFERRED_SIZE,
														107,
														GroupLayout.PREFERRED_SIZE)
												.addComponent(
														jPanel2,
														GroupLayout.PREFERRED_SIZE,
														107,
														GroupLayout.PREFERRED_SIZE))
								.addContainerGap(25, Short.MAX_VALUE)));

		JLabel lblFrequenz = new JLabel("Frequenz:");
		lblFrequenz.setFont(new Font("Dialog", Font.PLAIN, 11));

		frequenzS = new LogarithmicJSlider(SwingConstants.HORIZONTAL, 1,
				5000000,1);
		frequenzS.setFont(new Font("Dialog", Font.PLAIN, 11));
		frequenzS.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		frequenzS.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				frequenz.setValue(frequenzS.getValue() * 0.01);
				if (!frequenzS.getValueIsAdjusting())
					g.setFrequenz(frequenzS.getValue() * 0.01);
			}
		});

		frequenz = new JFormattedTextField(Util.createMinMaxFormatter(0.01, 50000, 2));
		frequenz.setColumns(5);
		frequenz.setFont(new Font("Dialog", Font.PLAIN, 11));
		frequenz.setToolTipText("Frequenz zwischen 0,01 und 50000 Hz");
		frequenz.addPropertyChangeListener("value",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						Number wert = (Number) evt.getNewValue();
						frequenzS.setValue((int) (100 * wert.doubleValue()));
					}
				});

		JLabel lblHz = new JLabel("Hz");
		lblHz.setFont(new Font("Dialog", Font.PLAIN, 11));

		JLabel lblAmplitude = new JLabel("Amplitude:");
		lblAmplitude.setFont(new Font("Dialog", Font.PLAIN, 11));

		amplitudeS = new JSlider(SwingConstants.HORIZONTAL, -1000, 1000, 0);
		amplitudeS.setFont(new Font("Dialog", Font.PLAIN, 11));
		amplitudeS.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				amplitude.setValue(istStrom? amplitudeS.getValue()/1000.0
						: amplitudeS.getValue()/100.0);
				if (!amplitudeS.getValueIsAdjusting() && g != null)
					g.setAmplitude(istStrom? amplitudeS.getValue()/1000.0
							: amplitudeS.getValue()/100.0);
			}
		});

		amplitude = new JFormattedTextField();
		amplitude.setFont(new Font("Dialog", Font.PLAIN, 11));
		amplitude.setColumns(5);
		amplitude.addPropertyChangeListener("value",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						Number wert = (Number) evt.getNewValue();
						amplitudeS.setValue((int)(istStrom? wert.doubleValue()*1000.0
								: wert.doubleValue()*100.0));
					}
				});
		amplitudeEinheit = new JLabel("V ");
		amplitudeEinheit.setFont(new Font("Dialog", Font.PLAIN, 11));

		JLabel lblOffset = new JLabel("Offset:");
		lblOffset.setFont(new Font("Dialog", Font.PLAIN, 11));

		offsetS = new JSlider(SwingConstants.HORIZONTAL, -1000, 1000, 0);
		offsetS.setFont(new Font("Dialog", Font.PLAIN, 11));
		offsetS.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				offset.setValue(istStrom? offsetS.getValue()/1000.0
						: offsetS.getValue()/100.0);
				if (!offsetS.getValueIsAdjusting() && g != null)
					g.setFormOffset(istStrom? offsetS.getValue()/1000.0
							: offsetS.getValue()/100.0);
			}
		});

		offset = new JFormattedTextField();
		offset.setColumns(5);
		offset.setFont(new Font("Dialog", Font.PLAIN, 11));
		offset.addPropertyChangeListener("value", new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				Number wert = (Number) evt.getNewValue();
				offsetS.setValue((int)(istStrom? wert.doubleValue()*1000.0
						: wert.doubleValue()*100.0));
			}
		});

		offsetEinheit = new JLabel("V");
		offsetEinheit.setFont(new Font("Dialog", Font.PLAIN, 11));

		JLabel lblAnstieg = new JLabel("Anstieg:");
		lblAnstieg.setFont(new Font("Dialog", Font.PLAIN, 11));

		anstiegS = new JSlider(SwingConstants.HORIZONTAL, 0, 100, 50);
		anstiegS.setFont(new Font("Dialog", Font.PLAIN, 11));
		anstiegS.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				anstieg.setValue(anstiegS.getValue());
				if (!anstiegS.getValueIsAdjusting() && g!=null)
					g.setAnstieg(anstiegS.getValue());
			}
		});

		anstieg = new JFormattedTextField(Util.createMinMaxFormatter(0, 100, 0));
		anstieg.setColumns(5);
		anstieg.setFont(new Font("Dialog", Font.PLAIN, 11));
		anstieg.addPropertyChangeListener("value",
				new PropertyChangeListener() {
					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						Number wert = (Number) evt.getNewValue();
						anstiegS.setValue(wert.intValue());
					}
				});

		JLabel label = new JLabel("%");
		label.setFont(new Font("Dialog", Font.PLAIN, 11));
		GroupLayout gl_panel_1 = new GroupLayout(panel_1);
		gl_panel_1
				.setHorizontalGroup(gl_panel_1
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_panel_1
										.createSequentialGroup()
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																lblFrequenz)
														.addComponent(
																lblAmplitude)
														.addComponent(lblOffset)
														.addComponent(
																lblAnstieg))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.LEADING,
																false)
														.addComponent(anstiegS,
																0, 0,
																Short.MAX_VALUE)
														.addComponent(offsetS,
																0, 0,
																Short.MAX_VALUE)
														.addComponent(
																amplitudeS, 0,
																0,
																Short.MAX_VALUE)
														.addComponent(
																frequenzS,
																GroupLayout.DEFAULT_SIZE,
																163,
																Short.MAX_VALUE))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.LEADING)
														.addGroup(
																gl_panel_1
																		.createSequentialGroup()
																		.addGroup(
																				gl_panel_1
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addComponent(
																								offset,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								frequenz,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								amplitude,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE))
																		.addPreferredGap(
																				ComponentPlacement.RELATED,
																				GroupLayout.DEFAULT_SIZE,
																				Short.MAX_VALUE)
																		.addGroup(
																				gl_panel_1
																						.createParallelGroup(
																								Alignment.LEADING)
																						.addComponent(
																								lblHz)
																						.addComponent(
																								amplitudeEinheit)
																						.addComponent(
																								offsetEinheit,
																								GroupLayout.DEFAULT_SIZE,
																								16,
																								Short.MAX_VALUE))
																		.addContainerGap(
																				87,
																				Short.MAX_VALUE))
														.addGroup(
																gl_panel_1
																		.createSequentialGroup()
																		.addComponent(
																				anstieg,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				ComponentPlacement.RELATED)
																		.addComponent(
																				label)
																		.addGap(40)))));
		gl_panel_1
				.setVerticalGroup(gl_panel_1
						.createParallelGroup(Alignment.LEADING)
						.addGroup(
								gl_panel_1
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.TRAILING)
														.addComponent(
																lblFrequenz)
														.addGroup(
																gl_panel_1
																		.createParallelGroup(
																				Alignment.LEADING)
																		.addGroup(
																				gl_panel_1
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								frequenz,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								lblHz))
																		.addComponent(
																				frequenzS,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																lblAmplitude)
														.addComponent(
																amplitudeS,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addGroup(
																gl_panel_1
																		.createParallelGroup(
																				Alignment.BASELINE)
																		.addComponent(
																				amplitude,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addComponent(
																				amplitudeEinheit)))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(lblOffset)
														.addGroup(
																gl_panel_1
																		.createParallelGroup(
																				Alignment.TRAILING)
																		.addGroup(
																				gl_panel_1
																						.createParallelGroup(
																								Alignment.BASELINE)
																						.addComponent(
																								offset,
																								GroupLayout.PREFERRED_SIZE,
																								GroupLayout.DEFAULT_SIZE,
																								GroupLayout.PREFERRED_SIZE)
																						.addComponent(
																								offsetEinheit))
																		.addComponent(
																				offsetS,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)))
										.addPreferredGap(
												ComponentPlacement.RELATED)
										.addGroup(
												gl_panel_1
														.createParallelGroup(
																Alignment.LEADING)
														.addComponent(
																lblAnstieg)
														.addComponent(
																anstiegS,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addGroup(
																gl_panel_1
																		.createParallelGroup(
																				Alignment.BASELINE)
																		.addComponent(
																				anstieg,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addComponent(
																				label)))
										.addContainerGap(85, Short.MAX_VALUE)));
		panel_1.setLayout(gl_panel_1);

		momentanwerte.setHorizontalAlignment(SwingConstants.LEFT);
		momentanwerte.setFont(new Font("Dialog", Font.PLAIN, 11));
		momentanwerte.setAction(actionMap.get("aufnahme_momentan"));
		momentanwerte.setText("Momentanwerte");
		aufnahme.add(momentanwerte);

		mittelwerte.setHorizontalAlignment(SwingConstants.LEFT);
		mittelwerte.setFont(new Font("Dialog", Font.PLAIN, 11));
		mittelwerte.setAction(actionMap.get("aufnahme_mittelwerte"));
		mittelwerte.setText("Mittelwerte");
		aufnahme.add(mittelwerte);

		effektivwerte.setFont(new Font("Dialog", Font.PLAIN, 11));
		effektivwerte.setHorizontalAlignment(SwingConstants.LEFT);
		effektivwerte.setAction(actionMap.get("aufnahme_effektivwerte"));
		effektivwerte.setText("Effektivwerte");
		aufnahme.add(effektivwerte);

		GroupLayout gl_panel = new GroupLayout(panel);
		gl_panel.setHorizontalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup()
						.addGroup(
								gl_panel.createParallelGroup(Alignment.LEADING)
										.addComponent(momentanwerte)
										.addComponent(mittelwerte)
										.addComponent(effektivwerte))
						.addContainerGap(68, Short.MAX_VALUE)));
		gl_panel.setVerticalGroup(gl_panel.createParallelGroup(
				Alignment.LEADING).addGroup(
				gl_panel.createSequentialGroup().addComponent(momentanwerte)
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addComponent(mittelwerte)
						.addPreferredGap(ComponentPlacement.UNRELATED)
						.addComponent(effektivwerte).addGap(48)));
		panel.setLayout(gl_panel);
		this.setLayout(layout);
	}

	protected javax.swing.JLabel jLabel2;
	protected javax.swing.JLabel jLabel3;
	protected javax.swing.JLabel messbereich;
	protected javax.swing.JPanel jPanel2;
	protected javax.swing.JLabel amplitudeEinheit;
	protected javax.swing.JLabel offsetEinheit;
	protected javax.swing.ButtonGroup aufnahme;
	protected javax.swing.JRadioButton momentanwerte;
	protected javax.swing.JRadioButton mittelwerte;
	protected javax.swing.JRadioButton effektivwerte;
	protected javax.swing.JCheckBox starteSynchron;
	protected javax.swing.JComboBox<String> form;
	protected LogarithmicJSlider frequenzS;
	protected javax.swing.JSlider amplitudeS, anstiegS, offsetS;
	protected javax.swing.JFormattedTextField frequenz, amplitude, anstieg,
			offset;
	protected FormelFormattedTextField formel;

	@Override
	protected void setzeMessbereichListe(String[] liste) {
		// Schaltet die Methode von StandardboxPanel aus.
		return;
	}

	@Action
	public void nullpunkt_links() {
		super.nullpunkt_links();
	}

	@Action
	public void nullpunkt_mittig() {
		super.nullpunkt_mittig();
	}

	@Action
	public void nullpunkt_rechts() {
		super.nullpunkt_rechts();
	}

	@Action
	public void aufnahme_momentan() {
		super.aufnahme_momentan();
	}

	@Action
	public void aufnahme_mittelwerte() {
		super.aufnahme_mittelwerte();
	}

	@Action
	public void aufnahme_effektivwerte() {
		super.aufnahme_effektivwerte();
	}

	@Action
	public void starte_synchron() {
		if (DEBUG)
			System.out
					.println("[StellgroessePanel] - starte_synchron: synchron("
							+ starteSynchron.isSelected() + ")");
		g.setFormSynchron(starteSynchron.isSelected());
	}

	@Action
	public void messgroesse_setzen() {
		if(firePC) {
			int index = messgroesse.getSelectedIndex();
			((IAuswahl) messkanal.getEingang()).setAuswahl(index);
		}
	}

	@Override
	protected void setzeMessgroessenListe() {
		if (DEBUG)
			System.out.println("[StellgroessePanel] - setzeMessgroessenListe");
		messgroesse.setModel(new DefaultComboBoxModel<String>(new String[] {
				namen[0] + "_" + a.getNr(), namen[1] + "_" + a.getNr() }));
		messgroesse
				.setSelectedIndex(g.getMessGroesse() == Messgroesse.Spannung ? 0
						: 1);
	}

	@Override
	public void setMessbereich(int index) {
		messbereich.setText(messkanal.getMessbereich().getName());
		if (DEBUG)
			System.out.println("[StellgroessePanel] - setMessbereich: index("
					+ index + ")");
	}

	// This method cannot be used from superclass, otherwise wrong instance of aufnahme is used.
	@Override
	protected void setzeMesswertAufnahme() {
		switch (g.getAufnahme()) {
		case momentan:
			aufnahme.setSelected(momentanwerte.getModel(), true);
			break;
		case gemittelt:
			aufnahme.setSelected(mittelwerte.getModel(), true);
			break;
		case effektiv:
			aufnahme.setSelected(effektivwerte.getModel(), true);
			break;
		}
	}

	@Action
	public void signalform() {
		int index = form.getSelectedIndex();
		if (index < 0 && index >= form.getModel().getSize())
			return;
		g.setForm(Waveforms.forValue(index));
		if (DEBUG)
			System.out.println("[StellgroessePanel] - signalform: form("
					+ g.getForm() + ")");
	}

	@Override
	protected void adjustComponents() {
		// disable adjustComponents from CASSYBoxPanel, because intervall not existing
	}
}
