/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.SensorPanel;

import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IAuswahl;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.cassy.EingangCASSY;
import ghmessen.groesse.Groesse;

import java.awt.event.ActionEvent;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import ld.api.ios.ICASSYTimerQuantity;
import ld.api.ios.ICollision;
import ld.api.ios.IGateTime;
import ld.api.ios.IRadius;
import ld.api.ios.IRangeMax;

import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;

public class TimerBoxPanel extends SensorBoxPanel implements DocumentListener {

	private static final long serialVersionUID = -8941696654605773539L;
	private static final boolean DEBUG = false;

	private static ActionMap actionMap;
	private static ResourceMap resourceMap;

	private enum Eigenschaft {
		messMaximum, radius, fahne, torzeit, flanke;
	}

	@Override
	public boolean hatGroessen() {
		return true;
	}

	/** Creates new form TimerBoxPanel */
	public TimerBoxPanel(Groesse g, IAdapter a, IMesskanal messkanal) {
		super(g, a, messkanal);
		initTimerBoxValues();
		if (DEBUG)
			System.out.println("Konstruktor TimerBoxPanel...");
		setzeMessgroessenListe();
		adjustComponents();
	}

	private void initTimerBoxValues() {
		if (messkanal != null && g != null) {
			messMaximum.setValue(g.getBis());
			radius.setValue(g.getRadius());
			if (messkanal instanceof ICollision) {
				if (messkanal instanceof IRadius) {
					torzeit.setValue(g.getFahne());
					torzeitEinheit.setText("mm");
					lblTorzeit.setText(resourceMap.getString("fahne.text"));
				} else {
					torzeit.setValue(g.getRadius());
					torzeitEinheit.setText("m");
					lblTorzeit.setText(resourceMap.getString("radius.text"));
				}
			} else {
				torzeit.setValue(g.getZeitraum());
				torzeitEinheit.setText("s");
				lblTorzeit.setText(resourceMap.getString("torzeit.text"));
			}
			if (messkanal instanceof ICollision && messkanal instanceof IRadius)
				messbereichEinheit.setText("rad/s");
			else if (messkanal instanceof ICASSYTimerQuantity)
				messbereichEinheit.setText("ms");
			else
				messbereichEinheit.setText(g.getMessGroesse().getEinheit().toString());

		}
	}

	protected void initComponents() {
		messgroesseLabel = new javax.swing.JLabel();
		messgroesse = new javax.swing.JComboBox<String>();

		actionMap = Application.getInstance(ghmessen.GHMessen.class).getContext().getActionMap(TimerBoxPanel.class,
				this);

		resourceMap = Application.getInstance(ghmessen.GHMessen.class).getContext().getResourceMap(TimerBoxPanel.class);

		messgroesseLabel.setText(resourceMap.getString("messgroesse.text"));
		messgroesse.setAction(actionMap.get("messgroesse_setzen"));

		lblMessbereich = new JLabel(resourceMap.getString("messbereich.text"));

		messMaximum = new JFormattedTextField();
		messMaximum.getDocument().addDocumentListener(this);
		messMaximum.getDocument().putProperty("name", Eigenschaft.messMaximum);

		torzeit = new JFormattedTextField();
		torzeitEinheit = new JLabel("mm");
		lblTorzeit = new JLabel(resourceMap.getString("fahne.text"));
		torzeit.getDocument().addDocumentListener(this);
		torzeit.getDocument().putProperty("name", Eigenschaft.torzeit);
		messbereichEinheit = new JLabel("xxx");

		setzeOptionenPanel();

		radius = new JFormattedTextField();
		radius.getDocument().addDocumentListener(this);
		radius.getDocument().putProperty("name", Eigenschaft.radius);

		lblRadiusBruch = new JLabel("/");

		radiusEinheit = new JLabel("m");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING)
				.addGroup(layout.createSequentialGroup().addContainerGap()
						.addGroup(layout.createParallelGroup(Alignment.LEADING)
								.addComponent(opts, GroupLayout.DEFAULT_SIZE, 332, Short.MAX_VALUE)
								.addGroup(layout.createSequentialGroup()
										.addComponent(messgroesseLabel, GroupLayout.PREFERRED_SIZE, 81,
												GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(ComponentPlacement.RELATED)
										.addComponent(messgroesse, 0, 239, Short.MAX_VALUE))
								.addGroup(layout.createSequentialGroup()
										.addGroup(layout.createParallelGroup(Alignment.LEADING)
												.addComponent(lblMessbereich).addComponent(lblTorzeit))
										.addPreferredGap(ComponentPlacement.RELATED)
										.addGroup(layout.createParallelGroup(Alignment.LEADING, false)
												.addComponent(messMaximum, GroupLayout.PREFERRED_SIZE, 95,
														GroupLayout.PREFERRED_SIZE)
												.addGroup(layout.createSequentialGroup()
														.addComponent(torzeit, GroupLayout.PREFERRED_SIZE, 60,
																GroupLayout.PREFERRED_SIZE)
														.addPreferredGap(ComponentPlacement.RELATED)
														.addComponent(torzeitEinheit, GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
										.addPreferredGap(ComponentPlacement.RELATED)
										.addGroup(layout.createParallelGroup(Alignment.LEADING)
												.addGroup(layout.createSequentialGroup()
														.addComponent(lblRadiusBruch, GroupLayout.PREFERRED_SIZE, 18,
																GroupLayout.PREFERRED_SIZE)
														.addPreferredGap(ComponentPlacement.RELATED)
														.addComponent(radius, GroupLayout.PREFERRED_SIZE, 63,
																GroupLayout.PREFERRED_SIZE)
														.addPreferredGap(ComponentPlacement.RELATED)
														.addComponent(radiusEinheit))
												.addComponent(messbereichEinheit))))
						.addContainerGap()));
		layout.setVerticalGroup(layout.createParallelGroup(Alignment.LEADING)
				.addGroup(layout.createSequentialGroup().addContainerGap()
						.addGroup(layout.createParallelGroup(Alignment.BASELINE)
								.addComponent(messgroesse, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(messgroesseLabel))
						.addGap(18)
						.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(lblMessbereich)
								.addComponent(messMaximum, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(messbereichEinheit))
						.addGap(18)
						.addGroup(layout.createParallelGroup(Alignment.BASELINE).addComponent(lblTorzeit)
								.addComponent(torzeit, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(torzeitEinheit).addComponent(lblRadiusBruch)
								.addComponent(radius, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(radiusEinheit))
						.addGap(18).addComponent(opts, GroupLayout.DEFAULT_SIZE, 194, Short.MAX_VALUE)
						.addContainerGap()));
		this.setLayout(layout);
	}// </editor-fold>

	// Variables declaration - do not modify

	protected javax.swing.JLabel messgroesseLabel;
	protected javax.swing.JComboBox<String> messgroesse;
	protected JFormattedTextField messMaximum;
	protected JLabel messbereichEinheit;
	protected JLabel lblTorzeit;
	protected JFormattedTextField torzeit;
	protected JLabel torzeitEinheit;
	protected TimerBoxPanelOptionenStandard opts;
	protected JFormattedTextField radius;
	protected JLabel radiusEinheit;
	private JLabel lblRadiusBruch;
	private JLabel lblMessbereich;

	// End of variables declaration

	private void adjustComponents() {
		messMaximum.setVisible(messkanal instanceof IRangeMax);
		torzeit.setVisible(messkanal instanceof IGateTime);
		radius.setVisible(messkanal instanceof IRadius);
		if (messkanal instanceof ICollision && messkanal instanceof IRadius)
			lblTorzeit.setText("Radius");
		else if (messkanal instanceof ICollision)
			lblTorzeit.setText("Fahne");
		else
			lblTorzeit.setText("Torzeit");
		opts.adjustComponents();
		adjustLabels();
	}

	private void adjustLabels() {
		lblRadiusBruch.setVisible(radius.isVisible());
		radiusEinheit.setVisible(radius.isVisible());
		torzeitEinheit.setVisible(torzeit.isVisible());
		lblTorzeit.setVisible(torzeit.isVisible());
		messbereichEinheit.setVisible(messMaximum.isVisible());
		lblMessbereich.setVisible(messMaximum.isVisible());
	}

	protected void setzeMessgroessenListe() {
		messgroesse.setModel(new DefaultComboBoxModel<String>(
				((EingangCASSY) messkanal.getEingang()).getListe().toArray(new String[] {})));
		messgroesse.setSelectedIndex(((EingangCASSY) messkanal.getEingang()).getAuswahl());
	}

	@Action
	public void messgroesse_setzen(ActionEvent e) {
		if (e.getSource() == messgroesse) {
			waehleMessgroesse(messgroesse.getSelectedIndex());
		}
	}

	private void waehleMessgroesse(int index) {
		IAuswahl auswahl = ((IAuswahl) messkanal.getEingang());
		auswahl.setAuswahl(index);
		setzeOptionenPanel();
		opts.leseWerte();
		adjustComponents();
	}

	private void setzeOptionenPanel() {
		if (messkanal instanceof ICollision)
			opts = new TimerBoxPanelOptionenStoss(g, messkanal);
		else
			opts = new TimerBoxPanelOptionenStandard(g, messkanal);
	}

	private void wertSchreiben(DocumentEvent e) {
		switch ((Eigenschaft) e.getDocument().getProperty("name")) {
		case messMaximum:
			g.setBis((Double) messMaximum.getValue());
			break;
		case torzeit:
			if (messkanal instanceof IRadius)
				g.setRadius((Double) torzeit.getValue());
			else if (messkanal instanceof ICollision)
				g.setFahne((Double) torzeit.getValue());
			else
				g.setZeitraum((Double) torzeit.getValue());
			break;
		case radius:
			g.setRadius((Double) radius.getValue());
			break;
		default:
			break;
		}
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		wertSchreiben(e);

	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		wertSchreiben(e);

	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		return;
	}
}
