/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.SensorPanel;

import ghmessen.GHMessen;
import ghmessen.Messparameter;
import ghmessen.adapter.IAdapter;
import ghmessen.adapter.IAuswahl;
import ghmessen.adapter.IMesskanal;
import ghmessen.adapter.cassy.EingangXRay;
import ghmessen.components.DoubleFieldFocusAdapter;
import ghmessen.groesse.Groesse;

import java.util.HashMap;

import javax.swing.ActionMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.TitledBorder;

import org.jdesktop.application.Action;

public class XRayPanel extends SensorBoxPanel {

	private static final long serialVersionUID = -1047130303774678066L;

	@SuppressWarnings("unused")
	private static final boolean DEBUG = false;

	public static HashMap<String, Double> gitterAbstaende;
	public static String STANDARDGITTER = "564,0 pm (NaCl)";
	public static HashMap<String, Double> energieUmrechnungen;
	public static final int STANDARDANODE = 3;

	private ActionMap actionMap;

	@Override
	public boolean hatBereiche() {
		return false;
	}

	@Override
	public boolean hatNullpunkt() {
		return false;
	}

	/** Creates new form StromSpannungBoxPanel */
	public XRayPanel(Groesse g, IAdapter a, IMesskanal messkanal) {
		super(g, a, messkanal);
		initXRayValues();
		setzeMessgroessenListe();
	}

	private void initXRayValues() {
		if (g != null) {
			messMaximum.setValue(g.getBis());
			Messparameter mp = GHMessen.getApplication().getMainView().mp;
			if (mp != null)
				g.setZeitraum(mp.getDT());
			torzeit.setValue(g.getZeitraum());
		}
		gitterabstand.setModel(
				new javax.swing.DefaultComboBoxModel<String>(gitterAbstaende.keySet().toArray(new String[] {})));
		gitterabstand.setSelectedIndex(3);
		anode.setModel(new DefaultComboBoxModel<String>(
				new String[] { "für Fe-Anode", "für Cu-Anode", "für W-Anode", "aus", "für Ag-Anode", "für Mo-Anode" }));
		anode.setSelectedIndex(5);
	}

	@Override
	protected void initComponents() {
		actionMap = org.jdesktop.application.Application.getInstance().getContext().getActionMap(XRayPanel.class, this);

		jLabel2 = new javax.swing.JLabel();
		messgroesse = new javax.swing.JComboBox<String>();

		jLabel2.setText("Messgröße:");
		jLabel2.setName("jLabel2"); // NOI18N

		messgroesse.setName("messgroesse"); // NOI18N
		messgroesse.addActionListener(actionMap.get("messgroesse_setzen"));

		JPanel counterPanel = new JPanel();
		counterPanel
				.setBorder(new TitledBorder(null, "Z\u00E4hler", TitledBorder.LEADING, TitledBorder.TOP, null, null));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup()
				.addContainerGap()
				.addGroup(layout.createParallelGroup(Alignment.LEADING).addGroup(layout.createSequentialGroup()
						.addComponent(counterPanel, GroupLayout.DEFAULT_SIZE, 426, Short.MAX_VALUE).addContainerGap())
						.addGroup(layout.createSequentialGroup()
								.addComponent(jLabel2, GroupLayout.PREFERRED_SIZE, 81, GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(ComponentPlacement.RELATED)
								.addComponent(messgroesse, 0, 206, Short.MAX_VALUE)
								.addContainerGap(139, Short.MAX_VALUE)))));
		layout.setVerticalGroup(layout.createParallelGroup(Alignment.LEADING)
				.addGroup(layout.createSequentialGroup().addContainerGap()
						.addGroup(layout.createParallelGroup(Alignment.BASELINE)
								.addComponent(messgroesse, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(jLabel2))
						.addGap(45).addComponent(counterPanel, GroupLayout.DEFAULT_SIZE, 198, Short.MAX_VALUE)
						.addContainerGap()));

		JLabel lblMessMaximum = new JLabel("Messmaximum:");

		messMaximum = new JFormattedTextField();
		messMaximum.getDocument().putProperty("feld", messMaximum);
		messMaximum.getDocument().putProperty("minimum", new Double(1.0));
		messMaximum.getDocument().putProperty("maximum", new Double(9999.0));
		messMaximum.addFocusListener(new DoubleFieldFocusAdapter() {
			public void schreiben(double d) {
				wertSchreiben(0, d);
			}
		});

		JLabel lblTorzeit = new JLabel("Torzeit:");

		torzeit = new JFormattedTextField();
		torzeit.getDocument().putProperty("feld", torzeit);
		torzeit.getDocument().putProperty("minimum", new Double(0.1));
		torzeit.getDocument().putProperty("maximum", new Double(9999.0));
		torzeit.addFocusListener(new DoubleFieldFocusAdapter() {
			public void schreiben(double d) {
				wertSchreiben(1, d);
			}
		});

		JLabel lblS = new JLabel("s");

		JLabel lblGitterabstandd = new JLabel("Gitterabstand 2d:");

		gitterabstand = new JComboBox<String>();
		gitterabstand.setAction(actionMap.get("setzeGitterabstand"));

		JLabel lblEnergie = new JLabel("Anode:");

		anode = new JComboBox<String>();
		anode.setAction(actionMap.get("setzeAnode"));

		alleOrdnungen = new JCheckBox("Alle Ordnungen");

		GroupLayout gl_counterPanel = new GroupLayout(counterPanel);
		gl_counterPanel.setHorizontalGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_counterPanel.createSequentialGroup().addContainerGap().addGroup(gl_counterPanel
						.createParallelGroup(Alignment.LEADING)
						.addGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING, false)
								.addGroup(gl_counterPanel.createSequentialGroup()
										.addGroup(gl_counterPanel.createParallelGroup(Alignment.TRAILING, false)
												.addComponent(lblTorzeit, Alignment.LEADING, GroupLayout.DEFAULT_SIZE,
														GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
												.addComponent(lblMessMaximum, Alignment.LEADING,
														GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE))
										.addPreferredGap(ComponentPlacement.UNRELATED)
										.addGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING, false)
												.addComponent(messMaximum, GroupLayout.PREFERRED_SIZE, 153,
														GroupLayout.PREFERRED_SIZE)
												.addGroup(gl_counterPanel.createSequentialGroup()
														.addComponent(torzeit, GroupLayout.PREFERRED_SIZE, 107,
																GroupLayout.PREFERRED_SIZE)
														.addPreferredGap(ComponentPlacement.RELATED).addComponent(lblS,
																GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))))
								.addGroup(gl_counterPanel.createSequentialGroup()
										.addGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING)
												.addComponent(lblGitterabstandd).addComponent(lblEnergie))
										.addPreferredGap(ComponentPlacement.UNRELATED)
										.addGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING)
												.addComponent(anode, 0, 154, Short.MAX_VALUE).addComponent(
														gitterabstand, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
						.addComponent(alleOrdnungen)).addContainerGap(124, Short.MAX_VALUE)));
		gl_counterPanel.setVerticalGroup(gl_counterPanel.createParallelGroup(Alignment.LEADING).addGroup(gl_counterPanel
				.createSequentialGroup()
				.addGroup(gl_counterPanel
						.createParallelGroup(Alignment.BASELINE).addComponent(lblMessMaximum).addComponent(messMaximum,
								GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
				.addGap(18)
				.addGroup(gl_counterPanel.createParallelGroup(Alignment.BASELINE).addComponent(lblTorzeit)
						.addComponent(torzeit, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addComponent(lblS))
				.addGap(18)
				.addGroup(gl_counterPanel.createParallelGroup(Alignment.BASELINE).addComponent(lblGitterabstandd)
						.addComponent(gitterabstand, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE))
				.addGap(18)
				.addGroup(gl_counterPanel.createParallelGroup(Alignment.BASELINE).addComponent(lblEnergie).addComponent(
						anode, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
				.addPreferredGap(ComponentPlacement.UNRELATED).addComponent(alleOrdnungen)
				.addContainerGap(11, Short.MAX_VALUE)));
		gitterabstand.setName("gitterabstand"); // NOI18N
		anode.setName("anode"); // NOI18N

		alleOrdnungen.setText("Alle Ordnungen zeigen");
		alleOrdnungen.setName("alleOrdnungen"); // NOI18N

		counterPanel.setLayout(gl_counterPanel);
		this.setLayout(layout);
	}

	protected javax.swing.JLabel jLabel2;
	protected javax.swing.JPanel jPanel2;
	protected javax.swing.JComboBox<String> messgroesse;
	protected javax.swing.JFormattedTextField torzeit;
	protected javax.swing.JFormattedTextField messMaximum;
	protected javax.swing.JComboBox<String> gitterabstand;
	protected javax.swing.JComboBox<String> anode;
	protected javax.swing.JCheckBox alleOrdnungen;

	@Action
	public void messgroesse_setzen() {
		int index = messgroesse.getSelectedIndex();
		((IAuswahl) messkanal.getEingang()).setAuswahl(index);
	}

	protected void setzeMessgroessenListe() {
		messgroesse.setModel(new DefaultComboBoxModel<String>(
				((EingangXRay) messkanal.getEingang()).getGroessenStringList(g.getEF())));
	}

	@Action
	public void setzeGitterabstand() {
		if (g != null) {
			String s = (String) gitterabstand.getSelectedItem();
			g.setD_pm(gitterAbstaende.get(s));
		}
	}

	@Action
	public void setzeAnode() {
		if (g != null) {
			String s = (String) anode.getSelectedItem();
			g.setE_keV(energieUmrechnungen.get(s));
		}
	}

	private void wertSchreiben(int nummer, double d) {
		switch (nummer) {
		case 0:
			g.setBis(d);
			break;
		case 1:
			Messparameter mp = GHMessen.getApplication().getMainView().mp;
			g.setZeitraum(d);
			if (mp != null)
				mp.setDT(d);
			break;
		}
	}

	static {
		gitterAbstaende = new HashMap<String, Double>() {
			private static final long serialVersionUID = 1L;

			{
				put("aus", 1.0);
				put(STANDARDGITTER, 564.0);
				put("402,7 pm (LiF)", 402.7);
				put("659,7 pm (KBr)", 659.7);
			}
		};

		energieUmrechnungen = new HashMap<String, Double>() {
			private static final long serialVersionUID = 1L;

			{
				put("aus", 1.0);
				put("für Fe-Anode", 0.0329896907216);
				put("für Cu-Anode", 0.052140077821);
				put("für Mo-Anode", 0.244725738397);
				put("für Ag-Anode", 0.393939393939);
				put("für W-Anode", 0.0566891891892);
			}
		};

	}
}
