/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import ghmessen.util.HTML;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;

public abstract class Anzeige extends JLabel implements ComponentListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 6223563536456813959L;
	private static final boolean DEBUG = false;
	protected double wert;
	
	public Anzeige() {
		this(0.0, Color.RED);
	}

	public Anzeige(double wert){
		this(wert, Color.RED);
	}
	
	public Anzeige(double wert, Color farbe) {
		super(String.format("%f",wert));
		if(DEBUG)
			System.out.println("[Anzeige] - Konstruktor.");
		this.wert = wert;
		if(farbe != null)
			setForeground(farbe);
		setOpaque(true);
		setHorizontalAlignment(SwingConstants.LEADING);
		setBorder(new BevelBorder(BevelBorder.LOWERED, null, null,
				null, null));
		setBackground(Color.WHITE);
		addComponentListener(this);
	}

	private int calcMaxPtSize(java.awt.Component com) {
		Dimension size = com.getSize();
		Font font = com.getFont();
		FontMetrics fm = com.getGraphics().getFontMetrics(font);
		int h = fm.getHeight();
		int w = fm.stringWidth(getText());
		double yf = size.getHeight() / h;
		double xf = size.getWidth() / w;
		double f = Math.min(yf, xf);
		int scale = (HTML.enthaeltHTML(getText())) ? 3 : 1;
		int pt = (int) Math.round(Math.floor(0.8 * f * font.getSize()));
		return scale*pt;
	}

	// Eigene Methoden

	public void setWert(double wert) {
		this.wert = wert;
		setText(String.format("%f",wert));
	}

	public double getWert() {
		return wert;
	}

	@Override
	public void componentResized(ComponentEvent e) {
		setFont(new Font(Font.DIALOG, Font.PLAIN,
				calcMaxPtSize(this)));
	}

	@Override
	public void componentMoved(ComponentEvent e) {
		return;
		
	}

	@Override
	public void componentShown(ComponentEvent e) {
		return;
		
	}

	@Override
	public void componentHidden(ComponentEvent e) {
		return;
		
	}
}
