/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import static ghmessen.groesse.Aufnahme.momentan;
import ghmessen.groesse.Aufnahme;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

public class DragZeigeranzeige extends Zeigeranzeige implements MouseListener,
		MouseMotionListener {
	private static final long serialVersionUID = 836510869375053927L;
	private static final boolean DEBUG = false;

	public DragZeigeranzeige() {
		this(0.0, Nullpunkt.mittig, -10, 10, 5, momentan, Color.MAGENTA);
	}

	public DragZeigeranzeige(double wert, Nullpunkt nullpunkt,
			double minimum, double maximum, double unterteilung, Color farbe) {
		this(wert, nullpunkt, minimum, maximum, unterteilung, momentan, farbe);
	}

	public DragZeigeranzeige(double wert, Nullpunkt nullpunkt,
			double minimum, double maximum, double unterteilung, Aufnahme aufnahme, Color farbe) {
		super(wert, nullpunkt, minimum, maximum, unterteilung, aufnahme, farbe);
		if (DEBUG)
			System.out.println("[DragZeigeranzeige] - Konstruktor.");
		addMouseListener(this);
		zeiger.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		zeiger.addMouseMotionListener(this);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		int x = e.getX();
		x = x - rand;
		x = Math.max(x, 0);
		x = Math.min(x, width);
		double alt = wert;
		setWert(skala.inversTransform(x));
		firePropertyChange("wert", alt, wert);
	}

	@Override
	public void mousePressed(MouseEvent e) {
		return;
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		return;
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		return;
	}

	@Override
	public void mouseExited(MouseEvent e) {
		return;
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		Component c = e.getComponent();
		int x = c.getX() + e.getX();
		c.setLocation(x, c.getY());
		x = x - rand;
		x = Math.max(x, 0);
		x = Math.min(x, width);
		double alt = wert;
		setWert(skala.inversTransform(x));
		firePropertyChange("wert", alt, wert);
		repaint();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		return;
	}
}
