/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import ghmessen.util.Util;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.GroupLayout;
import javax.swing.JFormattedTextField;

public class EingabeWerteanzeige extends Werteanzeige implements MouseListener, ActionListener {

	private static final long serialVersionUID = 4607391183094132917L;
	private static final boolean DEBUG = false;
	private WerteEingabe eingabe;

	public EingabeWerteanzeige(){
		this("m", "%2.2f", "U_0", 0.0);
	}
	public EingabeWerteanzeige(String einheit, String format, String groesse,
			double wert) {
		super(einheit, format, groesse, wert);
		if (DEBUG)
			System.out.println("[EingabeWerteanzeige] - Konstruktor: einheit("
					+ einheit + ") format(" + format + ") groesse(" + groesse
					+ ") wert(" + wert + ")");
		setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		addMouseListener(this);
	}

	public void actionPerformed(ActionEvent e) {
		double alterWert = getWert();
		setWert((Double) eingabe.getValue());
		firePropertyChange("wert", alterWert, wert);
		if(e.getSource() instanceof Component){
			Component c = (Component)e.getSource();
			if(c.getParent().getLayout() instanceof GroupLayout){
				GroupLayout gl = (GroupLayout)c.getParent().getLayout();
				gl.replace(eingabe, this);
			}
		}
	}

	private class WerteEingabe extends JFormattedTextField {
		private static final long serialVersionUID = 1L;
		public WerteEingabe(EingabeWerteanzeige anzeige) {
			super(Util.doubleFormat);
			setValue(wert);
			addActionListener(anzeige);
		}
		
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON1
				&& e.getClickCount() == 2) {
			if(eingabe == null) 
				eingabe = new WerteEingabe(this);
			eingabe.setFont(getFont());
			if(this instanceof Component){
				Component c = (Component)this;
				if(c.getParent().getLayout() instanceof GroupLayout){
					GroupLayout gl = (GroupLayout)c.getParent().getLayout();
					gl.replace(this, eingabe);
				}
			}
			eingabe.requestFocusInWindow();
		}
	}

	@Override
	public void mousePressed(MouseEvent e) { }

	@Override
	public void mouseReleased(MouseEvent e) { }

	@Override
	public void mouseEntered(MouseEvent e) { }

	@Override
	public void mouseExited(MouseEvent e) { }

}
