/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import ghmessen.groesse.Groesse;
import ghmessen.util.Util;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;

import javax.swing.ActionMap;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;

import org.jdesktop.application.Action;

import java.awt.Dimension;

public class KorrekturDlg extends JDialog {
	/**
	 * 
	 */
	private static final long serialVersionUID = 5558529485579762521L;
	private static final boolean DEBUG = false;
	
	private class WertPropertyHandler implements Serializable, PropertyChangeListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			handleWertEvent(evt);
		}
	}
	
	private JLabel faktor1;
	private JFormattedTextField faktor2;
	private JLabel wert1;
	private JLabel wert2;
	private JFormattedTextField verschiebung1;
	private JLabel verschiebung2;
	private JFormattedTextField sollwert1;
	private JFormattedTextField sollwert2;
	private JButton offset;
	private JButton faktor;
	private JButton aus;
	private JButton schliessen;

	private Groesse g;
	private String einheit;

	private WertPropertyHandler wertPH;
	
	public KorrekturDlg(Frame parent, Groesse g, String einheit) {
		super(parent);
		setMinimumSize(new Dimension(600, 250));
		setPreferredSize(new Dimension(600, 250));
		wertPH = new WertPropertyHandler();
		this.g = g;
		this.einheit = " "+einheit;
		initComponents();
		if (this.g != null) {
			wert1.setText(String.format("%1.3f", this.g.getWert())+einheit);
			wert2.setText(String.format("%1.3f", this.g.getWert())+einheit);
			faktor1.setText(Double.toString(this.g.getFaktor()));
			faktor2.setValue(this.g.getFaktor());
			verschiebung1.setValue(this.g.getOffset());
			verschiebung2.setText(Double.toString(this.g.getOffset())+einheit);
		}
		this.g.addPropertyChangeListener("wert", wertPH);
	}

	public void initComponents() {
		ActionMap actionMap = org.jdesktop.application.Application
				.getInstance(ghmessen.GHMessen.class).getContext()
				.getActionMap(ghmessen.instrumente.komponenten.KorrekturDlg.class, this);
		setTitle("Messwerte korrigieren");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setAlwaysOnTop(true);
		setModal(true);
		setModalityType(ModalityType.APPLICATION_MODAL);
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[] { 112, 25, 26, 85, 0, 0, 0, 24,
				81, 98, 0 };
		gridBagLayout.rowHeights = new int[] { 35, 48, 50, 51, 0 };
		gridBagLayout.columnWeights = new double[] { 0.0, 0.0, 0.0, 0.0, 0.0,
				0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE };
		gridBagLayout.rowWeights = new double[] { 0.0, 0.0, 0.0, 0.0,
				Double.MIN_VALUE };
		getContentPane().setLayout(gridBagLayout);

		JLabel lblFaktor = new JLabel("Faktor");
		GridBagConstraints gbc_lblFaktor = new GridBagConstraints();
		gbc_lblFaktor.insets = new Insets(0, 0, 5, 5);
		gbc_lblFaktor.gridx = 0;
		gbc_lblFaktor.gridy = 0;
		getContentPane().add(lblFaktor, gbc_lblFaktor);

		JLabel label = new JLabel("*");
		GridBagConstraints gbc_label = new GridBagConstraints();
		gbc_label.insets = new Insets(0, 0, 5, 5);
		gbc_label.gridx = 1;
		gbc_label.gridy = 0;
		getContentPane().add(label, gbc_label);

		JLabel label_1 = new JLabel("(");
		GridBagConstraints gbc_label_1 = new GridBagConstraints();
		gbc_label_1.insets = new Insets(0, 0, 5, 5);
		gbc_label_1.gridx = 2;
		gbc_label_1.gridy = 0;
		getContentPane().add(label_1, gbc_label_1);

		JLabel lblIstwert = new JLabel("Istwert");
		GridBagConstraints gbc_lblIstwert = new GridBagConstraints();
		gbc_lblIstwert.insets = new Insets(0, 0, 5, 5);
		gbc_lblIstwert.gridx = 3;
		gbc_lblIstwert.gridy = 0;
		getContentPane().add(lblIstwert, gbc_lblIstwert);

		JLabel label_2 = new JLabel("+");
		GridBagConstraints gbc_label_2 = new GridBagConstraints();
		gbc_label_2.insets = new Insets(0, 0, 5, 5);
		gbc_label_2.gridx = 4;
		gbc_label_2.gridy = 0;
		getContentPane().add(label_2, gbc_label_2);

		JLabel lblVerschiebung = new JLabel("Verschiebung");
		GridBagConstraints gbc_lblVerschiebung = new GridBagConstraints();
		gbc_lblVerschiebung.insets = new Insets(0, 0, 5, 5);
		gbc_lblVerschiebung.gridx = 5;
		gbc_lblVerschiebung.gridy = 0;
		getContentPane().add(lblVerschiebung, gbc_lblVerschiebung);

		JLabel label_3 = new JLabel(")");
		GridBagConstraints gbc_label_3 = new GridBagConstraints();
		gbc_label_3.insets = new Insets(0, 0, 5, 5);
		gbc_label_3.gridx = 6;
		gbc_label_3.gridy = 0;
		getContentPane().add(label_3, gbc_label_3);

		JLabel label_4 = new JLabel("=");
		GridBagConstraints gbc_label_4 = new GridBagConstraints();
		gbc_label_4.insets = new Insets(0, 0, 5, 5);
		gbc_label_4.gridx = 7;
		gbc_label_4.gridy = 0;
		getContentPane().add(label_4, gbc_label_4);

		JLabel lblSollwert = new JLabel("Sollwert");
		GridBagConstraints gbc_lblSollwert = new GridBagConstraints();
		gbc_lblSollwert.insets = new Insets(0, 0, 5, 5);
		gbc_lblSollwert.gridx = 8;
		gbc_lblSollwert.gridy = 0;
		getContentPane().add(lblSollwert, gbc_lblSollwert);

		faktor1 = new JLabel("1");
		GridBagConstraints gbc_faktor1 = new GridBagConstraints();
		gbc_faktor1.insets = new Insets(0, 0, 5, 5);
		gbc_faktor1.gridx = 0;
		gbc_faktor1.gridy = 1;
		getContentPane().add(faktor1, gbc_faktor1);

		JLabel label_8 = new JLabel("*");
		GridBagConstraints gbc_label_8 = new GridBagConstraints();
		gbc_label_8.insets = new Insets(0, 0, 5, 5);
		gbc_label_8.gridx = 1;
		gbc_label_8.gridy = 1;
		getContentPane().add(label_8, gbc_label_8);

		JLabel label_10 = new JLabel("(");
		GridBagConstraints gbc_label_10 = new GridBagConstraints();
		gbc_label_10.insets = new Insets(0, 0, 5, 5);
		gbc_label_10.gridx = 2;
		gbc_label_10.gridy = 1;
		getContentPane().add(label_10, gbc_label_10);

		wert1 = new JLabel("new label");
		GridBagConstraints gbc_wert1 = new GridBagConstraints();
		gbc_wert1.insets = new Insets(0, 0, 5, 5);
		gbc_wert1.gridx = 3;
		gbc_wert1.gridy = 1;
		getContentPane().add(wert1, gbc_wert1);

		JLabel label_12 = new JLabel("+");
		GridBagConstraints gbc_label_12 = new GridBagConstraints();
		gbc_label_12.insets = new Insets(0, 0, 5, 5);
		gbc_label_12.gridx = 4;
		gbc_label_12.gridy = 1;
		getContentPane().add(label_12, gbc_label_12);

		verschiebung1 = new JFormattedTextField(Util.doubleFormat);
		verschiebung1.setColumns(7);
		GridBagConstraints gbc_verschiebung = new GridBagConstraints();
		gbc_verschiebung.fill = GridBagConstraints.HORIZONTAL;
		gbc_verschiebung.insets = new Insets(0, 0, 5, 5);
		gbc_verschiebung.gridx = 5;
		gbc_verschiebung.gridy = 1;
		getContentPane().add(verschiebung1, gbc_verschiebung);

		JLabel label_14 = new JLabel(")");
		GridBagConstraints gbc_label_14 = new GridBagConstraints();
		gbc_label_14.insets = new Insets(0, 0, 5, 5);
		gbc_label_14.gridx = 6;
		gbc_label_14.gridy = 1;
		getContentPane().add(label_14, gbc_label_14);

		JLabel label_6 = new JLabel("=");
		GridBagConstraints gbc_label_6 = new GridBagConstraints();
		gbc_label_6.insets = new Insets(0, 0, 5, 5);
		gbc_label_6.gridx = 7;
		gbc_label_6.gridy = 1;
		getContentPane().add(label_6, gbc_label_6);

		sollwert1 = new JFormattedTextField(Util.doubleFormat);
		sollwert1.setColumns(7);
		GridBagConstraints gbc_sollwert1 = new GridBagConstraints();
		gbc_sollwert1.fill = GridBagConstraints.HORIZONTAL;
		gbc_sollwert1.insets = new Insets(0, 0, 5, 5);
		gbc_sollwert1.gridx = 8;
		gbc_sollwert1.gridy = 1;
		getContentPane().add(sollwert1, gbc_sollwert1);

		offset = new JButton("Offset");
		offset.setAction(actionMap.get("offset"));
		GridBagConstraints gbc_offset = new GridBagConstraints();
		gbc_offset.insets = new Insets(0, 0, 5, 0);
		gbc_offset.gridx = 9;
		gbc_offset.gridy = 1;
		getContentPane().add(offset, gbc_offset);

		faktor2 = new JFormattedTextField(Util.doubleFormat);
		faktor2.setColumns(7);
		GridBagConstraints gbc_faktor2 = new GridBagConstraints();
		gbc_faktor2.fill = GridBagConstraints.HORIZONTAL;
		gbc_faktor2.insets = new Insets(0, 0, 5, 5);
		gbc_faktor2.gridx = 0;
		gbc_faktor2.gridy = 2;
		getContentPane().add(faktor2, gbc_faktor2);

		JLabel label_9 = new JLabel("*");
		GridBagConstraints gbc_label_9 = new GridBagConstraints();
		gbc_label_9.insets = new Insets(0, 0, 5, 5);
		gbc_label_9.gridx = 1;
		gbc_label_9.gridy = 2;
		getContentPane().add(label_9, gbc_label_9);

		JLabel label_11 = new JLabel("(");
		GridBagConstraints gbc_label_11 = new GridBagConstraints();
		gbc_label_11.insets = new Insets(0, 0, 5, 5);
		gbc_label_11.gridx = 2;
		gbc_label_11.gridy = 2;
		getContentPane().add(label_11, gbc_label_11);

		wert2 = new JLabel("New label");
		GridBagConstraints gbc_wert2 = new GridBagConstraints();
		gbc_wert2.insets = new Insets(0, 0, 5, 5);
		gbc_wert2.gridx = 3;
		gbc_wert2.gridy = 2;
		getContentPane().add(wert2, gbc_wert2);

		JLabel label_13 = new JLabel("+");
		GridBagConstraints gbc_label_13 = new GridBagConstraints();
		gbc_label_13.insets = new Insets(0, 0, 5, 5);
		gbc_label_13.gridx = 4;
		gbc_label_13.gridy = 2;
		getContentPane().add(label_13, gbc_label_13);

		verschiebung2 = new JLabel("New label");
		GridBagConstraints gbc_verschiebung2 = new GridBagConstraints();
		gbc_verschiebung2.insets = new Insets(0, 0, 5, 5);
		gbc_verschiebung2.gridx = 5;
		gbc_verschiebung2.gridy = 2;
		getContentPane().add(verschiebung2, gbc_verschiebung2);

		JLabel label_15 = new JLabel(")");
		GridBagConstraints gbc_label_15 = new GridBagConstraints();
		gbc_label_15.insets = new Insets(0, 0, 5, 5);
		gbc_label_15.gridx = 6;
		gbc_label_15.gridy = 2;
		getContentPane().add(label_15, gbc_label_15);

		JLabel label_7 = new JLabel("=");
		GridBagConstraints gbc_label_7 = new GridBagConstraints();
		gbc_label_7.insets = new Insets(0, 0, 5, 5);
		gbc_label_7.gridx = 7;
		gbc_label_7.gridy = 2;
		getContentPane().add(label_7, gbc_label_7);

		sollwert2 = new JFormattedTextField(Util.doubleFormat);
		sollwert2.setColumns(7);
		GridBagConstraints gbc_sollwert2 = new GridBagConstraints();
		gbc_sollwert2.fill = GridBagConstraints.HORIZONTAL;
		gbc_sollwert2.insets = new Insets(0, 0, 5, 5);
		gbc_sollwert2.gridx = 8;
		gbc_sollwert2.gridy = 2;
		getContentPane().add(sollwert2, gbc_sollwert2);

		faktor = new JButton("Faktor");
		faktor.setAction(actionMap.get("faktor"));
		GridBagConstraints gbc_faktor = new GridBagConstraints();
		gbc_faktor.insets = new Insets(0, 0, 5, 0);
		gbc_faktor.gridx = 9;
		gbc_faktor.gridy = 2;
		getContentPane().add(faktor, gbc_faktor);

		schliessen = new JButton("Schließen");
		schliessen.setAction(actionMap.get("schliessen"));
		GridBagConstraints gbc_schliessen = new GridBagConstraints();
		gbc_schliessen.gridwidth = 3;
		gbc_schliessen.insets = new Insets(0, 0, 0, 5);
		gbc_schliessen.gridx = 0;
		gbc_schliessen.gridy = 3;
		getContentPane().add(schliessen, gbc_schliessen);

		aus = new JButton("Aus");
		aus.setAction(actionMap.get("aus"));
		GridBagConstraints gbc_aus = new GridBagConstraints();
		gbc_aus.gridx = 9;
		gbc_aus.gridy = 3;
		getContentPane().add(aus, gbc_aus);
	}

	@Action
	public void offset() {
		Double v = (Double) verschiebung1.getValue();
		Double s = (Double) sollwert1.getValue();
		try {
			if (v != null && v.doubleValue() != 0) {
				g.setOffset(v);
				s = g.getFaktor() * (g.getWert() + g.getOffset());
			} else if (s != null) {
				g.setOffset(s / g.getFaktor() - g.getWert());
				v = g.getOffset();
			}
			verschiebung1.setValue(v);
			verschiebung2.setText(Double.toString(v)+einheit);
			sollwert1.setValue(s);
			sollwert2.setValue(s);
		} catch (NumberFormatException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	@Action
	public void faktor() {
		Double f = (Double) faktor2.getValue();
		Double s = (Double) sollwert2.getValue();
		try {
			if (f != null && f.doubleValue() != 1 && f.doubleValue() != 0) {
				g.setFaktor(f);
				s = g.getFaktor() * (g.getWert() + g.getOffset());
			} else if (s != null && s.doubleValue() != 0) {
				g.setOffset(s / g.getFaktor() - g.getWert());
				f = g.getOffset();
			}
			faktor2.setValue(f);
			faktor1.setText(Double.toString(f)+einheit);
			sollwert1.setValue(s);
			sollwert2.setValue(s);
		} catch (NumberFormatException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

	@Action
	public void aus() {
		g.setOffset(0);
		g.setFaktor(1);
		faktor1.setText(Double.toString(this.g.getFaktor())+einheit);
		faktor2.setValue(this.g.getFaktor());
		verschiebung1.setValue(this.g.getOffset());
		verschiebung2.setText(Double.toString(this.g.getOffset())+einheit);
		sollwert1.setValue(null);
		sollwert2.setValue(null);
	}

	@Action
	public void schliessen() {
		this.dispose();
	}

	protected void handleWertEvent(PropertyChangeEvent evt) {
		if(evt.getSource() instanceof Groesse){
			wert1.setText(String.format("%1.3f ", this.g.getWert())+einheit);
			wert2.setText(String.format("%1.3f ", this.g.getWert())+einheit);			
		}
	}
}
