/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import static ghmessen.Konstanten.FARBE_MESSGROESSE;
import ghmessen.util.HTML;

import java.awt.Color;

import javax.swing.SwingConstants;


public class Logikanzeige extends Anzeige {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5110810252824611634L;
	protected Color nullFarbe;
	protected Color einsFarbe;
	protected Color schriftFarbe;
	protected String groesse;

	public Logikanzeige() {
		this("R_0", false, Color.BLACK, Color.RED, Color.WHITE);
	}

	public Logikanzeige(int wert) {
		this("R_0", wert == 0 ? false : true, Color.BLACK, Color.RED,
				Color.WHITE);
	}

	public Logikanzeige(String groesse, double wert) {
		this(groesse, wert == 0, Color.BLACK, Color.RED, Color.WHITE);
	}

	public Logikanzeige(String groesse, boolean pegel, Color nullFarbe,
			Color einsFarbe, Color schriftFarbe) {
		super(pegel ? 1.0 : 0.0, FARBE_MESSGROESSE);
		this.groesse = groesse;
		this.nullFarbe = nullFarbe;
		this.einsFarbe = einsFarbe;
		this.schriftFarbe = schriftFarbe;
		setText();
	}

	protected void setPegel(boolean pegel) {
		setWert(pegel ? 1 : 0);
	}

	protected boolean getPegel() {
		return getWert() != 0;
	}

	@Override
	public void setWert(double wert) {
		this.wert = wert != 0 ? 1 : 0;
		setText();
	}

	protected void setNullFarbe(Color farbe) {
		nullFarbe = farbe;
		setText();
	}

	protected Color getNullFarbe() {
		return nullFarbe;
	}

	protected void setEinsFarbe(Color farbe) {
		einsFarbe = farbe;
		setText();
	}

	protected Color getEinsFarbe() {
		return einsFarbe;
	}

	protected void setSchriftFarbe(Color farbe) {
		schriftFarbe = farbe;
		setText();
	}

	protected Color getSchriftFarbe() {
		return schriftFarbe;
	}

	protected void setFarben(Color nullFarbe, Color einsFarbe,
			Color schriftFarbe) {
		this.nullFarbe = nullFarbe;
		this.einsFarbe = einsFarbe;
		this.schriftFarbe = schriftFarbe;
		setText();
	}

	protected void setPegelFarben(Color nullFarbe, Color einsFarbe) {
		this.nullFarbe = nullFarbe;
		this.einsFarbe = einsFarbe;
		setText();
	}

	protected void setText() {
		setHorizontalAlignment(SwingConstants.CENTER);
		setForeground(schriftFarbe);
		if (wert == 0) {
			setBackground(nullFarbe);
		} else {
			setBackground(einsFarbe);
		}
		setText("<html>" + HTML.symbolZuHTML(groesse) + " = "
					+ (wert == 0.0 ? "0" : "1") + "</html>");
	}

}
