/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import static ghmessen.groesse.Aufnahme.momentan;
import ghmessen.components.Intervall;
import ghmessen.groesse.Aufnahme;
import ghmessen.groesse.Messbereich;

import java.awt.Color;

public class NullpunktZeigeranzeige extends Zeigeranzeige {
	/**
	 * 
	 */
	private static final long serialVersionUID = 836510869375053927L;
	private static final boolean DEBUG = false;

	protected Nullpunkt nullpunkt;
	protected double gesamtMinimum, gesamtMaximum;

	public NullpunktZeigeranzeige() {
		this(0, Nullpunkt.mittig, -10, 10, 5, momentan, Color.RED);
	}

	public NullpunktZeigeranzeige(double wert, Nullpunkt nullpunkt,
			Messbereich messbereich, Aufnahme aufnahme, Color farbe) {
		this(wert, nullpunkt, messbereich.getMinimum(), messbereich
				.getMaximum(), messbereich.getUnterteilung(), aufnahme, farbe);
	}

	public NullpunktZeigeranzeige(double wert, Nullpunkt nullpunkt,
			double minimum, double maximum, double unterteilung, Color farbe) {
		this(wert, nullpunkt, minimum, maximum, unterteilung, momentan, farbe);
	}

	public NullpunktZeigeranzeige(double wert, Nullpunkt nullpunkt,
			double minimum, double maximum, double unterteilung,
			Aufnahme aufnahme, Color farbe) {
		super(wert, nullpunkt, minimum, maximum, unterteilung, aufnahme, farbe);
		if (DEBUG)
			System.out
					.println("[NullpunktZeigeranzeige] - Konstruktor: minimum("
							+ minimum + ") maximum(" + maximum + ")");
		gesamtMinimum = minimum;
		gesamtMaximum = maximum;
		this.nullpunkt = nullpunkt;
		if (gesamtMinimum != -gesamtMaximum)
			System.err
					.println("[NullpunktZeigeranzeige] - Konstruktur: Der Messbereich("
							+ gesamtMinimum
							+ "|"
							+ gesamtMaximum
							+ ") ist nicht symmetrisch.");
	}

	@Override
	public void setNullpunkt(Nullpunkt nullpunkt) {
		this.nullpunkt = nullpunkt;
		Intervall intervall = null;
		switch (nullpunkt) {
		case links:
			intervall = new Intervall(0.0, gesamtMaximum);
			break;
		case rechts:
			intervall = new Intervall(gesamtMinimum, 0.0);
			break;
		case mittig:
			intervall = new Intervall(gesamtMinimum, gesamtMaximum);
			break;
		}
		super.setIntervall(intervall, skala != null ? skala.getUnterteilung() : -1.0);
	}

	@Override
	public Nullpunkt getNullpunkt() {
		return nullpunkt;
	}

	@Override
	public void setIntervall(Intervall intervall, double unterteilung) {
		gesamtMinimum = minimum;
		gesamtMaximum = maximum;
		setNullpunkt(nullpunkt);
	}

	@Override
	public void setMinMax(double minimum, double maximum, double unterteilung) {
		gesamtMinimum = minimum;
		gesamtMaximum = maximum;
		setNullpunkt(nullpunkt);
	}

	@Override
	public void setMinimum(double minimum) {
		gesamtMinimum = minimum;
		setNullpunkt(nullpunkt);
	}

	@Override
	public void setMaximum(double maximum) {
		gesamtMaximum = maximum;
		setNullpunkt(nullpunkt);
	}

	@Override
	public void setMessbereich(Messbereich m) {
		gesamtMinimum = m.getMinimum();
		gesamtMaximum = m.getMaximum();
		super.setMessbereich(m);
		setNullpunkt(nullpunkt);
	}

}
