/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import ghmessen.util.HTML;

public class Werteanzeige extends Anzeige {
	/**
	 * 
	 */
	private static final long serialVersionUID = 6223563536456813959L;
	private static final boolean DEBUG = false;
	protected String einheit;
	protected String format;
	protected String groesse;

	public Werteanzeige() {
		this("V", "%2.2f", "U_A1", 0);
	}

	public Werteanzeige(String einheit, String format, String groesse, double wert) {
		super(wert);
		if (DEBUG)
			System.out.println("[Werteanzeige] - Konstruktor: einheit(" + einheit + ") format(" + format + ") groesse("
					+ groesse + ") wert(" + wert + ")");
		this.groesse = groesse;
		this.format = format;
		this.einheit = einheit;
		this.wert = wert;
		setText();
	}

	public void setGroesse(String groesse) {
		this.groesse = groesse;
		setText();
	}

	public String getGroesse() {
		return groesse;
	}

	public void setFormat(String format) {
		this.format = format;
		setText();
	}

	public String getFormat() {
		return format;
	}

	public void setEinheit(String einheit) {
		this.einheit = einheit;
		setText();
	}

	public String getEinheit() {
		return einheit;
	}

	public void setStellen(int stellen) {
		this.format = "%" + stellen + format.substring(format.indexOf('.'), format.length());
		setText();
	}

	public int getStellen() {
		return Integer.parseInt(format.substring(format.indexOf('%') + 1, format.indexOf('.')));
	}

	public void setDezimalStellen(int stellen) {
		this.format = "%" + format.substring(1, format.indexOf('.')) + "." + stellen + "f";
		setText();
	}

	public int getDezimalStellen() {
		return Integer.parseInt(format.substring(format.indexOf('.') + 1, format.indexOf('f')));
	}

	@Override
	public void setWert(double wert) {
		if (this.wert != wert) {
			this.wert = wert;
			setText();
		}
	}

	private void setText() {
		if (HTML.enthaeltHTML(groesse))
			setText(String.format(groesse + " = " + format + " " + einheit, wert));
		else
			setText(HTML
					.umgebeMitHTML(String.format(HTML.symbolZuHTML(groesse) + " = " + format + " " + einheit, wert)));
	}
}
