/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.instrumente.komponenten;

import static ghmessen.groesse.Aufnahme.momentan;
import ghmessen.components.Intervall;
import ghmessen.components.Skala;
import ghmessen.components.Skalentext;
import ghmessen.groesse.Aufnahme;
import ghmessen.groesse.Messbereich;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

public class Zeigeranzeige extends Anzeige {
	/**
	 * 
	 */
	private static final long serialVersionUID = 836510869375053927L;
	private static final boolean DEBUG = false;

	protected Skala skala;
	protected double minimum, maximum;

	Zeiger zeiger;
	ArtLabel art;
	int width = 0;
	int height = 0;
	int laengeKlein = 12;
	int laengeGross = 16;
	int fontHeight = 10; /* in pt */
	int rand = 10;

	public Zeigeranzeige() {
		this(0, null, 0, 10, 5, momentan, Color.RED);
	}

	public Zeigeranzeige(double wert, Messbereich m){
		this(wert,null,m.getMinimum(), m.getMaximum(), m.getUnterteilung(), momentan, Color.red);
	}
	
	public Zeigeranzeige(double wert, Messbereich m, Color farbe){
		this(wert,Nullpunkt.links, m, farbe);
	}
	
	public Zeigeranzeige(double wert, Nullpunkt nullpunkt, Messbereich m, Color farbe){
		this(wert, nullpunkt, m.getMinimum(), m.getMaximum(), m.getUnterteilung(), momentan, farbe);
	}
	
	public Zeigeranzeige(double wert, Nullpunkt nullpunkt, Messbereich m, Aufnahme aufnahme, Color farbe){
		this(wert, nullpunkt, m.getMinimum(), m.getMaximum(), m.getUnterteilung(), aufnahme, farbe);
	}
	
	public Zeigeranzeige(double wert, Nullpunkt nullpunkt, double minimum,
			double maximum, double unterteilung, Color farbe) {
		this(wert, nullpunkt, minimum, maximum, unterteilung, momentan, farbe);
	}

	public Zeigeranzeige(double wert, Nullpunkt nullpunkt, double minimum,
			double maximum, double unterteilung, Aufnahme aufnahme, Color farbe) {
		super(wert, farbe);
		if (DEBUG)
			System.out.println("[Zeigeranzeige] - Konstruktor.");
		this.skala = new Skala(minimum, maximum, unterteilung, false);
		this.minimum = minimum;
		this.maximum = maximum;
		this.zeiger = new Zeiger(farbe);
		add(zeiger);
		this.art = new ArtLabel(aufnahme);
		add(art);
		setBackground(Color.LIGHT_GRAY);
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				setSize();
				repaint();
			}
		});
	}

	public void setZeigerFarbe(Color zeigerFarbe) {
		if (zeiger != null)
			zeiger.setForeground(zeigerFarbe);
	}

	@Override
	public void componentResized(ComponentEvent e) {
		return;
	}

	public void setNullpunkt(Nullpunkt nullpunkt) {
		return;
	}

	public Nullpunkt getNullpunkt() {
		return Nullpunkt.links;
	}

	public String[] getZahlen() {
		return skala == null ? null : skala.getZahlen();
	}

	public void setIntervall(Intervall intervall, double unterteilung) {
		this.minimum = intervall.getMin();
		this.maximum = intervall.getMax();
		skala.setIntervall(intervall, unterteilung);
		updateZeiger();
		repaint();
	}

	public void setMinMax(double minimum, double maximum, double unterteilung) {
		this.minimum = minimum;
		this.maximum = maximum;
		skala.setMinMax(minimum, maximum, unterteilung);
		updateZeiger();
		repaint();
	}

	public void setMinimum(double minimum) {
		this.minimum = minimum;
		updateZeiger();
		repaint();
	}

	public double getMinimum() {
		return skala.getMinimum();
	}

	public void setMaximum(double maximum) {
		this.maximum = maximum;
		updateZeiger();
		repaint();
	}

	public double getMaximum() {
		return skala.getMaximum();
	}

	public void setMessbereich(Messbereich m) {
		this.minimum = m.getMinimum();
		this.maximum = m.getMaximum();
		skala.setMessbereich(m);
		updateZeiger();
		repaint();
	}

	public void setAufnahme(Aufnahme aufnahme) {
		if (art != null)
			art.setAufnahme(aufnahme);
	}

	@Override
	public void setWert(double wert) {
		if (DEBUG)
			System.out.println("[Zeigeranzeige] - setWert(" + wert + ")");
		this.wert = wert;
		updateZeiger();
	}

	protected void updateZeiger(){
		if (zeiger != null && skala != null) {
			zeiger.setLocation(rand + skala.transform(wert) - zeiger.getWidth()
					/ 2, zeiger.getY());
		}		
	}
	
	public void setSize() {
		fontHeight = calcPtSize(0.2, getGraphics());
		setFont(new Font(Font.DIALOG, Font.PLAIN, fontHeight));
		rand = getGraphics() == null ? 10 : getGraphics().getFontMetrics()
				.charsWidth(new char[] { '-', '1', '0', '0' }, 0, 4) / 2;
		width = getWidth() - 2 * rand;
		height = getHeight();
		int zeigerBreite = (int) Math.min(
				Math.max(11, Math.round(0.06 * width)), 0.4 * height);
		zeiger.setSize(zeigerBreite,
				(int) Math.max(11, Math.round(0.4 * height)));
		laengeGross = (int) Math.max(16, Math.round(0.2 * height));
		laengeKlein = (int) Math.round(0.75 * laengeGross);
		if (art != null) {
			this.art.setSize(new Dimension(width / 2, height / 2));
			this.art.setLocation(width, height / 2);
			this.art.setFont(getFont());
		}
		if (skala != null) {
			skala.setLaenge(width);
			zeiger.setLocation(rand + skala.transform(wert) - zeiger.getWidth()
					/ 2, height - zeiger.getHeight());
		}
	}

	/*********************************************************************
	 * 
	 * D A R S T E L L U N G
	 * 
	 *********************************************************************/
	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (width <= 1 || height <= 1)
			return;
		Graphics g1 = g.create();
		g1.setColor(Color.white);
		g1.fillRect(0, 0, getWidth(), getHeight());
		g1.translate(rand, 0); /* linker und rechter Rand */
		if (DEBUG) {
			Color saveColor = g.getColor();
			g.setColor(Color.CYAN);
			g.drawRect(rand, rand, width, height - 2 * rand);
			g.setColor(saveColor);
		}
		paintSkala(g1);
		paintZahlen(g1);
	}

	protected void paintSkala(Graphics g) {
		if (skala == null) {
			g.drawLine(0, 0, getWidth(), getHeight());
			g.drawLine(0, getHeight(), getWidth(), 0);
			return;
		}
		Graphics g1 = g.create();
		g1.translate(0, height - (int) Math.round(0.4 * height));
		g1.setColor(Color.BLACK);
		int[] striche = skala.getStricheKlein();
		for (int index = 0; index < striche.length; index++) {
			g1.drawLine(striche[index], 0, striche[index], -laengeKlein);
		}
		striche = skala.getStricheGross();
		for (int index = 0; index < striche.length; index++) {
			g1.drawLine(striche[index], 0, striche[index], 0 - laengeGross);
		}
	}

	protected void paintZahlen(Graphics g) {
		if (skala == null)
			return;
		Graphics g1 = g.create();
		g1.setColor(Color.BLACK);
		g1.translate(0, (int) Math.round(0.3 * height));
		int textBreite = 0;
		int x;
		Skalentext[] zahlen = skala.getSkalenText();
		for (int index = 0; index < zahlen.length; index++) {
			if (zahlen[index].text == null)
				continue;
			textBreite = g1.getFontMetrics().charsWidth(zahlen[index].toCharArray(),
					0, zahlen[index].length());
			x = (int) Math.round(zahlen[index].pos - textBreite / 2.0);
			if(index == 0)
				x = Math.max(-rand/2,x);
			if(index == zahlen.length-1)
				x = Math.min(x,width-textBreite+rand/2);
			g1.drawString(zahlen[index].text, x, 0);
		}
	}

	protected int calcPtSize(double faktor, Graphics g) {
		if (g == null)
			return 10;
		Font font = getFont();
		FontMetrics fm = g.getFontMetrics(font);
		double yf = faktor * getHeight() / fm.getHeight();
		int pt = (int) Math.round(Math.floor(yf * font.getSize()));
		return pt;
	}

}
