/** 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

grammar Formel;

@header{ 
package ghmessen.parser; 

}
funktion: ID ( '(' fvar ')' )? '=' expr KOMMENTAR? EOF
;
fvar: ID ( '[' expr ']' )?
;
start: expr KOMMENTAR? EOF
;
literal: '#e'                                  # leuler
    |    '#pi'                                 # lpi
	|	 DEZIMAL                               # ldezimal
	|	 date                                  # ldate
	|	 time                                  # ltime
;
date:   	INT '.' INT '.' INT?
;
time:   	INT ':' INT (':' INT)?
;
expr:
	'(' expr ')'                               # epar
	| literal                                  # elit
    | FUNC '(' expr ')'                        # efunc
	| 'new' '(' expr ')'                       # enew
	| 'int' '(' expr ')'                       # eint
	| expr '^' expr                            # eexp
	| expr op=('*'|'/') expr                   # emult
	| expr op=('+'|'-') expr                   # eadd
	| vz=('+'|'-') expr                        # epm
	| expr op=('='|'>'|'>='|'<'|'<=') expr     # erel
	| expr 'and' expr                          # eand
	| expr 'or' expr                           # eor
	| ID ('[' expr ']')?                       # eatom
;

FUNC:	('ramp'|'square'|'saw'|'shift'|'sin'|'cos'|'tan'|'arcsin'|'arccos'|'rsin'|'rcos'|
		'rtan'|'rarcsin'|'rarccos'|'rarctan'|'delta'|'random'|'sqr'|'exp'|'ln'|'log'|'frac'|
		'abs'|'sgn'|'odd'|'even'|'not'|'defined'|'sec'|'min'|'day')
;
DEZIMAL:	INT (',' INT)? (('e'|'E') ('+'|'-')? INT)?
;
INT:	'0'..'9'+
;
fragment GREEKLETTER: ('\u0391'..'\u03A9'|'\u03B1'..'\u03C9'|'\u03D1')
;
fragment IDFIRST 	: 	(GREEKLETTER|'a'..'z'|'A'..'Z')
;
fragment IDPART 	: (IDFIRST |'0'..'9')
;
ID:	     IDFIRST IDPART* ('_' (IDPART+))? ('#' INT)?
;
WS:  	 (' '|'\t')+ -> skip
;
KOMMENTAR: ';' (~('\r'|'\n'))*
;
