package ghmessen.parser;

import ghmessen.groesse.WerteTabelle;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.misc.NotNull;
import org.antlr.v4.runtime.tree.ParseTree;
import org.codehaus.janino.CompileException;
import org.codehaus.janino.ExpressionEvaluator;
import org.codehaus.janino.Parser.ParseException;
import org.codehaus.janino.Scanner.ScanException;

public class FormelGenerator extends FormelBaseVisitor<Void> {

	private static final boolean DEBUG = false;
	List<String> vars = Arrays.asList(new String[]{ "t", "n", "f", "\u03B2", "\u03BB", "E",
			"delta", "date", "time"});
	List<String> ids = new ArrayList<String>();
	String fname;
	public static ExpressionEvaluator NULLAUSDRUCK;
	public enum Baumart {GUELTIG, UNGUELTIG};
	public Baumart zustand;
	private ParseTree t;
	private String eingabe;
	private String html;
	private String formel;
	
	public FormelGenerator(){
		this("",null);
		zustand = Baumart.UNGUELTIG;
		t = null;
		eingabe = null;
	}
	
	public FormelGenerator(String name, String[] konstanten) {
		setName(name);
		setKonstanten(konstanten);
		zustand = Baumart.UNGUELTIG;
		t = null;
		eingabe = null;
	}

	public FormelGenerator(String[] konstanten, String s){
		setKonstanten(konstanten);
		zustand = Baumart.UNGUELTIG;
		t = null;
		eingabe = s;
	}
	
	public FormelGenerator(String s){
		zustand = Baumart.UNGUELTIG;
		t = null;
		eingabe = s;
	}
	
	public ParseTree getTree() {
		return t;
	}
	
	public void setTree(ParseTree t) {
		this.t = t;
	}
	
	public void setFormel(String s){
		zustand = Baumart.UNGUELTIG;
		t = null;
		eingabe = s;
	}
	
	public String getFormel() {
		return formel;
	}
	
	public void setName(String s) {
		fname = s;
	}

	public String getName() {
		return fname;
	}

	public void setKonstanten(String[] k){
		ids.clear();
		if(k != null)
			ids.addAll(Arrays.asList(k));
	}
	
	public String[] getKonstanten(){
		return ids.toArray(new String[]{});
	}

	public void addKonstante(String s) {
		if (s == null || s.equals(""))
			return;
		if (ids == null)
			ids = Arrays.asList(new String[]{s});
		else if (ids.contains(s))
				return;
		ids.add(s);
	}

	public void removeKonstante(String s) {
		if (s == null || s.equals("") || ids == null || ids.size() == 0)
			return;
		ids.remove(s);
	}

	public void replaceKonstante(String alt, String neu) {
		if (alt == null || neu == null || alt.equals("") || neu.equals(""))
			return;
		if(ids.indexOf(neu) > -1)
			removeKonstante(alt);
		else if(!(ids.indexOf(alt)>-1))
			addKonstante(neu);
		else
			ids.set(ids.indexOf(alt),neu);
	}
	
	public boolean parsen(String s){
		return parsen(s, false);
	}
	
	public boolean parsen(String s, boolean funktion){
		setFormel(s);
		return _parse(funktion);
	}
	
	private boolean _parse(boolean funktion){
		if(eingabe == null)
			return false;
        ANTLRInputStream input = new ANTLRInputStream(eingabe);
        FormelLexer lexer = new FormelLexer(input);
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        FormelParser parser = new FormelParser(tokens);
        parser.setBuildParseTree(true);
        parser.removeErrorListeners();
        parser.setErrorHandler(new FormelParserFehler());
        formel = "";
        html = "";
        try {
        	if(funktion){
				t = parser.funktion();
			}
			else {
				t = parser.start();
			}
        	this.visit(t);
        	zustand = Baumart.GUELTIG;
			return true;
        } catch (RuntimeException e) {
        	zustand = Baumart.UNGUELTIG;
        	return false;
        }
	}
	
	public ExpressionEvaluator getAusdruck(String s){
		setFormel(s);
		return getAusdruck();
	}
	
	public String getHTML(String s, boolean funktion){
		parsen(s, funktion);
		return html;
	}
	
	public String getHTML(){
		if(zustand != Baumart.GUELTIG)
			_parse(false);
		if(DEBUG)
			System.out.println("[FormelParser] - getHTML("+html+")");
		return html;
	}
		
	
	private String _generiere(){
		if(zustand != Baumart.GUELTIG)
			_parse(false);
		if(DEBUG)
			System.out.println("[FormelParser] - generiere("+formel+")");
		return formel;
	}
		
	public ExpressionEvaluator getAusdruck() {
		ExpressionEvaluator result = new ExpressionEvaluator();
		result.setExpressionType(double.class);
		result.setDefaultImports(new String[] { "static ghmessen.util.Math.*" });
		result.setParameters(new String[] { "t", "n", "f", "\u03B2", "\u03BB", "E", "delta",
				"date", "time", "k", "_werte" }, new Class[] { double.class,
				double.class, double.class, double.class,
				double.class, double.class, double.class,
				int.class, int.class, Double[].class, WerteTabelle.class });
		try {
			result.cook(_generiere());
		} catch (CompileException e) {
			if (DEBUG)
				e.printStackTrace();
			result = NULLAUSDRUCK;
		} catch (ParseException e) {
			if (DEBUG)
				e.printStackTrace();
			result = NULLAUSDRUCK;
		} catch (ScanException e) {
			if (DEBUG)
				e.printStackTrace();
			result = NULLAUSDRUCK;
		}
		return result;
	}
	
	/**
	 * Methoden von FormelBaseListener
	 * 
	 */
	
	@Override public Void visitDate(@NotNull FormelParser.DateContext ctx) {
		String datum = "0";
		try {
			datum = Integer.toString(Integer.parseInt(ctx.INT(0).getText())
				+ Integer.parseInt(ctx.INT(1).getText())*100
				+ (ctx.INT().size() < 3 ? 
						new GregorianCalendar().get(Calendar.YEAR) :
							Integer.parseInt(ctx.INT(2).getText()))*100000);
		} catch(NumberFormatException e){
			if(DEBUG)
				e.printStackTrace();
		}
		formel += datum;
		html += datum;
		return null;
	}

	@Override public Void visitEint(@NotNull FormelParser.EintContext ctx) {
		formel += "_int(";
		html += "<font color=\"#0000FF\">int</font>(";
		visit(ctx.expr());
		formel += ")";
		html += ")";
		return null;
	}

	@Override public Void visitEor(@NotNull FormelParser.EorContext ctx) {
		formel += "or(";
		visit(ctx.expr(0));
		formel += ",";
		html += " <b>or</b> ";
		visit(ctx.expr(1));
		formel += ")";
		return null;
	}
	
	private void _visitAtom(String id, @NotNull FormelParser.ExprContext expr) {
		boolean isId, isVar, hasIndex = false;
		isId = ids.contains(id);
		if(isId)
			hasIndex = expr != null;
		isVar = vars.contains(id);
		int nr = ids.indexOf(id);
		if(isId || isVar) {
			if(hasIndex) {
				formel += "getValueAt(_werte,";
				html += "<i>"+id+"[";
				visit(expr);
				formel += ","+nr+")";
				html += "]</i>";
			} else if(isId) {
				formel += "k["+nr+"]";
				html += "<i>"+id+"</i>";
			} else if(isVar) {
				formel += id;
				html += "<i>"+id+"</i>";
			}
		}
	}
	
	@Override public Void visitEatom(@NotNull FormelParser.EatomContext ctx) {
		_visitAtom(ctx.ID().getText(), ctx.expr());
		return null;
	}

	@Override public Void visitEfunc(@NotNull FormelParser.EfuncContext ctx) {
		formel += ctx.FUNC().getText()+"(";
		html += "<font color=\"#0000FF\">"+ctx.FUNC().getText()+"</font>(";
		visit(ctx.expr());
		formel += ")";
		html += ")";
		return null;
	}
	@Override public Void visitEmult(@NotNull FormelParser.EmultContext ctx) {
		visit(ctx.expr(0));
		formel += ctx.op.getText();
		if("*".equals(ctx.op.getText()))
			html += " &sdot; ";
		else
			html += " &frasl; ";
		visit(ctx.expr(1));
		return null;
	}

	@Override public Void visitLdezimal(@NotNull FormelParser.LdezimalContext ctx) {
		String komplett = ctx.DEZIMAL().getText();
		int idx1 = komplett.indexOf(','), 
				idx2 = Math.max(komplett.indexOf("e"), komplett.indexOf("E"));
		String ganz = idx1>-1?komplett.substring(0, idx1):
			(idx2>-1?komplett.substring(0,idx2):komplett);
		formel += ganz;
		if(idx1 > -1) {
			formel += "." + 
					(idx2 > - 1?
							komplett.substring(idx1+1,idx2):
								komplett.substring(idx1+1));
		}
		else {
			formel += ".0";
		}
		if(idx2 > -1) {
			formel += "E" + komplett.substring(idx2+1);
			html += komplett.substring(0,idx2);
			html += "&sdot;10<sup>"+komplett.substring(idx2+1)+"</sup>";
		}
		else {
			html += komplett;
		}
		return null;
	}
	
	@Override public Void visitFunktion(@NotNull FormelParser.FunktionContext ctx) {
		formel += ctx.ID().getText() + "(";
		html += "<i><b>"+ctx.ID().getText() + "</b>(";
		if(ctx.fvar() != null)
			visit(ctx.fvar());
		formel += ")";
		html += ")</i>";
		return null;
	}
	
	@Override public Void visitStart(@NotNull FormelParser.StartContext ctx) {
		visit(ctx.expr());
		if(ctx.KOMMENTAR() != null) {
			html += " <font size=\"-1\" color=\"#A4A4A4\">";
			visit(ctx.KOMMENTAR());
			html += "</font>";
		}
		return null; 
	}

	@Override public Void visitLeuler(@NotNull FormelParser.LeulerContext ctx) {
		formel += "_E";
		html += "&#x2107;";
		return null;
	}
	
	@Override public Void visitFvar(@NotNull FormelParser.FvarContext ctx) {
		_visitAtom(ctx.ID().getText(), ctx.expr());
		return null;
	}
	
	@Override public Void visitEpar(@NotNull FormelParser.EparContext ctx) {
		formel += "(";
		visit(ctx.expr());
		formel += ")";
		return null;
	}

	@Override public Void visitEadd(@NotNull FormelParser.EaddContext ctx) {
		visit(ctx.expr(0));
		formel += ctx.op.getText();
		if("+".equals(ctx.op.getText()))
			html += " + ";
		else
			html += " &minus; ";
		visit(ctx.expr(1));
		return null;
	}
	
	@Override public Void visitEnew(@NotNull FormelParser.EnewContext ctx) {
		formel += "_new(";
		html += "<font color=\"#0000FF\">new</font>(";
		visit(ctx.expr());
		formel += ")";
		html += ")";
		return null;
	}

	@Override public Void visitEexp(@NotNull FormelParser.EexpContext ctx) {
		formel += "pow(";
		visit(ctx.expr(0));
		formel += ",";
		html += "<sup>";
		visit(ctx.expr(1));
		html += "</sup>";
		formel += ")";
		return null;
	}

	private String _relation(String s){
			switch(s.charAt(0)){
			case '<':
				return s.length() == 1 ? "kleiner" : "kleinergl";
			default:
				return "gleich";
			case '>':
				return s.length() == 1 ? "groesser" : "groessergl";
			}
	}
	
	private String _relationHTML(String s){
		switch(s.charAt(0)){
		case '<':
			return s.length() == 1 ? "&lt;" : "&le;";
		default:
			return "=";
		case '>':
			return s.length() == 1 ? "&gt;" : "&ge;";
		}
}

	@Override public Void visitErel(@NotNull FormelParser.ErelContext ctx) {
		formel += _relation(ctx.op.getText()) + "(";
		visit(ctx.expr(0));
		formel += ",";
		html += " "+ _relationHTML(ctx.op.getText()) + " ";
		visit(ctx.expr(1));
		formel += ")";
		return null;
	}

	@Override public Void visitTime(@NotNull FormelParser.TimeContext ctx) {
		String zeit = "0";
		try {
			zeit = Integer.toString(Integer.parseInt(ctx.INT(0).getText())*3600
				+ Integer.parseInt(ctx.INT(1).getText())*60
				+ (ctx.INT().size() < 3? 0 : 
					Integer.parseInt(ctx.INT(2).getText())));
		}
		catch(NumberFormatException e){
			if(DEBUG)
				e.printStackTrace();
		}
		formel += zeit;
		html += zeit;
		return null;
	}

	@Override public Void visitEpm(@NotNull FormelParser.EpmContext ctx) {
		formel += ctx.vz.getText();
		html += ctx.vz.getText();
		return visit(ctx.expr());
	}

	@Override public Void visitLpi(@NotNull FormelParser.LpiContext ctx) {
		formel += "_PI";
		html += "&pi;";
		return null;
	}
	
	@Override public Void visitEand(@NotNull FormelParser.EandContext ctx) {
		formel += "and(";
		visit(ctx.expr(0));
		formel += ",";
		html += " <b>and</b> ";
		visit(ctx.expr(1));
		formel += ")";
		return null;
	}

	static {
		try {
			NULLAUSDRUCK = new ExpressionEvaluator("0", double.class,
				new String[]{ "t", "n", "f", "\u03B2", "\u03BB", "E",
			"delta", "date", "time", "k", "_werte"}, new Class[] { double.class,
				double.class, double.class, double.class,
				double.class, double.class, double.class,
				int.class, int.class, Double[].class, WerteTabelle.class });
		} catch (CompileException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (ParseException e) {
			if (DEBUG)
				e.printStackTrace();
		} catch (ScanException e) {
			if (DEBUG)
				e.printStackTrace();
		}
	}

}
