/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.util;

import static ghmessen.groesse.wertetabelle.XAchseTyp.UNABHAENGIG;
import ghmessen.groesse.WerteTabelle;

import java.util.Calendar;
import java.util.GregorianCalendar;

/*
 * Diese Klasse stellt alle Methoden bereit, die in Formeln benötigt werden.
 * 
 */
public class Math {

	private static final boolean DEBUG = false;

	private static double grad(double r) {
		return java.lang.Math.toDegrees(r);
	}

	private static double radiant(double g) {
		return java.lang.Math.toRadians(g);
	}

	private static double letzterWert_delta = 0.0;
	private static double letzterWert_new = 0.0;

	/**********************************************************
	 * Hilfsroutinen
	 *********************************************************/
	public static int datum(Calendar c) {
		return c.get(Calendar.YEAR) * 100000 + c.get(Calendar.MONTH) * 100
				+ c.get(Calendar.DAY_OF_MONTH);

	}

	public static int uhrzeit(Calendar c) {
		return c.get(Calendar.HOUR_OF_DAY) * 3600 + c.get(Calendar.MINUTE) * 60
				+ c.get(Calendar.SECOND);
	}

    /**********************************************************
	 * Exporte für Formeln
	 *********************************************************/

	public static double _PI = java.lang.Math.PI;
	public static double _E = java.lang.Math.E;

	public static double ramp(double x) {
		return frac(x);
	}

	public static double square(double x) {
		return ramp(x) < 0.5 ? 0.0 : 1.0;
	}

	public static double saw(double x) {
		if (0 < x && x < 0.5)
			return 2.0 * x;
		else if (0.5 < x && x < 1.0)
			return 2.0 * (1.0 - x);
		else
			return 0.0;
	}

	public static double shift(double x) {
		if (x < 0)
			return 0.0;
		else if (x > 1)
			return 1.0;
		else
			return x;
	}

	public static double sin(double g) {
		return java.lang.Math.sin(radiant(g));
	}

	public static double cos(double g) {
		return java.lang.Math.cos(radiant(g));
	}

	public static double tan(double g) {
		return java.lang.Math.tan(radiant(g));
	}

	public static double arcsin(double x) {
		return grad(java.lang.Math.asin(x));
	}

	public static double arccos(double x) {
		return grad(java.lang.Math.acos(x));
	}

	public static double arctan(double x) {
		return grad(java.lang.Math.atan(x));
	}

	public static double rsin(double r) {
		return java.lang.Math.sin(r);
	}

	public static double rcos(double r) {
		return java.lang.Math.cos(r);
	}

	public static double rtan(double r) {
		return java.lang.Math.tan(r);
	}

	public static double rarcsin(double x) {
		return java.lang.Math.asin(x);
	}

	public static double rarccos(double x) {
		return java.lang.Math.acos(x);
	}

	public static double rarctan(double x) {
		return java.lang.Math.atan(x);
	}

	public static double delta(double x) {
		double ret = x - letzterWert_delta;
		letzterWert_delta = x;
		return ret;
	}

	public static double _new(double x) {
		double ret = letzterWert_new == x ? 0.0 : 1.0;
		letzterWert_new = x;
		return ret;
	}

	public static double random(double x) {
		return java.lang.Math.random() * x;
	}

	public static double sqr(double x) {
		return java.lang.Math.sqrt(x);
	}

	public static double exp(double x) {
		return java.lang.Math.exp(x);
	}

	public static double ln(double x) {
		return java.lang.Math.log(x);
	}

	public static double log(double x) {
		return java.lang.Math.log10(x);
	}

	public static double _int(double x) {
		return java.lang.Math.floor(x);
	}

	public static double frac(double x) {
		return x - java.lang.Math.floor(x);
	}

	public static double abs(double x) {
		return java.lang.Math.abs(x);
	}

	public static double sgn(double x) {
		return java.lang.Math.signum(x);
	}

	public static double odd(double x) {
		return ((int) x) % 2;
	}

	public static double even(double x) {
		return ((int) x) % 2 == 0 ? 1.0 : 0.0;
	}

	public static double not(double x) {
		return x == 0.0 ? 1.0 : 0.0;
	}

	public static double defined(Double x) {
		if (x.isInfinite() || x.isNaN())
			return 0.0;
		else
			return 1.0;
	}

	public static double sec(double x) {
		return _int(x);
	}

	public static double min(double x) {
		return 60.0 * _int(x / 60.0);
	}

	public static double day(double x) {
		Calendar d = new GregorianCalendar();
		d.set((int) (x / 100000.0), ((int) (x / 100.0)) % 1000, ((int) x) % 100);
		int result = d.get(Calendar.DAY_OF_WEEK);
		if (result == 0)
			return 7;
		else
			return result;
	}

	public static double and(double x, double y) {
		if (x != 0 && y != 0)
			return 1.0;
		else
			return 0.0;
	}

	public static double or(double x, double y) {
		if (x != 0 || y != 0)
			return 1.0;
		else
			return 0.0;
	}

	public static double pow(double x, double y) {
		return java.lang.Math.pow(x, y);
	}

	public static double gleich(double x, double y) {
		return x == y ? 1.0 : 0.0;
	}

	public static double groesser(double x, double y) {
		return x > y ? 1.0 : 0.0;
	}

	public static double groessergl(double x, double y) {
		return x >= y ? 1.0 : 0.0;
	}

	public static double kleiner(double x, double y) {
		return x < y ? 1.0 : 0.0;
	}

	public static double kleinergl(double x, double y) {
		return x <= y ? 1.0 : 0.0;
	}

	public static double minus(double x, double y) {
		return x - y;
	}

	public static double plus(double x, double y) {
		return x + y;
	}

	public static double unminus(double x) {
		return -x;
	}

	public static double getValueAt(WerteTabelle w, double row, double col) {
		//Umrechnung der 1-basierten auf die 0-basierte Indizierung
		Double wert = (Double) w.getValueAt((int)(row-1),(int)(UNABHAENGIG + col));
		if (DEBUG)
			System.out.println("[Math] - getValueAt: w(" + w + ") row(" + row
					+ ") col(" + col + ") wert("+wert+")");
		if (wert != null)
			return wert;
		else
			return 0.0;
	}
}
