package ghmessen.util;

import static java.lang.Math.*;

import java.text.NumberFormat;

/*
 * Hier werden alle mathematischen Berechnungsmethoden bereitgestellt, die
 * an mehr als einer Stelle im Programm benötigt werden.
 */

public class MathUtil {

	public static double moduloGrenze(double grenze, double unterteilung,
			boolean aussen) {
		if (grenze == 0.0)
			return grenze;
		if (aussen)
			return signum(grenze) * ceil(abs(grenze / unterteilung)+0.0001)
					* unterteilung;
		else
			return signum(grenze) * floor(abs(grenze / unterteilung)+0.0001)
					* unterteilung;
	}

	public static String[] berechneSkalaListe(double minimum, double maximum) {
		return berechneSkalaListe(minimum, maximum,
				findeGrobeUnterteilung(minimum, maximum));
	}

	public static String[] berechneSkalaListe(double minimum, double maximum,
			double unterteilung) {
		double linkeGrenze = moduloGrenze(minimum, unterteilung, false);
		double rechteGrenze = moduloGrenze(maximum, unterteilung, false);
		String[] result = new String[(int) round((rechteGrenze - linkeGrenze)
				/ unterteilung + 1.0)];
		for (int i = 0; i < result.length; i++)
			result[i] = String.format(unterteilung >= 1 ? "%1.0f"
					: (unterteilung >= 0.1 ? "%1.1f"
							: (unterteilung >= 0.01 ? "%2.2f" : "%3.3f")),
					linkeGrenze + i * unterteilung);
		return result;
	}

	public static double findeExponent(double minimum, double maximum) {
		if (minimum == 0.0 && maximum == 0.0)
			return 0.0;
		if (minimum == 0.0)
			return floor(log10(abs(maximum)));
		else if (maximum == 0.0)
			return floor(log10(abs(minimum)));
		else
			return max(floor(log10(abs(minimum))), floor(log10(abs(maximum))));
	}

	public static double findeGrobeUnterteilung(double minimum, double maximum) {
		if (minimum != 0.0 && maximum != 0.0) {
			maximum = abs(maximum - minimum);
			minimum = 0.0;
		}
		double exponent = findeExponent(minimum, maximum);
		minimum /= pow(10.0, exponent);
		maximum /= pow(10.0, exponent);
		maximum = round(max(abs(minimum), abs(maximum)));
		if (maximum < 1.5)
			return 0.2 * pow(10.0, exponent);
		else if (maximum <= 3)
			return 1.0 * pow(10.0, exponent);
		else if (maximum <= 6)
			return 1.0 * pow(10.0, exponent);
		else
			return 2.0 * pow(10.0, exponent);
	}

	public static double findeMittlereUnterteilung(double minimum,
			double maximum) {
		if (minimum != 0.0 && maximum != 0.0) {
			maximum = abs(maximum - minimum);
			minimum = 0.0;
		}
		double exponent = findeExponent(minimum, maximum);
		minimum /= pow(10.0, exponent);
		maximum /= pow(10.0, exponent);
		maximum = round(max(abs(minimum), abs(maximum)));
		if (maximum < 1.5)
			return 0.1 * pow(10.0, exponent);
		else if (maximum <= 3)
			return 0.5 * pow(10.0, exponent);
		else
			return 1.0 * pow(10.0, exponent);
	}

	public static double findeFeineUnterteilung(double minimum, double maximum) {
		if (minimum != 0.0 && maximum != 0.0) {
			maximum = abs(maximum - minimum);
			minimum = 0.0;
		}
		double exponent = findeExponent(minimum, maximum);
		minimum /= pow(10.0, exponent);
		maximum /= pow(10.0, exponent);
		maximum = round(max(abs(minimum), abs(maximum)));
		if (maximum <= 1.5)
			return 0.05 * pow(10.0, exponent);
		else if (maximum <= 3)
			return 0.1 * pow(10.0, exponent);
		else
			return 0.5 * pow(10.0, exponent);

	}

    // Berechnung einer logarithmischen Skala
    private static final double LOG10SCALE = 1/Math.log(10);
    public static double log10(double val) { return Math.log(val) * LOG10SCALE; }
    public static double exp10(double val) { return Math.exp(val / LOG10SCALE); }
    public static float flog10(double val) { return (float)log10(val); }
    public static float fexp10(double val) { return (float)exp10(val); }

    private static NumberFormat numberFormat = null;
    public static String formatNum(double num, int numfracdigits) {
        if (numberFormat == null) {
            numberFormat = NumberFormat.getInstance();
            numberFormat.setGroupingUsed(false);
        }
        numberFormat.setMinimumFractionDigits(numfracdigits);
        numberFormat.setMaximumFractionDigits(numfracdigits);
        return numberFormat.format(num);
    }

    public static int numFracDigits(double num) {
        int numdigits = 0;
        while (numdigits <= 15 && num != java.lang.Math.floor(num)) {
            num *= 10.0;
            numdigits += 1;
        }
        return numdigits;
    }
}
