/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.util;

public class UnsignedUtil {
	public static final int DOUBLESIZE = 8;
	public static final int INTSIZE = 4;

	public static short unsignedByte(int b) {
		return (short) (((short) b + 256) & 255);
	}

	public static long unsignedLongByte(long b) {
		return (long) (((long) b + 256) & 255);
	}

	public static int writeIntToBuffer(int daten, byte[] buffer, int start) {
		if (buffer == null || buffer.length - start < INTSIZE)
			return 0;
		for (int i = 0; i < INTSIZE; i++)
			buffer[start + i] = (byte) ((daten >> (8 * (3 - i))) & 0xff);
		return INTSIZE;
	}

	public static int readIntFromBuffer(byte[] buffer, int start) {
		if (buffer == null || buffer.length - start < INTSIZE)
			return 0;
		return unsignedByte(buffer[start + 3])
				| (unsignedByte(buffer[start + 2]) << 8)
				| (unsignedByte(buffer[start + 1]) << 16)
				| (unsignedByte(buffer[start]) << 24);
	}

	public static int writeDoubleToBuffer(double daten, byte[] buffer, int start) {
		if (buffer == null || buffer.length - start < DOUBLESIZE)
			return 0;
		long d = Double.doubleToRawLongBits(daten);
		for (int i = 0; i < DOUBLESIZE; i++)
			buffer[start + i] = (byte) ((d >> (8 * (7 - i))) & 0xff);
		return DOUBLESIZE;
	}

	public static double readDoubleFromBuffer(byte[] buffer, int start) {
		if (buffer == null || buffer.length - start < DOUBLESIZE)
			return 0;
		return Double.longBitsToDouble(unsignedLongByte(buffer[start + 7])
				| (unsignedLongByte(buffer[start + 6]) << 8)
				| (unsignedLongByte(buffer[start + 5]) << 16)
				| (unsignedLongByte(buffer[start + 4]) << 24)
				| (unsignedLongByte(buffer[start + 3]) << 32)
				| (unsignedLongByte(buffer[start + 2]) << 40)
				| (unsignedLongByte(buffer[start + 1]) << 48)
				| (unsignedLongByte(buffer[start]) << 56));
	}
}
