/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.util;

import ghmessen.groesse.Art;
import ghmessen.groesse.Aufnahme;
import ghmessen.groesse.Einheit;
import ghmessen.groesse.Groesse;
import ghmessen.groesse.Groessenliste;
import ghmessen.groesse.Messgroesse;
import ghmessen.groesse.Skalierung;
import ghmessen.groesse.darstellung.AnpassungEnum;
import ghmessen.groesse.wertetabelle.XAchseTyp;
import ghmessen.instrumente.komponenten.Nullpunkt;

import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.PixelGrabber;
import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JSpinner;
import javax.swing.text.DefaultFormatter;
import javax.swing.text.NumberFormatter;

import ld.api.TriggerBits;
import ld.api.Waveforms;

public class Util {

	public static NumberFormatter doubleFormat;
	public static NumberFormatter posIntFormat;

	public static NumberFormatter createMinMaxFormatter(double min, double max,
			int stellen) {
		NumberFormat wFormat = NumberFormat.getInstance();
		wFormat.setMaximumFractionDigits(stellen);
		wFormat.setRoundingMode(RoundingMode.HALF_UP);
		NumberFormatter wFormatter = new NumberFormatter(wFormat);
		wFormatter.setMinimum(min);
		wFormatter.setMaximum(max);
		return wFormatter;
	}

	public static NumberFormatter createIntegerMinMaxFormatter(int min, int max) {
		NumberFormat wFormat = NumberFormat.getIntegerInstance();
		NumberFormatter wFormatter = new NumberFormatter(wFormat);
		wFormatter.setMinimum(min);
		wFormatter.setMaximum(max);
		return wFormatter;
	}

	public static void spinnerSetCommitsOnValidEdit(JSpinner spinner,
			boolean set) {
		JComponent comp = spinner.getEditor();
		JFormattedTextField field = (JFormattedTextField) comp.getComponent(0);
		DefaultFormatter formatter = (DefaultFormatter) field.getFormatter();
		formatter.setCommitsOnValidEdit(true);
	}

	public static String findUniqueName(List<?> liste, String name) {
		return findUniqueName(liste, name, true);
	}

	/*
	 * Findet einen eindeutigen String durch anhängen von Ziffern.
	 * 
	 * @param ohneNummer true: Es ist auch ein String ohne angehängte Ziffer
	 * erlaubt. false: Jeder String muss mit mindestens einer Ziffer enden.
	 */
	public static String findUniqueName(List<?> liste, String name,
			boolean ohneNummer) {
		boolean gefunden = true;
		name = stripNumbers(name);
		if (ohneNummer) {
			for (int i = 0; i < liste.size(); i++) {
				if (name.equals((String) liste.get(i).toString())) {
					gefunden = false;
				}
			}
			if (gefunden)
				return name;
		}
		String tmpName = name;
		for (int j = 1; j < 100; j++) {
			tmpName = name + j;
			gefunden = true;
			for (int i = 0; i < liste.size(); i++) {
				if (tmpName.equals((String) liste.get(i).toString())) {
					gefunden = false;
				}
			}
			if (gefunden)
				return tmpName;
		}
		return tmpName;
	}

	public static String findUniqueSymbol(Groessenliste liste, String symbol) {
		return findUniqueSymbol(liste, symbol, true);
	}

	public static String findUniqueSymbol(Groessenliste liste, String symbol,
			boolean ohneNummer) {
		boolean gefunden = true;
		symbol = stripNumbers(symbol);
		if (ohneNummer) {
			for (int i = 0; i < liste.getSize(); i++) {
				if (symbol.equals((String) liste.get(i).getSymbol())) {
					gefunden = false;
				}
			}
			if (gefunden)
				return symbol;
		}
		String tmpName = symbol;
		for (int j = 1; j < 100; j++) {
			tmpName = symbol + j;
			gefunden = true;
			for (int i = 0; i < liste.getSize(); i++) {
				if (tmpName.equals((String) liste.get(i).getSymbol())) {
					gefunden = false;
				}
			}
			if (gefunden)
				return tmpName;
		}
		return tmpName;
	}

	public static String stripNumbers(String s) {
		return s.replaceAll("\\d*$", "");
	}

	public static int stringIndexOf(String[] arr, String targetValue) {
		for(int index = 0; index < arr.length; index++){
			if(arr[index].equals(targetValue))
				return index;
		}
		return -1;
	}
	public static void selectButton(ButtonGroup bg, String property, String wert) {
		Enumeration<AbstractButton> buttons = bg.getElements();
		while (buttons.hasMoreElements()) {
			AbstractButton b = buttons.nextElement();
			if (wert.equals(b.getClientProperty(property))) {
				bg.setSelected(b.getModel(), true);
				break;
			}
		}
	}

	public static void selectButton(ButtonGroup bg, String property, Object wert) {
		Enumeration<AbstractButton> buttons = bg.getElements();
		while (buttons.hasMoreElements()) {
			AbstractButton b = buttons.nextElement();
			if (wert.equals(b.getClientProperty(property))) {
				bg.setSelected(b.getModel(), true);
				break;
			}
		}
	}

	public static Groesse findeGroesse(Groessenliste gl, String s) {
		return findeGroesse(gl, s, false);
	}

	public static Groesse findeGroesse(Groessenliste gl, String s,
			boolean autoCreate) {
		for (int i = 0; i < gl.getSize(); i++)
			if (gl.get(i).getSymbol().equals(s))
				return gl.get(i);
		if (autoCreate) {
			Groesse g = new Groesse(s);
			gl.add(g);
			return g;
		} else
			return null;
	}

	public static Groesse findeCassyGroesse(Groessenliste gl, int nr, int input, int ef) {
		return findeCassyGroesse(gl, nr, input, ef, false);
	}

	public static Groesse findeCassyGroesse(Groessenliste gl, int nr,
			int input, int ef, boolean autoCreate) {
		for (Groesse g : gl)
			if (g.getNr() == nr && g.getAB() == input && g.getEF() == ef)
				return g;
		if (autoCreate) {
			Groesse g = new Groesse(nr, input, ef);
			return g;
		}
		return null;
	}

	public static boolean hasAlpha(Image image) {
		if (image instanceof BufferedImage) {
			BufferedImage bimage = (BufferedImage) image;
			return bimage.getColorModel().hasAlpha();
		}
		PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
		try {
			pg.grabPixels();
		} catch (InterruptedException e) {
		}
		ColorModel cm = pg.getColorModel();
		return cm.hasAlpha();
	}

	public static BufferedImage toBufferedImage(Image image) {
		if (image instanceof BufferedImage) {
			return (BufferedImage) image;
		}
		image = new ImageIcon(image).getImage();
		boolean hasAlpha = hasAlpha(image);
		BufferedImage bimage = null;
		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		try {
			int transparency = Transparency.OPAQUE;
			if (hasAlpha) {
				transparency = Transparency.BITMASK;
			}
			GraphicsDevice gs = ge.getDefaultScreenDevice();
			GraphicsConfiguration gc = gs.getDefaultConfiguration();
			bimage = gc.createCompatibleImage(image.getWidth(null),
					image.getHeight(null), transparency);
		} catch (HeadlessException e) {
		}
		if (bimage == null) {
			int type = BufferedImage.TYPE_INT_RGB;
			if (hasAlpha) {
				type = BufferedImage.TYPE_INT_ARGB;
			}
			bimage = new BufferedImage(image.getWidth(null),
					image.getHeight(null), type);
		}
		Graphics g = bimage.createGraphics();
		g.drawImage(image, 0, 0, null);
		g.dispose();
		return bimage;
	}

	public static File createTempDir() {
		final String baseTempPath = System.getProperty("java.io.tmpdir");

		Random rand = new Random();
		int randomInt = 1 + rand.nextInt();

		File tempDir = new File(baseTempPath + File.separator + "tempDir"
				+ randomInt);
		if (tempDir.exists() == false) {
			tempDir.mkdir();
		}

		tempDir.deleteOnExit();

		return tempDir;
	}

	public static String htmlSup(String s) {
		String result = (s.startsWith("<html>") ? "" : "<html>")
				+ (s.contains("^") ? s.replace("^", "<sup>") + "</sup>" : s)
				+ (s.endsWith("</html>") ? "" : "</html>");
		return result;
	}

	public static String htmlSub(String s) {
		if(s == null)
			return "<html></html>";
		String result = (s.startsWith("<html>") ? "" : "<html>")
				+ (s.contains("_") ? s.replace("_", "<sub>") + "</sub>" : s)
				+ (s.endsWith("</html>") ? "" : "</html>");
		return result;
	}

	public static String htmlSubSup(String s) {
		return htmlSup(htmlSub(s));
	}

	public static String htmlSymbolUndEinheit(String symbol, String einheit) {
		String result;
		if (symbol.contains("_^^'"))
			result = symbol.replace("_^^'", "</sub>'").replace("_", "<sub>");
		else if (symbol.contains("_"))
			result = symbol.replace("_", "<sub>") + "</sub>";
		else
			result = symbol;
		result = "<html>" + result + " / " + einheit + "</html>";
		return result;
	}

	public static String htmlSupRemove(String s) {
		String result = s.replace("<html>", "").replace("<sup>", "^")
				.replace("</sup>", "").replace("</html>", "");
		return result;
	}

	public static String htmlSubRemove(String s) {
		String result = s.replace("<html>", "").replace("<sub>", "_")
				.replace("</sub>", "").replace("</html>", "");
		return result;
	}

	public static String htmlSubSupRemove(String s) {
		return htmlSupRemove(htmlSubRemove(s));
	}

	private static String toSetter(String field) {
		char[] stringArray = field.toCharArray();
		stringArray[0] = Character.toUpperCase(stringArray[0]);
		field = new String(stringArray);
		return "set" + field;
	}

	public static void setFieldValue(Object o, Field field, String value) {
		Method m;
		try {
			if (field.getType().isAssignableFrom(String.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, value);
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { String.class });
					m.invoke(o, new Object[] { value });
				}
			} else if (field.getType().isAssignableFrom(Integer.TYPE)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Integer.parseInt(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Integer.TYPE });
					m.invoke(o, new Object[] { Integer.parseInt(value) });
				}
			} else if (field.getType().isAssignableFrom(Long.TYPE)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Long.parseLong(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Long.TYPE });
					m.invoke(o, new Object[] { Long.parseLong(value) });
				}
			} else if (field.getType().isAssignableFrom(Boolean.TYPE)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Boolean.parseBoolean(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Boolean.TYPE });
					m.invoke(o, new Object[] { Boolean.parseBoolean(value) });
				}
			} else if (field.getType().isAssignableFrom(Double.TYPE)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Double.parseDouble(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Double.TYPE });
					m.invoke(o, new Object[] { Double.parseDouble(value) });
				}
			} else if (field.getType().isAssignableFrom(Float.TYPE)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Float.parseFloat(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Float.TYPE });
					m.invoke(o, new Object[] { Float.parseFloat(value) });
				}
			} else if (field.getType().isAssignableFrom(Art.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Art.toArt(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Art.class });
					m.invoke(o, new Object[] { Art.toArt(value) });
				}
			} else if (field.getType().isAssignableFrom(XAchseTyp.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, XAchseTyp.valueOf(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { XAchseTyp.class });
					m.invoke(o, new Object[] { XAchseTyp.valueOf(value) });
				}
			} else if (field.getType().isAssignableFrom(Einheit.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o,
							"".equals(value) ? null : Einheit.toEinheit(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Einheit.class });
					m.invoke(o, new Object[] { "".equals(value) ? null
							: Einheit.toEinheit(value) });
				}
			} else if (field.getType().isAssignableFrom(Messgroesse.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Messgroesse.toMessgroesse(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Messgroesse.class });
					m.invoke(o,
							new Object[] { Messgroesse.toMessgroesse(value) });
				}
			} else if (field.getType().isAssignableFrom(Nullpunkt.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Nullpunkt.valueOf(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Nullpunkt.class });
					m.invoke(o, new Object[] { Nullpunkt.valueOf(value) });
				}
			} else if (field.getType().isAssignableFrom(Aufnahme.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Aufnahme.valueOf(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Aufnahme.class });
					m.invoke(o, new Object[] { Aufnahme.valueOf(value) });
				}
			} else if (field.getType().isAssignableFrom(Waveforms.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Waveforms.valueOf(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Waveforms.class });
					m.invoke(o, new Object[] { Waveforms.valueOf(value) });
				}
			} else if (field.getType().isAssignableFrom(TriggerBits.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, TriggerBits.forValue(Integer.parseInt(value)));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { TriggerBits.class });
					m.invoke(o, new Object[] { TriggerBits.forValue(Integer.parseInt(value)) });
				}
			} else if (field.getType().isAssignableFrom(Aufnahme.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, Aufnahme.valueOf(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { Aufnahme.class });
					m.invoke(o, new Object[] { Aufnahme.valueOf(value) });
				}
			} else if (field.getType().isAssignableFrom(AnpassungEnum.class)) {
				if (Modifier.isPublic(field.getModifiers()))
					field.set(o, AnpassungEnum.valueOf(value));
				else {
					m = o.getClass().getMethod(toSetter(field.getName()),
							new Class[] { AnpassungEnum.class });
					m.invoke(o, new Object[] { AnpassungEnum.valueOf(value) });
				}
			} else
				throw new NumberFormatException(
						"[Util] - setFieldValue: Object(" + o + ") Field("
								+ field.getName() + ") Value(" + value
								+ ") nicht konvertierbar!");
		} catch (InvocationTargetException e) {
			System.err.println("[Util] - setFieldValue: Object(" + o
					+ ") Field(" + field.getName() + ") Value(" + value + ")"
					+ e.getMessage());
		} catch (IllegalArgumentException e) {
			System.err.println("[Util] - setFieldValue: Object(" + o
					+ ") Field(" + field.getName() + ") Value(" + value + ")"
					+ e.getMessage());
		} catch (IllegalAccessException e) {
			System.err.println("[Util] - setFieldValue: Object(" + o
					+ ") Field(" + field.getName() + ") Value(" + value + ")"
					+ e.getMessage());
		} catch (SecurityException e) {
			System.err.println("[Util] - setFieldValue: Object(" + o
					+ ") Field(" + field.getName() + ") Value(" + value + ")"
					+ e.getMessage());
		} catch (NoSuchMethodException e) {
			System.err.println("[Util] - setFieldValue: Object(" + o
					+ ") Field(" + field.getName() + ") Value(" + value + ")"
					+ e.getMessage());
		}
	}

	public static String writeDoublesToXml(double[] a) {
		String s = "";
		for (double d : a)
			s += "\"" + Double.toString(d) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeDoublesToXml(Double[] a) {
		String s = "";
		for (Double d : a)
			s += "\"" + (d == null ? "" : d.toString()) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeDoublesToXml(Vector<Double> a) {
		String s = "";
		for (Double d : a)
			s += "\"" + (d == null ? "" : d.toString()) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeIntegersToXml(Vector<Integer> a) {
		String s = "";
		for (Integer i : a)
			s += "\"" + (i == null ? "" : i.toString()) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeBooleansToXml(Vector<Boolean> a) {
		String s = "";
		for (Boolean b : a)
			s += "\"" + (b == null ? "" : b.toString()) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeBooleansToXml(Boolean[] a) {
		String s = "";
		for (Boolean b : a)
			s += "\"" + (b == null ? "" : b.toString()) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeBooleansToXml(boolean[] a) {
		String s = "";
		for (boolean b : a)
			s += "\"" + Boolean.toString(b) + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String writeSkalierungsToXml(Skalierung[] a) {
		String s = "";
		for (Skalierung d : a)
			s += "\"" + d.toString() + "\",";
		if (s.length() > 0)
			s = s.substring(0, s.length() - 1);
		return s;
	}

	public static String[] aStringFromXml(String liste) {
		return aStringFromXml(liste, false);
	}
	
	public static String[] aStringFromXml(String liste, boolean ignoriereLeere) {
		Vector<String> ret = new Vector<String>();
		String s = "";
		boolean quoted = false;
		char quote = '"';
		char sep = ',';
		int pos = 0;
		while (pos < liste.length()) {
			char c = liste.charAt(pos++);
			if (c == quote) {
				quoted = !quoted;
			} else if (quoted) {
				s += c;
			} else if (c == sep) {
				ret.add(s);
				s = "";
			}
		}
		if (!ignoriereLeere || !"".equals(s))
			ret.add(s);
		return ret.toArray(new String[] {});
	}

	public static Double[] dataAbsoluteFromRel(Double[] rel, int von, int bis, Groessenliste gl) {
		Double[] ret = Arrays.copyOfRange(rel, von, bis);
		for(int i=0;i<ret.length;i++)
			if(ret[i] != null)
				ret[i] *= gl.get(i).getEinheit().getFaktor();
		return ret;
	}
	
	public static Vector<Double> doubleVectorFromXml(String liste) {
		Vector<Double> ret = new Vector<Double>();
		for (String s : aStringFromXml(liste))
			ret.add(s == null || s.equals("") ? null : Double.parseDouble(s));
		return ret;
	}

	public static Double[] doubleAFromXml(String liste) {
		Vector<Double> ret = doubleVectorFromXml(liste);
		return ret == null ? null : ret.toArray(new Double[] {});
	}

	public static double[] prdoubleAFromXml(String liste) {
		Vector<Double> ret = doubleVectorFromXml(liste);
		double[] r = new double[ret.size()];
		for (int index = 0; index < r.length; index++)
			r[index] = ret.get(index) == null ? 0.0 : ret.get(index);
		return r;
	}

	public static Vector<Integer> intVectorFromXml(String liste) {
		Vector<Integer> ret = new Vector<Integer>();
		for (String s : aStringFromXml(liste))
			ret.add(Integer.parseInt(s));
		return ret;
	}

	public static Vector<Boolean> booleanVectorFromXml(String liste) {
		Vector<Boolean> ret = new Vector<Boolean>();
		for (String s : aStringFromXml(liste))
			ret.add(Boolean.parseBoolean(s));
		return ret;
	}

	public static boolean[] prbooleanAFromXml(String liste) {
		Vector<Boolean> ret = booleanVectorFromXml(liste);
		boolean[] r = new boolean[ret.size()];
		for (int index = 0; index < r.length; index++)
			r[index] = ret.get(index) == null ? false : ret.get(index);
		return r;
	}

	public static Skalierung[] skalierungAFromXml(String liste) {
		String[] sa = aStringFromXml(liste);
		Skalierung[] ret = new Skalierung[sa.length];
		for (int index = 0; index < ret.length; index++)
			ret[index] = Skalierung.toSkalierung(sa[index]);
		return ret;
	}

	public static void main(String[] args) {
		Vector<String> liste = new Vector<String>();
		System.out.println("Util\n========");
		System.out.println("findUniqueName(liste,Standard1,true)->"
				+ findUniqueName(liste, "Standard1", true));
		System.out.println("findUniqueName(liste,Willi,false)->"
				+ findUniqueName(liste, "Willi", false));
		System.out.println("findUniqueName(liste,Willi1,false)->"
				+ findUniqueName(liste, "Willi1", false));
	}

	static {
		doubleFormat = new NumberFormatter();
		doubleFormat.setValueClass(Double.class);
		posIntFormat = new NumberFormatter();
		posIntFormat.setValueClass(Integer.class);
		posIntFormat.setMinimum(0);
	}

}
