/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package libGoIO;

import java.io.Serializable;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class GoIOCalibrationPage implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 8347987132622361426L;
	float calibrationCoefficientA;
	float calibrationCoefficientB;
	float calibrationCoefficientC;
	String units;
	
	private static final String CALIBRATION = "Calibration";

	public GoIOCalibrationPage(tagGCalibrationPage page){
		calibrationCoefficientA = page.CalibrationCoefficientA();
		calibrationCoefficientB = page.CalibrationCoefficientB();
		calibrationCoefficientC = page.CalibrationCoefficientC();
		units = page.Units().getCString();
	}
	
	public GoIOCalibrationPage(){
		calibrationCoefficientA = 0.0f;
		calibrationCoefficientB = 0.0f;
		calibrationCoefficientC = 0.0f;
		units = "";
	}
	
	public static GoIOCalibrationPage toGoIOCalibrationPage(tagGCalibrationPage page){
		return new GoIOCalibrationPage(page);
	}
	
	public tagGCalibrationPage toGCalibrationPage(){
		tagGCalibrationPage ret = new tagGCalibrationPage();
		return ret;
	}

	public static GoIOCalibrationPage readFromXml(XMLEventReader eventReader)
			throws XMLStreamException {
		GoIOCalibrationPage gcp = new GoIOCalibrationPage();
		while (eventReader.hasNext()) {
			XMLEvent event = eventReader.nextEvent();
			if (event.isStartElement()) {
				StartElement startElement = event.asStartElement();
				if ("CalibrationCoefficientA".equals(startElement.getName().getLocalPart())) {
					event = eventReader.nextEvent();
					gcp.calibrationCoefficientA = Float.parseFloat(event.toString());
				} else if("CalibrationCoefficientB".equals(startElement.getName().getLocalPart())) {
					event = eventReader.nextEvent();
					gcp.calibrationCoefficientB = Float.parseFloat(event.toString());
				} else if("CalibrationCoefficientC".equals(startElement.getName().getLocalPart())) {
					event = eventReader.nextEvent();
					gcp.calibrationCoefficientC = Float.parseFloat(event.toString());
				} else if("Units".equals(startElement.getName().getLocalPart())) {
					event = eventReader.nextEvent();
					gcp.units = event.toString();
				}
			}
			if (event.isEndElement()
					&& event.asEndElement().getName().getLocalPart() == (CALIBRATION))
				break;
		}
		return gcp;
	}

	public float getCalibrationCoefficientA() {
		return calibrationCoefficientA;
	}

	public void setCalibrationCoefficientA(float calibrationCoefficientA) {
		this.calibrationCoefficientA = calibrationCoefficientA;
	}

	public float getCalibrationCoefficientB() {
		return calibrationCoefficientB;
	}

	public void setCalibrationCoefficientB(float calibrationCoefficientB) {
		this.calibrationCoefficientB = calibrationCoefficientB;
	}

	public float getCalibrationCoefficientC() {
		return calibrationCoefficientC;
	}

	public void setCalibrationCoefficientC(float calibrationCoefficientC) {
		this.calibrationCoefficientC = calibrationCoefficientC;
	}

	public String getUnits() {
		return units;
	}

	public void setUnits(String units) {
		this.units = units;
	}

}
