/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package libGoIO;

import ghmessen.groesse.Messbereich;

import java.util.Vector;

public class GoIOCalibrationVector extends Vector<GoIOCalibration>{

	private static final long serialVersionUID = -6617744714491681822L;

	public GoIOCalibration getCalibration(int sensorNumber, int ddsIndex){
		for(GoIOCalibration gc : this)
			if(sensorNumber == gc.getSensorNumber() && ddsIndex == gc.getDdsIndex())
				return gc;
		return null;
	}
	
	public GoIOCalibrationVector getCalibrationVector(int sensorNumber) {
		GoIOCalibrationVector ret = new GoIOCalibrationVector();
		for(GoIOCalibration gc : this){
			if(gc.getSensorNumber() != sensorNumber)
				continue;
			if(gc.getDdsIndex() >= ret.size())
				ret.add(gc);
			else {
				int index = ret.size()-1;
				while(index > 0 && gc.getDdsIndex()>ret.get(index).getDdsIndex())
					index--;
				ret.add(index, gc);
			}
		}
		return ret;
	}
	
	public Vector<Messbereich> getMessbereiche(int sensorNumber) {
		Vector<Messbereich> ret = new Vector<Messbereich>();
		Vector<Integer> mbIndex = new Vector<Integer>();
		for(GoIOCalibration gc : this){
			if(gc.getSensorNumber() != sensorNumber)
				continue;
			if(gc.getDdsIndex() >= ret.size()){
				ret.add(gc.getMessbereich());
				mbIndex.add(gc.getDdsIndex());
			} else {
				int index = ret.size()-1;
				while(index > 0 && gc.getDdsIndex()>mbIndex.get(index))
					index--;
				ret.add(index, gc.getMessbereich());
				mbIndex.add(index,gc.getDdsIndex());
			}
		}
		return ret;
	}
	

}
