/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package libGoIO;

import java.awt.Color;
import java.awt.Component;
import java.util.HashMap;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.JTree;
import javax.swing.ListCellRenderer;
import javax.swing.border.LineBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

@SuppressWarnings("serial")
public class GoIOSensorCellRenderer extends DefaultTreeCellRenderer
		implements ListCellRenderer<String> {

	private HashMap<String, Icon> icon;
	private String prefix;
	private String resourcePath;
	private String extension;

	public GoIOSensorCellRenderer() {
		this("sensor_", "resources/", ".png");
	}

	public GoIOSensorCellRenderer(String prefix, String resourcePath,
			String extension) {
		this.prefix = prefix;
		this.resourcePath = resourcePath;
		this.extension = extension;
		icon = new HashMap<String, Icon>();
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean sel, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {

		super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
				row, hasFocus);
		if (leaf
				&& value instanceof DefaultMutableTreeNode
				&& ((DefaultMutableTreeNode) value).getUserObject() instanceof GoIOSensorInfo) {
			GoIOSensorInfo sensorNode = (GoIOSensorInfo) ((DefaultMutableTreeNode) value)
					.getUserObject();
			if (!icon.containsKey(sensorNode.getSensorID())) {
				icon.put(sensorNode.getSensorID(), createImageIcon(resourcePath
						+ prefix + sensorNode.getSensorID() + extension));
			}
			setIcon(icon.get(sensorNode.getSensorID()));
		}
		return this;
	}

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = GoIOSensorCellRenderer.class
				.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	@Override
	public Component getListCellRendererComponent(JList<? extends String> list, String value,
			int index, boolean isSelected, boolean cellHasFocus) {
		setText((String) value);
		setBackground(Color.lightGray);
		if(isSelected) {
			setBorder(new LineBorder(Color.yellow));
			setOpaque(true);
		} else {
			setBorder(null);
			setOpaque(false);
		}
		if (((String)value).contains("(") && ((String)value).contains(")")) {
			value = ((String)value).substring(((String)value).indexOf("(") + 1, ((String)value).indexOf(")"));
			if (value != null && !"".equals(value)) {
				if (!icon.containsKey(value)) {
					icon.put((String) value, createImageIcon(resourcePath + prefix + value
							+ extension));
				}
				setIcon(icon.get(value));
			}
		}
		return this;
	}
}
