/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package libGoIO;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class GoIOSensorDDSRec implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1464187443449305412L;
	byte memMapVersion;
	byte manufacturerID;
	byte sensorNumber;
	byte[] sensorSerialNumber;
	byte[] sensorLotCode;
	String sensorLongName;
	String sensorShortName;
	byte uncertainty;
	byte significantFigures;
	byte currentRequirement;
	byte averaging;
	float minSamplePeriod;
	float typSamplePeriod;
	short typNumberOfSamples;
	short warmUpTime;
	byte experimentType;
	byte operationType;
	byte calibrationEquation;
	float yminValue;
	float ymaxValue;
	byte yScale;
	byte highestValidCalPageIndex;
	byte activeCalPage;
	Vector<GoIOCalibrationPage> calibrationPage;
	byte checksum;

	/* CalibrationList */
	private static final String CALIBRATION = "Calibration";
	private static final String CALIBRATIONLIST = "CalibrationList";

	/* SensorBurnRec */
	private static final String SENSORBURNREC = "SensorBurnRec";

	public GoIOSensorDDSRec() {
		super();
		sensorSerialNumber = new byte[3];
		sensorLotCode = new byte[2];
		sensorLongName = "";
		sensorShortName = "";
		calibrationPage = new Vector<GoIOCalibrationPage>();
		calibrationPage.add(new GoIOCalibrationPage());
	}

	@SuppressWarnings("unchecked")
	@Override
	public GoIOSensorDDSRec clone(){
		GoIOSensorDDSRec r = new GoIOSensorDDSRec();
		r.memMapVersion = memMapVersion;
		r.manufacturerID = manufacturerID;
		r.sensorNumber = sensorNumber;
		r.sensorSerialNumber = sensorSerialNumber.clone();
		r.sensorLotCode = sensorLotCode.clone();
		r.sensorLongName = sensorLongName;
		r.sensorShortName = sensorShortName;
		r.uncertainty = uncertainty;
		r.significantFigures = significantFigures;
		r.currentRequirement = currentRequirement;
		r.averaging = averaging;
		r.minSamplePeriod = minSamplePeriod;
		r.typSamplePeriod = typSamplePeriod;
		r.typNumberOfSamples = typNumberOfSamples;
		r.warmUpTime = warmUpTime;
		r.experimentType = experimentType;
		r.operationType = operationType;
		r.calibrationEquation = calibrationEquation;
		r.yminValue = yminValue;
		r.ymaxValue = ymaxValue;
		r.yScale = yScale;
		r.highestValidCalPageIndex = highestValidCalPageIndex;
		r.activeCalPage = activeCalPage;
		r.calibrationPage = (Vector<GoIOCalibrationPage>) calibrationPage.clone();
		r.checksum = checksum; 
		return r;
	}
	
	public GoIOSensorDDSRec(tagGSensorDDSRec src) {
		memMapVersion = src.MemMapVersion();
		manufacturerID = src.ManufacturerID();
		sensorNumber = src.SensorNumber();
		sensorSerialNumber = src.SensorSerialNumber().getBytes();
		sensorLotCode = src.SensorLotCode().getBytes();
		sensorLongName = src.SensorLongName().getCString();
		sensorShortName = src.SensorShortName().getCString();
		uncertainty = src.Uncertainty();
		significantFigures = src.SignificantFigures();
		currentRequirement = src.CurrentRequirement();
		averaging = src.Averaging();
		minSamplePeriod = src.MinSamplePeriod();
		typSamplePeriod = src.TypSamplePeriod();
		typNumberOfSamples = src.TypNumberofSamples();
		warmUpTime = src.WarmUpTime();
		experimentType = src.ExperimentType();
		operationType = src.OperationType();
		calibrationEquation = src.CalibrationEquation();
		yminValue = src.YminValue();
		ymaxValue = src.YmaxValue();
		yScale = src.Yscale();
		highestValidCalPageIndex = src.HighestValidCalPageIndex();
		activeCalPage = src.ActiveCalPage();
		calibrationPage = new Vector<GoIOCalibrationPage>();
		for (int index = 0; index <= highestValidCalPageIndex; index++)
			calibrationPage.add(new GoIOCalibrationPage(src.CalibrationPage()
					.get(index)));
		checksum = src.Checksum();
	}

	public tagGSensorDDSRec toGSensorDDSRec() {
		tagGSensorDDSRec ret = new tagGSensorDDSRec();
		ret.MemMapVersion(memMapVersion);
		ret.ManufacturerID(manufacturerID);
		ret.SensorNumber(sensorNumber);
		ret.SensorSerialNumber().setBytes(sensorSerialNumber);
		ret.SensorLotCode().setBytes(sensorLotCode);
		if(sensorLongName != null)
			ret.SensorLongName().setBytes(sensorLongName.getBytes());
		if(sensorShortName != null)
			ret.SensorShortName().setBytes(sensorShortName.getBytes());
		ret.Uncertainty(uncertainty);
		ret.SignificantFigures(significantFigures);
		ret.CurrentRequirement(currentRequirement);
		ret.Averaging(averaging);
		ret.MinSamplePeriod(minSamplePeriod);
		ret.TypSamplePeriod(typSamplePeriod);
		ret.TypNumberofSamples(typNumberOfSamples);
		ret.WarmUpTime(warmUpTime);
		ret.ExperimentType(experimentType);
		ret.OperationType(operationType);
		ret.CalibrationEquation(calibrationEquation);
		ret.YminValue(yminValue);
		ret.YmaxValue(ymaxValue);
		ret.Yscale(yScale);
		ret.HighestValidCalPageIndex(highestValidCalPageIndex);
		ret.ActiveCalPage(activeCalPage);
		for (int index = 0; index <= highestValidCalPageIndex; index++)
			ret.CalibrationPage().set(index,
					calibrationPage.get(index).toGCalibrationPage());
		return ret;
	}

	public static GoIOSensorDDSRec readFromXml(XMLEventReader eventReader)
			throws XMLStreamException {
		GoIOSensorDDSRec ret = new GoIOSensorDDSRec();
		while (eventReader.hasNext()) {
			XMLEvent event = eventReader.nextEvent();
			if (event.isStartElement()) {
				StartElement startElement = event.asStartElement();
				if (startElement.getName().getLocalPart() == (SENSORBURNREC)) {
					continue;
				} else if (startElement.getName().getLocalPart() == (CALIBRATIONLIST)) {
					ret.calibrationPage = parseBurnRecCalibrationList(eventReader);
					ret.highestValidCalPageIndex = (byte) (ret.calibrationPage.size()-1);
					continue;
				}
				event = eventReader.nextEvent();
				ret.parseBurnRecElement(startElement.getName().getLocalPart(),
						event);
			}
			if (event.isEndElement()
					&& event.asEndElement().getName().getLocalPart() == (SENSORBURNREC))
				break;
		}
		return ret;

	}

	private static Vector<GoIOCalibrationPage> parseBurnRecCalibrationList(XMLEventReader eventReader)
			throws XMLStreamException {
		Vector<GoIOCalibrationPage> gcp = new Vector<GoIOCalibrationPage>();
		while (eventReader.hasNext()) {
			XMLEvent event = eventReader.nextEvent();
			if (event.isStartElement()) {
				StartElement startElement = event.asStartElement();
				if (startElement.getName().getLocalPart() == (CALIBRATION)) {
					int index = parseBurnRecCalibrationPageAttribs(startElement);
					gcp.add(index, GoIOCalibrationPage.readFromXml(eventReader));
					continue;
				}
			}
			if (event.isEndElement()
					&& event.asEndElement().getName().getLocalPart() == (CALIBRATIONLIST))
				break;
		}
		return gcp;
	}

	private static int parseBurnRecCalibrationPageAttribs(StartElement startElement){
		int index = 0;
		@SuppressWarnings("unchecked")
		Iterator<Attribute> attributes = startElement.getAttributes();
		while (attributes.hasNext()) {
			Attribute attribute = attributes.next();
			String s = attribute.getName().toString();
			if (s.equals("DDSIndex")){
				index = Integer.parseInt(attribute.getValue());
				break;
			}
		}
		return index;
	}
	
	private void parseBurnRecElement(String n, XMLEvent event) {
		if ("MemMapVersion".equals(n))
			memMapVersion = Byte.parseByte(event.toString());
		else if ("ManufacturerID".equals(n))
			manufacturerID = Byte.parseByte(event.toString());
		else if ("SensorLongName".equals(n))
			sensorLongName = event.toString();
		else if ("SensorShortName".equals(n))
			sensorShortName =
					event.toString();
		else if ("Uncertainty".equals(n))
			uncertainty = Byte.parseByte(event.toString());
		else if ("SignificantFigures".equals(n))
			significantFigures = Byte.parseByte(event.toString());
		// else if("Precision".equals(n))
		// 
		else if ("CurrentRequirement".equals(n))
			currentRequirement = Byte.parseByte(event.toString());
		else if ("Averaging".equals(n))
			averaging = Byte.parseByte(event.toString());
		else if ("MinSamplePeriod".equals(n))
			minSamplePeriod = Float.parseFloat(event.toString());
		else if ("TypSamplePeriod".equals(n))
			typSamplePeriod = Float.parseFloat(event.toString());
		else if ("TypNumberofSamples".equals(n))
			typNumberOfSamples = Short.parseShort(event.toString());
		else if ("WarmUpTime".equals(n))
			warmUpTime = Short.parseShort(event.toString());
		else if ("ExperimentType".equals(n))
			experimentType = Byte.parseByte(event.toString());
		else if ("OperationType".equals(n))
			operationType = Byte.parseByte(event.toString());
		else if ("CalibrationEquation".equals(n))
			calibrationEquation = Byte.parseByte(event.toString());
		else if ("YminValue".equals(n))
			yminValue = Float.parseFloat(event.toString());
		else if ("YmaxValue".equals(n))
			ymaxValue = Float.parseFloat(event.toString());
		else if ("Yscale".equals(n))
			yScale = Byte.parseByte(event.toString());
		else if ("ActiveCalPage".equals(n))
			activeCalPage = Byte.parseByte(event.toString());
	}

	public byte getMemMapVersion() {
		return memMapVersion;
	}

	public void setMemMapVersion(byte memMapVersion) {
		this.memMapVersion = memMapVersion;
	}

	public byte getManufacturerID() {
		return manufacturerID;
	}

	public void setManufacturerID(byte manufacturerID) {
		this.manufacturerID = manufacturerID;
	}

	public byte getSensorNumber() {
		return sensorNumber;
	}

	public void setSensorNumber(byte sensorNumber) {
		this.sensorNumber = sensorNumber;
	}

	public byte[] getSensorSerialNumber() {
		return sensorSerialNumber;
	}

	public void setSensorSerialNumber(byte[] sensorSerialNumber) {
		this.sensorSerialNumber = sensorSerialNumber;
	}

	public byte[] getSensorLotCode() {
		return sensorLotCode;
	}

	public void setSensorLotCode(byte[] sensorLotCode) {
		this.sensorLotCode = sensorLotCode;
	}

	public String getSensorLongName() {
		return sensorLongName;
	}

	public void setSensorLongName(String sensorLongName) {
		this.sensorLongName = sensorLongName;
	}

	public String getSensorShortName() {
		return sensorShortName;
	}

	public void setSensorShortName(String sensorShortName) {
		this.sensorShortName = sensorShortName;
	}

	public byte getUncertainty() {
		return uncertainty;
	}

	public void setUncertainty(byte uncertainty) {
		this.uncertainty = uncertainty;
	}

	public byte getSignificantFigures() {
		return significantFigures;
	}

	public void setSignificantFigures(byte significantFigures) {
		this.significantFigures = significantFigures;
	}

	public byte getCurrentRequirement() {
		return currentRequirement;
	}

	public void setCurrentRequirement(byte currentRequirement) {
		this.currentRequirement = currentRequirement;
	}

	public byte getAveraging() {
		return averaging;
	}

	public void setAveraging(byte averaging) {
		this.averaging = averaging;
	}

	public float getMinSamplePeriod() {
		return minSamplePeriod;
	}

	public void setMinSamplePeriod(float minSamplePeriod) {
		this.minSamplePeriod = minSamplePeriod;
	}

	public float getTypSamplePeriod() {
		return typSamplePeriod;
	}

	public void setTypSamplePeriod(float typSamplePeriod) {
		this.typSamplePeriod = typSamplePeriod;
	}

	public short getTypNumberOfSamples() {
		return typNumberOfSamples;
	}

	public void setTypNumberOfSamples(short typNumberOfSamples) {
		this.typNumberOfSamples = typNumberOfSamples;
	}

	public short getWarmUpTime() {
		return warmUpTime;
	}

	public void setWarmUpTime(short warmUpTime) {
		this.warmUpTime = warmUpTime;
	}

	public byte getExperimentType() {
		return experimentType;
	}

	public void setExperimentType(byte experimentType) {
		this.experimentType = experimentType;
	}

	public byte getOperationType() {
		return operationType;
	}

	public void setOperationType(byte operationType) {
		this.operationType = operationType;
	}

	public byte getCalibrationEquation() {
		return calibrationEquation;
	}

	public void setCalibrationEquation(byte calibrationEquation) {
		this.calibrationEquation = calibrationEquation;
	}

	public float getYminValue() {
		return yminValue;
	}

	public void setYminValue(float yminValue) {
		this.yminValue = yminValue;
	}

	public float getYmaxValue() {
		return ymaxValue;
	}

	public void setYmaxValue(float ymaxValue) {
		this.ymaxValue = ymaxValue;
	}

	public byte getyScale() {
		return yScale;
	}

	public void setyScale(byte yScale) {
		this.yScale = yScale;
	}

	public byte getHighestValidCalPageIndex() {
		return highestValidCalPageIndex;
	}

	public void setHighestValidCalPageIndex(byte highestValidCalPageIndex) {
		this.highestValidCalPageIndex = highestValidCalPageIndex;
	}

	public byte getActiveCalPage() {
		return activeCalPage;
	}

	public void setActiveCalPage(byte activeCalPage) {
		this.activeCalPage = activeCalPage;
	}

	public Vector<GoIOCalibrationPage> getCalibrationPage() {
		return calibrationPage;
	}

	public void setCalibrationPage(Vector<GoIOCalibrationPage> calibrationPage) {
		this.calibrationPage = calibrationPage;
	}

	public byte getChecksum() {
		return checksum;
	}

	public void setChecksum(byte checksum) {
		this.checksum = checksum;
	}

}
