/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package libGoIO;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class GoIOSensorFactory {

	private static boolean DEBUG = false;

	private static String quelle = "sensormap.xml";
	private static String erweiterung = "sensormapextensions.xml";
	private static boolean erweiterungVerwenden = false;

	/* XML Elements */
	private static final String SENSOR = "Sensor";
	private static final String SENSORID = "ID_Name";
	private static final String SENSORNAME = "LocalName";
	private static final String RUBRIK = "SensorFamily";
	private static final String MESSGROESSE = "lname_base";

	private static SensorMap strings;

	public static String getString(String s) {
		if (strings == null)
			strings = new SensorMap();
		return strings.getString(s);
	}

	public static DefaultTreeModel createSensorTreeModel(
			DefaultMutableTreeNode root) {
		try {
			InputStream in;
			in = GoIOSensorFactory.class.getResourceAsStream(quelle);
			if (strings == null)
				strings = new SensorMap();
			root = parseInputFile(in, root, strings);
			if (erweiterungVerwenden) {
				in = GoIOSensorFactory.class.getResourceAsStream(erweiterung);
				root = parseInputFile(in, root, strings);
			}
			if (!root.isLeaf())
				sortChildren(root);
		} catch (XMLStreamException e) {
			if (DEBUG)
				e.printStackTrace();
		}

		return new DefaultTreeModel(root);
	}

	public static HashMap<String, String> createSensorMap() {
		HashMap<String, String> map = new HashMap<String, String>();
		try {
			InputStream in;
			in = GoIOSensorFactory.class.getResourceAsStream(quelle);
			if (strings == null)
				strings = new SensorMap();
			map = parseInputFileToMap(in, map, strings);
			if (erweiterungVerwenden) {
				in = GoIOSensorFactory.class.getResourceAsStream(erweiterung);
				map = parseInputFileToMap(in, map, strings);
			}
		} catch (XMLStreamException e) {
			if (DEBUG)
				e.printStackTrace();
		}

		return map;
	}

	private static void sortChildren(DefaultMutableTreeNode node) {
		@SuppressWarnings("unchecked")
		ArrayList<TreeNode> children = Collections.list(node
				.children());
		ArrayList<String> orgCnames = new ArrayList<String>();
		ArrayList<String> cNames = new ArrayList<String>();
		DefaultMutableTreeNode temParent = new DefaultMutableTreeNode();
		for (Object child : children) {
			DefaultMutableTreeNode ch = (DefaultMutableTreeNode) child;
			temParent.insert(ch, 0);
			cNames.add(ch.toString().toUpperCase());
			orgCnames.add(ch.toString().toUpperCase());
		}
		Collections.sort(cNames);
		for (String name : cNames) {
			int indx = orgCnames.indexOf(name);
			node.insert((DefaultMutableTreeNode) children.get(indx),
					node.getChildCount());
		}
	}

	private static DefaultMutableTreeNode parseInputFile(InputStream in,
			DefaultMutableTreeNode root, SensorMap strings)
			throws XMLStreamException {
		if (in == null || root == null)
			throw new XMLStreamException("InputStream or TreeNode is Null");
		// First create a new XMLInputFactory
		XMLInputFactory inputFactory = XMLInputFactory.newInstance();
		// Setup a new eventReader
		XMLEventReader eventReader = inputFactory.createXMLEventReader(in);
		String sensorID = null, sensorName = null, rubrik = null;
		// Read the XML document
		while (eventReader.hasNext()) {
			XMLEvent event = eventReader.nextEvent();

			if (event.isStartElement()) {
				StartElement startElement = event.asStartElement();
				if (startElement.getName().getLocalPart() == (SENSOR)) {
					@SuppressWarnings("unchecked")
					Iterator<Attribute> attributes = startElement
							.getAttributes();
					while (attributes.hasNext()) {
						Attribute attribute = attributes.next();
						if (attribute.getName().toString().equals(SENSORID)) {
							sensorID = attribute.getValue();
							break;
						}
					}
					continue;
				} else if (startElement.getName().getLocalPart() == (SENSORNAME)) {
					event = eventReader.nextEvent();
					sensorName = event.asCharacters().getData();
					sensorName = strings.getString(sensorName);
					continue;
				} else if ((rubrik == null || "".equals(rubrik))
						&& startElement.getName().getLocalPart() == (MESSGROESSE)) {
					event = eventReader.nextEvent();
					try {
						rubrik = event.asCharacters().getData();
						rubrik = strings.getString(rubrik);
					} catch (ClassCastException e) {
					}
					continue;
				} else if (startElement.getName().getLocalPart() == (RUBRIK)) {
					event = eventReader.nextEvent();
					try {
						rubrik = event.asCharacters().getData();
						rubrik = strings.getString(rubrik);
					} catch (ClassCastException e) {
					}
					continue;
				}
			}

			if (event.isEndElement()) {
				EndElement endElement = event.asEndElement();
				if (endElement.getName().getLocalPart() == (SENSOR)) {
					if (DEBUG && (sensorName == null))
						System.out
								.println("[GoIOSensorFactory] - createSensorMap: SensorId("
										+ sensorID
										+ ") sensorName("
										+ sensorName + ")");
					if (sensorID != null && sensorName != null) {
						if (rubrik == null)
							rubrik = sensorName;
						if ("".equals(rubrik)) {
							root.add(new DefaultMutableTreeNode(
									new GoIOSensorInfo(sensorName, sensorID,
											rubrik)));
						} else {
							// Rubrik suchen bzw. hinzufügen
							Enumeration<TreeNode> rubriken = root
									.children();
							DefaultMutableTreeNode rubrikNode = null;
							while (rubriken.hasMoreElements()) {
								DefaultMutableTreeNode neu = (DefaultMutableTreeNode) rubriken
										.nextElement();
								if (neu.toString().equals(rubrik)) {
									rubrikNode = neu;
									break;
								}
							}
							if (rubrikNode == null) {
								rubrikNode = new DefaultMutableTreeNode(rubrik);
								root.add(rubrikNode);
							}
							// Blatt hinzufügen
							rubrikNode.add(new DefaultMutableTreeNode(
									new GoIOSensorInfo(sensorName, sensorID,
											rubrik)));
						}
					}
					sensorID = null;
					sensorName = null;
					rubrik = null;
				}
			}

		}
		return root;

	}

	private static HashMap<String, String> parseInputFileToMap(InputStream in,
			HashMap<String, String> map, SensorMap strings)
			throws XMLStreamException {
		if (in == null)
			throw new XMLStreamException("InputStream or TreeNode is Null");
		if (map == null)
			map = new HashMap<String, String>();
		// First create a new XMLInputFactory
		XMLInputFactory inputFactory = XMLInputFactory.newInstance();
		// Setup a new eventReader
		XMLEventReader eventReader = inputFactory.createXMLEventReader(in);
		StringWriter inhalt = null;
		String sensorName = null, sensorID = null;

		// Read the XML document
		while (eventReader.hasNext()) {
			XMLEvent event = eventReader.nextEvent();
			if (inhalt != null)
				event.writeAsEncodedUnicode(inhalt);
			if (event.isStartElement()) {
				StartElement startElement = event.asStartElement();
				// If we have a sensor element we create a new node
				if (startElement.getName().getLocalPart() == (SENSOR)) {
					inhalt = new StringWriter();
					startElement.writeAsEncodedUnicode(inhalt);
					@SuppressWarnings("unchecked")
					Iterator<Attribute> attributes = startElement
							.getAttributes();
					while (attributes.hasNext()) {
						Attribute attribute = attributes.next();
						if (attribute.getName().toString().equals(SENSORID)) {
							sensorID = attribute.getValue();
							break;
						}
					}
					continue;
				} else if (startElement.getName().getLocalPart() == (SENSORNAME)) {
					event = eventReader.nextEvent();
					event.writeAsEncodedUnicode(inhalt);
					sensorName = event.asCharacters().getData();
					sensorName = strings.getString(sensorName);
					continue;
				}
			}

			if (event.isEndElement()
					&& event.asEndElement().getName().getLocalPart() == (SENSOR)) {
				if (sensorName != null && sensorID != null)
					map.put(sensorName + "(" + sensorID + ")",
							inhalt.toString());
				sensorName = null;
				sensorID = null;
				if (inhalt != null)
					try {
						inhalt.close();
					} catch (IOException e) {
						if (DEBUG)
							e.printStackTrace();
					}
				inhalt = null;
			}

		}
		return map;

	}

	public static void main(String[] args) {
		System.out.println("[GoIOSensorFactory] - main: ");
		/*
		 * Test: createSensorMap
		 */
		// HashMap<String, String> map = createSensorMap();
		// System.out.println("[GoIOSensorFactory] - main: SensorMap (");
		// for(String s : map.keySet()){
		// System.out.println("\tSensorname:\t"+s);
		// }
		// System.out.println(") erstellt.");
		/*
		 * Test: createTreeModel
		 */
		TreeModel tree = createSensorTreeModel(new DefaultMutableTreeNode(
				"Testbaum"));
		System.out.println("[GoIOSensorFactory] - main: SensorTreeModel (");
		for (Enumeration<? extends TreeNode> e = ((TreeNode)tree.getRoot()).children(); e.hasMoreElements();) {
			TreeNode rubrik;
			System.out.println("\tRubrik\t" + (rubrik = e.nextElement()));
			for (Enumeration<? extends TreeNode> ee = rubrik.children(); ee.hasMoreElements();) {
				System.out.println("\t\tSensor:\t" + ee.nextElement());
			}
		}
		System.out.println(") erstellt.");
	}

}
