package test;

import static libGoIO.GoIOLibrary.*;
import static libGoIO.GoIOLibrary.EEquationType.*;

import org.bridj.Pointer;

public class GoIODeviceCheck {

	public static int MAX_NUM_MEASUREMENTS = 100;
	public static String[] deviceDesc = new String[]
			{"?", "?", "Go! Temp", "Go! Link", "Go! Motion", "?", "?", "Mini GC"};

	public static int VERNIER_DEFAULT_VENDOR_ID  = 0x08F7;
	public static int LABPRO_DEFAULT_PRODUCT_ID = 0x0001,
			USB_DIRECT_TEMP_DEFAULT_PRODUCT_ID = 0x0002,	//aka GoTemp
			SKIP_DEFAULT_PRODUCT_ID = 0x0003,				//aka GoLink
			CYCLOPS_DEFAULT_PRODUCT_ID = 0x0004,			//aka GoMotion
			NGI_DEFAULT_PRODUCT_ID = 0x0005,				//aka LabQuest
			LOWCOST_SPEC_DEFAULT_PRODUCT_ID = 0x0006,		//aka CK Spectrometer
			MINI_GC_DEFAULT_PRODUCT_ID = 0x0007,			//aka Vernier Mini Gas Chromatograph
			STANDALONE_DAQ_DEFAULT_PRODUCT_ID = 0x0008;
	public static byte SKIP_CMD_ID_START_MEASUREMENTS = 0x18;
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Pointer<Byte> deviceName;
		deviceName = Pointer.allocateBytes(GOIO_MAX_SIZE_DEVICE_NAME);
		Pointer<Integer> vendorId = Pointer.allocateInt() ;		//USB vendor id
		Pointer<Integer> productId = Pointer.allocateInt();		//USB product id
		Pointer<Byte> tmpstring = Pointer.allocateBytes(100);
		Pointer<Short> MajorVersion = Pointer.allocateShort();
		Pointer<Short> MinorVersion = Pointer.allocateShort();
		Pointer<Byte> units = Pointer.allocateBytes(20);
		Pointer<Byte> equationType = Pointer.allocateByte();
		equationType.set((byte) 0);

		Pointer<Integer> rawMeasurements = Pointer.allocateInts(MAX_NUM_MEASUREMENTS);
		double[] volts = new double[MAX_NUM_MEASUREMENTS];
		double[] calbMeasurements = new double[MAX_NUM_MEASUREMENTS];
		int numMeasurements, i;
		double averageCalbMeasurement;

		System.out.printf("goIODeviceCheck version 1.0\n");
		
		goIOInit();

		goIOGetDLLVersion(MajorVersion, MinorVersion);
		System.out.printf("This app is linked to GoIO lib version %d.%d .\n", MajorVersion.get(), MinorVersion.get());

		boolean bFoundDevice = GetAvailableDeviceName(deviceName, GOIO_MAX_SIZE_DEVICE_NAME, vendorId, productId);
		if (!bFoundDevice)
			System.out.printf("No Go devices found.\n");
		else
		{
			System.out.println("The device name is "+deviceName.get());
			Pointer<?> hDevice = goIOSensorOpen(deviceName, vendorId.get(), productId.get(), 0);
			if (hDevice != null)
			{
				System.out.printf("Successfully opened %s device %s .\n", deviceDesc[productId.get()], deviceName.getCString());

				Pointer<Byte> charId = Pointer.allocateByte();
				goIOSensorDDSMemGetSensorNumber(hDevice, charId, 0, 0);
				System.out.printf("Sensor id = %d", charId.get());

				goIOSensorDDSMemGetLongName(hDevice, tmpstring, (short)tmpstring.getValidBytes());
				if (tmpstring.getCString().length() != 0)
					System.out.printf("(%s)", tmpstring.getCString());
				System.out.printf("\n");

				goIOSensorSetMeasurementPeriod(hDevice, 0.040, SKIP_TIMEOUT_MS_DEFAULT);//40 milliseconds measurement period.
				goIOSensorSendCmdAndGetResponse(hDevice, SKIP_CMD_ID_START_MEASUREMENTS, null, 0, null, null, SKIP_TIMEOUT_MS_DEFAULT);
				OSSleep(1000); //Wait 1 second.

				numMeasurements = goIOSensorReadRawMeasurements(hDevice, rawMeasurements, MAX_NUM_MEASUREMENTS);
				System.out.printf("%d measurements received after about 1 second.\n", numMeasurements);
				averageCalbMeasurement = 0.0;
				for (i = 0; i < numMeasurements; i++)
				{
					volts[i] = goIOSensorConvertToVoltage(hDevice, rawMeasurements.get(i));
					calbMeasurements[i] = goIOSensorCalibrateData(hDevice, volts[i]);
					averageCalbMeasurement += calbMeasurements[i];
				}
				if (numMeasurements > 1)
					averageCalbMeasurement = averageCalbMeasurement/numMeasurements;

				goIOSensorDDSMemGetCalibrationEquation(hDevice, equationType);
				if (equationType.get() != kEquationType_Linear.ordinal())
					units.setCString("volts");
				else
				{
					Pointer<Float> a = Pointer.allocateFloat(), b = Pointer.allocateFloat(), 
							c = Pointer.allocateFloat();
					Pointer<Byte> activeCalPage = Pointer.allocateByte();
					activeCalPage.set((byte) 0);
					goIOSensorDDSMemGetActiveCalPage(hDevice, activeCalPage);
					goIOSensorDDSMemGetCalPage(hDevice, activeCalPage.get(), a, b, c, units, (short)units.getValidBytes());
				}
				System.out.printf("Average measurement = %8.3f %s .\n", averageCalbMeasurement, units.getCString());

				goIOSensorClose(hDevice);
			}
		}
		goIOUninit();
		System.exit(0);
	}

	public static void OSSleep(int ms){
		try {
			Thread.sleep(ms);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	public static boolean GetAvailableDeviceName(Pointer<Byte> deviceName, int nameLength, Pointer<Integer> pVendorId, 
			Pointer<Integer> pProductId)
	{
		boolean bFoundDevice = false;
		deviceName.set((byte) 0);
		int numSkips = goIOUpdateListOfAvailableDevices(VERNIER_DEFAULT_VENDOR_ID, SKIP_DEFAULT_PRODUCT_ID);
		int numJonahs = goIOUpdateListOfAvailableDevices(VERNIER_DEFAULT_VENDOR_ID, USB_DIRECT_TEMP_DEFAULT_PRODUCT_ID);
		int numCyclopses = goIOUpdateListOfAvailableDevices(VERNIER_DEFAULT_VENDOR_ID, CYCLOPS_DEFAULT_PRODUCT_ID);
		int numMiniGCs = goIOUpdateListOfAvailableDevices(VERNIER_DEFAULT_VENDOR_ID, MINI_GC_DEFAULT_PRODUCT_ID);

		if (numSkips > 0)
		{
			goIOGetNthAvailableDeviceName(deviceName, nameLength, VERNIER_DEFAULT_VENDOR_ID, SKIP_DEFAULT_PRODUCT_ID, 0);
			pVendorId.set(VERNIER_DEFAULT_VENDOR_ID);
			pProductId.set(SKIP_DEFAULT_PRODUCT_ID);
			bFoundDevice = true;
		}
		else if (numJonahs > 0)
		{
			goIOGetNthAvailableDeviceName(deviceName, nameLength, VERNIER_DEFAULT_VENDOR_ID, USB_DIRECT_TEMP_DEFAULT_PRODUCT_ID, 0);
			pVendorId.set(VERNIER_DEFAULT_VENDOR_ID);
			pProductId.set(USB_DIRECT_TEMP_DEFAULT_PRODUCT_ID);
			bFoundDevice = true;
		}
		else if (numCyclopses > 0)
		{
			goIOGetNthAvailableDeviceName(deviceName, nameLength, VERNIER_DEFAULT_VENDOR_ID, CYCLOPS_DEFAULT_PRODUCT_ID, 0);
			pVendorId.set(VERNIER_DEFAULT_VENDOR_ID);
			pProductId.set(CYCLOPS_DEFAULT_PRODUCT_ID);
			bFoundDevice = true;
		}
		else if (numMiniGCs > 0)
		{
			goIOGetNthAvailableDeviceName(deviceName, nameLength, VERNIER_DEFAULT_VENDOR_ID, MINI_GC_DEFAULT_PRODUCT_ID, 0);
			pVendorId.set(VERNIER_DEFAULT_VENDOR_ID);
			pProductId.set(MINI_GC_DEFAULT_PRODUCT_ID);
			bFoundDevice = true;
		}

		return bFoundDevice;
	}

}
