package test;

import java.io.IOException;

import ld.api.CASSY;
import ld.api.PowerCASSY;
import ld.api.Waveforms;
import ld.api.ios.Box524011;
import ld.api.ios.CASSYIOs;
import ld.api.ios.CASSYQuantity;
import ld.api.ios.IWaveform;
import ld.api.ios.ScanMode;

public class TestFunctionGenerator {
    /*
     * Function generator example with Power-CASSY.
     * If you are using a serial connection, please specify the Connection and CommPort properties.
     */
    public static void main(String[] args) throws IOException
    {
        CASSY cassy = new PowerCASSY();                 // class of attached CASSY
        CASSYIOs cassyios = new CASSYIOs();             // class of all CASSYIOs
        if(cassy != null && cassyios != null)
        {
            cassyios.setAutoDefaults(true);             // change oscilloscope defaults automatically after selecting the first quantity
            //cassy.Connection = Connection.Serial;
            //cassy.CommPort = "COM1";
            cassy.open();
            if (cassy.getCASSYType() != cassy.getCASSYClassType())
            {
                System.out.println("Power-CASSY not found.");
                cassyios.close();
                cassy.close();
                return;
            }

            Box524011 io = new Box524011(cassy);        // Power-CASSY IO
            cassyios.add(io);
            cassyios.scan(ScanMode.ScanOnly);           // sets all SensorBoxValids and Power-CASSY's PowerGood property
            if (!cassy.getPowerGood())
            {
                System.out.println("Power-CASSY is not connected to its own power supply.");
                cassyios.close();
                return;
            }

            CASSYQuantity quantityU = io.getQuantityOutputU();
            CASSYQuantity quantityI = io.getQuantityInputI();
            quantityU.setSelected(true);                 // select voltage as output
            quantityI.setSelected(true);                 // select current as input

            IWaveform waveform = quantityU instanceof IWaveform? (IWaveform) quantityU : null;
            if (waveform != null)                        // sets waveform of output
            {
                waveform.setSingleShot(false);           // continous output
                waveform.setWaveform(Waveforms.SqrSym);
                waveform.getFrequencyParser().setValue(30);
                waveform.getAmplitudeParser().setValue(5);
                waveform.getOffsetParser().setValue(0);
                waveform.getRatioParser().setValue(50);
                /*
                 * Define waveform for Waveform == Waveforms.User
                 *
                double[] userWaveForm = new double[4000];
                for (int j = 0; j < userWaveForm.Length; j++)
                    userWaveForm[j] = Math.Sin(2 * Math.PI * j / userWaveForm.Length);
                waveform.WaveformParser.SetWaveform(userWaveForm);
                 */
            }

            cassyios.setOscilloscopeInterval(0.001);     // dt = 1 ms
            cassyios.setOscilloscopeCount(201);          // max. 200*dt = 200 ms
            /*
            cassyios.Trigger = true;                    // trigger switched on
            cassyios.TriggerQuantity = quantityU;       // start with 0° phase
            cassyios.TriggerValue = 0;
            cassyios.TriggerRising = true;
             */

            switch (cassyios.getOscilloscopeError())
            {
                case IncompatibleCASSYIO:
                    System.out.println("Incompatible inputs or outputs.");
                    cassyios.close();
                    return;
                case IncompatibleQuantities:
                	System.out.println("Incompatible quantities.");
                	cassyios.close();
                    return;
                case IncompatibleOscilloscopeInterval:
                	System.out.println("Not supported oscilloscope interval.");
                	cassyios.close();
                    return;
                default:
                	break;
            }

            int i = 0;
            System.out.println("Waiting for trigger signal.");
            while (!cassyios.doOscilloscope())          // start the oscilloscope and output the values
                for (; i < Math.min(quantityU.getValues(), quantityI.getValues()); i++)
                    System.out.printf("t=%f s\tU=%f V\tI=%f A\n", cassyios.getTimes(i), 
                    		quantityU.getValues(i), quantityI.getValues(i));
            cassyios.close();
        }
    }
}
