package test;

import java.io.IOException;

import ld.api.*;
import ld.api.ios.*;

/** 
	Pocket-CASSY mit US-Box 524070.
*/
public class TestUSBox {
	public static void main(String[] args) throws IOException, InterruptedException {
		try(CASSYs cassys = new CASSYs()) {
			cassys.scan(); // scan all CASSYs
			if (cassys.size() == 0) {
				System.out.println("No CASSY module found.");
				return;
			}
			if (cassys.get(0).getCASSYClassType() != CASSYTypes.Pocket) {
				System.out.println("Pocket-CASSY not found.");
				return;
			}

			CASSYIOs cassyios = cassys.getCASSYIOs(); // collection of all CASSYIOs
			cassyios.setAutoDefaults(true); // change measurement defaults automatically after selecting the first quantity
			Box524070 input = new Box524070(cassys.get(0),0);
			cassyios.add(input);
			cassyios.scan(ScanMode.ScanOnly); // sets all SensorBoxValids
			if (!input.getSensorBoxValid()) {
				System.out.printf("Expected sensor box %1$s not found.", input.getSensorBox(), "\r\n");
				return;
			}
	
			CASSYQuantity quantityS = input.getQuantityS();
			//CASSYQuantity quantityTime = input.getQuantityTime();
			
			quantityS.setSelected(true); // select voltage
			
			cassyios.setOscilloscopeInterval(0.1); // dt = 100 ms
			cassyios.setOscilloscopeCount(11); // max. 10*dt = 1000 ms
			
			switch (cassyios.getOscilloscopeError()) {
				case IncompatibleCASSYIO:
					System.out.println("Incompatible inputs or outputs.");
					return;
				case IncompatibleQuantities:
					System.out.println("Incompatible quantities.");
					return;
				case IncompatibleOscilloscopeInterval:
					System.out.println("Not supported oscilloscope interval.");
					return;
				default:
					break;
			}

			int i = 0;
			System.out.println("Waiting for trigger signal.");
			while (!cassyios.doOscilloscope()) { // start the oscilloscope and output the values
				for (; i < quantityS.getValues(); i++) {
					System.out.printf("s\ts=%2$s m", quantityS.getValues(i), "\r\n");
				}
			}
		}
	}
}